# LLM Docs - Documentation Generator

Documentation generation utilities for TypeScript and API projects. Provides basic documentation extraction and static site generation capabilities.

**Status**: ✅ Production Ready - v0.1.2 - Build successful
**Technology**: Docusaurus 3.x, React, TypeScript, Static site generation

## Technical Overview

Provides documentation generation from TypeScript source code and OpenAPI specifications. Includes basic static site generation and markdown processing capabilities.

### Recommended Replacements
1. **Docusaurus** - For documentation websites
   - Better UI/UX out of the box
   - Larger ecosystem and plugin support
   - Active development and community
   - Built-in versioning and i18n

2. **TypeDoc** - For TypeScript API documentation
   - Better TypeScript support
   - More accurate type extraction
   - Standard in TypeScript ecosystem
   - Better plugin ecosystem

### Migration Guide
```bash
# For documentation sites:
npx create-docusaurus@latest my-docs classic
# Move your markdown files to docs/

# For API documentation:
npm install --save-dev typedoc
npx typedoc --out docs src/index.ts
```

## Current Status

This project provides basic documentation tooling. While functional, it lacks the features and polish of established alternatives.

### Current Capabilities
- TypeScript/JavaScript source code documentation extraction
- OpenAPI/Swagger specification processing
- Markdown file processing and enhancement
- Static HTML site generation using Vite
- JSDoc and TSDoc comment parsing
- Basic full-text search implementation
- Template-based documentation structure

### Development Status
- Model registry API documentation (incomplete)
- Cross-project linking (basic implementation)
- PDF/DOCX export (experimental)
- Search indexing optimization (in progress)
- TypeScript complex type parsing (partial support)
- Plugin system (basic framework)

### Limitations
- No interactive API playground
- Manual versioning only
- Basic search functionality
- No analytics or tracking
- Limited plugin ecosystem
- Requires manual configuration

## Installation

```bash
# Clone the repository
git clone <repository>
cd llm-docs

# Install dependencies
npm install

# Build the project
npm run build

# Start development server
npm run dev
```

## Usage

### Generate Documentation

```bash
# Generate docs for a TypeScript project
npm run docs:generate -- --input ./src --output ./docs

# Generate API documentation
npm run docs:api -- --spec ./openapi.yaml --output ./docs/api

# Build static documentation site
npm run docs:build

# Serve documentation locally
npm run docs:serve
```

### Documentation Structure

```
docs/
├── api/              # API documentation
├── guides/           # User guides
├── reference/        # Technical reference
├── examples/         # Code examples
└── migration/        # Migration guides
```

## Features

### Documentation Generation
- Automatic extraction from TypeScript/JavaScript
- JSDoc and TSDoc support
- Markdown enhancement
- Code example extraction
- Type documentation

### API Documentation
- OpenAPI/Swagger support
- Interactive API explorer
- Request/response examples
- Authentication documentation
- Error code reference

### Static Site Features
- Fast static site generation
- Full-text search
- Syntax highlighting
- Mobile responsive design
- Dark mode support

## Configuration

Create a `llm-docs.config.js` file in your project root:

```javascript
module.exports = {
  input: './src',
  output: './docs',
  theme: 'default',
  features: {
    search: true,
    apiPlayground: true,
    darkMode: true
  },
  plugins: [
    // Add plugins here
  ]
};
```

## Project Structure

```
llm-docs/
├── src/
│   ├── core/           # Core documentation engine
│   ├── generators/     # Documentation generators
│   ├── parsers/        # Code parsers
│   ├── templates/      # Documentation templates
│   └── utils/          # Utility functions
├── scripts/            # Build and utility scripts
├── examples/           # Example projects
├── strategic/          # Strategic documentation
└── model-registry-api/ # Model registry API specs
```

## API

### Core API

```typescript
import { DocumentationGenerator } from '@llm/docs';

const generator = new DocumentationGenerator({
  input: './src',
  output: './docs'
});

await generator.generate();
```

### Plugin API

```typescript
export interface LLMDocsPlugin {
  name: string;
  transform?: (content: string, metadata: any) => string;
  generate?: (config: any) => Promise<void>;
}
```

## Development

```bash
# Install dependencies
npm install

# Run tests
npm test

# Run linting
npm run lint

# Build for production
npm run build

# Watch mode for development
npm run dev
```

## Templates

LLM Docs includes several documentation templates:

- `standard.md` - Standard documentation template
- `api.md` - API documentation template
- `architecture.md` - Architecture documentation template
- `db_schema.md` - Database schema documentation

## Technical Limitations

- Incomplete model registry API integration
- Complex TypeScript type parsing issues
- Performance issues with large codebases
- PDF export formatting problems
- Limited error handling in parsers
- No real-time update capabilities

## Contributing

This is an internal tool. When contributing:

1. Follow the existing code style
2. Add tests for new features
3. Update documentation
4. Submit pull requests to the development branch

## Migration Notes

If migrating from an older version:
- Check the migration guides in `docs/migration/`
- Update configuration files to new format
- Regenerate documentation after upgrading

## License

Proprietary - Internal Use Only 