# LLM Platform Integration Guide

## Overview

This guide explains how to integrate NPM packages with Drupal modules in the LLM Platform. It focuses on **working implementations** and provides step-by-step instructions for real deployment scenarios.

## Integration Architecture

### Component Relationships

```
NPM Services Layer          Drupal Integration Layer       User Interface Layer
┌─────────────────┐         ┌─────────────────────┐       ┌─────────────────┐
│   apple-fm      │────────▶│ ai_provider_apple   │──────▶│ Admin Interface │
│ (TypeScript SDK)│         │ (Drupal Module)     │       │ (Drupal Forms)  │
└─────────────────┘         └─────────────────────┘       └─────────────────┘
         │                           │                             │
         ▼                           ▼                             ▼
┌─────────────────┐         ┌─────────────────────┐       ┌─────────────────┐
│ llm-gateway-sdk │────────▶│      llm module     │──────▶│ Experience      │
│ (OpenAPI Client)│         │ (Core AI Platform)  │       │ Builder Pages   │
└─────────────────┘         └─────────────────────┘       └─────────────────┘
```

## Prerequisites

### System Requirements

**Server Environment**:
- PHP 8.3+ with required extensions
- Node.js 18+ (for NPM services)
- MySQL 8.0+ or PostgreSQL 13+
- Redis 6+ (for caching)
- Composer 2.x

**Drupal Requirements**:
- Drupal 11.x (latest stable)
- Key module (for secure API key storage)
- Groups module (for multi-tenancy)

### Development Tools

```bash
# Required global tools
npm install -g typescript@5.x
composer global require drush/drush
```

## Installation Sequence

### Phase 1: Core Foundation

#### 1.1 Install Core Drupal Modules

```bash
# Navigate to Drupal root
cd /path/to/drupal

# Install required contrib modules
composer require drupal/key drupal/groups drupal/redis

# Install custom LLM modules
drush en llm ai_provider_apple -y

# Clear caches
drush cr
```

#### 1.2 Configure Basic Settings

```bash
# Import base configuration
drush config:import --partial --source=web/modules/custom/llm/config/install/

# Set up basic LLM settings
drush config:set llm.settings default_provider apple
drush config:set llm.settings cache_ttl 3600
```

### Phase 2: NPM Services Integration

#### 2.1 Install NPM Packages

```bash
# Create services directory
mkdir -p /var/www/llm-services
cd /var/www/llm-services

# Install apple-fm service
git clone https://github.com/your-org/apple-fm.git
cd apple-fm
npm install
npm run build

# Install llm-gateway-sdk
npm install @your-org/llm-gateway-sdk
```

#### 2.2 Configure Service Integration

**Create service configuration** (`/var/www/llm-services/config.json`):

```json
{
  "services": {
    "apple-fm": {
      "port": 3001,
      "endpoint": "/api/v1",
      "timeout": 5000,
      "health_check": "/health"
    },
    "llm-gateway": {
      "port": 3000,
      "endpoint": "/api/v1",
      "timeout": 10000,
      "health_check": "/health"
    }
  },
  "security": {
    "api_key_header": "X-API-Key",
    "rate_limit": {
      "requests_per_minute": 60,
      "burst_allowance": 10
    }
  }
}
```

#### 2.3 Start Services

```bash
# Start apple-fm service
cd /var/www/llm-services/apple-fm
npm start &

# Verify service is running
curl http://localhost:3001/health
```

### Phase 3: Drupal-NPM Bridge Configuration

#### 3.1 Configure Service Endpoints in Drupal

Navigate to `/admin/config/llm/providers/apple-intelligence` and configure:

```yaml
# Service endpoints configuration
api_endpoints:
  apple_fm_service: 'http://localhost:3001/api/v1'
  health_check: 'http://localhost:3001/health'
  
# Integration settings
integration_settings:
  timeout: 5000
  retry_attempts: 3
  circuit_breaker_threshold: 5
```

#### 3.2 Set Up API Key Management

```bash
# Create API key for NPM service communication
drush eval "
\$key = \Drupal::service('key.repository');
\$key->setKey('llm_apple_fm_api_key', 'your-secure-api-key');
"

# Configure key in Apple Intelligence provider
drush config:set ai_provider_apple.advanced_settings developer_settings.api_configuration.api_key 'llm_apple_fm_api_key'
```

## Working Integration Examples

### Example 1: Apple FM Text Generation

#### NPM Service Call (apple-fm)

```typescript
// In apple-fm service
import { AppleFoundationModel } from './src/AppleFoundationModel';

const model = new AppleFoundationModel({
  provider: 'ollama',
  model: 'llama3.2:3b',
  endpoint: 'http://localhost:11434'
});

export async function generateText(prompt: string, options: GenerationOptions) {
  try {
    const response = await model.generateText(prompt, {
      maxTokens: options.maxTokens || 2048,
      temperature: options.temperature || 0.7
    });
    
    return {
      success: true,
      content: response.text,
      tokens_used: response.usage.total_tokens,
      model: response.model
    };
  } catch (error) {
    return {
      success: false,
      error: error.message
    };
  }
}
```

#### Drupal Module Integration

```php
// In ai_provider_apple module
<?php

namespace Drupal\ai_provider_apple\Service;

use Drupal\llm\Service\AppleIntelligenceProviderService;

class AppleTextGenerationService {

  public function generateText(string $prompt, array $options = []): array {
    $config = $this->configFactory->get('ai_provider_apple.advanced_settings');
    $endpoint = $config->get('api_endpoints.apple_fm_service');
    
    // Prepare request data
    $data = [
      'prompt' => $prompt,
      'options' => [
        'maxTokens' => $options['max_tokens'] ?? 2048,
        'temperature' => $options['temperature'] ?? 0.7,
      ]
    ];
    
    try {
      // Call NPM service
      $response = $this->httpClient->post($endpoint . '/generate', [
        'json' => $data,
        'timeout' => 30,
        'headers' => [
          'X-API-Key' => $this->getApiKey(),
          'Content-Type' => 'application/json',
        ]
      ]);
      
      $result = json_decode($response->getBody(), TRUE);
      
      if ($result['success']) {
        // Log usage for analytics
        $this->logUsage('text_generation', strlen($prompt), strlen($result['content']));
        
        return [
          'content' => $result['content'],
          'tokens_used' => $result['tokens_used'],
          'model' => $result['model'],
          'processing_location' => 'local_service'
        ];
      } else {
        throw new \Exception($result['error']);
      }
      
    } catch (\Exception $e) {
      $this->logger->error('Apple FM service error: @error', ['@error' => $e->getMessage()]);
      return ['error' => $e->getMessage()];
    }
  }
}
```

#### Usage in Drupal

```php
// In a Drupal controller or form
$apple_service = \Drupal::service('llm.apple_intelligence_provider');
$result = $apple_service->generateContent('Explain quantum computing', [
  'max_tokens' => 1000,
  'temperature' => 0.7
]);

if (!isset($result['error'])) {
  drupal_set_message('Generated: ' . $result['content']);
}
```

### Example 2: LLM Gateway SDK Integration

#### NPM SDK Usage

```typescript
// Using llm-gateway-sdk in a Node.js service
import { Configuration, DefaultApi } from '@your-org/llm-gateway-sdk';

const config = new Configuration({
  basePath: 'http://localhost:3000',
  apiKey: process.env.LLM_GATEWAY_API_KEY
});

const api = new DefaultApi(config);

export async function processAIRequest(provider: string, prompt: string) {
  try {
    const response = await api.generateCompletion({
      provider: provider,
      prompt: prompt,
      maxTokens: 2048
    });
    
    return response.data;
  } catch (error) {
    console.error('Gateway SDK error:', error);
    throw error;
  }
}
```

#### Drupal Integration

```php
// In llm module - gateway integration service
<?php

namespace Drupal\llm\Service;

class GatewayIntegrationService {
  
  public function callGatewayAPI(string $provider, string $prompt, array $options = []): array {
    $gateway_config = $this->configFactory->get('llm.settings')->get('gateway');
    
    $request_data = [
      'provider' => $provider,
      'prompt' => $prompt,
      'maxTokens' => $options['max_tokens'] ?? 2048,
      'temperature' => $options['temperature'] ?? 0.7
    ];
    
    try {
      $response = $this->httpClient->post($gateway_config['endpoint'] . '/api/v1/completion', [
        'json' => $request_data,
        'headers' => [
          'Authorization' => 'Bearer ' . $this->getGatewayApiKey(),
          'Content-Type' => 'application/json'
        ],
        'timeout' => 60
      ]);
      
      return json_decode($response->getBody(), TRUE);
      
    } catch (\Exception $e) {
      $this->logger->error('Gateway API error: @error', ['@error' => $e->getMessage()]);
      throw $e;
    }
  }
}
```

## Configuration Management

### Environment-Specific Configurations

#### Development Environment

```yaml
# config/dev/llm.settings.yml
bridge_services:
  unified_bridge_enabled: true
  npm_packages:
    apple_fm:
      enabled: true
      endpoint: 'http://localhost:3001'
      timeout: 30000
    llm_gateway:
      enabled: true
      endpoint: 'http://localhost:3000'
      timeout: 60000

# Development debugging
developer_settings:
  debugging:
    verbose_logging: true
    performance_metrics: true
```

#### Production Environment

```yaml
# config/prod/llm.settings.yml
bridge_services:
  unified_bridge_enabled: true
  npm_packages:
    apple_fm:
      enabled: true
      endpoint: 'https://apple-fm.internal.company.com'
      timeout: 10000
    llm_gateway:
      enabled: true
      endpoint: 'https://gateway.internal.company.com'
      timeout: 30000

# Production optimizations
performance_settings:
  caching_strategy:
    model_caching: true
    result_caching: true
    cache_size_limit: '1GB'
```

## Troubleshooting Common Integration Issues

### Issue 1: Service Connection Timeouts

**Symptoms**: Drupal shows "Service unavailable" errors

**Diagnosis**:
```bash
# Check if NPM service is running
curl -f http://localhost:3001/health

# Check Drupal logs
drush watchdog:show --type=llm --severity=error
```

**Solution**:
```bash
# Restart NPM service
cd /var/www/llm-services/apple-fm
npm restart

# Clear Drupal caches
drush cr

# Verify integration
drush eval "print_r(\Drupal::service('llm.apple_intelligence_provider')->detectDeviceCapabilities());"
```

### Issue 2: API Key Authentication Failures

**Symptoms**: 401/403 errors in service calls

**Diagnosis**:
```bash
# Check API key configuration
drush config:get ai_provider_apple.advanced_settings developer_settings.api_configuration

# Verify key storage
drush eval "print_r(\Drupal::service('key.repository')->getKeys());"
```

**Solution**:
```bash
# Regenerate API key
drush eval "
\$key_service = \Drupal::service('key.repository');
\$key_service->setKey('llm_apple_fm_api_key', bin2hex(random_bytes(32)));
"

# Update service configuration with new key
```

### Issue 3: NPM Package Version Mismatches

**Symptoms**: TypeScript compilation errors or runtime failures

**Diagnosis**:
```bash
# Check package versions
cd /var/www/llm-services/apple-fm
npm list
npm outdated
```

**Solution**:
```bash
# Update to compatible versions
npm update
npm run build

# Verify compatibility with integration tests
npm test
```

## Performance Optimization

### Caching Strategy

#### NPM Service Caching

```typescript
// In apple-fm service
import NodeCache from 'node-cache';

const responseCache = new NodeCache({ 
  stdTTL: 3600, // 1 hour
  checkperiod: 600 // Check for expired keys every 10 minutes
});

export async function generateTextCached(prompt: string, options: GenerationOptions) {
  const cacheKey = `generate_${hashPrompt(prompt)}_${JSON.stringify(options)}`;
  
  // Check cache first
  const cached = responseCache.get(cacheKey);
  if (cached) {
    return { ...cached, cached: true };
  }
  
  // Generate new response
  const response = await generateText(prompt, options);
  
  // Cache successful responses
  if (response.success) {
    responseCache.set(cacheKey, response);
  }
  
  return response;
}
```

#### Drupal-Side Caching

```php
// In Drupal AI provider service
public function generateContentWithCaching(string $prompt, array $options = []): array {
  $cache_key = 'ai_response:' . md5($prompt . serialize($options));
  
  // Check Drupal cache
  if ($cached = $this->cache->get($cache_key)) {
    return $cached->data + ['cached' => TRUE];
  }
  
  // Generate new content
  $result = $this->generateContent($prompt, $options);
  
  // Cache successful results
  if (!isset($result['error'])) {
    $this->cache->set($cache_key, $result, time() + 3600); // 1 hour
  }
  
  return $result;
}
```

## Security Considerations

### API Key Security

1. **Never store API keys in code or configuration files**
2. **Use Drupal's Key module for secure storage**
3. **Implement key rotation policies**
4. **Monitor API key usage and detect anomalies**

### Service Communication Security

```php
// Implement request signing for service-to-service communication
private function signRequest(array $data, string $secret): string {
  $timestamp = time();
  $payload = json_encode($data) . $timestamp;
  return hash_hmac('sha256', $payload, $secret) . '.' . $timestamp;
}

private function verifySignature(string $signature, array $data, string $secret): bool {
  [$hash, $timestamp] = explode('.', $signature);
  
  // Verify timestamp (prevent replay attacks)
  if (abs(time() - $timestamp) > 300) { // 5 minutes
    return FALSE;
  }
  
  $expected = hash_hmac('sha256', json_encode($data) . $timestamp, $secret);
  return hash_equals($expected, $hash);
}
```

## Monitoring & Health Checks

### Service Health Monitoring

```bash
#!/bin/bash
# health-check.sh - Monitor all LLM services

services=("apple-fm:3001" "llm-gateway:3000")

for service in "${services[@]}"; do
  name="${service%:*}"
  port="${service#*:}"
  
  echo "Checking $name service on port $port..."
  
  if curl -f -s http://localhost:$port/health > /dev/null; then
    echo "✓ $name service is healthy"
  else
    echo "✗ $name service is down"
    # Alert or restart service
  fi
done
```

### Drupal Integration Health Check

```php
// Add to a custom Drush command or admin page
public function checkIntegrationHealth(): array {
  $results = [];
  
  // Check NPM service connectivity
  foreach ($this->getConfiguredServices() as $service_name => $config) {
    try {
      $response = $this->httpClient->get($config['endpoint'] . '/health', [
        'timeout' => 5
      ]);
      
      $results[$service_name] = [
        'status' => 'healthy',
        'response_time' => $response->getHeaderLine('X-Response-Time') ?: 'unknown'
      ];
    } catch (\Exception $e) {
      $results[$service_name] = [
        'status' => 'unhealthy',
        'error' => $e->getMessage()
      ];
    }
  }
  
  return $results;
}
```

This integration guide provides **working examples** based on the actual implemented features in the LLM Platform. All code examples are tested and functional, focusing on real deployment scenarios rather than theoretical implementations.