---
id: api-documentation
---
# API Documentation and Testing Guide

<!-- title: API Documentation and Testing Guide -->
<!-- description: Practical guide for API documentation and testing in the LLM Platform Platform -->
<!-- lastUpdated: 2024-03-19 -->

## Overview

This guide covers API documentation and testing for the LLM Platform Platform, including OpenAPI specification, documentation generation, API testing, and validation.

## OpenAPI Specification

### OpenAPI Service

```typescript
// src/api/documentation/service.ts
import { OpenAPI, OpenAPIMetrics } from './types';
import { MetricsService } from '../../monitoring/metrics/service';
import { Logger } from '../../logging/service';

export class OpenAPIService {
  constructor(
    private metrics: MetricsService,
    private logger: Logger,
    private options: OpenAPIServiceOptions
  ) {}

  async generateSpecification(): Promise<OpenAPI> {
    const startTime = Date.now();

    try {
      // Initialize specification
      const spec = await this.initializeSpecification();

      // Add servers
      await this.addServers(spec);

      // Add paths
      await this.addPaths(spec);

      // Add components
      await this.addComponents(spec);

      // Add security
      await this.addSecurity(spec);

      // Add tags
      await this.addTags(spec);

      // Validate specification
      await this.validateSpecification(spec);

      // Record metrics
      this.recordGenerationMetrics(spec, startTime);

      // Log generation
      await this.logGeneration(spec);

      return spec;
    } catch (error) {
      // Record error
      this.recordGenerationError(error, startTime);

      // Log error
      await this.logGenerationError(error);

      throw error;
    }
  }

  private async initializeSpecification(): Promise<OpenAPI> {
    return {
      openapi: '3.1.0',
      info: {
        title: this.options.title,
        version: this.options.version,
        description: this.options.description,
        contact: this.options.contact,
        license: this.options.license
      },
      servers: [],
      paths: {},
      components: {
        schemas: {},
        responses: {},
        parameters: {},
        examples: {},
        requestBodies: {},
        headers: {},
        securitySchemes: {}
      },
      security: [],
      tags: []
    };
  }

  private async addServers(spec: OpenAPI): Promise<void> {
    for (const server of this.options.servers) {
      spec.servers.push({
        url: server.url,
        description: server.description,
        variables: server.variables
      });
    }
  }

  private async addPaths(spec: OpenAPI): Promise<void> {
    for (const route of this.options.routes) {
      const path = this.normalizePath(route.path);

      if (!spec.paths[path]) {
        spec.paths[path] = {};
      }

      spec.paths[path][route.method.toLowerCase()] = {
        summary: route.summary,
        description: route.description,
        operationId: route.operationId,
        tags: route.tags,
        parameters: await this.generateParameters(route),
        requestBody: await this.generateRequestBody(route),
        responses: await this.generateResponses(route),
        security: route.security,
        deprecated: route.deprecated
      };
    }
  }

  private async addComponents(spec: OpenAPI): Promise<void> {
    // Add schemas
    for (const schema of this.options.schemas) {
      spec.components.schemas[schema.name] = schema.schema;
    }

    // Add responses
    for (const response of this.options.responses) {
      spec.components.responses[response.name] = response.response;
    }

    // Add parameters
    for (const parameter of this.options.parameters) {
      spec.components.parameters[parameter.name] = parameter.parameter;
    }

    // Add examples
    for (const example of this.options.examples) {
      spec.components.examples[example.name] = example.example;
    }

    // Add request bodies
    for (const body of this.options.requestBodies) {
      spec.components.requestBodies[body.name] = body.body;
    }

    // Add headers
    for (const header of this.options.headers) {
      spec.components.headers[header.name] = header.header;
    }

    // Add security schemes
    for (const scheme of this.options.securitySchemes) {
      spec.components.securitySchemes[scheme.name] = scheme.scheme;
    }
  }

  private async addSecurity(spec: OpenAPI): Promise<void> {
    for (const security of this.options.security) {
      spec.security.push(security);
    }
  }

  private async addTags(spec: OpenAPI): Promise<void> {
    for (const tag of this.options.tags) {
      spec.tags.push({
        name: tag.name,
        description: tag.description,
        externalDocs: tag.externalDocs
      });
    }
  }

  private async validateSpecification(
    spec: OpenAPI
  ): Promise<void> {
    // Validate against schema
    const validator = new OpenAPIValidator({
      schema: spec
    });

    const result = await validator.validate();

    if (!result.valid) {
      throw new OpenAPIValidationError(result.errors);
    }

    // Validate against rules
    const rules = new OpenAPIRules(this.options.rules);

    const violations = await rules.validate(spec);

    if (violations.length > 0) {
      throw new OpenAPIRuleViolationError(violations);
    }
  }

  private async generateParameters(
    route: Route
  ): Promise<OpenAPIParameter[]> {
    const parameters: OpenAPIParameter[] = [];

    // Add path parameters
    for (const param of route.pathParams) {
      parameters.push(
        await this.generateParameter(param, 'path')
      );
    }

    // Add query parameters
    for (const param of route.queryParams) {
      parameters.push(
        await this.generateParameter(param, 'query')
      );
    }

    // Add header parameters
    for (const param of route.headerParams) {
      parameters.push(
        await this.generateParameter(param, 'header')
      );
    }

    // Add cookie parameters
    for (const param of route.cookieParams) {
      parameters.push(
        await this.generateParameter(param, 'cookie')
      );
    }

    return parameters;
  }

  private async generateParameter(
    param: Parameter,
    location: string
  ): Promise<OpenAPIParameter> {
    return {
      name: param.name,
      in: location,
      description: param.description,
      required: param.required,
      deprecated: param.deprecated,
      allowEmptyValue: param.allowEmptyValue,
      style: param.style,
      explode: param.explode,
      allowReserved: param.allowReserved,
      schema: param.schema,
      example: param.example,
      examples: param.examples
    };
  }

  private async generateRequestBody(
    route: Route
  ): Promise<OpenAPIRequestBody | undefined> {
    if (!route.body) {
      return undefined;
    }

    return {
      description: route.body.description,
      required: route.body.required,
      content: await this.generateContent(route.body.content)
    };
  }

  private async generateContent(
    content: Content
  ): Promise<Record<string, OpenAPIMediaType>> {
    const result: Record<string, OpenAPIMediaType> = {};

    for (const [type, mediaType] of Object.entries(content)) {
      result[type] = {
        schema: mediaType.schema,
        example: mediaType.example,
        examples: mediaType.examples,
        encoding: mediaType.encoding
      };
    }

    return result;
  }

  private async generateResponses(
    route: Route
  ): Promise<Record<string, OpenAPIResponse>> {
    const responses: Record<string, OpenAPIResponse> = {};

    for (const [code, response] of Object.entries(
      route.responses
    )) {
      responses[code] = {
        description: response.description,
        headers: await this.generateHeaders(response.headers),
        content: await this.generateContent(response.content),
        links: response.links
      };
    }

    return responses;
  }

  private async generateHeaders(
    headers: Record<string, Header>
  ): Promise<Record<string, OpenAPIHeader>> {
    const result: Record<string, OpenAPIHeader> = {};

    for (const [name, header] of Object.entries(headers)) {
      result[name] = {
        description: header.description,
        required: header.required,
        deprecated: header.deprecated,
        allowEmptyValue: header.allowEmptyValue,
        style: header.style,
        explode: header.explode,
        allowReserved: header.allowReserved,
        schema: header.schema,
        example: header.example,
        examples: header.examples
      };
    }

    return result;
  }

  private normalizePath(path: string): string {
    return path.replace(/\/+/g, '/');
  }

  private recordGenerationMetrics(
    spec: OpenAPI,
    startTime: number
  ): void {
    const duration = Date.now() - startTime;

    this.metrics.recordMetric({
      name: 'openapi_generation_duration_seconds',
      value: duration / 1000,
      labels: {
        version: spec.info.version,
        environment: this.options.environment,
        service: this.options.service,
        version: this.options.version
      },
      type: 'histogram',
      timestamp: new Date()
    });

    this.metrics.recordMetric({
      name: 'openapi_paths_total',
      value: Object.keys(spec.paths).length,
      labels: {
        version: spec.info.version,
        environment: this.options.environment,
        service: this.options.service,
        version: this.options.version
      },
      type: 'gauge',
      timestamp: new Date()
    });

    this.metrics.recordMetric({
      name: 'openapi_components_total',
      value: this.countComponents(spec),
      labels: {
        version: spec.info.version,
        environment: this.options.environment,
        service: this.options.service,
        version: this.options.version
      },
      type: 'gauge',
      timestamp: new Date()
    });
  }

  private recordGenerationError(
    error: Error,
    startTime: number
  ): void {
    const duration = Date.now() - startTime;

    this.metrics.recordMetric({
      name: 'openapi_generation_error_duration_seconds',
      value: duration / 1000,
      labels: {
        error: error.name,
        environment: this.options.environment,
        service: this.options.service,
        version: this.options.version
      },
      type: 'histogram',
      timestamp: new Date()
    });
  }

  private async logGeneration(
    spec: OpenAPI
  ): Promise<void> {
    await this.logger.logGeneration({
      spec,
      timestamp: new Date()
    });
  }

  private async logGenerationError(
    error: Error
  ): Promise<void> {
    await this.logger.logGenerationError({
      error,
      timestamp: new Date()
    });
  }

  private countComponents(spec: OpenAPI): number {
    return (
      Object.keys(spec.components.schemas).length +
      Object.keys(spec.components.responses).length +
      Object.keys(spec.components.parameters).length +
      Object.keys(spec.components.examples).length +
      Object.keys(spec.components.requestBodies).length +
      Object.keys(spec.components.headers).length +
      Object.keys(spec.components.securitySchemes).length
    );
  }
}

interface OpenAPIServiceOptions {
  environment: string;
  service: string;
  version: string;
  title: string;
  description: string;
  contact: OpenAPIContact;
  license: OpenAPILicense;
  servers: OpenAPIServer[];
  routes: Route[];
  schemas: Schema[];
  responses: Response[];
  parameters: Parameter[];
  examples: Example[];
  requestBodies: RequestBody[];
  headers: Header[];
  securitySchemes: SecurityScheme[];
  security: OpenAPISecurityRequirement[];
  tags: OpenAPITag[];
  rules: OpenAPIRule[];
}

interface OpenAPIContact {
  name: string;
  url: string;
  email: string;
}

interface OpenAPILicense {
  name: string;
  url: string;
}

interface OpenAPIServer {
  url: string;
  description: string;
  variables: Record<string, OpenAPIServerVariable>;
}

interface OpenAPIServerVariable {
  default: string;
  enum?: string[];
  description?: string;
}

interface Route {
  path: string;
  method: string;
  summary: string;
  description: string;
  operationId: string;
  tags: string[];
  pathParams: Parameter[];
  queryParams: Parameter[];
  headerParams: Parameter[];
  cookieParams: Parameter[];
  body?: RequestBody;
  responses: Record<string, Response>;
  security?: OpenAPISecurityRequirement[];
  deprecated?: boolean;
}

interface Parameter {
  name: string;
  description: string;
  required: boolean;
  deprecated?: boolean;
  allowEmptyValue?: boolean;
  style?: string;
  explode?: boolean;
  allowReserved?: boolean;
  schema: OpenAPISchema;
  example?: any;
  examples?: Record<string, OpenAPIExample>;
}

interface RequestBody {
  description: string;
  required: boolean;
  content: Content;
}

interface Content {
  [key: string]: MediaType;
}

interface MediaType {
  schema: OpenAPISchema;
  example?: any;
  examples?: Record<string, OpenAPIExample>;
  encoding?: Record<string, OpenAPIEncoding>;
}

interface Response {
  description: string;
  headers?: Record<string, Header>;
  content?: Content;
  links?: Record<string, OpenAPILink>;
}

interface Header {
  description: string;
  required?: boolean;
  deprecated?: boolean;
  allowEmptyValue?: boolean;
  style?: string;
  explode?: boolean;
  allowReserved?: boolean;
  schema: OpenAPISchema;
  example?: any;
  examples?: Record<string, OpenAPIExample>;
}

interface Schema {
  name: string;
  schema: OpenAPISchema;
}

interface OpenAPISchema {
  type?: string;
  format?: string;
  title?: string;
  description?: string;
  default?: any;
  multipleOf?: number;
  maximum?: number;
  exclusiveMaximum?: boolean;
  minimum?: number;
  exclusiveMinimum?: boolean;
  maxLength?: number;
  minLength?: number;
  pattern?: string;
  maxItems?: number;
  minItems?: number;
  uniqueItems?: boolean;
  maxProperties?: number;
  minProperties?: number;
  required?: string[];
  enum?: any[];
  properties?: Record<string, OpenAPISchema>;
  items?: OpenAPISchema;
  allOf?: OpenAPISchema[];
  oneOf?: OpenAPISchema[];
  anyOf?: OpenAPISchema[];
  not?: OpenAPISchema;
  additionalProperties?: boolean | OpenAPISchema;
  nullable?: boolean;
  readOnly?: boolean;
  writeOnly?: boolean;
  xml?: OpenAPIXML;
  externalDocs?: OpenAPIExternalDocumentation;
  example?: any;
  deprecated?: boolean;
}

interface OpenAPIXML {
  name?: string;
  namespace?: string;
  prefix?: string;
  attribute?: boolean;
  wrapped?: boolean;
}

interface OpenAPIExternalDocumentation {
  description?: string;
  url: string;
}

interface OpenAPIExample {
  value: any;
  summary?: string;
  description?: string;
  externalValue?: string;
}

interface OpenAPIEncoding {
  contentType?: string;
  headers?: Record<string, OpenAPIHeader>;
  style?: string;
  explode?: boolean;
  allowReserved?: boolean;
}

interface OpenAPILink {
  operationRef?: string;
  operationId?: string;
  parameters?: Record<string, any>;
  requestBody?: any;
  description?: string;
  server?: OpenAPIServer;
}

interface OpenAPISecurityRequirement {
  [key: string]: string[];
}

interface OpenAPITag {
  name: string;
  description?: string;
  externalDocs?: OpenAPIExternalDocumentation;
}

interface OpenAPIRule {
  name: string;
  description: string;
  validate: (spec: OpenAPI) => Promise<OpenAPIViolation[]>;
}

interface OpenAPIViolation {
  rule: string;
  path: string;
  message: string;
}

class OpenAPIValidationError extends Error {
  constructor(errors: OpenAPIValidationError[]) {
    super('OpenAPI validation failed');
    this.name = 'OpenAPIValidationError';
    this.errors = errors;
  }

  errors: OpenAPIValidationError[];
}

class OpenAPIRuleViolationError extends Error {
  constructor(violations: OpenAPIViolation[]) {
    super('OpenAPI rule violations');
    this.name = 'OpenAPIRuleViolationError';
    this.violations = violations;
  }

  violations: OpenAPIViolation[];
}

interface OpenAPIMetrics {
  generations: number;
  validations: number;
  violations: number;
  errors: number;
}
```

## API Testing

### API Test Service

```typescript
// src/api/testing/service.ts
import { APITest, APITestMetrics } from './types';
import { MetricsService } from '../../monitoring/metrics/service';
import { Logger } from '../../logging/service';

export class APITestService {
  constructor(
    private metrics: MetricsService,
    private logger: Logger,
    private options: APITestServiceOptions
  ) {}

  async runTests(tests: APITest[]): Promise<APITestResult[]> {
    const startTime = Date.now();
    const results: APITestResult[] = [];

    try {
      // Initialize tests
      await this.initializeTests(tests);

      // Run tests
      for (const test of tests) {
        const result = await this.runTest(test);
        results.push(result);
      }

      // Record metrics
      this.recordTestMetrics(tests, results, startTime);

      // Log results
      await this.logResults(tests, results);

      return results;
    } catch (error) {
      // Record error
      this.recordTestError(error, startTime);

      // Log error
      await this.logTestError(error);

      throw error;
    }
  }

  private async runTest(
    test: APITest
  ): Promise<APITestResult> {
    const startTime = Date.now();

    try {
      // Initialize test
      await this.initializeTest(test);

      // Execute request
      const response = await this.executeRequest(test);

      // Validate response
      await this.validateResponse(test, response);

      // Record metrics
      this.recordTestMetrics(test, response, startTime);

      // Log test
      await this.logTest(test, response);

      return {
        test,
        response,
        success: true,
        duration: Date.now() - startTime,
        timestamp: new Date()
      };
    } catch (error) {
      // Record error
      this.recordTestError(test, error, startTime);

      // Log error
      await this.logTestError(test, error);

      return {
        test,
        error,
        success: false,
        duration: Date.now() - startTime,
        timestamp: new Date()
      };
    }
  }

  private async executeRequest(
    test: APITest
  ): Promise<APITestResponse> {
    const request = this.buildRequest(test);

    const response = await this.options.client.request(request);

    return this.buildResponse(response);
  }

  private buildRequest(test: APITest): APITestRequest {
    return {
      method: test.method,
      url: this.buildUrl(test),
      headers: this.buildHeaders(test),
      body: this.buildBody(test),
      timeout: test.timeout
    };
  }

  private buildUrl(test: APITest): string {
    const baseUrl = this.options.baseUrl;
    const path = test.path;
    const query = this.buildQuery(test);

    return `${baseUrl}${path}${query}`;
  }

  private buildQuery(test: APITest): string {
    if (!test.query) {
      return '';
    }

    const params = new URLSearchParams();

    for (const [key, value] of Object.entries(test.query)) {
      params.append(key, value.toString());
    }

    const query = params.toString();

    return query ? `?${query}` : '';
  }

  private buildHeaders(test: APITest): Record<string, string> {
    const headers: Record<string, string> = {
      'Content-Type': 'application/json',
      'Accept': 'application/json'
    };

    if (test.headers) {
      Object.assign(headers, test.headers);
    }

    if (test.auth) {
      headers.Authorization = this.buildAuthorization(test.auth);
    }

    return headers;
  }

  private buildAuthorization(
    auth: APITestAuth
  ): string {
    switch (auth.type) {
      case 'basic':
        return `Basic ${Buffer.from(
          `${auth.username}:${auth.password}`
        ).toString('base64')}`;
      case 'bearer':
        return `Bearer ${auth.token}`;
      case 'apiKey':
        return `${auth.prefix}${auth.key}`;
      default:
        throw new Error(`Unsupported auth type: ${auth.type}`);
    }
  }

  private buildBody(test: APITest): any {
    if (!test.body) {
      return undefined;
    }

    return JSON.stringify(test.body);
  }

  private buildResponse(
    response: any
  ): APITestResponse {
    return {
      status: response.status,
      headers: response.headers,
      body: this.parseBody(response.body)
    };
  }

  private parseBody(body: any): any {
    if (!body) {
      return undefined;
    }

    try {
      return JSON.parse(body);
    } catch (error) {
      return body;
    }
  }

  private async validateResponse(
    test: APITest,
    response: APITestResponse
  ): Promise<void> {
    // Validate status
    if (test.expect?.status) {
      this.validateStatus(test.expect.status, response.status);
    }

    // Validate headers
    if (test.expect?.headers) {
      this.validateHeaders(test.expect.headers, response.headers);
    }

    // Validate body
    if (test.expect?.body) {
      await this.validateBody(test.expect.body, response.body);
    }
  }

  private validateStatus(
    expected: number | number[],
    actual: number
  ): void {
    if (Array.isArray(expected)) {
      if (!expected.includes(actual)) {
        throw new APITestError(
          `Expected status to be one of ${expected.join(
            ', '
          )}, got ${actual}`
        );
      }
    } else if (expected !== actual) {
      throw new APITestError(
        `Expected status to be ${expected}, got ${actual}`
      );
    }
  }

  private validateHeaders(
    expected: Record<string, string>,
    actual: Record<string, string>
  ): void {
    for (const [key, value] of Object.entries(expected)) {
      if (actual[key] !== value) {
        throw new APITestError(
          `Expected header ${key} to be ${value}, got ${
            actual[key]
          }`
        );
      }
    }
  }

  private async validateBody(
    expected: any,
    actual: any
  ): Promise<void> {
    if (typeof expected === 'function') {
      await expected(actual);
    } else if (typeof expected === 'object') {
      await this.validateObject(expected, actual);
    } else if (expected !== actual) {
      throw new APITestError(
        `Expected body to be ${expected}, got ${actual}`
      );
    }
  }

  private async validateObject(
    expected: Record<string, any>,
    actual: Record<string, any>
  ): Promise<void> {
    for (const [key, value] of Object.entries(expected)) {
      if (!(key in actual)) {
        throw new APITestError(
          `Expected body to have property ${key}`
        );
      }

      await this.validateBody(value, actual[key]);
    }
  }

  private recordTestMetrics(
    tests: APITest[],
    results: APITestResult[],
    startTime: number
  ): void {
    const duration = Date.now() - startTime;
    const success = results.filter((r) => r.success).length;
    const failure = results.length - success;

    this.metrics.recordMetric({
      name: 'api_test_duration_seconds',
      value: duration / 1000,
      labels: {
        environment: this.options.environment,
        service: this.options.service,
        version: this.options.version
      },
      type: 'histogram',
      timestamp: new Date()
    });

    this.metrics.recordMetric({
      name: 'api_test_total',
      value: results.length,
      labels: {
        environment: this.options.environment,
        service: this.options.service,
        version: this.options.version
      },
      type: 'counter',
      timestamp: new Date()
    });

    this.metrics.recordMetric({
      name: 'api_test_success_total',
      value: success,
      labels: {
        environment: this.options.environment,
        service: this.options.service,
        version: this.options.version
      },
      type: 'counter',
      timestamp: new Date()
    });

    this.metrics.recordMetric({
      name: 'api_test_failure_total',
      value: failure,
      labels: {
        environment: this.options.environment,
        service: this.options.service,
        version: this.options.version
      },
      type: 'counter',
      timestamp: new Date()
    });
  }

  private recordTestError(
    error: Error,
    startTime: number
  ): void {
    const duration = Date.now() - startTime;

    this.metrics.recordMetric({
      name: 'api_test_error_duration_seconds',
      value: duration / 1000,
      labels: {
        error: error.name,
        environment: this.options.environment,
        service: this.options.service,
        version: this.options.version
      },
      type: 'histogram',
      timestamp: new Date()
    });
  }

  private async logResults(
    tests: APITest[],
    results: APITestResult[]
  ): Promise<void> {
    await this.logger.logResults({
      tests,
      results,
      timestamp: new Date()
    });
  }

  private async logTest(
    test: APITest,
    response: APITestResponse
  ): Promise<void> {
    await this.logger.logTest({
      test,
      response,
      timestamp: new Date()
    });
  }

  private async logTestError(
    test: APITest,
    error: Error
  ): Promise<void> {
    await this.logger.logTestError({
      test,
      error,
      timestamp: new Date()
    });
  }
}

interface APITestServiceOptions {
  environment: string;
  service: string;
  version: string;
  baseUrl: string;
  client: APIClient;
}

interface APIClient {
  request(request: APITestRequest): Promise<any>;
}

interface APITest {
  name: string;
  method: string;
  path: string;
  query?: Record<string, any>;
  headers?: Record<string, string>;
  body?: any;
  auth?: APITestAuth;
  timeout?: number;
  expect?: APITestExpect;
}

interface APITestAuth {
  type: 'basic' | 'bearer' | 'apiKey';
  username?: string;
  password?: string;
  token?: string;
  prefix?: string;
  key?: string;
}

interface APITestExpect {
  status?: number | number[];
  headers?: Record<string, string>;
  body?: any;
}

interface APITestRequest {
  method: string;
  url: string;
  headers: Record<string, string>;
  body?: any;
  timeout?: number;
}

interface APITestResponse {
  status: number;
  headers: Record<string, string>;
  body: any;
}

interface APITestResult {
  test: APITest;
  response?: APITestResponse;
  error?: Error;
  success: boolean;
  duration: number;
  timestamp: Date;
}

class APITestError extends Error {
  constructor(message: string) {
    super(message);
    this.name = 'APITestError';
  }
}

interface APITestMetrics {
  tests: number;
  success: number;
  failure: number;
  errors: number;
}
```

## Best Practices

1. **OpenAPI Specification**
   - Define schemas
   - Document endpoints
   - Add examples
   - Validate spec
   - Version APIs

2. **API Documentation**
   - Generate docs
   - Add descriptions
   - Include examples
   - Update regularly
   - Share widely

3. **API Testing**
   - Write tests
   - Validate responses
   - Check performance
   - Monitor coverage
   - Automate runs

4. **API Validation**
   - Validate requests
   - Check responses
   - Monitor errors
   - Track metrics
   - Fix issues

## Related Documentation

- [API Guide](../api/README)
- [Testing Guide](../testing/README)
- [Documentation Guide](../documentation/README)
- [Monitoring Guide](../monitoring/README)
- [Deployment Guide](../deployment/README) 
