---
id: architecture-data-schema
---
# Database Schema Documentation

This document provides detailed information about the database schemas used across the LLM Platform AI Platform services. It includes entity relationship diagrams, schema definitions, data models, and database optimization guidelines.

## Overview

The LLM Platform AI Platform uses a polyglot persistence approach with different database technologies optimized for specific use cases:

1. **PostgreSQL**: Primary relational database for structured data
2. **Redis**: Cache and message broker for high-speed operations
3. **MongoDB**: Document storage for flexible schema data
4. **Qdrant/Pinecone**: Vector database for embeddings and similarity search
5. **Drupal Database**: MySQL/PostgreSQL for the Drupal control plane

## Core Schema Entities

### Entity Relationship Diagram

```
┌─────────────────┐       ┌───────────────────┐      ┌───────────────────┐
│                 │       │                   │      │                   │
│  Model Registry │───────┤ Model Provider    │──────┤ Model Settings    │
│                 │       │                   │      │                   │
└────────┬────────┘       └───────────────────┘      └───────────────────┘
         │                          ▲                          ▲
         │                          │                          │
         ▼                          │                          │
┌─────────────────┐                 │                          │
│                 │                 │                          │
│  Model Version  │─────────────────┘                          │
│                 │                                            │
└────────┬────────┘                                            │
         │                                                     │
         │                                                     │
         ▼                                                     │
┌─────────────────┐       ┌───────────────────┐      ┌────────┴──────────┐
│                 │       │                   │      │                   │
│  Inference Log  │───────┤  Usage Metrics    │──────┤  User Account     │
│                 │       │                   │      │                   │
└─────────────────┘       └───────────────────┘      └───────────────────┘
         ▲                          ▲                          ▲
         │                          │                          │
         │                          │                          │
┌────────┴────────┐       ┌────────┴──────────┐      ┌────────┴──────────┐
│                 │       │                   │      │                   │
│  Agent Config   │───────┤  Tool Registry    │──────┤  Tool Execution   │
│                 │       │                   │      │  Log              │
└─────────────────┘       └───────────────────┘      └───────────────────┘
         ▲                          ▲
         │                          │
         │                          │
┌────────┴────────┐       ┌────────┴──────────┐
│                 │       │                   │
│  Content        │───────┤  Template         │
│  Template       │       │  Variable         │
└─────────────────┘       └───────────────────┘
```

## PostgreSQL Schemas

### Model Registry Schema

```sql
CREATE TABLE model_registry (
  id SERIAL PRIMARY KEY,
  model_id VARCHAR(255) NOT NULL UNIQUE,
  display_name VARCHAR(255) NOT NULL,
  description TEXT,
  model_type VARCHAR(50) NOT NULL,
  capabilities JSONB,
  created_at TIMESTAMP WITH TIME ZONE DEFAULT CURRENT_TIMESTAMP,
  updated_at TIMESTAMP WITH TIME ZONE DEFAULT CURRENT_TIMESTAMP,
  is_active BOOLEAN DEFAULT TRUE
);

CREATE TABLE model_version (
  id SERIAL PRIMARY KEY,
  model_id VARCHAR(255) REFERENCES model_registry(model_id),
  version VARCHAR(50) NOT NULL,
  provider_id INTEGER REFERENCES model_provider(id),
  model_file_path VARCHAR(255),
  parameter_count BIGINT,
  context_window INTEGER,
  input_tokens_per_second INTEGER,
  output_tokens_per_second INTEGER,
  created_at TIMESTAMP WITH TIME ZONE DEFAULT CURRENT_TIMESTAMP,
  is_default BOOLEAN DEFAULT FALSE,
  UNIQUE(model_id, version)
);

CREATE TABLE model_provider (
  id SERIAL PRIMARY KEY,
  provider_name VARCHAR(255) NOT NULL UNIQUE,
  provider_type VARCHAR(50) NOT NULL,
  api_endpoint VARCHAR(255),
  auth_config JSONB,
  rate_limit_per_minute INTEGER,
  created_at TIMESTAMP WITH TIME ZONE DEFAULT CURRENT_TIMESTAMP,
  updated_at TIMESTAMP WITH TIME ZONE DEFAULT CURRENT_TIMESTAMP,
  is_active BOOLEAN DEFAULT TRUE
);

CREATE TABLE model_settings (
  id SERIAL PRIMARY KEY,
  model_id VARCHAR(255) REFERENCES model_registry(model_id),
  user_id INTEGER REFERENCES user_account(id),
  settings JSONB NOT NULL,
  created_at TIMESTAMP WITH TIME ZONE DEFAULT CURRENT_TIMESTAMP,
  updated_at TIMESTAMP WITH TIME ZONE DEFAULT CURRENT_TIMESTAMP,
  UNIQUE(model_id, user_id)
);

CREATE INDEX idx_model_registry_model_id ON model_registry(model_id);
CREATE INDEX idx_model_version_model_id ON model_version(model_id);
CREATE INDEX idx_model_settings_model_id_user_id ON model_settings(model_id, user_id);
```

### Usage Tracking Schema

```sql
CREATE TABLE inference_log (
  id SERIAL PRIMARY KEY,
  request_id UUID NOT NULL UNIQUE,
  model_id VARCHAR(255) REFERENCES model_registry(model_id),
  user_id INTEGER REFERENCES user_account(id),
  prompt_tokens INTEGER NOT NULL,
  completion_tokens INTEGER NOT NULL,
  total_tokens INTEGER NOT NULL,
  latency_ms INTEGER NOT NULL,
  started_at TIMESTAMP WITH TIME ZONE NOT NULL,
  completed_at TIMESTAMP WITH TIME ZONE NOT NULL,
  status VARCHAR(50) NOT NULL,
  error_message TEXT,
  metadata JSONB
);

CREATE TABLE usage_metrics (
  id SERIAL PRIMARY KEY,
  user_id INTEGER REFERENCES user_account(id),
  model_id VARCHAR(255) REFERENCES model_registry(model_id),
  date DATE NOT NULL,
  prompt_tokens BIGINT NOT NULL DEFAULT 0,
  completion_tokens BIGINT NOT NULL DEFAULT 0,
  total_tokens BIGINT NOT NULL DEFAULT 0,
  request_count INTEGER NOT NULL DEFAULT 0,
  average_latency_ms INTEGER,
  error_count INTEGER NOT NULL DEFAULT 0,
  cost_usd NUMERIC(10,6) NOT NULL DEFAULT 0,
  UNIQUE(user_id, model_id, date)
);

CREATE TABLE model_metrics (
  id SERIAL PRIMARY KEY,
  model_id VARCHAR(255) REFERENCES model_registry(model_id),
  date DATE NOT NULL,
  prompt_tokens BIGINT NOT NULL DEFAULT 0,
  completion_tokens BIGINT NOT NULL DEFAULT 0,
  total_tokens BIGINT NOT NULL DEFAULT 0,
  request_count INTEGER NOT NULL DEFAULT 0,
  average_latency_ms INTEGER,
  p95_latency_ms INTEGER,
  p99_latency_ms INTEGER,
  error_count INTEGER NOT NULL DEFAULT 0,
  error_rate NUMERIC(5,4) NOT NULL DEFAULT 0,
  UNIQUE(model_id, date)
);

CREATE INDEX idx_inference_log_model_id_user_id ON inference_log(model_id, user_id);
CREATE INDEX idx_inference_log_started_at ON inference_log(started_at);
CREATE INDEX idx_usage_metrics_user_id_date ON usage_metrics(user_id, date);
CREATE INDEX idx_model_metrics_model_id_date ON model_metrics(model_id, date);
```

### Tool Registry Schema

```sql
CREATE TABLE tool_registry (
  id SERIAL PRIMARY KEY,
  tool_id VARCHAR(255) NOT NULL UNIQUE,
  display_name VARCHAR(255) NOT NULL,
  description TEXT,
  version VARCHAR(50) NOT NULL,
  api_endpoint VARCHAR(255) NOT NULL,
  authentication_type VARCHAR(50) NOT NULL,
  input_schema JSONB NOT NULL,
  output_schema JSONB NOT NULL,
  capabilities JSONB,
  created_at TIMESTAMP WITH TIME ZONE DEFAULT CURRENT_TIMESTAMP,
  updated_at TIMESTAMP WITH TIME ZONE DEFAULT CURRENT_TIMESTAMP,
  is_active BOOLEAN DEFAULT TRUE
);

CREATE TABLE tool_execution_log (
  id SERIAL PRIMARY KEY,
  execution_id UUID NOT NULL UNIQUE,
  tool_id VARCHAR(255) REFERENCES tool_registry(tool_id),
  user_id INTEGER REFERENCES user_account(id),
  inference_id UUID REFERENCES inference_log(request_id),
  input_data JSONB NOT NULL,
  output_data JSONB,
  latency_ms INTEGER NOT NULL,
  started_at TIMESTAMP WITH TIME ZONE NOT NULL,
  completed_at TIMESTAMP WITH TIME ZONE NOT NULL,
  status VARCHAR(50) NOT NULL,
  error_message TEXT,
  metadata JSONB
);

CREATE TABLE tool_metrics (
  id SERIAL PRIMARY KEY,
  tool_id VARCHAR(255) REFERENCES tool_registry(tool_id),
  date DATE NOT NULL,
  execution_count INTEGER NOT NULL DEFAULT 0,
  success_count INTEGER NOT NULL DEFAULT 0,
  error_count INTEGER NOT NULL DEFAULT 0,
  average_latency_ms INTEGER,
  p95_latency_ms INTEGER,
  p99_latency_ms INTEGER,
  UNIQUE(tool_id, date)
);

CREATE INDEX idx_tool_registry_tool_id ON tool_registry(tool_id);
CREATE INDEX idx_tool_execution_log_tool_id ON tool_execution_log(tool_id);
CREATE INDEX idx_tool_execution_log_inference_id ON tool_execution_log(inference_id);
CREATE INDEX idx_tool_metrics_tool_id_date ON tool_metrics(tool_id, date);
```

### Agent Configuration Schema

```sql
CREATE TABLE agent_config (
  id SERIAL PRIMARY KEY,
  agent_id VARCHAR(255) NOT NULL UNIQUE,
  display_name VARCHAR(255) NOT NULL,
  description TEXT,
  model_id VARCHAR(255) REFERENCES model_registry(model_id),
  system_prompt TEXT,
  temperature NUMERIC(3,2),
  top_p NUMERIC(3,2),
  max_tokens INTEGER,
  tool_configuration JSONB,
  created_at TIMESTAMP WITH TIME ZONE DEFAULT CURRENT_TIMESTAMP,
  updated_at TIMESTAMP WITH TIME ZONE DEFAULT CURRENT_TIMESTAMP,
  created_by INTEGER REFERENCES user_account(id),
  is_active BOOLEAN DEFAULT TRUE
);

CREATE TABLE agent_execution_log (
  id SERIAL PRIMARY KEY,
  execution_id UUID NOT NULL UNIQUE,
  agent_id VARCHAR(255) REFERENCES agent_config(agent_id),
  user_id INTEGER REFERENCES user_account(id),
  inference_id UUID REFERENCES inference_log(request_id),
  latency_ms INTEGER NOT NULL,
  started_at TIMESTAMP WITH TIME ZONE NOT NULL,
  completed_at TIMESTAMP WITH TIME ZONE NOT NULL,
  status VARCHAR(50) NOT NULL,
  tools_used JSONB,
  error_message TEXT,
  metadata JSONB
);

CREATE INDEX idx_agent_config_agent_id ON agent_config(agent_id);
CREATE INDEX idx_agent_execution_log_agent_id ON agent_execution_log(agent_id);
CREATE INDEX idx_agent_execution_log_inference_id ON agent_execution_log(inference_id);
```

### Content Management Schema

```sql
CREATE TABLE content_template (
  id SERIAL PRIMARY KEY,
  template_id VARCHAR(255) NOT NULL UNIQUE,
  display_name VARCHAR(255) NOT NULL,
  description TEXT,
  template_content TEXT NOT NULL,
  format VARCHAR(50) NOT NULL,
  created_at TIMESTAMP WITH TIME ZONE DEFAULT CURRENT_TIMESTAMP,
  updated_at TIMESTAMP WITH TIME ZONE DEFAULT CURRENT_TIMESTAMP,
  created_by INTEGER REFERENCES user_account(id),
  is_active BOOLEAN DEFAULT TRUE
);

CREATE TABLE template_variable (
  id SERIAL PRIMARY KEY,
  template_id VARCHAR(255) REFERENCES content_template(template_id),
  variable_name VARCHAR(255) NOT NULL,
  variable_type VARCHAR(50) NOT NULL,
  description TEXT,
  is_required BOOLEAN DEFAULT FALSE,
  default_value TEXT,
  UNIQUE(template_id, variable_name)
);

CREATE TABLE document (
  id SERIAL PRIMARY KEY,
  document_id UUID NOT NULL UNIQUE,
  template_id VARCHAR(255) REFERENCES content_template(template_id),
  user_id INTEGER REFERENCES user_account(id),
  document_name VARCHAR(255) NOT NULL,
  document_content TEXT NOT NULL,
  variables_used JSONB,
  format VARCHAR(50) NOT NULL,
  created_at TIMESTAMP WITH TIME ZONE DEFAULT CURRENT_TIMESTAMP,
  updated_at TIMESTAMP WITH TIME ZONE DEFAULT CURRENT_TIMESTAMP,
  generation_settings JSONB
);

CREATE INDEX idx_content_template_template_id ON content_template(template_id);
CREATE INDEX idx_template_variable_template_id ON template_variable(template_id);
CREATE INDEX idx_document_template_id ON document(template_id);
CREATE INDEX idx_document_user_id ON document(user_id);
```

### User Account Schema

```sql
CREATE TABLE user_account (
  id SERIAL PRIMARY KEY,
  username VARCHAR(255) NOT NULL UNIQUE,
  email VARCHAR(255) NOT NULL UNIQUE,
  password_hash VARCHAR(255) NOT NULL,
  first_name VARCHAR(255),
  last_name VARCHAR(255),
  role VARCHAR(50) NOT NULL,
  created_at TIMESTAMP WITH TIME ZONE DEFAULT CURRENT_TIMESTAMP,
  updated_at TIMESTAMP WITH TIME ZONE DEFAULT CURRENT_TIMESTAMP,
  last_login TIMESTAMP WITH TIME ZONE,
  status VARCHAR(50) NOT NULL DEFAULT 'active',
  token_limit_per_month INTEGER
);

CREATE TABLE api_key (
  id SERIAL PRIMARY KEY,
  user_id INTEGER REFERENCES user_account(id),
  api_key VARCHAR(255) NOT NULL UNIQUE,
  display_name VARCHAR(255),
  expires_at TIMESTAMP WITH TIME ZONE,
  created_at TIMESTAMP WITH TIME ZONE DEFAULT CURRENT_TIMESTAMP,
  last_used TIMESTAMP WITH TIME ZONE,
  permissions JSONB,
  is_active BOOLEAN DEFAULT TRUE
);

CREATE INDEX idx_user_account_username ON user_account(username);
CREATE INDEX idx_user_account_email ON user_account(email);
CREATE INDEX idx_api_key_user_id ON api_key(user_id);
```

## MongoDB Schemas

### Vector Embeddings Schema

```javascript
// Collection: embeddings
{
  _id: ObjectId,
  content_id: String,  // Unique identifier for the content
  content_type: String,  // document, chunk, image, etc.
  embedding: Array,  // Vector embedding
  metadata: {
    source: String,
    title: String,
    author: String,
    created_at: Date,
    updated_at: Date,
    tags: Array
  },
  content: String,  // Actual text content
  user_id: String,  // Owner
  project_id: String,  // Project association
  model_id: String,  // Model used for embedding
  dimensions: Number,  // Embedding dimensions
  created_at: Date,
  updated_at: Date,
  is_active: Boolean
}

// Indexes
db.embeddings.createIndex({ content_id: 1 }, { unique: true })
db.embeddings.createIndex({ user_id: 1 })
db.embeddings.createIndex({ project_id: 1 })
db.embeddings.createIndex({ content_type: 1 })
```

### Model Cache Schema

```javascript
// Collection: model_cache
{
  _id: ObjectId,
  model_id: String,
  prompt_hash: String,  // Hash of the prompt for cache lookup
  prompt: String,
  parameters: {
    temperature: Number,
    top_p: Number,
    max_tokens: Number,
    // Other parameters
  },
  completion: String,
  tokens: {
    prompt_tokens: Number,
    completion_tokens: Number,
    total_tokens: Number
  },
  created_at: Date,
  expires_at: Date,
  hit_count: Number
}

// Indexes
db.model_cache.createIndex({ model_id: 1, prompt_hash: 1 }, { unique: true })
db.model_cache.createIndex({ expires_at: 1 }, { expireAfterSeconds: 0 })
```

### Content Storage Schema

```javascript
// Collection: content_storage
{
  _id: ObjectId,
  content_id: String,
  user_id: String,
  content_type: String,  // document, image, audio, etc.
  format: String,  // pdf, docx, png, mp3, etc.
  name: String,
  description: String,
  content: Binary,  // Binary content
  file_size: Number,
  metadata: {
    original_filename: String,
    mime_type: String,
    dimensions: {
      width: Number,
      height: Number
    },
    duration: Number,  // For audio/video
    page_count: Number,  // For documents
    tags: Array,
    custom_metadata: Object
  },
  storage_location: {
    type: String,  // local, s3, azure, etc.
    path: String
  },
  created_at: Date,
  updated_at: Date,
  is_active: Boolean
}

// Indexes
db.content_storage.createIndex({ content_id: 1 }, { unique: true })
db.content_storage.createIndex({ user_id: 1 })
db.content_storage.createIndex({ content_type: 1 })
```

## Redis Schema

### Cache Keys Structure

```
# Model Response Cache
model:{model_id}:response:{prompt_hash}
  -> JSON string of model response

# Rate Limiting
ratelimit:user:{user_id}:{endpoint}
  -> sorted set with timestamps

# Token Counters
tokens:user:{user_id}:daily:{date}
  -> integer counter
tokens:user:{user_id}:monthly:{month}
  -> integer counter

# Authentication
auth:token:{token_id}
  -> JSON string with token data
auth:refresh:{refresh_token}
  -> user_id string

# Circuit Breaker
circuit:service:{service_name}
  -> JSON string with status data
  -> { "status": "closed|open|half-open", "failures": 0, "last_failure": timestamp, "retry_at": timestamp }
```

### Pub/Sub Channels

```
# Event notifications
events:model:inference
events:tool:execution
events:user:signup
events:user:login

# Service health
health:service:{service_name}

# Configuration updates
config:update:{config_type}

# Scaling events
scaling:service:{service_name}
```

## Drupal Database Schemas

### LLM Module Schema

```sql
CREATE TABLE llm_model (
  id INT AUTO_INCREMENT PRIMARY KEY,
  uuid CHAR(36) NOT NULL UNIQUE,
  model_id VARCHAR(255) NOT NULL UNIQUE,
  label VARCHAR(255) NOT NULL,
  description TEXT,
  type VARCHAR(50) NOT NULL,
  provider VARCHAR(255) NOT NULL,
  capabilities JSON,
  status TINYINT(1) NOT NULL DEFAULT 1,
  created BIGINT NOT NULL,
  changed BIGINT NOT NULL
);

CREATE TABLE llm_agent (
  id INT AUTO_INCREMENT PRIMARY KEY,
  uuid CHAR(36) NOT NULL UNIQUE,
  agent_id VARCHAR(255) NOT NULL UNIQUE,
  label VARCHAR(255) NOT NULL,
  description TEXT,
  model_id VARCHAR(255) NOT NULL,
  system_prompt TEXT,
  parameters JSON,
  tools JSON,
  status TINYINT(1) NOT NULL DEFAULT 1,
  created BIGINT NOT NULL,
  changed BIGINT NOT NULL
);

CREATE TABLE llm_tool (
  id INT AUTO_INCREMENT PRIMARY KEY,
  uuid CHAR(36) NOT NULL UNIQUE,
  tool_id VARCHAR(255) NOT NULL UNIQUE,
  label VARCHAR(255) NOT NULL,
  description TEXT,
  endpoint VARCHAR(255) NOT NULL,
  auth_type VARCHAR(50) NOT NULL,
  input_schema JSON NOT NULL,
  output_schema JSON NOT NULL,
  status TINYINT(1) NOT NULL DEFAULT 1,
  created BIGINT NOT NULL,
  changed BIGINT NOT NULL
);

CREATE TABLE llm_settings (
  id INT AUTO_INCREMENT PRIMARY KEY,
  uuid CHAR(36) NOT NULL UNIQUE,
  name VARCHAR(255) NOT NULL UNIQUE,
  value TEXT,
  created BIGINT NOT NULL,
  changed BIGINT NOT NULL
);

CREATE INDEX idx_llm_model_model_id ON llm_model(model_id);
CREATE INDEX idx_llm_agent_agent_id ON llm_agent(agent_id);
CREATE INDEX idx_llm_tool_tool_id ON llm_tool(tool_id);
```

## Vector Database Schema (Qdrant)

```json
{
  "collections": [
    {
      "name": "document_embeddings",
      "vectors": {
        "size": 1536,
        "distance": "Cosine"
      },
      "schema": {
        "content_id": "keyword",
        "content_type": "keyword",
        "title": "text",
        "description": "text",
        "metadata": "json",
        "user_id": "keyword",
        "project_id": "keyword",
        "created_at": "datetime",
        "updated_at": "datetime"
      },
      "indexes": [
        {
          "field": "content_id",
          "field_schema": "keyword"
        },
        {
          "field": "user_id",
          "field_schema": "keyword"
        },
        {
          "field": "project_id",
          "field_schema": "keyword"
        }
      ]
    },
    {
      "name": "model_embeddings",
      "vectors": {
        "size": 1536,
        "distance": "Cosine"
      },
      "schema": {
        "model_id": "keyword",
        "version": "keyword",
        "feature_vector": "dense",
        "capabilities": "json",
        "performance_metrics": "json"
      },
      "indexes": [
        {
          "field": "model_id",
          "field_schema": "keyword"
        }
      ]
    }
  ]
}
```

## Database Optimization Guidelines

### Index Optimization

1. **Identify Query Patterns**: Analyze most frequent queries and optimize indexes for them
2. **Composite Indexes**: Use composite indexes for queries with multiple WHERE conditions
3. **Covering Indexes**: Create covering indexes that include all fields in query
4. **Index Maintenance**: Regularly rebuild indexes to prevent fragmentation
5. **Partial Indexes**: Use partial indexes for queries that filter on subsets of data

### Query Optimization

1. **Limit Column Selection**: Only select required columns
2. **Use Pagination**: Implement limit/offset or cursor pagination
3. **Join Optimization**: Ensure proper join order and types
4. **Use Prepared Statements**: Leverage query caching
5. **Avoid N+1 Query Problems**: Use eager loading where appropriate

### Connection Management

1. **Connection Pooling**: Implement connection pooling
2. **Connection Timeout**: Set appropriate connection timeouts
3. **Max Connections**: Configure max connections based on workload
4. **Monitoring**: Monitor connection usage and errors

### Data Partitioning

1. **Time-Based Partitioning**: Partition by date for time-series data
2. **Range Partitioning**: Partition large tables by value ranges
3. **Hash Partitioning**: Distribute data evenly across partitions
4. **Partition Pruning**: Ensure queries benefit from partitioning

## Backup and Recovery

### Backup Strategy

1. **Full Backups**: Daily full database backups
2. **Incremental Backups**: Hourly incremental backups
3. **Transaction Logs**: Continuous transaction log backups
4. **Cross-Region Replication**: Replicate to multiple regions

### Recovery Procedures

1. **Point-in-Time Recovery**: Ability to restore to any point in time
2. **Recovery Testing**: Regular testing of recovery procedures
3. **Automated Recovery**: Scripts for automated recovery processes
4. **Recovery Time Objectives**: Well-defined RTOs for different scenarios

## Schema Evolution Guidelines

1. **Backward Compatibility**: Maintain backward compatibility with schema changes
2. **Blue-Green Deployments**: Use blue-green deployments for schema migrations
3. **Feature Flags**: Implement feature flags for gradual schema transitions
4. **Version Control**: Track schema changes in version control
5. **Migration Testing**: Test migrations with production-like data volumes

## Database Security

1. **Encryption at Rest**: Encrypt all database files
2. **Encryption in Transit**: Use TLS for all database connections
3. **Row-Level Security**: Implement row-level security for multi-tenant data
4. **Audit Logging**: Enable comprehensive audit logging
5. **Access Control**: Implement principle of least privilege for database access

## Conclusion

The database schema design for the LLM Platform AI Platform provides a robust foundation for:

1. **Scalability**: Horizontally and vertically scalable database architecture
2. **Performance**: Optimized schemas for high-throughput AI operations
3. **Flexibility**: Support for structured, semi-structured, and vector data
4. **Security**: Comprehensive security controls at all levels
5. **Reliability**: High availability and disaster recovery capabilities

This schema design supports the current features while allowing for future expansion and optimization.
