---
id: architecture-integration
---
# Integration Architecture Guide

<!-- title: Integration Architecture Guide -->
<!-- description: Guide for integration architecture in the LLM Platform Platform -->
<!-- lastUpdated: 2024-03-19 -->

## Overview

This guide details the integration architecture of the LLM Platform Platform. It covers Drupal integration, LLM integration, and integration patterns.

## Drupal Integration

### Module Architecture

```php
// src/Plugin/ModelProvider/DrupalProvider.php
namespace Drupal\bfllm\Plugin\ModelProvider;

use Drupal\Core\Plugin\PluginBase;
use Drupal\bfllm\ModelProviderInterface;

/**
 * @ModelProvider(
 *   id = "drupal",
 *   label = @Translation("Drupal Provider"),
 *   description = @Translation("Drupal model provider")
 * )
 */
class DrupalProvider extends PluginBase implements ModelProviderInterface {
  public function getModels(): array {
    return [
      'drupal-11' => [
        'name' => 'Drupal 11',
        'type' => 'llm',
        'provider' => 'drupal',
        'version' => '11.0.0',
        'capabilities' => [
          'content_generation',
          'entity_learning',
          'pattern_analysis'
        ]
      ]
    ];
  }
  
  public function execute(
    string $modelId,
    string $input,
    array $config
  ): array {
    // Execute model
    $result = $this->modelService->execute($modelId, $input, $config);
    
    return [
      'output' => $result['output'],
      'metadata' => $result['metadata']
    ];
  }
}
```

### API Integration

```php
// src/Controller/ApiController.php
namespace Drupal\bfllm\Controller;

use Drupal\Core\Controller\ControllerBase;
use Symfony\Component\HttpFoundation\JsonResponse;

class ApiController extends ControllerBase {
  public function execute(string $modelId): JsonResponse {
    $input = $this->request->getContent();
    $config = $this->request->query->all();
    
    $result = $this->modelService->execute($modelId, $input, $config);
    
    return new JsonResponse($result);
  }
  
  public function train(string $modelId): JsonResponse {
    $data = $this->request->getContent();
    $config = $this->request->query->all();
    
    $result = $this->modelService->train($modelId, $data, $config);
    
    return new JsonResponse($result);
  }
}
```

### Field Integration

```php
// src/Plugin/Field/FieldType/AIField.php
namespace Drupal\bfllm\Plugin\Field\FieldType;

use Drupal\Core\Field\FieldItemBase;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\Core\TypedData\DataDefinition;

/**
 * @FieldType(
 *   id = "ai_field",
 *   label = @Translation("AI Field"),
 *   description = @Translation("AI-powered field type"),
 *   default_widget = "ai_widget",
 *   default_formatter = "ai_formatter"
 * )
 */
class AIField extends FieldItemBase {
  public static function propertyDefinitions(
    FieldStorageDefinitionInterface $field_definition
  ): array {
    $properties['value'] = DataDefinition::create('string')
      ->setLabel(t('Value'))
      ->setRequired(TRUE);
    
    $properties['metadata'] = DataDefinition::create('map')
      ->setLabel(t('Metadata'))
      ->setRequired(FALSE);
    
    return $properties;
  }
  
  public static function schema(
    FieldStorageDefinitionInterface $field_definition
  ): array {
    return [
      'columns' => [
        'value' => [
          'type' => 'text',
          'size' => 'big'
        ],
        'metadata' => [
          'type' => 'json'
        ]
      ]
    ];
  }
}
```

## LLM Integration

### Model Management

```typescript
// src/integration/llm/model.service.ts
@Injectable()
export class ModelService {
  constructor(
    private http: HttpClient,
    private cache: CacheService
  ) {}
  
  async list(): Promise<Model[]> {
    return this.http.get<Model[]>('/api/models').toPromise();
  }
  
  async get(id: string): Promise<Model> {
    return this.http.get<Model>(`/api/models/${id}`).toPromise();
  }
  
  async execute(
    modelId: string,
    input: string,
    config: ModelConfig
  ): Promise<ExecutionResult> {
    return this.http.post<ExecutionResult>(
      `/api/models/${modelId}/execute`,
      { input, config }
    ).toPromise();
  }
  
  async train(
    modelId: string,
    data: TrainingData,
    config: TrainingConfig
  ): Promise<TrainingResult> {
    return this.http.post<TrainingResult>(
      `/api/models/${modelId}/train`,
      { data, config }
    ).toPromise();
  }
}
```

### Provider Integration

```typescript
// src/integration/llm/provider.service.ts
@Injectable()
export class ProviderService {
  constructor(
    private http: HttpClient,
    private configService: ConfigService
  ) {}
  
  async list(): Promise<Provider[]> {
    return this.http.get<Provider[]>('/api/providers').toPromise();
  }
  
  async get(id: string): Promise<Provider> {
    return this.http.get<Provider>(`/api/providers/${id}`).toPromise();
  }
  
  async register(
    provider: ProviderConfig
  ): Promise<Provider> {
    return this.http.post<Provider>(
      '/api/providers',
      provider
    ).toPromise();
  }
  
  async update(
    id: string,
    config: ProviderConfig
  ): Promise<Provider> {
    return this.http.put<Provider>(
      `/api/providers/${id}`,
      config
    ).toPromise();
  }
}
```

### Execution Service

```typescript
// src/integration/llm/execution.service.ts
@Injectable()
export class ExecutionService {
  constructor(
    private http: HttpClient,
    private modelService: ModelService,
    private memoryService: MemoryService
  ) {}
  
  async execute(
    agentId: string,
    input: string,
    config: ExecutionConfig
  ): Promise<ExecutionResult> {
    // Get agent
    const agent = await this.getAgent(agentId);
    
    // Execute model
    const result = await this.modelService.execute(
      agent.modelId,
      input,
      config
    );
    
    // Store in memory
    await this.memoryService.store(agentId, result);
    
    return result;
  }
  
  async stream(
    agentId: string,
    input: string,
    config: ExecutionConfig
  ): Observable<ExecutionChunk> {
    return this.http.post<ExecutionChunk[]>(
      `/api/agents/${agentId}/stream`,
      { input, config }
    ).pipe(
      mergeMap(chunks => from(chunks))
    );
  }
}
```

## Integration Patterns

### Adapter Pattern

```typescript
// src/integration/patterns/adapter.ts
interface ModelAdapter {
  execute(input: string, config: any): Promise<any>;
  train(data: any, config: any): Promise<any>;
}

class DrupalAdapter implements ModelAdapter {
  async execute(input: string, config: any): Promise<any> {
    // Adapt Drupal execution
  }
  
  async train(data: any, config: any): Promise<any> {
    // Adapt Drupal training
  }
}

class LLMAdapter implements ModelAdapter {
  async execute(input: string, config: any): Promise<any> {
    // Adapt LLM execution
  }
  
  async train(data: any, config: any): Promise<any> {
    // Adapt LLM training
  }
}
```

### Bridge Pattern

```typescript
// src/integration/patterns/bridge.ts
interface ModelBridge {
  connect(): Promise<void>;
  disconnect(): Promise<void>;
  execute(input: string): Promise<any>;
}

class DrupalBridge implements ModelBridge {
  async connect(): Promise<void> {
    // Connect to Drupal
  }
  
  async disconnect(): Promise<void> {
    // Disconnect from Drupal
  }
  
  async execute(input: string): Promise<any> {
    // Execute in Drupal
  }
}

class LLMBridge implements ModelBridge {
  async connect(): Promise<void> {
    // Connect to LLM
  }
  
  async disconnect(): Promise<void> {
    // Disconnect from LLM
  }
  
  async execute(input: string): Promise<any> {
    // Execute in LLM
  }
}
```

### Facade Pattern

```typescript
// src/integration/patterns/facade.ts
class IntegrationFacade {
  constructor(
    private drupalService: DrupalService,
    private llmService: LLMService,
    private memoryService: MemoryService
  ) {}
  
  async execute(
    agentId: string,
    input: string,
    config: any
  ): Promise<any> {
    // Get agent
    const agent = await this.getAgent(agentId);
    
    // Execute based on type
    if (agent.type === 'drupal') {
      return this.drupalService.execute(agent, input, config);
    } else {
      return this.llmService.execute(agent, input, config);
    }
  }
  
  async train(
    agentId: string,
    data: any,
    config: any
  ): Promise<any> {
    // Get agent
    const agent = await this.getAgent(agentId);
    
    // Train based on type
    if (agent.type === 'drupal') {
      return this.drupalService.train(agent, data, config);
    } else {
      return this.llmService.train(agent, data, config);
    }
  }
}
```

## Best Practices

1. **Drupal Integration**
   - Use plugin system
   - Follow Drupal standards
   - Implement services
   - Use dependency injection

2. **LLM Integration**
   - Abstract providers
   - Handle errors
   - Implement caching
   - Use streaming

3. **Integration Patterns**
   - Use adapters
   - Implement bridges
   - Apply facades
   - Follow SOLID

4. **Security**
   - Validate input
   - Sanitize output
   - Handle errors
   - Implement logging

## Related Documentation

- [Architecture Overview](overview)
- [Component Architecture](components)
- [Security Architecture](security)
- [Deployment Architecture](deployment)
- [API Guide](../api/README)
- [Development Guide](../development/README)
- [Operations Guide](../operations/README) 
