# BFC Implementation Guide for Project Owners

This guide provides step-by-step instructions for implementing BFC components in your independent projects.

## Overview

BFC (BlueFly Components) provides a standardized approach to CI/CD configuration and project organization. This guide will help you implement BFC components in your project with minimal disruption.

## Prerequisites

- Access to your project repository
- GitLab account with permissions to push changes
- Basic understanding of GitLab CI/CD

## Implementation Steps

### 1. Determine Your Project Type

First, identify which project type best matches your project:

- **Node.js Project**: TypeScript/JavaScript applications and libraries
- **Drupal Module**: Drupal modules and plugins
- **ML-specific Project**: Projects with machine learning components

### 2. Add GitLab CI Configuration

#### For Node.js Projects

1. Create or update `.gitlab-ci.yml` in your project root:

```yaml
# .gitlab-ci.yml for Node.js projects
include:
  - component: gitlab.bluefly.io/llm/bfcicomponents/components/nodejs/npm_golden_ci_package@main
    inputs:
      node_version: "20"
      npm_token: "$NPM_TOKEN"
      min_test_coverage: 90
      package_manager: "npm"  # or "yarn", "pnpm"
```

2. Customize the inputs as needed for your project.

#### For Drupal Module Projects

1. Create or update `.gitlab-ci.yml` in your project root:

```yaml
# .gitlab-ci.yml for Drupal modules
include:
  - component: gitlab.bluefly.io/llm/bfcicomponents/components/drupal/drupal_golden_ci_module@main
    inputs:
      php_version: "8.3"
      drupal_version_constraint: "^11"
      module_name: "your_module_name"
      min_test_coverage: 90
```

2. Replace `your_module_name` with your actual module name.
3. Adjust other inputs as needed.

#### For ML-specific Projects

1. Create or update `.gitlab-ci.yml` in your project root:

```yaml
# .gitlab-ci.yml for ML projects
include:
  - component: gitlab.bluefly.io/llm/bfcicomponents/components/nodejs/ml_node_package@main
    inputs:
      node_version: "20"
      npm_token: "$NPM_TOKEN"
      min_test_coverage: 85
      ml_framework: "tensorflow"  # or "pytorch", "huggingface", etc.
```

2. Adjust the inputs for your specific ML framework and requirements.

### 3. Add CLAUDE.md Configuration

Adding a CLAUDE.md file helps Claude Code understand your project structure and development patterns.

1. Copy the template that matches your project type:
   - Node.js: `/components/bfc/templates/nodejs/CLAUDE.md`
   - Drupal: `/components/bfc/templates/drupal/CLAUDE.md`
   - ML: `/components/bfc/templates/ml/CLAUDE.md`

2. Customize the CLAUDE.md file with your project-specific information.

### 4. Test the Implementation

1. Commit and push the changes to a feature branch:
```bash
git checkout -b feature/bfc-implementation
git add .gitlab-ci.yml CLAUDE.md
git commit -m "Implement BFC components"
git push -u origin feature/bfc-implementation
```

2. Create a merge request to verify the CI/CD pipeline runs successfully.

3. Review the pipeline results and make any necessary adjustments.

### 5. Complete the Implementation

Once the pipeline passes successfully:

1. Merge the changes into your main branch
2. Document the BFC implementation in your project README.md

## Customization Options

### Extending the Pipeline

You can add custom jobs to your `.gitlab-ci.yml` while still using the BFC components:

```yaml
# Include BFC component
include:
  - component: gitlab.bluefly.io/llm/bfcicomponents/components/nodejs/npm_golden_ci_package@main
    inputs:
      node_version: "20"
      # other inputs...

# Add custom job
custom_job:
  stage: test
  image: node:20-alpine
  script:
    - npm ci
    - npm run custom-script
  rules:
    - if: $CI_COMMIT_BRANCH == "main"
```

### Disabling Features

You can disable certain features by setting their inputs to `false`:

```yaml
include:
  - component: gitlab.bluefly.io/llm/bfcicomponents/components/nodejs/npm_golden_ci_package@main
    inputs:
      node_version: "20"
      enable_security_scan: "false"
      enable_api_docs: "false"
      # other inputs...
```

## Troubleshooting

### Common Issues

1. **Pipeline fails with "component not found"**:
   - Ensure the component path is correct
   - Check that you have access to the bfcicomponents repository

2. **Tests fail in the pipeline**:
   - Review test failures and fix issues in your code
   - Adjust test coverage requirements if necessary

3. **Missing dependencies**:
   - Ensure your project has all required dependencies
   - Check that package.json or composer.json is properly configured

### Getting Help

If you encounter issues implementing BFC components:

1. Check the [BFC documentation](/docs/BFC_STRUCTURE.md)
2. Review [Golden Components Documentation](/docs/GOLDEN_COMPONENTS.md)
3. Contact the BFC team for assistance

## Maintenance

Once you've implemented BFC components, maintain them by:

1. Periodically updating to the latest component versions
2. Following the standard practices outlined in the components
3. Ensuring new features adhere to the established patterns

## Conclusion

Implementing BFC components standardizes your CI/CD configuration and improves consistency across projects. By following this guide, you've taken an important step toward establishing enterprise-grade CI/CD practices in your project.