# BFCIComponents - Component Index

This document provides a comprehensive index of all the components available in the BFCIComponents repository. Components are organized by category and provide reusable CI/CD configurations for various project types.

## Quick Links

- [Configuration Components](#configuration-components)
- [Kubernetes Components](#kubernetes-components)
- [llmcli Components](#llmcli-components)
- [LLM Components](#llm-components)
- [GitLab Utilities](#gitlab-utilities)
- [Code Quality Components](#code-quality-components)
- [Golden Components](#golden-components)
- [Project Templates](#project-templates)

## Configuration Components

Components for standardizing project configurations.

| Component | Description | Inputs |
|-----------|-------------|--------|
| [eslint-config](../components/ci/eslint-config) | ESLint configuration utilities with support for both flat and legacy formats, TypeScript, React, and Jest integration. | `project_type`, `environment`, `config_format`, `typescript`, `react`, `jest` |
| [jest-config](../components/ci/jest-config) | Jest configuration utilities with multiple profiles, customizable coverage thresholds, and reporting options. | `project_type`, `config_profile`, `coverage_threshold`, `coverage_reporting`, `test_path`, `setup_file` |

## Kubernetes Components

Components for Kubernetes deployments and configurations.

| Component | Description | Inputs |
|-----------|-------------|--------|
| [k8s-base](../components/k8s/base) | Base Kubernetes resources including namespace definitions and configmaps with parameterized configuration. | `environment`, `namespace`, `version`, `node_env`, etc. |
| [k8s-llm-gateway](../components/k8s/llm-gateway) | Kubernetes deployment for llm-gateway service with configurable resources and monitoring. | `environment`, `namespace`, `replicas`, `image_tag`, etc. |
| [llmcli](../components/k8s/llmcli) | Kubernetes resources for the CLI service | Various deployment parameters |
| [bfllm](../components/k8s/bfllm) | Kubernetes resources for the LLM service | Various deployment parameters |
| [llm-mcp](../components/k8s/llm-mcp) | Kubernetes resources for the Model Control Plane | Various deployment parameters |
| [bfrfp](../components/k8s/bfrfp) | Kubernetes resources for the RFP service | Various deployment parameters |
| [bfui](../components/k8s/bfui) | Kubernetes resources for the UI service | Various deployment parameters |

## llmcli Components

Components for llmcli integration and automation.

| Component | Description | Inputs |
|-----------|-------------|--------|
| [llmcli-automation](../components/llmcli/automation) | llmcli automation services for test generation, component adaptation, and performance optimization. | `project_type`, `environment`, `ai_provider`, etc. |
| [llmcli-project-templates](../components/llmcli/project-templates) | Project templates for llmcli project setup commands, providing ready-to-use GitLab CI/CD configurations. | `project_type`, `output_path` |

## LLM Components

Components for Large Language Model integration and services.

| Component | Description | Inputs |
|-----------|-------------|--------|
| [llm-platform-integration](../components/llm/platform-integration) | LLM Platform integration services for recipe validation, deployment, and monitoring. | `environment`, `recipe_path`, `ai_providers`, etc. |

## GitLab Utilities

Components for GitLab CI/CD integration and utilities.

| Component | Description | Inputs |
|-----------|-------------|--------|
| [gitlab-ci-utilities](../components/gitlab/ci-utilities) | PHP utility classes for working with GitLab CI configurations programmatically. | `mode`, `composer_namespace`, `php_version` |

## API Components

Components for API services and validation.

| Component | Description | Path |
|-----------|-------------|------|
| [api-service](../components/api/api-service) | API Service template for microservices | `components/api/api-service` |
| [llmcli-api-service](../components/api/llmcli-api-service) | llmcli Enhanced API Service | `components/api/llmcli-api-service` |
| [openapi-validation](../components/api/openapi-validation) | OpenAPI schema validation | `components/api/openapi-validation` |

## CI Automation Components

| Component | Description | Path |
|-----------|-------------|------|
| [ai-test-recommendations](../components/ci/ai-test-recommendations) | AI-powered test recommendations | `components/ci/ai-test-recommendations` |
| [ci-automation](../components/ci/ci-automation) | CI/CD automation tools and utilities | `components/ci/ci-automation` |
| [ci-config](../components/ci/ci-config) | CI configuration management utilities | `components/ci/ci-config` |
| [drupal-ai-integration](../components/ci/drupal-ai-integration) | Drupal AI integration tooling | `components/ci/drupal-ai-integration` |
| [drupal-tdd-scaffolding](../components/ci/drupal-tdd-scaffolding) | Drupal TDD scaffolding generators | `components/ci/drupal-tdd-scaffolding` |
| [tdd-dashboard](../components/ci/tdd-dashboard) | TDD dashboard and visualization tools | `components/ci/tdd-dashboard` |
| [template-management](../components/ci/template-management) | GitLab CI template management tools | `components/ci/template-management` |

## Code Quality Components

Components for code quality analysis and reporting.

| Component | Description | Inputs |
|-----------|-------------|--------|
| [code-quality](../components/code-quality) | Comprehensive code quality analysis with integrated TODO management, static analysis, and quality reporting. | `code_quality_fail_on_error`, `code_quality_eslint_enabled`, etc. |
| [todo-aggregator](../components/todo-aggregator) | Aggregates and analyzes TODOs from your codebase with priority detection, visualization, and CI integration. | `todo_max_critical`, `todo_max_high`, `todo_max_per_file`, etc. |

## Monitoring Components

Components for monitoring and observability.

| Component | Description | Path |
|-----------|-------------|------|
| [alertmanager](../components/monitoring/alertmanager) | Alert management and notification system | `components/monitoring/alertmanager` |
| [grafana](../components/monitoring/grafana) | Grafana dashboards and visualization | `components/monitoring/grafana` |
| [prometheus](../components/monitoring/prometheus) | Prometheus monitoring configuration | `components/monitoring/prometheus` |
| [service-monitors](../components/monitoring/service-monitors) | Service monitor configurations | `components/monitoring/service-monitors` |
| [slo](../components/monitoring/slo) | Service Level Objectives configuration | `components/monitoring/slo` |
| [tracing](../components/monitoring/tracing) | Distributed tracing with Jaeger | `components/monitoring/tracing` |

## Node.js Components

Components for Node.js projects.

| Component | Description | Path |
|-----------|-------------|------|
| [library](../components/nodejs/library) | Node.js library CI/CD with NPM publishing | `components/nodejs/library` |
| [ml_node_package](../components/nodejs/ml_node_package) | ML-specific Node.js package CI/CD | `components/nodejs/ml_node_package` |
| [nodejs](../components/nodejs/nodejs) | Node.js application CI/CD pipeline | `components/nodejs/nodejs` |
| [nodejs-enhanced](../components/nodejs/nodejs-enhanced) | Enhanced Node.js CI/CD pipeline | `components/nodejs/nodejs-enhanced` |
| [nodejs-npm](../components/nodejs/nodejs-npm) | Node.js NPM package CI/CD | `components/nodejs/nodejs-npm` |
| [nodejs-npm-ci](../components/nodejs/nodejs-npm-ci) | Node.js NPM CI pipeline | `components/nodejs/nodejs-npm-ci` |
| [npm_golden_ci_package](../components/nodejs/npm_golden_ci_package) | Golden CI pipeline for NPM packages | `components/nodejs/npm_golden_ci_package` |

## Drupal Components

Components for Drupal projects.

| Component | Description | Path |
|-----------|-------------|------|
| [ai-platform](../components/drupal/ai-platform) | Drupal AI Platform deployment | `components/drupal/ai-platform` |
| [drupal-lint](../components/drupal/drupal-lint) | Drupal code linting and quality checks | `components/drupal/drupal-lint` |
| [drupal-module-ci](../components/drupal/drupal-module-ci) | Drupal module CI pipeline | `components/drupal/drupal-module-ci` |
| [drupal-recipe-tester](../components/drupal/drupal-recipe-tester) | Drupal recipe testing framework | `components/drupal/drupal-recipe-tester` |
| [drupal-test-unit](../components/drupal/drupal-test-unit) | Drupal unit testing with PHPUnit | `components/drupal/drupal-test-unit` |
| [drupal-tdd](../components/drupal/drupal-tdd) | Drupal Test-Driven Development workflow | `components/drupal/drupal-tdd` |

## Golden Components

High-level components that combine multiple specialized components for complete CI/CD pipelines.

| Component | Description | Included Components |
|-----------|-------------|---------------------|
| [drupal_golden_ci_module](../components/drupal/drupal_golden_ci_module) | Golden Recipe CI/CD component for custom Drupal module development. | drupal-lint, drupal-test-unit, drupal-test-kernel, drupal-test-functional, drupal-packagist, security |
| [drupal_golden_ci_recipe](../components/drupal/drupal_golden_ci_recipe) | Golden Recipe CI/CD component for comprehensive Drupal project pipelines. | drupal-lint, drupal-test-unit, drupal-recipe-tester, security |
| [drupal_golden_ci_site](../components/drupal/drupal_golden_ci_site) | All-encompassing Golden Recipe CI/CD component for complete Drupal site projects. | drupal-lint, drupal-e2e-testing, drupal-security, drupal-performance, security |
| [npm_golden_ci_package](../components/nodejs/npm_golden_ci_package) | Golden Recipe CI/CD component for npm package development. | eslint-config, jest-config, nodejs-npm-ci, security, semantic-release |

## Project Templates

Templates for quickly setting up CI/CD pipelines for different project types.

| Template | Description | Location |
|----------|-------------|----------|
| [drupal-golden-module-template](../components/llmcli/project-templates/templates/drupal-golden-module-template.gitlab-ci.yml) | Template for Drupal modules | `components/llmcli/project-templates/templates/` |
| [drupal-golden-recipe-template](../components/llmcli/project-templates/templates/drupal-golden-recipe-template.gitlab-ci.yml) | Template for Drupal recipes | `components/llmcli/project-templates/templates/` |
| [drupal-golden-site-template](../components/llmcli/project-templates/templates/drupal-golden-site-template.gitlab-ci.yml) | Template for Drupal sites | `components/llmcli/project-templates/templates/` |
| [nodejs-golden-package-template](../components/llmcli/project-templates/templates/nodejs-golden-package-template.gitlab-ci.yml) | Template for Node.js packages | `components/llmcli/project-templates/templates/` |

## Usage Examples

### Basic Component Usage

```yaml
include:
  - component: gitlab.bluefly.io/llm/bfcicomponents/components/ci/eslint-config@main
    inputs:
      config_format: flat
      typescript: true
```

### Golden Component Usage

```yaml
include:
  - component: gitlab.bluefly.io/llm/bfcicomponents/components/drupal/drupal_golden_ci_module@main
    inputs:
      php_version: "8.1"
      module_name: "my_module"
```

### Multiple Component Usage

```yaml
include:
  - component: gitlab.bluefly.io/llm/bfcicomponents/components/ci/eslint-config@main
    inputs:
      config_format: flat
      typescript: true
  - component: gitlab.bluefly.io/llm/bfcicomponents/components/ci/jest-config@main
    inputs:
      config_profile: typescript
      coverage_threshold: 85
```

## Component Development

For information on developing new components or contributing to existing ones, see [CONTRIBUTING.md](./CONTRIBUTING.md).