# Deployment Workflow Guide

This document outlines the standardized deployment workflow for the LLM Platform, including npm packages, Drupal modules, and AI model registration.

## Table of Contents

- [Overview](#overview)
- [GitLab CI Components](#gitlab-ci-components)
- [Drupal Module Deployment](#drupal-module-deployment)
- [NPM Package Publishing](#npm-package-publishing)
- [AI Model Registry](#ai-model-registry)
- [Container Registry](#container-registry)
- [Security Considerations](#security-considerations)
- [CI/CD Variables](#cicd-variables)
- [Troubleshooting](#troubleshooting)

## Overview

The LLM Platform uses a standardized deployment workflow based on GitLab CI/CD components to ensure consistent, reliable, and secure deployments across all projects. The workflow includes:

1. **Test-Driven Development (TDD)** - All code must pass comprehensive tests before deployment
2. **Semantic Versioning** - Automated version increments based on commit messages
3. **Package Registry Integration** - Automated publishing to Packagist and npm
4. **Model Registry** - AI model version tracking and deployment
5. **Container Registry** - Docker container builds and deployment

## GitLab CI Components

The following GitLab CI components are available for deployment:

| Component | Path | Purpose |
|-----------|------|---------|
| `drupal-packagist` | `/components/drupal/drupal-packagist` | Drupal module publishing to Packagist |
| `nodejs-npm` | `/components/nodejs/nodejs-npm` | npm package publishing |
| `drupal-tdd` | `/components/drupal/drupal-tdd` | Test-driven development for Drupal |
| `nodejs` | `/components/nodejs/nodejs` | Node.js application workflow |
| `docker` | `/components/shared/docker` | Docker container builds |

## Drupal Module Deployment

### Setup

1. Create a `.gitlab-ci.yml` file in your Drupal module directory:

```yaml
include:
  - project: 'llm/bfcicomponents'
    file: 'templates/drupal-module-ci.yml'

variables:
  MODULE_NAME: "your_module_name"
  DRUPAL_VERSION: "11.x"
  PHP_VERSION: "8.3"
```

2. Configure your `composer.json` file:

```json
{
  "name": "drupal/your_module_name",
  "description": "Your module description",
  "type": "drupal-module",
  "license": "GPL-2.0-or-later",
  "version": "1.0.0",
  ...
}
```

3. Set up GitLab CI/CD variables:
   - `PACKAGIST_TOKEN` - Your Packagist API token

### Deployment Process

1. Write tests following TDD methodology
2. Commit changes using conventional commit format:
   - `feat: add new feature` (minor version bump)
   - `fix: resolve issue` (patch version bump)
   - `docs: update documentation` (no version bump)
3. Push to your release branch (default: `main`)
4. GitLab CI will automatically:
   - Run tests
   - Generate documentation
   - Create a semantic release
   - Publish to Packagist
   - Register AI models (if applicable)
   - Build Docker container (if applicable)

## NPM Package Publishing

### Setup

1. Create a `.gitlab-ci.yml` file in your npm package directory:

```yaml
include:
  - project: 'llm/bfcicomponents'
    file: 'templates/nodejs-npm-ci.yml'

variables:
  PACKAGE_NAME: "your-package-name"
  NODE_VERSION: "20"
  PACKAGE_MANAGER: "npm"
```

2. Configure your `package.json` file:

```json
{
  "name": "@bluefly/your-package-name",
  "version": "1.0.0",
  "description": "Your package description",
  "main": "dist/index.js",
  "types": "dist/index.d.ts",
  ...
}
```

3. Set up GitLab CI/CD variables:
   - `NPM_TOKEN` - Your npm registry token

### Deployment Process

1. Write tests following TDD methodology
2. Commit changes using conventional commit format
3. Push to your release branch (default: `main`)
4. GitLab CI will automatically:
   - Run tests
   - Analyze bundle size
   - Create a semantic release
   - Publish to npm
   - Publish to GitLab Package Registry

## AI Model Registry

The platform supports registering AI models in the GitLab Package Registry:

1. Create a `models` directory in your module or package
2. Add model definition files in YAML format:

```yaml
name: your-model-name
version: 1.0.0
type: language-model
description: "Model description"
framework: "custom"
task: "text-generation"
metrics:
  accuracy: 0.92
  latency_ms: 150
parameters:
  parameter1: value1
  parameter2: value2
```

3. The CI/CD pipeline will automatically register models when you push to the release branch

## Container Registry

For modules or packages that include a Dockerfile, containers are automatically built and pushed to the GitLab Container Registry:

1. Add a `Dockerfile` to your project
2. Containers will be tagged with:
   - Latest version: `registry.gitlab.bluefly..io/your-project:latest`
   - Specific version: `registry.gitlab.bluefly..io/your-project:1.0.0`

## Security Considerations

All deployments include security scanning:

- **Drupal Modules**: PHP Security Checker, PHPCS Security Audit
- **NPM Packages**: npm audit, Snyk vulnerability scanning
- **Docker Containers**: Container scanning

## CI/CD Variables

Configure these variables in GitLab CI/CD settings:

| Variable | Description | Required By |
|----------|-------------|------------|
| `PACKAGIST_TOKEN` | Packagist API token | Drupal modules |
| `NPM_TOKEN` | npm registry token | npm packages |
| `COMPOSER_AUTH` | Composer authentication JSON | Drupal modules with private dependencies |
| `DOCKER_REGISTRY_USER` | Docker registry username | Container builds |
| `DOCKER_REGISTRY_PASSWORD` | Docker registry password | Container builds |

## Troubleshooting

### Common Issues

1. **Packagist Publishing Fails**
   - Verify your Packagist token is correct
   - Ensure your package name is unique
   - Check that your composer.json has all required fields

2. **npm Publishing Fails**
   - Verify your npm token is correct
   - Check package scope permissions
   - Ensure version hasn't already been published

3. **Tests Fail**
   - Follow TDD approach to fix failing tests
   - Ensure test coverage meets thresholds
   - Run tests locally before pushing

4. **Model Registration Fails**
   - Verify model YAML format
   - Check GitLab permissions for package registry
   - Ensure model name and version are unique

### Support Resources

- GitLab CI/CD Documentation: [GitLab CI/CD](https://docs.gitlab.bluefly..io/ee/ci/)
- Project Repositories: [LLM Platform GitLab](https://gitlab.bluefly.io/llm)
- Internal Documentation: [bfdocs](https://gitlab.bluefly.io/llm/bfdocs)

---

**Version**: 1.0.0  
**Last Updated**: May 2025  
**Author**: Bluefly LLM Platform Engineering