# Independent Project Standardization Guide

This document provides guidance for standardizing independent projects to use BFC components without requiring shared directories or project grouping.

## Project Types

Our ecosystem includes these independent project types:

1. **Main Service Projects**
   - `llm-gateway` - API Gateway service
   - `llm-gatewayclient` - API Client library
   - `bfcicomponents` - CI/CD Component catalog
   - `llmcli` - CLI utility
   - `bfdocs` - Documentation system
   - `bfllm` - LLM service
   - `llm-mcp` - Model Control Plane
   - `bfrfp` - Request For Proposal service
   - `bfui` - User Interface components
   - `llm_platform_recipe` - Drupal platform recipe

2. **NPM Projects**
   - `native-ai` - TypeScript AI framework
   - `secure_project` - Security tooling
   - `reelcode` - Code standards validation

3. **Drupal Module Projects**
   - `ai_swarm_intelligence` - AI swarm functionality
   - `drupal_native_ai` - Native AI integration
   - `llm` - LLM integration
   - `secure_drupal` - Security module

## Standardization Approach

Since these are independent projects, our standardization will focus on:

1. **CI/CD Configuration**: Including BFC components
2. **Development Standards**: Documentation and code practices
3. **Project-specific AI Configuration**: CLAUDE.md files

## Implementation Guidelines

### 1. GitLab CI/CD Configuration

Each project should include a `.gitlab-ci.yml` file that references the appropriate BFC golden component:

#### Node.js Project Example:

```yaml
# .gitlab-ci.yml for llm-gateway
include:
  - component: gitlab.bluefly.io/llm/bfcicomponents/components/nodejs/npm_golden_ci_package@main
    inputs:
      node_version: "20"
      npm_token: "$NPM_TOKEN"
      min_test_coverage: 90
      package_manager: "npm"
```

#### Drupal Module Example:

```yaml
# .gitlab-ci.yml for ai_swarm_intelligence
include:
  - component: gitlab.bluefly.io/llm/bfcicomponents/components/drupal/drupal_golden_ci_module@main
    inputs:
      php_version: "8.2"
      drupal_version_constraint: "^10"
      module_name: "ai_swarm_intelligence"
      min_test_coverage: 90
```

#### ML-specific Project Example:

```yaml
# .gitlab-ci.yml for bfllm
include:
  - component: gitlab.bluefly.io/llm/bfcicomponents/components/nodejs/ml_node_package@main
    inputs:
      node_version: "20"
      npm_token: "$NPM_TOKEN"
      min_test_coverage: 85
      ml_framework: "tensorflow"
```

### 2. CLAUDE.md Standardization

Each project should include a `CLAUDE.md` file in its root directory with:

1. **Project-specific information**
2. **Standard development practices**
3. **Environment configuration**
4. **Common commands**

#### Example CLAUDE.md Template:

```markdown
# CLAUDE.md for [Project Name]

This document provides guidance to Claude Code (claude.ai/code) when working with code in this repository.

## Project Overview

[Brief description of the project]

## Development Environment

- Node.js version: 20.x
- Package manager: npm
- Main technologies: [List key technologies]
- Testing framework: Jest
- Documentation standard: JSDoc

## Common Commands

```bash
# Installation
npm install

# Development
npm run dev

# Testing
npm run test
npm run test:coverage

# Linting and Formatting
npm run lint
npm run format

# Building
npm run build
```

## Project Structure

- `/src` - Source code
- `/test` - Test files
- `/docs` - Documentation
- `/config` - Configuration

## CI/CD Pipeline

This project uses BFC components from the GitLab CI/CD Component Catalog:

```yaml
include:
  - component: gitlab.bluefly.io/llm/bfcicomponents/components/nodejs/npm_golden_ci_package@main
```
```

### 3. Documentation Standards

Each project should maintain:

1. **README.md** - Project overview and getting started guide
2. **CONTRIBUTING.md** - Contribution guidelines
3. **docs/** directory - Detailed documentation

## Project-Specific Implementation Guidance

### Node.js Projects

For projects like `llm-gateway`, `llmcli`, etc.:

1. Update `.gitlab-ci.yml` to use `npm_golden_ci_package`
2. Add standardized `CLAUDE.md`
3. Ensure package.json has standard scripts:
   - test
   - lint
   - build
   - format

### Drupal Module Projects

For projects like `ai_swarm_intelligence`, `drupal_native_ai`, etc.:

1. Update `.gitlab-ci.yml` to use `drupal_golden_ci_module`
2. Add standardized `CLAUDE.md`
3. Ensure PHPUnit configuration exists
4. Ensure composer.json is properly configured

### ML-specific Projects

For projects like `bfllm`:

1. Update `.gitlab-ci.yml` to use `ml_node_package`
2. Add standardized `CLAUDE.md`
3. Ensure ML-specific configuration is documented

## Implementation Process

Project owners should:

1. **Review** the appropriate template files
2. **Adapt** them to their project's specific needs
3. **Commit** the updated files to their project
4. **Validate** with a CI/CD run

## Templates

Standard template files are available in:
`./bfcicomponents/templates/standardization/`

## Benefits

By standardizing independent projects:

1. **Consistency** across the platform
2. **Reduced maintenance** through shared components
3. **Improved quality** through standard practices
4. **Better discoverability** for developers
5. **Simplified onboarding** for new team members

## Support

For assistance with implementation, contact the BFC team or refer to the [BFC documentation](docs/BFC_STRUCTURE.md).