# GitLab CI/CD Components Library Reorganization Plan

## Overview

This plan reorganizes the bfcicomponents directory to follow GitLab CI/CD component best practices, creating a world-class component library focused on Drupal 11, TypeScript, Python, AI, and NPM packages.

## New Structure

```
bfcicomponents/
├── .gitlab-ci.yml                 # Component catalog definition
├── README.md                      # Library documentation
├── CONTRIBUTING.md                # Contribution guidelines
├── LICENSE                        # License file
├── CHANGELOG.md                   # Version history
│
├── components/                    # All CI/CD components
│   ├── README.md                 # Component overview
│   │
│   ├── drupal/                   # Drupal-specific components
│   │   ├── build/
│   │   │   └── template.yml      # Drupal build component
│   │   ├── test/
│   │   │   └── template.yml      # Drupal test component
│   │   ├── deploy/
│   │   │   └── template.yml      # Drupal deployment
│   │   ├── security/
│   │   │   └── template.yml      # Drupal security scanning
│   │   ├── quality/
│   │   │   └── template.yml      # Drupal code quality
│   │   └── module/
│   │       └── template.yml      # Drupal module CI/CD
│   │
│   ├── nodejs/                   # Node.js/TypeScript components
│   │   ├── build/
│   │   │   └── template.yml      # Node.js build
│   │   ├── test/
│   │   │   └── template.yml      # Jest/Vitest testing
│   │   ├── publish/
│   │   │   └── template.yml      # NPM publishing
│   │   ├── lint/
│   │   │   └── template.yml      # ESLint/Prettier
│   │   └── security/
│   │       └── template.yml      # npm audit
│   │
│   ├── python/                   # Python components
│   │   ├── build/
│   │   │   └── template.yml      # Python build
│   │   ├── test/
│   │   │   └── template.yml      # pytest
│   │   ├── lint/
│   │   │   └── template.yml      # flake8/black
│   │   └── publish/
│   │       └── template.yml      # PyPI publishing
│   │
│   ├── ai/                       # AI/ML components
│   │   ├── ollama/
│   │   │   └── template.yml      # Ollama integration
│   │   ├── langchain/
│   │   │   └── template.yml      # LangChain pipelines
│   │   ├── model-training/
│   │   │   └── template.yml      # Model training
│   │   └── documentation/
│   │       └── template.yml      # AI doc generation
│   │
│   ├── docker/                   # Container components
│   │   ├── build/
│   │   │   └── template.yml      # Docker build
│   │   ├── scan/
│   │   │   └── template.yml      # Security scanning
│   │   └── publish/
│   │       └── template.yml      # Registry push
│   │
│   ├── kubernetes/               # K8s components
│   │   ├── deploy/
│   │   │   └── template.yml      # K8s deployment
│   │   ├── helm/
│   │   │   └── template.yml      # Helm charts
│   │   └── validate/
│   │       └── template.yml      # Manifest validation
│   │
│   └── shared/                   # Shared base components
│       ├── setup/
│       │   └── template.yml      # Common setup
│       ├── cache/
│       │   └── template.yml      # Caching strategies
│       └── artifacts/
│           └── template.yml      # Artifact handling
│
├── tests/                        # Component tests
│   ├── unit/                     # Unit tests
│   ├── integration/              # Integration tests
│   └── fixtures/                 # Test fixtures
│
├── docs/                         # Documentation
│   ├── components/               # Component docs
│   ├── examples/                 # Usage examples
│   ├── guides/                   # How-to guides
│   └── api/                      # API reference
│
├── tools/                        # Development tools
│   ├── validate-components.js    # Component validator
│   ├── generate-catalog.js       # Catalog generator
│   └── create-component.js       # Component creator
│
└── examples/                     # Example projects
    ├── drupal-module/            # Example Drupal module
    ├── nodejs-package/           # Example Node.js package
    └── python-package/           # Example Python package
```

## Component Template Structure

Each component follows this structure:

```yaml
# components/[category]/[component]/template.yml
spec:
  inputs:
    # Comprehensive input definitions with defaults, validation, and descriptions
    stage:
      type: string
      default: 'test'
      description: 'Pipeline stage name'
      options: ['build', 'test', 'deploy']
    
    node_version:
      type: string
      default: '20'
      description: 'Node.js version'
      pattern: '^\d+(\.\d+)?$'
    
    environment:
      type: string
      default: 'production'
      description: 'Deployment environment'
      options: ['development', 'staging', 'production']
    
    enable_cache:
      type: boolean
      default: true
      description: 'Enable dependency caching'

---
# Component implementation
.component-name:
  stage: $[[ inputs.stage ]]
  image: node:$[[ inputs.node_version ]]
  variables:
    NODE_ENV: $[[ inputs.environment ]]
  cache:
    - if: $[[ inputs.enable_cache ]]
      key: "[CI_COMMIT_REF_SLUG]"
      paths:
        - node_modules/
  script:
    - echo "Running component"
```

## Key Components to Create

### 1. Drupal Components

#### drupal/build
```yaml
spec:
  inputs:
    php_version:
      default: '8.3'
    composer_version:
      default: '2'
    drupal_version:
      default: '11'
    enable_patches:
      default: true
    custom_repositories:
      type: array
      default: []
```

#### drupal/test
```yaml
spec:
  inputs:
    test_suite:
      default: 'all'
      options: ['unit', 'kernel', 'functional', 'javascript', 'all']
    coverage:
      default: true
    phpunit_version:
      default: '10'
    browser:
      default: 'chrome'
      options: ['chrome', 'firefox']
```

#### drupal/deploy
```yaml
spec:
  inputs:
    deploy_strategy:
      default: 'rolling'
      options: ['rolling', 'blue-green', 'canary']
    drush_commands:
      type: array
      default: ['updb', 'cim', 'cr']
    backup_database:
      default: true
```

### 2. Node.js/TypeScript Components

#### nodejs/build
```yaml
spec:
  inputs:
    package_manager:
      default: 'npm'
      options: ['npm', 'yarn', 'pnpm']
    typescript:
      default: true
    build_command:
      default: 'build'
    output_directory:
      default: 'dist'
```

#### nodejs/publish
```yaml
spec:
  inputs:
    registry:
      default: 'npmjs'
      options: ['npmjs', 'github', 'gitlab', 'custom']
    access:
      default: 'public'
      options: ['public', 'restricted']
    tag:
      default: 'latest'
    dry_run:
      default: false
```

### 3. AI Components

#### ai/ollama
```yaml
spec:
  inputs:
    model:
      default: 'mixtral:latest'
    pull_model:
      default: true
    base_url:
      default: 'http://localhost:11434'
    context_length:
      default: 4096
```

#### ai/documentation
```yaml
spec:
  inputs:
    provider:
      default: 'ollama'
      options: ['ollama', 'openai', 'anthropic']
    mode:
      default: 'enhance'
      options: ['create', 'enhance', 'review', 'translate']
    languages:
      type: array
      default: ['en']
    output_format:
      default: 'markdown'
      options: ['markdown', 'html', 'pdf']
```

### 4. Docker/Container Components

#### docker/build
```yaml
spec:
  inputs:
    dockerfile:
      default: 'Dockerfile'
    context:
      default: '.'
    platforms:
      type: array
      default: ['linux/amd64', 'linux/arm64']
    push:
      default: true
    cache_from:
      type: array
      default: []
```

### 5. Package Publishing Components

#### packagist/publish
```yaml
spec:
  inputs:
    composer_auth:
      required: true
      secret: true
    package_name:
      required: true
    stability:
      default: 'stable'
      options: ['stable', 'RC', 'beta', 'alpha', 'dev']
```

## Migration Steps

1. **Create new structure**
   ```bash
   ./tools/reorganize-components.sh
   ```

2. **Migrate components**
   - Extract component definitions from nested directories
   - Standardize input specifications
   - Add comprehensive validation

3. **Update catalog**
   ```bash
   ./tools/generate-catalog.js
   ```

4. **Create tests**
   - Unit tests for each component
   - Integration tests for workflows
   - Performance benchmarks

5. **Generate documentation**
   - Auto-generate from component specs
   - Create usage examples
   - Build interactive docs

## Component Features

### Input Validation
- Type checking (string, boolean, array, object)
- Pattern matching with regex
- Enum validation with options
- Required/optional with defaults
- Secret handling

### Reusability
- Inheritance with extends
- Composition with includes
- Variable interpolation
- Conditional execution

### Testing
- Automated validation
- Integration testing
- Performance metrics
- Security scanning

### Documentation
- Auto-generated from specs
- Interactive examples
- Best practices
- Troubleshooting guides

## Benefits

1. **Standards Compliance**: Follows GitLab CI/CD component best practices
2. **Comprehensive Coverage**: Supports Drupal, Node.js, Python, AI, containers
3. **Type Safety**: Strong input validation and type checking
4. **Reusability**: Components work across projects
5. **Maintainability**: Clear structure and documentation
6. **Testing**: Comprehensive test coverage
7. **Security**: Built-in security scanning and best practices

## Next Steps

1. Review and approve this plan
2. Create migration scripts
3. Set up new directory structure
4. Migrate existing components
5. Add comprehensive tests
6. Deploy component catalog
7. Create usage documentation

This reorganization will transform bfcicomponents into a world-class GitLab CI/CD component library!