# TDD Quick Start Guide

##  Get Started with TDD in 5 Minutes

### Step 1: Choose Your TDD Mode

Add this to your `.gitlab-ci.yml`:

```yaml
include:
  - component: gitlab.bluefly.io/llm/bfcicomponents/drupal-tdd@main
    inputs:
      tdd_mode: "progressive"  # Choose: strict, progressive, optional, disabled
      min_test_coverage: "60"
```

### Step 2: TDD Mode Explained

| Mode | When to Use | Pipeline Behavior | Best For |
|------|-------------|-------------------|----------|
| **strict** | New projects, TDD-ready teams | ❌ Fails without tests | Greenfield projects |
| **progressive** | Existing projects adopting TDD |  Warns, doesn't fail | **RECOMMENDED** for most |
| **optional** | Legacy projects, exploration |  Manual jobs only | Learning TDD |
| **disabled** | No TDD support needed | ➡️ Basic pipeline only | Legacy/maintenance |

### Step 3: What Happens Next?

The component will:
1. 🔍 **Analyze** your modules for testing opportunities
2. 🏗️ **Create** test scaffolding if missing
3. 🧪 **Run** existing tests (unit, kernel, functional, security)
4.  **Report** coverage and provide recommendations
5. 📋 **Generate** a TDD migration plan

## 📋 Quick Migration Path

### Week 1: Exploration
```yaml
inputs:
  tdd_mode: "optional"
  min_test_coverage: "30"
```
- Run manual TDD analysis
- Create test scaffolding
- Write your first unit test

### Week 2-4: Progressive Adoption
```yaml
inputs:
  tdd_mode: "progressive"
  min_test_coverage: "50"
```
- Add unit tests for service classes
- Implement kernel tests for integrations
- Target 50% coverage

### Week 5-8: Comprehensive Testing
```yaml
inputs:
  tdd_mode: "progressive"
  min_test_coverage: "70"
  test_frameworks: "phpunit,behat"
```
- Add functional tests for workflows
- Include security tests
- Target 70% coverage

### Week 9+: TDD Excellence
```yaml
inputs:
  tdd_mode: "strict"
  min_test_coverage: "80"
  enable_mutation_testing: "true"
```
- Enforce all test requirements
- Enable mutation testing
- Achieve 80%+ coverage

## 🧪 Test Types Quick Reference

### Unit Tests (`tests/src/Unit/`)
**Purpose**: Test individual methods/classes in isolation
```php
public function testServiceMethod(): void {
  $service = new MyService();
  $result = $service->calculate(10, 5);
  $this->assertEquals(15, $result);
}
```

### Kernel Tests (`tests/src/Kernel/`)
**Purpose**: Test service integration with Drupal
```php
public function testServiceIntegration(): void {
  $service = $this->container->get('my_module.service');
  $this->assertInstanceOf(MyServiceInterface::class, $service);
}
```

### Functional Tests (`tests/src/Functional/`)
**Purpose**: Test complete user workflows
```php
public function testUserCanCreateContent(): void {
  $this->drupalLogin($this->drupalCreateUser(['create content']));
  $this->drupalGet('/node/add/article');
  $this->assertSession()->statusCodeEquals(200);
}
```

### Security Tests (`tests/src/Unit/Security/`)
**Purpose**: Test security measures
```php
public function testInputSanitization(): void {
  $malicious_input = '<script>alert("xss")</script>';
  $sanitized = $this->validator->sanitize($malicious_input);
  $this->assertStringNotContainsString('<script>', $sanitized);
}
```

##  Common Commands

### Run Tests Locally
```bash
# Run all tests
vendor/bin/phpunit web/modules/custom/

# Run specific test type
vendor/bin/phpunit web/modules/custom/*/tests/src/Unit/
vendor/bin/phpunit web/modules/custom/*/tests/src/Functional/

# Run with coverage
vendor/bin/phpunit --coverage-html=coverage web/modules/custom/
```

### Generate Test Scaffolding
```bash
# Manual scaffolding (or use pipeline job)
mkdir -p web/modules/custom/my_module/tests/src/{Unit,Kernel,Functional}
```

##  Manual Pipeline Jobs

When using `tdd_mode: "optional"`, these jobs are available manually:

- 🔍 **analyze_tdd_structure** - Assess current testing state
- 🏗️ **create_test_scaffolding** - Generate test templates
- 🧪 **run_sample_tests** - Try running existing tests
-  **explore_testing_opportunities** - Find testing candidates

## 🆘 Troubleshooting

### "No modules discovered"
- Check `modules_path` input (default: `web/modules/custom`)
- Ensure `.info.yml` files exist

### "Tests failed"
- Start with `tdd_mode: "optional"` to explore
- Use manual jobs to debug
- Check test dependencies and autoloading

### "Coverage too low"
- Add more unit tests (easiest coverage boost)
- Lower `min_test_coverage` temporarily
- Focus on testing service classes first

### "Pipeline too slow"
- Use caching (automatically enabled)
- Run tests in parallel (default behavior)
- Consider reducing test scope initially

##  Resources

- **Full Documentation**: [docs/components/drupal-tdd.md](./components/drupal-tdd.md)
- **Examples**: [examples/drupal-tdd-*.yml](../examples/)
- **Drupal Testing**: https://www.drupal.org/docs/testing
- **PHPUnit**: https://phpunit.de/documentation.html

##  Success Metrics

Track your TDD progress:

| Metric | Beginner | Intermediate | Advanced |
|--------|----------|--------------|----------|
| **Coverage** | 30%+ | 60%+ | 80%+ |
| **Test Types** | Unit only | Unit + Kernel | All types |
| **Pipeline Mode** | Optional | Progressive | Strict |
| **Quality** | Tests pass | Good coverage | Mutation testing |

##  Pro Tips

1. **Start Small**: One unit test is better than no tests
2. **Test Behavior**: Test what the code should do, not how it does it
3. **Mock Dependencies**: Unit tests should be isolated
4. **Descriptive Names**: Test names should explain what they test
5. **Red-Green-Refactor**: Write failing test → make it pass → improve code

##  Getting Help

- **Component Issues**: Create issue in `llm/bfcicomponents`
- **TDD Questions**: Check generated reports and recommendations
- **Team Training**: Use `tdd_mode: "optional"` for learning
- **Migration Support**: Follow the weekly migration plan

---

**Remember**: TDD is a journey, not a destination. Start where you are, progress at your pace! 