# Todo Aggregator Component

[![pipeline status](https://gitlab.com/blueflyio/bfcicomponents/badges/main/pipeline.svg)](https://gitlab.com/blueflyio/bfcicomponents/-/commits/main)

> Aggregates and analyzes TODOs from your codebase with priority detection, visualization, and CI integration.

This component provides comprehensive todo management capabilities for your codebase, including automatic priority and category detection, interactive visualizations, and GitLab CI integration.

## What it does

The Todo Aggregator component helps teams manage and track TODOs in their codebase by:

- Scanning files for TODOs and similar comments
- Detecting priorities and categories automatically
- Generating organized roadmap documentation
- Providing interactive visualizations for todo analysis
- Enforcing policies around critical TODOs in CI pipelines
- Tracking changes in TODOs over time

## Key Features

- **Priority Detection**: Automatically detects priorities based on text content (Critical, High, Medium, Low)
- **Category Classification**: Categorizes TODOs by type (bug, feature, security, etc.)
- **Interactive Dashboard**: Visualizes TODOs with charts and filtering capabilities
- **CI/CD Integration**: Enforces policies around critical TODOs in GitLab pipelines
- **Trend Analysis**: Tracks changes in TODOs over time
- **Project Management Integration**: Create issues from TODOs in systems like JIRA
- **MCP Integration**: Connect with the Model Context Protocol for AI-powered workflows
- **Advanced EcosystemicIntelligence**: AI-powered deep analysis and insights

## Usage

### 1. Basic Usage in GitLab CI

Include the Todo Aggregator CI template in your `.gitlab-ci.yml`:

```yaml
include:
  - project: 'blueflyio/bfcicomponents'
    ref: main
    file: 'components/todo-aggregator/examples/gitlab-ci-template.yml'

variables:
  TODO_MAX_CRITICAL: 3        # Maximum allowed critical TODOs
  TODO_MAX_HIGH: 15           # Maximum allowed high priority TODOs
  TODO_MAX_PER_FILE: 8        # Maximum allowed TODOs per file
  TODO_FAIL_ON_THRESHOLD: "false"  # Don't fail the build initially (for adoption)
```

### 2. Local CLI Usage

```bash
# Install the package
npm install @bfcicomponents/todo-aggregator

# Basic usage - scan files and generate a roadmap
npx bfcicomponents-todo-aggregator

# Specify glob patterns and output file
npx bfcicomponents-todo-aggregator --patterns "src/**/*.ts,lib/**/*.js" --out docs/roadmap.md

# Filter by priority
npx bfcicomponents-todo-aggregator --priority HIGH

# Filter by category
npx bfcicomponents-todo-aggregator --category security

# Start the interactive dashboard
npx bfcicomponents-todo-aggregator --dashboard

# Run CI check
npx bfcicomponents-todo-aggregator --ci-check
```

### 3. API Usage

```typescript
import { 
  scanFiles, 
  enhanceTodos, 
  aggregateTodosToRoadmap,
  createTodoMcpConnector
} from '@bfcicomponents/todo-aggregator';

// Scan files for TODOs
const todos = await scanFiles(['src/**/*.ts']);

// Enhance TODOs with priority and category
const enhancedTodos = enhanceTodos(todos);

// Filter TODOs by priority
const criticalTodos = enhancedTodos.filter(todo => 
  todo.priority === 'CRITICAL'
);

// Generate a roadmap markdown file
await aggregateTodosToRoadmap(['src/**/*.ts'], 'roadmap.md');

// Connect to MCP server
const connector = createTodoMcpConnector({
  mcpServerUrl: 'http://localhost:3001',
  apiKey: 'your-api-key',
  toolPrefix: 'todo-aggregator'
});
await connector.initialize();
```

## Configuration

### GitLab CI Configuration Options

| Variable | Description | Default |
|----------|-------------|---------|
| `TODO_MAX_CRITICAL` | Maximum allowed critical TODOs | 5 |
| `TODO_MAX_HIGH` | Maximum allowed high priority TODOs | 20 |
| `TODO_MAX_PER_FILE` | Maximum allowed TODOs per file | 10 |
| `TODO_FAIL_ON_THRESHOLD` | Whether to fail the build if thresholds are exceeded | "true" |
| `TODO_INCLUDE_PATTERNS` | Comma-separated glob patterns to include | "**/*.ts,**/*.js,**/*.tsx,**/*.jsx" |
| `TODO_EXCLUDE_PATTERNS` | Comma-separated glob patterns to exclude | "**/node_modules/**,**/dist/**,**/build/**" |

### Custom Configuration File

Create a custom configuration file (`todo-ci-config.json`):

```json
{
  "maxCriticalTodos": 5,
  "maxHighPriorityTodos": 20,
  "maxTodosPerFile": 10,
  "reportPath": "todo-report.md",
  "failOnThresholdExceeded": true,
  "criticalCategories": ["security", "crash", "data-loss"],
  "includePatterns": ["**/*.ts", "**/*.js", "**/*.tsx", "**/*.jsx"],
  "excludePatterns": ["**/node_modules/**", "**/dist/**", "**/build/**"]
}
```

Then use it with the CLI:

```bash
npx bfcicomponents-todo-aggregator --ci-check --config todo-ci-config.json
```

## Examples

The component includes multiple examples demonstrating various features:

1. **Priority Demo**: Basic priority and category detection
2. **Custom Patterns**: Customizing priority and category detection patterns
3. **JIRA Integration**: Integration with project management systems
4. **Visualize Priorities**: ASCII chart visualization
5. **Interactive Dashboard**: Web-based dashboard
6. **Trend Analysis**: Tracking changes over time
7. **CI Integration**: Integrating with GitLab CI
8. **MCP Integration**: Integrating with the Model Context Protocol

Run the examples:
```bash
npm run examples
```

## Best Practices

1. **Start with Informational Reports**: Begin with non-failing checks to establish baselines
2. **Gradually Tighten Thresholds**: Reduce thresholds over time to drive improvement
3. **Different Thresholds for Different Branches**: Consider stricter rules for main branches
4. **Regular Dashboard Reviews**: Use the dashboard for team visibility and planning
5. **Track Trends Over Time**: Use trend analysis to track progress

## Related Components

- **[ESLint Component](../eslint)**: Code linting and style enforcement
- **[Drupal TDD Component](../drupal/drupal-tdd)**: Test-driven development for Drupal

## Documentation

For more detailed information, see:

- [Priority Detection Feature](./PRIORITY_DETECTION_FEATURE.md)
- [GitLab CI Integration Guide](./docs/CI_INTEGRATION.md)
- [MCP Integration Guide](./docs/MCP_INTEGRATION.md)

## License

MIT