# Drupal CI/CD Components Architecture

## Overview

This document outlines the comprehensive architecture for scalable Drupal CI/CD components that follow Drupal standards and best practices while being flexible for all kinds of Drupal sites.

## Component Design Philosophy

### 1. Scalability First
- **Minimal Individual CI Configuration**: Projects should have minimal `.gitlab-ci.yml` files
- **Component-Based Architecture**: All logic centralized in reusable components
- **Progressive Enhancement**: Components can be enhanced without breaking existing implementations

### 2. Drupal Standards Compliance
- **Drupal Coding Standards**: PHPCS with Drupal and DrupalPractice standards
- **Test Structure**: Unit, Kernel, Functional, and Behavioral tests
- **Security Best Practices**: Automated security scanning and validation
- **Performance Standards**: Automated performance testing and optimization

### 3. Flexibility & Adaptability
- **Multi-Site Support**: Components work for modules, themes, distributions, and full sites
- **Version Flexibility**: Support for Drupal 10.x and 11.x
- **Environment Agnostic**: Works across development, staging, and production environments

## Core Component Architecture

### 1. Scheduling Components

#### `drupal-cron`
```yaml
# templates/drupal-cron/template.yml
spec:
  inputs:
    schedule:
      default: "0 */6 * * *"
      type: string
      description: "Cron schedule expression"
    
    drupal_root:
      default: "web"
      type: string
      description: "Drupal installation directory"
    
    environment:
      default: "production"
      type: string
      options: ["development", "staging", "production"]
    
    timeout:
      default: 3600
      type: number
      description: "Maximum execution time in seconds"
```

#### `drupal-scheduled-tests`
```yaml
# Nightly comprehensive testing
spec:
  inputs:
    test_schedule:
      default: "0 2 * * *"
      type: string
      description: "When to run comprehensive tests"
    
    browsers:
      default: "chrome,firefox"
      type: string
      description: "Browsers for functional testing"
    
    performance_budget:
      default: true
      type: boolean
      description: "Enable performance budget testing"
```

### 2. Preparation Components

#### `drupal-prepare`
```yaml
# templates/drupal-prepare/template.yml
spec:
  inputs:
    php_version:
      default: "8.3"
      type: string
      options: ["8.1", "8.2", "8.3", "8.4"]
    
    drupal_version:
      default: "11.x"
      type: string
      options: ["10.x", "11.x", "latest"]
    
    database:
      default: "mysql:8.0"
      type: string
      options: ["mysql:5.7", "mysql:8.0", "mariadb:10.6", "postgres:14"]
    
    cache_strategy:
      default: "composer"
      type: string
      options: ["composer", "npm", "vendor", "all"]
    
    install_dev_dependencies:
      default: true
      type: boolean
      description: "Install development dependencies"
```

#### `drupal-environment-setup`
```yaml
# Environment-specific preparation
spec:
  inputs:
    environment_type:
      default: "testing"
      type: string
      options: ["testing", "staging", "production"]
    
    enable_xdebug:
      default: false
      type: boolean
      description: "Enable Xdebug for debugging"
    
    memory_limit:
      default: "512M"
      type: string
      description: "PHP memory limit"
    
    upload_max_filesize:
      default: "64M"
      type: string
      description: "Maximum upload file size"
```

### 3. Build Components

#### `drupal-build`
```yaml
# templates/drupal-build/template.yml
spec:
  inputs:
    build_type:
      default: "full"
      type: string
      options: ["minimal", "full", "production"]
    
    optimize_autoloader:
      default: true
      type: boolean
      description: "Optimize Composer autoloader"
    
    install_suggest:
      default: false
      type: boolean
      description: "Install suggested packages"
    
    clear_cache:
      default: true
      type: boolean
      description: "Clear Drupal caches after build"
    
    build_assets:
      default: true
      type: boolean
      description: "Build frontend assets (CSS/JS)"
    
    asset_build_command:
      default: "npm run build"
      type: string
      description: "Command to build frontend assets"
```

#### `drupal-theme-build`
```yaml
# Specialized theme building
spec:
  inputs:
    theme_path:
      default: "web/themes/custom"
      type: string
      description: "Path to custom themes"
    
    build_tools:
      default: "webpack"
      type: string
      options: ["webpack", "gulp", "npm", "yarn"]
    
    scss_compilation:
      default: true
      type: boolean
      description: "Compile SCSS to CSS"
    
    js_minification:
      default: true
      type: boolean
      description: "Minify JavaScript files"
```

### 4. Lint Components

#### `drupal-lint`
```yaml
# templates/drupal-lint/template.yml
spec:
  inputs:
    phpcs_standards:
      default: "Drupal,DrupalPractice"
      type: string
      description: "PHPCS coding standards to apply"
    
    eslint_enabled:
      default: true
      type: boolean
      description: "Enable ESLint for JavaScript"
    
    stylelint_enabled:
      default: true
      type: boolean
      description: "Enable Stylelint for CSS/SCSS"
    
    phpstan_level:
      default: 7
      type: number
      description: "PHPStan analysis level (0-9)"
    
    custom_rules:
      default: ""
      type: string
      description: "Path to custom linting rules"
    
    fix_automatically:
      default: false
      type: boolean
      description: "Attempt to fix linting issues automatically"
```

#### `drupal-security-lint`
```yaml
# Security-focused linting
spec:
  inputs:
    security_checker:
      default: true
      type: boolean
      description: "Run Symfony security checker"
    
    psalm_enabled:
      default: false
      type: boolean
      description: "Enable Psalm static analysis"
    
    custom_security_rules:
      default: ""
      type: string
      description: "Custom security scanning rules"
```

### 5. Test Components

#### `drupal-test-unit`
```yaml
# templates/drupal-test-unit/template.yml
spec:
  inputs:
    phpunit_config:
      default: "phpunit.xml"
      type: string
      description: "PHPUnit configuration file"
    
    test_suites:
      default: "unit"
      type: string
      description: "Test suites to run (comma-separated)"
    
    coverage_format:
      default: "html,clover"
      type: string
      description: "Coverage report formats"
    
    coverage_threshold:
      default: 80
      type: number
      description: "Minimum coverage percentage"
    
    parallel_testing:
      default: true
      type: boolean
      description: "Enable parallel test execution"
```

#### `drupal-test-kernel`
```yaml
# Kernel testing with database
spec:
  inputs:
    database_url:
      default: "mysql://drupal:drupal@mysql:3306/drupal"
      type: string
      description: "Database connection string"
    
    test_modules:
      default: ""
      type: string
      description: "Additional test modules to enable"
    
    memory_limit:
      default: "1G"
      type: string
      description: "Memory limit for kernel tests"
```

#### `drupal-test-functional`
```yaml
# Functional testing with browser
spec:
  inputs:
    browser:
      default: "chrome"
      type: string
      options: ["chrome", "firefox", "edge"]
    
    headless:
      default: true
      type: boolean
      description: "Run browser in headless mode"
    
    screen_resolution:
      default: "1920x1080"
      type: string
      description: "Browser screen resolution"
    
    test_base_url:
      default: "http://localhost:8080"
      type: string
      description: "Base URL for functional tests"
```

#### `drupal-test-behavioral`
```yaml
# Behat behavioral testing
spec:
  inputs:
    behat_config:
      default: "behat.yml"
      type: string
      description: "Behat configuration file"
    
    test_profiles:
      default: "default"
      type: string
      description: "Behat profiles to run"
    
    features_path:
      default: "tests/features"
      type: string
      description: "Path to feature files"
```

### 6. Deploy Components

#### `drupal-deploy`
```yaml
# templates/drupal-deploy/template.yml
spec:
  inputs:
    deployment_strategy:
      default: "rolling"
      type: string
      options: ["rolling", "blue-green", "canary"]
    
    target_environment:
      default: "staging"
      type: string
      options: ["development", "staging", "production"]
    
    backup_before_deploy:
      default: true
      type: boolean
      description: "Create backup before deployment"
    
    run_database_updates:
      default: true
      type: boolean
      description: "Run database updates during deployment"
    
    clear_cache_after_deploy:
      default: true
      type: boolean
      description: "Clear all caches after deployment"
    
    health_check_enabled:
      default: true
      type: boolean
      description: "Perform health checks after deployment"
```

#### `drupal-database-deploy`
```yaml
# Database-specific deployment
spec:
  inputs:
    migration_strategy:
      default: "incremental"
      type: string
      options: ["incremental", "full", "config-only"]
    
    backup_retention:
      default: 7
      type: number
      description: "Days to retain backups"
    
    sync_files:
      default: false
      type: boolean
      description: "Sync files directory"
```

### 7. Quality Assurance Components

#### `drupal-qa-comprehensive`
```yaml
# Complete quality assurance suite
spec:
  inputs:
    performance_testing:
      default: true
      type: boolean
      description: "Run performance tests"
    
    accessibility_testing:
      default: true
      type: boolean
      description: "Run accessibility tests (WCAG 2.1)"
    
    load_testing:
      default: false
      type: boolean
      description: "Run load testing"
    
    mutation_testing:
      default: false
      type: boolean
      description: "Run mutation testing"
    
    visual_regression:
      default: false
      type: boolean
      description: "Visual regression testing"
```

## Component Usage Patterns

### 1. Simple Module Development
```yaml
# .gitlab-ci.yml for a simple module
include:
  - component: gitlab.bluefly.io/llm/bfcicomponents/drupal-prepare@main
    inputs:
      drupal_version: "11.x"
      php_version: "8.3"
  
  - component: gitlab.bluefly.io/llm/bfcicomponents/drupal-lint@main
    inputs:
      phpcs_standards: "Drupal,DrupalPractice"
      phpstan_level: 7
  
  - component: gitlab.bluefly.io/llm/bfcicomponents/drupal-test-unit@main
    inputs:
      coverage_threshold: 80
```

### 2. Full Site Development
```yaml
# .gitlab-ci.yml for a complete site
include:
  - component: gitlab.bluefly.io/llm/bfcicomponents/drupal-prepare@main
  - component: gitlab.bluefly.io/llm/bfcicomponents/drupal-build@main
  - component: gitlab.bluefly.io/llm/bfcicomponents/drupal-lint@main
  - component: gitlab.bluefly.io/llm/bfcicomponents/drupal-test-unit@main
  - component: gitlab.bluefly.io/llm/bfcicomponents/drupal-test-kernel@main
  - component: gitlab.bluefly.io/llm/bfcicomponents/drupal-test-functional@main
  - component: gitlab.bluefly.io/llm/bfcicomponents/drupal-deploy@main
    inputs:
      target_environment: "[CI_COMMIT_REF_NAME]"
```

### 3. Enterprise Site with Full QA
```yaml
# .gitlab-ci.yml for enterprise site
include:
  - component: gitlab.bluefly.io/llm/bfcicomponents/drupal-prepare@main
  - component: gitlab.bluefly.io/llm/bfcicomponents/drupal-build@main
    inputs:
      build_type: "production"
      optimize_autoloader: true
  
  - component: gitlab.bluefly.io/llm/bfcicomponents/drupal-lint@main
  - component: gitlab.bluefly.io/llm/bfcicomponents/drupal-security-lint@main
  
  - component: gitlab.bluefly.io/llm/bfcicomponents/drupal-test-unit@main
  - component: gitlab.bluefly.io/llm/bfcicomponents/drupal-test-kernel@main
  - component: gitlab.bluefly.io/llm/bfcicomponents/drupal-test-functional@main
  - component: gitlab.bluefly.io/llm/bfcicomponents/drupal-test-behavioral@main
  
  - component: gitlab.bluefly.io/llm/bfcicomponents/drupal-qa-comprehensive@main
    inputs:
      performance_testing: true
      accessibility_testing: true
      load_testing: true
  
  - component: gitlab.bluefly.io/llm/bfcicomponents/drupal-deploy@main
    inputs:
      deployment_strategy: "blue-green"
      backup_before_deploy: true

# Scheduled jobs
drupal_cron:
  extends: .drupal-cron
  only:
    - schedules
  variables:
    SCHEDULE_TYPE: "cron"

nightly_tests:
  extends: .drupal-scheduled-tests
  only:
    - schedules
  variables:
    SCHEDULE_TYPE: "nightly"
```

## Implementation Benefits

### 1. Reduced Maintenance
- **Centralized Logic**: All CI logic maintained in components
- **Automatic Updates**: Components can be updated without touching individual projects
- **Consistent Standards**: All projects automatically follow latest best practices

### 2. Enhanced Flexibility
- **Modular Architecture**: Pick and choose components as needed
- **Progressive Enhancement**: Add more components as projects mature
- **Environment Specific**: Components adapt to different environments

### 3. Drupal Best Practices
- **Standards Compliance**: Automatic enforcement of Drupal coding standards
- **Test Coverage**: Comprehensive testing across all Drupal test types
- **Security First**: Built-in security scanning and best practices
- **Performance Optimized**: Automatic performance testing and optimization

### 4. Developer Experience
- **Minimal Configuration**: Projects need minimal CI configuration
- **Clear Documentation**: Each component is well-documented
- **Easy Debugging**: Components provide clear error messages and logs
- **Fast Feedback**: Optimized for quick feedback loops

This architecture ensures that all Drupal projects in the ecosystem follow best practices while maintaining maximum flexibility and minimal maintenance overhead.