# Drupal TDD Component Documentation

## Overview

The `drupal-tdd.yml` component extends the base `drupal-modules.yml` component with comprehensive Test-Driven Development (TDD) capabilities. It provides flexible TDD enforcement levels to allow progressive adoption without breaking existing projects.

## Features

- **Progressive TDD Adoption**: Choose from strict, progressive, optional, or disabled modes
- **Comprehensive Test Coverage**: Unit, kernel, functional, and security testing
- **Quality Gates**: Automated validation of test coverage and TDD practices
- **Mutation Testing**: Validate the quality of your tests
- **Security Testing**: Built-in security validation and vulnerability scanning
- **Flexible Configuration**: Adapt to your project's current testing maturity

## TDD Modes

### 1. Strict Mode (`tdd_mode: "strict"`)
- **Use Case**: New projects or projects with established TDD practices
- **Requirements**: All test types must be present and pass
- **Coverage**: Enforces minimum coverage requirements
- **Behavior**: Pipeline fails if any TDD requirements are not met

### 2. Progressive Mode (`tdd_mode: "progressive"`) - **RECOMMENDED**
- **Use Case**: Existing projects adopting TDD incrementally
- **Requirements**: Tests are encouraged but failures don't block deployment
- **Coverage**: Reports coverage but allows lower percentages
- **Behavior**: Warnings for missing tests, but pipeline continues

### 3. Optional Mode (`tdd_mode: "optional"`)
- **Use Case**: Legacy projects with minimal testing
- **Requirements**: All testing jobs are manual
- **Coverage**: No coverage requirements
- **Behavior**: All TDD jobs can be run manually when needed

### 4. Disabled Mode (`tdd_mode: "disabled"`)
- **Use Case**: Projects that cannot support testing infrastructure
- **Requirements**: No TDD requirements
- **Behavior**: Falls back to basic drupal-modules.yml functionality

## Usage Examples

### Basic Configuration

```yaml
include:
  - component: gitlab.bluefly.io/llm/bfcicomponents/drupal-tdd@main
    inputs:
      tdd_mode: "progressive"
      min_test_coverage: "70"
      modules_path: "web/modules/custom"
```

### Advanced Configuration

```yaml
include:
  - component: gitlab.bluefly.io/llm/bfcicomponents/drupal-tdd@main
    inputs:
      tdd_mode: "strict"
      min_test_coverage: "85"
      test_frameworks: "phpunit,behat,nightwatch"
      enable_mutation_testing: "true"
      test_data_fixtures: "true"
      performance_testing: "true"
      php_version: "8.3"
      drupal_version: "11.x"
```

### Legacy Project Migration

```yaml
# Start with optional mode
include:
  - component: gitlab.bluefly.io/llm/bfcicomponents/drupal-tdd@main
    inputs:
      tdd_mode: "optional"
      min_test_coverage: "30"
      
# After implementing some tests, move to progressive
# tdd_mode: "progressive"
# min_test_coverage: "50"

# Eventually reach strict mode
# tdd_mode: "strict"  
# min_test_coverage: "80"
```

## Test Structure Requirements

The component expects the following directory structure for each module:

```
web/modules/custom/your_module/
├── src/
│   ├── Service/
│   ├── Controller/
│   ├── Form/
│   └── Security/
├── tests/
│   └── src/
│       ├── Unit/
│       │   ├── Service/
│       │   ├── Controller/
│       │   └── Security/
│       ├── Kernel/
│       │   ├── Service/
│       │   └── Integration/
│       ├── Functional/
│       │   ├── Controller/
│       │   ├── Form/
│       │   └── Dashboard/
│       └── FunctionalJavascript/
└── your_module.info.yml
```

## Test Types Explained

### Unit Tests
- **Purpose**: Test individual classes and methods in isolation
- **Location**: `tests/src/Unit/`
- **Requirements**: Mock all dependencies, no database access
- **Example**: Testing service logic, validators, utilities

### Kernel Tests  
- **Purpose**: Test service integration and dependency injection
- **Location**: `tests/src/Kernel/`
- **Requirements**: Drupal kernel loaded, services available
- **Example**: Testing service container registration, configuration

### Functional Tests
- **Purpose**: Test complete user workflows and UI interactions
- **Location**: `tests/src/Functional/`
- **Requirements**: Full Drupal installation, database
- **Example**: Testing forms, controllers, user permissions

### Security Tests
- **Purpose**: Validate security measures and vulnerability protection
- **Location**: `tests/src/Unit/Security/` or `tests/src/Functional/Security/`
- **Requirements**: Test input validation, access control, data protection
- **Example**: XSS protection, SQL injection prevention, API key security

## Coverage Analysis

The component provides detailed coverage analysis including:

- **Line Coverage**: Percentage of code lines executed by tests
- **Branch Coverage**: Percentage of code branches tested
- **Method Coverage**: Percentage of methods with test coverage
- **Class Coverage**: Percentage of classes with tests

### Coverage Thresholds

| TDD Mode | Minimum Coverage | Recommended Coverage |
|----------|------------------|---------------------|
| Strict | 80% | 90%+ |
| Progressive | 60% | 70%+ |
| Optional | 30% | 50%+ |
| Disabled | N/A | N/A |

## Quality Gates

The component implements several quality gates:

1. **TDD Score**: Calculated based on test type presence (25 points each)
2. **Coverage Threshold**: Minimum coverage percentage
3. **Security Tests**: Presence of security-focused tests
4. **Mutation Score**: Quality of tests (when enabled)

## Mutation Testing

When enabled, mutation testing validates test quality by:
- Introducing small code changes (mutations)
- Running tests against mutated code
- Ensuring tests fail when code is broken
- Calculating mutation score (% of mutations caught)

### Mutation Testing Configuration

```yaml
inputs:
  enable_mutation_testing: "true"
  # Requires higher test coverage for meaningful results
  min_test_coverage: "80"
```

## Integration with Existing Projects

### Step 1: Assessment
Run with `tdd_mode: "optional"` to assess current state:

```bash
# The component will generate a TDD analysis report
# showing current test coverage and recommendations
```

### Step 2: Incremental Improvement
Move to `tdd_mode: "progressive"` and start adding tests:

1. Begin with unit tests for service classes
2. Add kernel tests for service integration
3. Implement functional tests for user workflows
4. Add security tests for critical functionality

### Step 3: Full TDD Adoption
Switch to `tdd_mode: "strict"` when ready:
- All modules have comprehensive test coverage
- Team is comfortable with TDD practices
- CI/CD pipeline includes test requirements

## Troubleshooting

### Common Issues

1. **"No modules discovered"**
   - Check `modules_path` input parameter
   - Ensure `.info.yml` files exist in module directories

2. **"Unit tests failed"**
   - Verify PHPUnit configuration
   - Check test dependencies and autoloading
   - Ensure proper test isolation

3. **"Coverage below threshold"**
   - Add more comprehensive tests
   - Remove dead code or document exclusions
   - Consider lowering threshold temporarily

4. **"Security tests missing"**
   - Add security-focused test classes
   - Test input validation and sanitization
   - Validate access control and permissions

### Debug Mode

Enable debug output by setting:

```yaml
variables:
  CI_DEBUG_TRACE: "true"
```

## Best Practices

### For New Projects
1. Start with `tdd_mode: "strict"`
2. Set minimum coverage to 80%+
3. Enable mutation testing
4. Include all test frameworks

### For Existing Projects
1. Begin with assessment using `tdd_mode: "optional"`
2. Gradually increase coverage requirements
3. Migrate to `tdd_mode: "progressive"`
4. Eventually adopt `tdd_mode: "strict"`

### Test Design
1. Follow the Testing Pyramid: More unit tests, fewer functional tests
2. Test behavior, not implementation details
3. Use descriptive test names
4. Include edge cases and error conditions
5. Mock external dependencies in unit tests

## Performance Considerations

- **Caching**: Component uses optimized caching strategies
- **Parallel Execution**: Tests run in parallel when possible
- **Selective Testing**: Only runs relevant tests for changed modules
- **Artifact Management**: Efficient artifact storage and cleanup

## Security Features

- **Automated Security Scanning**: Built-in vulnerability detection
- **Input Validation Testing**: Ensures proper sanitization
- **Access Control Testing**: Validates permission systems
- **Security Regression Testing**: Prevents security vulnerabilities

## Migration from Basic Drupal Component

Replace your existing component inclusion:

```yaml
# Before
include:
  - component: gitlab.bluefly.io/llm/bfcicomponents/drupal-modules@main

# After  
include:
  - component: gitlab.bluefly.io/llm/bfcicomponents/drupal-tdd@main
    inputs:
      tdd_mode: "progressive"  # Start here for existing projects
```

The TDD component is fully backward compatible and will not break existing pipelines when used in `progressive` or `optional` modes.

## Support and Contributing

- **Documentation**: Full examples in `/docs/examples/`
- **Issues**: Report issues in the bfcicomponents repository
- **Contributions**: Follow the contribution guidelines
- **Support**: Reach out to the Bluefly AI Platform Engineering team

## Changelog

- **v1.0.0**: Initial release with comprehensive TDD support
- **v1.1.0**: Added mutation testing and security validation
- **v1.2.0**: Enhanced progressive mode and quality gates
- **v1.3.0**: Performance optimization and parallel test execution