# Drupal TDD CI/CD Component

A comprehensive GitLab CI/CD component for Drupal test-driven development with full testing pipeline.

## Overview

The Drupal TDD component provides a complete testing environment for Drupal modules, themes, and sites. It includes unit testing, kernel testing, functional testing, code quality validation, and coverage reporting.

## Features

- 🧪 **Complete Test Suite**: Unit, kernel, and functional tests
-  **Coverage Reporting**: Configurable coverage thresholds
- 🔍 **Code Quality**: PHPStan analysis and Drupal coding standards
- 🐘 **Multi-PHP Support**: PHP 8.1, 8.2, 8.3, 8.4
- 🗄️ **Database Testing**: MySQL 5.7, 8.0, 8.4 support
- 📦 **Module Packaging**: Automated deployment packaging

## Usage

```yaml
include:
  - component: $CI_SERVER_FQDN/llm/bfcicomponents/drupal-tdd@main
    inputs:
      php_version: "8.3"
      drupal_version: "11.x"
      test_coverage: true
      coverage_threshold: 80
```

## Input Parameters

| Parameter | Type | Default | Description |
|-----------|------|---------|-------------|
| `php_version` | string | `"8.3"` | PHP version to use |
| `drupal_version` | string | `"11.x"` | Drupal version to test against |
| `test_coverage` | boolean | `true` | Enable test coverage reporting |
| `phpstan_enabled` | boolean | `true` | Enable PHPStan static analysis |
| `coding_standards` | boolean | `true` | Enable Drupal coding standards check |
| `coverage_threshold` | number | `80` | Minimum test coverage percentage |
| `test_types` | string | `"unit,kernel,functional"` | Types of tests to run |
| `mysql_version` | string | `"8.0"` | MySQL version for testing |

## Pipeline Stages

1. **setup**: Sets up Drupal environment and dependencies
2. **validate**: Runs code quality checks (PHPStan, coding standards)
3. **test**: Executes unit, kernel, and functional tests
4. **coverage**: Generates and validates coverage reports
5. **deploy**: Packages module for deployment

## Examples

### Basic Module Testing
```yaml
include:
  - component: $CI_SERVER_FQDN/llm/bfcicomponents/drupal-tdd@main
    inputs:
      php_version: "8.3"
      drupal_version: "11.x"
      test_types: "unit,kernel"
```

### Full Testing with High Coverage
```yaml
include:
  - component: $CI_SERVER_FQDN/llm/bfcicomponents/drupal-tdd@main
    inputs:
      php_version: "8.3"
      drupal_version: "11.x"
      test_coverage: true
      coverage_threshold: 90
      test_types: "unit,kernel,functional"
```

### Legacy Drupal Support
```yaml
include:
  - component: $CI_SERVER_FQDN/llm/bfcicomponents/drupal-tdd@main
    inputs:
      php_version: "8.1"
      drupal_version: "10.x"
      mysql_version: "5.7"
      coding_standards: false
```

### Code Quality Only
```yaml
include:
  - component: $CI_SERVER_FQDN/llm/bfcicomponents/drupal-tdd@main
    inputs:
      test_types: ""
      phpstan_enabled: true
      coding_standards: true
      test_coverage: false
```

## Test Types

- **unit**: Fast isolated tests without database
- **kernel**: Tests with database but no web server
- **functional**: Full browser-based tests with web server

## Requirements

- GitLab 17.0 or later
- MySQL service configured
- Drupal module/theme with proper test structure
- PHPUnit configuration (uses Drupal core config by default)

## File Structure

```
your-module/
├── composer.json
├── your_module.info.yml
├── src/
├── tests/
│   ├── src/
│   │   ├── Unit/
│   │   ├── Kernel/
│   │   └── Functional/
│   └── modules/
└── phpunit.xml (optional)
```

## Coverage Reports

Coverage reports are generated in multiple formats:
- Clover XML for GitLab integration
- HTML reports for detailed analysis
- Console output with percentage

## Troubleshooting

### Test Failures
1. Check Drupal version compatibility
2. Verify test database configuration
3. Ensure proper test class namespacing

### Coverage Issues
1. Verify Xdebug is enabled for coverage
2. Check file paths in coverage configuration
3. Adjust coverage threshold if needed

### Code Quality Failures
1. Run `phpcs --standard=Drupal` locally
2. Fix coding standard violations
3. Address PHPStan analysis issues

## Performance Optimization

- Uses Composer caching for faster builds
- Parallel test execution where possible
- Optimized Docker images for PHP versions
- Smart artifact management

## License

MIT License - see [LICENSE.md](../../LICENSE.md) for details.