# Drupal Golden CI Module Component

This component provides a golden path CI/CD pipeline specifically designed for the development, testing, and packaging of custom Drupal modules. It incorporates best practices for module quality, security, and distribution.

## Usage

```yaml
include:
  - component: $CI_SERVER_FQDN/$CI_PROJECT_PATH/drupal_golden_ci_module@main # Or specific version
    inputs:
      php_version: "8.3"
      drupal_version: "11.x"
      module_name: "my_custom_module" # Machine name of the module
      module_path: "web/modules/custom/my_custom_module" # Path to the module directory
      # packagist_token: "your_packagist_token" # If publishing to packagist.org
      # release_branch: "main" # Branch for automated releases
```

## Purpose

This component is intended for repositories that either contain a single Drupal module or where you want to run CI for a specific module within a larger Drupal project. It focuses on:

-   **Module-Specific Linting**: Applying Drupal coding standards and static analysis.
-   **Targeted Testing**: Running unit, kernel, and functional tests relevant to the module.
-   **Code Coverage**: Ensuring adequate test coverage for the module's codebase.
-   **Security Scanning**: Checking for vulnerabilities in the module and its dependencies.
-   **Packaging**: Preparing the module for distribution (e.g., as a tarball or for Composer).
-   **Documentation**: Generating API documentation for the module.
-   **Release Management**: Optional integration for semantic versioning and publishing to repositories like Packagist or GitLab Package Registry.

## Included/Referenced Components (Conceptual)

This golden module recipe will leverage functionalities typically found in components like:

-   `drupal-prepare`: For setting up a consistent PHP and Drupal (minimal) environment.
-   `drupal-lint`: For PHPCS, PHPStan, and other linting.
-   `drupal-test-unit`, `drupal-test-kernel`, `drupal-test-functional`: For executing different types of tests against the module.
-   `drupal-security`: For security-focused scans.
-   `drupal-packagist` (or similar logic): For release and publishing.
-   (Logic inspired by `drupal-modules` for identifying module specifics if not explicitly provided).

## Inputs

Key inputs for this component might include:

| Input                         | Description                                                                 | Type    | Default          |
| ----------------------------- | --------------------------------------------------------------------------- | ------- | ---------------- |
| `php_version`                 | PHP version for testing and packaging.                                      | string  | `"8.3"`          |
| `drupal_version`              | Drupal core version compatibility (e.g., `^10 || ^11`).                 | string  | `"11.x"`         |
| `module_name`                 | The machine name of the module (e.g., `custom_module`).                   | string  |                  |
| `module_path`                 | Path to the module's root directory relative to the project root.         | string  | `." `            |
| `composer_json_path`          | Path to the module's `composer.json` if it has one (relative to `module_path`). | string | `composer.json`  |
| `has_frontend_assets`         | Set to true if the module includes JS/CSS assets requiring a build step.    | boolean | `false`          |
| `min_test_coverage`           | Minimum required code coverage percentage.                                  | number  | `80`             |
| `enable_security_scan`        | Flag to enable security scanning.                                           | boolean | `true`           |
| `enable_documentation`        | Flag to enable API documentation generation.                                | boolean | `true`           |
| `release_branch`              | Branch from which to create releases (e.g., `main`, `master`).              | string  | `main`           |
| `packagist_token`             | Packagist.org API token for publishing (leave empty if not used).           | string  | `""`             |
| `gitlab_release`              | Set to true to create GitLab releases.                                      | boolean | `true`           |

*(This is a representative list; actual inputs will be defined in `template.yml`)*

## Features

-   Streamlined CI for individual Drupal module development.
-   Enforces Drupal coding standards and best practices.
-   Automated testing and coverage reporting.
-   Optional automated release and packaging workflow.

## Requirements

-   GitLab 17.0+ for component support.
-   The module should follow standard Drupal module structure.
-   A `composer.json` file is recommended if the module has external PHP dependencies or is intended for Packagist.
-   PHPUnit tests set up for unit, kernel, and/or functional testing.

## License

MIT License - see `LICENSE.md` for details. 