# GitLab MR Automation

This component provides tools for automating GitLab Merge Request workflows, including creation, approval, and merging.

## Features

- Automated MR creation from branches
- Auto-approval based on configurable rules
- Auto-merging of approved MRs
- Support for custom MR templates
- Integration with GitLab CI/CD pipelines

## Usage

### Installation

Add this component to your project:

```yaml
include:
  - component: 'gitlab/mr-automation'
    from: 'bfcicomponents'
```

### Configuration

Configure the component in your `.gitlab-ci.yml` file:

```yaml
variables:
  AUTO_MERGE_LABEL: auto-merge
  AUTO_APPROVE_LABEL: auto-approve
  MR_TEMPLATE_DIR: .gitlab/merge_request_templates
```

### MR Templates

Create merge request templates in your template directory:

```markdown
# .gitlab/merge_request_templates/default.md
## What does this MR do?

## Why was this MR needed?

## What are the relevant tickets?

## Screenshots (if relevant)

## Does this MR meet the acceptance criteria?

- [ ] Tests have been added/updated
- [ ] Documentation has been updated
- [ ] Changelog has been updated
```

### Auto-Approval Rules

Configure auto-approval rules in your GitLab CI configuration:

```yaml
auto_approve_rules:
  - condition: "branch ~= /^dependabot\//"
    label: security-update
  - condition: "has_label('trivial-change') && files.count < 5"
```

### Auto-Merge Rules

Configure auto-merge rules:

```yaml
auto_merge_rules:
  - condition: "approved_by >= 2 && pipeline.status == 'success'"
  - condition: "has_label('auto-merge') && approved_by >= 1"
```

### Commands

#### Create MR

```bash
gitlab-mr create --source-branch=feature/new-feature --target-branch=main --title="Add new feature" --description="This MR adds a new feature"
```

#### Auto-Approve MRs

```bash
gitlab-mr auto-approve --project-id=12345
```

#### Auto-Merge MRs

```bash
gitlab-mr auto-merge --project-id=12345
```

## Integration with llmcli

This component is fully integrated with llmcli. You can use it with the following commands:

```bash
llmcli gitlab mr create
llmcli gitlab mr approve
llmcli gitlab mr merge
```

## Examples

### Creating an MR in a Pipeline

```yaml
create_mr:
  stage: deploy
  script:
    - llmcli gitlab mr create --source-branch=$CI_COMMIT_REF_NAME --target-branch=main --title="[Auto] Release $CI_COMMIT_REF_NAME" --template=release
  only:
    - /^release\/.*$/
```

### Auto-Approving Dependency Updates

```yaml
auto_approve:
  stage: approve
  script:
    - llmcli gitlab mr approve --rules="branch.startsWith('dependabot/') && changes.count < 10"
  only:
    - schedules
```

### Auto-Merging Approved MRs

```yaml
auto_merge:
  stage: merge
  script:
    - llmcli gitlab mr merge --rules="approved_by >= 2 && pipeline.status == 'success'"
  only:
    - schedules
```

## Advanced Usage

### Custom MR Creation Logic

You can implement custom logic for MR creation:

```javascript
// .gitlab/scripts/custom-mr-creator.js
module.exports = async function createMR(gitlab, options) {
  // Custom logic here
  const branchName = options.sourceBranch;
  const ticketId = branchName.match(/([A-Z]+-\d+)/)?.[1];
  
  const title = ticketId ? `[[ticketId]] [options.title]` : options.title;
  
  return gitlab.createMR({
    ...options,
    title,
    labels: [...options.labels, 'from-pipeline']
  });
};
```

### Conditional MR Approval

You can implement conditional approval logic:

```javascript
// .gitlab/scripts/conditional-approval.js
module.exports = async function shouldApprove(mr, gitlab) {
  // Check if it's a dependency update
  if (mr.source_branch.startsWith('dependabot/')) {
    const changes = await gitlab.getMRChanges(mr.id);
    
    // Only auto-approve if changes are limited to package.json and package-lock.json
    const changedFiles = changes.map(c => c.filename);
    return changedFiles.every(file => 
      file === 'package.json' || file === 'package-lock.json'
    );
  }
  
  return false;
};
```

## Support

For issues or feature requests, please contact the LLM Platform team.