# GitLab Template Management

This component provides a standardized way to manage GitLab CI/CD templates across projects. It includes tools for validation, registration, and synchronization of templates.

## Features

- Template validation with GitLab CI Lint
- Template registry for version tracking
- Synchronization of templates with GitLab
- Support for template inheritance and extension

## Usage

### Installation

Add this component to your project:

```yaml
include:
  - component: 'gitlab/template-management'
    from: 'bfcicomponents'
```

### Configuration

Configure the component in your `.gitlab-ci.yml` file:

```yaml
variables:
  TEMPLATE_DIR: .gitlab/templates
  TEMPLATE_REGISTRY: .gitlab/template-registry.yml
```

### Creating Templates

Create templates in your template directory:

```yaml
# .gitlab/templates/drupal-module.yml
stages:
  - validate
  - test
  - build
  - deploy

lint:
  stage: validate
  script:
    - php vendor/bin/phpcs
  tags:
    - php
```

### Template Registry

The template registry tracks template versions and metadata:

```yaml
# .gitlab/template-registry.yml
templates:
  drupal-module:
    version: 1.0.0
    path: .gitlab/templates/drupal-module.yml
    description: Standard pipeline for Drupal modules
  npm-package:
    version: 1.2.0
    path: .gitlab/templates/npm-package.yml
    description: Standard pipeline for NPM packages
```

### Commands

#### Validate Templates

```bash
gitlab-template validate
```

#### Register Templates

```bash
gitlab-template register
```

#### Sync Templates

```bash
gitlab-template sync
```

## Integration with llmcli

This component is fully integrated with llmcli. You can use it with the following commands:

```bash
llmcli gitlab template validate
llmcli gitlab template register
llmcli gitlab template sync
```

## Examples

### Using a Template in a Project

```yaml
# .gitlab-ci.yml
include:
  - project: 'bfcicomponents/components'
    ref: main
    file: 'gitlab/template-management/template.yml'
  - local: .gitlab/templates/drupal-module.yml

variables:
  TEMPLATE_DIR: .gitlab/templates
```

### Extending a Template

```yaml
# .gitlab-ci.yml
include:
  - project: 'bfcicomponents/templates'
    ref: main
    file: 'drupal/base.yml'

# Extend with custom configuration
stages:
  - validate
  - test
  - build
  - deploy
  - publish

publish:
  stage: publish
  script:
    - composer publish
  only:
    - tags
```

## Advanced Usage

### Template Inheritance

Templates can inherit from other templates:

```yaml
# .gitlab/templates/advanced-drupal-module.yml
include:
  - local: .gitlab/templates/drupal-module.yml

# Add additional jobs
security_scan:
  stage: validate
  script:
    - security-scanner scan
  allow_failure: true
```

### Template Variables

Templates can use variables that are defined in the including pipeline:

```yaml
# .gitlab/templates/configurable-template.yml
test:
  script:
    - echo "Running tests with [TEST_FRAMEWORK]"
    - [TEST_COMMAND]
```

## Support

For issues or feature requests, please contact the LLM Platform team.