# Model Registry Component

[![NPM version](https://img.shields.io/npm/v/@bluefly/model-registry.svg)](https://www.npmjs.com/package/@bluefly/model-registry)
[![License: MIT](https://img.shields.io/badge/License-MIT-blue.svg)](LICENSE)

A comprehensive Model Registry component for managing LLM models with built-in validation against standards. Fully integrated with GitLab ML Model Registry.

## Component Overview

The Model Registry component is a React-based library for managing LLM models with built-in validation against standards. It provides a complete UI for creating, updating, managing, and deploying LLM model configurations.

Key features include:
1. **Model Management** - CRUD operations for LLM models
2. **Standards Validation** - Validates models against predefined standards
3. **Auto-Fix** - Automatically fixes common configuration issues
4. **Environment-Specific Rules** - Different validation rules for different environments
5. **Provider-Specific Rules** - Custom validation for different model providers
6. **GitLab Integration** - Native integration with GitLab ML Model Registry

The component architecture consists of:
1. **Core Components**:
   - ModelRegistry: Main component for displaying models
   - ModelForm: For creating/editing models
   - ModelDeploymentForm: For deploying models
   - ModelStandardsValidator: Validates models against standards

2. **Services**:
   - GitLabModelRegistryApi: Service for interacting with GitLab's ML Model Registry

3. **Utils**:
   - ModelStandardsUtility: Validation utilities for model standards

4. **Context Providers**:
   - ModelRegistryProvider: Provides state management and API access

The component integrates with GitLab's ML Model Registry via the MLflow compatibility layer, allowing users to:
1. Create and manage models directly in GitLab
2. Track model versions
3. Deploy models to different environments
4. Validate models against standards
5. Roll back to previous versions

It uses Chakra UI for the interface components and has a peer dependency on @bluefly/reelcode for standards validation.

This component is part of a larger architecture for managing LLM models within the Bluefly ecosystem, providing a standardized interface for model management, deployment, and validation.

## Features

- 📋 **Model Management**: Create, update, and manage LLM model configurations
-  **Standards Validation**: Validate models against predefined standards
-  **Auto-Fix**: Automatically fix common configuration issues
- 🔄 **Environment-Specific Rules**: Different validation rules for different environments
- 🏷️ **Provider-Specific Rules**: Custom validation for different model providers
- 🔄 **GitLab Integration**: Native integration with GitLab ML Model Registry
-  **Authentication**: Secure API token management for GitLab

## Installation

```bash
npm install @bluefly/model-registry
```

## Usage

### Basic Model Registry

```tsx
import [ModelRegistry] from '@bluefly/model-registry';

function App() {
  return (
    <div className="app">
      <ModelRegistry 
        onModelSelect={handleModelSelect}
        showActions=[true]
        searchable=[true]
        filterable=[true]
      />
    </div>
  );
}
```

### GitLab Integration

The component can connect directly to GitLab's ML Model Registry:

```tsx
import { 
  ModelRegistryProvider, 
  ModelRegistry, 
  GitLabConfigForm,
  GitLabConfigStatus 
} from '@bluefly/model-registry';

function App() {
  return (
    <ModelRegistryProvider>
      <div className="app">
        <header>
          <h1>Model Registry</h1>
          <GitLabConfigStatus />
        </header>
        
        {/* GitLab Configuration Form */}
        <GitLabConfigForm 
          onConfigSaved={() => console.log('GitLab config saved')} 
        />
        
        [/* Model Registry Component */]
        <ModelRegistry 
          showActions=[true]
          searchable=[true]
          filterable=[true]
        />
      </div>
    </ModelRegistryProvider>
  );
}
```

### Model Form with Validation

```tsx
import [ModelForm] from '@bluefly/model-registry';

function CreateModel() {
  const handleSubmit = (data) => {
    console.log('Model data:', data);
    // Submit to API
  };

  return (
    <div className="create-model">
      <h1>Create New Model</h1>
      <ModelForm 
        onSubmit=[handleSubmit]
        onCancel={() => navigate(-1)}
      />
    </div>
  );
}
```

### Standalone Model Validation

```tsx
import [validateModelStandards] from '@bluefly/model-registry';

async function validateModel(model) {
  const result = await validateModelStandards(model);
  
  if (result.valid) {
    console.log('Model is valid!');
  } else {
    console.log('Validation errors:', result.errors);
    console.log('Warnings:', result.warnings);
  }
}
```

## Component API

### `<ModelRegistry>`

Main component for displaying and managing models.

**Props:**
- `className?: string` - Additional CSS class
- `onModelSelect?: (model: LlmModel) => void` - Called when a model is selected
- `onModelCreate?: (model: LlmModel) => void` - Called when a model is created
- `onModelUpdate?: (model: LlmModel) => void` - Called when a model is updated
- `onModelDelete?: (modelId: string) => void` - Called when a model is deleted
- `showActions?: boolean` - Whether to show action buttons
- `searchable?: boolean` - Enable search functionality
- `filterable?: boolean` - Enable filtering
- `pageSize?: number` - Number of models per page

### `<ModelRegistryProvider>`

Provider component for state management with GitLab integration support.

**Props:**
- `children: ReactNode` - Child components
- `initialModels?: LlmModel[]` - Initial models to display

**Context:**
- `gitlabConfig?: GitLabConfig` - GitLab configuration
- `setGitlabConfig: (config?: GitLabConfig) => void` - Update GitLab configuration

### `<GitLabConfigForm>`

Form component for configuring GitLab integration.

**Props:**
- `onConfigSaved?: () => void` - Called when configuration is saved
- `className?: string` - Additional CSS class

### `<GitLabConfigStatus>`

Status indicator for GitLab integration.

### `<ModelForm>`

Form component for creating and editing models.

**Props:**
- `model?: LlmModel` - Model data for editing (omit for create mode)
- `onSubmit: (data: ModelCreateRequest | ModelUpdateRequest) => void` - Submit handler
- `onCancel: () => void` - Cancel handler
- `loading?: boolean` - Loading state
- `error?: string` - Error message

### `<ModelStandardsValidator>`

Component that validates models against standards.

**Props:**
- `model?: LlmModel | ModelCreateRequest | ModelUpdateRequest` - Model to validate
- `onChange?: (isValid: boolean, results: ValidationResult) => void` - Validation result handler
- `showWarnings?: boolean` - Whether to display warnings

### Utility Functions

- `validateModelStandards(model)` - Validate a model against standards
- `applyModelStandards(model)` - Apply standards to fix model issues
- `generateModelStandardsConfig(projectId)` - Generate standards configuration for AI assistants

### GitLab API Service

The `GitLabModelRegistryApi` service provides direct access to GitLab's Model Registry API:

```tsx
import { GitLabModelRegistryApi, GitLabConfig } from '@bluefly/model-registry';

// Initialize the service
const config: GitLabConfig = {
  baseUrl: 'https://gitlab.example.com',
  projectId: '12345',
  token: 'your-access-token'
};

const api = new GitLabModelRegistryApi(config);

// Fetch models
const models = await api.fetchModels();

// Create a model
const newModel = await api.createModel({
  name: 'my-model',
  version: '1.0.0',
  provider: 'openai'
});
```

## GitLab Integration

The component integrates with GitLab's ML Model Registry via the MLflow compatibility layer. This enables:

1. **Model Management**: Create, update, delete models in your GitLab project
2. **Version Control**: Track model versions with proper semantic versioning
3. **Deployment Tracking**: Monitor model deployments across environments
4. **Metrics Storage**: Store and visualize model performance metrics
5. **Standards Validation**: Validate models against your organization's standards

### Configuration

To connect to GitLab, you'll need:

1. **GitLab URL**: Your GitLab instance URL (e.g., https://gitlab.example.com)
2. **Project ID**: The ID of your GitLab project
3. **Access Token**: A personal access token with the `api` scope

The token is stored only in the browser memory and never persisted to disk.

## Standards Configuration

Model validation rules are defined in YAML format:

```yaml
rules:
  naming:
    model_name:
      pattern: "^[a-zA-Z0-9\\-_]+$"
      min_length: 3
      max_length: 100
  
  versioning:
    pattern: "^\\d+\\.\\d+\\.\\d+(?:-[a-zA-Z0-9\\-\\.]+)?(?:\\+[a-zA-Z0-9\\-\\.]+)?$"
  
  parameters:
    temperature:
      min: 0
      max: 1
      recommended_max: 0.8
```

The component uses [@bluefly/reelcode](https://www.npmjs.com/package/@bluefly/reelcode) for standards validation.

## License

MIT © Bluefly Engineering