# Node.js CI/CD Component

A comprehensive GitLab CI/CD component for Node.js projects with modern package manager support and best practices.

## Overview

The Node.js component provides a complete CI/CD pipeline for Node.js applications with support for multiple package managers, testing, linting, building, and packaging. It follows modern Node.js development practices and GitLab CI/CD best practices.

## Features

-  **Multiple Node.js Versions**: Support for Node.js 18, 20, 22, and latest
- 📦 **Package Manager Flexibility**: npm, yarn, pnpm, and bun support
- 🧪 **Comprehensive Testing**: Test execution with coverage reporting
- 🔍 **Code Quality**: ESLint and TypeScript type checking
- 🏗️ **Build Pipeline**: Automated building and artifact collection
- 💾 **Smart Caching**: Optimized dependency caching strategies
-  **Coverage Reporting**: Cobertura and JUnit test reports
-  **Flexible Configuration**: Highly configurable for different project needs

## Usage

```yaml
include:
  - component: $CI_SERVER_FQDN/llm/bfcicomponents/nodejs@main
    inputs:
      node_version: "20"
      package_manager: "npm"
      test_coverage: true
      enable_linting: true
```

## Input Parameters

| Parameter | Type | Default | Description | Options |
|-----------|------|---------|-------------|---------|
| `node_version` | string | `"20"` | Node.js version to use | `18`, `20`, `22`, `latest` |
| `package_manager` | string | `"npm"` | Package manager to use | `npm`, `yarn`, `pnpm`, `bun` |
| `test_coverage` | boolean | `true` | Enable test coverage reporting | `true`, `false` |
| `enable_linting` | boolean | `true` | Enable code linting | `true`, `false` |
| `build_enabled` | boolean | `true` | Enable build stage | `true`, `false` |
| `cache_strategy` | string | `"aggressive"` | Caching strategy for dependencies | `none`, `basic`, `aggressive` |
| `coverage_threshold` | number | `80` | Minimum test coverage percentage | Any positive integer |

## Pipeline Stages

1. **install**: Install dependencies with package manager-specific optimizations
2. **quality**: Code linting and TypeScript type checking
3. **test**: Run tests with coverage reporting
4. **build**: Build the project for distribution
5. **package**: Create distribution packages (on tags or manual)

## Examples

### Basic TypeScript Project
```yaml
include:
  - component: $CI_SERVER_FQDN/llm/bfcicomponents/nodejs@main
    inputs:
      node_version: "20"
      package_manager: "npm"
      enable_linting: true
      test_coverage: true
```

### React Application with Yarn
```yaml
include:
  - component: $CI_SERVER_FQDN/llm/bfcicomponents/nodejs@main
    inputs:
      node_version: "18"
      package_manager: "yarn"
      build_enabled: true
      coverage_threshold: 85
```

### Modern Project with pnpm
```yaml
include:
  - component: $CI_SERVER_FQDN/llm/bfcicomponents/nodejs@main
    inputs:
      node_version: "22"
      package_manager: "pnpm"
      cache_strategy: "aggressive"
```

### Library Project with bun
```yaml
include:
  - component: $CI_SERVER_FQDN/llm/bfcicomponents/nodejs@main
    inputs:
      node_version: "latest"
      package_manager: "bun"
      build_enabled: true
      test_coverage: true
```

## Package Manager Support

### npm
- Uses `npm ci` for faster, reliable installs
- Configured with local cache directory
- Supports all standard npm scripts

### yarn
- Uses `yarn install --frozen-lockfile` for reproducible builds
- Custom cache folder configuration
- Supports yarn-specific optimizations

### pnpm
- Automatically installs pnpm globally
- Uses `--frozen-lockfile` for consistency
- Efficient disk space usage

### bun
- Installs bun runtime globally
- Fast package installation and execution
- Compatible with npm ecosystem

## Project Structure Requirements

Your Node.js project should have:

```
├── package.json          # Required
├── package-lock.json     # For npm
├── yarn.lock            # For yarn  
├── pnpm-lock.yaml       # For pnpm
├── bun.lockb            # For bun
├── src/                 # Source code
├── test/ or __tests__/  # Test files
└── dist/ or build/      # Build output (created by pipeline)
```

## Required package.json Scripts

The component expects these scripts (optional):

```json
{
  "scripts": {
    "test": "jest",
    "lint": "eslint src/",
    "typecheck": "tsc --noEmit",
    "build": "webpack --mode production"
  }
}
```

## Artifacts

The component collects:
- **Dependencies**: `node_modules/` (1 hour)
- **Test Reports**: Coverage and JUnit reports (1 week)
- **Build Output**: `dist/`, `build/`, `lib/` directories (1 week)
- **Packages**: `.tgz` files from npm pack (1 week)

## Caching Strategy

- **File-based Keys**: Uses lockfiles for cache invalidation
- **Multi-layer Caching**: Dependencies, build tools, and artifacts
- **Package Manager Specific**: Optimized for each package manager

## Requirements

- GitLab 17.0 or later
- Node.js project with `package.json`
- Appropriate lockfile for chosen package manager

## Troubleshooting

### Common Issues

1. **Missing Scripts**: Component gracefully handles missing npm scripts
2. **Lockfile Mismatch**: Ensure lockfile matches package manager
3. **Cache Issues**: Try different cache strategies if builds are slow
4. **Coverage Failures**: Adjust `coverage_threshold` if needed

### Debug Mode

Enable verbose logging by adding to your project's `.gitlab-ci.yml`:

```yaml
variables:
  NODE_ENV: development
  DEBUG: "*"
```

## License

MIT License - see [LICENSE.md](../../LICENSE.md) for details.