# Docker CI/CD Component

A comprehensive GitLab CI/CD component for building, testing, and deploying Docker containers.

## Overview

The Docker component provides a complete container lifecycle pipeline including building, testing, security scanning, and deployment. It supports modern Docker features like BuildKit, multi-stage builds, and layer caching.

## Features

- 🐳 **Modern Docker Support**: Uses Docker 24+ with BuildKit
-  **Multi-stage Builds**: Optimized build process support
- 🔒 **Security Scanning**: Built-in container vulnerability scanning
- 📦 **Registry Management**: Automatic tagging and pushing
-  **Layer Caching**: Improved build performance
-  **Flexible Configuration**: Customizable for various use cases

## Usage

```yaml
include:
  - component: $CI_SERVER_FQDN/llm/bfcicomponents/docker@main
    inputs:
      base_image: "node:20-alpine"
      app_port: "3000"
      registry: "$CI_REGISTRY_IMAGE"
      security_scan: true
```

## Input Parameters

| Parameter | Type | Default | Description |
|-----------|------|---------|-------------|
| `base_image` | string | `"alpine:latest"` | Base Docker image to use |
| `app_port` | string | `"3000"` | Port the application runs on |
| `registry` | string | `"$CI_REGISTRY_IMAGE"` | Docker registry to push to |
| `build_context` | string | `"."` | Docker build context path |
| `dockerfile_path` | string | `"./Dockerfile"` | Path to Dockerfile |
| `enable_cache` | boolean | `true` | Enable Docker layer caching |
| `multi_stage` | boolean | `false` | Enable multi-stage build optimization |
| `security_scan` | boolean | `true` | Enable container security scanning |

## Pipeline Stages

1. **build**: Builds the Docker image with optional caching
2. **test**: Tests the built image functionality
3. **scan**: Performs security vulnerability scanning
4. **deploy**: Pushes to registry and deploys (manual trigger)

## Examples

### Basic Web Application
```yaml
include:
  - component: $CI_SERVER_FQDN/llm/bfcicomponents/docker@main
    inputs:
      base_image: "node:20-alpine"
      app_port: "3000"
      dockerfile_path: "./Dockerfile"
```

### Multi-stage Build with Caching
```yaml
include:
  - component: $CI_SERVER_FQDN/llm/bfcicomponents/docker@main
    inputs:
      base_image: "node:20-alpine"
      multi_stage: true
      enable_cache: true
      security_scan: true
```

### Custom Registry
```yaml
include:
  - component: $CI_SERVER_FQDN/llm/bfcicomponents/docker@main
    inputs:
      registry: "my-registry.com/myapp"
      build_context: "./app"
      dockerfile_path: "./app/Dockerfile.prod"
```

## Requirements

- GitLab 17.0 or later
- Docker-in-Docker service enabled
- Container registry access configured
- Dockerfile present in the repository

## Security

- Automatic vulnerability scanning with Trivy
- Registry authentication handling
- Secure credential management
- Layer optimization for minimal attack surface

## Troubleshooting

### Build Failures
1. Check Dockerfile syntax and base image availability
2. Verify build context contains all required files
3. Ensure sufficient disk space for Docker builds

### Registry Issues
1. Verify `CI_REGISTRY_USER` and `CI_REGISTRY_PASSWORD` variables
2. Check registry permissions and network access
3. Confirm registry URL format

### Performance Optimization
1. Enable Docker layer caching with `enable_cache: true`
2. Use multi-stage builds for production images
3. Optimize Dockerfile for better layer caching

## License

MIT License - see [LICENSE.md](../../LICENSE.md) for details.