# Drupal Deployment Metrics Documentation

## Overview

The bluefly.io Deployment Metrics system provides comprehensive monitoring and analysis of Drupal deployments. This documentation covers the metrics collection, analysis, and reporting capabilities of the system.

## Metrics Collection

### 1. Performance Metrics

#### PHP-FPM Metrics
```php
[
  'memory_usage' => memory_get_usage(true),
  'peak_memory' => memory_get_peak_usage(true),
  'opcache_status' => opcache_get_status(false),
  'php_version' => PHP_VERSION,
  'extensions' => get_loaded_extensions()
]
```

#### Drupal Performance
```php
[
  'cache_hits' => \Drupal::service('cache.default')->get('cache_metrics'),
  'cache_misses' => \Drupal::service('cache.default')->get('cache_metrics_miss'),
  'database_queries' => \Drupal::database()->query('SHOW STATUS LIKE "Questions"'),
  'page_cache_hits' => \Drupal::service('page_cache_kill_switch')->isTriggered(),
  'module_count' => count(\Drupal::service('extension.list.module')->getAllInstalledInfo()),
  'theme_count' => count(\Drupal::service('extension.list.theme')->getAllInstalledInfo())
]
```

#### System Metrics
```php
[
  'cpu_usage' => sys_getloadavg()[0],
  'disk_usage' => disk_free_space('/') / disk_total_space('/') * 100,
  'memory_usage' => memory_get_usage(true) / ini_get('memory_limit') * 100
]
```

### 2. Error Metrics

#### PHP Errors
```php
[
  'error_count' => count(error_get_last()),
  'error_log' => file_exists('/var/log/php_errors.log') ? file_get_contents('/var/log/php_errors.log') : null,
  'fatal_errors' => count(array_filter(error_get_last(), function($error) { 
    return $error['type'] === E_ERROR; 
  }))
]
```

#### Drupal Errors
```php
[
  'watchdog_errors' => \Drupal::database()->query("SELECT COUNT(*) FROM [watchdog] WHERE severity <= 3"),
  'recent_errors' => \Drupal::database()->query("SELECT * FROM [watchdog] WHERE severity <= 3 ORDER BY wid DESC LIMIT 10"),
  'module_errors' => array_filter(\Drupal::service('extension.list.module')->getAllInstalledInfo(), function($module) {
    return !\Drupal::service('module_handler')->moduleExists($module['name']);
  })
]
```

### 3. Cache Metrics

#### Cache Bin Statistics
```php
[
  'cache_bins' => array_map(function($bin) {
    return [
      'name' => $bin,
      'hits' => \Drupal::service('cache.' . $bin)->get('cache_metrics'),
      'misses' => \Drupal::service('cache.' . $bin)->get('cache_metrics_miss'),
      'size' => \Drupal::service('cache.' . $bin)->get('cache_size')
    ];
  }, array_keys(\Drupal::service('cache')->getBins()))
]
```

### 4. Database Metrics

#### Database Statistics
```php
[
  'query_count' => \Drupal::database()->query('SHOW STATUS LIKE "Questions"'),
  'slow_queries' => \Drupal::database()->query('SHOW STATUS LIKE "Slow_queries"'),
  'connections' => \Drupal::database()->query('SHOW STATUS LIKE "Threads_connected"'),
  'table_stats' => array_map(function($table) {
    return [
      'name' => $table,
      'rows' => \Drupal::database()->query("SELECT COUNT(*) FROM {$table}"),
      'size' => \Drupal::database()->query("SELECT data_length + index_length FROM information_schema.TABLES WHERE table_name = '[$table]'")
    ];
  }, \Drupal::database()->query("SHOW TABLES")->fetchCol())
]
```

## Metrics Reporting

### 1. JSON Reports

#### Report Structure
```json
{
  "metadata": {
    "timestamp": "2024-03-14T12:00:00Z",
    "environment": "production",
    "deployment_id": "12345",
    "commit_sha": "abc123",
    "branch": "main"
  },
  "performance": [// Performance metrics],
  "errors": [// Error metrics],
  "cache": [// Cache metrics],
  "database": [// Database metrics]
}
```

### 2. HTML Reports

#### Report Components
- Performance charts
- Error summaries
- Cache statistics
- Database metrics
- Historical trends

#### Visualization
```python
# Example chart generation
plt.figure(figsize=(12, 6))
df['performance'].plot(kind='bar')
plt.title('Deployment Performance Metrics')
plt.savefig('performance-metrics.png')
```

## Integration

### 1. Metrics Endpoint

#### API Configuration
```yaml
variables:
  METRICS_ENDPOINT: "https://metrics.bluefly.io/api/v1/metrics"
  METRICS_API_KEY: [METRICS_API_KEY]
  METRICS_RETENTION: "30d"
```

#### Endpoint Usage
```bash
curl -X POST \
  -H "Content-Type: application/json" \
  -H "Authorization: Bearer $METRICS_API_KEY" \
  -d "$METRICS_DATA" \
  "$METRICS_ENDPOINT"
```

### 2. Notification Integration

#### Slack Notifications
```yaml
variables:
  SLACK_WEBHOOK: [SLACK_WEBHOOK]
  NOTIFICATION_CHANNELS: "slack"
```

#### Email Notifications
```yaml
variables:
  EMAIL_RECIPIENTS: [EMAIL_RECIPIENTS]
  NOTIFICATION_CHANNELS: "email"
```

## Analysis and Alerts

### 1. Performance Thresholds

#### Memory Usage
```yaml
variables:
  MEMORY_THRESHOLD: 80  # Percentage
  MEMORY_ALERT: true
```

#### Cache Performance
```yaml
variables:
  CACHE_HIT_RATIO_THRESHOLD: 0.8  # 80% hit ratio
  CACHE_ALERT: true
```

#### Database Performance
```yaml
variables:
  SLOW_QUERY_THRESHOLD: 1  # Second
  QUERY_COUNT_THRESHOLD: 1000
  DATABASE_ALERT: true
```

### 2. Alert Configuration

#### Alert Types
- Performance degradation
- Error rate increase
- Cache miss ratio
- Database slow queries

#### Alert Channels
- Slack
- Email
- Webhook
- Dashboard

## Dashboard Integration

### 1. Metrics Dashboard

#### Components
- Real-time metrics
- Historical trends
- Performance graphs
- Error reports

#### Access
```yaml
environment:
  name: metrics
  url: https://metrics.bluefly.io/dashboard
```

### 2. Custom Dashboards

#### Configuration
```yaml
variables:
  DASHBOARD_TEMPLATE: "drupal-metrics"
  DASHBOARD_REFRESH: "5m"
  DASHBOARD_EXPORT: true
```

## Best Practices

### 1. Metrics Collection

- Collect metrics at appropriate intervals
- Use efficient collection methods
- Store metrics securely
- Implement data retention policies

### 2. Analysis

- Set meaningful thresholds
- Monitor trends over time
- Correlate metrics with deployments
- Identify performance patterns

### 3. Reporting

- Generate clear, actionable reports
- Include historical context
- Highlight significant changes
- Provide recommendations

### 4. Alerts

- Configure appropriate thresholds
- Use multiple notification channels
- Include relevant context
- Enable quick response

## Troubleshooting

### 1. Collection Issues

```bash
# Check metrics collection
drush @self eval "print_r(opcache_get_status());"

# Verify database access
drush @self sql-query "SHOW STATUS"

# Test cache access
drush @self cache:rebuild
```

### 2. Reporting Issues

```bash
# Check report generation
ls -l deployment-report-*.json

# Verify HTML report
cat deployment-report.html

# Test metrics endpoint
curl -I $METRICS_ENDPOINT
```

### 3. Alert Issues

```bash
# Test Slack webhook
curl -X POST -H "Content-type: application/json" \
  --data '{"text":"Test alert"}' \
  $SLACK_WEBHOOK

# Verify email configuration
echo "Test alert" | mail -s "Test" $EMAIL_RECIPIENTS
```

## Contributing

1. Fork the repository
2. Create a feature branch
3. Add your metrics
4. Update documentation
5. Submit a merge request

## Support

For support and questions:
- Email: metrics-support@bluefly.io
- Documentation: https://docs.bluefly.io/metrics
- GitLab Issues: https://gitlab.bluefly.io/llm/BFCIComponents/issues 