# CLAUDE.md

This file provides guidance to Claude Code (claude.ai/code) when working with code in this repository.

## Ecosystem Overview

The LLM directory contains the **Bluefly AI Platform**, a microservices-based ecosystem for enterprise AI orchestration. Projects work together while maintaining clear separation of concerns.

### Core Platform Architecture

```
┌─────────────────────────────────────────────────────────────────┐
│                        API Gateway (llm-gateway)                       │
│                 Routes requests to microservices                 │
└─────────────────────────────────────────────────────────────────┘
                               │
┌─────────────────┬─────────────────┬─────────────────┬───────────┐
│     bfllm       │      llm-mcp      │      bfrfp      │   bfui    │
│  Core LLM       │  Model Control  │    Content      │    UI     │
│  Inference      │     Plane       │   Generation    │Components │
└─────────────────┴─────────────────┴─────────────────┴───────────┘
                               │
┌─────────────────────────────────────────────────────────────────┐
│                    Drupal Control Plane                          │
│              Central orchestration & management                  │
└─────────────────────────────────────────────────────────────────┘
```

### Integration Rules

**CRITICAL**: Services communicate via published NPM packages, NOT file imports:
```typescript
//  CORRECT - Use published packages
import [ModelRegistry] from '@bluefly/llm';
import [ToolOrchestrator] from '@bluefly/mcp';

// ❌ WRONG - Never use file imports across projects
import [ModelRegistry] from '../bfllm/src/models/registry';
```

### Common Development Commands

#### All Node.js Projects Support:
```bash
npm run dev               # Development server with hot reload
npm run build             # Production build
npm run build:force       # Build bypassing TypeScript errors (dev only)
npm run test              # Run tests with coverage
npm run test:watch        # Tests in watch mode for TDD
npm run lint:fix          # Fix linting issues
npm run typecheck         # TypeScript validation
npm run ci                # Full CI pipeline locally
```

#### Platform Management (llmcli):
```bash
# Quick Platform Setup
llmcli llm:platform install --with-ddev --with-vector-db --with-ai-swarm

# Token Management (required for AI providers)
llmcli token load          # Load tokens from .env files
llmcli token validate      # Verify token configuration

# Development
llmcli ddev setup          # Set up DDEV environment
llmcli drupal install      # Install Drupal with recipe
```

#### Drupal Development:
```bash
# From llm_platform_recipe directory
ddev start
ddev composer install
ddev drush site-install --account-name=admin --account-pass=admin
ddev drush recipe:apply llm_platform
ddev drush cr            # Clear cache after changes
```

### Testing Standards

**TDD is mandatory** - Write tests first:
```bash
# Run specific test file
npm test -- src/services/ModelRegistry.test.ts

# Run tests matching pattern
npm test -- --testNamePattern="should register model"

# PHP/Drupal testing
./vendor/bin/phpunit --group=unit
./vendor/bin/phpunit --coverage-html coverage
```

### Build System Features

1. **TypeScript Errors During Development**: Use `npm run build:force` or `npm run build:esbuild` to bypass TypeScript errors temporarily
2. **Dual Build Support**: All projects support both CommonJS and ESM
3. **Fallback Mechanisms**: bfllm includes automatic fallbacks for failed builds

### API Development

All APIs must have OpenAPI specs:
```bash
npm run openapi:validate  # Validate OpenAPI spec
npm run openapi:docs      # Generate documentation
```

### Environment Setup

1. **Required**: Node.js 20.x LTS, Docker, Git
2. **For Drupal**: DDEV, Composer, PHP 8.3+
3. **For AI Features**: Valid API tokens for providers (OpenAI, Anthropic, etc.)

### Project-Specific Commands

**llm-gateway (API Gateway)**:
```bash
npm run generate:sdk      # Generate client SDKs
npm run swagger          # Validate API specifications
```

**bfllm (LLM Service)**:
```bash
npm run create:fallback  # Create fallback model implementations
npm run test:fallback    # Test fallback systems
```

**llm-mcp (Model Control Plane)**:
```bash
npm run test:streaming   # Test streaming implementations
npm run mcp:upgrade      # Upgrade to latest MCP protocol
```

**bfui (UI Components)**:
```bash
npm run storybook        # Component development environment
npm run test:accessibility # Accessibility compliance tests
```

### Security & Compliance

- JWT authentication with 15-minute expiry
- All inputs validated against OpenAPI schemas
- Compliance frameworks: HIPAA, GDPR, PCI, FedRAMP, SOC 2
- Never commit tokens or secrets

### File Preservation

**Never delete files**. Move to `__DELETE_LATER` with date suffix:
```bash
mv old-file.ts __DELETE_LATER/old-file_2024-01-07.ts
```

### Shared Utilities

Import from `@bluefly/cli/shared` for:
- `BlueflyLogger` - Structured logging
- `HttpClient` - Resilient HTTP client  
- `createError` - Standard error handling
- Configuration, metrics, and observability utilities

### Quick Debugging

1. **Build Issues**: Try `npm run build:force` first
2. **Type Errors**: Use `npm run typecheck` to see all errors
3. **Test Failures**: Run with `npm test -- --verbose`
4. **API Issues**: Check `npm run openapi:validate`
5. **Drupal Issues**: Always `ddev drush cr` after changes

### Performance Requirements

- API response time: <200ms
- Test coverage: 90%+ 
- Build time: <60s
- Memory usage: <2GB per service

## AI Integration Standards

### Drupal AI Module Integration

When integrating with the Drupal AI module, follow these patterns:

#### Provider Plugin Architecture
```php
namespace Drupal\your_module\Plugin\AiProvider;

use Drupal\ai\Attribute\AiProvider;
use Drupal\ai\Base\AiProviderPluginBase;

#[AiProvider(
  id: 'bluefly_ai',
  label: 'Bluefly AI Provider',
  description: 'Integrates Bluefly AI services with Drupal AI module'
)]
class BlueflyAiProvider extends AiProviderPluginBase [// Implementation following drupal/ai patterns]
```

#### Service Communication Patterns
```
┌─────────────────────────────────────────────────────────────┐
│                     Drupal AI Module                        │
│                  (Community Standard)                       │
└─────────────────────────────────────────────────────────────┘
                              │
┌─────────────────────────────┴───────────────────────────────┐
│                  Bluefly AI Provider                         │
│              (Bridges to llm-gateway/bfllm)                       │
└─────────────────────────────────────────────────────────────┘
                              │
┌──────────────┬──────────────┼──────────────┬───────────────┐
│   AI Field   │    Agent     │   Security   │    Recipes    │
│    Suite     │ Orchestrator │ Integration  │  Deployment   │
└──────────────┴──────────────┴──────────────┴───────────────┘
```

### Recipe Development Standards

#### Recipe YAML Structure
```yaml
name: 'AI Platform'
description: 'Enterprise AI platform with Drupal integration'
type: 'Site'
install:
  - ai
  - drupal_native_ai
  - ai_swarm_intelligence
  - secure_drupal
config:
  import:
    ai.settings:
      '*'
  actions:
    ai.settings:
      simple_config_update:
        default_provider: 'bluefly_ai'
```

### Security Patterns for AI Operations

#### Key Module Integration
```php
// Store API keys securely
$key = \Drupal::service('key.repository')->getKey('openai_api_key');
$api_key = $key->getKeyValue();

// Never store keys in config or database directly
```

#### Input Sanitization
```php
// Sanitize all AI inputs
$sanitized_prompt = Html::escape($user_input);
$filtered_prompt = \Drupal::service('ai.security')->filterPrompt($sanitized_prompt);
```

#### Audit Logging
```php
// Log all AI operations
\Drupal::logger('ai_audit')->info('AI operation: @operation', [
  '@operation' => $operation_type,
  'user' => \Drupal::currentUser()->id(),
  'provider' => $provider_id,
  'tokens_used' => $token_count,
]);
```

### Testing Framework

#### AI Provider Tests
```php
class BlueflyAiProviderTest extends KernelTestBase {
  protected static $modules = ['ai', 'drupal_native_ai', 'key'];
  
  public function testProviderRegistration() {
    $provider = \Drupal::service('ai.provider')->createInstance('bluefly_ai');
    $this->assertInstanceOf(BlueflyAiProvider::class, $provider);
  }
}
```

#### Recipe Application Tests
```bash
# Test recipe application
ddev drush site:install minimal --yes
ddev drush recipe:apply recipes/llm_platform
ddev drush config:status
```

### Development Workflow

#### Daily AI Development Routine
- **Morning (2h)**: Review drupal/ai module code, study provider implementations
- **Afternoon (3h)**: Implement features, write tests, integrate services
- **Evening (1h)**: Community engagement, documentation, code reviews

#### Worker Coordination Pattern
When working with multiple parallel tasks:
1. Each worker focuses on one module/component
2. Use Git branches named `worker-N-description`
3. Coordinate via merge requests
4. Test integration points frequently

### Environment Variables

Required environment variables for AI features:
```bash
# AI Provider Tokens (use Key module in production)
OPENAI_API_KEY=sk-...
ANTHROPIC_API_KEY=sk-ant-...
GEMINI_API_KEY=...

# Service URLs
llm-gateway_URL=http://localhost:3000
BFLLM_URL=http://localhost:3001
llm-mcp_URL=http://localhost:3002

# Feature Flags
ENABLE_AI_STREAMING=true
ENABLE_AI_CACHING=true
AI_RATE_LIMIT_PER_USER=100
```

### Performance Optimization

#### Caching Strategy
```php
// Cache AI responses
$cid = 'ai:response:' . md5($prompt);
if ($cache = \Drupal::cache()->get($cid)) {
  return $cache->data;
}
$response = $this->generateResponse($prompt);
\Drupal::cache()->set($cid, $response, time() + 3600);
```

#### Queue Processing
```php
// Use Drupal's queue for async AI tasks
$queue = \Drupal::queue('ai_processing');
$queue->createItem([
  'prompt' => $prompt,
  'callback' => 'process_ai_response',
]);
```

### Compliance Requirements

- **HIPAA**: PII detection and redaction required
- **GDPR**: User consent and data deletion capabilities
- **SOC 2**: Comprehensive audit logging
- **FedRAMP**: Encryption at rest and in transit

### Community Standards

When preparing for drupal.org contribution:
1. Follow Drupal coding standards (use phpcs)
2. Minimum 80% test coverage
3. Security team review required
4. Documentation in README.md
5. Hook implementations must be documented

### Quick Reference Commands

```bash
# AI Module Development
ddev composer require drupal/ai
ddev drush en ai ai_explorer -y
ddev drush config:set ai.settings default_provider bluefly_ai

# Testing AI Integration
ddev drush ai:explorer
ddev drush queue:run ai_processing

# Security Checks
ddev drush security:review
ddev drush audit:log ai_operations
```