# bluefly.io Domain Configuration

This document outlines all domains used in the bluefly.io LLM Platform ecosystem. It serves as the central reference for domain configurations across all projects.

## Domain Overview

| Domain | Project | Purpose | Environment | Configuration |
|--------|---------|---------|------------|---------------|
| llm - docs.bluefly.io | BFDOCS | Central documentation | Production | GitLab Pages |
| api.bluefly.io | llm-gateway (ARCHIVED - migrate to LangChain ecosystem) | Kubernetes |
| cli.bluefly.io | llmcli | CLI documentation | Production | GitLab Pages |
| insights.bluefly.io | Bluefly_insights | Analytics dashboard | Production | Drupal |
| dashboard.bluefly.io | acui | Admin dashboard | Production | Kubernetes |
| registry.bluefly.io | bfllm | Model registry | Production | Kubernetes |
| mlflow.bluefly.io | bfllm | MLflow tracking | Production | Kubernetes |
| mcp.bluefly.io | llm-mcp | MCP server | Production | Kubernetes |

## Domain Configuration Details

### llm - docs.bluefly.io

**Project**: BFDOCS
**Purpose**: Central documentation hub for all bluefly.io components
**Deployment**: GitLab Pages via CI / CD pipeline
**Configuration**:

- CNAME record pointing to GitLab Pages
- SSL certificate managed by GitLab
- Documentation built with MkDocs

**CI / CD Integration**:

```yaml
pages:
  stage: deploy
  script:
    - pip install -r requirements.txt
    - mkdocs build
  artifacts:
    paths:
      - public
  only:
    - main
```text

### api.bluefly.io
**Project**: llm-gateway
**Purpose**: API endpoints for bluefly.io platform
**Deployment**: Kubernetes cluster
**Configuration**:
- A record pointing to Kubernetes ingress
- SSL certificate managed by Let's Encrypt
- API gateway with rate limiting and monitoring

**CI / CD Integration**:

```yaml
deploy_production:
  stage: deploy
  script:
    - kubectl apply -f k8s / deployment.yaml
    - kubectl apply -f k8s / service.yaml
    - kubectl apply -f k8s / ingress.yaml
  environment:
    name: production
    url: https://api.bluefly.io
  only:
    - main
```text

### cli.bluefly.io
**Project**: llmcli
**Purpose**: CLI documentation and downloads
**Deployment**: GitLab Pages
**Configuration**:
- CNAME record pointing to GitLab Pages
- SSL certificate managed by GitLab
- Documentation built with VuePress

**CI / CD Integration**:

```yaml
pages:
  stage: deploy
  script:
    - npm ci
    - npm run docs:build
  artifacts:
    paths:
      - public
  only:
    - main
```text

### insights.bluefly.io
**Project**: Bluefly_insights
**Purpose**: Analytics dashboard and insights
**Deployment**: Drupal site on managed hosting
**Configuration**:
- A record pointing to hosting provider
- SSL certificate managed by hosting provider
- Drupal 9 with custom modules

**CI / CD Integration**:

```yaml
deploy_production:
  stage: deploy
  script:
    - composer install --no - dev --optimize - autoloader
    - drush deploy
  environment:
    name: production
    url: https://insights.bluefly.io
  only:
    - main
```text

### dashboard.bluefly.io
**Project**: acui
**Purpose**: Admin dashboard for platform management
**Deployment**: Kubernetes cluster
**Configuration**:
- A record pointing to Kubernetes ingress
- SSL certificate managed by Let's Encrypt
- React application with SSR

**CI / CD Integration**:

```yaml
deploy_production:
  stage: deploy
  script:
    - npm ci
    - npm run build
    - kubectl apply -f k8s / deployment.yaml
    - kubectl apply -f k8s / service.yaml
    - kubectl apply -f k8s / ingress.yaml
  environment:
    name: production
    url: https://dashboard.bluefly.io
  only:
    - main
```text

### registry.bluefly.io
**Project**: bfllm
**Purpose**: Model registry for trained models
**Deployment**: Kubernetes cluster
**Configuration**:
- A record pointing to Kubernetes ingress
- SSL certificate managed by Let's Encrypt
- Container with model registry server

**CI / CD Integration**:

```yaml
deploy_registry:
  stage: deploy
  script:
    - docker build -t registry.gitlab.bluefly.io / Bluefly / bfllm / registry:$CI_COMMIT_SHA -f Dockerfile.registry .
    - docker push registry.gitlab.bluefly.io / Bluefly / bfllm / registry:$CI_COMMIT_SHA
    - kubectl set image deployment / registry - deployment registry = registry.gitlab.bluefly.io / Bluefly / bfllm / registry:$CI_COMMIT_SHA
  environment:
    name: production
    url: https://registry.bluefly.io
  only:
    - main
```text

### mlflow.bluefly.io
**Project**: bfllm
**Purpose**: MLflow tracking server for experiment tracking
**Deployment**: Kubernetes cluster
**Configuration**:
- A record pointing to Kubernetes ingress
- SSL certificate managed by Let's Encrypt
- MLflow server with database backend

**CI / CD Integration**:

```yaml
deploy_mlflow:
  stage: deploy
  script:
    - docker build -t registry.gitlab.bluefly.io / Bluefly / bfllm / mlflow:$CI_COMMIT_SHA -f Dockerfile.mlflow .
    - docker push registry.gitlab.bluefly.io / Bluefly / bfllm / mlflow:$CI_COMMIT_SHA
    - kubectl set image deployment / mlflow - deployment mlflow = registry.gitlab.bluefly.io / Bluefly / bfllm / mlflow:$CI_COMMIT_SHA
  environment:
    name: production
    url: https://mlflow.bluefly.io
  only:
    - main
```text

### mcp.bluefly.io
**Project**: llm-mcp
**Purpose**: Model Context Protocol server
**Deployment**: Kubernetes cluster
**Configuration**:
- A record pointing to Kubernetes ingress
- SSL certificate managed by Let's Encrypt
- MCP server with tool registration

**CI / CD Integration**:

```yaml
deploy_production:
  stage: deploy
  script:
    - npm ci
    - npm run build
    - docker build -t registry.gitlab.bluefly.io / Bluefly / llm-mcp:$CI_COMMIT_SHA .
    - docker push registry.gitlab.bluefly.io / Bluefly / llm-mcp:$CI_COMMIT_SHA
    - kubectl set image deployment / mcp - deployment mcp = registry.gitlab.bluefly.io / Bluefly / llm-mcp:$CI_COMMIT_SHA
  environment:
    name: production
    url: https://mcp.bluefly.io
  only:
    - main
```text

## Non - Production Environments

### Staging Environment
Each domain has a staging version using the pattern `service-name - staging.bluefly.io`. For example:
- api - staging.bluefly.io
- dashboard - staging.bluefly.io
- mcp - staging.bluefly.io

Staging environments are updated automatically from the `develop`branch.

**CI / CD Integration**:```yaml
deploy_staging:
  stage: deploy
  script:
    - # Deployment scripts similar to production
  environment:
    name: staging
    url: https://[CI_PROJECT_NAME] - staging.bluefly.io
  only:
    - develop

```text

### Development Environment
For local development, the following patterns are used:

1. **Local Development**:
   - API Server: http://localhost:3000
   - UI Dashboard: http://localhost:3001
   - Documentation: http://localhost:8000
   - MCP Server: http://localhost:3002

2. **Review Environments**:
   When a merge request is created, dynamic review environments are provisioned with the following pattern:
   - `[CI_PROJECT_NAME] - [CI_MERGE_REQUEST_ID].review.bluefly.io`**CI / CD Integration**:
  ```yaml

   deploy_review:
     stage: deploy
     script:
       - # Deployment scripts for review environment
     environment:
       name: review / $CI_MERGE_REQUEST_ID
       url: https://[CI_PROJECT_NAME] - [CI_MERGE_REQUEST_ID].review.bluefly.io
       on_stop: stop_review
     only:
       - merge_requests

   ```text

## DNS Configuration Management
DNS records for all domains are managed through GitLab infrastructure - as - code located in the `Bluefly - infrastructure`repository.

Example DNS configuration:```yaml

# Route53 configuration example

resource "aws_route53_record" "api" [zone_id  =  aws_route53_zone.Bluefly_io.zone_id
  name     =  "api.bluefly.io"
  type     =  "A"

  alias {
    name                    =  kubernetes_ingress.api_ingress.load_balancer_ingress[0].hostname
    zone_id                 =  kubernetes_ingress.api_ingress.load_balancer_ingress[0].zone_id
    evaluate_target_health  =  true]
}
```text

## SSL Certificate Management
SSL certificates are managed through the following methods:

1. **GitLab Pages Domains**: Managed automatically by GitLab for *.pages.gitlab.io domains
2. **Kubernetes Ingress**: Managed through cert - manager with Let's Encrypt integration
3. **Drupal Hosting**: Managed by the hosting provider

## Domain Validation
A validation script is available in the BFCIComponents repository to verify domain configurations:

```bash

# Validate all domain configurations

llmcli domains verify - all

# Validate a specific domain

llmcli domains verify api.bluefly.io
```text

## Adding New Domains
To add a new domain to the bluefly.io ecosystem:

1. Create a merge request in the `Bluefly - infrastructure`repository with the DNS configuration
2. Update this domains.md file with the new domain information
3. Configure the appropriate CI / CD pipeline for deployment
4. Verify the domain is accessible with`llmcli domains verify <new - domain > `All domains must follow the` * .bluefly.io` pattern for consistency and must be associated with a specific project in the ecosystem.
