# Component Cleanup and Reorganization Plan

## Current Issues

- 137 template files across 37+ directories
- Multiple duplicates (nodejs, drupal, docs, etc.)
- Inconsistent naming conventions
- Poor categorization
- Legacy/unused components mixed with active ones

## Proposed Clean Structure

```
components/
├── README.md                    # Component usage guide
├── ai/                         # AI & ML Components
│   ├── base/                   # Foundation AI template
│   ├── openai/                 # OpenAI integration
│   ├── anthropic/              # Anthropic integration
│   ├── rag/                    # Retrieval Augmented Generation
│   ├── vector-stores/          # Vector database management
│   ├── embedding/              # Embedding services
│   └── model-training/         # Model training pipelines
├── ci/                         # CI/CD Pipeline Components
│   ├── base/                   # Base CI template
│   ├── testing/                # Test automation
│   ├── quality/                # Code quality checks
│   ├── security/               # Security scanning
│   ├── deployment/             # Deployment automation
│   └── monitoring/             # Pipeline monitoring
├── platforms/                  # Platform-specific Components
│   ├── drupal/                 # Drupal-specific templates
│   ├── nodejs/                 # Node.js templates
│   ├── python/                 # Python templates
│   ├── php/                    # PHP templates
│   └── docker/                 # Docker/containerization
├── infrastructure/             # Infrastructure Components
│   ├── k8s/                    # Kubernetes deployments
│   ├── terraform/              # Terraform automation
│   ├── monitoring/             # Infrastructure monitoring
│   └── backup/                 # Backup and recovery
├── api/                        # API Components
│   ├── rest/                   # REST API templates
│   ├── graphql/                # GraphQL templates
│   ├── validation/             # API validation
│   └── documentation/          # API docs generation
└── shared/                     # Shared Utilities
    ├── base/                   # Base template utilities
    ├── auth/                   # Authentication helpers
    ├── logging/                # Logging utilities
    └── config/                 # Configuration management
```

## Cleanup Actions

### 1. Move to __DELETE_LATER (Date: 2025-06-10)

- `nodejs-base/` (duplicate of nodejs/base)
- `nodejs-build/` (duplicate functionality)
- `nodejs-test/` (duplicate of ci/testing)
- `docs/docs-legacy/` (legacy)
- `docs/docs/` (duplicate)
- `drupal/drupal-tdd-legacy/` (legacy)
- `documentation/` (duplicate of docs)
- `base-template/` (merge into shared/base)
- `validation/` (merge into ci/quality)
- `optimization/` (merge into ci/performance)
- `model-management/` (merge into ai/)
- `llm/` (merge into ai/)
- `CIMLPipeline/` (poorly named, merge into ci/)
- `claude-autotest/` (merge into ci/testing)
- `tdd-validation/` (merge into ci/testing)
- `shared-utils/` (merge into shared/)

### 2. Consolidate Duplicates

- Merge all nodejs variants into `platforms/nodejs/`
- Merge all drupal variants into `platforms/drupal/`
- Merge all docs variants into `api/documentation/`
- Merge monitoring variants into `infrastructure/monitoring/`

### 3. Rename and Reorganize

- `security-scan/` → `ci/security/`
- `terraform-deploy/` → `infrastructure/terraform/`
- `dependency-update/` → `ci/maintenance/`
- `docker-service/` → `platforms/docker/`
- `npm-package/` → `platforms/nodejs/package/`
- `authentication/` → `shared/auth/`
- `logging/` → `shared/logging/`

### 4. Keep These Core Categories

- `ai/` (newly organized)
- `k8s/` → `infrastructure/k8s/`
- `gitlab/` → `ci/gitlab/`
- `llmcli/` (project-specific)
- `todo-aggregator/` (utility)
- `model-registry/` (utility)

## Benefits After Cleanup

- ~25 clean component categories instead of 37+ messy ones
- Clear naming conventions
- Logical grouping by purpose
- No duplicates
- Easier discovery and maintenance
- Better documentation structure

## Implementation Timeline

### Phase 1: Analysis and Planning (Week 1)

- [ ] Audit all existing components
- [ ] Identify duplicates and legacy components
- [ ] Create detailed migration plan
- [ ] Set up backup of current structure

### Phase 2: Cleanup and Consolidation (Week 2)

- [ ] Move legacy components to __DELETE_LATER
- [ ] Consolidate duplicate components
- [ ] Rename components according to new structure
- [ ] Update component documentation

### Phase 3: Reorganization (Week 3)

- [ ] Create new directory structure
- [ ] Move components to new locations
- [ ] Update all references and imports
- [ ] Test all components in new structure

### Phase 4: Documentation and Validation (Week 4)

- [ ] Update component documentation
- [ ] Create new component index
- [ ] Validate all components work correctly
- [ ] Update CI/CD pipelines

## Success Criteria

- [ ] All components follow new naming convention
- [ ] No duplicate components exist
- [ ] All legacy components moved to __DELETE_LATER
- [ ] Component documentation is up to date
- [ ] All CI/CD pipelines pass with new structure
- [ ] Component discovery is improved
- [ ] Maintenance overhead is reduced
