# GitLab CI/CD Components Library Reorganization Plan

## Overview

This plan reorganizes the bfcicomponents directory to follow GitLab CI/CD component best practices, creating a world-class component library focused on Drupal 11, TypeScript, Python, AI, and NPM packages.

## New Structure

```
bfcicomponents/
├── .gitlab-ci.yml                 # Component catalog definition
├── README.md                      # Library documentation
├── CONTRIBUTING.md                # Contribution guidelines
├── LICENSE                        # License file
├── CHANGELOG.md                   # Version history
│
├── components/                    # All CI/CD components
│   ├── README.md                 # Component overview
│   │
│   ├── drupal/                   # Drupal-specific components
│   │   ├── build/
│   │   │   └── template.yml      # Drupal build component
│   │   ├── test/
│   │   │   └── template.yml      # Drupal test component
│   │   ├── deploy/
│   │   │   └── template.yml      # Drupal deployment
│   │   ├── security/
│   │   │   └── template.yml      # Drupal security scanning
│   │   ├── quality/
│   │   │   └── template.yml      # Drupal code quality
│   │   └── module/
│   │       └── template.yml      # Drupal module CI/CD
│   │
│   ├── nodejs/                   # Node.js/TypeScript components
│   │   ├── build/
│   │   │   └── template.yml      # Node.js build
│   │   ├── test/
│   │   │   └── template.yml      # Jest/Vitest testing
│   │   ├── publish/
│   │   │   └── template.yml      # NPM publishing
│   │   ├── lint/
│   │   │   └── template.yml      # ESLint/Prettier
│   │   └── security/
│   │       └── template.yml      # npm audit
│   │
│   ├── python/                   # Python components
│   │   ├── build/
│   │   │   └── template.yml      # Python build
│   │   ├── test/
│   │   │   └── template.yml      # pytest
│   │   ├── lint/
│   │   │   └── template.yml      # flake8/black
│   │   └── publish/
│   │       └── template.yml      # PyPI publishing
│   │
│   ├── ai/                       # AI/ML components
│   │   ├── ollama/
│   │   │   └── template.yml      # Ollama integration
│   │   ├── langchain/
│   │   │   └── template.yml      # LangChain pipelines
│   │   ├── model-training/
│   │   │   └── template.yml      # Model training
│   │   └── documentation/
│   │       └── template.yml      # AI doc generation
│   │
│   ├── docker/                   # Container components
│   │   ├── build/
│   │   │   └── template.yml      # Docker build
│   │   ├── scan/
│   │   │   └── template.yml      # Security scanning
│   │   └── publish/
│   │       └── template.yml      # Registry push
│   │
│   ├── kubernetes/               # K8s components
│   │   ├── deploy/
│   │   │   └── template.yml      # K8s deployment
│   │   ├── helm/
│   │   │   └── template.yml      # Helm charts
│   │   └── validate/
│   │       └── template.yml      # Manifest validation
│   │
│   └── shared/                   # Shared base components
│       ├── setup/
│       │   └── template.yml      # Common setup
│       ├── cache/
│       │   └── template.yml      # Caching strategies
│       └── artifacts/
│           └── template.yml      # Artifact handling
│
├── tests/                        # Component tests
│   ├── unit/                     # Unit tests
│   ├── integration/              # Integration tests
│   └── fixtures/                 # Test fixtures
│
├── docs/                         # Documentation
│   ├── components/               # Component docs
│   ├── examples/                 # Usage examples
│   ├── guides/                   # How-to guides
│   └── api/                      # API reference
│
├── tools/                        # Development tools
│   ├── validate-components.js    # Component validator
│   ├── generate-catalog.js       # Catalog generator
│   └── create-component.js       # Component creator
│
└── examples/                     # Example projects
    ├── drupal-module/            # Example Drupal module
    ├── nodejs-package/           # Example Node.js package
    └── python-package/           # Example Python package
```

## Component Template Structure

Each component follows this structure:

```yaml
# components/[category]/[component]/template.yml
spec:
  inputs:
    # Comprehensive input definitions with defaults, validation, and descriptions
    stage:
      type: string
      default: 'test'
      description: 'Pipeline stage name'
      options: ['build', 'test', 'deploy']
    
    node_version:
      type: string
      default: '20'
      description: 'Node.js version'
      pattern: '^\d+(\.\d+)?$'
    
    environment:
      type: string
      default: 'production'
      description: 'Deployment environment'
      options: ['development', 'staging', 'production']
    
    enable_cache:
      type: boolean
      default: true
      description: 'Enable dependency caching'

---
# Component implementation
.component-name:
  stage: $[[ inputs.stage ]]
  image: node:$[[ inputs.node_version ]]
  variables:
    NODE_ENV: $[[ inputs.environment ]]
  cache:
    - if: $[[ inputs.enable_cache ]]
      key: "[CI_COMMIT_REF_SLUG]"
      paths:
        - node_modules/
  script:
    - echo "Running component"
```

## Key Components to Create

### 1. Drupal Components

#### drupal/build

```yaml
spec:
  inputs:
    php_version:
      default: '8.3'
    composer_version:
      default: '2'
    drupal_version:
      default: '11'
    enable_patches:
      default: true
    custom_repositories:
      type: array
      default: []
```

#### drupal/test

```yaml
spec:
  inputs:
    test_suite:
      default: 'all'
      options: ['unit', 'kernel', 'functional', 'javascript', 'all']
    coverage:
      default: true
    phpunit_version:
      default: '10'
    browser:
      default: 'chrome'
      options: ['chrome', 'firefox']
```

#### drupal/deploy

```yaml
spec:
  inputs:
    environment:
      default: 'production'
      options: ['development', 'staging', 'production']
    deployment_method:
      default: 'rsync'
      options: ['rsync', 'git', 'composer']
    backup_enabled:
      default: true
```

### 2. Node.js Components

#### nodejs/build

```yaml
spec:
  inputs:
    node_version:
      default: '20'
    package_manager:
      default: 'npm'
      options: ['npm', 'yarn', 'pnpm']
    build_script:
      default: 'build'
    enable_cache:
      default: true
```

#### nodejs/test

```yaml
spec:
  inputs:
    test_framework:
      default: 'jest'
      options: ['jest', 'vitest', 'mocha']
    coverage_threshold:
      default: 80
    parallel_tests:
      default: true
```

### 3. AI Components

#### ai/ollama

```yaml
spec:
  inputs:
    model:
      default: 'llama2'
    context_length:
      default: 4096
    temperature:
      default: 0.7
    max_tokens:
      default: 1000
```

#### ai/langchain

```yaml
spec:
  inputs:
    chain_type:
      default: 'llm'
      options: ['llm', 'retrieval', 'conversation']
    vector_store:
      default: 'chroma'
    embedding_model:
      default: 'sentence-transformers'
```

## Implementation Strategy

### Phase 1: Foundation (Week 1-2)

- [ ] Set up new directory structure
- [ ] Create base component templates
- [ ] Implement component validation tools
- [ ] Set up testing framework

### Phase 2: Core Components (Week 3-4)

- [ ] Create Drupal components
- [ ] Create Node.js components
- [ ] Create Python components
- [ ] Create shared components

### Phase 3: Advanced Components (Week 5-6)

- [ ] Create AI/ML components
- [ ] Create Docker components
- [ ] Create Kubernetes components
- [ ] Create infrastructure components

### Phase 4: Documentation & Examples (Week 7-8)

- [ ] Complete component documentation
- [ ] Create usage examples
- [ ] Create migration guides
- [ ] Set up component catalog

## Benefits

### For Developers

- **Consistent Patterns**: All components follow the same structure
- **Easy Discovery**: Clear categorization and naming
- **Comprehensive Inputs**: Well-defined parameters with validation
- **Reusable**: Components can be used across multiple projects

### For Projects

- **Faster Setup**: Pre-built components reduce setup time
- **Best Practices**: Components implement industry standards
- **Maintainable**: Centralized component management
- **Scalable**: Easy to add new components and categories

### For CI/CD

- **Standardized**: Consistent pipeline structure
- **Reliable**: Tested and validated components
- **Flexible**: Configurable for different use cases
- **Observable**: Built-in monitoring and logging

## Migration Plan

### From Current Structure

1. **Audit**: Identify all existing components
2. **Categorize**: Map to new structure
3. **Migrate**: Move and update components
4. **Test**: Validate all components work
5. **Document**: Update all documentation

### Timeline

- **Week 1**: Audit and planning
- **Week 2**: Foundation setup
- **Week 3-4**: Core component migration
- **Week 5-6**: Advanced component migration
- **Week 7-8**: Documentation and validation

## Success Metrics

- [ ] All components follow new structure
- [ ] Component catalog is functional
- [ ] Documentation is complete
- [ ] Examples are working
- [ ] Migration is successful
- [ ] Community adoption begins
