# Semantic Release Component

The `semantic - release.yml`component provides automated semantic versioning and release management for GitLab CI / CD pipelines. It automatically determines the next version number, generates release notes, and publishes the package to the GitLab package registry.

## Overview

This component follows the [Semantic Versioning](<https://semver.org> / ) specification and uses [semantic - release](<https://semantic> - release.gitbook.io / semantic - release / ) to automate the versioning process. It analyzes commit messages to determine the appropriate version bump (major, minor, or patch) and generates release notes based on the commit history.

## Usage

To use this component in your GitLab CI / CD pipeline, include it in your`.gitlab - ci.yml`file:

```yaml
include:
  - component: 'gitlab://bluefly / BFCIComponents / components / semantic - release.yml'
    inputs:
      release_branches: 'main'
      create_gitlab_release: true
      publish_to_registry: true
```text

## Input Parameters
| Parameter               | Type    | Default                                                                                          | Options                                 | Description                                       |
| ----------------------- | ------- | ------------------------------------------------------------------------------------------------ | --------------------------------------- | ------------------------------------------------- |
| `enabled`| boolean |`true`|`true`, `false`| Enable semantic release processing                |
|`release_branches`| string  |`main`| any branch name                         | Branches that trigger a release                   |
|`default_bump`| string  |`patch`|`patch`, `minor`, `major`| Default bump when no conventional commit is found |
|`create_gitlab_release`| boolean |`true`|`true`, `false`| Create a GitLab release for each release          |
|`annotated_tags`| boolean |`true`|`true`, `false`| Create annotated git tags                         |
|`publish_to_registry`| boolean |`true`|`true`, `false`| Publish packages to the GitLab registry           |
|`npm_token_access`| string  |`automation`|`automation`, `maintainer`, `developer`| NPM token access level                            |
|`use_custom_image`| boolean |`true`|`true`, `false`| Use the Bluefly custom Docker image               |
|`custom_image_path`| string  |`registry.gitlab.bluefly.io / bluefly / internal - projects / core - assets / Bluefly - docker - image:latest` | any Docker image path                   | Path to the custom Docker image                   |

## Jobs
This component defines the following jobs:

- **prepare_release**: Analyzes commits to determine the next version number (dry run)
- **semantic_release**: Creates the release, updates the package version, creates a Git tag, and publishes the package

## Commit Message Format
This component uses the [Conventional Commits](https://www.conventionalcommits.org / ) format to determine version bumps:

- **major**: Breaking changes (`BREAKING CHANGE:` in the commit body)
- **minor**: New features (`feat:` prefix)
- **patch**: Bug fixes (`fix:` prefix), documentation changes (`docs:`prefix), and other changes

Example commit messages:

```text
feat: add new feature X
fix: resolve issue with Y
docs: update documentation for Z
style: improve code formatting
refactor: restructure component A
perf: optimize function B
test: add tests for C
build: update build process
ci: improve CI pipeline
chore: update dependencies
```text

## Requirements
To use this component, your project should:

1. Use Node.js and have a `package.json`file
2. Use Git for version control
3. Have access to the GitLab API for creating releases
4. Have access to the GitLab Package Registry for publishing packages

## Configuration

### GitLab CI / CD Variables
Set the following GitLab CI / CD variables in your project settings:

-`CI_JOB_TOKEN`(automatically set by GitLab)
-`NPM_TOKEN`(for publishing to the GitLab Package Registry, set automatically)

### .releaserc.json
The component automatically creates a`.releaserc.json`file with the following configuration:

```json
{
  "branches": ["main"],
  "plugins": [
    "@semantic - release / commit - analyzer",
    "@semantic - release / release - notes - generator",
    "@semantic - release / changelog",
    "@semantic - release / npm",
    "@semantic - release / gitlab",
    "@semantic - release / git"
  ]
}
```text

## Example Configuration

```yaml
include:
  - component: 'gitlab://bluefly / BFCIComponents / components / nodejs - base.yml'
    inputs:
      node_version: '20'
      project_type: 'app'
      use_custom_image: true
      custom_image_path: 'registry.gitlab.bluefly.io / bluefly / internal - projects / core - assets / Bluefly - docker - image:latest'

  - component: 'gitlab://bluefly / BFCIComponents / components / semantic - release.yml'
    inputs:
      release_branches: 'main'
      create_gitlab_release: true
      publish_to_registry: true

# Additional jobs specific to your project

test:
  extends: .node_base
  stage: test
  script:
    - npm run test
```text

## Changelog Management
This component automatically updates the `CHANGELOG.md` file with each release, which includes:

- Version number
- Release date
- Detailed list of changes, grouped by type (features, fixes, etc.)
- Links to issues and merge requests

## Troubleshooting

### No Release is Created
If no release is created, check the following:

1. Ensure your commit messages follow the [Conventional Commits](https://www.conventionalcommits.org / ) format
2. Verify that you're pushing to the release branch specified in the component configuration
3. Check the CI / CD logs for any errors from the semantic - release tool

### Release Creation Fails
If the release creation fails, check:

1. GitLab API access permissions
2. GitLab Package Registry access permissions
3. Git push permissions for the release branch
