# GitLab CI / CD Workflow with llmcli

This document explains how to use the llmcli workflow commands to manage CI / CD components and automate common GitLab tasks.

## Installation

Make sure you have the llmcli installed:

```bash
npm install -g @bluefly / llmcli
```text

## CI / CD Component Commands
llmcli provides dedicated commands for working with CI / CD components in the BFCIComponents repository.

### Validating Components
Validate that all components conform to the required structure and standards:

```bash

# Validate all components

llmcli workflow validate - components

# Validate components in a specific repository

llmcli workflow validate - components --project BFCIComponents
```text

The validation checks:

- Proper component format with spec section
- Required inputs definition
- Component metadata
- Custom image support
- Default values for inputs
- Documentation

### Listing Components
List all available CI / CD components:

```bash

# List all components

llmcli workflow list - components

# List components in a specific repository

llmcli workflow list - components --project BFCIComponents
```text

The output shows:

- Component name
- File path
- Description
- Whether it has proper metadata

### Publishing Components
Publish a component to the GitLab CI / CD Catalog:

```bash

# Publish a component with automatic versioning

llmcli workflow publish - component nodejs - base

# Publish with a specific tag and message

llmcli workflow publish - component nodejs - base --tag v1.2.0 --message "Add support for Node.js 20"

# Publish to a specific project

llmcli workflow publish - component nodejs - base --project BFCIComponents

# Publish with detailed description

llmcli workflow publish - component nodejs - base --description "This release adds support for Node.js 20 and improves caching"

# Skip validation (not recommended)

llmcli workflow publish - component nodejs - base --no - validate
```text

Publishing:

1. Validates the component (unless --no - validate is specified)
2. Creates a git tag
3. Pushes the tag to the remote repository
4. Creates a GitLab release
5. Makes the component available in the CI / CD Catalog

## Other Workflow Commands
llmcli includes many additional workflow commands for GitLab:

### Merge Request Commands

```bash

# Create a merge request

llmcli workflow mr - create --title "Fix build issue" --description "This fixes the CI build failure"

# List all merge requests

llmcli workflow mr - list

# Update a merge request

llmcli workflow mr - update 123 --title "New title"

# Merge a merge request

llmcli workflow mr - merge 123 --delete - branch

# Close a merge request

llmcli workflow mr - close 123
```text

### Pipeline Commands

```bash

# Run a GitLab pipeline

llmcli workflow pipeline - run --branch main

# List pipelines

llmcli workflow pipeline - list --status running
```text

### Issue Commands

```bash

# Create an issue

llmcli workflow issue - create --title "Fix documentation" --description "The docs are incomplete"

# List issues

llmcli workflow issue - list
```text

### Release Commands

```bash

# Create a release

llmcli workflow release - create --tag v1.0.0 --name "Version 1.0.0" --description "Initial release"
```text

## Using in CI / CD Pipelines
The workflow commands can be used in CI / CD pipelines by installing llmcli in the CI environment:

```yaml
before_script:
  - npm install -g @bluefly / llmcli
  - llmcli --version

validate - job:
  script:
    - llmcli workflow validate - components
```text

## Automation Tips
1. To trigger a component release from CI, include a special prefix in your commit message:

   ```text
   release: Update nodejs - base component
   ```text

2. To trigger documentation updates, include a special prefix:

   ```text
   docs: Update component documentation
   ```text

3. For bulk operations across multiple projects, use the `--projects`option:

  ```bash
   llmcli workflow setup - all --projects llm-gateway, bfllm, acui
   ```text
