---
id: components-bfcicomponents-components-security
---
# Security Component

The security component provides standardized security scanning and compliance checks for your GitLab CI/CD pipelines.

## Usage

Include the security component in your `.gitlab-ci.yml`:

```yaml
include:
  - component: ${COMPONENT_REGISTRY}/security@1.0.0
    inputs:
      scan_type: 'sast'
      target_dirs: 'src'
```

## Configuration

### Required Inputs

| Input       | Description           | Default |
| ----------- | --------------------- | ------- |
| `scan_type` | Type of security scan | None    |

### Optional Inputs

| Input                | Description                        | Default |
| -------------------- | ---------------------------------- | ------- |
| `target_dirs`        | Directories to scan                | `src`   |
| `severity_threshold` | Minimum severity to report         | `low`   |
| `fail_on_severity`   | Severity level that causes failure | `high`  |
| `scan_timeout`       | Scan timeout in seconds            | `1800`  |
| `exclude_patterns`   | Patterns to exclude                | None    |
| `report_format`      | Format for scan reports            | `json`  |

## Scan Types

### SAST (Static Application Security Testing)

```yaml
security-sast:
  extends: .security
  variables:
    SCAN_TYPE: 'sast'
    TARGET_DIRS: 'src'
    SEVERITY_THRESHOLD: 'low'
```

### Dependency Scanning

```yaml
security-deps:
  extends: .security
  variables:
    SCAN_TYPE: 'dependency'
    TARGET_DIRS: '.'
    FAIL_ON_SEVERITY: 'critical'
```

### Container Scanning

```yaml
security-container:
  extends: .security
  variables:
    SCAN_TYPE: 'container'
    TARGET_IMAGE: '${CI_REGISTRY_IMAGE}:${CI_COMMIT_SHA}'
```

### Secret Detection

```yaml
security-secrets:
  extends: .security
  variables:
    SCAN_TYPE: 'secret'
    EXCLUDE_PATTERNS: '*.test.js,*.spec.js'
```

## Reports

Security reports are automatically generated:

```yaml
artifacts:
  reports:
    security: gl-security-report.json
    dependency: gl-dependency-scanning-report.json
    container: gl-container-scanning-report.json
    sast: gl-sast-report.json
```

## Environment Variables

### Available Variables

| Variable        | Description             |
| --------------- | ----------------------- |
| `GITLAB_TOKEN`  | GitLab API token        |
| `SCAN_REGISTRY` | Security tools registry |
| `REPORT_PATH`   | Path for report output  |
| `SCAN_RULES`    | Custom scanning rules   |

### Custom Variables

You can set custom environment variables:

```yaml
variables:
  MY_SECURITY_VAR: 'value'
```

## Best Practices

1. Scan Configuration

   - Set appropriate thresholds
   - Configure exclusions
   - Use multiple scan types

2. Report Management

   - Review reports regularly
   - Track security trends
   - Document false positives

3. Security Policies
   - Define security standards
   - Implement compliance checks
   - Maintain audit logs

## Compliance

### Standards Support

- OWASP Top 10
- CWE/SANS Top 25
- PCI DSS
- HIPAA
- SOC 2

### Compliance Reporting

```yaml
security-compliance:
  extends: .security
  variables:
    SCAN_TYPE: 'compliance'
    COMPLIANCE_STANDARD: 'pci-dss'
```

## Troubleshooting

### Common Issues

1. Scan Failures

   - Check scan logs
   - Verify tool versions
   - Review configurations

2. False Positives

   - Document exceptions
   - Update exclusions
   - Tune thresholds

3. Performance Issues
   - Optimize scan scope
   - Adjust timeouts
   - Use caching

## Support

Need help? Check our [Support Guide](../guide/support) or [create an issue](https://gitlab.bluefly.io/blueflyio/internal-projects/llm/bfcicomponents/-/issues).
