---
id: components-bfcicomponents-documentation
---
# Documentation Component

The `documentation` component provides standardized jobs for generating, publishing, and validating documentation in GitLab CI/CD pipelines.

## Purpose

This component:

- Generates project documentation
- Publishes documentation to a central repository
- Validates documentation consistency
- Supports API documentation generation
- Supports user guide generation
- Optionally creates PDF documentation

## Inputs

| Input                    | Description                                                   | Default                     | Options                       |
| ------------------------ | ------------------------------------------------------------- | --------------------------- | ----------------------------- | -------------------------------------- | ----------------- |
| `stage`                  | Stage for documentation jobs                                  | `"docs"`                    | Any valid stage name          |
| `docs_source_dir`        | Directory containing documentation source files               | `"docs"`                    | Any directory path            |
| `docs_dest_dir`          | Destination directory in central documentation repository     | `"projects/{project_name}"` | Any directory path            |
| `docs_repo`              | Central documentation repository path                         | `"blueflyio/bfdocs"`        | Any valid GitLab project path |
| `docs_branch`            | Branch in central documentation repository to push to         | `"main"`                    | Any branch name               |
| `enable_api_docs`        | Whether to enable API documentation generation                | `false`                     | `true`, `false`               |
| `api_docs_source`        | API documentation source directory or file                    | `"docs/api"`                | Any directory or file path    |
| `api_docs_dest`          | Destination path for API documentation in central repository  | `"api/{project_name}"`      | Any directory path            |
| `enable_user_docs`       | Whether to enable user documentation generation               | `false`                     | `true`, `false`               |
| `user_docs_source`       | User documentation source directory or file                   | `"docs/user"`               | Any directory or file path    |
| `user_docs_dest`         | Destination path for user documentation in central repository | `"guides/{project_name}"`   | Any directory path            |
| `generate_pdf`           | Whether to generate PDF versions of documentation             | `false`                     | `true`, `false`               |
| `docs_consistency_check` | Whether to check documentation for consistency                | `true`                      | `true`, `false`               |
| `docs_generate_commands` | Commands to run for documentation generation                  | `"npm run docs:generate     |                               | echo 'No docs:generate script found'"` | Any valid command |

## Provided Jobs

| Job                      | Description                                   | Conditions                                                 |
| ------------------------ | --------------------------------------------- | ---------------------------------------------------------- |
| `generate_docs`          | Generates project documentation               | Always runs when documentation files change                |
| `publish_docs`           | Publishes documentation to central repository | Runs on main branch when documentation files change        |
| `generate_api_docs`      | Generates API documentation                   | Runs when `enable_api_docs` is true and API docs change    |
| `generate_user_docs`     | Generates user documentation                  | Runs when `enable_user_docs` is true and user docs change  |
| `check_docs_consistency` | Checks documentation consistency              | Runs when `docs_consistency_check` is true and docs change |

## Reusable Job Templates

The component provides these reusable job templates that can be extended in your pipeline:

| Template    | Description                          |
| ----------- | ------------------------------------ |
| `.docs_job` | Base template for documentation jobs |

## Documentation Structure

The component expects documentation to be structured as follows:

```
docs/              # Main documentation directory
├── api/           # API documentation
├── user/          # User documentation
├── technical/     # Technical documentation
└── README.md      # Main documentation entry point
```

## Usage Example

```yaml
include:
  - project: 'blueflyio/bfcicomponents'
    file: '/components/documentation.yml'
    ref: 'main'

variables:
  # Configure documentation component
  DOCS_SOURCE_DIR: 'docs'
  DOCS_DEST_DIR: 'projects/myproject'
  DOCS_REPO: 'blueflyio/bfdocs'
  ENABLE_API_DOCS: 'true'
  API_DOCS_SOURCE: 'docs/api'
  API_DOCS_DEST: 'api/myproject'
```

## Advanced Configuration

### Custom Documentation Generation

To use a custom documentation generation process:

```yaml
variables:
  DOCS_GENERATE_COMMANDS: 'mkdocs build && cp -r site/* public/docs/'
```

### PDF Documentation

To enable PDF generation for user documentation:

```yaml
variables:
  ENABLE_USER_DOCS: 'true'
  GENERATE_PDF: 'true'
```

Note: This requires a `docs:pdf` script in your package.json.

### Custom Publication Job

You can create a custom publication job that extends from the template:

```yaml
custom_docs_publisher:
  extends: .docs_job
  script:
    - echo "Publishing to custom location..."
    - ./scripts/publish-custom-docs.sh
  rules:
    - if: $CI_COMMIT_TAG
```

## Troubleshooting

### Documentation Not Published

If documentation isn't being published:

1. Check that you have the correct permissions for the central documentation repository
2. Verify that your documentation source directory exists and contains files
3. Ensure that the destination directory is configured correctly

### API Documentation Issues

If API documentation isn't generating correctly:

1. Verify that you have a proper OpenAPI or Swagger JSON file
2. Ensure that the API documentation source path is correct
3. Check that the `docs:api` script is defined in package.json if using a custom script
