---
id: components-bfcicomponents-nodejs-test
---
# Node.js Test Component

The `nodejs-test` component provides standardized test jobs for Node.js projects in GitLab CI/CD pipelines.

## Purpose

This component:

- Provides configurable test jobs for various testing types (unit, integration, e2e)
- Handles code linting
- Generates and validates test coverage reports
- Supports multiple testing frameworks
- Works seamlessly with the `nodejs-base` component

## Inputs

| Input                   | Description                                                     | Default  | Options                                     |
| ----------------------- | --------------------------------------------------------------- | -------- | ------------------------------------------- |
| `stage`                 | Stage for test jobs                                             | `"test"` | Any valid stage name                        |
| `test_framework`        | Test framework to use                                           | `"jest"` | `"jest"`, `"mocha"`, `"vitest"`, `"custom"` |
| `custom_test_command`   | Custom test command to run if test_framework is set to 'custom' | `""`     | Any valid command                           |
| `run_unit_tests`        | Whether to run unit tests                                       | `true`   | `true`, `false`                             |
| `run_integration_tests` | Whether to run integration tests                                | `false`  | `true`, `false`                             |
| `run_e2e_tests`         | Whether to run end-to-end tests                                 | `false`  | `true`, `false`                             |
| `run_lint`              | Whether to run linting                                          | `true`   | `true`, `false`                             |
| `coverage`              | Whether to generate coverage reports                            | `true`   | `true`, `false`                             |
| `coverage_threshold`    | Minimum coverage percentage to pass                             | `"70"`   | Any valid percentage                        |
| `fail_on_coverage`      | Whether to fail the job when coverage is below threshold        | `false`  | `true`, `false`                             |

## Provided Jobs

| Job                | Description            | Conditions                                |
| ------------------ | ---------------------- | ----------------------------------------- |
| `test:unit`        | Runs unit tests        | Runs when `run_unit_tests` is true        |
| `test:integration` | Runs integration tests | Runs when `run_integration_tests` is true |
| `test:e2e`         | Runs end-to-end tests  | Runs when `run_e2e_tests` is true         |
| `lint`             | Runs linting           | Runs when `run_lint` is true              |

## Reusable Job Templates

The component provides these reusable job templates that can be extended in your pipeline:

| Template    | Description               |
| ----------- | ------------------------- |
| `.test_job` | Template for test jobs    |
| `.lint_job` | Template for linting jobs |

## Coverage Reports

When coverage is enabled, the component:

- Generates coverage reports in the `coverage/` directory
- Adds coverage reports to job artifacts
- Configures GitLab coverage visualization
- Optionally enforces a minimum coverage threshold

## Usage Example

```yaml
include:
  - project: 'blueflyio/bfcicomponents'
    file: '/components/nodejs-base.yml'
    ref: 'main'
  - project: 'blueflyio/bfcicomponents'
    file: '/components/nodejs-test.yml'
    ref: 'main'

variables:
  # Configure test component
  TEST_FRAMEWORK: 'jest'
  RUN_INTEGRATION_TESTS: 'true'
  COVERAGE_THRESHOLD: '80'
  FAIL_ON_COVERAGE: 'true'
```

## Advanced Configuration

### Custom Test Framework

To use a custom test command:

```yaml
variables:
  TEST_FRAMEWORK: 'custom'
  CUSTOM_TEST_COMMAND: 'nyc mocha --recursive --timeout 10000'
```

### Different Test Types

To run different types of tests:

```yaml
variables:
  RUN_UNIT_TESTS: 'true'
  RUN_INTEGRATION_TESTS: 'true'
  RUN_E2E_TESTS: 'true'
```

### Custom Test Jobs

You can create custom test jobs that extend from the templates:

```yaml
browser_tests:
  extends: .test_job
  stage: browser_test
  image: selenium/standalone-chrome:latest
  script:
    - echo "Running browser tests..."
    - npm run test:browser
  rules:
    - if: $CI_COMMIT_BRANCH == "main"
```

## Troubleshooting

### Test Failures

If tests are failing:

1. Run the failing tests locally to debug the issue
2. Check that test dependencies are properly installed
3. Verify that the correct test framework is configured

### Coverage Issues

If coverage reporting isn't working:

1. Ensure your test command supports coverage reporting
2. Verify that coverage reports are generated in the correct format
3. Check that the coverage threshold isn't set too high

### Integration Tests Not Running

If integration tests aren't running:

1. Verify that you have a `test:integration` script in your package.json
2. Ensure all necessary environment variables are set
3. Check that external dependencies are available in the test environment
