---
id: components-bfrfp-INSTALLATION
---
# BFRFP Installation Guide

This guide provides comprehensive instructions for installing and configuring the LLM Platform RFP Response Platform (BFRFP), with special attention to the multi-model AI integration system.

## Table of Contents

- [Prerequisites](#prerequisites)
- [Installation Options](#installation-options)
  - [Standard Installation](#standard-installation)
  - [Global Installation](#global-installation)
  - [Docker Installation](#docker-installation)
  - [AI-Only Installation](#ai-only-installation)
- [Configuration](#configuration)
  - [Environment Variables](#environment-variables)
  - [AI Model Configuration](#ai-model-configuration)
- [Running the System](#running-the-system)
- [Verification](#verification)
- [Troubleshooting](#troubleshooting)
- [Updating](#updating)
- [Uninstalling](#uninstalling)
- [Advanced Topics](#advanced-topics)

## Prerequisites

Before you install BFRFP, make sure your system meets the following requirements:

- **Node.js**: v20.0.0 or higher
- **npm**: v9.0.0 or higher
- **Disk Space**: At least 500MB available
- **Memory**: Minimum 1GB RAM, recommended 2GB+
- **Operating System**: Linux, macOS, or Windows with WSL

To check your Node.js and npm versions:

```bash
node --version
npm --version
```

## Installation Options

BFRFP can be installed in several ways depending on your needs.

### Standard Installation

The standard installation sets up BFRFP locally in the current directory:

```bash
# Clone the repository (if not already done)
git clone https://gitlab.com/bluefly/BFRFP.git
cd BFRFP

# Run the installation script
./install.sh

# Configure your environment
cp .env.example .env
nano .env  # Edit with your settings
```

### Global Installation

For command-line access from any directory:

```bash
./install.sh --global
```

This installs BFRFP globally and adds shell completions, allowing you to run `bfrfp` commands from anywhere.

### Docker Installation

For containerized deployment:

```bash
./install.sh --docker

# Start the Docker container
docker-compose up -d
```

### AI-Only Installation

If you're only interested in the multi-model AI integration:

```bash
./install.sh --ai-only

# or to set up AI components specifically
node scripts/setup-ai-models.js
```

## Configuration

### Environment Variables

BFRFP uses environment variables for configuration. Copy the example file and edit as needed:

```bash
cp .env.example .env
nano .env
```

Key environment variables:

| Variable                | Description                                | Default                   | Required          |
| ----------------------- | ------------------------------------------ | ------------------------- | ----------------- |
| `PORT`                  | HTTP server port                           | `3000`                    | No                |
| `NODE_ENV`              | Environment mode (development, production) | `development`             | No                |
| `CLAUDE_API_KEY`        | Anthropic Claude API key                   | None                      | For Claude AI     |
| `CLAUDE_MODEL`          | Default Claude model to use                | `claude-3-haiku-20240307` | No                |
| `HUGGINGFACE_API_TOKEN` | Hugging Face API token                     | None                      | For HF models     |
| `LOCAL_MODEL_URL`       | URL for local AI model server              | None                      | For local models  |
| `REMOTE_MODEL_URL`      | URL for remote AI model                    | None                      | For remote models |
| `CLAUDE_MOCK_MODE`      | Use mock responses for Claude              | `false`                   | No                |
| `FREE_MODEL_MOCK_MODE`  | Use mock responses for free models         | `false`                   | No                |
| `MAX_DAILY_COST`        | Maximum daily cost in USD                  | `10.0`                    | No                |
| `MAX_DAILY_TOKENS`      | Maximum daily token usage                  | `1000000`                 | No                |

### AI Model Configuration

For optimal performance of the multi-model AI system:

1. **Claude Integration**:

   - Sign up at Anthropic to get an API key
   - Add to your .env: `CLAUDE_API_KEY=your-key-here`
   - Select model: `CLAUDE_MODEL=claude-3-haiku-20240307`

2. **Free Models**:

   - Hugging Face: `HUGGINGFACE_API_TOKEN=your-token-here`
   - Local model: `LOCAL_MODEL_URL=http://localhost:8000/api/generate`
   - Remote model: `REMOTE_MODEL_URL=https://example.com/api/v1/generate`

3. **Mock Mode** (for development without API keys):
   ```
   CLAUDE_MOCK_MODE=true
   FREE_MODEL_MOCK_MODE=true
   ```

## Running the System

### Start the Server

```bash
# Standard installation
npm run server

# Global installation
bfrfp server

# Docker installation
docker-compose up -d
```

### Server Options

```bash
# Development mode with auto-reload
npm run server:dev

# Specify port
npm run server -- --port=8080

# Global installation with port
bfrfp server --port=8080
```

## Verification

After installation, verify the system is working correctly:

```bash
# Run the troubleshooting script
node scripts/troubleshoot.js

# Test the server by making a simple request
curl http://localhost:3000/api/ai/completion \
  -H "Content-Type: application/json" \
  -d '{"messages":[{"role":"user","content":"Hello, what is BFRFP?"}]}'

# Check token usage
curl http://localhost:3000/api/ai/usage
```

## Troubleshooting

If you encounter issues during installation or operation:

```bash
# Run the diagnostic tool
node scripts/troubleshoot.js

# Try to fix issues automatically
node scripts/troubleshoot.js --fix

# AI-specific diagnostics
node scripts/troubleshoot.js --ai-only

# Check server status
node scripts/troubleshoot.js --port=3000
```

### Common Issues and Solutions

#### API Keys Not Working

- Verify the key format is correct
- Check for whitespace or newlines in the environment variables
- For Claude: Ensure you have an active subscription

#### Server Won't Start

- Check if something else is using the same port
- Verify node_modules exists (`npm install`)
- Ensure required directories are writable (`./logs`, `./data/usage`)

#### Out of Memory Errors

- Increase the Node.js memory limit: `NODE_OPTIONS="--max-old-space-size=4096" npm run server`
- For Docker, adjust memory limits in docker-compose.yml

## Updating

To update BFRFP to the latest version:

```bash
# Pull latest code
git pull

# Reinstall dependencies
npm install

# Run the installation script again
./install.sh
```

## Uninstalling

To remove BFRFP from your system:

```bash
# Local installation (preserves data)
./scripts/uninstall.sh

# Complete removal (including data)
./scripts/uninstall.sh --complete

# Remove global installation
./scripts/uninstall.sh --global
```

## Advanced Topics

### Shell Integration

The installation script configures shell integration for Bash, Zsh, and Fish shells including command completion. To manually set up:

```bash
# Add to .bashrc or .zshrc
source /path/to/BFRFP/completions/bfrfp.sh

# For fish shell
cp /path/to/BFRFP/completions/bfrfp.fish ~/.config/fish/completions/
```

### Docker Customization

Edit `docker-compose.yml` to customize the Docker setup:

```yaml
version: '3.8'

services:
  bfrfp:
    build:
      context: .
      dockerfile: Dockerfile
    ports:
      - '8000:3000' # Map to different port
    volumes:
      - ./data:/app/data
      - ./logs:/app/logs
      - ./output:/app/output
    environment:
      - NODE_ENV=production
      - CLAUDE_API_KEY=your-key-here
    restart: unless-stopped
    # Add custom resource limits
    deploy:
      resources:
        limits:
          memory: 2G
        reservations:
          memory: 1G
```

### Load Balancing

For high-availability deployments:

```yaml
# In docker-compose.yml
services:
  bfrfp:
    # ... other config
    deploy:
      replicas: 3
      update_config:
        parallelism: 1
        delay: 10s
      restart_policy:
        condition: on-failure
    networks:
      - bfrfp-network

  nginx:
    image: nginx:latest
    volumes:
      - ./nginx.conf:/etc/nginx/conf.d/default.conf
    ports:
      - '80:80'
    depends_on:
      - bfrfp
    networks:
      - bfrfp-network

networks:
  bfrfp-network:
    driver: bridge
```

### Custom AI Model Integration

To integrate additional AI models:

1. Create a service implementation in `src/services/`
2. Update the model dispatcher configuration
3. Add environment variables for the new model
4. Restart the server to apply changes

See `AI_README.md` for detailed documentation on the multi-model AI architecture.

---

For more information, refer to the [API Reference](./docs/api/api-reference) and [AI Documentation](./AI_README).

If you encounter any issues not covered in this guide, please open an issue in the GitLab repository or contact the LLM Platform support team.
