---
id: components-bfrfp-api-api-reference
---
# BFRFP API Reference

## Introduction

The BFRFP API provides programmatic access to RFP analysis and response generation capabilities. This reference guide details the available endpoints, request formats, and response structures.

## Base URL

```
http://localhost:8000/api
```

For production environments, use:

```
https://api.bluefly.io/rfp
```

## Authentication

All API requests require authentication using an API key.

Include the API key in the request header:

```
Authorization: Bearer YOUR_API_KEY
```

To obtain an API key, contact your LLM Platform administrator or generate one through the BFRFP web interface.

## API Versioning

The current API version is `v1`. Include the version in the URL path:

```
/api/v1/analyze
```

## Rate Limiting

Default rate limits:

- 100 requests per minute
- 5,000 requests per day

Rate limit headers are included in all responses:

- `X-RateLimit-Limit`: Maximum requests allowed in the current period
- `X-RateLimit-Remaining`: Remaining requests in the current period
- `X-RateLimit-Reset`: Seconds until the rate limit resets

## Endpoints

### Document Analysis

#### Analyze Document

```
POST /api/v1/analyze
```

Analyze an RFP document and extract structured information.

**Request Body**

| Parameter         | Type   | Description                                                             |
| ----------------- | ------ | ----------------------------------------------------------------------- |
| `documentUrl`     | string | URL to the document (optional if `documentContent` is provided)         |
| `documentContent` | string | Base64-encoded document content (optional if `documentUrl` is provided) |
| `documentType`    | string | Document type (`pdf`, `txt`, `docx`)                                    |
| `options`         | object | Analysis options (see below)                                            |

**Analysis Options**

| Option                | Type    | Description                     | Default |
| --------------------- | ------- | ------------------------------- | ------- |
| `extractRequirements` | boolean | Extract individual requirements | `true`  |
| `extractMetadata`     | boolean | Extract document metadata       | `true`  |
| `extractTimeline`     | boolean | Extract timeline information    | `true`  |
| `extractSections`     | boolean | Extract document sections       | `true`  |
| `useAI`               | boolean | Use AI-powered analysis         | `true`  |

**Response**

```json
{
  "status": "success",
  "data": {
    "documentId": "doc_12345",
    "title": "Request for Proposal: Digital Transformation Services",
    "issuer": "Sample Government Agency",
    "dueDate": "2025-05-15",
    "sections": [
      {
        "title": "Introduction",
        "summary": "Overview of the agency's digital transformation initiative",
        "content": "Full section content..."
      }
    ],
    "requirements": [
      {
        "id": "req_1",
        "text": "Contractor shall provide website redesign services",
        "type": "technical",
        "priority": "high"
      }
    ],
    "qualifications": [
      "Minimum 5 years experience with government projects",
      "Previous successful CMS implementations"
    ],
    "evaluationCriteria": {
      "technical": 40,
      "experience": 30,
      "cost": 20,
      "timeline": 10
    },
    "timeline": {
      "rfpRelease": "2025-04-01",
      "questionsDeadline": "2025-04-15",
      "proposalsDue": "2025-05-15",
      "award": "2025-06-15",
      "projectStart": "2025-07-01"
    },
    "contact": {
      "name": "Jane Smith",
      "email": "jsmith@agency.gov",
      "phone": "555-123-4567"
    },
    "budgetInformation": "Project budget not to exceed $500,000",
    "metadata": {
      "pageCount": 45,
      "wordCount": 12500,
      "analysisDate": "2025-04-02T14:30:00Z"
    }
  }
}
```

**Example Request**

```bash
curl -X POST https://api.bluefly.io/rfp/v1/analyze \
  -H "Authorization: Bearer YOUR_API_KEY" \
  -H "Content-Type: application/json" \
  -d '{
    "documentUrl": "https://example.com/rfp.pdf",
    "documentType": "pdf",
    "options": {
      "extractRequirements": true,
      "extractTimeline": true
    }
  }'
```

#### Get Document Analysis

```
GET /api/v1/analysis/:documentId
```

Retrieve the analysis results for a previously analyzed document.

**Path Parameters**

| Parameter    | Description                 |
| ------------ | --------------------------- |
| `documentId` | ID of the analyzed document |

**Response**

Same as the response from the `POST /api/v1/analyze` endpoint.

**Example Request**

```bash
curl -X GET https://api.bluefly.io/rfp/v1/analysis/doc_12345 \
  -H "Authorization: Bearer YOUR_API_KEY"
```

### Response Generation

#### Generate Response

```
POST /api/v1/generate
```

Generate a response to an RFP based on analysis results.

**Request Body**

| Parameter    | Type   | Description                                       |
| ------------ | ------ | ------------------------------------------------- |
| `documentId` | string | ID of the analyzed document                       |
| `template`   | string | Template name to use (optional)                   |
| `sections`   | array  | Sections to include (optional)                    |
| `format`     | string | Output format (`markdown`, `pdf`, `docx`, `html`) |
| `options`    | object | Generation options (see below)                    |

**Generation Options**

| Option               | Type    | Description                                      | Default            |
| -------------------- | ------- | ------------------------------------------------ | ------------------ |
| `includeGraphics`    | boolean | Include graphics in the response                 | `true`             |
| `includeCaseStudies` | boolean | Include case studies                             | `true`             |
| `includePricing`     | boolean | Include pricing information                      | `true`             |
| `customizationLevel` | string  | Level of customization (`low`, `medium`, `high`) | `medium`           |
| `targetWordCount`    | number  | Target word count for the response               | Varies by template |

**Response**

```json
{
  "status": "success",
  "data": {
    "responseId": "resp_12345",
    "documentId": "doc_12345",
    "format": "markdown",
    "sections": ["executive-summary", "technical-approach", "past-performance"],
    "wordCount": 5280,
    "downloadUrl": "https://api.bluefly.io/rfp/v1/responses/resp_12345/download",
    "generatedAt": "2025-04-02T15:45:00Z"
  }
}
```

**Example Request**

```bash
curl -X POST https://api.bluefly.io/rfp/v1/generate \
  -H "Authorization: Bearer YOUR_API_KEY" \
  -H "Content-Type: application/json" \
  -d '{
    "documentId": "doc_12345",
    "template": "full-response",
    "format": "markdown",
    "options": {
      "includeCaseStudies": true,
      "customizationLevel": "high"
    }
  }'
```

#### Get Response

```
GET /api/v1/responses/:responseId
```

Retrieve information about a generated response.

**Path Parameters**

| Parameter    | Description                  |
| ------------ | ---------------------------- |
| `responseId` | ID of the generated response |

**Response**

Same as the response from the `POST /api/v1/generate` endpoint.

**Example Request**

```bash
curl -X GET https://api.bluefly.io/rfp/v1/responses/resp_12345 \
  -H "Authorization: Bearer YOUR_API_KEY"
```

#### Download Response

```
GET /api/v1/responses/:responseId/download
```

Download a generated response in the specified format.

**Path Parameters**

| Parameter    | Description                  |
| ------------ | ---------------------------- |
| `responseId` | ID of the generated response |

**Query Parameters**

| Parameter | Description                                       | Default                       |
| --------- | ------------------------------------------------- | ----------------------------- |
| `format`  | Output format (`markdown`, `pdf`, `docx`, `html`) | Format used during generation |

**Response**

The response content in the requested format with appropriate Content-Type header.

**Example Request**

```bash
curl -X GET https://api.bluefly.io/rfp/v1/responses/resp_12345/download?format=pdf \
  -H "Authorization: Bearer YOUR_API_KEY" \
  -o response.pdf
```

### Templates

#### List Templates

```
GET /api/v1/templates
```

List available response templates.

**Response**

```json
{
  "status": "success",
  "data": {
    "templates": [
      {
        "id": "full-response",
        "name": "Full Response",
        "description": "Complete RFP response template",
        "sections": [
          "executive-summary",
          "technical-approach",
          "past-performance",
          "team-qualifications",
          "pricing"
        ]
      },
      {
        "id": "technical-approach",
        "name": "Technical Approach",
        "description": "Technical approach section template",
        "sections": [
          "problem-understanding",
          "proposed-solution",
          "implementation-plan",
          "timeline"
        ]
      }
    ]
  }
}
```

**Example Request**

```bash
curl -X GET https://api.bluefly.io/rfp/v1/templates \
  -H "Authorization: Bearer YOUR_API_KEY"
```

#### Get Template

```
GET /api/v1/templates/:templateId
```

Get details about a specific template.

**Path Parameters**

| Parameter    | Description        |
| ------------ | ------------------ |
| `templateId` | ID of the template |

**Response**

```json
{
  "status": "success",
  "data": {
    "id": "technical-approach",
    "name": "Technical Approach",
    "description": "Technical approach section template",
    "sections": ["problem-understanding", "proposed-solution", "implementation-plan", "timeline"],
    "content": "# Technical Approach\n\n## Understanding of the Problem\n{problem_understanding}\n\n## Proposed Solution\n{proposed_solution}\n\n## Implementation Plan\n{implementation_plan}\n\n## Timeline\n{timeline}\n"
  }
}
```

**Example Request**

```bash
curl -X GET https://api.bluefly.io/rfp/v1/templates/technical-approach \
  -H "Authorization: Bearer YOUR_API_KEY"
```

### Reference Data

#### List Case Studies

```
GET /api/v1/reference/case-studies
```

List available case studies for response generation.

**Query Parameters**

| Parameter  | Description                  | Default |
| ---------- | ---------------------------- | ------- |
| `industry` | Filter by industry           | None    |
| `service`  | Filter by service type       | None    |
| `limit`    | Maximum number of results    | `20`    |
| `offset`   | Result offset for pagination | `0`     |

**Response**

```json
{
  "status": "success",
  "data": {
    "caseStudies": [
      {
        "id": "cs_gov_portal",
        "title": "State Government Portal Modernization",
        "industry": "government",
        "services": ["web-development", "content-migration"],
        "summary": "Modernized legacy state portal to improve citizen access",
        "results": ["40% increase in user engagement", "65% faster content updates"],
        "testimonial": "LLM Platform's team delivered exceptional results ahead of schedule."
      }
    ],
    "pagination": {
      "total": 24,
      "limit": 20,
      "offset": 0
    }
  }
}
```

**Example Request**

```bash
curl -X GET "https://api.bluefly.io/rfp/v1/reference/case-studies?industry=government&limit=10" \
  -H "Authorization: Bearer YOUR_API_KEY"
```

#### List Team Members

```
GET /api/v1/reference/team
```

List team members available for response generation.

**Query Parameters**

| Parameter | Description                  | Default |
| --------- | ---------------------------- | ------- |
| `role`    | Filter by role               | None    |
| `skill`   | Filter by skill              | None    |
| `limit`   | Maximum number of results    | `20`    |
| `offset`  | Result offset for pagination | `0`     |

**Response**

```json
{
  "status": "success",
  "data": {
    "team": [
      {
        "id": "tm_123",
        "name": "Jane Smith",
        "title": "Senior Project Manager",
        "role": "project-manager",
        "skills": [
          { "name": "agile", "level": 5 },
          { "name": "government-projects", "level": 4 }
        ],
        "certifications": ["PMP", "Agile Certified Practitioner"],
        "experience": "12+ years in digital transformation projects"
      }
    ],
    "pagination": {
      "total": 35,
      "limit": 20,
      "offset": 0
    }
  }
}
```

**Example Request**

```bash
curl -X GET "https://api.bluefly.io/rfp/v1/reference/team?role=project-manager&limit=10" \
  -H "Authorization: Bearer YOUR_API_KEY"
```

## Error Handling

All errors follow a standardized format:

```json
{
  "status": "error",
  "error": {
    "code": "invalid_document",
    "message": "The document could not be processed",
    "details": "PDF file is password protected"
  }
}
```

### Common Error Codes

| Code               | Description                     |
| ------------------ | ------------------------------- |
| `unauthorized`     | Missing or invalid API key      |
| `invalid_request`  | Invalid request parameters      |
| `invalid_document` | Document could not be processed |
| `not_found`        | Requested resource not found    |
| `rate_limited`     | Rate limit exceeded             |
| `server_error`     | Internal server error           |

### HTTP Status Codes

| Code  | Description           |
| ----- | --------------------- |
| `200` | Success               |
| `400` | Bad Request           |
| `401` | Unauthorized          |
| `403` | Forbidden             |
| `404` | Not Found             |
| `429` | Too Many Requests     |
| `500` | Internal Server Error |

## Webhook Notifications

BFRFP can send webhook notifications for asynchronous operations.

Configure webhooks in the BFRFP web interface or via the API:

```
POST /api/v1/webhooks
```

**Request Body**

```json
{
  "url": "https://your-server.com/webhook",
  "events": ["analysis.completed", "response.generated"],
  "secret": "your_webhook_secret"
}
```

**Webhook Payload**

```json
{
  "event": "analysis.completed",
  "timestamp": "2025-04-02T14:35:00Z",
  "data": {
    "documentId": "doc_12345",
    "status": "completed",
    "resultUrl": "https://api.bluefly.io/rfp/v1/analysis/doc_12345"
  }
}
```

## SDKs and Client Libraries

Official client libraries:

- **JavaScript/TypeScript**: [bfrfp-js](https://github.com/blueflyio/bfrfp-js)
- **Python**: [bfrfp-python](https://github.com/blueflyio/bfrfp-python)
- **Java**: [bfrfp-java](https://github.com/blueflyio/bfrfp-java)

### JavaScript Example

```javascript
import { BFRFPClient } from 'bfrfp-js';

const client = new BFRFPClient({
  apiKey: 'YOUR_API_KEY',
  baseUrl: 'https://api.bluefly.io/rfp',
});

// Analyze a document
const analysis = await client.analyze({
  documentUrl: 'https://example.com/rfp.pdf',
});

// Generate a response
const response = await client.generate({
  documentId: analysis.documentId,
  template: 'full-response',
  format: 'markdown',
});

// Download the response
const responseContent = await client.downloadResponse({
  responseId: response.responseId,
  format: 'pdf',
});
```

## Best Practices

1. **Efficient API Usage**

   - Cache responses when appropriate
   - Use webhooks for long-running operations
   - Implement retries with exponential backoff

2. **Security**

   - Keep API keys secure
   - Use HTTPS for all requests
   - Validate webhook signatures

3. **Performance**
   - Optimize document size before uploading
   - Use batch operations where available
   - Implement pagination for large result sets

## Support and Feedback

For API support and feedback:

- Email: api-support@bluefly.io
- API Status: [status.bluefly.io](https://status.bluefly.io)
- Developer Forum: [developers.bluefly.io](https://developers.bluefly.io)

## Changelog

### v1.0.0 (2025-04-01)

- Initial API release

### v1.1.0 (Coming Soon)

- Batch processing capabilities
- Enhanced template customization
- Improved document analysis`v
