/**
 * Integration tests for the full RFP workflow
 */
import fs from 'fs';
import path from 'path';
import { init } from '../../src/index.js';
import { analyzeDocument } from '../../src/parser.js';
import { generateResponse } from '../../src/generator.js';

describe('RFP Workflow Integration', () => {
  const testOutputDir = path.join(process.cwd(), 'test-output');
  const testOutputFile = path.join(testOutputDir, 'test-response.md');

  beforeAll(async () => {
    // Create test output directory if it doesn't exist
    if (!fs.existsSync(testOutputDir)) {
      fs.mkdirSync(testOutputDir, { recursive: true });
    }
  });

  afterAll(() => {
    // Clean up test files
    if (fs.existsSync(testOutputFile)) {
      fs.unlinkSync(testOutputFile);
    }
  });

  test('should analyze document and generate response', async () => {
    // Skip this test if running in CI environment
    if (process.env.CI) {
      console.log('Skipping integration test in CI environment');
      return;
    }

    // Create a mock sample RFP file if it doesn't exist
    const testFilePath = global.testSetup.testFilePath;

    if (!fs.existsSync(testFilePath)) {
      const fixtureDir = path.dirname(testFilePath);
      if (!fs.existsSync(fixtureDir)) {
        fs.mkdirSync(fixtureDir, { recursive: true });
      }

      const sampleRfp = `REQUEST FOR PROPOSAL
Digital Transformation Services

SECTION 1: INTRODUCTION
This is a sample RFP document for testing purposes.

SECTION 2: REQUIREMENTS
- Requirement 1
- Requirement 2
- Requirement 3

SECTION 3: SUBMISSION
Please submit your proposal by 2025-12-31.`;

      fs.writeFileSync(testFilePath, sampleRfp);
      console.log(`Created sample RFP file at: ${testFilePath}`);
    }

    // Analyze sample RFP
    const analysisResult = await analyzeDocument(testFilePath);

    // Verify analysis result
    expect(analysisResult).toBeDefined();
    expect(analysisResult.title).toBeDefined();
    expect(analysisResult.sections).toBeInstanceOf(Array);

    // Generate response
    const generationResult = await generateResponse(testOutputFile, {
      rfpTitle: analysisResult.title,
    });

    // Verify generation result
    expect(generationResult).toBeDefined();
    expect(generationResult.path).toBe(testOutputFile);
    expect(generationResult.sections).toBeInstanceOf(Array);
    expect(generationResult.wordCount).toBeGreaterThan(0);

    // Verify output file exists
    expect(fs.existsSync(testOutputFile)).toBe(true);
  }, 60000); // Longer timeout for this test
});
