---
id: components-bfrfp-technical-CI-TROUBLESHOOTING
---
# CI Troubleshooting Guide

This guide helps resolve common issues with the GitLab CI pipeline for BFRFP.

## Quick Start

If you're experiencing CI failures, run the following commands to diagnose and fix the issues:

```bash
# Test CI locally to see all failing jobs
./scripts/test-ci-locally.sh

# Automatically fix common issues
node scripts/fix-ci-issues.js

# Run CI tests again to verify fixes
./scripts/test-ci-locally.sh
```

## Common Issues

### ESLint Errors

ESLint errors typically occur due to code style violations:

```
error  'someVar' is assigned a value but never used  no-unused-vars
```

#### Solution

Run ESLint locally to identify and fix issues:

```bash
npm run lint
```

Or to automatically fix issues:

```bash
npm run lint -- --fix
```

### TypeScript Errors

TypeScript errors occur when types don't match or imports are incorrect:

```
TS2307: Cannot find module './some-module' or its corresponding type declarations.
```

#### Solution

Verify your imports and run TypeScript check locally:

```bash
npm run typecheck
```

Common TypeScript issues in BFRFP:

1. **ES Module Syntax**: BFRFP uses ES modules, so make sure your imports use the `.js` extension:

   ```typescript
   // Correct:
   import { something } from './module.js';

   // Incorrect:
   import { something } from './module';
   ```

2. **Type Definitions**: If you're using external libraries, ensure their types are properly defined or installed.

3. **CommonJS vs ES Modules**: Make sure your `package.json` has `"type": "module"` and `tsconfig.json` has `"module": "esnext"`.

### Build Failures

Build failures often occur due to TypeScript errors or module format issues.

#### Solution

Check your tsconfig.json configuration and ensure it's set up for ES modules:

```json
{
  "compilerOptions": {
    "module": "esnext",
    "moduleResolution": "node",
    "esModuleInterop": true
  }
}
```

Also make sure your `package.json` has the correct build script:

```json
"scripts": {
  "build": "tsc"
}
```

### Test Failures

Test failures indicate that some tests are not passing.

#### Solution

Run tests locally to identify the failing tests:

```bash
npm test
```

Common test issues in BFRFP:

1. **Environment Variables**: Some tests may require environment variables to be set.

   ```bash
   # Create a .env.test file and then run tests
   cp .env.example .env.test
   npm test
   ```

2. **Jest Configuration**: Make sure your jest.config.js is properly configured for ES modules:
   ```javascript
   export default {
     transform: {},
     extensionsToTreatAsEsm: ['.ts'],
     moduleNameMapper: {
       '^(\\.{1,2}/.*)\\.js$': '$1',
     },
     testEnvironment: 'node',
   };
   ```

### Documentation Issues

Documentation jobs might fail if required documentation files are missing.

#### Solution

Ensure the documentation structure is correct:

- docs/index.md
- docs/api/API.md
- Other required documentation files

You can use our automated tool to create the basic documentation structure:

```bash
node scripts/fix-ci-issues.js
```

## GitLab CI Configuration

### Understanding the Pipeline

Our CI pipeline is defined in `.gitlab-ci.yml` and uses shared configurations from the BFCommon repository:

```yaml
include:
  # Include base Node.js configuration
  - project: 'bluefly/bfcommon'
    file: '/ci/node-base.yml'
    ref: main

  # Other included configurations...
```

This means the pipeline includes jobs from the BFCommon repository that may not be directly visible in our repository.

### Pipeline Stages

The pipeline consists of several stages:

1. **Build**: Compiles TypeScript code and builds the project
2. **Test**: Runs linting, type checking, and unit tests
3. **Integration**: Runs integration tests
4. **Docs**: Builds and validates documentation
5. **Deploy**: Deploys to staging/production environments

### Variables

CI variables are defined in the `.gitlab-ci.yml` file:

```yaml
variables:
  # Project-specific variables
  PROJ_NAME: 'bfrfp'
  NODE_VERSION: '20'
  # ...
```

You can modify these variables to change how the CI pipeline behaves.

## Advanced Troubleshooting

### Running CI Locally with GitLab Runner

If you have GitLab Runner installed, you can run the CI pipeline locally:

```bash
# Install GitLab Runner if needed
# brew install gitlab-runner

# Run a specific job
gitlab-runner exec docker build

# Run with specific variables
gitlab-runner exec docker build --env NODE_VERSION=20
```

### Debugging CI Failures

To add more verbose output to CI:

1. **Environment Debugging**: Add debugging to your scripts

   ```javascript
   console.debug('Variable value:', someVar);
   ```

2. **CI Trace Debugging**: Enable CI_DEBUG_TRACE in GitLab

   ```yaml
   variables:
     CI_DEBUG_TRACE: 'true'
   ```

3. **Script Trace**: Add `set -x` to shell scripts to trace execution
   ```bash
   #!/bin/bash
   set -x
   # Rest of script
   ```

### Cache Issues

If you're experiencing caching issues:

```bash
# Clear GitLab CI cache for this project
# (Requires GitLab CLI tool)
glab ci-job run clean_cache

# In CI config, disable cache for debugging
cache:
  enabled: false
```

## Contributing to CI

If you make changes to the CI configuration:

1. Test locally using `./scripts/test-ci-locally.sh`
2. Document your changes in this guide
3. Update the CI configuration in `.gitlab-ci.yml`

## Further Help

If you're still experiencing issues:

- Contact the DevOps team
- Check the GitLab documentation
- Refer to the BFCommon repository for shared CI configurations
