---
id: components-bfrfp-user-USER_GUIDE
---
# BFRFP User Guide

## Introduction

Welcome to BFRFP (LLM Platform RFP Response Platform), your powerful tool for automating and enhancing the RFP response process. This guide will help you understand how to use BFRFP effectively to analyze RFP documents and generate high-quality responses.

## Getting Started

### Installation

There are several ways to install BFRFP:

#### Node.js Installation

```bash
# Install dependencies
npm install

# Run the development server
npm run dev
```

#### Docker Installation

```bash
# Pull the Docker image
docker pull blueflyio/bfrfp:latest

# Run the container
docker run -p 8000:8000 --env-file .env blueflyio/bfrfp
```

### Configuration

Before using BFRFP, you need to configure it with your environment variables:

1. Create a `.env` file in the root directory
2. Add the following variables:

```env
# Server configuration
PORT=8000
NODE_ENV=development

# API integration
API_URL=http://localhost:3000/api
API_KEY=your-api-key-here

# AI model configuration
MODEL_VERSION=latest
CLAUDE_API_KEY=your-claude-api-key

# Storage paths
TEMPLATES_DIR=./templates
DATA_DIR=./data
OUTPUT_DIR=./output
```

## Document Analysis

BFRFP can analyze various document formats to extract structured information from RFPs.

### Supported Formats

- PDF (`.pdf`)
- Plain Text (`.txt`)
- Word Documents (`.docx`) - Coming soon

### Running Document Analysis

You can analyze an RFP document using the command line:

```bash
npm run analyze -- --file=path/to/rfp.pdf
```

Or use the web interface by:

1. Starting the server: `npm run dev`
2. Opening `http://localhost:8000` in your browser
3. Uploading a document through the interface

### Analysis Output

After analyzing a document, BFRFP will generate a structured JSON output containing:

- Document title and issuer
- Key sections with summaries
- Requirements list with classification
- Timeline information
- Evaluation criteria
- Contact information
- Budget details

### Viewing Analysis Results

Analysis results can be viewed in several ways:

1. **Command Line**: Results are displayed in the console
2. **Web Interface**: Interactive visualization of document structure
3. **JSON Export**: Save results to a JSON file for further processing

## Response Generation

BFRFP can generate tailored responses to RFPs based on templates and AI-powered content.

### Templates

Response templates are stored in the `templates/` directory and include:

- `full-response.md`: Complete RFP response
- `technical-approach.md`: Technical approach section
- `past-performance.md`: Past performance and case studies

You can create custom templates by:

1. Creating a new Markdown file in the `templates/` directory
2. Using placeholder variables like `{section_name}`
3. Adding sections for AI-generated content

### Generating Responses

Generate a response using:

```bash
npm run generate -- --output=path/to/output.md
```

Options include:

- `--section=<section-name>`: Generate a specific section
- `--template=<template-name>`: Use a custom template
- `--format=<format>`: Output format (markdown, pdf, docx, html)
- `--data=<data-path>`: Custom data for response generation

### Response Customization

You can customize responses in several ways:

1. **Data Overrides**: Create JSON files in the `data/` directory
2. **Template Modification**: Edit template files to change structure
3. **AI Guidance**: Provide specific prompts for AI-generated content

## Data Management

BFRFP uses reference data to enhance response generation.

### Data Structure

Data is organized in the `data/` directory:

- `company/`: Company information and capabilities
- `case-studies/`: Past project case studies
- `services/`: Service descriptions and offerings
- `team/`: Team member profiles and skills

### Updating Data

You can update reference data by:

1. Editing JSON files in the respective directories
2. Adding new files for additional information
3. Using the web interface to manage data entries

## Web Interface

BFRFP includes a web interface for easier document management.

### Accessing the Interface

1. Start the server: `npm run dev`
2. Open `http://localhost:8000` in your browser

### Interface Features

- Document upload and analysis
- Interactive document viewer
- Response generation and editing
- Template management
- Data administration

## Integration

BFRFP integrates with other LLM Platform platforms.

### API Integration

The platform exposes RESTful APIs for integration:

```javascript
// Example API request
const response = await fetch('http://localhost:8000/api/analyze', {
  method: 'POST',
  headers: {
    'Content-Type': 'application/json',
    Authorization: 'Bearer YOUR_API_KEY',
  },
  body: JSON.stringify({
    documentUrl: 'https://example.com/rfp.pdf',
  }),
});

const result = await response.json();
```

### CLI Integration

Use llmcli to interact with BFRFP:

```bash
llmcli rfp analyze --file=path/to/rfp.pdf
llmcli rfp generate --output=path/to/output.md
```

## Troubleshooting

### Common Issues

1. **Document Parsing Errors**

   - Ensure the document is not password-protected
   - Check that the file format is supported
   - For large documents, consider splitting into smaller files

2. **AI Generation Issues**

   - Verify your Claude API key is valid
   - Check network connectivity to AI services
   - Ensure prompt length is within limits

3. **Template Errors**
   - Validate template syntax for proper variable formatting
   - Check for missing closing brackets in variable placeholders
   - Ensure template files are in the correct directory

### Getting Help

For additional support:

- Check the [API Reference](../api/api-reference)
- Review the [Template System](../templates/README) documentation
- Open an issue in the GitLab repository
- Contact the LLM Platform support team

## Best Practices

### Effective RFP Analysis

1. **Document Preparation**

   - Ensure documents are text-searchable PDFs
   - Remove any password protection
   - Split very large RFPs into logical sections

2. **Analysis Strategies**
   - Start with a high-level analysis of the entire document
   - Pay special attention to evaluation criteria
   - Note any unique requirements or constraints

### Response Generation

1. **Template Selection**

   - Choose the appropriate template for your RFP type
   - Customize templates for specific industries or clients
   - Keep templates updated with latest capabilities

2. **Content Review**

   - Always review AI-generated content for accuracy
   - Customize technical details to match the specific RFP
   - Ensure past performance examples are relevant to the requirements

3. **Collaboration**
   - Share response drafts with subject matter experts
   - Collect feedback in a structured format
   - Maintain version control for response iterations

## Advanced Usage

### Custom Prompts

You can provide custom prompts for AI-generated content:

```bash
npm run generate -- --output=output.md --prompt="Focus on cloud migration experience"
```

### Response Automation

Set up automated workflows using the API:

```javascript
// Automated RFP processing
async function processRfp(documentUrl) {
  // Analyze document
  const analysis = await analyzeDocument(documentUrl);

  // Generate response
  const response = await generateResponse(analysis);

  // Notify team
  await notifyTeam(response);

  return response;
}
```

### Batch Processing

Process multiple RFPs at once:

```bash
npm run batch -- --dir=path/to/rfps --output=path/to/outputs
```

## Conclusion

BFRFP is a powerful tool that can significantly streamline your RFP response process. By leveraging its document analysis and AI-powered content generation capabilities, you can create high-quality responses in less time, allowing your team to focus on strategy and customization rather than routine content creation.

For additional information, refer to the other documentation sections:

- [API Reference](../api/api-reference)
- [Template System](../templates/README)
- [ML Integration](../ml/ML_INTEGRATION)
- [Technical Documentation](../technical/TESTING)
