---
title: llm-mcp Model Integration
description: Guide for integrating LLM models with the LLM Platform MCP server
id: components-llm-mcp-model-integration
---

# Integrating Models with llm-mcp

This guide provides detailed instructions for registering and deploying LLM models with the LLM Platform Master Control Program (MCP) server.

## Overview

The llm-mcp server provides a centralized system for managing models and services across the LLM Platform platform. The integration tools in the LLM directory allow you to:

- Register models and services with a local llm-mcp server
- Deploy models to remote llm-mcp servers
- Test and validate model registration and deployment

## Prerequisites

- Node.js 20.x or later
- Access to the llm-mcp server (local or remote)
- API access credentials (for remote servers)

## Integration Tools

The LLM directory contains several tools for llm-mcp integration:

### Core Configuration Files

- **mcp-config.json**: Defines models and services for registration
- **.env**: Environment variables for connections and authentication
- **package.json**: NPM package configuration with scripts
- **.gitignore**: Standard Git ignore patterns

### Main Scripts

- **register-with-mcp.js**: Script to register models with local or remote MCP servers
- **deploy-to-server.js**: Script for deploying to remote servers with validation

### Testing Tools

- **jest.config.js**: Jest testing configuration
- **test/register-test.js**: Unit tests for the registration script
- **test/setup.js**: Test environment setup

## Setup Guide

### 1. Clone and Install

```bash
cd ~/Sites
git clone https://gitlab.com/bluefly-llm/LLM.git
cd LLM
npm install
```

### 2. Configure Environment

Edit the `.env` file with your specific configuration:

```
# Local MCP Server
LOCAL_MCP_URL=http://localhost:8081/api/v1

# Remote MCP Server
REMOTE_MCP_URL=https://mcp.example.com/api/v1

# Authentication
MCP_API_KEY=your_api_key_here

# Deployment Environment
DEPLOYMENT_ENV=development
```

### 3. Configure Models

Customize the model and service definitions in `mcp-config.json` to match your LLM models:

```json
{
  "name": "LLM-Models",
  "description": "LLM models and services registry",
  "baseUrl": "http://localhost:8081/api/v1",
  "models": [
    {
      "id": "llm-model-1",
      "name": "LLM Default Model",
      "version": "1.0.0",
      "description": "Default LLM model for text generation",
      "type": "text-generation",
      "status": "active",
      "metadata": {
        "provider": "bluefly",
        "capabilities": ["text-generation", "chat"]
      }
    }
  ],
  "services": [
    {
      "id": "llm-service-1", 
      "name": "llm-text-generation",
      "description": "Text generation service for LLM models",
      "status": "running",
      "metadata": {
        "version": "1.0.0",
        "supported_models": ["llm-model-1"]
      }
    }
  ]
}
```

## Usage Guide

### Local Registration

To register your LLM models and services with a local MCP server:

```bash
cd ~/Sites/LLM
npm run register-local
```

The script will:
1. Connect to your local llm-mcp server
2. Register each model defined in your configuration
3. Register each service defined in your configuration
4. Provide status updates for each registration

### Remote Registration

To register with a remote MCP server:

```bash
npm run register-remote -- --remote-url https://mcp.example.com/api/v1
```

If you've configured `REMOTE_MCP_URL` in your `.env` file, you can simply run:

```bash
npm run register-remote
```

### Deployment to Production

To deploy your models and services to a remote server:

```bash
npm run deploy -- --server-url https://mcp.example.com/api/v1 --env production
```

If you've configured `REMOTE_MCP_URL` in your `.env` file, you can simply run:

```bash
npm run deploy -- --env production
```

Additional deployment options:

```bash
npm run deploy -- --env production --skip-tests
```

## Model Definition Reference

### Model Object

| Field | Type | Description | Required |
|-------|------|-------------|----------|
| id | string | Unique identifier for the model | Yes |
| name | string | Human-readable name | Yes |
| version | string | Model version in semver format | Yes |
| description | string | Detailed description | No |
| type | string | Model type (e.g., 'text-generation') | Yes |
| status | string | Current status ('active', 'inactive', 'deprecated') | Yes |
| metadata | object | Additional model metadata | No |

### Service Object

| Field | Type | Description | Required |
|-------|------|-------------|----------|
| id | string | Unique identifier for the service | Yes |
| name | string | Human-readable name | Yes |
| description | string | Detailed description | No |
| status | string | Current status ('running', 'stopped', 'error') | Yes |
| metadata | object | Additional service metadata | No |

## Testing

To verify your integration is working correctly:

1. Start the local llm-mcp server (refer to [llm-mcp documentation](./index))
2. Register your models: `npm run register-local`
3. Check the MCP UI or API to confirm models and services are registered
4. Test service functionality through the MCP API

To run unit tests:

```bash
npm test
```

## Troubleshooting

### Common Issues

- **Connection failures**: Check that the MCP server is running and accessible
- **Authentication errors**: Verify your API key is correct in the `.env` file
- **Registration failures**: Ensure your model and service definitions match the MCP schema
- **Server validation errors**: Check server logs for detailed error messages

### Logs

Both registration and deployment scripts provide detailed logs to help diagnose issues.

## CI/CD Integration

For GitLab CI integration, add the following to your `.gitlab-ci.yml`:

```yaml
deploy_mcp:
  stage: deploy
  script:
    - cd LLM
    - npm install
    - npm run deploy -- --env $CI_ENVIRONMENT_NAME
  environment:
    name: $CI_ENVIRONMENT_NAME
  only:
    - main
    - tags
```

Set the required environment variables in your CI/CD settings:
- `REMOTE_MCP_URL`
- `MCP_API_KEY`

## Advanced Configuration

### Custom Model Types

You can define custom model types by extending the basic model schema:

```json
{
  "id": "custom-model-1",
  "name": "Custom Model",
  "version": "1.0.0",
  "type": "custom-type",
  "status": "active",
  "metadata": {
    "custom_field": "custom_value",
    "framework": "pytorch",
    "capabilities": ["custom-capability"]
  }
}
```

### Environment-specific Configurations

You can maintain multiple configuration files for different environments:

```bash
# Using a specific configuration file
npm run register-local -- --config mcp-config.production.json
```

## Security Considerations

- Always use environment variables for sensitive information
- Do not commit API keys to version control
- Use secure connections for remote servers (HTTPS)
- Implement proper authentication for all API calls
