---
id: components-llmcli-CLAUDE
---
# CLAUDE.md - LLM Platform Projects Guidelines

## Project Management Commands

The llmcli now includes a comprehensive set of commands for managing multiple
LLM Platform projects:

### Basic Git Operations

- `llmcli dev status-all` - Check git status across all projects
- `llmcli dev pull-all` - Pull latest changes for all projects
- `llmcli dev push-all` - Push changes to all projects
- `llmcli dev branch-all <branch-name>` - Create a new branch in all projects

### Semantic Release and CI Integration

- `llmcli dev setup-semantic-release` - Set up semantic-release in all projects
- `llmcli dev verify-release` - Verify semantic-release configuration in all
  projects
- `llmcli dev migrate-ci` - Migrate from BFCommon CI components to BFCIComponents
- `llmcli dev verify-ci` - Verify Docker image usage in CI configurations

### Project Setup and Maintenance

- `llmcli dev install-all` - Install dependencies for all projects
- `llmcli dev run-all <command>` - Run a custom command in all projects
- `llmcli dev script-all <script-name>` - Run an npm script in all projects
- `llmcli dev setup` - Set up a new development environment

### Enhanced Management Features

- `llmcli dev commit-all <message>` - Commit changes with semantic commit
  messages to all projects
- `llmcli dev setup-remotes` - Set up Git remotes for all projects
- `llmcli dev replace-bfmodel` - Replace all instances of BFModel with BFLLM
  across the codebase

## Usage Examples

### Semantic Commits Workflow

```bash
# Make changes to multiple projects
cd /path/to/LLM

# Commit changes with a semantic commit message
llmcli dev commit-all "fix: update file paths in CI configuration"

# Push changes to all repositories
llmcli dev push-all
```

### Setting Up Semantic Release

```bash
# Check which projects need semantic-release setup
llmcli dev verify-release

# Set up semantic-release for all projects
llmcli dev setup-semantic-release

# Commit the changes
llmcli dev commit-all "ci: add semantic-release configuration"
```

### BFModel to BFLLM Migration

```bash
# Replace all references to BFModel with BFLLM
llmcli dev replace-bfmodel

# Commit the changes
llmcli dev commit-all "fix: rename BFModel to BFLLM throughout the codebase"
```

## Semantic Release

The llmcli includes commands to set up semantic versioning and automated
releases:

```bash
# Set up semantic release for all projects
llmcli dev setup-semantic-release

# Set up semantic release for specific projects
llmcli dev setup-semantic-release --projects BFAPI,llm-ui
```

This configures projects with:

- Automatic versioning based on conventional commits
- Automatic changelog generation
- GitLab release creation
- Package registry publishing

## Build/Test/Lint Commands

- `npm run build` - Build TypeScript projects
- `npm run dev` - Run development server with hot reload
- `npm run lint` - Run ESLint checks
- `npm run typecheck` - Verify TypeScript without emitting files
- `npm test` - Run all tests
- `npm test -- -t "test name"` - Run specific test
- `npm run test:watch` - Run tests in watch mode
- `npm run test:integration` - Run integration tests
- `npm run mcp` - Start Model Context Protocol server

## Code Style Guidelines

- **Node.js Version**: Use v20 or above for all projects
- **Module System**: ES Modules with TypeScript, `type: "module"` in
  package.json
- **Imports**: Order: 1) Node modules 2) External libs 3) Local imports
- **Types**: Use strict TypeScript with interfaces/types for all code
- **Error Handling**: Use `LLM PlatformError` class from BFCommon
- **Naming**: camelCase for variables/functions, PascalCase for classes
- **Formatting**: 2 spaces indentation, semicolons, max 100 chars width
- **Environment**: Use ENV object from BFCommon for config values
- **Exports**: Prefer named exports over default exports
- **Documentation**: JSDoc comments for public APIs and complex functions
- **Commits**: Follow the Conventional Commits standard:
  - `feat:` - New feature (minor version bump)
  - `fix:` - Bug fix (patch version bump)
  - `docs:` - Documentation changes
  - `style:` - Code style changes (formatting)
  - `refactor:` - Code refactoring
  - `perf:` - Performance improvements
  - `test:` - Adding or updating tests
  - `build:` - Build system changes
  - `ci:` - CI/CD changes
  - `chore:` - Maintenance tasks
  - Use `BREAKING CHANGE:` in commit body for major version bumps

## Architecture

- **Shared Code**: Use BFCommon library for utilities across projects
- **TypeScript**: Keep strict mode enabled in tsconfig.json
- **Error Codes**: Use standard error codes from BFCommon
- **Logging**: Use structured logging from BFCommon utilities
- **API Clients**: Use shared API client from BFCommon
- **Testing**: Jest with ESM support via ts-jest

## AI/ML Integration

- Keep AI providers interchangeable with consistent interfaces
- Use model registry for versioning and tracking models
- Standard model name across codebase: `bluefly-rfp-server`
- Cache AI responses when appropriate
- Always validate AI outputs before use
