---
id: components-llmcli-api
---
# llmcli API Reference

This section documents the programmatic API of llmcli for developers who want to
integrate with or extend the CLI.

## Core API

llmcli exposes a core API that can be used in other Node.js applications:

```javascript
import { llmcli } from 'llmcli';

// Initialize the CLI programmatically
const cli = new llmcli({
  configPath: './config.json',
  silent: false,
  logLevel: 'info',
});

// Execute a command
const result = await cli.execute(['model', 'list']);
console.log(result);
```

## Configuration API

The configuration API allows you to manage llmcli settings:

```javascript
import { ConfigManager } from 'llmcli/config';

// Load configuration
const config = new ConfigManager();

// Get configuration value
const apiKey = config.get('apiKey');

// Set configuration value
config.set('apiUrl', 'https://api.bluefly.io');

// Save configuration
config.save();
```

## Command Registration API

You can extend llmcli with custom commands:

```javascript
import { registerCommand } from 'llmcli/commands';

// Register a custom command
registerCommand({
  name: 'custom',
  description: 'A custom command',
  action: async args => {
    // Command implementation
    return { success: true, data: 'Custom command executed' };
  },
});
```

## Plugin API

llmcli supports plugins that can add new functionality:

```javascript
import { registerPlugin } from 'llmcli/plugins';

// Register a plugin
registerPlugin({
  name: 'my-plugin',
  version: '1.0.0',
  commands: [
    // Plugin commands
  ],
  hooks: {
    // Plugin hooks
    beforeCommand: (commandName, args) => {
      console.log(`Running command: ${commandName}`);
    },
  },
});
```

## Utility Functions

llmcli provides utility functions that can be used in custom commands and
plugins:

- `formatOutput(data, format)` - Format output in different formats (JSON,
  table, etc.)
- `parseArguments(args)` - Parse command arguments
- `validateInput(input, schema)` - Validate user input against a schema

## Type Definitions

Type definitions are available for TypeScript users:

```typescript
// Example type definition
interface CommandOptions {
  name: string;
  description: string;
  action: (args: string[]) => Promise<CommandResult>;
  options?: CommandOption[];
}
```

For detailed type definitions, see the
[TypeScript definition files](https://gitlab.bluefly.io/blueflyio/internal-projects/llm/llmcli/blob/main/src/types).
