---
id: components-llmcli-technical-WORKFLOW
---
# Workflow Module Technical Documentation

The workflow module in llmcli provides integration with GitLab workflows through
the GitLab CLI (`glab`). This document describes the technical implementation
details and how to extend or modify this functionality.

## Overview

The workflow module consists of the following components:

1. **Command Definition**: In `llmcli/src/index.js`, the `setupWorkflowCommands`
   function defines all available workflow commands.
2. **Command Implementation**: In `llmcli/src/commands/workflow.js`, the
   implementations for each command are defined in the `workflowCommands`
   object.
3. **Utility Functions**: In `llmcli/src/utils/gitlab.js`, reusable functions for
   interacting with GitLab are defined.

## Dependencies

The workflow module has the following dependencies:

- `glab`: The GitLab CLI (installed automatically if not found)
- `child_process`: For executing shell commands
- `ora`: For displaying spinner animations during long-running operations
- `chalk`: For colored output in the terminal

## Integration with GitLab CLI

The workflow module integrates with GitLab CLI (glab) by:

1. Checking if `glab` is installed using `isGlabInstalled()`
2. Installing it if necessary using `installGlab()`
3. Ensuring the user is authenticated with `authenticate()`
4. Using the `glab` CLI to execute GitLab operations

## Project Management

The workflow commands use the project utilities from
`llmcli/src/utils/projects.js` to manage multi-repository projects:

- `getProjectPath(project)`: Retrieves the local file path for a project
- `projects` object: Contains configuration for all known projects

This allows workflow commands to:

1. Find the correct local repository path for each project
2. Execute GitLab commands within the right directory context
3. Apply operations across multiple repositories when needed

## Command Structure

Each workflow command follows a similar pattern:

1. Validate required parameters (project name, etc.)
2. Get the local path for the project
3. Change to the project directory
4. Ensure GitLab CLI is installed and authenticated
5. Execute the GitLab CLI command with appropriate arguments
6. Parse and display the results

## GitLab Utility Functions

The `gitlab.js` utility file provides the following reusable functions:

- **Authentication**: `isAuthenticated()`, `authenticate()`
- **Installation**: `isGlabInstalled()`, `installGlab()`
- **Merge Requests**: `createMergeRequest()`, `updateMergeRequest()`,
  `getMergeRequests()`
- **Pipelines**: `runPipeline()`, `getPipelines()`
- **Issues**: `createIssue()`
- **Releases**: `createRelease()`
- **Project Management**: `configureProjectWorkflow()`, `getProjectUrl()`,
  `cloneProject()`

## Error Handling

Errors are handled by:

1. Catching exceptions in each command function
2. Logging detailed error messages using the logger
3. Returning appropriate exit codes for CLI commands

## Adding New Commands

To add a new workflow command:

1. Update `setupWorkflowCommands()` in `llmcli/src/index.js`:

   ```js
   workflow
     .command('new-command')
     .description('Description of the new command')
     .option('-o, --option <value>', 'Option description')
     .action(async options => {
       try {
         await workflowCommands.newCommand(options);
       } catch (error) {
         logger.error(`Failed to execute new command: ${error.message}`);
         process.exit(1);
       }
     });
   ```

2. Add the implementation to `workflowCommands` in
   `llmcli/src/commands/workflow.js`:

   ```js
   newCommand: async options => {
     const project = options.project || null;
     if (!project) {
       throw new Error('Project name is required (--project)');
     }

     try {
       logger.info(`Executing new command for ${project}...`);
       const projectPath = getProjectPath(project);
       if (!projectPath) {
         throw new Error(`Project ${project} not found locally`);
       }

       process.chdir(projectPath);
       await ensureGlabAuth();

       // Implement the command logic
       const result = execSync('glab some-command', { encoding: 'utf8' });
       logger.success(`Successfully executed new command for ${project}`);
       console.log(result);
     } catch (error) {
       logger.error(`Failed to execute new command for ${project}: ${error.message}`);
       throw error;
     }
   };
   ```

3. Add utility functions to `llmcli/src/utils/gitlab.js` if needed.

## Testing

To test workflow commands:

1. Ensure you have a local clone of the repository
2. Install the GitLab CLI (`glab`)
3. Authenticate with GitLab (`glab auth login`)
4. Create test projects or use existing projects
5. Execute the workflow commands with appropriate parameters

Example test:

```bash
# Create a test branch
git checkout -b workflow-test

# Make changes
echo "Test" > test.txt
git add test.txt
git commit -m "test: Add test file"

# Test creating a merge request
llmcli workflow mr-create --project YourProject --title "Test MR" --source workflow-test --target main
```
