---
id: database-management
---
# Database Management Guide

<!-- title: Database Management Guide -->
<!-- description: Practical guide for database management and migrations in the LLM Platform Platform -->
<!-- lastUpdated: 2024-03-19 -->

## Overview

This guide covers practical database management for the LLM Platform Platform, including schema management, migrations, backups, and optimization.

## Schema Management

### TypeORM Configuration

```typescript
// src/config/database.config.ts
import { TypeOrmModuleOptions } from '@nestjs/typeorm';
import { ConfigService } from '@nestjs/config';

export const getDatabaseConfig = (
  configService: ConfigService
): TypeOrmModuleOptions => ({
  type: 'postgres',
  url: configService.get('DATABASE_URL'),
  ssl: {
    rejectUnauthorized: false
  },
  entities: ['dist/**/*.entity{.ts,.js}'],
  migrations: ['dist/migrations/*{.ts,.js}'],
  synchronize: false,
  logging: configService.get('NODE_ENV') === 'development',
  maxQueryExecutionTime: 1000,
  extra: {
    max: 20,
    connectionTimeoutMillis: 5000,
    idleTimeoutMillis: 30000
  }
});

// src/app.module.ts
@Module({
  imports: [
    TypeOrmModule.forRootAsync({
      imports: [ConfigModule],
      useFactory: getDatabaseConfig,
      inject: [ConfigService]
    })
  ]
})
export class AppModule {}
```

### Entity Definitions

```typescript
// src/entities/agent.entity.ts
@Entity('agents')
export class Agent {
  @PrimaryGeneratedColumn('uuid')
  id: string;
  
  @Column()
  name: string;
  
  @Column()
  model: string;
  
  @Column({
    type: 'jsonb',
    nullable: true
  })
  config: Record<string, any>;
  
  @Column({
    type: 'enum',
    enum: AgentStatus,
    default: AgentStatus.INACTIVE
  })
  status: AgentStatus;
  
  @Column({
    type: 'timestamp',
    default: () => 'CURRENT_TIMESTAMP'
  })
  createdAt: Date;
  
  @Column({
    type: 'timestamp',
    default: () => 'CURRENT_TIMESTAMP',
    onUpdate: 'CURRENT_TIMESTAMP'
  })
  updatedAt: Date;
  
  @OneToMany(() => Execution, execution => execution.agent)
  executions: Execution[];
  
  @ManyToOne(() => User, user => user.agents)
  owner: User;
  
  @BeforeInsert()
  @BeforeUpdate()
  updateTimestamps() {
    this.updatedAt = new Date();
  }
}

// src/entities/execution.entity.ts
@Entity('executions')
export class Execution {
  @PrimaryGeneratedColumn('uuid')
  id: string;
  
  @Column('text')
  input: string;
  
  @Column('text')
  output: string;
  
  @Column('int')
  tokens: number;
  
  @Column({
    type: 'jsonb',
    nullable: true
  })
  metadata: Record<string, any>;
  
  @Column({
    type: 'timestamp',
    default: () => 'CURRENT_TIMESTAMP'
  })
  createdAt: Date;
  
  @ManyToOne(() => Agent, agent => agent.executions)
  agent: Agent;
  
  @ManyToOne(() => User)
  user: User;
}
```

## Migrations

### Migration Generation

```typescript
// src/migrations/1234567890-CreateAgentsTable.ts
import { MigrationInterface, QueryRunner } from 'typeorm';

export class CreateAgentsTable1234567890 implements MigrationInterface {
  name = 'CreateAgentsTable1234567890';
  
  public async up(queryRunner: QueryRunner): Promise<void> {
    await queryRunner.query(`
      CREATE TYPE agent_status AS ENUM (
        'active',
        'inactive',
        'error'
      );
      
      CREATE TABLE agents (
        id UUID PRIMARY KEY DEFAULT gen_random_uuid(),
        name VARCHAR(255) NOT NULL,
        model VARCHAR(255) NOT NULL,
        config JSONB,
        status agent_status NOT NULL DEFAULT 'inactive',
        created_at TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP,
        updated_at TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP,
        owner_id UUID REFERENCES users(id)
      );
      
      CREATE INDEX idx_agents_owner ON agents(owner_id);
      CREATE INDEX idx_agents_status ON agents(status);
      CREATE INDEX idx_agents_created ON agents(created_at);
    `);
  }
  
  public async down(queryRunner: QueryRunner): Promise<void> {
    await queryRunner.query(`
      DROP TABLE agents;
      DROP TYPE agent_status;
    `);
  }
}

// src/migrations/1234567891-CreateExecutionsTable.ts
import { MigrationInterface, QueryRunner } from 'typeorm';

export class CreateExecutionsTable1234567891 implements MigrationInterface {
  name = 'CreateExecutionsTable1234567891';
  
  public async up(queryRunner: QueryRunner): Promise<void> {
    await queryRunner.query(`
      CREATE TABLE executions (
        id UUID PRIMARY KEY DEFAULT gen_random_uuid(),
        input TEXT NOT NULL,
        output TEXT NOT NULL,
        tokens INTEGER NOT NULL,
        metadata JSONB,
        created_at TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP,
        agent_id UUID REFERENCES agents(id),
        user_id UUID REFERENCES users(id)
      );
      
      CREATE INDEX idx_executions_agent ON executions(agent_id);
      CREATE INDEX idx_executions_user ON executions(user_id);
      CREATE INDEX idx_executions_created ON executions(created_at);
    `);
  }
  
  public async down(queryRunner: QueryRunner): Promise<void> {
    await queryRunner.query(`
      DROP TABLE executions;
    `);
  }
}
```

### Migration Management

```typescript
// src/database/migration.service.ts
@Injectable()
export class MigrationService {
  constructor(
    private dataSource: DataSource,
    private logger: Logger
  ) {}
  
  async runMigrations(): Promise<void> {
    try {
      this.logger.log('Running database migrations...');
      
      const migrations = await this.dataSource.runMigrations();
      
      this.logger.log(
        `Successfully ran ${migrations.length} migrations`
      );
    } catch (error) {
      this.logger.error('Migration failed:', error);
      throw error;
    }
  }
  
  async revertMigration(): Promise<void> {
    try {
      this.logger.log('Reverting last migration...');
      
      await this.dataSource.undoLastMigration();
      
      this.logger.log('Successfully reverted last migration');
    } catch (error) {
      this.logger.error('Migration revert failed:', error);
      throw error;
    }
  }
  
  async showMigrations(): Promise<void> {
    try {
      const migrations = await this.dataSource.showMigrations();
      
      this.logger.log('Pending migrations:');
      migrations.forEach(migration => {
        this.logger.log(`- ${migration.name}`);
      });
    } catch (error) {
      this.logger.error('Failed to show migrations:', error);
      throw error;
    }
  }
}
```

## Backup and Recovery

### Backup Script

```typescript
// scripts/backup.ts
import { exec } from 'child_process';
import { promisify } from 'util';
import { S3 } from 'aws-sdk';
import { format } from 'date-fns';

const execAsync = promisify(exec);
const s3 = new S3();

async function backupDatabase(): Promise<void> {
  const timestamp = format(new Date(), 'yyyy-MM-dd-HH-mm');
  const filename = `backup-${timestamp}.sql`;
  
  try {
    // Create backup
    await execAsync(
      `PGPASSWORD=$DB_PASSWORD pg_dump -h $DB_HOST -U $DB_USER -d $DB_NAME -F c -f ${filename}`
    );
    
    // Upload to S3
    await s3.upload({
      Bucket: process.env.BACKUP_BUCKET,
      Key: `database/${filename}`,
      Body: require('fs').createReadStream(filename)
    }).promise();
    
    // Clean up
    await execAsync(`rm ${filename}`);
    
    console.log(`Backup completed: ${filename}`);
  } catch (error) {
    console.error('Backup failed:', error);
    throw error;
  }
}

// Run backup
backupDatabase().catch(console.error);
```

### Recovery Script

```typescript
// scripts/recover.ts
import { exec } from 'child_process';
import { promisify } from 'util';
import { S3 } from 'aws-sdk';

const execAsync = promisify(exec);
const s3 = new S3();

async function recoverDatabase(backupKey: string): Promise<void> {
  try {
    // Download from S3
    const { Body } = await s3.getObject({
      Bucket: process.env.BACKUP_BUCKET,
      Key: backupKey
    }).promise();
    
    // Save to file
    const filename = backupKey.split('/').pop();
    require('fs').writeFileSync(filename, Body);
    
    // Restore database
    await execAsync(
      `PGPASSWORD=$DB_PASSWORD pg_restore -h $DB_HOST -U $DB_USER -d $DB_NAME -c ${filename}`
    );
    
    // Clean up
    await execAsync(`rm ${filename}`);
    
    console.log(`Recovery completed: ${backupKey}`);
  } catch (error) {
    console.error('Recovery failed:', error);
    throw error;
  }
}

// Run recovery
const backupKey = process.argv[2];
if (!backupKey) {
  console.error('Please provide backup key');
  process.exit(1);
}

recoverDatabase(backupKey).catch(console.error);
```

## Query Optimization

### Index Management

```typescript
// src/database/index.service.ts
@Injectable()
export class IndexService {
  constructor(
    private dataSource: DataSource,
    private logger: Logger
  ) {}
  
  async analyzeIndexes(): Promise<void> {
    const queryRunner = this.dataSource.createQueryRunner();
    
    try {
      // Get index usage statistics
      const stats = await queryRunner.query(`
        SELECT
          schemaname,
          tablename,
          indexname,
          idx_scan,
          idx_tup_read,
          idx_tup_fetch
        FROM pg_stat_user_indexes
        ORDER BY idx_scan DESC;
      `);
      
      // Analyze unused indexes
      const unused = stats.filter(
        stat => stat.idx_scan === 0
      );
      
      if (unused.length > 0) {
        this.logger.warn('Unused indexes found:', unused);
      }
      
      // Analyze index sizes
      const sizes = await queryRunner.query(`
        SELECT
          schemaname,
          tablename,
          indexname,
          pg_size_pretty(pg_relation_size(schemaname || '.' || indexname::text)) as size
        FROM pg_indexes
        ORDER BY pg_relation_size(schemaname || '.' || indexname::text) DESC;
      `);
      
      this.logger.log('Index sizes:', sizes);
    } finally {
      await queryRunner.release();
    }
  }
  
  async createIndex(
    table: string,
    columns: string[],
    options: {
      unique?: boolean;
      concurrent?: boolean;
      where?: string;
    } = {}
  ): Promise<void> {
    const queryRunner = this.dataSource.createQueryRunner();
    
    try {
      const indexName = `idx_${table}_${columns.join('_')}`;
      const unique = options.unique ? 'UNIQUE' : '';
      const concurrent = options.concurrent ? 'CONCURRENTLY' : '';
      const where = options.where ? `WHERE ${options.where}` : '';
      
      await queryRunner.query(`
        CREATE ${unique} INDEX ${concurrent} ${indexName}
        ON ${table} (${columns.join(', ')})
        ${where};
      `);
      
      this.logger.log(`Created index: ${indexName}`);
    } finally {
      await queryRunner.release();
    }
  }
}
```

### Query Analysis

```typescript
// src/database/query.service.ts
@Injectable()
export class QueryService {
  constructor(
    private dataSource: DataSource,
    private logger: Logger
  ) {}
  
  async analyzeQuery(query: string): Promise<any> {
    const queryRunner = this.dataSource.createQueryRunner();
    
    try {
      // Get query plan
      const plan = await queryRunner.query(`
        EXPLAIN (ANALYZE, BUFFERS, FORMAT JSON)
        ${query};
      `);
      
      // Analyze plan
      const analysis = this.analyzePlan(plan[0]['QUERY PLAN']);
      
      this.logger.log('Query analysis:', analysis);
      
      return analysis;
    } finally {
      await queryRunner.release();
    }
  }
  
  private analyzePlan(plan: any): any {
    return {
      executionTime: plan.Execution_Time,
      planningTime: plan.Planning_Time,
      bufferUsage: plan.Buffers,
      nodeTypes: this.getNodeTypes(plan.Plan),
      recommendations: this.getRecommendations(plan.Plan)
    };
  }
  
  private getNodeTypes(plan: any): string[] {
    const types = new Set<string>();
    
    const traverse = (node: any) => {
      types.add(node.Node_Type);
      if (node.Plans) {
        node.Plans.forEach(traverse);
      }
    };
    
    traverse(plan);
    return Array.from(types);
  }
  
  private getRecommendations(plan: any): string[] {
    const recommendations: string[] = [];
    
    const traverse = (node: any) => {
      // Check for sequential scans
      if (node.Node_Type === 'Seq Scan') {
        recommendations.push(
          `Consider adding an index for ${node.Relation_Name}`
        );
      }
      
      // Check for nested loops
      if (node.Node_Type === 'Nested Loop') {
        recommendations.push(
          'Consider optimizing join conditions'
        );
      }
      
      // Check for sort operations
      if (node.Node_Type === 'Sort') {
        recommendations.push(
          'Consider adding an index to avoid sorting'
        );
      }
      
      if (node.Plans) {
        node.Plans.forEach(traverse);
      }
    };
    
    traverse(plan);
    return recommendations;
  }
}
```

## Best Practices

1. **Schema Management**
   - Use migrations
   - Version control
   - Review changes
   - Test migrations

2. **Backup and Recovery**
   - Regular backups
   - Test recovery
   - Monitor space
   - Secure storage

3. **Query Optimization**
   - Use indexes
   - Analyze queries
   - Monitor performance
   - Regular maintenance

4. **Data Management**
   - Data validation
   - Data cleanup
   - Data archiving
   - Data security

## Related Documentation

- [Architecture Guide](../architecture/README)
- [Development Guide](../development/README)
- [Deployment Guide](../deployment/README)
- [Security Guide](../security/README)
- [Operations Guide](../operations/README) 
