---
id: development-pipeline
---
# CI/CD Pipeline

<!-- title: CI/CD Pipeline -->
<!-- description: Developer guide for continuous integration, deployment, and release automation in the LLM Platform Platform -->
<!-- lastUpdated: 2024-03-19 -->

This guide covers the CI/CD pipeline implementation for the LLM Platform Platform, including build automation, testing, deployment strategies, and release management.

## Architecture

### Pipeline Components

```typescript
// Pipeline Stage Interface
interface PipelineStage {
  name: string;
  description: string;
  jobs: PipelineJob[];
  dependencies?: string[];
  conditions?: PipelineCondition[];
}

// Pipeline Job Interface
interface PipelineJob {
  name: string;
  stage: string;
  script: string[];
  artifacts?: ArtifactConfig;
  environment?: EnvironmentConfig;
  rules?: JobRule[];
  needs?: string[];
  timeout?: number;
}

// Environment Configuration
interface EnvironmentConfig {
  name: string;
  url?: string;
  deployment_tier: 'development' | 'staging' | 'production';
  variables: Record<string, string>;
  kubernetes?: KubernetesConfig;
}

// Artifact Configuration
interface ArtifactConfig {
  paths: string[];
  reports?: {
    junit?: string[];
    coverage?: string[];
    security?: string[];
  };
  expire_in?: string;
}

// Pipeline Condition
interface PipelineCondition {
  type: 'branch' | 'tag' | 'variable' | 'changes';
  value: string | string[];
  operator?: 'equals' | 'contains' | 'starts_with' | 'ends_with';
}
```

### Directory Structure

```
.gitlab/
├── ci/                    # CI/CD configuration
│   ├── templates/        # Reusable job templates
│   ├── stages/          # Stage definitions
│   └── variables/       # Pipeline variables
├── charts/               # Helm charts
│   ├── bfapi/          # API service chart
│   ├── bfagent/        # Agent service chart
│   └── llm-mcp/          # MCP service chart
└── environments/        # Environment configurations
    ├── development/    # Development environment
    ├── staging/       # Staging environment
    └── production/    # Production environment
```

## Pipeline Implementation

### Base Pipeline Configuration

```yaml
# .gitlab-ci.yml
include:
  - project: 'bluefly/platform-templates'
    ref: v1.0.0
    file: '/templates/base-pipeline.yml'
  - local: '.gitlab/ci/templates/*.yml'
  - local: '.gitlab/ci/stages/*.yml'

variables:
  DOCKER_REGISTRY: registry.bluefly.io
  DOCKER_IMAGE_PREFIX: bluefly
  KUBERNETES_NAMESPACE: ${CI_ENVIRONMENT_SLUG}
  HELM_RELEASE_PREFIX: bf

stages:
  - validate
  - test
  - build
  - security
  - deploy
  - release

workflow:
  rules:
    - if: $CI_PIPELINE_SOURCE == "merge_request"
    - if: $CI_COMMIT_BRANCH == $CI_DEFAULT_BRANCH
    - if: $CI_COMMIT_TAG =~ /^v\d+\.\d+\.\d+$/

default:
  image: node:18
  before_script:
    - npm ci
    - npm run setup
  after_script:
    - npm run cleanup
```

### Stage Templates

```yaml
# .gitlab/ci/templates/validate.yml
.validate:
  stage: validate
  script:
    - npm run lint
    - npm run type-check
    - npm run validate:openapi
  artifacts:
    reports:
      codequality: gl-code-quality-report.json

# .gitlab/ci/templates/test.yml
.test:
  stage: test
  services:
    - name: redis:latest
      alias: redis
    - name: elasticsearch:latest
      alias: elasticsearch
  variables:
    REDIS_HOST: redis
    ELASTICSEARCH_URL: http://elasticsearch:9200
  script:
    - npm run test:unit
    - npm run test:integration
    - npm run test:e2e
  coverage: '/Coverage: \d+\.\d+% of statements/'
  artifacts:
    reports:
      coverage_report:
        coverage_format: cobertura
        path: coverage/cobertura-coverage.xml
      junit: junit.xml

# .gitlab/ci/templates/build.yml
.build:
  stage: build
  image: docker:20.10
  services:
    - docker:20.10-dind
  variables:
    DOCKER_TLS_CERTDIR: "/certs"
  script:
    - docker build -t $DOCKER_REGISTRY/$DOCKER_IMAGE_PREFIX/$CI_PROJECT_NAME:$CI_COMMIT_SHA .
    - docker push $DOCKER_REGISTRY/$DOCKER_IMAGE_PREFIX/$CI_PROJECT_NAME:$CI_COMMIT_SHA
  artifacts:
    paths:
      - .dockerignore
      - Dockerfile
```

### Environment-Specific Configurations

```yaml
# .gitlab/ci/stages/deploy-development.yml
deploy:development:
  extends: .deploy
  stage: deploy
  environment:
    name: development
    url: https://dev.bluefly.io
    deployment_tier: development
  variables:
    KUBERNETES_NAMESPACE: bluefly-dev
    HELM_RELEASE_NAME: bf-dev
  script:
    - helm upgrade --install $HELM_RELEASE_NAME .gitlab/charts/$CI_PROJECT_NAME
      --namespace $KUBERNETES_NAMESPACE
      --set image.tag=$CI_COMMIT_SHA
      --set environment=development
      --values .gitlab/environments/development/values.yaml

# .gitlab/ci/stages/deploy-staging.yml
deploy:staging:
  extends: .deploy
  stage: deploy
  environment:
    name: staging
    url: https://staging.bluefly.io
    deployment_tier: staging
  variables:
    KUBERNETES_NAMESPACE: bluefly-staging
    HELM_RELEASE_NAME: bf-staging
  script:
    - helm upgrade --install $HELM_RELEASE_NAME .gitlab/charts/$CI_PROJECT_NAME
      --namespace $KUBERNETES_NAMESPACE
      --set image.tag=$CI_COMMIT_SHA
      --set environment=staging
      --values .gitlab/environments/staging/values.yaml
  rules:
    - if: $CI_COMMIT_BRANCH == $CI_DEFAULT_BRANCH
      when: manual
      allow_failure: true

# .gitlab/ci/stages/deploy-production.yml
deploy:production:
  extends: .deploy
  stage: deploy
  environment:
    name: production
    url: https://bluefly.io
    deployment_tier: production
  variables:
    KUBERNETES_NAMESPACE: bluefly-prod
    HELM_RELEASE_NAME: bf-prod
  script:
    - helm upgrade --install $HELM_RELEASE_NAME .gitlab/charts/$CI_PROJECT_NAME
      --namespace $KUBERNETES_NAMESPACE
      --set image.tag=$CI_COMMIT_SHA
      --set environment=production
      --values .gitlab/environments/production/values.yaml
  rules:
    - if: $CI_COMMIT_TAG =~ /^v\d+\.\d+\.\d+$/
      when: manual
      allow_failure: false
```

### Release Automation

```yaml
# .gitlab/ci/stages/release.yml
release:
  stage: release
  image: registry.bluefly.io/tools/release-manager:latest
  script:
    - release-manager create
      --version $CI_COMMIT_TAG
      --changelog CHANGELOG.md
      --assets "dist/*"
      --release-notes "docs/releases/$CI_COMMIT_TAG.md"
  rules:
    - if: $CI_COMMIT_TAG =~ /^v\d+\.\d+\.\d+$/
  environment:
    name: production
```

## Helm Charts

### Base Chart Template

```yaml
# .gitlab/charts/templates/_helpers.tpl
{{/*
Expand the name of the chart.
*/}}
{{- define "bluefly.name" -}}
{{- default .Chart.Name .Values.nameOverride | trunc 63 | trimSuffix "-" }}
{{- end }}

{{/*
Create a default fully qualified app name.
*/}}
{{- define "bluefly.fullname" -}}
{{- if .Values.fullnameOverride }}
{{- .Values.fullnameOverride | trunc 63 | trimSuffix "-" }}
{{- else }}
{{- $name := default .Chart.Name .Values.nameOverride }}
{{- if contains $name .Release.Name }}
{{- .Release.Name | trunc 63 | trimSuffix "-" }}
{{- else }}
{{- printf "%s-%s" .Release.Name $name | trunc 63 | trimSuffix "-" }}
{{- end }}
{{- end }}
{{- end }}

{{/*
Create chart name and version as used by the chart label.
*/}}
{{- define "bluefly.chart" -}}
{{- printf "%s-%s" .Chart.Name .Chart.Version | replace "+" "_" | trunc 63 | trimSuffix "-" }}
{{- end }}
```

### Service Chart

```yaml
# .gitlab/charts/bfapi/Chart.yaml
apiVersion: v2
name: bfapi
description: LLM Platform API Service
type: application
version: 0.1.0
appVersion: "1.0.0"

# .gitlab/charts/bfapi/values.yaml
replicaCount: 1

image:
  repository: registry.bluefly.io/bluefly/bfapi
  tag: latest
  pullPolicy: IfNotPresent

service:
  type: ClusterIP
  port: 80

ingress:
  enabled: true
  className: nginx
  annotations:
    kubernetes.io/ingress.class: nginx
    cert-manager.io/cluster-issuer: letsencrypt-prod
  hosts:
    - host: api.bluefly.io
      paths:
        - path: /
          pathType: Prefix

resources:
  limits:
    cpu: 1000m
    memory: 1Gi
  requests:
    cpu: 500m
    memory: 512Mi

autoscaling:
  enabled: true
  minReplicas: 2
  maxReplicas: 10
  targetCPUUtilizationPercentage: 80
  targetMemoryUtilizationPercentage: 80

env:
  - name: NODE_ENV
    value: production
  - name: REDIS_HOST
    valueFrom:
      secretKeyRef:
        name: bfapi-secrets
        key: redis-host
  - name: ELASTICSEARCH_URL
    valueFrom:
      secretKeyRef:
        name: bfapi-secrets
        key: elasticsearch-url

# .gitlab/charts/bfapi/templates/deployment.yaml
apiVersion: apps/v1
kind: Deployment
metadata:
  name: {{ include "bluefly.fullname" . }}
  labels:
    {{- include "bluefly.labels" . | nindent 4 }}
spec:
  replicas: {{ .Values.replicaCount }}
  selector:
    matchLabels:
      {{- include "bluefly.selectorLabels" . | nindent 6 }}
  template:
    metadata:
      labels:
        {{- include "bluefly.selectorLabels" . | nindent 8 }}
    spec:
      containers:
        - name: {{ .Chart.Name }}
          image: "{{ .Values.image.repository }}:{{ .Values.image.tag }}"
          imagePullPolicy: {{ .Values.image.pullPolicy }}
          ports:
            - name: http
              containerPort: 80
              protocol: TCP
          livenessProbe:
            httpGet:
              path: /health
              port: http
            initialDelaySeconds: 30
            periodSeconds: 10
          readinessProbe:
            httpGet:
              path: /ready
              port: http
            initialDelaySeconds: 5
            periodSeconds: 5
          resources:
            {{- toYaml .Values.resources | nindent 12 }}
          env:
            {{- toYaml .Values.env | nindent 12 }}
```

## Deployment Strategies

### Rolling Updates

```yaml
# .gitlab/charts/bfapi/templates/deployment.yaml
spec:
  strategy:
    type: RollingUpdate
    rollingUpdate:
      maxSurge: 1
      maxUnavailable: 0
```

### Blue-Green Deployment

```yaml
# .gitlab/ci/stages/deploy-blue-green.yml
deploy:blue-green:
  extends: .deploy
  stage: deploy
  script:
    # Deploy new version
    - helm upgrade --install $HELM_RELEASE_NAME-green .gitlab/charts/$CI_PROJECT_NAME
      --namespace $KUBERNETES_NAMESPACE
      --set image.tag=$CI_COMMIT_SHA
      --set ingress.host=green.bluefly.io
    
    # Wait for new deployment to be ready
    - kubectl rollout status deployment/$HELM_RELEASE_NAME-green -n $KUBERNETES_NAMESPACE
    
    # Run smoke tests
    - npm run test:smoke -- --url https://green.bluefly.io
    
    # Switch traffic to new version
    - kubectl patch ingress $HELM_RELEASE_NAME -n $KUBERNETES_NAMESPACE
      --type=json
      -p='[{"op": "replace", "path": "/spec/rules/0/host", "value": "bluefly.io"}]'
    
    # Clean up old version
    - helm delete $HELM_RELEASE_NAME-blue -n $KUBERNETES_NAMESPACE
```

## Monitoring and Observability

### Pipeline Metrics

```typescript
// src/metrics/pipeline-metrics.ts
import { Metrics } from '../utils/metrics';

export class PipelineMetrics {
  private metrics: Metrics;
  
  constructor() {
    this.metrics = new Metrics('pipeline');
  }
  
  recordPipelineStart(pipelineId: string, branch: string): void {
    this.metrics.increment('pipeline_start_total', {
      branch,
      pipeline_id: pipelineId
    });
  }
  
  recordPipelineSuccess(pipelineId: string, duration: number): void {
    this.metrics.increment('pipeline_success_total');
    this.metrics.observe('pipeline_duration_seconds', duration, {
      status: 'success'
    });
  }
  
  recordPipelineFailure(pipelineId: string, error: Error): void {
    this.metrics.increment('pipeline_failure_total', {
      error: error.name
    });
  }
  
  recordDeploymentSuccess(environment: string, duration: number): void {
    this.metrics.increment('deployment_success_total', {
      environment
    });
    this.metrics.observe('deployment_duration_seconds', duration, {
      environment,
      status: 'success'
    });
  }
  
  recordDeploymentFailure(environment: string, error: Error): void {
    this.metrics.increment('deployment_failure_total', {
      environment,
      error: error.name
    });
  }
}
```

### Pipeline Dashboard

```yaml
# .gitlab/dashboards/pipeline-metrics.json
{
  "dashboard": {
    "title": "Pipeline Metrics",
    "panels": [
      {
        "title": "Pipeline Success Rate",
        "type": "graph",
        "datasource": "Prometheus",
        "targets": [
          {
            "expr": "rate(pipeline_success_total[1h]) / rate(pipeline_start_total[1h]) * 100",
            "legendFormat": "Success Rate"
          }
        ]
      },
      {
        "title": "Pipeline Duration",
        "type": "graph",
        "datasource": "Prometheus",
        "targets": [
          {
            "expr": "histogram_quantile(0.95, rate(pipeline_duration_seconds_bucket[1h]))",
            "legendFormat": "95th Percentile"
          }
        ]
      },
      {
        "title": "Deployment Success Rate",
        "type": "graph",
        "datasource": "Prometheus",
        "targets": [
          {
            "expr": "rate(deployment_success_total[1h]) / (rate(deployment_success_total[1h]) + rate(deployment_failure_total[1h])) * 100",
            "legendFormat": "{{environment}}"
          }
        ]
      }
    ]
  }
}
```

## Best Practices

1. **Pipeline Organization**
   - Use templates for reusable jobs
   - Separate stages logically
   - Implement proper dependencies
   - Use meaningful job names
   - Maintain clear documentation

2. **Build Process**
   - Optimize build caching
   - Use multi-stage builds
   - Implement artifact management
   - Version all dependencies
   - Sign and verify artifacts

3. **Testing Strategy**
   - Run tests in parallel
   - Implement test retries
   - Use appropriate test types
   - Maintain test data
   - Monitor test coverage

4. **Deployment**
   - Use deployment strategies
   - Implement rollback procedures
   - Monitor deployments
   - Validate deployments
   - Maintain deployment history

5. **Security**
   - Scan dependencies
   - Validate container images
   - Secure secrets management
   - Implement access control
   - Monitor security metrics

## Related Documentation

- [Testing Framework](./testing)
- [Security Guide](../operations/security)
- [Monitoring Guide](../operations/monitoring)
- [Release Guide](../operations/release) 
