---
id: development-reelcode-integration
---
# ReelCode Integration Guide

<!-- title: ReelCode Integration Guide -->
<!-- description: Guide for integrating ReelCode AI-powered coding standards with the LLM Platform Platform -->
<!-- lastUpdated: 2024-03-19 -->

## Overview

ReelCode is a comprehensive AI-powered coding standards management system that defines, enforces, and distributes coding standards for AI assistants across the LLM Platform Platform. It serves as the foundation for maintaining code quality, security, and consistency across our microservices architecture.

## Architecture

### Core Components

- **StandardsManager**: Central standards management
- **ValidationEngine**: Code validation
- **AIAssistantAdapter**: AI integration
- **CI Integration**: Pipeline tools

### Directory Structure

```
src/
├── core/           # Core functionality
├── validation/     # Code validation
├── ai/            # AI integration
├── ci/            # CI/CD utilities
└── cli/           # CLI implementation
```

## Integration Guide

### Installation

```bash
# Install dependencies
npm install @bluefly/reelcode

# Initialize in project
npx reelcode init
```

### Development Workflow

```bash
# Development
npm run dev:watch    # Watch mode
npm run build       # Build
npm run start       # Start

# Testing
npm test            # Run tests
npm run test:coverage  # Coverage
npm run test:watch  # Watch mode

# Code Quality
npm run lint        # Lint
npm run lint:fix    # Fix issues
npm run format      # Format
npm run typecheck   # Type check
```

### AI Assistant Integration

1. **Configuration Generation**
   ```bash
   # Generate AI config
   reelcode generate:ai-config
   ```

2. **Adapter Implementation**
   ```typescript
   // src/ai/adapters/claude-adapter.ts
   export class ClaudeAdapter implements AIAssistantAdapter {
     async generateConfig(standards: Standards): Promise<Config> {
       // Generate Claude-specific config
     }
     
     async applyConfig(config: Config): Promise<void> {
       // Apply to project
     }
   }
   ```

3. **Standards Definition**
   ```yaml
   # .reelcode/standards.yml
   extends: base-standards
   language: typescript
   rules:
     naming:
       pattern: camelCase
     testing:
       coverage: 90
     structure:
       src: required
   ```

### CI/CD Integration

1. **GitLab CI**
   ```bash
   # Generate configuration
   reelcode ci:generate gitlab --output .gitlab-ci.yml
   ```

2. **GitHub Actions**
   ```bash
   # Generate workflow
   reelcode ci:generate github --output .github/workflows/reelcode.yml
   ```

3. **Pipeline Validation**
   ```yaml
   # .gitlab-ci.yml
   standards:
     stage: validate
     script:
       - reelcode validate
     rules:
       - if: $CI_PIPELINE_SOURCE == "merge_request"
   ```

## Development Standards

### Test-Driven Development

1. Write tests first
2. Implement minimal code
3. Refactor with coverage
4. Repeat for features

### Code Quality

- 90%+ test coverage
- TypeScript for safety
- Document public APIs
- ESLint + Prettier

### Project Phases

1. **Phase 1**: Core Standards Engine
   - Standards management
   - Basic validation
   - CLI foundation

2. **Phase 2**: Project Integration
   - CI/CD integration
   - Multi-language support
   - Advanced validation

3. **Phase 3**: Advanced Features
   - AI assistant adapters
   - Custom rules
   - External usage

## Security Guidelines

- No sensitive data in commits
- Authenticate all APIs
- Least privilege principle
- Input validation

## File Management

### Preservation Policy

1. Move to `__DELETE_LATER`
2. Preserve structure
3. Add date suffix
4. Document changes

### Task Management

- Use TodoWrite
- Track completion
- Update docs

## CLI Reference

### Core Commands

```bash
# Validation
reelcode validate [path]

# AI Configuration
reelcode generate:ai-config [options]

# Project Integration
reelcode integrate [project]

# Analysis
reelcode analysis [type]
```

### Command Development

1. Create in `src/cli/commands/`
2. Register in `src/cli/index.ts`
3. Add tests
4. Update docs

## Related Documentation

- [Development Guide](../development/README)
- [Testing Guide](../testing/README)
- [CI/CD Guide](../development/pipeline)
- [Security Guide](../operations/security)
- [Monitoring Guide](../operations/monitoring) 
