---
id: drupal-DRUPAL_INTEGRATION
---
# Drupal Integration with Native AI.js and BFLLM

This document provides a comprehensive guide to the integration between Drupal and the LLM ecosystem components, specifically focusing on Native AI.js and BFLLM (DEPRECATED - migrate to LangChain) AI capabilities within Drupal sites while maintaining a clean architecture and separation of concerns.

## Architecture Overview

The integration follows a layered architecture that connects Drupal with the LLM ecosystem:

```mermaid
graph TB
    A[Drupal] -->|drupal_native_ai module| B[Bridge Layer]
    B -->|API| C[Native AI.js]
    C -->|Provider Interface| D[BFLLM]
    D -->|FederatedSwarmIntelligence| E[llm-mcp]
    
    F[field_eca_bridge module] -->|Events| A
    G[llm_platform_recipe module] -->|Installation| A
    
    subgraph "Drupal Layer"
        A
        F
        G
    end
    
    subgraph "Integration Layer"
        B
    end
    
    subgraph "AI Framework Layer"
        C
        D
        E
    end
```

## Key Components

### 1. drupal_native_ai Module

The `drupal_native_ai` module serves as the primary integration point within Drupal, providing:

- **AI-Enhanced Fields**: Custom field types for AI-enhanced content
- **Service Layer**: PHP services for AI operations
- **JavaScript Bridge**: Client-side integration with Native AI.js
- **Admin Interfaces**: Configuration and management UIs
- **Entity Processing**: AI-powered entity operations
- **Batch Operations**: Efficient processing of large content sets

#### Module Structure

```
drupal_native_ai/
├── drupal_native_ai.info.yml         # Module information
├── drupal_native_ai.module           # Core module hooks
├── drupal_native_ai.install          # Installation hooks
├── drupal_native_ai.libraries.yml    # JS/CSS libraries
├── drupal_native_ai.services.yml     # Service definitions
├── drupal_native_ai.permissions.yml  # Permission definitions
├── drupal_native_ai.routing.yml      # Route definitions
├── drupal_native_ai.recipe.yml       # Installation recipe
├── config/                           # Configuration schemas and defaults
├── src/                              # PHP source code
│   ├── Controller/                   # Controllers for routes
│   ├── Form/                         # Admin forms
│   ├── Plugin/                       # Plugin implementations
│   │   ├── Field/                    # Custom field types
│   │   ├── AIProvider/               # AI provider plugins
│   │   └── FieldEcaBridge/           # ECA integration
│   ├── Service/                      # Service classes
│   ├── Adapter/                      # Provider adapters
│   ├── Integration/                  # Integration classes
│   └── Event/                        # Event classes
├── js/                               # JavaScript files
├── css/                              # CSS files
├── templates/                        # Twig templates
├── tests/                            # Unit and functional tests
└── docs/                             # Documentation
```

### 2. Native AI.js Integration

Native AI.js serves as the core AI framework, providing:

- **Unified API**: Consistent interface for model interactions
- **Model Registry**: Discovery and registration of AI models
- **Provider Interface**: Standardized access to different AI providers
- **Type Safety**: Strong TypeScript types for reliable integration

#### Integration Structure

```typescript
// Native AI.js provider interface
interface IModelProvider {
  id: string;
  name: string;
  models: Model[];
  
  initialize(): Promise<void>;
  generateCompletion(request: CompletionRequest): Promise<CompletionResponse>;
  generateEmbeddings(request: EmbeddingRequest): Promise<EmbeddingResponse>;
  
  // Additional provider methods...
}

// Native AI.js implementation of BFLLM provider
class BFLLMProvider implements IModelProvider {
  constructor(config: ProviderConfig) {
    // Initialize BFLLM provider
  }
  
  // Implementation of provider methods...
}
```

### 3. BFLLM Integration

BFLLM (DEPRECATED - migrate to LangChain) AI capabilities including:

- **Federated Swarm Intelligence**: Coordinated agent networks
- **Emergence Detection**: Pattern detection in agent operations
- **Byzantine Consensus**: Fault-tolerant decision making
- **Model Orchestration**: Intelligent model selection and fallback

#### Integration with Drupal

The integration between BFLLM and Drupal focuses on:

1. **Field-Level Intelligence**: Processing field data with AI
2. **Entity Analysis**: Analyzing entire entities with agent swarms
3. **Event Dispatching**: Triggering Drupal events based on AI insights
4. **Pattern Storage**: Storing detected patterns as Drupal entities

## Configuration and Setup

### Installation

1. **Install Required Modules**:

```bash
composer require drupal/drupal_native_ai
composer require drupal/field_eca_bridge
composer require drupal/llm_platform_recipe
drush en drupal_native_ai field_eca_bridge llm_platform_recipe
```

2. **Apply Installation Recipe**:

```bash
drush recipe:apply native_ai
```

3. **Configure Server Connection**:

Navigate to `/admin/config/services/native-ai` and configure:
- Server URL
- Authentication settings
- Default models

### Connecting to BFLLM

Configure the connection to BFLLM through the Native AI.js provider:

```php
// settings.php or similar configuration
$settings['drupal_native_ai_settings'] = [
  'provider_settings' => [
    'bfllm' => [
      'url' => 'http://localhost:3002',
      'api_key' => 'your-api-key',
      'default_model' => 'llama-2-7b',
      'swarm_intelligence' => true,
      'emergence_detection' => true,
    ],
  ],
];
```

## Integration Features

### 1. AI-Enhanced Fields

The integration provides custom field types for AI-enhanced content:

```php
// Add an AI-enhanced text field to a content type
$field_storage = FieldStorageConfig::create([
  'field_name' => 'field_ai_content',
  'entity_type' => 'node',
  'type' => 'ai_enhanced_text',
  'settings' => [
    'enhancement_types' => ['improve', 'summarize', 'expand'],
  ],
]);
$field_storage->save();

$field = FieldConfig::create([
  'field_storage' => $field_storage,
  'bundle' => 'article',
  'label' => 'AI-Enhanced Content',
  'settings' => [
    'default_model' => 'llama-2-7b',
    'auto_enhance' => false,
  ],
]);
$field->save();
```

### 2. Entity Processing

Process entire entities with AI:

```php
// Process a node with AI
$node = Node::load(1);
$result = \Drupal::service('drupal_native_ai.entity_processor')
  ->processEntity($node, 'analyze', [
    'model' => 'llama-2-7b',
    'fields' => ['body', 'field_description'],
  ]);
```

### 3. Emergence Detection Integration

Use BFLLM's Emergence Detection system with Drupal:

```php
// Detect patterns in content
$patterns = \Drupal::service('drupal_native_ai.emergence_detector')
  ->detectPatterns($node, [
    'fields' => ['body'],
    'threshold' => 0.7,
    'store_results' => true,
  ]);

// Access pattern information
foreach ($patterns as $pattern) {
  $type = $pattern->getType();
  $impact = $pattern->getImpact();
  $frequency = $pattern->getFrequency();
  // Process pattern...
}
```

### 4. JavaScript Integration

Client-side integration for interactive AI features:

```javascript
// Initialize the Native AI.js bridge
Drupal.behaviors.nativeAI = {
  attach: function (context, settings) {
    if (context === document && settings.nativeAI) {
      Drupal.NativeAI.init(settings.nativeAI);
      
      // Add AI-powered features to the page
      $('.ai-enhance-button', context).once('ai-enhance').on('click', function() {
        const $content = $(this).closest('.field').find('.field-content');
        const text = $content.text();
        
        Drupal.NativeAI.generate(text, {
          model: 'llama-2-7b',
          parameters: {
            temperature: 0.7,
            max_tokens: 500,
          },
          operation: 'enhance',
        }).then(result => {
          if (result.success) {
            $content.html(result.output);
          } else {
            console.error('AI enhancement failed:', result.error);
          }
        });
      });
    }
  }
};
```

### 5. Field ECA Bridge Integration

Integrate with Event-Condition-Action workflows:

```php
// Register AI events with ECA
$events = [
  'enhancement_completed' => [
    'label' => t('AI enhancement completed'),
    'entity_type_id' => 'node',
  ],
  'pattern_detected' => [
    'label' => t('AI pattern detected'),
    'entity_type_id' => 'node',
  ],
];

foreach ($events as $id => $event) {
  \Drupal::service('plugin.manager.eca.event')->registerEvent(
    'drupal_native_ai',
    $id,
    $event['label'],
    $event['entity_type_id']
  );
}
```

## API Reference

### PHP Services

The integration provides several Drupal services:

#### NativeAIService

Primary service for AI operations:

```php
/**
 * Service for Native AI operations.
 */
class NativeAIService {
  /**
   * Generate text using an AI model.
   *
   * @param string $prompt
   *   The prompt to send to the model.
   * @param array $options
   *   Generation options.
   *
   * @return array
   *   The generation result.
   */
  public function generateText($prompt, array $options = []);

  /**
   * Generate embeddings for text.
   *
   * @param string|array $text
   *   The text to embed.
   * @param array $options
   *   Embedding options.
   *
   * @return array
   *   The embedding result.
   */
  public function generateEmbeddings($text, array $options = []);
}
```

#### ModelRegistry

Service for managing AI models:

```php
/**
 * Registry for AI models.
 */
class ModelRegistry {
  /**
   * Get all available models.
   *
   * @return array
   *   The available models.
   */
  public function getModels();

  /**
   * Get a specific model by ID.
   *
   * @param string $model_id
   *   The model ID.
   *
   * @return array|null
   *   The model data or NULL if not found.
   */
  public function getModel($model_id);

  /**
   * Refresh the model registry from the server.
   */
  public function refreshModels();
}
```

#### EntityProcessor

Service for processing entities with AI:

```php
/**
 * Service for processing entities with AI.
 */
class EntityProcessor {
  /**
   * Process an entity with AI.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity to process.
   * @param string $operation
   *   The operation to perform.
   * @param array $options
   *   Processing options.
   *
   * @return array
   *   The processing result.
   */
  public function processEntity(EntityInterface $entity, $operation, array $options = []);
}
```

### JavaScript API

The JavaScript bridge provides client-side integration:

```javascript
/**
 * Native AI.js bridge for Drupal.
 */
Drupal.NativeAI = {
  /**
   * Initialize the bridge.
   *
   * @param {Object} settings
   *   The initialization settings.
   */
  init: function(settings) {},

  /**
   * Generate text with AI.
   *
   * @param {string} prompt
   *   The prompt to send to the model.
   * @param {Object} options
   *   Generation options.
   *
   * @return {Promise<Object>}
   *   A promise that resolves to the generation result.
   */
  generate: function(prompt, options = {}) {},

  /**
   * Generate text with streaming response.
   *
   * @param {string} prompt
   *   The prompt to send to the model.
   * @param {Function} callback
   *   Callback function for each chunk of the response.
   * @param {Object} options
   *   Generation options.
   *
   * @return {Function}
   *   Unsubscribe function to stop streaming.
   */
  streamGenerate: function(prompt, callback, options = {}) {},

  /**
   * Generate embeddings for text.
   *
   * @param {string} text
   *   The text to embed.
   * @param {Object} options
   *   Embedding options.
   *
   * @return {Promise<Object>}
   *   A promise that resolves to the embedding result.
   */
  embed: function(text, options = {}) {},
};
```

## Use Cases and Examples

### 1. Content Enhancement

Automatically enhance content with AI:

```php
// ECA action to enhance content on save
$action = [
  'id' => 'enhance_body_on_save',
  'label' => 'Enhance body content on save',
  'events' => [
    'entity:save:node:article',
  ],
  'conditions' => [
    [
      'plugin' => 'entity_field_value',
      'configuration' => [
        'field' => 'body',
        'operator' => 'not_empty',
      ],
    ],
  ],
  'actions' => [
    [
      'plugin' => 'drupal_native_ai_enhance_text',
      'configuration' => [
        'entity_type' => 'node',
        'field' => 'body',
        'enhancement_type' => 'improve',
        'model' => 'llama-2-7b',
      ],
    ],
  ],
];
```

### 2. Semantic Search

Implement semantic search using embeddings:

```php
// Search content by semantic similarity
$query = "climate change solutions";
$results = \Drupal::service('drupal_native_ai.semantic_search')
  ->search($query, [
    'entity_types' => ['node'],
    'bundles' => ['article', 'page'],
    'fields' => ['title', 'body'],
    'limit' => 10,
    'model' => 'embedding-v3',
  ]);
```

### 3. Pattern Detection in User Behavior

Using BFLLM's Emergence Detection to analyze user behavior:

```php
// Detect patterns in user behavior
$userActivities = $userActivityRepository->getRecentActivities($user->id(), 100);

$patterns = \Drupal::service('drupal_native_ai.emergence_detector')
  ->detectPatterns($userActivities, [
    'pattern_types' => ['usage_pattern', 'interest_pattern'],
    'threshold' => 0.6,
  ]);

// Use patterns for personalization
if (!empty($patterns)) {
  $personalization = \Drupal::service('personalization.manager')
    ->personalizeContent($node, $patterns);
}
```

### 4. AI-Powered Forms

Enhance Drupal forms with AI capabilities:

```php
/**
 * Implements hook_form_alter().
 */
function mymodule_form_alter(&$form, FormStateInterface $form_state, $form_id) {
  if ($form_id == 'node_article_form') {
    // Add AI title generator
    $form['title_wrapper']['generate_title'] = [
      '#type' => 'button',
      '#value' => t('Generate Title'),
      '#attributes' => [
        'class' => ['ai-generate-title'],
        'data-target' => 'edit-title-0-value',
        'data-source' => 'edit-body-0-value',
      ],
      '#attached' => [
        'library' => ['drupal_native_ai/form_enhancement'],
      ],
    ];
  }
}
```

## Security Considerations

### Authentication and Authorization

The integration implements secure authentication:

1. **API Key Management**: Secure storage of API keys
2. **Permission Control**: Granular permissions for AI features
3. **Role-Based Access**: Limiting AI capabilities by user role

### Data Privacy

Ensuring data privacy in AI operations:

1. **Local Models**: Option to use local models for sensitive data
2. **Data Retention**: Controls for how long data is stored
3. **Anonymization**: Options to anonymize data before processing

### Content Moderation

AI-powered content requires moderation:

1. **Content Filtering**: Filtering inappropriate AI-generated content
2. **Human Review**: Options for human review of AI enhancements
3. **Revision Control**: Tracking changes made by AI

## Performance Optimization

### Caching

Optimize performance with caching:

1. **Response Caching**: Caching AI responses for similar inputs
2. **Embedding Caching**: Storing embeddings for efficient retrieval
3. **Model Caching**: Caching model parameters for faster inference

### Batch Processing

Efficient processing of large content sets:

1. **Drupal Batch API**: Using Drupal's batch system for processing
2. **Queue Workers**: Background processing with queue workers
3. **Progressive Operations**: Showing progress for long-running operations

## Troubleshooting

### Common Issues

1. **Connection Failures**:
   - Check server URL configuration
   - Verify API key validity
   - Check server logs for errors

2. **Performance Issues**:
   - Adjust batch size for large operations
   - Enable caching for repetitive operations
   - Consider using local models for faster response

3. **Integration Errors**:
   - Verify compatible versions of all components
   - Check for proper installation of dependencies
   - Review configuration settings

### Logging and Monitoring

The integration provides comprehensive logging:

```php
/**
 * Log AI operations for monitoring.
 */
$logger = \Drupal::logger('drupal_native_ai');
$logger->notice('AI operation completed: @operation on @entity_type @entity_id', [
  '@operation' => $operation,
  '@entity_type' => $entity_type,
  '@entity_id' => $entity_id,
]);
```

## Conclusion

The integration between Drupal and the LLM ecosystem (Native AI.js and BFLLM (DEPRECATED - migrate to LangChain) AI capabilities while maintaining Drupal's core strengths in content management.

This integration represents a significant advancement in bringing state-of-the-art AI capabilities to Drupal sites, enabling content enhancement, pattern detection, semantic search, and much more.

## References

- [Native AI.js Documentation](native_aijs/API_INTERFACES)
- [BFLLM Documentation](bfllm/docs/index)
- [Drupal Native AI Module](bfapi/drupal_native_ai/README)
- [Field ECA Bridge Module](https://www.drupal.org/project/field_eca_bridge)
- [LLM Platform Recipe](llm_platform_recipe/README)
- [Emergence Detection System](bfllm/docs/EMERGENCE_DETECTION)
