---
id: drupal-agent-ui-guide
---
# Agent UI Guide

<!-- title: Agent UI Guide -->
<!-- description: Guide for implementing and using the Agent UI in Drupal Native AI -->
<!-- lastUpdated: 2024-03-19 -->

## Overview

This guide details the implementation and usage of the Agent UI in the Drupal Native AI module. The UI provides a comprehensive interface for managing and executing AI agents within Drupal's administrative interface.

## Architecture

### Core Components

1. **Entity Types**
   - Agent config entity type
   - Agent interface
   - Entity conversion
   - Schema definition

2. **UI Components**
   - List builder
   - Forms
   - Controllers
   - Templates
   - Assets

3. **Integration Points**
   - Agent registry
   - Provider system
   - LLM module
   - Theme system

## Implementation Guide

### Entity Implementation

```php
// src/Entity/Agent.php
namespace Drupal\drupal_native_ai\Entity;

use Drupal\Core\Config\Entity\ConfigEntityBase;

/**
 * @ConfigEntityType(
 *   id = "drupal_native_ai_agent",
 *   label = @Translation("AI Agent"),
 *   handlers = {
 *     "list_builder" = "Drupal\drupal_native_ai\AgentListBuilder",
 *     "form" = {
 *       "add" = "Drupal\drupal_native_ai\Form\AgentForm",
 *       "edit" = "Drupal\drupal_native_ai\Form\AgentForm",
 *       "delete" = "Drupal\drupal_native_ai\Form\AgentDeleteForm"
 *     }
 *   }
 * )
 */
class Agent extends ConfigEntityBase implements AgentInterface {
  // Entity implementation
}
```

### List Builder

```php
// src/AgentListBuilder.php
namespace Drupal\drupal_native_ai;

use Drupal\Core\Config\Entity\ConfigEntityListBuilder;

class AgentListBuilder extends ConfigEntityListBuilder {
  public function buildHeader() {
    // Build header
  }
  
  public function buildRow(EntityInterface $entity) {
    // Build row
  }
  
  public function getDefaultOperations(EntityInterface $entity) {
    // Get operations
  }
}
```

### Forms

```php
// src/Form/AgentForm.php
namespace Drupal\drupal_native_ai\Form;

use Drupal\Core\Form\FormBase;

class AgentForm extends FormBase {
  public function buildForm(array $form, FormStateInterface $form_state) {
    // Build form
  }
  
  public function submitForm(array &$form, FormStateInterface $form_state) {
    // Submit form
  }
}
```

### Controllers

```php
// src/Controller/AgentController.php
namespace Drupal\drupal_native_ai\Controller;

use Drupal\Core\Controller\ControllerBase;

class AgentController extends ControllerBase {
  public function dashboard() {
    // Dashboard
  }
  
  public function execute($agent) {
    // Execute agent
  }
}
```

### Templates

```twig
{# templates/agent-dashboard.html.twig #}
<div class="agent-dashboard">
  <h2>{{ 'Agent Dashboard'|t }}</h2>
  
  <div class="agent-list">
    {% for agent in agents %}
      <div class="agent-item">
        {{ agent.name }}
      </div>
    {% endfor %}
  </div>
</div>
```

### JavaScript

```javascript
// js/agent-ui.js
(function ($, Drupal) {
  'use strict';
  
  Drupal.behaviors.agentUI = {
    attach: function (context, settings) {
      // Initialize UI
    }
  };
  
  Drupal.agentUI = {
    execute: function (agentId, input) {
      // Execute agent
    },
    
    stream: function (agentId, input, callback) {
      // Stream response
    }
  };
})(jQuery, Drupal);
```

## Integration Guide

### Agent Registry

1. **Entity to Contract**
   ```php
   public function toContract(): ContractAgent {
     return new ContractAgent(
       $this->id(),
       $this->getProvider(),
       $this->getModel()
     );
   }
   ```

2. **Registry Sync**
   ```php
   public function syncWithRegistry(): void {
     $registry = \Drupal::service('native_ai.agent_registry');
     $registry->register($this->toContract());
   }
   ```

### Provider Integration

1. **Model Selection**
   ```php
   public function getAvailableModels(): array {
     $provider = $this->getProvider();
     return $provider->getModels();
   }
   ```

2. **Execution**
   ```php
   public function execute(string $input): array {
     $provider = $this->getProvider();
     return $provider->process($input);
   }
   ```

### Theme Integration

1. **Theme Hooks**
   ```php
   function drupal_native_ai_theme() {
     return [
       'agent_dashboard' => [
         'variables' => ['agents' => []],
       ],
     ];
   }
   ```

2. **Preprocess**
   ```php
   function template_preprocess_agent_dashboard(&$variables) {
     // Preprocess variables
   }
   ```

## Development Workflow

### Testing

1. **Unit Tests**
   ```php
   namespace Drupal\Tests\drupal_native_ai\Unit;
   
   class AgentTest extends UnitTestCase {
     public function testAgentCreation() {
       // Test creation
     }
   }
   ```

2. **Functional Tests**
   ```php
   namespace Drupal\Tests\drupal_native_ai\Functional;
   
   class AgentUITest extends BrowserTestBase {
     public function testAgentDashboard() {
       // Test dashboard
     }
   }
   ```

### Field Integration

1. **Formatter**
   ```php
   class AgentFormatter extends FormatterBase {
     public function viewElements(FieldItemListInterface $items) {
       // Format field
     }
   }
   ```

2. **Widget**
   ```php
   class AgentWidget extends WidgetBase {
     public function formElement(FieldItemListInterface $items) {
       // Build widget
     }
   }
   ```

## Best Practices

1. **Entity Usage**
   - Use config entities for agent configuration
   - Implement proper interfaces
   - Follow entity API patterns

2. **Form Integration**
   - Use Form API consistently
   - Implement AJAX where appropriate
   - Validate input properly

3. **Theme System**
   - Use templates for presentation
   - Implement preprocess functions
   - Follow Drupal theming patterns

4. **Service Architecture**
   - Use dependency injection
   - Follow service patterns
   - Maintain clean architecture

5. **Testing**
   - Write unit tests
   - Implement functional tests
   - Test JavaScript components

## Related Documentation

- [Development Guide](../development/README)
- [Testing Guide](../testing/README)
- [Security Guide](../operations/security)
- [Monitoring Guide](../operations/monitoring)
- [Native AI Integration](../integration/native-ai-integration) 
