---
id: drupal-modules
---
# Module Development

<!-- title: Module Development -->
<!-- description: Developer guide for creating Drupal modules for the LLM Platform Platform -->
<!-- lastUpdated: 2024-03-19 -->

This guide covers the development of custom Drupal modules for the LLM Platform Platform, including module structure, plugin system, entity types, and best practices.

## Module Structure

### Basic Structure

```
modules/custom/bluefly_custom/
├── bluefly_custom.info.yml        # Module info
├── bluefly_custom.module         # Module file
├── bluefly_custom.install        # Install/update hooks
├── bluefly_custom.services.yml   # Service definitions
├── bluefly_custom.routing.yml    # Route definitions
├── bluefly_custom.permissions.yml # Permission definitions
├── src/
│   ├── Plugin/                   # Plugin definitions
│   │   ├── Block/               # Block plugins
│   │   ├── Field/               # Field plugins
│   │   ├── Action/              # Action plugins
│   │   └── Agent/               # Agent plugins
│   ├── Entity/                  # Entity types
│   ├── Form/                    # Forms
│   ├── Controller/              # Controllers
│   └── Service/                 # Services
├── config/
│   ├── install/                 # Default config
│   └── schema/                  # Config schema
└── tests/
    ├── Kernel/                  # Kernel tests
    └── Functional/              # Functional tests
```

### Module Info

```yaml
# bluefly_custom.info.yml
name: LLM Platform Custom
type: module
description: 'Custom LLM Platform Platform integration features'
package: LLM Platform
core_version_requirement: ^9 || ^10
dependencies:
  - drupal:bluefly
  - drupal:jsonapi
  - drupal:rest
```

## Plugin System

### Block Plugin

```php
// src/Plugin/Block/AgentBlock.php
namespace Drupal\bluefly_custom\Plugin\Block;

use Drupal\Core\Block\BlockBase;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\bluefly\Service\LLM PlatformServiceInterface;

/**
 * @Block(
 *   id = "bluefly_agent",
 *   admin_label = @Translation("LLM Platform Agent"),
 *   category = @Translation("LLM Platform")
 * )
 */
class AgentBlock extends BlockBase implements ContainerFactoryPluginInterface {
  private LLM PlatformServiceInterface $blueflyService;
  
  public function __construct(
    array $configuration,
    string $plugin_id,
    array $plugin_definition,
    LLM PlatformServiceInterface $blueflyService
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->blueflyService = $blueflyService;
  }
  
  public static function create(
    ContainerInterface $container,
    array $configuration,
    string $plugin_id,
    array $plugin_definition
  ) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('bluefly.service')
    );
  }
  
  public function build(): array {
    $agents = $this->blueflyService->getAvailableAgents();
    
    return [
      '#theme' => 'bluefly_agent_block',
      '#agents' => $agents,
      '#cache' => [
        'max-age' => 0,
        'contexts' => ['user.permissions']
      ]
    ];
  }
}
```

### Field Plugin

```php
// src/Plugin/Field/FieldType/AgentField.php
namespace Drupal\bluefly_custom\Plugin\Field\FieldType;

use Drupal\Core\Field\FieldItemBase;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\Core\TypedData\DataDefinition;

/**
 * @FieldType(
 *   id = "bluefly_agent",
 *   label = @Translation("LLM Platform Agent"),
 *   description = @Translation("Field for storing agent configuration"),
 *   default_widget = "bluefly_agent_widget",
 *   default_formatter = "bluefly_agent_formatter"
 * )
 */
class AgentField extends FieldItemBase {
  public static function propertyDefinitions(FieldStorageDefinitionInterface $field_definition): array {
    $properties['agent_id'] = DataDefinition::create('string')
      ->setLabel(t('Agent ID'))
      ->setRequired(TRUE);
    
    $properties['configuration'] = DataDefinition::create('string')
      ->setLabel(t('Configuration'))
      ->setRequired(FALSE);
    
    return $properties;
  }
  
  public static function schema(FieldStorageDefinitionInterface $field_definition): array {
    return [
      'columns' => [
        'agent_id' => [
          'type' => 'varchar',
          'length' => 255,
          'not null' => TRUE
        ],
        'configuration' => [
          'type' => 'text',
          'size' => 'big',
          'not null' => FALSE
        ]
      ],
      'indexes' => [
        'agent_id' => ['agent_id']
      ]
    ];
  }
}
```

### Action Plugin

```php
// src/Plugin/Action/AgentAction.php
namespace Drupal\bluefly_custom\Plugin\Action;

use Drupal\Core\Action\ActionBase;
use Drupal\Core\Session\AccountInterface;
use Drupal\bluefly\Service\LLM PlatformServiceInterface;

/**
 * @Action(
 *   id = "bluefly_agent_action",
 *   label = @Translation("Execute LLM Platform Agent"),
 *   type = "node",
 *   confirm = TRUE
 * )
 */
class AgentAction extends ActionBase {
  private LLM PlatformServiceInterface $blueflyService;
  
  public function __construct(
    array $configuration,
    string $plugin_id,
    array $plugin_definition,
    LLM PlatformServiceInterface $blueflyService
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->blueflyService = $blueflyService;
  }
  
  public function execute($entity = NULL): void {
    if (!$entity) {
      return;
    }
    
    try {
      $this->blueflyService->executeTask('content', [
        'action' => 'process',
        'entity' => [
          'type' => $entity->getEntityTypeId(),
          'id' => $entity->id(),
          'data' => $entity->toArray()
        ]
      ]);
    } catch (\Exception $e) {
      \Drupal::logger('bluefly_custom')->error('Error executing agent action: @error', [
        '@error' => $e->getMessage()
      ]);
    }
  }
  
  public function access($object, AccountInterface $account = NULL, $return_as_object = FALSE) {
    return $account->hasPermission('execute bluefly tasks');
  }
}
```

## Entity Types

### Custom Entity

```php
// src/Entity/AgentTask.php
namespace Drupal\bluefly_custom\Entity;

use Drupal\Core\Entity\ContentEntityBase;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Field\BaseFieldDefinition;

/**
 * @ContentEntityType(
 *   id = "bluefly_agent_task",
 *   label = @Translation("Agent Task"),
 *   handlers = {
 *     "view_builder" = "Drupal\Core\Entity\EntityViewBuilder",
 *     "list_builder" = "Drupal\bluefly_custom\AgentTaskListBuilder",
 *     "form" = {
 *       "default" = "Drupal\bluefly_custom\Form\AgentTaskForm",
 *       "add" = "Drupal\bluefly_custom\Form\AgentTaskForm",
 *       "edit" = "Drupal\bluefly_custom\Form\AgentTaskForm",
 *       "delete" = "Drupal\Core\Entity\ContentEntityDeleteForm"
 *     },
 *     "route_provider" = {
 *       "html" = "Drupal\Core\Entity\Routing\AdminHtmlRouteProvider"
 *     }
 *   },
 *   base_table = "bluefly_agent_task",
 *   admin_permission = "administer bluefly tasks",
 *   entity_keys = {
 *     "id" = "id",
 *     "label" = "title",
 *     "uuid" = "uuid"
 *   },
 *   links = {
 *     "canonical" = "/admin/bluefly/task/{bluefly_agent_task}",
 *     "add-form" = "/admin/bluefly/task/add",
 *     "edit-form" = "/admin/bluefly/task/{bluefly_agent_task}/edit",
 *     "delete-form" = "/admin/bluefly/task/{bluefly_agent_task}/delete",
 *     "collection" = "/admin/bluefly/tasks"
 *   }
 * )
 */
class AgentTask extends ContentEntityBase {
  public static function baseFieldDefinitions(EntityTypeInterface $entity_type): array {
    $fields = parent::baseFieldDefinitions($entity_type);
    
    $fields['title'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Title'))
      ->setRequired(TRUE)
      ->setDisplayOptions('form', [
        'type' => 'string_textfield',
        'weight' => 0
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);
    
    $fields['agent'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Agent'))
      ->setRequired(TRUE)
      ->setDisplayOptions('form', [
        'type' => 'string_textfield',
        'weight' => 1
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);
    
    $fields['status'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Status'))
      ->setDefaultValue('pending')
      ->setDisplayOptions('form', [
        'type' => 'options_select',
        'weight' => 2,
        'settings' => [
          'options' => [
            'pending' => t('Pending'),
            'running' => t('Running'),
            'completed' => t('Completed'),
            'failed' => t('Failed')
          ]
        ]
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);
    
    return $fields;
  }
}
```

## Forms

### Task Form

```php
// src/Form/AgentTaskForm.php
namespace Drupal\bluefly_custom\Form;

use Drupal\Core\Entity\EntityForm;
use Drupal\Core\Form\FormStateInterface;
use Drupal\bluefly\Service\LLM PlatformServiceInterface;

class AgentTaskForm extends EntityForm {
  private LLM PlatformServiceInterface $blueflyService;
  
  public function __construct(LLM PlatformServiceInterface $blueflyService) {
    $this->blueflyService = $blueflyService;
  }
  
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('bluefly.service')
    );
  }
  
  public function form(array $form, FormStateInterface $form_state): array {
    $form = parent::form($form, $form_state);
    $task = $this->entity;
    
    $form['title'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Title'),
      '#required' => TRUE,
      '#default_value' => $task->get('title')->value
    ];
    
    $form['agent'] = [
      '#type' => 'select',
      '#title' => $this->t('Agent'),
      '#required' => TRUE,
      '#options' => $this->getAgentOptions(),
      '#default_value' => $task->get('agent')->value
    ];
    
    $form['configuration'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Configuration'),
      '#default_value' => $task->get('configuration')->value,
      '#description' => $this->t('JSON configuration for the agent')
    ];
    
    return $form;
  }
  
  public function validateForm(array &$form, FormStateInterface $form_state): void {
    parent::validateForm($form, $form_state);
    
    $configuration = $form_state->getValue('configuration');
    if (!empty($configuration)) {
      json_decode($configuration);
      if (json_last_error() !== JSON_ERROR_NONE) {
        $form_state->setErrorByName('configuration', $this->t('Invalid JSON configuration'));
      }
    }
  }
  
  public function save(array $form, FormStateInterface $form_state): int {
    $task = $this->entity;
    $status = $task->save();
    
    if ($status === SAVED_NEW) {
      $this->messenger()->addStatus($this->t('Created task %label.', [
        '%label' => $task->label()
      ]));
    } else {
      $this->messenger()->addStatus($this->t('Updated task %label.', [
        '%label' => $task->label()
      ]));
    }
    
    $form_state->setRedirect('entity.bluefly_agent_task.collection');
    return $status;
  }
  
  private function getAgentOptions(): array {
    $agents = $this->blueflyService->getAvailableAgents();
    $options = [];
    
    foreach ($agents as $agent) {
      $options[$agent->getName()] = $agent->getLabel();
    }
    
    return $options;
  }
}
```

## Services

### Task Service

```php
// src/Service/TaskService.php
namespace Drupal\bluefly_custom\Service;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\bluefly\Service\LLM PlatformServiceInterface;
use Drupal\bluefly_custom\Entity\AgentTask;

class TaskService {
  private EntityTypeManagerInterface $entityTypeManager;
  private LLM PlatformServiceInterface $blueflyService;
  
  public function __construct(
    EntityTypeManagerInterface $entityTypeManager,
    LLM PlatformServiceInterface $blueflyService
  ) {
    $this->entityTypeManager = $entityTypeManager;
    $this->blueflyService = $blueflyService;
  }
  
  public function createTask(array $data): AgentTask {
    $task = $this->entityTypeManager
      ->getStorage('bluefly_agent_task')
      ->create($data);
    
    $task->save();
    return $task;
  }
  
  public function executeTask(AgentTask $task): void {
    try {
      $task->set('status', 'running')->save();
      
      $result = $this->blueflyService->executeTask(
        $task->get('agent')->value,
        json_decode($task->get('configuration')->value, TRUE)
      );
      
      $task->set('status', 'completed')
        ->set('result', json_encode($result))
        ->save();
    } catch (\Exception $e) {
      $task->set('status', 'failed')
        ->set('error', $e->getMessage())
        ->save();
      
      throw $e;
    }
  }
  
  public function getTaskStatus(AgentTask $task): array {
    return [
      'id' => $task->id(),
      'status' => $task->get('status')->value,
      'result' => $task->get('result')->value,
      'error' => $task->get('error')->value
    ];
  }
}
```

## Testing

### Kernel Tests

```php
// tests/Kernel/TaskServiceTest.php
namespace Drupal\Tests\bluefly_custom\Kernel;

use Drupal\KernelTests\KernelTestBase;
use Drupal\bluefly_custom\Service\TaskService;
use Drupal\bluefly_custom\Entity\AgentTask;

class TaskServiceTest extends KernelTestBase {
  protected static $modules = [
    'bluefly_custom',
    'bluefly',
    'system',
    'user'
  ];
  
  private TaskService $service;
  
  protected function setUp(): void {
    parent::setUp();
    
    $this->installEntitySchema('bluefly_agent_task');
    $this->service = $this->container->get('bluefly_custom.task_service');
  }
  
  public function testCreateTask(): void {
    $task = $this->service->createTask([
      'title' => 'Test Task',
      'agent' => 'content',
      'configuration' => json_encode([
        'action' => 'generate',
        'type' => 'article'
      ])
    ]);
    
    $this->assertInstanceOf(AgentTask::class, $task);
    $this->assertEquals('Test Task', $task->get('title')->value);
    $this->assertEquals('content', $task->get('agent')->value);
  }
}
```

### Functional Tests

```php
// tests/Functional/TaskFormTest.php
namespace Drupal\Tests\bluefly_custom\Functional;

use Drupal\Tests\BrowserTestBase;

class TaskFormTest extends BrowserTestBase {
  protected static $modules = ['bluefly_custom'];
  
  protected function setUp(): void {
    parent::setUp();
    
    $this->drupalCreateUser(['administer bluefly tasks']);
  }
  
  public function testTaskForm(): void {
    $this->drupalLogin($this->createUser(['administer bluefly tasks']));
    
    $this->drupalGet('/admin/bluefly/task/add');
    $this->assertSession()->statusCodeEquals(200);
    
    $edit = [
      'title' => 'Test Task',
      'agent' => 'content',
      'configuration' => json_encode([
        'action' => 'generate',
        'type' => 'article'
      ])
    ];
    
    $this->submitForm($edit, 'Save');
    $this->assertSession()->pageTextContains('Created task Test Task');
  }
}
```

## Best Practices

1. **Module Organization**
   - Follow Drupal coding standards
   - Use proper namespacing
   - Implement dependency injection
   - Document code thoroughly
   - Include tests

2. **Plugin Development**
   - Use plugin annotations
   - Implement interfaces
   - Handle dependencies
   - Validate inputs
   - Cache when appropriate

3. **Entity Development**
   - Define proper fields
   - Implement access control
   - Use entity hooks
   - Handle revisions
   - Include translations

4. **Form Development**
   - Validate inputs
   - Handle errors
   - Use form states
   - Implement AJAX
   - Cache form data

## Related Documentation

- [Drupal Integration](./integration)
- [API Development](../development/api)
- [Testing Guide](../development/testing)
- [Security Guide](../operations/security) 
