---
id: error-recovery
---
# Error Handling and Recovery Guide

<!-- title: Error Handling and Recovery Guide -->
<!-- description: Practical guide for error handling and recovery in the LLM Platform Platform -->
<!-- lastUpdated: 2024-03-19 -->

## Overview

This guide covers practical error handling and recovery for the LLM Platform Platform, including error classification, handling strategies, recovery procedures, and monitoring.

## Error Classification

### Error Types

```typescript
// src/error/types.ts
export class BaseError extends Error {
  constructor(
    message: string,
    public code: string,
    public status: number,
    public details?: any
  ) {
    super(message);
    this.name = this.constructor.name;
    Error.captureStackTrace(this, this.constructor);
  }
}

export class ValidationError extends BaseError {
  constructor(message: string, details?: any) {
    super(message, 'VALIDATION_ERROR', 400, details);
  }
}

export class AuthenticationError extends BaseError {
  constructor(message: string, details?: any) {
    super(message, 'AUTHENTICATION_ERROR', 401, details);
  }
}

export class AuthorizationError extends BaseError {
  constructor(message: string, details?: any) {
    super(message, 'AUTHORIZATION_ERROR', 403, details);
  }
}

export class NotFoundError extends BaseError {
  constructor(message: string, details?: any) {
    super(message, 'NOT_FOUND_ERROR', 404, details);
  }
}

export class ConflictError extends BaseError {
  constructor(message: string, details?: any) {
    super(message, 'CONFLICT_ERROR', 409, details);
  }
}

export class RateLimitError extends BaseError {
  constructor(message: string, details?: any) {
    super(message, 'RATE_LIMIT_ERROR', 429, details);
  }
}

export class InternalError extends BaseError {
  constructor(message: string, details?: any) {
    super(message, 'INTERNAL_ERROR', 500, details);
  }
}

export class ServiceUnavailableError extends BaseError {
  constructor(message: string, details?: any) {
    super(message, 'SERVICE_UNAVAILABLE_ERROR', 503, details);
  }
}

export class DatabaseError extends BaseError {
  constructor(message: string, details?: any) {
    super(message, 'DATABASE_ERROR', 500, details);
  }
}

export class ExternalServiceError extends BaseError {
  constructor(
    message: string,
    public service: string,
    details?: any
  ) {
    super(message, 'EXTERNAL_SERVICE_ERROR', 502, {
      service,
      ...details
    });
  }
}

export class TimeoutError extends BaseError {
  constructor(message: string, details?: any) {
    super(message, 'TIMEOUT_ERROR', 504, details);
  }
}
```

### Error Factory

```typescript
// src/error/factory.ts
import {
  BaseError,
  ValidationError,
  AuthenticationError,
  AuthorizationError,
  NotFoundError,
  ConflictError,
  RateLimitError,
  InternalError,
  ServiceUnavailableError,
  DatabaseError,
  ExternalServiceError,
  TimeoutError
} from './types';

export class ErrorFactory {
  static create(
    type: ErrorType,
    message: string,
    details?: any
  ): BaseError {
    switch (type) {
      case 'VALIDATION_ERROR':
        return new ValidationError(message, details);
      
      case 'AUTHENTICATION_ERROR':
        return new AuthenticationError(message, details);
      
      case 'AUTHORIZATION_ERROR':
        return new AuthorizationError(message, details);
      
      case 'NOT_FOUND_ERROR':
        return new NotFoundError(message, details);
      
      case 'CONFLICT_ERROR':
        return new ConflictError(message, details);
      
      case 'RATE_LIMIT_ERROR':
        return new RateLimitError(message, details);
      
      case 'INTERNAL_ERROR':
        return new InternalError(message, details);
      
      case 'SERVICE_UNAVAILABLE_ERROR':
        return new ServiceUnavailableError(message, details);
      
      case 'DATABASE_ERROR':
        return new DatabaseError(message, details);
      
      case 'EXTERNAL_SERVICE_ERROR':
        return new ExternalServiceError(
          message,
          details.service,
          details
        );
      
      case 'TIMEOUT_ERROR':
        return new TimeoutError(message, details);
      
      default:
        return new InternalError(
          'Unknown error type',
          { type, message, details }
        );
    }
  }
  
  static fromException(
    error: Error,
    context?: any
  ): BaseError {
    // Handle known error types
    if (error instanceof BaseError) {
      return error;
    }
    
    // Handle database errors
    if (error.name === 'SequelizeError') {
      return new DatabaseError(error.message, {
        original: error,
        context
      });
    }
    
    // Handle HTTP errors
    if (error.name === 'AxiosError') {
      const axiosError = error as any;
      const status = axiosError.response?.status;
      
      switch (status) {
        case 400:
          return new ValidationError(
            axiosError.response?.data?.message ?? error.message,
            { original: error, context }
          );
        
        case 401:
          return new AuthenticationError(
            axiosError.response?.data?.message ?? error.message,
            { original: error, context }
          );
        
        case 403:
          return new AuthorizationError(
            axiosError.response?.data?.message ?? error.message,
            { original: error, context }
          );
        
        case 404:
          return new NotFoundError(
            axiosError.response?.data?.message ?? error.message,
            { original: error, context }
          );
        
        case 409:
          return new ConflictError(
            axiosError.response?.data?.message ?? error.message,
            { original: error, context }
          );
        
        case 429:
          return new RateLimitError(
            axiosError.response?.data?.message ?? error.message,
            { original: error, context }
          );
        
        case 503:
          return new ServiceUnavailableError(
            axiosError.response?.data?.message ?? error.message,
            { original: error, context }
          );
        
        case 504:
          return new TimeoutError(
            axiosError.response?.data?.message ?? error.message,
            { original: error, context }
          );
        
        default:
          return new ExternalServiceError(
            axiosError.response?.data?.message ?? error.message,
            axiosError.config?.url,
            { original: error, context }
          );
      }
    }
    
    // Handle timeout errors
    if (error.name === 'TimeoutError') {
      return new TimeoutError(error.message, {
        original: error,
        context
      });
    }
    
    // Handle unknown errors
    return new InternalError(error.message, {
      original: error,
      context
    });
  }
}

type ErrorType =
  | 'VALIDATION_ERROR'
  | 'AUTHENTICATION_ERROR'
  | 'AUTHORIZATION_ERROR'
  | 'NOT_FOUND_ERROR'
  | 'CONFLICT_ERROR'
  | 'RATE_LIMIT_ERROR'
  | 'INTERNAL_ERROR'
  | 'SERVICE_UNAVAILABLE_ERROR'
  | 'DATABASE_ERROR'
  | 'EXTERNAL_SERVICE_ERROR'
  | 'TIMEOUT_ERROR';
```

## Error Handling

### Error Handler

```typescript
// src/error/handler.ts
import { Request, Response, NextFunction } from 'express';
import { BaseError, ErrorFactory } from './factory';
import { ErrorLogger } from './logger';
import { ErrorMetrics } from './metrics';

export class ErrorHandler {
  constructor(
    private logger: ErrorLogger,
    private metrics: ErrorMetrics
  ) {}
  
  handle(
    error: Error,
    req: Request,
    res: Response,
    next: NextFunction
  ): void {
    // Convert to BaseError
    const baseError = ErrorFactory.fromException(error, {
      request: {
        method: req.method,
        url: req.url,
        headers: req.headers,
        body: req.body,
        params: req.params,
        query: req.query
      }
    });
    
    // Log error
    this.logger.error(baseError, {
      request: {
        method: req.method,
        url: req.url,
        headers: req.headers,
        body: req.body,
        params: req.params,
        query: req.query
      }
    });
    
    // Record metrics
    this.metrics.recordError(baseError);
    
    // Send response
    res.status(baseError.status).json({
      error: {
        code: baseError.code,
        message: baseError.message,
        details: baseError.details
      }
    });
  }
  
  handleUncaughtException(
    error: Error
  ): void {
    // Convert to BaseError
    const baseError = ErrorFactory.fromException(error, {
      type: 'uncaughtException'
    });
    
    // Log error
    this.logger.error(baseError, {
      type: 'uncaughtException'
    });
    
    // Record metrics
    this.metrics.recordError(baseError);
    
    // Exit process
    process.exit(1);
  }
  
  handleUnhandledRejection(
    reason: any
  ): void {
    // Convert to BaseError
    const baseError = ErrorFactory.fromException(
      reason instanceof Error ? reason : new Error(String(reason)),
      {
        type: 'unhandledRejection'
      }
    );
    
    // Log error
    this.logger.error(baseError, {
      type: 'unhandledRejection'
    });
    
    // Record metrics
    this.metrics.recordError(baseError);
  }
}
```

### Error Logger

```typescript
// src/error/logger.ts
import { BaseError } from './types';
import { ElasticsearchClient } from '@elastic/elasticsearch';

export class ErrorLogger {
  constructor(
    private elasticsearch: ElasticsearchClient,
    private options: LoggerOptions
  ) {}
  
  async error(
    error: BaseError,
    context?: any
  ): Promise<void> {
    const document = {
      timestamp: new Date(),
      error: {
        name: error.name,
        code: error.code,
        message: error.message,
        stack: error.stack,
        details: error.details
      },
      context,
      environment: this.options.environment,
      service: this.options.service,
      version: this.options.version
    };
    
    try {
      await this.elasticsearch.index({
        index: this.getIndexName(),
        document
      });
    } catch (err) {
      console.error('Failed to log error:', err);
    }
  }
  
  async search(
    query: ErrorSearchQuery
  ): Promise<ErrorLog[]> {
    const { from = 0, size = 10, ...filters } = query;
    
    const response = await this.elasticsearch.search({
      index: this.getIndexName(),
      from,
      size,
      query: {
        bool: {
          must: Object.entries(filters).map(
            ([field, value]) => ({
              match: { [field]: value }
            })
          )
        }
      },
      sort: [
        { timestamp: { order: 'desc' } }
      ]
    });
    
    return response.hits.hits.map((hit) => hit._source as ErrorLog);
  }
  
  private getIndexName(): string {
    const date = new Date();
    const year = date.getFullYear();
    const month = String(date.getMonth() + 1).padStart(2, '0');
    
    return `errors-${this.options.environment}-${year}.${month}`;
  }
}

interface LoggerOptions {
  environment: string;
  service: string;
  version: string;
}

interface ErrorSearchQuery {
  from?: number;
  size?: number;
  'error.code'?: string;
  'error.name'?: string;
  'environment'?: string;
  'service'?: string;
  'version'?: string;
  'timestamp'?: {
    gte?: string;
    lte?: string;
  };
}

interface ErrorLog {
  timestamp: Date;
  error: {
    name: string;
    code: string;
    message: string;
    stack?: string;
    details?: any;
  };
  context?: any;
  environment: string;
  service: string;
  version: string;
}
```

### Error Metrics

```typescript
// src/error/metrics.ts
import { BaseError } from './types';
import { PrometheusClient } from 'prom-client';

export class ErrorMetrics {
  private errorCounter: PrometheusClient.Counter;
  private errorLatency: PrometheusClient.Histogram;
  
  constructor(
    private prometheus: PrometheusClient,
    private options: MetricsOptions
  ) {
    // Initialize counters
    this.errorCounter = new prometheus.Counter({
      name: 'error_total',
      help: 'Total number of errors',
      labelNames: [
        'environment',
        'service',
        'version',
        'code',
        'name'
      ]
    });
    
    this.errorLatency = new prometheus.Histogram({
      name: 'error_latency_seconds',
      help: 'Error latency in seconds',
      labelNames: [
        'environment',
        'service',
        'version',
        'code',
        'name'
      ],
      buckets: [0.1, 0.5, 1, 2, 5]
    });
  }
  
  recordError(
    error: BaseError,
    context?: any
  ): void {
    const labels = {
      environment: this.options.environment,
      service: this.options.service,
      version: this.options.version,
      code: error.code,
      name: error.name
    };
    
    // Increment counter
    this.errorCounter.inc(labels);
    
    // Record latency if available
    if (context?.latency) {
      this.errorLatency.observe(labels, context.latency);
    }
  }
  
  async getMetrics(): Promise<ErrorMetrics> {
    const [counter, latency] = await Promise.all([
      this.prometheus.register.getMetricsAsJSON(),
      this.errorLatency.get()
    ]);
    
    return {
      total: counter.find(
        (metric) => metric.name === 'error_total'
      ),
      latency: latency
    };
  }
}

interface MetricsOptions {
  environment: string;
  service: string;
  version: string;
}

interface ErrorMetrics {
  total: PrometheusClient.Metric;
  latency: PrometheusClient.Histogram;
}
```

## Recovery Procedures

### Retry Strategy

```typescript
// src/error/retry.ts
import { BaseError } from './types';
import { ErrorLogger } from './logger';
import { ErrorMetrics } from './metrics';

export class RetryStrategy {
  constructor(
    private logger: ErrorLogger,
    private metrics: ErrorMetrics,
    private options: RetryOptions
  ) {}
  
  async execute<T>(
    operation: () => Promise<T>,
    context?: any
  ): Promise<T> {
    let lastError: BaseError | undefined;
    let attempt = 0;
    
    while (attempt < this.options.maxAttempts) {
      try {
        const startTime = Date.now();
        const result = await operation();
        const latency = (Date.now() - startTime) / 1000;
        
        // Record success
        this.metrics.recordError(
          new BaseError('Success', 'SUCCESS', 200),
          { latency, attempt }
        );
        
        return result;
      } catch (error) {
        lastError = error as BaseError;
        attempt++;
        
        // Log attempt
        this.logger.error(lastError, {
          attempt,
          context
        });
        
        // Record failure
        this.metrics.recordError(lastError, {
          latency: 0,
          attempt
        });
        
        // Check if should retry
        if (!this.shouldRetry(lastError, attempt)) {
          break;
        }
        
        // Calculate delay
        const delay = this.calculateDelay(attempt);
        
        // Wait before retry
        await new Promise((resolve) =>
          setTimeout(resolve, delay)
        );
      }
    }
    
    throw lastError;
  }
  
  private shouldRetry(
    error: BaseError,
    attempt: number
  ): boolean {
    // Check max attempts
    if (attempt >= this.options.maxAttempts) {
      return false;
    }
    
    // Check retryable errors
    if (
      !this.options.retryableErrors.includes(
        error.code
      )
    ) {
      return false;
    }
    
    // Check custom retry condition
    if (
      this.options.shouldRetry &&
      !this.options.shouldRetry(error, attempt)
    ) {
      return false;
    }
    
    return true;
  }
  
  private calculateDelay(attempt: number): number {
    const { minDelay, maxDelay, factor } = this.options;
    
    // Exponential backoff with jitter
    const delay = Math.min(
      minDelay * Math.pow(factor, attempt - 1),
      maxDelay
    );
    
    const jitter = Math.random() * delay * 0.1;
    return delay + jitter;
  }
}

interface RetryOptions {
  maxAttempts: number;
  minDelay: number;
  maxDelay: number;
  factor: number;
  retryableErrors: string[];
  shouldRetry?: (
    error: BaseError,
    attempt: number
  ) => boolean;
}
```

### Fallback Strategy

```typescript
// src/error/fallback.ts
import { BaseError } from './types';
import { ErrorLogger } from './logger';
import { ErrorMetrics } from './metrics';

export class FallbackStrategy {
  constructor(
    private logger: ErrorLogger,
    private metrics: ErrorMetrics,
    private options: FallbackOptions
  ) {}
  
  async execute<T>(
    primary: () => Promise<T>,
    fallback: () => Promise<T>,
    context?: any
  ): Promise<T> {
    try {
      const startTime = Date.now();
      const result = await primary();
      const latency = (Date.now() - startTime) / 1000;
      
      // Record success
      this.metrics.recordError(
        new BaseError('Success', 'SUCCESS', 200),
        { latency, source: 'primary' }
      );
      
      return result;
    } catch (error) {
      const primaryError = error as BaseError;
      
      // Log primary failure
      this.logger.error(primaryError, {
        source: 'primary',
        context
      });
      
      // Record primary failure
      this.metrics.recordError(primaryError, {
        latency: 0,
        source: 'primary'
      });
      
      // Check if should fallback
      if (!this.shouldFallback(primaryError)) {
        throw primaryError;
      }
      
      try {
        const startTime = Date.now();
        const result = await fallback();
        const latency = (Date.now() - startTime) / 1000;
        
        // Record fallback success
        this.metrics.recordError(
          new BaseError('Success', 'SUCCESS', 200),
          { latency, source: 'fallback' }
        );
        
        return result;
      } catch (error) {
        const fallbackError = error as BaseError;
        
        // Log fallback failure
        this.logger.error(fallbackError, {
          source: 'fallback',
          context
        });
        
        // Record fallback failure
        this.metrics.recordError(fallbackError, {
          latency: 0,
          source: 'fallback'
        });
        
        throw fallbackError;
      }
    }
  }
  
  private shouldFallback(error: BaseError): boolean {
    // Check fallbackable errors
    if (
      !this.options.fallbackableErrors.includes(
        error.code
      )
    ) {
      return false;
    }
    
    // Check custom fallback condition
    if (
      this.options.shouldFallback &&
      !this.options.shouldFallback(error)
    ) {
      return false;
    }
    
    return true;
  }
}

interface FallbackOptions {
  fallbackableErrors: string[];
  shouldFallback?: (error: BaseError) => boolean;
}
```

## Best Practices

1. **Error Classification**
   - Use proper error types
   - Include error codes
   - Add error details
   - Handle all cases
   - Document errors

2. **Error Handling**
   - Log all errors
   - Record metrics
   - Handle uncaught
   - Use middleware
   - Format responses

3. **Recovery**
   - Implement retries
   - Use fallbacks
   - Set timeouts
   - Handle timeouts
   - Monitor recovery

4. **Monitoring**
   - Track errors
   - Alert on issues
   - Analyze patterns
   - Set thresholds
   - Review logs

## Related Documentation

- [Architecture Guide](../architecture/README)
- [Monitoring Guide](../monitoring/README)
- [Security Guide](../security/README)
- [Testing Guide](../testing/README)
- [Deployment Guide](../deployment/README) 
