---
id: global-deployment-gitlab-federation
---
# GitLab Federation Setup Guide

## Overview

This guide details the setup and configuration of federated GitLab instances for global deployment across 1000s of servers with multi-region coordination and ECA integration.

## Federation Architecture

### Hub-and-Spoke Model

```
Global GitLab Hub (Primary)
├── Regional GitLab Federation Nodes
│   ├── US Federation Node (us-east-1, us-west-2)
│   ├── EU Federation Node (eu-west-1, eu-central-1)
│   ├── APAC Federation Node (ap-southeast-1, ap-northeast-1)
│   ├── Canada Federation Node (ca-central-1)
│   ├── India Federation Node (ap-south-1)
│   ├── Brazil Federation Node (sa-east-1)
│   └── Middle East Federation Node (me-south-1)
└── Edge GitLab Instances (Per Data Center)
    ├── Compliance-Specific Instances
    ├── High-Security Instances
    └── Development/Staging Instances
```

## Core Federation Components

### 1. Global GitLab Hub Configuration

#### Primary GitLab Instance Setup
```ruby
# /etc/gitlab/gitlab.rb - Global Hub Configuration

# External URL and basic settings
external_url 'https://gitlab-global-hub.bluefly-ai.com'
gitlab_rails['gitlab_shell_ssh_port'] = 22

# Federation-specific configuration
gitlab_rails['federation_enabled'] = true
gitlab_rails['federation_role'] = 'hub'
gitlab_rails['federation_regions'] = [
  'us-east', 'us-west', 'eu-west', 'eu-central',
  'apac-southeast', 'apac-northeast', 'canada',
  'india', 'brazil', 'middle-east'
]

# High Availability Configuration
gitlab_rails['db_adapter'] = 'postgresql'
gitlab_rails['db_host'] = 'postgres-cluster.internal'
gitlab_rails['db_port'] = 5432
gitlab_rails['db_database'] = 'gitlab_global_hub'
gitlab_rails['db_username'] = 'gitlab'

# Redis Cluster Configuration
gitlab_rails['redis_host'] = 'redis-cluster.internal'
gitlab_rails['redis_port'] = 6379
gitlab_rails['redis_sentinels'] = [
  {'host' => 'redis-sentinel-1.internal', 'port' => 26379},
  {'host' => 'redis-sentinel-2.internal', 'port' => 26379},
  {'host' => 'redis-sentinel-3.internal', 'port' => 26379}
]

# Federation API Configuration
gitlab_rails['federation_api_enabled'] = true
gitlab_rails['federation_api_port'] = 8080
gitlab_rails['federation_api_authentication'] = 'jwt'
gitlab_rails['federation_api_rate_limit'] = 1000  # requests per minute

# Cross-Region Synchronization
gitlab_rails['federation_sync_enabled'] = true
gitlab_rails['federation_sync_interval'] = 300  # 5 minutes
gitlab_rails['federation_sync_batch_size'] = 100
gitlab_rails['federation_conflict_resolution'] = 'hub_wins'

# Monitoring and Metrics
prometheus['enable'] = true
prometheus['listen_address'] = '0.0.0.0:9090'
grafana['enable'] = true
grafana['admin_password'] = ENV['GRAFANA_ADMIN_PASSWORD']

# Security Configuration
gitlab_rails['omniauth_enabled'] = true
gitlab_rails['omniauth_allow_single_sign_on'] = ['saml', 'oauth2_generic']
gitlab_rails['omniauth_block_auto_created_users'] = false
gitlab_rails['omniauth_auto_link_ldap_user'] = true

# Compliance and Audit Logging
gitlab_rails['audit_events_enabled'] = true
gitlab_rails['audit_events_retention_period'] = 2557  # 7 years
gitlab_rails['compliance_framework_enabled'] = true
```

### 2. Regional Federation Node Configuration

#### EU Federation Node (GDPR Compliance)
```ruby
# /etc/gitlab/gitlab.rb - EU Federation Node

external_url 'https://gitlab-eu.bluefly-ai.com'

# Federation Configuration
gitlab_rails['federation_enabled'] = true
gitlab_rails['federation_role'] = 'regional_node'
gitlab_rails['federation_hub_url'] = 'https://gitlab-global-hub.bluefly-ai.com'
gitlab_rails['federation_region'] = 'eu'
gitlab_rails['federation_data_residency'] = 'strict'

# GDPR Compliance Configuration
gitlab_rails['gdpr_enabled'] = true
gitlab_rails['gdpr_data_retention_period'] = 2557  # 7 years
gitlab_rails['gdpr_right_to_be_forgotten'] = true
gitlab_rails['gdpr_data_portability'] = true
gitlab_rails['gdpr_consent_management'] = true

# Data Residency Enforcement
gitlab_rails['data_residency_enforcement'] = 'strict'
gitlab_rails['cross_border_data_transfer'] = 'prohibited'
gitlab_rails['adequacy_decision_countries'] = [
  'andorra', 'argentina', 'canada', 'faroe_islands',
  'guernsey', 'israel', 'isle_of_man', 'japan',
  'jersey', 'new_zealand', 'republic_of_korea',
  'switzerland', 'united_kingdom', 'uruguay'
]

# EU-Specific Security
gitlab_rails['encryption_at_rest'] = 'aes_256_gcm'
gitlab_rails['encryption_in_transit'] = 'tls_1_3_only'
gitlab_rails['key_management'] = 'hsm'  # Hardware Security Module
gitlab_rails['certificate_authority'] = 'eu_qualified_ca'

# Compliance Reporting
gitlab_rails['compliance_reporting_enabled'] = true
gitlab_rails['compliance_report_schedule'] = '0 2 * * MON'  # Weekly Monday 2 AM
gitlab_rails['compliance_frameworks'] = ['gdpr', 'iso27001', 'soc2_type2']
```

#### US Federation Node (HIPAA/SOC2 Compliance)
```ruby
# /etc/gitlab/gitlab.rb - US Federation Node

external_url 'https://gitlab-us.bluefly-ai.com'

# Federation Configuration
gitlab_rails['federation_enabled'] = true
gitlab_rails['federation_role'] = 'regional_node'
gitlab_rails['federation_hub_url'] = 'https://gitlab-global-hub.bluefly-ai.com'
gitlab_rails['federation_region'] = 'us'

# HIPAA Compliance Configuration
gitlab_rails['hipaa_enabled'] = true
gitlab_rails['hipaa_baa_required'] = true  # Business Associate Agreement
gitlab_rails['hipaa_audit_logging'] = 'comprehensive'
gitlab_rails['hipaa_encryption_required'] = true
gitlab_rails['hipaa_access_controls'] = 'role_based'
gitlab_rails['hipaa_data_backup_encryption'] = true

# SOC2 Type II Configuration
gitlab_rails['soc2_enabled'] = true
gitlab_rails['soc2_control_framework'] = 'type_ii'
gitlab_rails['soc2_availability_monitoring'] = true
gitlab_rails['soc2_processing_integrity'] = true
gitlab_rails['soc2_confidentiality_controls'] = true

# FedRAMP Configuration (if applicable)
gitlab_rails['fedramp_enabled'] = false  # Set to true for government clients
gitlab_rails['fedramp_authorization_level'] = 'moderate'  # low, moderate, high
gitlab_rails['fedramp_continuous_monitoring'] = true
```

### 3. Federation Synchronization Service

#### Federation Sync Manager
```php
<?php
// llmcli/src/Services/GitLabFederationManager.php

namespace llmcli\Services;

use GuzzleHttp\Client;
use Psr\Log\LoggerInterface;

class GitLabFederationManager {

  protected $httpClient;
  protected $logger;
  protected $hubUrl;
  protected $federationToken;
  protected $regionConfig;

  public function __construct(
    Client $http_client,
    LoggerInterface $logger,
    array $config
  ) {
    $this->httpClient = $http_client;
    $this->logger = $logger;
    $this->hubUrl = $config['hub_url'];
    $this->federationToken = $config['federation_token'];
    $this->regionConfig = $config['regions'];
  }

  /**
   * Synchronize projects across federation nodes.
   */
  public function synchronizeProjects(): array {
    $sync_results = [];
    
    foreach ($this->regionConfig as $region => $config) {
      try {
        $region_sync = $this->syncRegionProjects($region, $config);
        $sync_results[$region] = $region_sync;
        
        $this->logger->info('Federation sync completed for region: {region}', [
          'region' => $region,
          'projects_synced' => count($region_sync['synced_projects']),
          'conflicts_resolved' => count($region_sync['conflicts_resolved']),
        ]);
        
      } catch (\Exception $e) {
        $this->logger->error('Federation sync failed for region: {region}', [
          'region' => $region,
          'error' => $e->getMessage(),
        ]);
        
        $sync_results[$region] = [
          'status' => 'failed',
          'error' => $e->getMessage(),
        ];
      }
    }
    
    return $sync_results;
  }

  /**
   * Sync projects for a specific region.
   */
  protected function syncRegionProjects(string $region, array $config): array {
    $regional_gitlab = new Client([
      'base_uri' => $config['gitlab_url'],
      'timeout' => 30,
      'headers' => [
        'Authorization' => 'Bearer ' . $config['access_token'],
        'Content-Type' => 'application/json',
      ],
    ]);

    // Get projects from regional node
    $regional_projects = $this->getRegionalProjects($regional_gitlab);
    
    // Get projects from hub
    $hub_projects = $this->getHubProjects($region);
    
    // Identify sync operations needed
    $sync_operations = $this->identifySyncOperations($regional_projects, $hub_projects);
    
    // Execute sync operations
    $sync_results = $this->executeSyncOperations($sync_operations, $regional_gitlab);
    
    return $sync_results;
  }

  /**
   * Execute CI/CD pipeline federation.
   */
  public function federatePipelines(array $pipeline_config): array {
    $federation_results = [];
    
    foreach ($this->regionConfig as $region => $config) {
      if ($this->shouldRunInRegion($pipeline_config, $region)) {
        $pipeline_result = $this->executeFederatedPipeline($region, $config, $pipeline_config);
        $federation_results[$region] = $pipeline_result;
      }
    }
    
    // Coordinate results across regions
    $coordination_result = $this->coordinateFederatedResults($federation_results);
    
    return $coordination_result;
  }

  /**
   * Handle cross-region merge requests.
   */
  public function handleCrossRegionMergeRequest(array $mr_data): array {
    $source_region = $mr_data['source_region'];
    $target_regions = $mr_data['target_regions'];
    
    // Validate compliance for cross-region merge
    $compliance_check = $this->validateCrossRegionCompliance($mr_data);
    if (!$compliance_check['compliant']) {
      throw new \Exception('Cross-region merge request failed compliance check: ' . $compliance_check['reason']);
    }
    
    $merge_results = [];
    
    foreach ($target_regions as $target_region) {
      // Check if merge is allowed between these regions
      if ($this->isCrossRegionMergeAllowed($source_region, $target_region)) {
        $merge_result = $this->executeCrossRegionMerge($source_region, $target_region, $mr_data);
        $merge_results[$target_region] = $merge_result;
      } else {
        $merge_results[$target_region] = [
          'status' => 'blocked',
          'reason' => 'Cross-region merge not allowed due to compliance restrictions',
        ];
      }
    }
    
    return $merge_results;
  }

  /**
   * Validate cross-region compliance for operations.
   */
  protected function validateCrossRegionCompliance(array $operation_data): array {
    $compliance_frameworks = [
      'gdpr' => $this->validateGdprCompliance($operation_data),
      'hipaa' => $this->validateHipaaCompliance($operation_data),
      'pipeda' => $this->validatePipedaCompliance($operation_data),
      'soc2' => $this->validateSoc2Compliance($operation_data),
    ];
    
    $overall_compliant = true;
    $compliance_issues = [];
    
    foreach ($compliance_frameworks as $framework => $result) {
      if (!$result['compliant']) {
        $overall_compliant = false;
        $compliance_issues[] = $framework . ': ' . $result['reason'];
      }
    }
    
    return [
      'compliant' => $overall_compliant,
      'frameworks' => $compliance_frameworks,
      'issues' => $compliance_issues,
      'reason' => implode('; ', $compliance_issues),
    ];
  }
}
```

### 4. GitLab-ECA Integration Webhooks

#### Advanced Webhook Processing
```php
<?php
// _DrupalModules/llm/src/Service/GitlabWebhookProcessor.php

namespace Drupal\llm\Service;

use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\llm\Service\EcaWorkflowTrigger;
use Drupal\secure_drupal\Service\HmacValidator;

class GitlabWebhookProcessor {

  protected $ecaWorkflowTrigger;
  protected $hmacValidator;
  protected $logger;

  public function __construct(
    EcaWorkflowTrigger $eca_workflow_trigger,
    HmacValidator $hmac_validator,
    LoggerChannelFactoryInterface $logger_factory
  ) {
    $this->ecaWorkflowTrigger = $eca_workflow_trigger;
    $this->hmacValidator = $hmac_validator;
    $this->logger = $logger_factory->get('gitlab_webhook_processor');
  }

  /**
   * Process GitLab federation webhooks.
   */
  public function processFederationWebhook(array $webhook_data, string $signature): array {
    // Validate webhook signature
    if (!$this->hmacValidator->validateWebhookSignature($webhook_data, $signature)) {
      throw new \Exception('Invalid webhook signature');
    }

    $webhook_type = $webhook_data['object_kind'];
    $source_region = $webhook_data['region'] ?? 'unknown';
    
    switch ($webhook_type) {
      case 'push':
        return $this->processPushWebhook($webhook_data, $source_region);
        
      case 'merge_request':
        return $this->processMergeRequestWebhook($webhook_data, $source_region);
        
      case 'pipeline':
        return $this->processPipelineWebhook($webhook_data, $source_region);
        
      case 'deployment':
        return $this->processDeploymentWebhook($webhook_data, $source_region);
        
      case 'system_hook':
        return $this->processSystemWebhook($webhook_data, $source_region);
        
      default:
        $this->logger->warning('Unknown webhook type: {type}', ['type' => $webhook_type]);
        return ['status' => 'ignored', 'reason' => 'Unknown webhook type'];
    }
  }

  /**
   * Process deployment webhooks for cross-region coordination.
   */
  protected function processDeploymentWebhook(array $webhook_data, string $source_region): array {
    $deployment_data = $webhook_data['deployment'] ?? [];
    $project_data = $webhook_data['project'] ?? [];
    
    // Extract deployment information
    $deployment_info = [
      'deployment_id' => $deployment_data['id'],
      'environment' => $deployment_data['environment'],
      'status' => $deployment_data['status'],
      'source_region' => $source_region,
      'project_id' => $project_data['id'],
      'project_name' => $project_data['name'],
      'ref' => $deployment_data['ref'],
      'sha' => $deployment_data['sha'],
      'created_at' => $deployment_data['created_at'],
      'updated_at' => $deployment_data['updated_at'],
    ];

    // Determine if this requires cross-region coordination
    $requires_federation = $this->requiresFederationCoordination($deployment_info);
    
    if ($requires_federation) {
      // Trigger ECA workflow for federated deployment
      $eca_result = $this->ecaWorkflowTrigger->triggerWorkflow(
        'federated_deployment_coordinator',
        $deployment_info
      );
      
      return [
        'status' => 'federated',
        'deployment_info' => $deployment_info,
        'eca_workflow_id' => $eca_result['workflow_id'],
        'coordination_status' => $eca_result['status'],
      ];
    } else {
      // Handle as regional deployment
      $eca_result = $this->ecaWorkflowTrigger->triggerWorkflow(
        'regional_deployment_handler',
        $deployment_info
      );
      
      return [
        'status' => 'regional',
        'deployment_info' => $deployment_info,
        'eca_workflow_id' => $eca_result['workflow_id'],
      ];
    }
  }

  /**
   * Process pipeline webhooks for CI/CD coordination.
   */
  protected function processPipelineWebhook(array $webhook_data, string $source_region): array {
    $pipeline_data = $webhook_data['object_attributes'] ?? [];
    $project_data = $webhook_data['project'] ?? [];
    
    $pipeline_info = [
      'pipeline_id' => $pipeline_data['id'],
      'status' => $pipeline_data['status'],
      'source_region' => $source_region,
      'project_id' => $project_data['id'],
      'project_name' => $project_data['name'],
      'ref' => $pipeline_data['ref'],
      'sha' => $pipeline_data['sha'],
      'stages' => $webhook_data['builds'] ?? [],
      'created_at' => $pipeline_data['created_at'],
      'finished_at' => $pipeline_data['finished_at'],
    ];

    // Check if pipeline requires cross-region testing
    if ($this->requiresCrossRegionTesting($pipeline_info)) {
      $eca_result = $this->ecaWorkflowTrigger->triggerWorkflow(
        'cross_region_pipeline_coordinator',
        $pipeline_info
      );
      
      return [
        'status' => 'cross_region_testing',
        'pipeline_info' => $pipeline_info,
        'eca_workflow_id' => $eca_result['workflow_id'],
      ];
    }

    // Handle pipeline status changes
    if (in_array($pipeline_data['status'], ['failed', 'success', 'canceled'])) {
      $eca_result = $this->ecaWorkflowTrigger->triggerWorkflow(
        'pipeline_status_handler',
        $pipeline_info
      );
      
      return [
        'status' => 'status_handled',
        'pipeline_info' => $pipeline_info,
        'eca_workflow_id' => $eca_result['workflow_id'],
      ];
    }

    return [
      'status' => 'processed',
      'pipeline_info' => $pipeline_info,
    ];
  }

  /**
   * Determine if deployment requires federation coordination.
   */
  protected function requiresFederationCoordination(array $deployment_info): bool {
    // Check if environment indicates global deployment
    $global_environments = ['production', 'staging-global', 'pre-production'];
    if (in_array($deployment_info['environment'], $global_environments)) {
      return true;
    }

    // Check if project is configured for federated deployment
    $federated_projects = $this->getFederatedProjectsList();
    if (in_array($deployment_info['project_id'], $federated_projects)) {
      return true;
    }

    // Check deployment configuration for federation flags
    if (isset($deployment_info['federation_required']) && $deployment_info['federation_required']) {
      return true;
    }

    return false;
  }

  /**
   * Check if pipeline requires cross-region testing.
   */
  protected function requiresCrossRegionTesting(array $pipeline_info): bool {
    // Check for cross-region testing stages
    foreach ($pipeline_info['stages'] as $stage) {
      if (strpos($stage['name'], 'cross-region') !== false || 
          strpos($stage['name'], 'global-test') !== false) {
        return true;
      }
    }

    // Check project configuration
    $cross_region_projects = $this->getCrossRegionTestingProjects();
    return in_array($pipeline_info['project_id'], $cross_region_projects);
  }
}
```

### 5. Federation CI/CD Pipeline Templates

#### Global Deployment Pipeline
```yaml
# .gitlab-ci-federation-templates/global-deployment.yml

# Federation-aware deployment pipeline template
.federation_deployment_template: &federation_deployment
  stage: deploy
  image: registry.gitlab.bluefly..io/bluefly-ai/deployment-tools:latest
  variables:
    FEDERATION_MODE: "true"
    DEPLOYMENT_STRATEGY: "blue_green"
    COMPLIANCE_VALIDATION: "strict"
  before_script:
    - echo "Initializing federation deployment..."
    - source /opt/federation-tools/setup.sh
    - federation-auth --region $CI_REGION
  script:
    - |
      # Validate deployment compliance
      if ! federation-compliance-check --project $CI_PROJECT_ID --target-regions "$DEPLOYMENT_REGIONS"; then
        echo "Compliance check failed. Deployment aborted."
        exit 1
      fi
      
      # Coordinate with federation hub
      FEDERATION_ID=$(federation-coordinator start-deployment \
        --project $CI_PROJECT_ID \
        --commit $CI_COMMIT_SHA \
        --regions "$DEPLOYMENT_REGIONS" \
        --strategy $DEPLOYMENT_STRATEGY)
      
      echo "Federation deployment ID: $FEDERATION_ID"
      
      # Execute regional deployments
      for region in $(echo $DEPLOYMENT_REGIONS | tr ',' ' '); do
        echo "Deploying to region: $region"
        
        federation-deploy \
          --federation-id $FEDERATION_ID \
          --region $region \
          --project $CI_PROJECT_ID \
          --commit $CI_COMMIT_SHA \
          --strategy $DEPLOYMENT_STRATEGY \
          --compliance-mode strict
          
        # Wait for deployment health check
        federation-health-check \
          --federation-id $FEDERATION_ID \
          --region $region \
          --timeout 300
      done
      
      # Finalize federation deployment
      federation-coordinator finalize-deployment \
        --federation-id $FEDERATION_ID \
        --success-threshold 0.95
  after_script:
    - |
      # Generate deployment report
      federation-report generate \
        --federation-id $FEDERATION_ID \
        --format json \
        --output federation-deployment-report.json
      
      # Trigger ECA webhook for deployment completion
      curl -X POST \
        -H "Content-Type: application/json" \
        -H "X-Federation-Token: $FEDERATION_WEBHOOK_TOKEN" \
        -d @federation-deployment-report.json \
        "$ECA_WEBHOOK_ENDPOINT/federation/deployment-complete"
  artifacts:
    reports:
      dotenv: federation-deployment.env
    paths:
      - federation-deployment-report.json
    expire_in: 7 days
  when: manual
  allow_failure: false

# Regional deployment jobs
deploy_us_regions:
  <<: *federation_deployment
  variables:
    CI_REGION: "us"
    DEPLOYMENT_REGIONS: "us-east-1,us-west-2"
    COMPLIANCE_FRAMEWORKS: "hipaa,soc2,fedramp"
  rules:
    - if: '$CI_COMMIT_BRANCH == "main" && $DEPLOY_TARGET == "production"'
    - if: '$CI_COMMIT_BRANCH =~ /^release\/.*/ && $DEPLOY_TARGET == "staging"'

deploy_eu_regions:
  <<: *federation_deployment
  variables:
    CI_REGION: "eu"
    DEPLOYMENT_REGIONS: "eu-west-1,eu-central-1"
    COMPLIANCE_FRAMEWORKS: "gdpr,iso27001,soc2"
    DATA_RESIDENCY: "strict"
  rules:
    - if: '$CI_COMMIT_BRANCH == "main" && $DEPLOY_TARGET == "production"'
    - if: '$CI_COMMIT_BRANCH =~ /^release\/.*/ && $DEPLOY_TARGET == "staging"'

deploy_apac_regions:
  <<: *federation_deployment
  variables:
    CI_REGION: "apac"
    DEPLOYMENT_REGIONS: "ap-southeast-1,ap-northeast-1,ap-south-1"
    COMPLIANCE_FRAMEWORKS: "local_dpla,soc2"
  rules:
    - if: '$CI_COMMIT_BRANCH == "main" && $DEPLOY_TARGET == "production"'
    - if: '$CI_COMMIT_BRANCH =~ /^release\/.*/ && $DEPLOY_TARGET == "staging"'

# Cross-region testing pipeline
cross_region_integration_test:
  stage: test
  image: registry.gitlab.bluefly..io/bluefly-ai/test-tools:latest
  variables:
    TEST_TYPE: "cross_region_integration"
  script:
    - |
      echo "Running cross-region integration tests..."
      
      # Test inter-region communication
      for source_region in us-east-1 eu-west-1 ap-southeast-1; do
        for target_region in us-west-2 eu-central-1 ap-northeast-1; do
          if [ "$source_region" != "$target_region" ]; then
            echo "Testing $source_region -> $target_region communication"
            
            federation-test communication \
              --source $source_region \
              --target $target_region \
              --test-suite inter-region-api \
              --timeout 30
              
            federation-test latency \
              --source $source_region \
              --target $target_region \
              --max-latency 200ms
          fi
        done
      done
      
      # Test data residency compliance
      federation-test data-residency \
        --all-regions \
        --compliance-frameworks gdpr,hipaa,pipeda \
        --strict-mode
      
      # Test cross-region failover
      federation-test failover \
        --primary-region us-east-1 \
        --backup-region us-west-2 \
        --test-duration 60s
  artifacts:
    reports:
      junit: cross-region-test-results.xml
    paths:
      - test-reports/
    expire_in: 3 days
  rules:
    - if: '$CI_PIPELINE_SOURCE == "merge_request_event"'
    - if: '$CI_COMMIT_BRANCH =~ /^feature\/cross-region-.*/'
```

### 6. Federation Monitoring and Alerting

#### Federation Health Dashboard Configuration
```yaml
# grafana/dashboards/federation-health.json
{
  "dashboard": {
    "title": "GitLab Federation Health Dashboard",
    "tags": ["federation", "gitlab", "global"],
    "timezone": "UTC",
    "panels": [
      {
        "title": "Federation Sync Status",
        "type": "stat",
        "targets": [
          {
            "expr": "gitlab_federation_sync_success_rate",
            "legendFormat": "{{region}} Success Rate"
          }
        ],
        "fieldConfig": {
          "defaults": {
            "color": {
              "mode": "thresholds"
            },
            "thresholds": {
              "steps": [
                {"color": "red", "value": 0},
                {"color": "yellow", "value": 0.95},
                {"color": "green", "value": 0.99}
              ]
            }
          }
        }
      },
      {
        "title": "Cross-Region Latency",
        "type": "timeseries",
        "targets": [
          {
            "expr": "gitlab_federation_cross_region_latency_seconds",
            "legendFormat": "{{source_region}} -> {{target_region}}"
          }
        ]
      },
      {
        "title": "Compliance Violations",
        "type": "table",
        "targets": [
          {
            "expr": "increase(gitlab_federation_compliance_violations_total[1h])",
            "legendFormat": "{{region}} - {{framework}}"
          }
        ]
      },
      {
        "title": "Federation Pipeline Success Rate",
        "type": "gauge",
        "targets": [
          {
            "expr": "gitlab_federation_pipeline_success_rate",
            "legendFormat": "Overall Success Rate"
          }
        ]
      }
    ]
  }
}
```

#### Federation Alert Rules
```yaml
# prometheus/rules/federation-alerts.yml
groups:
  - name: gitlab_federation_alerts
    rules:
      - alert: FederationSyncFailure
        expr: gitlab_federation_sync_success_rate < 0.95
        for: 5m
        labels:
          severity: critical
          component: federation
        annotations:
          summary: "GitLab federation sync failing in {{ $labels.region }}"
          description: "Federation sync success rate in {{ $labels.region }} is {{ $value }}%, below the 95% threshold"

      - alert: CrossRegionLatencyHigh
        expr: gitlab_federation_cross_region_latency_seconds > 0.5
        for: 2m
        labels:
          severity: warning
          component: federation
        annotations:
          summary: "High cross-region latency detected"
          description: "Latency from {{ $labels.source_region }} to {{ $labels.target_region }} is {{ $value }}s"

      - alert: ComplianceViolation
        expr: increase(gitlab_federation_compliance_violations_total[5m]) > 0
        for: 0m
        labels:
          severity: critical
          component: compliance
        annotations:
          summary: "Compliance violation detected in {{ $labels.region }}"
          description: "{{ $labels.framework }} compliance violation in {{ $labels.region }}: {{ $labels.violation_type }}"

      - alert: FederationDeploymentFailure
        expr: gitlab_federation_deployment_failure_rate > 0.05
        for: 1m
        labels:
          severity: critical
          component: deployment
        annotations:
          summary: "High federation deployment failure rate"
          description: "Federation deployment failure rate is {{ $value }}%, above the 5% threshold"
```

## Implementation Timeline

### Phase 1: Foundation Setup (Month 1)
- Deploy global GitLab hub with high availability
- Set up regional federation nodes
- Implement basic synchronization

### Phase 2: Compliance Integration (Month 2)
- Configure region-specific compliance frameworks
- Implement data residency enforcement
- Set up compliance monitoring and reporting

### Phase 3: Advanced Federation (Month 3)
- Deploy cross-region pipeline coordination
- Implement federated merge request handling
- Set up advanced webhook processing

### Phase 4: Monitoring and Optimization (Month 4)
- Deploy comprehensive federation monitoring
- Implement predictive failure detection
- Optimize cross-region performance

This GitLab federation architecture ensures enterprise-grade coordination across thousands of servers while maintaining strict compliance with regional data residency requirements and seamless integration with ECA workflows.
