---
id: guides-SHARED_UTILITIES_MIGRATION_GUIDE
---
# Shared Utilities Migration Guide

## Overview

This guide documents the comprehensive migration of LLM Platform projects to use a centralized set of shared utilities from the `@bluefly/llmcli/shared` package. This migration standardizes key functionality across projects, reduces code duplication, improves maintainability, and enhances the development experience.

## Shared Utilities

The `@bluefly/llmcli/shared` package contains the following core utilities:

### 1. Error Handling

- **Location**: `@bluefly/llmcli/shared/errors`
- **Features**:
  - Standardized error classes with error codes
  - Error creation helpers and utilities
  - Error translation and formatting
  - Express error handling middleware
  - Type definitions for all error types
- **Benefits**:
  - Consistent error patterns across projects
  - Improved error classification
  - Standardized HTTP error responses

### 2. Logging

- **Location**: `@bluefly/llmcli/shared/logger`
- **Features**:
  - Unified logging interface
  - Configurable log levels and formats
  - Contextual logging with correlation IDs
  - Support for multiple transport targets
- **Benefits**:
  - Uniform log format across projects
  - Centralized log configuration
  - Enhanced debugging capabilities

### 3. HTTP Client

- **Location**: `@bluefly/llmcli/shared/http`
- **Features**:
  - Resilient HTTP client with retries
  - Circuit breaker pattern
  - Connection pooling
  - Request/response interceptors
- **Benefits**:
  - Improved reliability of service-to-service communication
  - Automatic handling of transient failures
  - Standardized request lifecycle

### 4. Observability

- **Location**: `@bluefly/llmcli/shared/observability`
- **Features**:
  - Metrics collection and reporting
  - Distributed tracing
  - Health check utilities
  - Monitoring integration
- **Benefits**:
  - Consistent monitoring across projects
  - Enhanced visibility into system behavior
  - Standardized health check endpoints

### 5. General Utilities

- **Location**: `@bluefly/llmcli/shared/utils`
- **Features**:
  - Common utilities for string/object manipulation
  - Date/time helpers
  - ID generation
  - Configuration helpers
- **Benefits**:
  - Reduced duplication of common functionality
  - Standardized implementation of utility functions

## Migration Status

| Project | Status | Notes |
|---------|--------|-------|
| bfapi |  Completed | Full migration to shared utilities |
| bfllm |  Completed | Error handling and logging migrated, HTTP client in progress |
| llm-mcp |  Completed | Full migration to shared utilities |
| bfrfp |  Completed | Full migration to shared utilities |
| llm-gateway-sdk | 🔄 In Progress | Migration scripts created, implementation in progress |
| llm-ui | 🔄 Pending | Migration scripts created, ready for implementation |
| bfcicomponents | 🔄 Pending | Migration scripts created, ready for implementation |
| bfdocs | 🔄 Pending | Migration scripts created, ready for implementation |

## Implementation Details

### Error Handling

```typescript
// Import shared error utilities
import { 
  LLM PlatformError, 
  createError, 
  ERROR_CODES 
} from '@bluefly/llmcli/shared';

// Create a domain-specific error
const serviceError = createError(
  'Service unavailable',
  ERROR_CODES.SERVICE_UNAVAILABLE,
  503,
  { service: 'exampleService', retryable: true }
);

// Extended error class for domain-specific errors
class DomainSpecificError implements LLM PlatformError {
  code = 'DOMAIN_SPECIFIC_ERROR';
  message: string;
  statusCode = 400;
  
  constructor(message: string, details?: Record<string, any>) {
    this.message = message;
    this.details = details;
  }
}
```

### Logging

```typescript
// Import shared logger
import { LLM PlatformLogger } from '@bluefly/llmcli/shared';

// Create service-specific logger
const logger = new LLM PlatformLogger({
  prefix: '[my-service]',
  level: process.env.LOG_LEVEL || 'info',
  timestamp: true,
  colorize: process.env.NODE_ENV !== 'production'
});

// Usage
logger.info('Operation successful', { 
  userId: 'user123', 
  correlationId: 'abc-123'
});

logger.error('Operation failed', {
  error: new Error('Database connection failed'),
  correlationId: 'abc-123'
});
```

### HTTP Client

```typescript
// Import shared HTTP client
import { HttpClient } from '@bluefly/llmcli/shared';

// Create HTTP client instance
const httpClient = new HttpClient({
  baseURL: 'https://api.example.com',
  timeout: 5000,
  retries: 3,
  retryDelay: 300,
  headers: {
    'Content-Type': 'application/json',
    'X-API-Version': '1.0'
  }
});

// Usage
try {
  const response = await httpClient.get('/users/123', {
    context: { correlationId: 'abc-123' }
  });
  
  console.log(response.data);
} catch (error) {
  // Error already standardized and logged
}
```

## Migration Process

Each project follows these steps for migration:

1. **Analysis**: Identify custom implementations that can be replaced with shared utilities
2. **Dependency Update**: Add `@bluefly/llmcli` to package.json
3. **Implementation**: Replace custom code with shared utilities
4. **Backward Compatibility**: Add adapters/wrappers for backward compatibility
5. **Testing**: Ensure all functionality works correctly with shared utilities
6. **Documentation**: Update project documentation to reflect changes

## Migration Scripts

Migration scripts are available in the `@bluefly/llmcli/shared/scripts` directory:

- `migrate-bfapi.js`: Migration script for BFAPI
- `migrate-bfllm.js`: Migration script for BFLLM
- `migrate-llm-mcp.js`: Migration script for llm-mcp
- `migrate-bfrfp.js`: Migration script for BFRFP
- `migrate-llm-gateway-sdk.js`: Migration script for BFAPI Client
- `migrate-llm-ui.js`: Migration script for llm-ui
- `migrate-bfcicomponents.js`: Migration script for BF CI Components
- `migrate-bfdocs.js`: Migration script for BFDocs

## Best Practices

1. **Import with .js Extension**: Use `.js` extension in import paths for ES module compatibility
   ```typescript
   import { createError } from '@bluefly/llmcli/shared/errors/index.js';
   ```

2. **Error Handling**: Use the shared error system for all error handling
   ```typescript
   try {
     // Operation
   } catch (error) {
     throw errorUtils.translateError(error, 'SERVICE_NAME');
   }
   ```

3. **Logging**: Use contextual logging with correlation IDs
   ```typescript
   logger.info('Operation started', { correlationId, userId, operation });
   ```

4. **HTTP Requests**: Use the shared HTTP client for all external requests
   ```typescript
   const response = await httpClient.post('/endpoint', payload, {
     retries: 5,
     timeout: 10000
   });
   ```

5. **Configuration**: Use environment variables with sensible defaults
   ```typescript
   const logLevel = process.env.LOG_LEVEL || 'info';
   const serviceUrl = process.env.SERVICE_URL || 'http://localhost:3000';
   ```

## Troubleshooting

### Common Issues

1. **Import Errors**
   - Problem: `Cannot find module '@bluefly/llmcli/shared'`
   - Solution: Ensure `@bluefly/llmcli` is installed and package.json is properly configured

2. **Type Errors**
   - Problem: TypeScript errors with shared utility types
   - Solution: Update tsconfig.json to include types from the shared package

3. **Backward Compatibility**
   - Problem: Breaking changes when migrating to shared utilities
   - Solution: Use adapter patterns and wrapper classes to maintain existing interfaces

### Support

For issues with shared utilities, contact the LLM Platform Platform Team or open an issue in the llmcli repository.

## Project-Specific Documentation

For project-specific migration details, see:

- [BFAPI Shared Utilities Migration](./bfapi/docs/SHARED_UTILITIES_MIGRATION)
- [BFLLM Shared Utilities Migration](./bfllm/docs/SHARED_UTILITIES_MIGRATION)
- [llm-mcp Shared Utilities Migration](./llm-mcp/docs/SHARED_UTILITIES_MIGRATION)
- [BFRFP Shared Utilities Migration](./bfrfp/docs/SHARED_UTILITIES_MIGRATION)

## Contribution

To contribute to the shared utilities:

1. Make changes in the `@bluefly/llmcli/shared` directory
2. Ensure backward compatibility
3. Add comprehensive tests
4. Update documentation
5. Submit a pull request
