---
id: guides-private-llm-implementation
---
# Private LLM Implementation Guide
*Using Existing Drupal AI Modules for Complete Data Privacy*

## 🎯 **Overview: Private LLM with Task Routing**

This guide shows how to implement a private, self-hosted LLM solution using existing Drupal modules. All data stays on your servers, no external API calls, complete privacy.

## 📋 **Prerequisites**

### **Server Requirements**
- **GPU Server**: For running large models (recommended: NVIDIA GPU with 24GB+ VRAM)
- **CPU Alternative**: Can run smaller models on CPU (slower but works)
- **RAM**: 32GB minimum, 64GB+ recommended
- **Storage**: 100GB+ for multiple models

### **Software Requirements**
- **Drupal 10.1+** or **Drupal 11**
- **Composer** for module installation
- **Drush** for configuration
- **Ollama** for local model management

## 🚀 **Step 1: Install Ollama (Model Server)**

### **Install Ollama on Your Server**
```bash
# Linux/WSL
curl -fsSL https://ollama.ai/install.sh | sh

# macOS
brew install ollama

# Start Ollama service
ollama serve
```

### **Configure Ollama for Network Access**
```bash
# Edit Ollama service to listen on all interfaces
# For systemd (Linux):
sudo systemctl edit ollama

# Add:
[Service]
Environment="OLLAMA_HOST=0.0.0.0"

# Restart service
sudo systemctl restart ollama
```

### **Pull Required Models**
```bash
# General content generation (70B parameters)
ollama pull llama2:70b

# Code analysis and generation (34B parameters)
ollama pull codellama:34b

# Fast summarization (7B parameters)
ollama pull mistral:7b

# Embeddings for semantic search (384 dimensions)
ollama pull nomic-embed-text

# Question answering (40B parameters)
ollama pull falcon:40b

# Verify models are installed
ollama list
```

## 📦 **Step 2: Install Drupal AI Modules**

### **Add Required Modules**
```bash
cd /path/to/drupal/root

# Core AI framework
composer require drupal/ai:^1.0@beta

# Ollama provider for local models
composer require drupal/ai_provider_ollama:^1.0@alpha

# ECA for workflow automation
composer require drupal/eca:^2.0

# Search API AI for embeddings/RAG
composer require drupal/search_api_ai:^1.0@alpha

# Enable modules
drush en ai ai_provider_ollama eca eca_base eca_content search_api_ai -y
```

## ⚙️ **Step 3: Configure AI Providers**

### **1. Configure Ollama Connection**
Navigate to: `/admin/config/ai/providers`

```yaml
Provider Type: Ollama
Label: Local Ollama Server
Host: http://localhost:11434  # Or your Ollama server IP
Authentication: None (local server)
```

### **2. Create Multiple Provider Instances**

#### **Content Generation Provider**
```yaml
Provider ID: content_generation
Label: Content Generation (Llama 2 70B)
Provider Type: Ollama
Model: llama2:70b
Default Temperature: 0.7
Max Tokens: 4096
```

#### **Code Analysis Provider**
```yaml
Provider ID: code_analysis  
Label: Code Analysis (CodeLlama 34B)
Provider Type: Ollama
Model: codellama:34b
Default Temperature: 0.1  # Lower for more deterministic code
Max Tokens: 8192
```

#### **Summarization Provider**
```yaml
Provider ID: summarization
Label: Fast Summarization (Mistral 7B)
Provider Type: Ollama
Model: mistral:7b
Default Temperature: 0.3
Max Tokens: 1024
```

#### **Embeddings Provider**
```yaml
Provider ID: embeddings
Label: Semantic Search Embeddings
Provider Type: Ollama
Model: nomic-embed-text
Dimensions: 384
```

## 🔄 **Step 4: Create Task Routing with ECA**

### **1. Install ECA Modeler**
```bash
# Install BPMN.io modeler (recommended)
composer require drupal/bpmn_io:^2.0
drush en bpmn_io -y
```

### **2. Create Workflow for Content Type Routing**

Navigate to: `/admin/config/workflow/eca`

#### **Workflow: AI Task Router**

**Event**: Entity presave (any entity)

**Conditions**:
1. **Check if Code**: Field type equals "code" OR content contains code blocks
2. **Check if Long Text**: Field type equals "text_long" 
3. **Check Task Type**: Custom field "ai_task_type" value

**Actions**:

**Branch 1: Code Analysis**
- Condition: Field contains code
- Action: Execute AI Provider (code_analysis)
- Store result in: field_ai_analysis

**Branch 2: Content Generation**  
- Condition: Field type is text_long AND ai_task_type = "generate"
- Action: Execute AI Provider (content_generation)
- Store result in: field_ai_generated

**Branch 3: Summarization**
- Condition: ai_task_type = "summarize"
- Action: Execute AI Provider (summarization)
- Store result in: field_ai_summary

### **3. Visual Workflow Example**
```
[Entity Presave Event]
         |
    [Condition: Check Task Type]
         |
    +---------+---------+
    |         |         |
[Code?]  [Generate?] [Summarize?]
    |         |         |
[CodeLlama] [Llama2]  [Mistral]
    |         |         |
[Save Result to Field]
```

## 🔍 **Step 5: Configure Search API AI for RAG**

### **1. Install Vector Database**
```bash
# Option 1: Milvus (recommended for production)
docker run -d --name milvus \
  -p 19530:19530 \
  -p 9091:9091 \
  milvusdb/milvus:latest

# Option 2: Use PostgreSQL with pgvector
docker run -d --name pgvector \
  -e POSTGRES_PASSWORD=vector123 \
  -p 5432:5432 \
  ankane/pgvector
```

### **2. Configure Search API**
Navigate to: `/admin/config/search/search-api`

1. **Add Server**:
   - Backend: "AI Embeddings Backend"
   - Vector Database: Milvus/PostgreSQL
   - Embedding Provider: embeddings (Ollama)

2. **Add Index**:
   - Datasource: Content
   - Fields to index: Title, Body, Custom fields
   - Enable: "Generate embeddings on save"

### **3. Create RAG-Enabled Search**
```php
// In your custom module or theme
$search_api = \Drupal::service('search_api_ai.search');
$results = $search_api->semanticSearch('your query', [
  'index' => 'your_index',
  'limit' => 10,
  'include_embeddings' => TRUE,
]);
```

## 🔐 **Step 6: Security Configuration**

### **1. Firewall Rules**
```bash
# Only allow Ollama access from Drupal server
sudo ufw allow from drupal_server_ip to any port 11434

# Block external access
sudo ufw deny 11434
```

### **2. Configure Audit Logging**
Using your existing `secure_drupal` module:

```php
// In settings.php
$config['secure_drupal.settings']['audit_ai_operations'] = TRUE;
$config['secure_drupal.settings']['log_prompts'] = FALSE; // Privacy
$config['secure_drupal.settings']['log_responses'] = FALSE; // Privacy
```

### **3. Access Control**
```yaml
# Create AI-specific permissions
ai use content generation:
  title: 'Use AI for content generation'
  
ai use code analysis:
  title: 'Use AI for code analysis'
  
ai use embeddings:
  title: 'Generate embeddings'
```

## 📊 **Step 7: Monitoring and Optimization**

### **1. Performance Monitoring**
```bash
# Monitor Ollama resource usage
ollama ps  # Show running models
nvidia-smi  # GPU usage (if using GPU)
htop  # CPU/RAM usage
```

### **2. Model Performance Tuning**
```yaml
# Adjust based on your hardware
# For GPU-constrained systems:
num_gpu: 1
gpu_layers: 35  # Adjust based on VRAM

# For CPU-only systems:
num_thread: 16  # Match CPU cores
use_mmap: true
use_mlock: false
```

### **3. Caching Configuration**
```php
// In settings.php
$settings['ai_cache_responses'] = TRUE;
$settings['ai_cache_ttl'] = 3600; // 1 hour
$settings['ai_cache_vary_by_user'] = TRUE;
```

## 🧪 **Step 8: Testing Your Setup**

### **1. Test Each Provider**
```bash
# Test via Drush
drush ai:test content_generation "Write a blog post about Drupal"
drush ai:test code_analysis "function calculateTax(\$amount) { return \$amount * 0.1; }"
drush ai:test summarization "Long text to summarize..."
```

### **2. Test ECA Workflows**
1. Create test content with different types
2. Verify correct model is used
3. Check results in designated fields

### **3. Test Search/RAG**
```bash
# Index content
drush search-api:index

# Test semantic search
drush search-api:search "find content about AI integration"
```

## 🚨 **Troubleshooting**

### **Common Issues**

**1. Ollama Connection Failed**
```bash
# Check Ollama is running
curl http://localhost:11434/api/tags

# Check firewall
sudo ufw status
```

**2. Model Too Large for Hardware**
```bash
# Use quantized versions
ollama pull llama2:13b  # Instead of 70b
ollama pull codellama:7b  # Instead of 34b
```

**3. Slow Response Times**
- Enable GPU acceleration
- Use smaller models for real-time tasks
- Implement response caching
- Load balance across multiple Ollama instances

## 📈 **Scaling Your Private LLM**

### **Multiple Ollama Instances**
```nginx
# nginx load balancer config
upstream ollama_cluster {
    server ollama1.local:11434;
    server ollama2.local:11434;
    server ollama3.local:11434;
}
```

### **Model Routing by Load**
```yaml
# In ECA workflow
Condition: System load < 50%
  Action: Use large model (70B)
Else:
  Action: Use medium model (13B)
```

## ✅ **Verification Checklist**

- [ ] Ollama installed and accessible
- [ ] All required models downloaded
- [ ] AI module configured with providers
- [ ] ECA workflows created for task routing
- [ ] Search API AI configured for embeddings
- [ ] Security rules implemented
- [ ] Audit logging enabled
- [ ] Performance monitoring active
- [ ] Test queries returning expected results

## 🎯 **Result: Complete Private LLM Platform**

You now have:
- **Complete privacy**: All processing on your servers
- **Task-specific routing**: Different models for different jobs
- **No API costs**: Using open-source models
- **Scalable**: Add more models or servers as needed
- **Integrated**: Works with all Drupal features
- **Secure**: Audit logging and access control

**Total implementation time: 2 weeks (vs 4+ weeks of custom development)**
