---
id: integration-code-quality
---
# Integration Code Quality Checklist

This document provides a comprehensive code quality checklist specifically focused on module integration points. Following these guidelines will ensure high-quality, maintainable, and robust cross-module integrations.

## Interface Implementation Quality

### Interface Compliance
- [ ] All methods defined in the interface are implemented
- [ ] Method signatures (parameters and return types) match exactly
- [ ] PHPDoc comments accurately describe implementation behavior
- [ ] Implementation respects interface contracts and semantics

### Type Safety
- [ ] Type hints are used consistently for parameters and return values
- [ ] Return values match the documented types
- [ ] Type-safe comparisons are used (`===` instead of `==` where appropriate)
- [ ] Null checks are performed before accessing potentially null objects

### Default Values and Optional Parameters
- [ ] Default values for optional parameters match interface definitions
- [ ] Implementation handles missing optional parameters gracefully
- [ ] Nullable parameters are explicitly marked with `?` type prefix

## Dependency Management

### Service Dependencies
- [ ] Required services are injected via constructor
- [ ] Optional dependencies use `?` nullable type hints
- [ ] Fallback behavior exists when optional dependencies are missing
- [ ] Implementation checks for service existence before using it

### Module Dependencies
- [ ] Uses `optional_dependencies` in .info.yml for non-critical dependencies
- [ ] Checks for module existence with `\Drupal::moduleHandler()->moduleExists()`
- [ ] Provides graceful degradation when dependent modules are not available
- [ ] Uses capability-based detection instead of module existence checks where possible

### Interface Dependencies
- [ ] Depends on interfaces rather than concrete implementations
- [ ] Uses shared interfaces from `llm_platform_interfaces` module
- [ ] Avoids circular dependencies between modules

## Error Handling and Logging

### Exception Handling
- [ ] Critical exceptions are caught and handled at integration boundaries
- [ ] Appropriate exception types are used (`\InvalidArgumentException`, etc.)
- [ ] Exception messages are clear and informative
- [ ] Exceptions include relevant context information

### Logging
- [ ] Uses consistent log levels across integration points:
  - EMERGENCY: System is unusable
  - ALERT: Action must be taken immediately
  - CRITICAL: Critical conditions
  - ERROR: Error conditions
  - WARNING: Warning conditions
  - NOTICE: Normal but significant condition
  - INFO: Informational messages
  - DEBUG: Debug-level messages
- [ ] Log messages include relevant context
- [ ] Sensitive data is not logged
- [ ] Uses structured logging with context arrays

### Fault Tolerance
- [ ] Implements circuit breaker pattern for external services
- [ ] Sets appropriate timeouts for external calls
- [ ] Provides sensible defaults when integration fails
- [ ] Maintains system stability during integration failures

## Service Discovery and Registration

### Service Tags
- [ ] Uses appropriate service tags for discovery
- [ ] Tag names follow consistent conventions
- [ ] Tag attributes are used to provide metadata
- [ ] Services are discoverable through the ServiceLocator

### Capability Declaration
- [ ] Capabilities are explicitly declared in service tags
- [ ] Capabilities follow consistent naming conventions
- [ ] Multiple capabilities are declared when appropriate
- [ ] Capability levels or scores are provided when relevant

### Service Registration
- [ ] Services are registered in the module's services.yml file
- [ ] Bridge services have appropriate arguments injected
- [ ] Bridge services are tagged with `ai_bridge` or similar tags
- [ ] Bridge services are public when needed by other modules

## Cross-Module Communication

### Event-Based Communication
- [ ] Uses Symfony EventDispatcher for loose coupling
- [ ] Event names follow consistent naming conventions
- [ ] Event classes extend appropriate base classes
- [ ] Event subscribers are properly registered

### Bridge Pattern Implementation
- [ ] Bridge class implements appropriate interface(s)
- [ ] Bridge provides complete implementations of all methods
- [ ] Bridge maintains state appropriately
- [ ] Bridge handles errors gracefully

### Service Locator Usage
- [ ] Uses ServiceLocator for runtime service discovery
- [ ] Caches discovery results appropriately
- [ ] Falls back gracefully when services are not found
- [ ] Uses capability-based discovery when appropriate

## Testing

### Unit Tests
- [ ] Unit tests exist for all interface implementations
- [ ] Tests verify behavior with missing dependencies
- [ ] Mock objects are used for external dependencies
- [ ] Edge cases are tested

### Integration Tests
- [ ] Tests verify cross-module behavior
- [ ] Tests cover error conditions and recovery
- [ ] Tests include performance benchmarks where appropriate
- [ ] Tests verify graceful degradation

### Test Coverage
- [ ] Integration points have near 100% test coverage
- [ ] Tests verify all possible execution paths
- [ ] Tests verify behavior with and without optional modules
- [ ] Test data includes realistic scenarios

## Performance and Caching

### Performance Considerations
- [ ] Integration points are optimized for performance
- [ ] Heavy operations are performed asynchronously when possible
- [ ] Resource usage is monitored and logged
- [ ] Performance bottlenecks are identified and addressed

### Caching Strategy
- [ ] Results are cached when appropriate
- [ ] Cache tags are used for proper invalidation
- [ ] Cache contexts are set correctly
- [ ] Cache lifetimes are appropriate for the data

## Security

### Input Validation
- [ ] All input from other modules is validated
- [ ] User-provided data is sanitized before use
- [ ] Security-sensitive operations are protected
- [ ] CSRF protection is implemented where needed

### Output Sanitization
- [ ] Data passed to other modules is sanitized
- [ ] HTML output is properly escaped
- [ ] Security tokens are handled properly
- [ ] Sensitive data is not exposed

### Authentication and Authorization
- [ ] Access checks are performed before operations
- [ ] Integration respects Drupal's permission system
- [ ] API keys and credentials are stored securely
- [ ] Principle of least privilege is followed

## Documentation

### Code Comments
- [ ] Methods have descriptive PHPDoc comments
- [ ] Complex code sections have inline comments
- [ ] Integration points have detailed documentation
- [ ] Examples are provided for common use cases

### External Documentation
- [ ] Integration patterns are documented in the project
- [ ] README.md includes integration examples
- [ ] Changes to integration points are documented in CHANGELOG.md
- [ ] Deprecation notices include migration paths

## Upgrade and Maintenance

### Versioning
- [ ] Follows semantic versioning for interface changes
- [ ] Breaking changes are clearly documented
- [ ] Maintains backward compatibility when possible
- [ ] Deprecation notices are added before removing features

### Refactoring
- [ ] Code is organized for maintainability
- [ ] Duplication is minimized
- [ ] Complex logic is encapsulated
- [ ] Code follows SOLID principles

## Example Integration Checklist for Bridge Classes

```php
/**
 * Example bridge implementation checklist.
 *
 * This is an annotated example of a bridge class showing key quality points.
 */
class ExampleBridge implements ExampleBridgeInterface {

  /**
   * The logger channel.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected $logger;

  /**
   * The optional service.
   *
   * @var \Drupal\example\OptionalServiceInterface|null
   */
  protected $optionalService;

  /**
   * Constructs a new ExampleBridge.
   *
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   The logger factory.
   * @param \Drupal\example\OptionalServiceInterface|null $optional_service
   *   The optional service, or NULL if not available.
   */
  public function __construct(
    LoggerChannelFactoryInterface $logger_factory,
    OptionalServiceInterface $optional_service = NULL
  ) {
    // Consistent logger channel name.
    $this->logger = $logger_factory->get('module_name');
    $this->optionalService = $optional_service;
    
    // Log initialization with appropriate level.
    $this->logger->info('ExampleBridge initialized');
  }

  /**
   * {@inheritdoc}
   */
  public function exampleMethod($required_param, $optional_param = NULL): array {
    try {
      // Validate input.
      if (empty($required_param)) {
        throw new \InvalidArgumentException('Required parameter cannot be empty');
      }
      
      // Use optional service with graceful degradation.
      if ($this->optionalService) {
        $result = $this->optionalService->process($required_param);
        $this->logger->debug('Used optional service for processing');
      }
      else {
        // Fallback implementation when service is not available.
        $result = $this->fallbackImplementation($required_param);
        $this->logger->info('Used fallback implementation');
      }
      
      return [
        'success' => TRUE,
        'data' => $result,
      ];
    }
    catch (\InvalidArgumentException $e) {
      // Log specific exception types with appropriate level.
      $this->logger->warning('Invalid argument: @message', [
        '@message' => $e->getMessage(),
      ]);
      
      // Return graceful error response.
      return [
        'success' => FALSE,
        'error' => $e->getMessage(),
      ];
    }
    catch (\Exception $e) {
      // Log unexpected exceptions.
      $this->logger->error('Unexpected error in exampleMethod: @message', [
        '@message' => $e->getMessage(),
        '@trace' => $e->getTraceAsString(),
      ]);
      
      // Return generic error for unexpected exceptions.
      return [
        'success' => FALSE,
        'error' => 'An unexpected error occurred',
      ];
    }
  }

  /**
   * Fallback implementation when optional service is not available.
   *
   * @param mixed $param
   *   The parameter to process.
   *
   * @return mixed
   *   The processed result.
   */
  protected function fallbackImplementation($param) {
    // Implement fallback logic.
    return $param;
  }

}
```

This comprehensive checklist provides a framework for ensuring high-quality integration between modules. By following these guidelines, developers can create robust, maintainable, and well-documented integrations that gracefully handle various conditions and dependencies.
