---
id: integration-submodule-restructuring
---
# AI Swarm Intelligence Submodule Restructuring Plan

## Current State

The AI Swarm Intelligence module currently includes two legacy submodules that are marked as deprecated:

1. **code_genesis**: Provides automated code generation and scaffolding
   - Focus: Single-agent code generation
   - Marked as deprecated in favor of swarm-based code generation
   - Migration path available via `drush ai-swarm:migrate-code-genesis`

2. **project_intelligence**: Provides project analytics and reporting
   - Focus: Phase-based project analysis
   - Marked as deprecated in favor of swarm-based project analysis
   - Migration path available via `drush ai-swarm:migrate-project-intelligence`

## Recommendation: Merge into `swarm_tools` Submodule

### Rationale

1. **Functional Alignment**: Both submodules provide tools that enhance swarm agent capabilities.
2. **Unified Workflow**: Both are primarily tooling for swarm agents rather than standalone functionality.
3. **Reduced Maintenance**: A single combined submodule reduces maintenance overhead.
4. **Consistent API**: Unified API for all swarm tooling functions.
5. **Better Resource Utilization**: Shared components can be efficiently reused.
6. **Simplified Mental Model**: "Tools for the swarm" rather than separating by output type.

### Implementation Plan

1. **Create New Submodule**:
   ```
   /ai_swarm_intelligence/modules/swarm_tools/
   ```

2. **Migrate Core Functionality**:
   - Move common tools and services to the new submodule
   - Restructure functionality around agent capabilities rather than output types
   - Create unified APIs for both code generation and project analysis
   
3. **Organization Structure**:
   ```
   swarm_tools/
   ├── src/
   │   ├── CodeGen/           # Code generation tools (from code_genesis)
   │   ├── ProjectAnalysis/   # Analysis tools (from project_intelligence)
   │   ├── Common/            # Shared tools used by both
   │   └── Plugin/            # Plugin definitions and managers
   ├── tests/
   ├── config/
   ├── swarm_tools.info.yml
   ├── swarm_tools.services.yml
   └── swarm_tools.module
   ```

4. **Updated Migration Paths**:
   - Update migration commands to point to the new unified module
   - Create backward compatibility layer for existing implementations

## Alternative Option: Rename with Clearer Purpose

If separation needs to be maintained for architectural reasons, rename the modules to better reflect their roles in the swarm ecosystem:

1. **code_genesis → swarm_codegen**
   - Clearer focus on code generation
   - Explicitly ties to swarm architecture
   - More descriptive of actual functionality

2. **project_intelligence → swarm_project_analytics**
   - Better describes the analytics focus
   - Clearly tied to swarm architecture
   - More descriptive and less generic

### Rationale for Separate Modules

- **Specialized Agent Types**: If the agent types are fundamentally different and rarely overlap in capabilities
- **Separate Development Teams**: If different teams maintain the different tool sets
- **Size Considerations**: If the combined module would be too large and unwieldy
- **Deployment Flexibility**: If some deployments need one set of tools but not the other

## Impact Analysis

### Advantages of Merging

1. **Simplified Architecture**: Single unified tooling module
2. **Reduced Cognitive Load**: One set of APIs to learn
3. **Improved Maintenance**: Single codebase to maintain
4. **Better Resource Sharing**: Shared services and utilities
5. **Consistent Updates**: All tools updated simultaneously

### Advantages of Renaming

1. **Clear Purpose**: Names directly describe functionality
2. **Maintained Separation**: Preserves logical separation of concerns
3. **Smaller Modules**: Each module remains focused and smaller
4. **Flexible Deployment**: Can deploy only what's needed
5. **Backward Compatibility**: Easier to maintain compatibility with existing code

## Recommendation

**Proceed with merging into `swarm_tools`** unless there are specific architectural or team reasons to maintain separation. The unified approach better reflects the evolution of the swarm intelligence architecture where tools serve the collective rather than individual agents.

If merging is not feasible due to architectural constraints, proceed with the renaming strategy to better align naming with actual functionality.

## Implementation Timeline

1. **Phase 1** (2 weeks): Create new module structure and API design
2. **Phase 2** (2 weeks): Migrate core functionality and tests
3. **Phase 3** (1 week): Create backward compatibility layer
4. **Phase 4** (1 week): Update documentation and migration paths
5. **Phase 5** (2 weeks): Testing and validation
6. **Phase 6** (Ongoing): Support for legacy modules during transition period

## Migration Support

Documentation for both approaches will be provided to help users transition to the new structure. Migration scripts will be enhanced to support automatic conversion of existing implementations.
