# API Documentation

## Overview

The TDD-AI Model provides comprehensive APIs for test generation, code analysis, and workflow orchestration. All APIs follow OpenAPI 3.1 specifications and support both synchronous and asynchronous operations.

## Available APIs

### 1. TDD-AI Model API
**Endpoint**: `/api/v2`  
**Specification**: [tddai-model.openapi.yaml](../../openapi/tddai-model.openapi.yaml)

#### Core Endpoints
- `POST /generate/test` - Generate AI-powered tests
- `POST /analyze/code` - Analyze code quality and security
- `POST /optimize/test` - Optimize test performance
- `POST /workflow/start` - Start TDD workflow
- `GET /workflow/item/status` - Get workflow status
- `GET /health` - Health check
- `GET /metrics` - Service metrics

### 2. MLflow Integration API  
**Endpoint**: `/api/v2/mlflow`  
**Specification**: [mlflow-integration.openapi.yaml](../../openapi/mlflow-integration.openapi.yaml)

#### MLflow Endpoints
- `POST /experiments` - Create experiment
- `POST /runs` - Start MLflow run
- `POST /models` - Register model
- `PUT /models/Name/versions/1.0.0/stage` - Update model stage
- `POST /deployment/deploy` - Deploy model
- `POST /evaluation/evaluate` - Evaluate model

## Authentication

All APIs use secure authentication:

```javascript
// API Key Authentication
const headers = {
  'X-API-Key': 'your-api-key',
  'Content-Type': 'application/json'
};

// JWT Authentication
const headers = {
  'Authorization': 'Bearer your-jwt-token',
  'Content-Type': 'application/json'
};
```

## Usage Examples

### Generate Tests

```javascript
const response = await fetch('/api/v2/generate/test', {
  method: 'POST',
  headers: {
    'X-API-Key': 'your-api-key',
    'Content-Type': 'application/json'
  },
  body: JSON.stringify({
    code: 'function add(a, b) { return a + b; }',
    language: 'javascript',
    framework: 'jest',
    options: {
      includeEdgeCases: true,
      coverageTarget: 95
    }
  })
});

const result = await response.json();
console.log(result.tests);
```

### Start TDD Workflow

```javascript
const workflow = await fetch('/api/v2/workflow/start', {
  method: 'POST',
  headers: {
    'X-API-Key': 'your-api-key',
    'Content-Type': 'application/json'
  },
  body: JSON.stringify({
    projectId: 'my-project',
    feature: 'user authentication',
    language: 'javascript',
    framework: 'jest'
  })
});

const workflowData = await workflow.json();
console.log('Workflow started:', workflowData.workflowId);
```

## Error Handling

All APIs use consistent error responses:

```json
{
  "error": "ValidationError",
  "message": "Invalid request parameters",
  "code": "INVALID_PARAMS",
  "timestamp": "2024-01-20T10:30:00Z",
  "details": {
    "field": "language",
    "expected": "javascript|typescript|python|php"
  }
}
```

## Rate Limiting

APIs implement rate limiting:
- **Default**: 100 requests per minute
- **Authenticated**: 1000 requests per minute
- **Premium**: 10000 requests per minute

Rate limit headers:
```
X-RateLimit-Limit: 100
X-RateLimit-Remaining: 95
X-RateLimit-Reset: 1705740600
```

## SDKs and Clients

### JavaScript/TypeScript SDK

```bash
npm install @bluefly/tddai-client
```

```javascript
import [TDDAIClient] from '@bluefly/tddai-client';

const client = new TDDAIClient({
  apiKey: 'your-api-key',
  baseURL: 'https://api.bluefly.ai'
});

const tests = await client.generateTests({
  code: 'function multiply(a, b) { return a * b; }',
  language: 'javascript',
  framework: 'jest'
});
```

### Python SDK

```bash
pip install tddai-python-client
```

```python
from tddai_client import TDDAIClient

client = TDDAIClient(
    api_key='your-api-key',
    base_url='https://api.bluefly.ai'
)

tests = client.generate_tests(
    code='def add(a, b): return a + b',
    language='python',
    framework='pytest'
)
```

## OpenAPI Specifications

### Interactive Documentation
- **Swagger UI**: Available at `/api/docs`
- **ReDoc**: Available at `/api/redoc`

### Specification Files
- [TDD-AI Model API](../../openapi/tddai-model.openapi.yaml)
- [MLflow Integration API](../../openapi/mlflow-integration.openapi.yaml)

### Validation Tools

```bash
# Validate OpenAPI specs
spectral lint openapi/*.yaml

# Generate client code
openapi-generator generate -i openapi/tddai-model.openapi.yaml -g typescript-axios -o client/
```

## Monitoring and Observability

### Health Checks
```bash
curl -X GET /api/v2/health
```

Response:
```json
{
  "status": "healthy",
  "version": "2.0.0",
  "uptime": 3600.5,
  "dependencies": {
    "mlflow": "healthy",
    "ollama": "healthy"
  }
}
```

### Metrics
```bash
curl -X GET /api/v2/metrics
```

### Tracing
APIs support distributed tracing with:
- OpenTelemetry
- Jaeger
- Zipkin

## Security

### API Security Features
- TLS 1.3 encryption
- API key rotation
- Rate limiting
- Input validation
- SQL injection prevention
- XSS protection

### Compliance
- GDPR compliant
- SOC 2 Type II
- ISO 27001

## Support

### Documentation
- [OpenAPI Specifications](../../openapi/)
- [SDK Documentation](https://docs.bluefly.ai/sdks)
- [Example Projects](https://github.com/bluefly/tddai-examples)

### Community
- [Discord](https://discord.gg/bluefly)
- [GitHub Discussions](https://github.com/bluefly/tddai/discussions)
- [Stack Overflow](https://stackoverflow.com/questions/tagged/tddai)

---

*Last Updated: January 2024*  
*API Version: v2.0.0*