---
title: tddai-model
description: "GitLab: https://gitlab.bluefly.io/llm/models/tddai-model"
sidebar_position: 1
---

# TDD-AI Core

**Real AI-powered Test-Driven Development assistant** - No mocks, no fakes, genuine AI integration.

[![NPM Version](https://img.shields.io/npm/v/tddai-core.svg)](https://www.npmjs.com/package/tddai-core)
[![License: MIT](https://img.shields.io/badge/License-MIT-yellow.svg)](https://opensource.org/licenses/MIT)
[![Node.js](https://img.shields.io/badge/Node.js-18%2B-green.svg)](https://nodejs.org/)

## 🚀 What Makes This Real

Unlike other "AI-powered" tools that use mocks or fake responses, TDD-AI makes **actual API calls** to:

- ✅ **OpenAI GPT-4** for genuine code analysis and test generation
- ✅ **MLflow REST APIs** for real experiment tracking and model registry
- ✅ **Real file operations** for actual test file creation and code modification

## 🎯 Features

### 🤖 **Real AI Integration**
- **Code Analysis**: GPT-4 powered quality assessment with actionable insights
- **Test Generation**: AI-generated comprehensive test suites (Jest, Vitest, Pytest, PHPUnit)
- **Refactoring Suggestions**: Intelligent code improvement recommendations
- **TDD Workflow**: AI-guided RED-GREEN-REFACTOR cycles

### 📊 **MLflow Integration**
- **Experiment Tracking**: Real metrics logging and parameter tracking
- **Model Registry**: Actual model versioning and deployment management
- **Artifact Storage**: File uploads and retrieval from MLflow servers

### 🛠️ **Developer Tools**
- **CLI Interface**: Feature-rich command-line tool
- **Multi-Language Support**: JavaScript, TypeScript, Python, PHP
- **Framework Support**: Jest, Vitest, Pytest, PHPUnit
- **Real-time Analysis**: AST parsing combined with AI insights

## 📦 Installation

```bash
npm install -g tddai-core
```

## 🔧 Setup

### Required for AI Features
```bash
export OPENAI_API_KEY="your_openai_api_key"
```

### Optional for MLflow Tracking
```bash
# Local MLflow server
pip install mlflow
mlflow server --host 0.0.0.0 --port 5000

# Or use remote MLflow
export MLFLOW_TRACKING_URI="https://your-mlflow-server.com"
```

## 🚀 Quick Start

### Basic Analysis (Free)
```bash
# AST-based code analysis
tddai analyze src/calculator.js

# Template-based test generation  
tddai generate src/calculator.js -f jest
```

### AI-Powered Analysis (Requires OpenAI API Key)
```bash
# Real GPT-4 code analysis
tddai analyze src/calculator.js --ai

# AI-generated intelligent tests
tddai generate src/calculator.js --ai -f jest -c 95

# Interactive TDD workflow
tddai workflow
```

## 📊 Example Output

### Real AI Code Analysis
```bash
$ tddai analyze src/calculator.js --ai
🔍 Analyzing code...
🤖 Using real AI analysis

📊 Analysis Results:
   Quality Score: 87/100
   Complexity: medium (8 functions)
   AI Model: gpt-4
   Tokens Used: 1,247
   Analysis Time: 2,341ms

🚨 Issues Found:
   1. [medium] Consider adding input validation for divide method
   2. [low] Add JSDoc comments for better documentation

💡 AI Suggestions:
   1. Add comprehensive error handling for edge cases
   2. Consider using TypeScript for better type safety
   3. Implement input sanitization for security
```

### Real AI Test Generation
```bash
$ tddai generate src/calculator.js --ai
🤖 Generating intelligent tests...
🤖 Using real AI test generation

✅ Test Generation Complete:
   Tests Generated: 15
   Estimated Coverage: 94%
   AI Model: gpt-4
   Tokens Used: 2,156
   Generation Time: 3,892ms

📝 Tests saved to: src/calculator.test.js
```

## 🔧 CLI Commands

```bash
# Code Analysis
tddai analyze <file> [--ai] [-l language]

# Test Generation  
tddai generate <file> [--ai] [-f framework] [-c coverage]

# Interactive TDD Workflow
tddai workflow

# System Information
tddai info

# Health Check
tddai health

# Initialize New Project
tddai init
```

## 💰 Cost Considerations

### OpenAI API Costs (Real Money)
- **Code Analysis**: ~$0.03-$0.09 per file
- **Test Generation**: ~$0.06-$0.15 per file
- **Complex Files**: Up to ~$0.24 per analysis

### Free Features (No API Key Required)
- Basic AST-based code analysis
- Template-based test generation
- File operations and CLI interface
- Quality scoring and metrics

## 📈 Usage Modes

| Feature | Basic Mode | AI Mode |
|---------|------------|---------|
| **Requirements** | None | OPENAI_API_KEY |
| **Cost** | Free | OpenAI API costs |
| **Code Analysis** | AST + heuristics | GPT-4 analysis |
| **Test Generation** | Templates | AI-generated |
| **Quality Assessment** | Algorithmic | AI-powered |
| **Suggestions** | Rule-based | AI insights |

## 🧪 Programmatic Usage

```javascript
const { TDDAIModel, RealAIService } = require('tddai-core');

// Initialize with real AI
const tddai = new TDDAIModel({
  aiModel: 'gpt-4'
});

// Real AI code analysis
const analysis = await tddai.analyzeCode({
  code: 'function add(a, b) { return a + b; }',
  language: 'javascript',
  filePath: 'calculator.js'
});

console.log(`Quality Score: [analysis.score]/100`);
console.log(`AI Model: [analysis.model]`);
console.log(`Tokens Used: [analysis.tokensUsed]`);

// Real AI test generation
const tests = await tddai.generateTests({
  code: 'function add(a, b) { return a + b; }',
  language: 'javascript',
  framework: 'jest'
});

console.log(`Tests Generated: [tests.tests.length]`);
console.log(`Coverage: [tests.totalCoverage]%`);
```

## 🔍 Verification

To verify this is a real implementation (not fake):

```bash
# Run verification script
npm run verify

# Expected output:
# ✅ Real OpenAI Integration: 5 API calls
# ✅ Real MLflow Integration: 12 REST endpoints  
# ✅ Real HTTP Client: 1 instances
# ✅ Fake Implementations: 0 found
```

## 🛡️ Security

- **API Keys**: Stored in environment variables only
- **No Logging**: API keys never logged or stored in files
- **HTTPS**: All API calls use secure connections
- **Rate Limiting**: Built-in retry logic and error handling

## 📚 Documentation

- [Real Implementation Guide](./REAL_IMPLEMENTATION.md)
- [Mission Accomplished](./MISSION_ACCOMPLISHED.md)
- [Working Features Demo](./WORKING_FEATURES.md)

## 🤝 Contributing

1. Fork the repository
2. Create a feature branch
3. Make your changes
4. Add tests for new functionality
5. Submit a pull request

## 📄 License

MIT License - see [LICENSE](./LICENSE) file for details.

## 🏆 Why TDD-AI is Different

**Other tools**: Mock responses, fake integrations, console.log implementations

**TDD-AI**: Real OpenAI API calls, actual MLflow integration, genuine file operations

**Proof**: Run `npm run verify` and see real authentication errors from actual API attempts.

---

**Built with real AI integration - no bullshit, no vaporware.**