# MLflow Integration for TDD-AI Model Registration

## Overview

The TDD-AI model now supports MLflow client integration for standardized model registration in GitLab's ML model registry. This provides a more robust and feature-rich approach to model management with comprehensive artifact logging and versioning.

## Features

### ✅ **MLflow Client Integration**
- Standardized model registration using MLflow client
- Comprehensive artifact logging and versioning
- Model version staging and transitions
- Rich metadata and tagging support
- GitLab CI/CD pipeline integration

### ✅ **Artifact Management**
- Automatic logging of model files and directories
- Version-specific artifact tracking
- Metadata preservation and documentation
- Model card generation and storage

### ✅ **Version Control**
- Git-based versioning with commit hashes
- CI/CD pipeline integration
- Tag-based version management
- Automated version transitions

## Quick Start

### Local Registration

```bash
# Install Python dependencies
pip install -r requirements.txt

# Register model using MLflow client
npm run model:register:mlflow

# Or with custom token
npm run model:register:mlflow:local
```

### GitLab CI/CD Integration

The MLflow registration is automatically triggered in the GitLab CI/CD pipeline:

```yaml
# Model registration using MLflow client
model_registration_mlflow:
  stage: register
  image: python:3.11-slim
  script:
    - pip install -r requirements.txt
    - python3 scripts/register-model-mlflow.py --token "$CI_JOB_TOKEN"
```

## Configuration

### Environment Variables

```bash
# Required
MLFLOW_TRACKING_URI=https://gitlab.bluefly.io/api/v4/projects/610/ml/mlflow/
MLFLOW_TRACKING_TOKEN=your_gitlab_token

# Optional (for CI/CD)
CI_COMMIT_SHA=git_commit_hash
CI_COMMIT_REF_NAME=git_branch_name
CI_PIPELINE_ID=pipeline_id
```

### Model Configuration

The MLflow registration script uses the following configuration:

```python
CONFIG = {
    'model_name': 'tddai-model',
    'description': 'TDD-AI: AI-powered test-driven development assistant',
    'gitlab_project_id': '610',
    'gitlab_host': 'gitlab.bluefly.io',
    'model_files': [
        'Modelfile',
        'tddai-config.yaml',
        'tddai-integration.yaml',
        'package.json',
        'README.md'
    ],
    'artifact_paths': [
        'docs/',
        'scripts/',
        'prompt-templates/',
        'behavior-injection/'
    ]
}
```

## Usage Examples

### Basic Registration

```python
from scripts.register_model_mlflow import TDDModelRegistry

# Initialize registry
registry = TDDModelRegistry(gitlab_token="your_token")

# Register model
result = registry.register_model()
print(f"Model registered: [result['model_name']] v[result['model_version']]")
```

### Command Line Usage

```bash
# Show help
python3 scripts/register-model-mlflow.py --help

# Register with custom token
python3 scripts/register-model-mlflow.py --token $GITLAB_TOKEN

# Register with custom stage
python3 scripts/register-model-mlflow.py --stage Production
```

### NPM Scripts

```bash
# Standard MLflow registration
npm run model:register:mlflow

# Local registration with token
npm run model:register:mlflow:local

# Check registration status
npm run model:status
```

## Artifact Logging

### Files Logged

The MLflow registration automatically logs the following artifacts:

- **Model Files**: Modelfile, configuration files, package.json
- **Documentation**: README.md, docs/ directory
- **Scripts**: All registration and deployment scripts
- **Templates**: Prompt templates and behavior injection files
- **Metadata**: Model metadata and registration summary

### Artifact Structure

```
model-version/
├── Modelfile
├── tddai-config.yaml
├── tddai-integration.yaml
├── package.json
├── README.md
├── docs/
│   ├── deployment-complete.md
│   ├── mlflow-integration.md
│   └── ...
├── scripts/
│   ├── register-model-mlflow.py
│   ├── deploy-model.js
│   └── ...
├── prompt-templates/
├── behavior-injection/
└── model-metadata.json
```

## Model Versioning

### Version Sources

The MLflow registration uses the following priority for versioning:

1. **Git Tags**: If a git tag exists, use it as version
2. **Git Commit Hash**: Use short commit hash (8 characters)
3. **CI Commit SHA**: Use CI/CD commit SHA
4. **Fallback**: Use 'latest' as version

### Version Tags

Each model version includes comprehensive tags:

```python
tags = {
    "gitlab.version": version,
    "gitlab.commit_hash": commit_hash,
    "gitlab.git_tag": git_tag,
    "gitlab.ci_commit_sha": ci_commit_sha,
    "gitlab.ci_commit_ref": ci_commit_ref,
    "gitlab.ci_pipeline_id": ci_pipeline_id,
    "model.type": "llm",
    "model.framework": "ollama",
    "model.architecture": "codellama",
    "tddai.version": "1.0.0",
    "created_at": timestamp
}
```

## Model Staging

### Stage Transitions

The MLflow registration automatically transitions model versions through stages:

1. **None**: Initial state
2. **Staging**: After successful registration
3. **Production**: After deployment validation
4. **Archived**: For deprecated versions

### Stage Management

```python
# Transition to staging
registry.transition_model_version(model_version, "Staging")

# Transition to production
registry.transition_model_version(model_version, "Production")

# Archive old version
registry.transition_model_version(model_version, "Archived")
```

## Error Handling

### Common Issues

1. **Authentication Errors**
   ```bash
   # Ensure GitLab token is valid
   export GITLAB_TOKEN="your_valid_token"
   ```

2. **Network Connectivity**
   ```bash
   # Check MLflow tracking URI
   curl -H "PRIVATE-TOKEN: $GITLAB_TOKEN" \
     "https://gitlab.bluefly.io/api/v4/projects/610/ml/mlflow/"
   ```

3. **Missing Dependencies**
   ```bash
   # Install Python dependencies
   pip install -r requirements.txt
   ```

### Debugging

```bash
# Enable verbose logging
export MLFLOW_VERBOSE=1

# Run with debug output
python3 scripts/register-model-mlflow.py --debug
```

## Integration with GitLab CI/CD

### Pipeline Integration

The MLflow registration is integrated into the GitLab CI/CD pipeline:

```yaml
stages:
  - validate
  - register  # MLflow registration happens here
  - test
  - deploy
```

### Artifacts

The pipeline generates the following artifacts:

- `mlflow-registration-report.md`: Registration summary
- `registration-summary.json`: JSON registration details
- `model-metadata.json`: Model metadata

### Environment Variables

The pipeline automatically sets required environment variables:

```yaml
variables:
  MLFLOW_TRACKING_URI: "https://gitlab.bluefly.io/api/v4/projects/$CI_PROJECT_ID/ml/mlflow/"
  MLFLOW_TRACKING_TOKEN: "$CI_JOB_TOKEN"
```

## Best Practices

### 1. **Version Management**
- Use semantic versioning for releases
- Tag important commits with version numbers
- Maintain version history in GitLab

### 2. **Artifact Organization**
- Keep model files organized and documented
- Include comprehensive metadata
- Log all relevant artifacts

### 3. **Security**
- Use secure tokens for authentication
- Validate model artifacts before registration
- Monitor registry access and usage

### 4. **Monitoring**
- Track model version performance
- Monitor artifact storage usage
- Validate model deployments

## Troubleshooting

### Registration Failures

```bash
# Check MLflow client setup
python3 -c "from mlflow import MlflowClient; print('MLflow OK')"

# Verify GitLab token
curl -H "PRIVATE-TOKEN: $GITLAB_TOKEN" \
  "https://gitlab.bluefly.io/api/v4/user"

# Test MLflow tracking URI
curl -H "PRIVATE-TOKEN: $GITLAB_TOKEN" \
  "$MLFLOW_TRACKING_URI"
```

### Artifact Issues

```bash
# Check file permissions
ls -la Modelfile tddai-config.yaml

# Verify file existence
find . -name "*.yaml" -o -name "*.json" -o -name "*.md"
```

### Version Conflicts

```bash
# Check existing versions
python3 -c "
from mlflow import MlflowClient
client = MlflowClient()
versions = client.search_model_versions('name = \"tddai-model\"')
for v in versions:
    print(f'Version: [v.version], Stage: [v.current_stage]')
"
```

## API Reference

### TDDModelRegistry Class

```python
class TDDModelRegistry:
    def __init__(self, gitlab_token=None)
    def get_version_info(self)
    def create_model(self)
    def create_model_version(self, version_info)
    def log_artifacts(self, model_version)
    def transition_model_version(self, model_version, stage)
    def register_model(self)
```

### Command Line Options

```bash
python3 scripts/register-model-mlflow.py [options]

Options:
    --help              Show help message
    --token TOKEN       GitLab token for authentication
    --stage STAGE       Target stage for model version
    --dry-run           Show what would be done without executing
```

## Conclusion

The MLflow integration provides a robust, standardized approach to TDD-AI model registration with comprehensive artifact management, versioning, and GitLab CI/CD integration. This ensures reliable model deployment and management in production environments.

For more information, see the [deployment-complete.md](deployment-complete.md) documentation. 