---
id: modules-development-drupal_migration_plus-CLAUDE
---
# CLAUDE.md

This file provides guidance to Claude Code (claude.ai/code) when working with code in this repository.

## Project Goals

**Primary Objective**: Perform a comprehensive audit of the existing Acquia Migrate Accelerate tool and rebuild it as a modern, self-healing Drupal 11 migration platform that can migrate content from Drupal 7, WordPress, and other open-source CMS platforms to Drupal 11 using best practices and modern Drupal recipes.

**Key Requirements**:

- Full Drupal 11 API compatibility with modern architecture patterns
- Multi-CMS source support (Drupal 7, WordPress, Joomla, etc.)
- Self-healing capabilities to adapt to new modules and deprecations
- Drupal.org contrib module standards compliance
- AI-powered migration intelligence using Drupal AI module
- GitLab CI/CD integration with model registry support
- Custom recipe generation for migration patterns
- Extensible submodule architecture

## Development Commands

### Backend (PHP)

```bash
# Install dependencies
composer install

# Linting and code quality
composer run-script cs                     # Run PHP CodeSniffer
composer run-script cs:fix                 # Fix code style issues
composer run-script stan                   # Run PHPStan static analysis
composer run-script rector                 # Run Rector analysis (dry run)
composer run-script rector:fix             # Apply Rector fixes
composer run-script normalize              # Format composer.json
composer run-script quality                # Run all quality checks (cs, stan, test:unit)
composer run-script quality:fix            # Run all fixers (cs:fix, rector:fix)

# Testing
composer run-script test                   # Run all tests
composer run-script test:unit              # Run unit tests only
composer run-script test:kernel            # Run kernel tests only
composer run-script test:functional        # Run functional tests only
composer run-script test:coverage          # Generate test coverage report

# Run a single test class
./vendor/bin/phpunit tests/src/Unit/Service/FieldMapperTest.php

# Run a specific test method
./vendor/bin/phpunit --filter testMethodName tests/src/Unit/Service/FieldMapperTest.php
```

### Frontend (UI)

```bash
# Root level markdownlint
npm run lint:md                  # Lint all markdown files
```

### Nightwatch Tests

```bash
# Run Nightwatch e2e tests (requires Chrome)
npx nightwatch                   # Run all Nightwatch tests
npx nightwatch tests/src/Nightwatch/01_source_discovery.js  # Run a specific test file
```

## Architecture

### Core Concepts

**Migration vs Migration Plugin**: The module distinguishes between "migrations" (user-facing clusters of related migration tasks) and "migration plugins" (individual Drupal core migration plugins). One migration contains multiple migration plugins.

**Migration Clustering**: The heart of the system is `MigrationClusterer` which groups Drupal's numerous migration plugins into meaningful "site builder" concepts using heuristics.

**Source Adapters**: The system includes adapters for various source systems (Drupal 7, WordPress, Joomla, etc.) that provide normalized access to source data.

**Recipe System**: The module includes a recipe-based approach to migration, with AI-powered recipe generation and execution.

**Parallel Processing**: A sophisticated parallel processing system to execute migrations at scale with workload partitioning and resource monitoring.

### Key Components

**Migration Repository** (`src/Service/MigrationRepository.php`):
- Central repository for all migrations
- Handles caching, dependency resolution, and migration status tracking

**MigrationClusterer** (`src/Service/MigrationClusterer.php`):
- Groups migration plugins into logical clusters using heuristics
- Two phases: get available plugins, then apply clustering heuristics
- Heuristics are in `src/Heuristic/` and determine cluster assignment

**SourceAdapterManager** (`src/Service/SourceAdapterManager.php`):
- Manages and discovers source adapters
- Provides factory methods for creating adapters

**RecipeManager** (`src/Service/RecipeManager.php`):
- Manages migration recipes
- Handles recipe generation, validation, and execution

**ParallelMigrationExecutor** (`src/Service/ParallelMigrationExecutor.php`):
- Coordinates parallel execution of migrations
- Manages worker processes and resource allocation

**AI Integration Services**:
- `AiFieldMappingService`: Intelligently maps source fields to destination fields
- `MultiModalAiOrchestrator`: Coordinates AI model usage for various tasks
- `VisualAiThemeParser`: Analyzes and migrates visual themes using AI

### Database Schema

- Migration tables track migration completion, timing, fingerprints
- Distributed queue tables for parallel processing

### Migration Process Flow

1. **Source Discovery**: Source adapters discover available content
2. **Analysis**: AI systems analyze content and suggest mappings
3. **Recipe Generation**: Migration recipes are generated based on analysis
4. **Validation**: Recipes are validated for consistency and completeness
5. **Execution**: Migrations are executed, potentially in parallel
6. **Verification**: Results are verified for correctness

### Submodule Organization

The module includes several submodules for specific functionalities:

- `drupal_migration_plus_wordpress`: WordPress-specific adapters and mappers
- `drupal_migration_plus_ai`: AI integration services
- `drupal_migration_plus_experience_builder`: Experience Builder integration
- `drupal_migration_plus_themes`: Theme migration tools
- Additional submodules for other CMS platforms (Joomla, AEM, Sitecore, etc.)

### Testing Organization

- `tests/src/Unit/`: Unit tests for isolated components
- `tests/src/Kernel/`: Kernel tests for integrated components with Drupal
- `tests/src/Functional/`: Functional tests for HTTP and basic browser testing
- `tests/src/FunctionalJavascript/`: Tests requiring JavaScript
- `tests/src/Nightwatch/`: End-to-end tests using Nightwatch.js

## Development Workflows

### Adding a New Source Adapter

1. Create a class in `src/SourceAdapter/` implementing `SourceAdapterInterface`
2. Register the adapter in the appropriate service provider
3. Implement discovery and data extraction methods
4. Add corresponding unit tests in `tests/src/Unit/SourceAdapter/`

### Adding a New Process Plugin

1. Create a class in `src/Plugin/migrate/process/`
2. Implement the transform method
3. Add unit tests in `tests/src/Unit/Plugin/migrate/process/`

### Adding a New Heuristic

1. Create a class in `src/Heuristic/` implementing `HeuristicInterface`
2. Register the heuristic in the appropriate service provider
3. Implement the applies() and getClusters() methods
4. Add unit tests in `tests/src/Unit/Heuristic/`

### Working with Parallel Processing

The module includes a sophisticated parallel processing system for scaling migrations:

1. Use `WorkloadPartitioner` to divide migration tasks
2. Use `DistributedQueueManager` to manage task queues
3. Use `ParallelMigrationExecutor` to execute tasks in parallel
4. Use `ResourceMonitor` to prevent system overload

Refer to `docs/PARALLEL_MIGRATION_README.md` for detailed usage. 
