---
id: modules-development-drupal_migration_plus-DRUPAL_TO_DRUPAL_MIGRATION
---
# Drupal-to-Drupal Migration Features

This document provides a comprehensive guide to the Drupal-to-Drupal migration capabilities of the Drupal Migration Plus module, with a specific focus on the Experience Builder integration for field mapping.

## Table of Contents

- [Overview](#overview)
- [Key Features](#key-features)
- [Architecture](#architecture)
- [Field Mapping Interface](#field-mapping-interface)
- [Field Type Compatibility](#field-type-compatibility)
- [API Documentation](#api-documentation)
- [ECA Workflow Integration](#eca-workflow-integration)
- [Advanced Use Cases](#advanced-use-cases)
- [Troubleshooting](#troubleshooting)

## Overview

The Drupal-to-Drupal migration features in Drupal Migration Plus provide a specialized system for migrating content between different Drupal installations. This system leverages the Experience Builder interface to create an intuitive visual field mapping tool, combined with intelligent field type detection, validation, and transformation capabilities.

The system supports migrations between:
- Drupal 7 to Drupal 9/10/11
- Drupal 8 to Drupal 9/10/11
- Drupal 9 to Drupal 9/10/11

## Key Features

- **Visual Field Mapping Interface**: Drag-and-drop interface for mapping fields between source and target Drupal sites
- **Intelligent Field Type Detection**: Automatic detection and suggestion of compatible field types
- **Field Transformation**: Advanced transformation capabilities for complex field types
- **Real-time Validation**: Immediate feedback on field mapping compatibility
- **Preview Capabilities**: Live preview of migration results
- **ECA Workflow Integration**: Automated workflows for migration tasks
- **Special Field Type Handling**: Enhanced support for complex field types like entity references, field collections, and file fields

## Architecture

The Drupal-to-Drupal migration system is built on the following architectural components:

### Core Components

1. **Field Mapping Interface**: React-based component integrated with Experience Builder
2. **Field Mapping Controllers**: API endpoints for field analysis and validation
3. **Field Type Conversion Service**: Handles field type detection and conversion
4. **Field Mapping Validation Service**: Provides real-time validation
5. **ECA Integration**: Event subscribers and workflow engine for automation

### Component Relationships

```
┌───────────────────────┐
│ Experience Builder UI │
└───────────┬───────────┘
            │
┌───────────▼───────────┐
│ Field Mapping API     │
│ Controllers           │
└───────────┬───────────┘
            │
┌───────────▼───────────┐
│ Field Mapping Services│
├───────────────────────┤
│ - Type Detection      │
│ - Validation          │
│ - Transformation      │
└───────────┬───────────┘
            │
┌───────────▼───────────┐
│ ECA Workflow Engine   │
└───────────┬───────────┘
            │
┌───────────▼───────────┐
│ Migration Execution   │
└───────────────────────┘
```

## Field Mapping Interface

The field mapping interface provides an intuitive visual environment for mapping fields between source and target Drupal sites.

### Accessing the Interface

1. Navigate to **Content → Migration → Drupal Migration**
2. Select the source Drupal site
3. Choose the content types to migrate
4. Click on "Field Mapping" to open the interface

### Using the Interface

The interface consists of the following components:

- **Source Fields Panel**: Displays available fields from the source site
- **Target Fields Panel**: Shows fields in the target content type
- **Mapping Canvas**: Visual area for creating field mappings
- **Validation Panel**: Real-time feedback on mapping validity
- **Transformation Controls**: Options for field transformation
- **Preview Panel**: Live preview of migration results

To create a mapping:
1. Drag a field from the source panel
2. Drop it onto a compatible field in the target panel
3. Configure any transformation options
4. Review the validation feedback
5. Save the mapping

## Field Type Compatibility

The system includes a comprehensive field type compatibility matrix to determine how fields can be mapped between different Drupal versions.

### Compatibility Matrix

Below is a simplified compatibility matrix for common field types:

| Source Field Type (D7) | Compatible Target Types (D9+) | Compatibility Score |
|------------------------|-------------------------------|---------------------|
| text                   | string, text, text_long       | 0.8 - 0.95          |
| varchar                | string, text                  | 0.8 - 0.95          |
| int                    | integer, number, list_integer | 0.85 - 0.95         |
| float                  | float, decimal, number        | 0.85 - 0.95         |
| node_reference         | entity_reference              | 0.9                 |
| file                   | file, image                   | 0.85 - 0.9          |
| image                  | image                         | 0.95                |
| taxonomy_term_reference| entity_reference              | 0.9                 |
| list                   | list_string, list_text        | 0.85 - 0.9          |
| list_number            | list_integer, list_float      | 0.85 - 0.9          |
| field_collection       | entity_reference_revisions    | 0.7                 |

### Special Field Type Handling

The system provides specialized handling for complex field types:

#### Entity References
- Automatic detection of reference target entity types
- Intelligent mapping of reference fields
- Support for migrating referenced entities

#### Field Collections (D7) to Paragraphs (D9+)
- Comprehensive transformation of field collections to paragraphs
- Field-by-field mapping within collection items
- Support for nested collections

#### File and Image Fields
- Automated file transfer
- Support for alt and title attributes
- Handling of file directory structures

## API Documentation

The Drupal-to-Drupal migration system exposes several API endpoints for integration with custom code and external systems.

### Field Analysis Endpoint

- **URL**: `/api/migration/field-mapping/drupal-to-drupal/analyze`
- **Method**: POST
- **Request Body**:
  ```json
  {
    "source_fields": [
      {
        "name": "field_example",
        "type": "text",
        "settings": { ... }
      }
    ],
    "target_entity_type": "node",
    "target_bundle": "article",
    "source_drupal_version": "7",
    "options": { ... }
  }
  ```
- **Response**:
  ```json
  {
    "direct_mappings": [ ... ],
    "field_transformations": [ ... ],
    "compatibility_analysis": [ ... ]
  }
  ```

### Validation Endpoint

- **URL**: `/api/migration/field-mapping/drupal-to-drupal/validate`
- **Method**: POST
- **Request Body**:
  ```json
  {
    "mappings": [
      {
        "source_field": "field_example",
        "target_field": "field_target",
        "transformation": "direct"
      }
    ],
    "target_entity_type": "node",
    "target_bundle": "article",
    "source_drupal_version": "7"
  }
  ```
- **Response**:
  ```json
  {
    "valid": true,
    "errors": [],
    "warnings": [],
    "transformation_notes": [],
    "field_specific_validations": [],
    "improvement_suggestions": []
  }
  ```

### Preview Endpoint

- **URL**: `/api/migration/field-mapping/drupal-to-drupal/preview`
- **Method**: POST
- **Request Body**:
  ```json
  {
    "mappings": [ ... ],
    "source_entity_type": "node",
    "source_bundle": "article",
    "source_entity_id": 123,
    "target_entity_type": "node",
    "target_bundle": "article",
    "source_drupal_version": "7"
  }
  ```
- **Response**:
  ```json
  {
    "preview": {
      "source_data": { ... },
      "transformed_data": { ... },
      "target_data": { ... }
    }
  }
  ```

## ECA Workflow Integration

The Drupal-to-Drupal migration system integrates with the Event-Condition-Action (ECA) module to provide automated workflows for migration tasks.

### Available Events

- `drupal_migration_plus.mcp.pre_migration`: Fired before a migration begins
- `drupal_migration_plus.mcp.post_migration`: Fired after a migration completes
- `drupal_migration_plus.mcp.entity_imported`: Fired when an entity is imported
- `drupal_migration_plus.field_mapping.analyzed`: Fired when field mapping analysis completes
- `drupal_migration_plus.field_mapping.validated`: Fired when field mapping validation completes
- `drupal_migration_plus.field_mapping.created`: Fired when a field mapping is created or updated

### Example Workflow

Below is an example of an ECA workflow for validating field mappings:

1. **Event**: `drupal_migration_plus.field_mapping.created`
2. **Condition**: Check if mappings contain complex field types
3. **Action**: Validate mappings and send notification if issues are found

### Creating Custom Workflows

Custom ECA workflows can be created through the ECA UI or in code:

1. Navigate to **Configuration → ECA → Workflows**
2. Click "Add Workflow"
3. Select a Drupal-to-Drupal migration event
4. Configure conditions and actions
5. Save the workflow

## Advanced Use Cases

### Handling Content Type Changes

When migrating between different content type structures:

1. Use the Field Mapping interface to create mapping templates
2. Apply transformations to adapt field data
3. Configure entity reference fields to maintain relationships
4. Use ECA workflows to execute post-migration cleanup

### Multi-site Migration

For migrating content from multiple Drupal sites:

1. Configure source connections for each site
2. Create separate migration configurations
3. Use the batch processing capabilities
4. Leverage ECA workflows for cross-site coordination

### Complex Field Transformations

For complex transformations like field collections to paragraphs:

1. Use the specialized transformation options in the interface
2. Configure nested field mappings
3. Preview the transformation results
4. Adjust settings as needed

## Troubleshooting

### Common Issues

#### Missing Field Mappings
- **Issue**: Some fields are not appearing in the mapping interface
- **Solution**: Verify field visibility settings in the source and target sites

#### Transformation Errors
- **Issue**: Field transformation fails during migration
- **Solution**: Check field settings compatibility and review validation warnings

#### Performance Issues
- **Issue**: Slow performance with large datasets
- **Solution**: Use batch processing and optimize field transformations

### Debugging

The system provides detailed logging for troubleshooting:

- Check Drupal logs under **Reports → Recent Log Messages**
- Enable debug mode in the module settings
- Review ECA workflow execution logs

### Getting Help

If you encounter issues not covered in this documentation:

- Review the issue queue on Drupal.org
- Join the Drupal Migration community channel
- Contact the module maintainers
