---
id: modules-development-drupal_migration_plus-PARALLEL_MIGRATION_README
---
# Parallel Migration Processing

## Overview

Drupal Migration Plus includes a high-performance parallel processing system for executing migrations at scale. This system enables distributing migration workloads across multiple processes to maximize throughput and efficiently utilize system resources.

## Features

- **Intelligent workload partitioning** that respects entity dependencies
- **Distributed task queue** for coordinated parallel processing
- **Multi-process execution** with dynamic worker management
- **Real-time monitoring** and progress reporting
- **Automatic error handling** with configurable retry mechanisms
- **Resource-aware scheduling** to prevent system overload
- **Cross-process synchronization** for data consistency

## Architecture

The parallel processing system consists of several integrated components:

1. **WorkloadPartitioner**: Analyzes migration tasks and divides them into balanced partitions based on dependencies and entity counts.

2. **DistributedQueueManager**: Manages task queues with priority-based scheduling and failure recovery.

3. **ParallelMigrationExecutor**: Coordinates the execution of tasks across multiple worker processes.

4. **Worker Script**: Standalone PHP script that continuously processes tasks from the queue.

5. **Monitoring System**: Tracks progress, resource usage, and worker health.

## Usage

### Through the UI

1. Navigate to **Admin → Configuration → Migration Plus → Parallel Settings** to configure system behavior.

2. In the Migration Dashboard, select migrations and click **Parallel Execute** to start a parallel migration.

3. Monitor progress in the Migration Dashboard's Parallel Execution tab.

### Using the API

```php
// Prepare a parallel migration
$executor = \Drupal::service('drupal_migration_plus.parallel_migration_executor');
$result = $executor->prepareMigration(['node:article', 'user:user']);

// Start the migration
$executor->executeMigration(4); // 4 worker processes

// Check status
$status = $executor->getStatus();

// Stop the migration
$executor->stopMigration();
```

### REST API

Parallel migrations can also be controlled through a REST API:

- `POST /admin/migrate/plus/api/parallel/prepare` - Prepare a migration
- `POST /admin/migrate/plus/api/parallel/start` - Start execution
- `POST /admin/migrate/plus/api/parallel/stop` - Stop execution
- `GET /admin/migrate/plus/api/parallel/status` - Get execution status
- `GET /admin/migrate/plus/api/parallel/queue-status` - Get queue details
- `POST /admin/migrate/plus/api/parallel/reset` - Reset a failed migration

## Configuration

### Worker Processes

The number of worker processes determines how many migrations can be executed in parallel. This should generally be set to:

- 1-2 workers per CPU core for optimal performance
- Lower if the system has limited memory
- Higher for I/O bound migrations on systems with fast storage

### Memory Management

The system actively monitors memory usage and will throttle worker processes if memory usage exceeds the configured threshold. Memory usage can be managed by:

- Setting a lower maximum memory percentage (e.g., 70%)
- Reducing the entity batch size
- Ensuring other system services have sufficient memory reserved

### Batch Size

The batch size controls how many entities are processed in a single operation:

- Larger batches are more efficient but use more memory
- Smaller batches use less memory but have higher overhead
- The optimal size depends on the complexity of your entities

### Error Handling

Failed tasks are automatically retried based on the configured retry limit:

- Set a lower retry limit for faster failure detection
- Set a higher limit for better resilience against transient failures
- Failed tasks can be manually reset through the UI or API

## Performance Considerations

### Database Connections

Each worker process uses its own database connection pool. Ensure your database server is configured to handle multiple simultaneous connections (typically 5 per worker).

### File System

For migrations with file operations, ensure the file system can handle concurrent access. Network file systems may become a bottleneck.

### Entity Caching

When migrating related entities, ensure entity caching is configured correctly to prevent memory exhaustion from entity cache buildup.

## Monitoring and Troubleshooting

### Worker Status

Worker processes report their status regularly, which can be viewed in the Migration Dashboard. If a worker has not reported activity for longer than the configured timeout, it will be considered stalled.

### Resource Usage

The system logs resource usage metrics to help identify bottlenecks:

- CPU usage per worker
- Memory consumption
- Database query counts and durations
- File operations

### Common Issues

1. **Workers terminating prematurely**: Check for PHP errors in the server logs and ensure memory_limit is set high enough.

2. **Slow performance**: Check if database is the bottleneck (look for high query times) or if file operations are causing delays.

3. **Memory exhaustion**: Reduce batch size or maximum worker count, and ensure entity caching is properly configured.

4. **Database connection errors**: Increase max_connections in your database configuration.

## Advanced Usage

### Custom Worker Scripts

You can create custom worker scripts for special processing needs by extending the base worker script and configuring it in the parallel settings.

### External Process Management

For very large migrations, the system can be integrated with external process managers like Supervisor or systemd for improved reliability and restart capabilities.

### Distributed Execution

For extremely large migrations, the system can be configured to distribute work across multiple servers by using shared queue storage and appropriate network configurations.
