---
id: modules-development-drupal_migration_plus-TESTING_STANDARDS
---
# Testing Standards

This document outlines the testing standards and practices for the Drupal Migration Plus module.

## Directory Structure

The module follows the standard Drupal testing directory structure:

```
drupal_migration_plus/
├── tests/
│   ├── src/
│   │   ├── Functional/       # Functional tests
│   │   ├── FunctionalJavascript/ # JavaScript-based functional tests
│   │   ├── Kernel/           # Kernel tests
│   │   └── Unit/             # Unit tests
│   └── fixtures/             # Test fixtures
└── __tests__/                # Migration path for old test structure
```

## Test Types

### Unit Tests

Unit tests verify individual components in isolation:

- Test individual methods and functions
- Mock dependencies
- Focus on logic and behavior
- Fast execution

Example:
```php
namespace Drupal\Tests\drupal_migration_plus\Unit;

use Drupal\Tests\UnitTestCase;
use Drupal\drupal_migration_plus\Service\FieldMapper;

/**
 * @coversDefaultClass \Drupal\drupal_migration_plus\Service\FieldMapper
 * @group drupal_migration_plus
 */
class FieldMapperTest extends UnitTestCase {

  /**
   * @covers ::mapField
   */
  public function testMapField() {
    // Test implementation.
  }

}
```

### Kernel Tests

Kernel tests verify integration between components:

- Test integration between services
- Include database operations
- Use the Drupal kernel
- More comprehensive than unit tests

Example:
```php
namespace Drupal\Tests\drupal_migration_plus\Kernel;

use Drupal\KernelTests\KernelTestBase;

/**
 * Tests the integration between migration components.
 *
 * @group drupal_migration_plus
 */
class MigrationIntegrationTest extends KernelTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'system',
    'user',
    'migrate',
    'migrate_drupal',
    'migrate_plus',
    'migrate_tools',
    'drupal_migration_plus',
  ];

  // Test implementation.
}
```

### Functional Tests

Functional tests verify features through the UI:

- Test full features
- Include HTTP requests
- Simulate user interactions
- Verify rendered output

Example:
```php
namespace Drupal\Tests\drupal_migration_plus\Functional;

use Drupal\Tests\BrowserTestBase;

/**
 * Tests the Drupal Migration Plus administrative UI.
 *
 * @group drupal_migration_plus
 */
class AdminUITest extends BrowserTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'system',
    'user',
    'migrate',
    'migrate_drupal',
    'migrate_plus',
    'migrate_tools',
    'drupal_migration_plus',
  ];

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  // Test implementation.
}
```

### JavaScript Functional Tests

JavaScript functional tests verify JavaScript behaviors:

- Test JavaScript interactions
- Simulate clicks and user input
- Verify dynamic UI changes
- Use headless browsers

Example:
```php
namespace Drupal\Tests\drupal_migration_plus\FunctionalJavascript;

use Drupal\FunctionalJavascriptTests\WebDriverTestBase;

/**
 * Tests the JavaScript-based features of the module.
 *
 * @group drupal_migration_plus
 */
class FieldMappingUITest extends WebDriverTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'system',
    'user',
    'migrate',
    'migrate_drupal',
    'migrate_plus',
    'migrate_tools',
    'drupal_migration_plus',
  ];

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  // Test implementation.
}
```

## Test Groups

All tests must use the `@group drupal_migration_plus` annotation to enable selective test execution.

Additional groups for specific components:
- `@group drupal_migration_plus_source`
- `@group drupal_migration_plus_process`
- `@group drupal_migration_plus_destination`
- `@group drupal_migration_plus_workflow`
- `@group drupal_migration_plus_ui`

## Running Tests

### Using PHPUnit

```bash
# Run all module tests
phpunit --group=drupal_migration_plus

# Run specific test types
phpunit --group=drupal_migration_plus --testsuite=unit
phpunit --group=drupal_migration_plus --testsuite=kernel
phpunit --group=drupal_migration_plus --testsuite=functional
phpunit --group=drupal_migration_plus_source

# Run with coverage
phpunit --group=drupal_migration_plus --coverage-html=coverage
```

### Using Drupal's Run-tests Script

```bash
# Run all module tests
php core/scripts/run-tests.sh --module=drupal_migration_plus

# Run specific test types
php core/scripts/run-tests.sh --module=drupal_migration_plus --types=PHPUnit-Unit
php core/scripts/run-tests.sh --module=drupal_migration_plus --types=PHPUnit-Kernel
php core/scripts/run-tests.sh --module=drupal_migration_plus --types=PHPUnit-Functional
```

### Using Drush

```bash
# Run all module tests
drush test:run drupal_migration_plus

# Run specific test classes
drush test:run "Drupal\Tests\drupal_migration_plus\Unit\FieldMapperTest"
```

## Test Fixtures

Place test fixtures in the `tests/fixtures` directory:

```
tests/fixtures/
├── sources/
│   ├── drupal7/
│   │   ├── drupal7_source.sql  # Sample Drupal 7 database
│   │   └── node_types.php      # Sample node type definitions
│   ├── wordpress/
│   │   ├── wordpress_source.sql # Sample WordPress database
│   │   └── post_types.php       # Sample post type definitions
│   └── joomla/
│       ├── joomla_source.sql    # Sample Joomla database
│       └── content_types.php    # Sample content type definitions
├── destinations/
│   ├── fields/
│   │   ├── field_mappings.yml   # Sample field mappings
│   │   └── field_types.yml      # Sample field type definitions
│   └── entities/
│       ├── entity_mappings.yml  # Sample entity mappings
│       └── entity_types.yml     # Sample entity type definitions
└── configs/
    ├── migration_recipes/       # Sample migration recipes
    ├── test_migration.yml       # Sample migration configuration
    └── test_mapping.yml         # Sample mapping configuration
```

## Migration-Specific Testing Considerations

### Testing Migrations

For testing full migrations:

1. Use the MigrateDumper to create fixture databases
2. Test both source plugins and process plugins separately
3. Test the full migration process with small datasets
4. Verify that rollbacks work correctly

### Testing Source Plugins

Example:
```php
/**
 * Tests WordPress source plugin.
 */
public function testWordPressSource() {
  $source = $this->createSourcePlugin('wordpress_posts');
  $this->assertEquals(10, $source->count());
  
  // Test field extraction.
  $row = $source->current();
  $this->assertEquals('Sample Post', $row->getSourceProperty('post_title'));
}
```

### Testing Process Plugins

Example:
```php
/**
 * Tests field splitting process plugin.
 */
public function testFieldSplitPlugin() {
  $plugin = $this->createProcessPlugin('field_split', [
    'delimiter' => ',',
    'source' => 'tags',
  ]);
  
  $value = $plugin->transform('tag1,tag2,tag3', $this->migrateExecutable, $this->row, 'tags');
  $this->assertEquals(['tag1', 'tag2', 'tag3'], $value);
}
```

## Coding Standards for Tests

- Follow PSR-4 autoloading standards
- Use camelCase for method names (e.g., `testMapField()`)
- Use snake_case for data provider methods (e.g., `provide_test_cases()`)
- Include descriptive doc blocks with `@covers` annotations for unit tests
- Group assertions by logical concepts
- Use descriptive assertion messages

## Test Coverage Requirements

- Minimum 85% code coverage for Unit tests
- All public service methods must have unit tests
- All forms must have functional tests
- All JavaScript behaviors must have JavaScript functional tests
- All migration plugins must have kernel or functional tests

## Best Practices

1. **Isolation**: Tests should not depend on each other
2. **Speed**: Optimize tests for speed, especially unit tests
3. **Readability**: Write clear test methods with descriptive names
4. **Data Providers**: Use data providers for testing multiple inputs
5. **Mocking**: Use mocks and stubs for dependencies in unit tests
6. **Cleanup**: Clean up all test artifacts
7. **Database Isolation**: Use transactions to isolate database tests
8. **Migration-Specific**: Always test rollbacks as well as imports

## Continuous Integration

- Tests are automatically run on every commit
- Code coverage reports are generated
- Tests must pass before merge
- Coverage must not decrease
