---
id: modules-development-drupal_migration_plus-TROUBLESHOOTING
---
# Troubleshooting Guide

## Common Issues and Solutions

### Installation Issues

#### Module Not Found After Installation

**Problem**: Module appears installed but doesn't show up in module list

**Symptoms**:
- Module not visible in `/admin/modules`
- "Module not found" errors
- Missing service definitions

**Solutions**:
1. Clear all caches:
```bash
drush cr
drush cc all
```

2. Rebuild module registry:
```bash
drush php-eval "system_rebuild_module_data();"
```

3. Check file permissions:
```bash
find modules/contrib/drupal_migration_plus -type d -exec chmod 755 {} \;
find modules/contrib/drupal_migration_plus -type f -exec chmod 644 {} \;
```

4. Verify composer autoloader:
```bash
composer dump-autoload
```

#### Dependency Installation Failures

**Problem**: Required dependencies fail to install

**Symptoms**:
- Composer dependency conflicts
- Missing required modules
- Version compatibility issues

**Solutions**:
1. Update composer to latest version:
```bash
composer self-update
```

2. Clear composer cache:
```bash
composer clear-cache
```

3. Install dependencies manually:
```bash
composer require drupal/ai --no-update
composer update drupal/ai
```

4. Check for conflicts:
```bash
composer why-not drupal/ai
```

### Configuration Issues

#### Database Connection Failures

**Problem**: Cannot connect to source database

**Symptoms**:
- "Connection refused" errors
- "Access denied" errors
- Migration discovery fails

**Solutions**:
1. Test database connection manually:
```bash
mysql -h hostname -u username -p database_name
```

2. Verify settings.php configuration:
```php
$databases['migrate']['default'] = [
  'database' => 'source_db',
  'username' => 'db_user', 
  'password' => 'db_password',
  'host' => 'localhost',
  'port' => '3306',
  'driver' => 'mysql',
  'charset' => 'utf8mb4',
];
```

3. Check database permissions:
```sql
GRANT SELECT ON source_db.* TO 'db_user'@'localhost';
FLUSH PRIVILEGES;
```

4. Test with Drush:
```bash
drush sql:connect --database=migrate
```

#### Permission Denied Errors

**Problem**: Access denied to migration features

**Symptoms**:
- "Access denied" on migration pages
- Missing admin menu items
- API endpoints return 403

**Solutions**:
1. Verify user permissions at `/admin/people/permissions`:
   -  Administer Drupal Migration Plus
   -  Access migration reports

2. Check role assignments:
```bash
drush user:role:add administrator username
```

3. Clear permission cache:
```bash
drush php-eval "\Drupal::service('kernel')->invalidateContainer();"
```

#### AI Provider Configuration Issues

**Problem**: AI features not working

**Symptoms**:
- "AI provider not configured" errors
- Field analysis returns empty results
- API key validation fails

**Solutions**:
1. Verify AI module installation:
```bash
drush en ai
```

2. Configure AI provider at `/admin/config/ai`:
   - Add valid API key
   - Select appropriate model
   - Test connection

3. Check API key permissions with provider:
   - OpenAI: Verify API key has sufficient credits
   - Ensure model access (GPT-4, GPT-3.5-turbo)

4. Test AI integration:
```bash
drush php-eval "
$ai = \Drupal::service('ai.provider_manager');
$provider = $ai->createInstance('openai');
var_dump($provider->isOperational());
"
```

### Migration Execution Issues

#### Memory Exhaustion

**Problem**: "Fatal error: Allowed memory size exhausted"

**Symptoms**:
- Migration stops with memory error
- Large content items fail to process
- Batch processing fails

**Solutions**:
1. Increase PHP memory limit in `php.ini`:
```ini
memory_limit = 2G
```

2. Increase memory limit for specific migration in `settings.php`:
```php
if (PHP_SAPI === 'cli') {
  ini_set('memory_limit', '4G');
}
```

3. Reduce batch size:
```php
$config['migrate_plus.migration_group.my_group']['batch_size'] = 25;
```

4. Enable memory limit monitoring:
```bash
drush config:set system.performance memory_limit_logging 1
```

#### Timeout Issues

**Problem**: Script execution timeout during migration

**Symptoms**:
- "Maximum execution time exceeded" errors
- Migration appears stuck
- Incomplete batch processing

**Solutions**:
1. Increase execution time in `php.ini`:
```ini
max_execution_time = 3600
```

2. For CLI operations, disable time limit:
```php
if (PHP_SAPI === 'cli') {
  set_time_limit(0);
}
```

3. Use background processing:
```bash
drush migrate:import migration_id --execute-dependencies
```

4. Enable proper batch processing:
```php
$batch = [
  'operations' => $operations,
  'finished' => 'migration_batch_finished',
  'title' => t('Processing migration'),
  'progressive' => TRUE,
];
batch_set($batch);
```

#### Field Mapping Errors

**Problem**: Field mapping validation fails

**Symptoms**:
- "Target field does not exist" errors
- Data type mismatch warnings
- Field configuration errors

**Solutions**:
1. Verify target field exists:
```bash
drush field:list node.article
```

2. Create missing fields:
```bash
drush generate:field
```

3. Check field type compatibility:
```php
$field_definitions = \Drupal::service('entity_field.manager')
  ->getFieldDefinitions('node', 'article');
foreach ($field_definitions as $name => $definition) {
  echo $name . ': ' . $definition->getType() . "\n";
}
```

4. Update field configuration:
```bash
drush config:edit field.field.node.article.field_name
```

### Performance Issues

#### Slow Migration Processing

**Problem**: Migration runs very slowly

**Symptoms**:
- Very low items per second
- High database load
- Excessive memory usage

**Solutions**:
1. Optimize database queries:
```sql
-- Add indexes for common migration queries
ALTER TABLE source_table ADD INDEX idx_migration (id, type, status);
```

2. Enable query logging to identify slow queries:
```php
$databases['migrate']['default']['log_queries'] = TRUE;
```

3. Use batch processing with optimal batch size:
```php
// Test different batch sizes
$batch_sizes = [10, 25, 50, 100];
foreach ($batch_sizes as $size) {
  // Time the processing and choose optimal size
}
```

4. Implement proper caching:
```php
$cache = \Drupal::cache('drupal_migration_plus');
$cached_data = $cache->get($cache_key);
if (!$cached_data) {
  $data = expensive_operation();
  $cache->set($cache_key, $data, CacheBackendInterface::CACHE_PERMANENT);
}
```

#### Database Lock Issues

**Problem**: Database locks during migration

**Symptoms**:
- "Lock wait timeout exceeded" errors
- Migration appears stuck
- Database connection issues

**Solutions**:
1. Increase lock timeout:
```sql
SET SESSION innodb_lock_wait_timeout = 300;
```

2. Use smaller transactions:
```php
// Process in smaller batches to reduce lock time
$batch_size = 10;
foreach (array_chunk($items, $batch_size) as $batch) {
  // Process batch in single transaction
}
```

3. Avoid long-running queries during migration:
```sql
-- Check for long-running queries
SHOW PROCESSLIST;
-- Kill problematic queries if needed
KILL query_id;
```

### API Issues

#### REST API Not Responding

**Problem**: API endpoints return errors

**Symptoms**:
- 404 errors on API endpoints
- JSON responses not returned
- CORS issues in browser

**Solutions**:
1. Clear routing cache:
```bash
drush router:rebuild
```

2. Verify route configuration:
```bash
drush route:debug | grep field-mapping
```

3. Check module weight and dependencies:
```bash
drush config:get core.extension.module
```

4. Test endpoint manually:
```bash
curl -X GET "http://your-site.com/api/migration/field-mapping/schema" \
  -H "Accept: application/json" \
  -H "Authorization: Bearer YOUR_TOKEN"
```

#### Authentication Issues

**Problem**: API authentication fails

**Symptoms**:
- 401 Unauthorized errors
- Token validation fails
- Session authentication not working

**Solutions**:
1. Verify CSRF token for session authentication:
```javascript
// Get CSRF token
const token = drupalSettings.csrf_token;
// Include in requests
headers: {
  'X-CSRF-Token': token
}
```

2. Check basic authentication configuration:
```bash
drush config:get basic_auth.settings
```

3. Verify user permissions:
```bash
drush user:information username
```

### Data Quality Issues

#### Character Encoding Problems

**Problem**: Special characters not migrating correctly

**Symptoms**:
- Question marks or squares instead of special characters
- Garbled text in migrated content
- UTF-8 encoding issues

**Solutions**:
1. Set proper database charset:
```sql
ALTER DATABASE source_db CHARACTER SET utf8mb4 COLLATE utf8mb4_unicode_ci;
```

2. Configure proper encoding in settings.php:
```php
$databases['migrate']['default']['charset'] = 'utf8mb4';
$databases['migrate']['default']['collation'] = 'utf8mb4_unicode_ci';
```

3. Clean content during migration:
```php
$clean_content = mb_convert_encoding($content, 'UTF-8', 'auto');
```

#### Missing Media Files

**Problem**: Images and files not migrating

**Symptoms**:
- Broken image links
- Missing file references
- File path errors

**Solutions**:
1. Verify source file paths:
```php
$settings['migrate_source_base_path'] = '/correct/path/to/files';
$settings['migrate_source_private_file_path'] = '/correct/path/to/private';
```

2. Check file permissions:
```bash
find /path/to/files -type f -exec chmod 644 {} \;
find /path/to/files -type d -exec chmod 755 {} \;
```

3. Verify file destination directory:
```bash
mkdir -p sites/default/files/migration
chmod 755 sites/default/files/migration
```

4. Test file copying:
```php
$source_path = '/source/files/image.jpg';
$dest_path = 'public://migration/image.jpg';
file_unmanaged_copy($source_path, $dest_path, FILE_EXISTS_REPLACE);
```

### Debug and Monitoring

#### Enable Debug Logging

Add to `settings.php`:
```php
// Enable verbose logging
$config['system.logging']['error_level'] = 'verbose';

// Migration-specific logging
$settings['container_yamls'][] = DRUPAL_ROOT . '/sites/development.services.yml';
```

Create `sites/development.services.yml`:
```yaml
services:
  logger.channel.drupal_migration_plus:
    class: Drupal\Core\Logger\LoggerChannel
    arguments: ['drupal_migration_plus']
```

#### Monitor Migration Progress

```bash
# Watch migration status
watch -n 5 'drush migrate:status'

# Monitor system resources
top -p $(pgrep -d',' php)

# Check database connections
mysqladmin processlist

# Monitor error logs
tail -f /var/log/apache2/error.log
```

#### Performance Profiling

```php
// Add to migration processing
$start_memory = memory_get_usage();
$start_time = microtime(true);

// ... migration processing ...

$end_memory = memory_get_usage();
$end_time = microtime(true);

$memory_used = $end_memory - $start_memory;
$time_taken = $end_time - $start_time;

\Drupal::logger('drupal_migration_plus')->info(
  'Migration batch processed: @items items, @memory memory, @time seconds',
  [
    '@items' => count($items),
    '@memory' => format_bytes($memory_used),
    '@time' => round($time_taken, 2),
  ]
);
```

### Recovery Procedures

#### Rollback Failed Migration

```bash
# Stop current migration
drush migrate:stop migration_id

# Rollback processed items
drush migrate:rollback migration_id

# Reset migration status
drush migrate:reset-status migration_id

# Clean up any partially migrated data
drush sql:query "DELETE FROM node WHERE type = 'article' AND created > UNIX_TIMESTAMP() - 3600"
```

#### Restore from Backup

```bash
# Restore database
mysql target_db < backup.sql

# Restore files
rsync -av backup/files/ sites/default/files/

# Clear caches
drush cr

# Rebuild registry
drush php-eval "system_rebuild_module_data();"
```

### Getting Additional Help

#### Log Analysis

Check these log files for more information:
- `/var/log/apache2/error.log` - Web server errors
- `/var/log/mysql/error.log` - Database errors
- Database watchdog table - Drupal system logs
- Migration-specific logs in admin interface

#### Community Resources

- **Issue Queue**: Report bugs with detailed error logs
- **Community Forums**: Search for similar issues
- **Documentation**: Check for updated troubleshooting information
- **Professional Support**: Consider professional assistance for complex issues

#### Debug Information to Collect

When reporting issues, include:
- Drupal version and Migration Plus version
- Source CMS type and version
- Server configuration (PHP, MySQL versions)
- Error messages and logs
- Steps to reproduce the issue
- Migration configuration details

This troubleshooting guide covers the most common issues encountered during migration setup and execution. For issues not covered here, refer to the community resources or consider professional support.
