---
id: modules-development-langchain-EXAMPLES
---
# LangChain Usage Examples

This document provides practical examples of how to use various components of the LangChain module in your Drupal application.

## Table of Contents

- [Basic Provider Usage](#basic-provider-usage)
- [Creating and Using Chains](#creating-and-using-chains)
- [Working with Agents](#working-with-agents)
- [Prompt Templates](#prompt-templates)
- [Self-Adapting Chains](#self-adapting-chains)
- [ECA Integration](#eca-integration)
- [RAG Implementation](#rag-implementation)
- [LangGraph Workflows](#langgraph-workflows)
- [Observability](#observability)
- [REST API Examples](#rest-api-examples)

## Basic Provider Usage

### Getting a Provider Instance

```php
// Get the provider manager service
$provider_manager = \Drupal::service('langchain.provider_manager');

// Get an instance of the OpenAI provider
$provider = $provider_manager->getProvider('openai');

// Configure the provider instance
$provider->setModel('gpt-4');
$provider->setTemperature(0.7);
```

### Simple Completion Request

```php
try {
  $response = $provider->complete('What is the capital of France?');
  $content = $response->getContent();
  \Drupal::messenger()->addStatus(t('AI response: @content', ['@content' => $content]));
} 
catch (\Exception $e) {
  \Drupal::messenger()->addError(t('Error: @message', ['@message' => $e->getMessage()]));
}
```

### Streaming Response

```php
// Set up a streaming callback
$callback = function ($chunk) {
  // Process each chunk as it arrives
  $partial_content = $chunk->getContent();
  // Append to response or process incrementally
};

try {
  $provider->completeStreaming('Generate a poem about Drupal.', $callback);
} 
catch (\Exception $e) {
  \Drupal::messenger()->addError(t('Streaming error: @message', ['@message' => $e->getMessage()]));
}
```

## Creating and Using Chains

### Programmatically Creating a Chain

```php
use Drupal\langchain\Entity\Chain;

// Create a new chain
$chain = Chain::create([
  'name' => 'Content Summarizer',
  'description' => 'Summarizes content in 3 bullet points',
  'langchain_provider' => 'openai',
  'model' => 'gpt-3.5-turbo',
  'prompt_template' => 'Summarize the following content in exactly 3 bullet points: {{content}}',
  'temperature' => 0.3,
  'status' => TRUE,
]);

// Save the chain
$chain->save();
```

### Executing a Chain

```php
// Get the LangChain service
$langchain_service = \Drupal::service('langchain.service');

// Load a chain by ID
$chain = Chain::load(1);

// Execute the chain with variables
$result = $langchain_service->executeChain($chain, [
  'content' => 'Drupal is a free and open-source content management system (CMS) written in PHP and distributed under the GNU General Public License. Drupal provides a back-end framework for at least 14% of the top 10,000 websites worldwide – ranging from personal blogs to corporate, political, and government sites. Systems also use Drupal for knowledge management and for business collaboration.',
]);

// Get the result content
$summary = $result->getContent();
```

### Chaining Multiple Steps

```php
use Drupal\langchain\Entity\Chain;

// Create a multi-step chain
$multi_step_chain = Chain::create([
  'name' => 'Content Analysis Pipeline',
  'description' => 'Analyzes content in multiple steps',
  'langchain_provider' => 'openai',
  'model' => 'gpt-4',
  'steps' => [
    [
      'name' => 'extract_topics',
      'prompt_template' => 'Extract the main topics from this content: {{content}}',
      'output_variable' => 'topics',
    ],
    [
      'name' => 'sentiment_analysis',
      'prompt_template' => 'Analyze the sentiment of this content: {{content}}',
      'output_variable' => 'sentiment',
    ],
    [
      'name' => 'final_summary',
      'prompt_template' => 'Create a summary that highlights the main topics ({{topics}}) and overall sentiment ({{sentiment}}) of the content.',
      'output_variable' => 'final_result',
    ],
  ],
  'status' => TRUE,
]);

$multi_step_chain->save();

// Execute the multi-step chain
$result = $langchain_service->executeChain($multi_step_chain, [
  'content' => 'Your content here...',
]);

// Get the final result and intermediate outputs
$final_summary = $result->getVariable('final_result');
$topics = $result->getVariable('topics');
$sentiment = $result->getVariable('sentiment');
```

## Working with Agents

### Creating an Agent

```php
use Drupal\langchain\Entity\Agent;

// Create a new agent
$agent = Agent::create([
  'name' => 'Content Assistant',
  'description' => 'Helps with content creation and editing',
  'langchain_provider' => 'openai',
  'model' => 'gpt-4',
  'system_prompt' => 'You are a helpful assistant that specializes in content creation and editing for Drupal websites. Help the user craft engaging and SEO-friendly content.',
  'tools' => ['search', 'create_content', 'check_seo'],
  'temperature' => 0.7,
  'status' => TRUE,
]);

// Save the agent
$agent->save();
```

### Running an Agent

```php
// Get the LangChain service
$langchain_service = \Drupal::service('langchain.service');

// Load an agent by ID
$agent = Agent::load(1);

// Run the agent with a user query
$result = $langchain_service->runAgent($agent, [
  'query' => 'Help me write a blog post about Drupal and AI integration.',
  'max_steps' => 10,
]);

// Get the agent's response
$response = $result->getFinalResponse();

// Get the full trace of agent actions
$steps = $result->getSteps();
foreach ($steps as $step) {
  $thought = $step->getThought();
  $action = $step->getAction();
  $observation = $step->getObservation();
  // Process or display the step information
}
```

## Prompt Templates

### Creating a Prompt Template

```php
use Drupal\langchain\Entity\PromptTemplate;

// Create a new prompt template
$template = PromptTemplate::create([
  'name' => 'SEO Title Generator',
  'description' => 'Generates SEO-friendly titles for content',
  'template' => 'Generate {{count}} SEO-friendly titles for a blog post about {{topic}} aimed at {{audience}}. Each title should be compelling and include relevant keywords.',
  'variables' => ['count', 'topic', 'audience'],
  'status' => TRUE,
]);

// Save the template
$template->save();
```

### Using a Prompt Template

```php
// Get the prompt template manager
$template_manager = \Drupal::service('langchain.prompt_template_manager');

// Load a template by ID
$template = PromptTemplate::load(1);

// Render the template with variables
$prompt = $template_manager->renderTemplate($template, [
  'count' => 5,
  'topic' => 'headless Drupal',
  'audience' => 'web developers',
]);

// Use the rendered prompt with a provider
$provider = $provider_manager->getProvider('openai');
$response = $provider->complete($prompt);
```

## Self-Adapting Chains

### Creating a Self-Adapting Chain

```php
use Drupal\langchain\Entity\SelfAdaptingChain;

// Create a new self-adapting chain
$adaptive_chain = SelfAdaptingChain::create([
  'name' => 'Smart Content Generator',
  'description' => 'Generates content and improves based on feedback',
  'langchain_provider' => 'openai',
  'model' => 'gpt-4',
  'base_prompt_template' => 'Create a {{content_type}} about {{topic}} for {{audience}}.',
  'feedback_prompt_template' => 'The previous response received the following feedback: {{feedback}}. Please improve the response based on this feedback.',
  'adaptation_strategies' => [
    'feedback_incorporation' => TRUE,
    'example_learning' => TRUE,
    'parameter_tuning' => TRUE,
  ],
  'learning_rate' => 0.5,
  'status' => TRUE,
]);

// Save the self-adapting chain
$adaptive_chain->save();
```

### Using a Self-Adapting Chain with Feedback

```php
// Get the self-optimizing service
$self_optimizing_service = \Drupal::service('langchain.self_optimizing');

// Load a self-adapting chain
$adaptive_chain = SelfAdaptingChain::load(1);

// Initial execution
$result = $self_optimizing_service->executeAdaptiveChain($adaptive_chain, [
  'content_type' => 'blog post',
  'topic' => 'digital experience platforms',
  'audience' => 'marketing professionals',
]);

$initial_content = $result->getContent();

// Provide feedback for improvement
$feedback_result = $self_optimizing_service->provideFeedback(
  $adaptive_chain,
  $result->getExecutionId(),
  'The content is good but could use more specific examples and statistics to support the points.'
);

$improved_content = $feedback_result->getContent();

// The chain will learn from this feedback for future executions
```

## ECA Integration

### Creating an ECA Rule for LangChain

```php
use Drupal\eca\Entity\Eca;
use Drupal\eca_content\Plugin\ECA\Event\ContentEvent;
use Drupal\langchain_eca\Plugin\ECA\Action\LangChainChainExecuteAction;

// Create an ECA rule that generates a summary when a node is created
$eca = Eca::create([
  'id' => 'auto_summarize_content',
  'label' => 'Automatically summarize content',
  'status' => TRUE,
  'events' => [
    'content_insert' => [
      'plugin' => 'content:insert',
      'label' => 'After saving new content',
      'configuration' => [
        'content_type' => 'article',
      ],
    ],
  ],
  'conditions' => [],
  'actions' => [
    'execute_summary_chain' => [
      'plugin' => 'langchain_execute_chain',
      'label' => 'Execute summary chain',
      'configuration' => [
        'chain_id' => '1', // ID of your summary chain
        'variable_mapping' => [
          'content' => '[node:body]',
        ],
        'result_token' => 'summary_result',
      ],
    ],
    'save_summary' => [
      'plugin' => 'content:set_field',
      'label' => 'Save summary to field',
      'configuration' => [
        'field_name' => 'field_summary',
        'field_value' => '[summary_result:content]',
      ],
    ],
  ],
]);

$eca->save();
```

## RAG Implementation

### Setting Up Document Processing

```php
// Get the document processor service
$document_processor = \Drupal::service('langchain_rag.document_processor');

// Process a node for the RAG system
$node = Node::load(1);
$result = $document_processor->processEntity('node', $node);

\Drupal::messenger()->addStatus(t('Processed @count chunks for RAG.', [
  '@count' => $result['chunk_count'],
]));
```

### Performing RAG Queries

```php
// Get the RAG service
$rag_service = \Drupal::service('langchain_rag.service');

// Perform a RAG query
$response = $rag_service->query(
  'What are the best practices for Drupal performance?',
  [
    'num_results' => 5,
    'similarity_threshold' => 0.75,
    'content_types' => ['article', 'page'],
    'provider' => 'openai',
    'model' => 'gpt-4',
  ]
);

// Get the AI response with retrieved context
$answer = $response->getAnswer();

// Get the sources used in the response
$sources = $response->getSources();
foreach ($sources as $source) {
  $entity_type = $source['entity_type'];
  $entity_id = $source['entity_id'];
  $relevance = $source['relevance_score'];
  // Process or display the source information
}
```

## LangGraph Workflows

### Creating a LangGraph Workflow

```php
use Drupal\langchain_langgraph\Entity\Workflow;
use Drupal\langchain_langgraph\Entity\WorkflowNode;
use Drupal\langchain_langgraph\Entity\WorkflowEdge;

// Create a new workflow
$workflow = Workflow::create([
  'name' => 'Content Creation Workflow',
  'description' => 'Multi-step workflow for content creation',
  'status' => TRUE,
]);
$workflow->save();

// Add nodes to the workflow
$ideation_node = WorkflowNode::create([
  'workflow' => $workflow->id(),
  'name' => 'ideation',
  'type' => 'llm_node',
  'configuration' => [
    'provider' => 'openai',
    'model' => 'gpt-4',
    'prompt_template' => 'Generate 3 content ideas about {{topic}} for {{audience}}.',
    'output_key' => 'ideas',
  ],
]);
$ideation_node->save();

$outline_node = WorkflowNode::create([
  'workflow' => $workflow->id(),
  'name' => 'outline',
  'type' => 'llm_node',
  'configuration' => [
    'provider' => 'openai',
    'model' => 'gpt-4',
    'prompt_template' => 'Create an outline for the following content idea: {{selected_idea}}',
    'output_key' => 'outline',
  ],
]);
$outline_node->save();

$draft_node = WorkflowNode::create([
  'workflow' => $workflow->id(),
  'name' => 'draft',
  'type' => 'llm_node',
  'configuration' => [
    'provider' => 'openai',
    'model' => 'gpt-4',
    'prompt_template' => 'Write a draft based on this outline: {{outline}}',
    'output_key' => 'draft',
  ],
]);
$draft_node->save();

$review_node = WorkflowNode::create([
  'workflow' => $workflow->id(),
  'name' => 'review',
  'type' => 'tool_node',
  'configuration' => [
    'tool' => 'content_review',
    'parameters' => [
      'content' => '{{draft}}',
      'criteria' => 'clarity,engagement,accuracy',
    ],
    'output_key' => 'review_result',
  ],
]);
$review_node->save();

$decision_node = WorkflowNode::create([
  'workflow' => $workflow->id(),
  'name' => 'decision',
  'type' => 'conditional',
  'configuration' => [
    'condition_plugin' => 'keyword_condition',
    'condition_configuration' => [
      'input_key' => 'review_result',
      'keywords' => 'revision needed,improve',
      'operator' => 'contains',
    ],
  ],
]);
$decision_node->save();

$final_node = WorkflowNode::create([
  'workflow' => $workflow->id(),
  'name' => 'finalize',
  'type' => 'llm_node',
  'configuration' => [
    'provider' => 'openai',
    'model' => 'gpt-4',
    'prompt_template' => 'Finalize this content with SEO optimizations: {{draft}}',
    'output_key' => 'final_content',
  ],
]);
$final_node->save();

$revision_node = WorkflowNode::create([
  'workflow' => $workflow->id(),
  'name' => 'revise',
  'type' => 'llm_node',
  'configuration' => [
    'provider' => 'openai',
    'model' => 'gpt-4',
    'prompt_template' => 'Revise this draft based on the following feedback: {{review_result}}. Original draft: {{draft}}',
    'output_key' => 'revised_draft',
  ],
]);
$revision_node->save();

// Connect the nodes with edges
$edges = [
  WorkflowEdge::create([
    'workflow' => $workflow->id(),
    'from_node' => $ideation_node->id(),
    'to_node' => $outline_node->id(),
  ]),
  WorkflowEdge::create([
    'workflow' => $workflow->id(),
    'from_node' => $outline_node->id(),
    'to_node' => $draft_node->id(),
  ]),
  WorkflowEdge::create([
    'workflow' => $workflow->id(),
    'from_node' => $draft_node->id(),
    'to_node' => $review_node->id(),
  ]),
  WorkflowEdge::create([
    'workflow' => $workflow->id(),
    'from_node' => $review_node->id(),
    'to_node' => $decision_node->id(),
  ]),
  WorkflowEdge::create([
    'workflow' => $workflow->id(),
    'from_node' => $decision_node->id(),
    'to_node' => $final_node->id(),
    'condition' => 'false', // If review is good
  ]),
  WorkflowEdge::create([
    'workflow' => $workflow->id(),
    'from_node' => $decision_node->id(),
    'to_node' => $revision_node->id(),
    'condition' => 'true', // If review indicates revision needed
  ]),
  WorkflowEdge::create([
    'workflow' => $workflow->id(),
    'from_node' => $revision_node->id(),
    'to_node' => $review_node->id(),
  ]),
];

foreach ($edges as $edge) {
  $edge->save();
}
```

### Executing a LangGraph Workflow

```php
// Get the workflow execution service
$workflow_execution = \Drupal::service('langchain_langgraph.execution');

// Execute a workflow
$execution = $workflow_execution->executeWorkflow(
  $workflow->id(),
  [
    'topic' => 'digital transformation',
    'audience' => 'enterprise decision makers',
    'selected_idea' => '{{ideas[0]}}', // Select the first idea from ideation
  ]
);

// Get the final result
$final_content = $execution->getOutput('final_content');

// Get the full execution trace
$trace = $execution->getTrace();
foreach ($trace as $step) {
  $node = $step->getNode();
  $input = $step->getInput();
  $output = $step->getOutput();
  // Process or display the step information
}
```

## Observability

### Setting Up Langfuse Observability

```php
// Get the Langfuse service
$langfuse = \Drupal::service('langfuse_observability.service');

// Start a trace for a user interaction
$trace = $langfuse->startTrace('content_generation', [
  'user_id' => \Drupal::currentUser()->id(),
  'content_type' => 'article',
]);

// Start a span for a specific operation
$span = $langfuse->startSpan($trace->getId(), 'draft_generation', [
  'model' => 'gpt-4',
  'temperature' => 0.7,
]);

// Log the prompt
$langfuse->logPrompt($span->getId(), [
  'prompt' => 'Generate a draft about sustainable technology for enterprise leaders.',
  'model' => 'gpt-4',
]);

// Your code to generate content here
$content = 'Generated content...';

// Log the response
$langfuse->logResponse($span->getId(), [
  'content' => $content,
  'tokens' => 850,
]);

// End the span
$langfuse->endSpan($span->getId());

// Score the quality of the generation
$langfuse->score($trace->getId(), 'quality', 0.85, [
  'criteria' => 'relevance',
]);

// End the trace
$langfuse->endTrace($trace->getId());
```

### Using the Observability Decorator

```php
// Get the decorator factory
$decorator_factory = \Drupal::service('langfuse_observability.decorator_factory');

// Get a decorated provider
$provider = $provider_manager->getProvider('openai');
$decorated_provider = $decorator_factory->decorateProvider($provider, [
  'trace_name' => 'article_generation',
  'metadata' => [
    'user_id' => \Drupal::currentUser()->id(),
    'content_type' => 'article',
  ],
]);

// Use the decorated provider
$response = $decorated_provider->complete('Generate an article about Drupal and AI.');

// All operations are automatically logged to Langfuse
```

## REST API Examples

### Chain Execution via REST API

```php
// Example using Guzzle HTTP client
use GuzzleHttp\Client;

$client = new Client([
  'base_uri' => 'https://example.com/api/',
  'headers' => [
    'Content-Type' => 'application/json',
    'X-CSRF-Token' => $csrf_token,
  ],
]);

// Execute a chain via REST API
$response = $client->post('langchain/chains/1/execute', [
  'json' => [
    'variables' => [
      'content' => 'Text to be processed by the chain',
      'tone' => 'professional',
    ],
  ],
]);

$result = json_decode($response->getBody()->getContents(), TRUE);
$content = $result['content'];
```

### Agent Execution via REST API

```php
// Execute an agent via REST API
$response = $client->post('langchain/agents/2/run', [
  'json' => [
    'query' => 'Help me create content about machine learning',
    'max_steps' => 10,
    'parameters' => [
      'detail_level' => 'technical',
    ],
  ],
]);

$result = json_decode($response->getBody()->getContents(), TRUE);
$response_content = $result['response'];
$steps = $result['steps'];
```

---

These examples demonstrate the basic usage patterns for the LangChain module. For more advanced usage and customization options, please refer to the API documentation and the module's source code.
