---
id: modules-development-langchain-TESTING_STANDARDS
---
# Testing Standards for LangChain

This document outlines the testing standards and methodologies for the LangChain module.

## Testing Structure

The module follows Drupal's standard testing structure:

```
langchain/
├── tests/
│   ├── src/
│   │   ├── Unit/               # Unit tests
│   │   ├── Kernel/             # Kernel tests
│   │   ├── Functional/         # Functional tests
│   │   └── FunctionalJavascript/ # JavaScript-enabled tests
│   └── fixtures/               # Test fixtures
```

## Test Types

### Unit Tests

Unit tests focus on testing individual components in isolation:

- Service methods
- Plugin implementations
- Utility functions
- Helper classes

Example unit test:

```php
namespace Drupal\Tests\langchain\Unit;

use Drupal\Tests\UnitTestCase;
use Drupal\langchain\Utility\PromptFormatter;

/**
 * @group langchain
 * @coversDefaultClass \Drupal\langchain\Utility\PromptFormatter
 */
class PromptFormatterTest extends UnitTestCase {

  /**
   * Tests prompt formatting with variables.
   */
  public function testFormatWithVariables() {
    $formatter = new PromptFormatter();
    $template = 'Hello, {{name}}!';
    $variables = ['name' => 'World'];
    
    $result = $formatter->format($template, $variables);
    
    $this->assertEquals('Hello, World!', $result);
  }
}
```

### Kernel Tests

Kernel tests validate interaction with Drupal's API:

- Config schemas
- Database operations
- Service interactions
- Plugin discovery and execution

Example kernel test:

```php
namespace Drupal\Tests\langchain\Kernel;

use Drupal\KernelTests\KernelTestBase;

/**
 * @group langchain
 */
class LlmProviderManagerTest extends KernelTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'system',
    'user',
    'key',
    'field',
    'langchain',
  ];

  /**
   * The LLM provider manager.
   *
   * @var \Drupal\langchain\LlmProviderManagerInterface
   */
  protected $providerManager;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();
    $this->installConfig(['langchain']);
    $this->providerManager = $this->container->get('plugin.manager.langchain.llm_provider');
  }

  /**
   * Tests provider plugin discovery.
   */
  public function testProviderDiscovery() {
    $definitions = $this->providerManager->getDefinitions();
    $this->assertNotEmpty($definitions);
    $this->assertArrayHasKey('openai', $definitions);
  }
}
```

### Functional Tests

Functional tests validate end-to-end functionality:

- Admin UI functionality
- Form submissions
- Integration with other modules
- End-to-end workflows

Example functional test:

```php
namespace Drupal\Tests\langchain\Functional;

use Drupal\Tests\BrowserTestBase;

/**
 * @group langchain
 */
class LangChainAdminTest extends BrowserTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'langchain',
  ];

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * Tests the settings form.
   */
  public function testSettingsForm() {
    $admin_user = $this->drupalCreateUser(['administer langchain']);
    $this->drupalLogin($admin_user);
    
    $this->drupalGet('admin/config/ai/langchain/settings');
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->pageTextContains('LangChain Settings');
    
    // Test form submission
    $edit = [
      'default_provider' => 'openai',
      'default_model' => 'gpt-3.5-turbo',
    ];
    $this->submitForm($edit, 'Save configuration');
    $this->assertSession()->pageTextContains('The configuration options have been saved.');
  }
}
```

### Functional JavaScript Tests

JavaScript-enabled tests validate UI interactions:

- AJAX functionality
- Interactive UI elements
- Real-time updates
- Client-side validation

Example functional JavaScript test:

```php
namespace Drupal\Tests\langchain\FunctionalJavascript;

use Drupal\FunctionalJavascriptTests\WebDriverTestBase;

/**
 * @group langchain
 */
class ChainBuilderTest extends WebDriverTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'langchain',
  ];

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * Tests the chain builder UI.
   */
  public function testChainBuilder() {
    $admin_user = $this->drupalCreateUser(['administer langchain']);
    $this->drupalLogin($admin_user);
    
    $this->drupalGet('admin/structure/langchain/chains/add');
    $this->assertSession()->pageTextContains('Add Chain');
    
    // Test step addition
    $this->click('button[name="add_step"]');
    $this->assertWaitOnAjaxRequest();
    $this->assertSession()->elementExists('css', '.langchain-step-form');
    
    // Select step type
    $this->selectFieldOption('step_type', 'llm');
    $this->assertWaitOnAjaxRequest();
    $this->assertSession()->elementExists('css', '.langchain-step-llm-form');
  }
}
```

## Running Tests

### Running All Tests

To run all tests for the module:

```bash
./vendor/bin/phpunit -c core/phpunit.xml modules/contrib/langchain
```

### Running Specific Test Suites

To run specific test suites:

```bash
# Unit tests only
./vendor/bin/phpunit -c core/phpunit.xml modules/contrib/langchain/tests/src/Unit

# Kernel tests only
./vendor/bin/phpunit -c core/phpunit.xml modules/contrib/langchain/tests/src/Kernel

# Functional tests only
./vendor/bin/phpunit -c core/phpunit.xml modules/contrib/langchain/tests/src/Functional

# Functional JavaScript tests only
./vendor/bin/phpunit -c core/phpunit.xml modules/contrib/langchain/tests/src/FunctionalJavascript
```

### Running Individual Tests

To run individual test classes:

```bash
./vendor/bin/phpunit -c core/phpunit.xml modules/contrib/langchain/tests/src/Unit/PromptFormatterTest.php
```

To run specific test methods:

```bash
./vendor/bin/phpunit -c core/phpunit.xml --filter testFormatWithVariables modules/contrib/langchain/tests/src/Unit/PromptFormatterTest.php
```

## Test Coverage

The module aims for comprehensive test coverage:

- Core services: 100% coverage
- Plugin implementations: 100% coverage
- UI components: 90%+ coverage
- Integration points: 90%+ coverage

To generate a test coverage report:

```bash
XDEBUG_MODE=coverage ./vendor/bin/phpunit -c core/phpunit.xml --coverage-html ./coverage modules/contrib/langchain
```

## Testing AI Functionality

Testing AI functionality presents unique challenges:

### Mocking AI Providers

For unit and kernel tests, AI providers should be mocked:

```php
// Example mock for OpenAI provider
$mock_provider = $this->createMock(LlmProviderInterface::class);
$mock_provider->method('complete')
  ->willReturn([
    'content' => 'Mocked response',
    'tokens' => 10,
    'model' => 'gpt-3.5-turbo',
  ]);
```

### Testing with Deterministic Responses

For functional tests that need predictable AI responses:

```php
// Override the provider factory service
$this->container->set('langchain.provider_factory', new TestProviderFactory([
  'response' => 'This is a deterministic test response',
  'tokens' => 10,
]));
```

### Testing API Integrations

For testing actual API integrations:

1. Use a test API key from environment variables
2. Create an API key for testing with limited quota
3. Use test mode flags to prevent actual API calls

```php
// Example test that conditionally calls real API
if (getenv('LANGCHAIN_TEST_REAL_API')) {
  $actual_response = $provider->complete($prompt, $options);
  $this->assertNotEmpty($actual_response['content']);
} else {
  $this->markTestSkipped('Skipping real API test');
}
```

## Test Fixtures

The module includes test fixtures for consistent testing:

- Example prompts and responses
- Mock API responses
- Sample chain configurations
- Test entity definitions

Example usage:

```php
// In your test class
protected function getFixtureContent($name) {
  return file_get_contents(__DIR__ . '/../../fixtures/' . $name);
}

// Usage
$prompt_template = $this->getFixtureContent('prompt_template.txt');
$mock_response = json_decode($this->getFixtureContent('openai_response.json'), TRUE);
```

## Continuous Integration

The module is set up for continuous integration with:

- PHPUnit for automated testing
- PHP_CodeSniffer for coding standards validation

### CI Configuration

Example GitHub Actions configuration:

```yaml
name: Test

on:
  push:
    branches: [ main ]
  pull_request:
    branches: [ main ]

jobs:
  test:
    runs-on: ubuntu-latest
    
    steps:
    - uses: actions/checkout@v2
    
    - name: Install dependencies
      run: composer install
    
    - name: Run tests
      run: ./vendor/bin/phpunit -c phpunit.xml.dist
    
    - name: Check coding standards
      run: ./vendor/bin/phpcs
```

## Test Documentation

All tests should include comprehensive documentation:

- Class-level documentation explaining the purpose of the test class
- Method-level documentation explaining the purpose of each test method
- Data provider documentation explaining test variations
- Comprehensive assertions with meaningful messages

Example:

```php
/**
 * Tests the PromptFormatter utility.
 *
 * This test class validates that the PromptFormatter correctly formats
 * prompt templates by replacing variables and handling special cases
 * like escape sequences and nested variables.
 *
 * @group langchain
 * @coversDefaultClass \Drupal\langchain\Utility\PromptFormatter
 */
class PromptFormatterTest extends UnitTestCase {

  /**
   * Tests prompt formatting with different variable types.
   *
   * @param string $template
   *   The prompt template to format.
   * @param array $variables
   *   The variables to replace in the template.
   * @param string $expected
   *   The expected formatted result.
   * @param string $description
   *   Description of the test case.
   *
   * @dataProvider providerFormatWithVariables
   */
  public function testFormatWithVariables($template, $variables, $expected, $description) {
    $formatter = new PromptFormatter();
    $result = $formatter->format($template, $variables);
    
    $this->assertEquals(
      $expected,
      $result,
      sprintf('Failed to correctly format prompt for case: %s', $description)
    );
  }

  /**
   * Data provider for testFormatWithVariables.
   *
   * @return array
   *   Test cases.
   */
  public function providerFormatWithVariables() {
    return [
      ['Hello, {{name}}!', ['name' => 'World'], 'Hello, World!', 'Simple variable'],
      ['{{greeting}}, {{name}}!', ['greeting' => 'Hi', 'name' => 'User'], 'Hi, User!', 'Multiple variables'],
      ['Value: {{number}}', ['number' => 42], 'Value: 42', 'Numeric variable'],
      ['Empty: {{value}}', ['value' => ''], 'Empty: ', 'Empty string variable'],
      ['Escaped: \\{{name}}', ['name' => 'World'], 'Escaped: {{name}}', 'Escaped variable syntax'],
    ];
  }
}
```

## Resources

- [Drupal Testing Documentation](https://www.drupal.org/docs/testing)
- [PHPUnit Documentation](https://phpunit.readthedocs.io/)
- [Drupal JavaScript Testing](https://www.drupal.org/docs/8/testing/javascript-testing)
