---
id: modules-development-langchain-controller-tests
---
# Controller and Routing Tests

This document provides information on testing controllers and routing in the LangChain module.

## Table of Contents

- [Overview](#overview)
- [Testing Approach](#testing-approach)
- [Unit Testing Controllers](#unit-testing-controllers)
- [Functional Testing Controllers](#functional-testing-controllers)
- [Testing REST Resources](#testing-rest-resources)
- [Testing Route Access](#testing-route-access)
- [Common Test Patterns](#common-test-patterns)

## Overview

The LangChain module includes several controllers that handle various aspects of the module's functionality:

- **LangChainController**: Main controller for general module functionality
- **LangChainAdminDashboardController**: Dashboard for module administration
- **LangChainProviderController**: Manages LLM provider integrations
- **LangChainRestController**: Handles REST API requests
- **SelfAdaptingChainController**: Manages self-adapting chain functionality
- **ConfigurationWizardController**: Provides a configuration wizard interface
- **AgentController**: Manages agent functionality
- **WorkflowController**: Manages workflow functionality
- **WorkflowExecutionController**: Handles workflow execution

## Testing Approach

Controllers in the LangChain module are tested using a combination of unit tests and functional tests:

- **Unit Tests**: Test controller methods in isolation with mocked dependencies
- **Functional Tests**: Test controllers in a full Drupal environment with browser simulation

## Unit Testing Controllers

Unit tests for controllers focus on testing the controller's logic in isolation from Drupal's routing and rendering systems.

### Example Unit Test for a Controller

```php
namespace Drupal\Tests\langchain\Unit;

use Drupal\Tests\UnitTestCase;
use Drupal\langchain\Controller\LangChainController;
use Drupal\langchain\LangChainService;
use Drupal\Core\Messenger\MessengerInterface;
use Prophecy\PhpUnit\ProphecyTrait;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * @coversDefaultClass \Drupal\langchain\Controller\LangChainController
 * @group langchain
 */
class LangChainControllerTest extends UnitTestCase {

  use ProphecyTrait;

  /**
   * The LangChain service.
   *
   * @var \Drupal\langchain\LangChainService|\Prophecy\Prophecy\ProphecyInterface
   */
  protected $langchainService;

  /**
   * The messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface|\Prophecy\Prophecy\ProphecyInterface
   */
  protected $messenger;

  /**
   * The controller being tested.
   *
   * @var \Drupal\langchain\Controller\LangChainController
   */
  protected $controller;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->langchainService = $this->prophesize(LangChainService::class);
    $this->messenger = $this->prophesize(MessengerInterface::class);

    $this->controller = new LangChainController(
      $this->langchainService->reveal(),
      $this->messenger->reveal()
    );
  }

  /**
   * @covers ::create
   */
  public function testCreate() {
    $container = $this->prophesize(ContainerInterface::class);
    $container->get('langchain.service')->willReturn($this->langchainService->reveal());
    $container->get('messenger')->willReturn($this->messenger->reveal());

    $controller = LangChainController::create($container->reveal());
    $this->assertInstanceOf(LangChainController::class, $controller);
  }

  /**
   * @covers ::content
   */
  public function testContent() {
    // Set up service method expectations
    $this->langchainService->getActiveProviders()->willReturn(['openai' => 'OpenAI']);

    // Call the controller method
    $build = $this->controller->content();

    // Assertions
    $this->assertIsArray($build);
    $this->assertArrayHasKey('#theme', $build);
    $this->assertEquals('langchain-admin-dashboard', $build['#theme']);
    $this->assertArrayHasKey('#providers', $build);
    $this->assertEquals(['openai' => 'OpenAI'], $build['#providers']);
  }
}
```

## Functional Testing Controllers

Functional tests for controllers verify that routes are accessible, permissions are enforced, and the UI functions correctly.

### Example Functional Test for a Controller

```php
namespace Drupal\Tests\langchain\Functional;

use Drupal\Tests\BrowserTestBase;

/**
 * Tests the LangChain administration UI.
 *
 * @group langchain
 */
class LangChainAdministrationTest extends BrowserTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'langchain',
    'block',
    'node',
  ];

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * A test user with administrative permissions.
   *
   * @var \Drupal\user\UserInterface
   */
  protected $adminUser;

  /**
   * A test user without administrative permissions.
   *
   * @var \Drupal\user\UserInterface
   */
  protected $regularUser;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    // Create and log in as admin user.
    $this->adminUser = $this->drupalCreateUser([
      'administer langchain',
      'access administration pages',
    ]);

    // Create a regular user without admin permissions.
    $this->regularUser = $this->drupalCreateUser([
      'access content',
    ]);
  }

  /**
   * Tests access to the admin dashboard.
   */
  public function testAdminDashboardAccess() {
    // Test access for admin user.
    $this->drupalLogin($this->adminUser);
    $this->drupalGet('admin/config/langchain/dashboard');
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->pageTextContains('LangChain Dashboard');

    // Test access for regular user.
    $this->drupalLogin($this->regularUser);
    $this->drupalGet('admin/config/langchain/dashboard');
    $this->assertSession()->statusCodeEquals(403);
  }

  /**
   * Tests the provider management page.
   */
  public function testProviderManagement() {
    $this->drupalLogin($this->adminUser);
    $this->drupalGet('admin/config/langchain/providers');
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->pageTextContains('LLM Providers');
    
    // Test provider listing.
    $this->assertSession()->elementExists('css', 'table.provider-list');
    
    // Additional assertions for specific UI elements...
  }
}
```

## Testing REST Resources

The LangChain module provides REST API endpoints that need to be tested to ensure they function correctly and securely.

### Example REST Resource Test

```php
namespace Drupal\Tests\langchain\Functional;

use Drupal\Tests\rest\Functional\CookieResourceTestTrait;
use Drupal\Tests\rest\Functional\ResourceTestBase;

/**
 * Tests the LangChain REST resources.
 *
 * @group langchain
 */
class LangChainRestResourceTest extends ResourceTestBase {

  use CookieResourceTestTrait;

  /**
   * {@inheritdoc}
   */
  protected static $modules = ['langchain', 'rest'];

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();
    
    // Set up authentication and REST configuration.
    $this->setUpAuthorization('cookie');
    
    // Create users with appropriate permissions.
    // ...
  }

  /**
   * Tests the GET method for listing providers.
   */
  public function testGetProviders() {
    $this->initAuthentication();
    $url = Url::fromRoute('rest.langchain_providers.GET');
    
    // Test as authenticated user with permissions.
    $this->drupalLogin($this->account);
    $response = $this->request('GET', $url, $this->getAuthenticationOptions());
    
    $this->assertResourceResponse(200, $response);
    $response_data = $this->getDecodedContents($response);
    
    // Assertions on response data...
  }
}
```

## Testing Route Access

It's important to test that route access is properly controlled based on user permissions.

### Access Control Testing

```php
/**
 * Tests access control for LangChain routes.
 */
public function testRouteAccess() {
  // Test routes that should be accessible by admin users.
  $admin_routes = [
    'langchain.dashboard',
    'langchain.providers',
    'langchain.settings',
  ];

  $this->drupalLogin($this->adminUser);
  foreach ($admin_routes as $route_name) {
    $this->drupalGet(Url::fromRoute($route_name));
    $this->assertSession()->statusCodeEquals(200);
  }

  // Test routes that should be denied for regular users.
  $this->drupalLogin($this->regularUser);
  foreach ($admin_routes as $route_name) {
    $this->drupalGet(Url::fromRoute($route_name));
    $this->assertSession()->statusCodeEquals(403);
  }
  
  // Test routes that should be accessible by specific role permissions.
  // ...
}
```

## Common Test Patterns

### Testing AJAX Controllers

For controllers that handle AJAX requests:

```php
/**
 * Tests AJAX functionality in the workflow editor.
 */
public function testWorkflowEditorAjax() {
  $this->drupalLogin($this->adminUser);
  
  // Create a test workflow entity.
  $workflow = Workflow::create([
    'name' => 'Test Workflow',
    'workflow_id' => 'test_workflow',
  ]);
  $workflow->save();
  
  // Test AJAX callback for adding a node.
  $edit = [
    'node_type' => 'llm_node',
    'node_label' => 'Test LLM Node',
  ];
  
  $this->drupalPostForm('admin/config/langchain/workflows/edit/' . $workflow->id(), $edit, 'Add Node');
  $this->assertSession()->assertWaitOnAjaxRequest();
  
  // Verify node was added in the UI.
  $this->assertSession()->elementExists('css', '.workflow-node[data-label="Test LLM Node"]');
  
  // Verify node was added in the database.
  $workflow = Workflow::load($workflow->id());
  $this->assertNotEmpty($workflow->getNodes());
  $nodes = $workflow->getNodes();
  $this->assertEquals('Test LLM Node', end($nodes)->label());
}
```

### Testing Form Submissions

For controllers that handle form submissions:

```php
/**
 * Tests the agent creation form.
 */
public function testAgentCreationForm() {
  $this->drupalLogin($this->adminUser);
  
  // Test form submission.
  $edit = [
    'name' => 'Test Agent',
    'agent_id' => 'test_agent',
    'description' => 'Test agent description',
    'provider' => 'openai',
    'model' => 'gpt-3.5-turbo',
  ];
  
  $this->drupalPostForm('admin/config/langchain/agents/add', $edit, 'Save');
  
  // Verify redirection.
  $this->assertSession()->addressEquals('admin/config/langchain/agents');
  
  // Verify success message.
  $this->assertSession()->pageTextContains('Agent Test Agent has been created.');
  
  // Verify entity was created.
  $agent = \Drupal::entityTypeManager()->getStorage('langchain_agent')->load('test_agent');
  $this->assertNotNull($agent);
  $this->assertEquals('Test Agent', $agent->label());
}
```
