---
id: modules-development-langchain-testing
---
# LangChain Module Testing Guide

This document outlines the testing approach, test types, and coverage information for the LangChain Drupal module.

## Table of Contents

- [Test Types](#test-types)
- [Running Tests](#running-tests)
- [Test Coverage](#test-coverage)
- [Writing Tests](#writing-tests)
- [Mocking Framework](#mocking-framework)
- [Common Test Patterns](#common-test-patterns)

## Test Types

The LangChain module employs three main types of tests following Drupal's testing conventions:

### Unit Tests

Unit tests focus on testing small, isolated components of the code with all dependencies mocked. These tests verify that individual methods function correctly in isolation.

- Located in `tests/src/Unit/`
- Extend `Drupal\Tests\UnitTestCase`
- Use Prophecy for mocking dependencies
- Do not require a Drupal installation

### Kernel Tests

Kernel tests verify integration between components in a lightweight Drupal environment. These tests validate that plugins are discovered correctly and that components interact properly with Drupal's API.

- Located in `tests/src/Kernel/`
- Extend `Drupal\KernelTests\KernelTestBase`
- Require a minimal Drupal installation
- Test discovery mechanisms and service integration

### Functional Tests

Functional tests verify end-to-end functionality by simulating browser interactions. These tests ensure the module functions correctly in a complete Drupal environment.

- Located in `tests/src/Functional/`
- Extend `Drupal\Tests\BrowserTestBase`
- Simulate browser interactions
- Test controllers, forms, and other user-facing features

## Running Tests

### Running with PHPUnit

```bash
# Run all tests
cd /path/to/drupal
vendor/bin/phpunit -c core modules/contrib/langchain

# Run a specific test type
vendor/bin/phpunit -c core modules/contrib/langchain/tests/src/Unit

# Run a specific test file
vendor/bin/phpunit -c core modules/contrib/langchain/tests/src/Unit/Plugin/ToolPluginManagerTest.php

# Run a specific test method
vendor/bin/phpunit -c core --filter testGetToolsByCategory modules/contrib/langchain/tests/src/Unit/Plugin/ToolPluginManagerTest.php
```

### Running with Drupal CLI

```bash
# Run all tests in the module
drupal test:unit --module=langchain

# Run a specific test group
drupal test:unit --group=langchain
```

## Test Coverage

The LangChain module aims for comprehensive test coverage across all its components:

### Plugin Systems

- **Tool Plugin System**: 100% coverage of manager classes and core plugins
- **Node Plugin System**: 100% coverage of manager classes and core node plugins
- **Condition Plugin System**: 100% coverage of manager classes and core condition plugins

### Core Services

- **LangChain Service**: 90%+ coverage of public methods
- **Provider Integration**: 80%+ coverage of provider plugins
- **Workflow Execution**: 80%+ coverage

### Entity Types

- **Agent**: 80%+ coverage of CRUD operations and business logic
- **Chain**: 80%+ coverage of CRUD operations and business logic
- **Workflow**: 80%+ coverage of CRUD operations and business logic

## Writing Tests

### Unit Test Best Practices

1. Use Prophecy for mocking dependencies
2. Test each method in isolation
3. Test edge cases and error conditions
4. Avoid testing private/protected methods directly
5. Use data providers for testing multiple scenarios

Example:

```php
/**
 * @dataProvider evaluateDataProvider
 */
public function testEvaluate($configuration, $source_output, $expected_result) {
  // Test implementation
}

public function evaluateDataProvider() {
  return [
    'scenario1' => [/* test data */],
    'scenario2' => [/* test data */],
  ];
}
```

### Kernel Test Best Practices

1. Install only the necessary configuration
2. Test plugin discovery and instantiation
3. Test service integration
4. Test error handling with invalid inputs
5. Verify schema validation

### Mocking External Services

For external services like LLM providers, use mock providers to avoid actual API calls:

```php
$provider = $this->prophesize(LangChainProviderInterface::class);
$provider->generate(Argument::any(), Argument::any())
  ->willReturn(['text' => 'Mocked response']);
```

## Mocking Framework

The LangChain module uses the [Prophecy](https://github.com/phpspec/prophecy) mocking framework for unit tests. Prophecy is included in PHPUnit and provides a more expressive syntax for creating test doubles.

Basic usage:

```php
// Create a prophecy (mock)
$service = $this->prophesize(SomeServiceInterface::class);

// Set up method behavior
$service->someMethod(Argument::any())->willReturn('result');

// Use the mock in your test
$this->someService = $service->reveal();
```

## Common Test Patterns

### Testing Plugin Managers

1. Test plugin discovery
2. Test plugin instantiation
3. Test error handling for invalid plugin IDs
4. Test any sorting or filtering methods

### Testing Plugins

1. Test plugin instantiation via factory methods
2. Test plugin methods with various inputs
3. Test configuration validation
4. Test error handling

### Testing Service Classes

1. Test public methods with various inputs
2. Test error handling and recovery
3. Test integration with other services using mocks
4. Test edge cases and boundary conditions
