---
id: modules-development-langchain-testing
---
# LangChain Module Testing & Code Coverage Guide

This document provides comprehensive information about testing the LangChain Drupal module, with special emphasis on achieving and maintaining 100% code coverage.

## Table of Contents

- [Testing Overview](#testing-overview)
- [Coverage Requirements](#coverage-requirements)
- [Running Tests with Coverage](#running-tests-with-coverage)
- [Interpreting Coverage Reports](#interpreting-coverage-reports)
- [Testing Best Practices](#testing-best-practices)
- [Continuous Integration](#continuous-integration)
- [Additional Resources](#additional-resources)

## Testing Overview

The LangChain module employs a comprehensive testing approach that includes:

- **Unit Tests** - Testing individual components in isolation
- **Kernel Tests** - Testing integration with Drupal's API
- **Functional Tests** - Testing user-facing functionality
- **Integration Tests** - Testing specific integration scenarios

All test files are organized according to Drupal's testing conventions:

```
tests/
├── src/
│   ├── Functional/   # Browser-based tests
│   ├── Kernel/       # Kernel tests
│   │   └── Integration/  # Integration-specific tests
│   └── Unit/         # Unit tests
```

Each submodule also contains its own tests following the same structure.

## Coverage Requirements

The LangChain module requires 100% code coverage for all production code. This means that every line of code, class, and method must be executed during test runs.

### Key Coverage Metrics

| Metric | Requirement | Description |
|--------|-------------|-------------|
| Line Coverage | 100% | All lines of code must be executed during tests |
| Method Coverage | 100% | All methods must be called during tests |
| Class Coverage | 100% | All classes must be instantiated during tests |
| Branch Coverage | ≥95% | At least 95% of all decision branches must be executed |

### Coverage Thresholds

- **Production Ready**: ≥90% coverage
- **Required for Merge**: ≥95% coverage
- **Target**: 100% coverage

### Components Requiring 100% Coverage

1. **Core Services**
   - LangChainService
   - LangChainProviderManager
   - All orchestration services

2. **Plugin Systems**
   - Provider plugins
   - Tool plugins
   - Node plugins
   - Condition plugins

3. **Entity Types**
   - Agent
   - Chain
   - PromptTemplate
   - SelfAdaptingChain
   - WorkflowConfiguration

4. **Controllers and APIs**
   - REST endpoints
   - Admin controllers
   - Workflow execution controllers

## Running Tests with Coverage

The module includes a dedicated script for running tests with code coverage analysis. This script is the preferred method for executing tests with coverage.

### Prerequisites

- PHP 8.2+
- Xdebug 3.0+ installed and enabled
- PHPUnit 10.0+
- Composer dependencies installed

### Using the Test Coverage Script

```bash
# Run all tests with HTML coverage report
./run-coverage-tests.sh

# Run only unit tests
./run-coverage-tests.sh -t unit

# Generate different report formats (html, text, clover, xml)
./run-coverage-tests.sh -f text
./run-coverage-tests.sh -f clover

# Show detailed coverage output
./run-coverage-tests.sh -f text -d
```

### Using Composer Scripts

The module's `composer.json` includes several scripts for running tests with coverage:

```bash
# Run all tests with HTML coverage report
composer test-coverage

# Run all tests with Clover XML report (for CI)
composer test-coverage-clover

# Display coverage report in terminal
composer test-coverage-text

# Run tests and static analysis
composer check
```

### Manual PHPUnit Execution

For more control, you can run PHPUnit directly:

```bash
# Navigate to the Drupal root directory
cd /path/to/drupal

# Run all tests with coverage
XDEBUG_MODE=coverage vendor/bin/phpunit -c core \
  --coverage-html modules/contrib/langchain/coverage/html \
  modules/contrib/langchain

# Run specific test suite with coverage
XDEBUG_MODE=coverage vendor/bin/phpunit -c core \
  --testsuite unit \
  --coverage-html modules/contrib/langchain/coverage/html \
  modules/contrib/langchain
```

## Interpreting Coverage Reports

PHPUnit generates various coverage reports that help identify untested code.

### HTML Reports

The HTML report is the most comprehensive and user-friendly format. To view it:

1. Open `coverage/html/index.html` in a web browser
2. Navigate through the file structure to inspect coverage
3. Pay attention to:
   - Red lines (not covered)
   - Yellow lines (partially covered branches)
   - Green lines (fully covered)

![HTML Coverage Report](https://www.phpunit.de/images/code-coverage.png)

### Coverage Analysis

When analyzing coverage reports, look for:

1. **Uncovered Classes** - Classes that were never instantiated
2. **Uncovered Methods** - Methods that were never called
3. **Uncovered Lines** - Lines that were never executed
4. **Partially Covered Branches** - Conditions where not all paths were tested

### Common Coverage Gaps

1. **Error handling paths** - Exception handling often lacks coverage
2. **Edge cases** - Extreme input values or unusual conditions
3. **Complex conditions** - Multiple branches in if/switch statements
4. **Early returns** - Methods that return early under certain conditions
5. **Dependency variations** - Code that behaves differently based on dependencies

## Testing Best Practices

### TDD Approach

1. Write tests before implementing functionality
2. Run tests frequently during development
3. Refactor code while maintaining test coverage
4. Use coverage reports to identify gaps

### Effective Test Design

1. **Test naming** - Use descriptive names that explain test purpose
2. **Data providers** - Test multiple scenarios efficiently
3. **Mocking** - Use prophecy to mock dependencies
4. **Boundary testing** - Test at and around boundary conditions
5. **Negative testing** - Test error handling paths

### Sample Test Structure

```php
/**
 * @covers \Drupal\langchain\Service\LangChainService
 * @group langchain
 */
class LangChainServiceTest extends UnitTestCase {

  /**
   * @covers ::generate
   * @dataProvider generateDataProvider
   */
  public function testGenerate($input, $expected) {
    // Test implementation
  }

  /**
   * @covers ::execute
   */
  public function testExecuteThrowsExceptionOnInvalidInput() {
    // Error handling test
  }

  /**
   * Data provider for testGenerate.
   */
  public function generateDataProvider() {
    return [
      'basic case' => [/* test data */],
      'edge case' => [/* test data */],
    ];
  }
}
```

## Continuous Integration

The LangChain module uses CI/CD pipelines to enforce code coverage requirements.

### CI Workflow

1. Run all tests with coverage
2. Generate Clover XML report
3. Analyze coverage metrics
4. Fail build if coverage thresholds aren't met

### Coverage Enforcement

- Pull requests must maintain or increase coverage percentage
- New code must have 100% coverage
- Overall coverage must not drop below 95%

### CI Configuration

The CI pipeline uses the following commands:

```bash
# Install dependencies
composer install

# Run PHPUnit with coverage
XDEBUG_MODE=coverage ./vendor/bin/phpunit --coverage-clover coverage/clover.xml

# Analyze coverage results
php coverage-check.php coverage/clover.xml 95
```

## Additional Resources

- [Test Documentation](../testing) - General testing guidelines
- [Controller Tests](../controller-tests) - Testing controller classes
- [Plugin Tests](../plugin-tests) - Testing plugin implementations

### Test Coverage Specifics

- [Code Coverage](testing/code-coverage) - Detailed coverage strategy
- [Coverage Exemptions](testing/coverage-exemptions) - List of exempted code
- [Coverage History](testing/coverage-history) - Historical coverage metrics
