---
id: operations-deployment
---
# Deployment Guides

This document provides comprehensive deployment guides for the LLM Platform AI Platform components.

## Deployment Options

### Recipe-Based Deployment (Recommended for Enterprise)

The fastest way to deploy the complete AI platform:

```bash
# Complete AI platform in 5 minutes
composer create-project bluefly/llm_platform_recipe my-ai-platform
cd my-ai-platform

# Apply the LLM Platform recipe
./vendor/bin/drush recipe:apply llm_platform

# Configure for production
./vendor/bin/drush llm:setup-wizard --environment=production
```

### Kubernetes Production Deployment

For container-based enterprise deployments:

```bash
# Enterprise container orchestration
helm install bluefly-ai charts/bluefly-ai \
  --set global.environment=production \
  --set scaling.enabled=true \
  --set monitoring.enabled=true
```

### Individual Service Deployment

For deploying specific microservices:

#### API Gateway (bfapi)

```bash
cd bfapi

# Build Docker image
npm run docker:build

# Deploy to Kubernetes
npm run helm:upgrade

# Or run locally
npm run build:prod
npm run start
```

#### LLM Inference Engine (bfllm)

```bash
cd bfllm

# Build Docker image
npm run docker:build

# Deploy to Kubernetes
npm run helm:upgrade

# Or run locally
npm run build:prod
npm run start
```

#### MCP Protocol Server (llm-mcp)

```bash
cd llm-mcp

# Build Docker image
npm run docker:build

# Deploy to Kubernetes
npm run helm:upgrade

# Or run locally
npm run build:prod
npm run start
```

#### Content Engine (bfrfp)

```bash
cd bfrfp

# Build Docker image
npm run docker:build

# Deploy to Kubernetes
npm run helm:upgrade

# Or run locally
npm run build:prod
npm run start
```

## Production Configuration

### Environment Configuration

Each service supports the following environment configuration:

```bash
# Configure service for specific environment
NODE_ENV=production npm run start

# Set log level
LOG_LEVEL=info npm run start

# Configure Redis connection for caching
REDIS_URL=redis://redis:6379 npm run start

# Configure JWT secret for authentication
JWT_SECRET=your-secure-secret npm run start
```

### Scaling Configuration

```yaml
# Example Kubernetes HPA configuration
apiVersion: autoscaling/v2
kind: HorizontalPodAutoscaler
metadata:
  name: bfapi-hpa
spec:
  scaleTargetRef:
    apiVersion: apps/v1
    kind: Deployment
    name: bfapi
  minReplicas: 3
  maxReplicas: 10
  metrics:
  - type: Resource
    resource:
      name: cpu
      target:
        type: Utilization
        averageUtilization: 75
  - type: Resource
    resource:
      name: memory
      target:
        type: Utilization
        averageUtilization: 80
```

### Security Configuration

```bash
# Configure HTTPS
SSL_CERT_PATH=/path/to/cert.pem
SSL_KEY_PATH=/path/to/key.pem

# Configure CORS
CORS_ALLOWED_ORIGINS=https://yourdomain.com,https://anotherdomain.com

# Configure rate limiting
RATE_LIMIT_WINDOW_MS=60000  # 1 minute
RATE_LIMIT_MAX_REQUESTS=100  # 100 requests per minute
```

## Monitoring Setup

### Prometheus & Grafana

```bash
# Deploy monitoring stack
cd bfcicomponents
npm run deploy:monitoring

# Access Grafana dashboards
echo "Grafana URL: http://grafana.yourdomain.com"
echo "Default credentials: admin/admin"
```

### Logging Configuration

```bash
# Configure centralized logging
cd bfapi
npm run deploy:logging

# Access logs in Loki
echo "Loki URL: http://loki.yourdomain.com"
```

## Deployment Validation

After deployment, validate that all services are functioning correctly:

```bash
# Check service health
curl https://api.yourdomain.com/health

# Validate OpenAPI specifications
npm run openapi:validate

# Run integration tests against deployed services
npm run test:integration
```

## Troubleshooting

### Common Issues

1. **Connection Refused**: Check service is running and ports are correctly configured
2. **Authentication Failure**: Verify JWT secrets match across services
3. **Missing Models**: Ensure model registry is properly configured
4. **Performance Issues**: Check resource allocation and scaling configuration

### Logs and Diagnostics

```bash
# View service logs
kubectl logs -f deployment/bfapi

# Get service status
kubectl describe pod -l app=bfapi

# Check resource usage
kubectl top pods -l app=bfapi
```

## Production Benefits

- **High Availability**: Load balancing, health checks, failover
- **Security**: JWT/OAuth2, RBAC, automated vulnerability scanning
- **Performance**: Response caching, connection pooling, CDN integration
- **Monitoring**: OpenTelemetry, Prometheus, Grafana dashboards
- **Compliance**: Audit logging, data protection, regulatory compliance
