---
id: operations-monitoring-observability
---
# Monitoring and Observability Guide

This document provides comprehensive guidance for implementing monitoring and observability across the LLM Platform AI Platform.

## Monitoring Architecture

The platform uses a multi-layered monitoring architecture:

```
┌─────────────────┐      ┌────────────────┐      ┌───────────────┐
│  Metrics Layer  │──────│  Logging Layer  │──────│  Tracing Layer│
└─────────────────┘      └────────────────┘      └───────────────┘
         │                       │                       │
         ▼                       ▼                       ▼
┌─────────────────┐      ┌────────────────┐      ┌───────────────┐
│    Prometheus   │      │      Loki      │      │     Jaeger    │
└─────────────────┘      └────────────────┘      └───────────────┘
         │                       │                       │
         └───────────────────────┼───────────────────────┘
                                 │
                                 ▼
                         ┌────────────────┐
                         │     Grafana    │
                         └────────────────┘
                                 │
                                 ▼
                         ┌────────────────┐
                         │ Alert Manager  │
                         └────────────────┘
```

## Metrics Collection

### OpenTelemetry Integration

All services expose metrics using OpenTelemetry:

```javascript
// Initialize OpenTelemetry
const { MeterProvider } = require('@opentelemetry/sdk-metrics');
const { Resource } = require('@opentelemetry/resources');
const { SemanticResourceAttributes } = require('@opentelemetry/semantic-conventions');
const { PrometheusExporter } = require('@opentelemetry/exporter-prometheus');

// Create a resource that identifies your service
const resource = new Resource({
  [SemanticResourceAttributes.SERVICE_NAME]: 'bfapi',
  [SemanticResourceAttributes.SERVICE_VERSION]: '4.0.0',
  [SemanticResourceAttributes.DEPLOYMENT_ENVIRONMENT]: process.env.NODE_ENV || 'development',
});

// Create Prometheus exporter
const exporter = new PrometheusExporter({
  port: 9464,
  startServer: true,
});

// Create meter provider
const meterProvider = new MeterProvider({
  resource: resource,
});
meterProvider.addMetricExporter(exporter);

// Create meter
const meter = meterProvider.getMeter('api-metrics');

// Register global provider
opentelemetry.metrics.setGlobalMeterProvider(meterProvider);
```

### Core Metrics

Implement the following metrics for all services:

#### HTTP Server Metrics

```javascript
// Request counter
const httpRequestCounter = meter.createCounter('http_requests_total', {
  description: 'Total number of HTTP requests',
});

// Request duration histogram
const httpRequestDuration = meter.createHistogram('http_request_duration_seconds', {
  description: 'Duration of HTTP requests in seconds',
});

// Active requests gauge
const httpRequestsActive = meter.createUpDownCounter('http_requests_active', {
  description: 'Number of currently active HTTP requests',
});

// Error counter
const httpRequestErrors = meter.createCounter('http_request_errors_total', {
  description: 'Total number of HTTP request errors',
});

// Middleware to track metrics
function metricsMiddleware(req, res, next) {
  const startTime = performance.now();
  httpRequestsActive.add(1);
  
  res.on('finish', () => {
    const duration = (performance.now() - startTime) / 1000;
    httpRequestCounter.add(1, {
      method: req.method,
      route: req.route?.path || 'unknown',
      status_code: res.statusCode.toString(),
    });
    
    httpRequestDuration.record(duration, {
      method: req.method,
      route: req.route?.path || 'unknown',
      status_code: res.statusCode.toString(),
    });
    
    httpRequestsActive.add(-1);
    
    if (res.statusCode >= 400) {
      httpRequestErrors.add(1, {
        method: req.method,
        route: req.route?.path || 'unknown',
        status_code: res.statusCode.toString(),
      });
    }
  });
  
  next();
}
```

#### System Metrics

```javascript
// CPU usage gauge
const cpuUsage = meter.createObservableGauge('system_cpu_usage', {
  description: 'CPU usage percentage',
});

// Memory usage gauge
const memoryUsage = meter.createObservableGauge('system_memory_usage_bytes', {
  description: 'Memory usage in bytes',
});

// Collect CPU and memory metrics
const collectSystemMetrics = () => {
  const cpuUsageValue = process.cpuUsage();
  const memUsageValue = process.memoryUsage();
  
  cpuUsage.record((cpuUsageValue.user + cpuUsageValue.system) / 1000000);
  memoryUsage.record(memUsageValue.rss);
};

// Update metrics every 5 seconds
setInterval(collectSystemMetrics, 5000);
```

### Service-Specific Metrics

#### bfapi Metrics

```javascript
// Rate limiting metrics
const rateLimitHits = meter.createCounter('rate_limit_hits_total', {
  description: 'Total number of rate limit hits',
});

// Authentication metrics
const authFailures = meter.createCounter('auth_failures_total', {
  description: 'Total number of authentication failures',
});

// Circuit breaker metrics
const circuitBreakerState = meter.createUpDownCounter('circuit_breaker_state', {
  description: 'Circuit breaker state (1 = open, 0 = closed)',
});

// Connection pool metrics
const connectionPoolActive = meter.createObservableGauge('connection_pool_active_connections', {
  description: 'Number of active connections in the pool',
});
```

#### bfllm Metrics

```javascript
// Inference metrics
const inferenceRequests = meter.createCounter('inference_requests_total', {
  description: 'Total number of inference requests',
});

const inferenceLatency = meter.createHistogram('inference_latency_seconds', {
  description: 'Inference latency in seconds',
});

const inferenceTokens = meter.createCounter('inference_tokens_total', {
  description: 'Total number of tokens processed',
  valueType: opentelemetry.metrics.ValueType.INT,
});

// Model loading metrics
const modelLoadingTime = meter.createHistogram('model_loading_time_seconds', {
  description: 'Model loading time in seconds',
});

const modelLoadingErrors = meter.createCounter('model_loading_errors_total', {
  description: 'Total number of model loading errors',
});
```

#### llm-mcp Metrics

```javascript
// Tool execution metrics
const toolExecutions = meter.createCounter('tool_executions_total', {
  description: 'Total number of tool executions',
});

const toolExecutionLatency = meter.createHistogram('tool_execution_latency_seconds', {
  description: 'Tool execution latency in seconds',
});

const toolExecutionErrors = meter.createCounter('tool_execution_errors_total', {
  description: 'Total number of tool execution errors',
});

// Tool discovery metrics
const toolDiscoveryLatency = meter.createHistogram('tool_discovery_latency_seconds', {
  description: 'Tool discovery latency in seconds',
});

// Protocol message metrics
const protocolMessages = meter.createCounter('protocol_messages_total', {
  description: 'Total number of protocol messages',
});
```

#### bfrfp Metrics

```javascript
// Document generation metrics
const documentGenerations = meter.createCounter('document_generations_total', {
  description: 'Total number of document generations',
});

const documentGenerationLatency = meter.createHistogram('document_generation_latency_seconds', {
  description: 'Document generation latency in seconds',
});

// Content processing metrics
const contentProcessingSize = meter.createHistogram('content_processing_size_bytes', {
  description: 'Size of processed content in bytes',
});
```

## Logging System

### Centralized Logging with ELK/Loki

Use structured logging with correlation IDs:

```javascript
const winston = require('winston');
const { format } = winston;

// Define log format
const logFormat = format.combine(
  format.timestamp(),
  format.json()
);

// Create logger
const logger = winston.createLogger({
  level: process.env.LOG_LEVEL || 'info',
  format: logFormat,
  defaultMeta: { service: 'bfapi' },
  transports: [
    new winston.transports.Console(),
    new winston.transports.File({ filename: 'logs/error.log', level: 'error' }),
    new winston.transports.File({ filename: 'logs/combined.log' }),
  ],
});

// Add request ID to logs
function requestLogger(req, res, next) {
  const requestId = req.headers['x-request-id'] || uuidv4();
  req.requestId = requestId;
  res.setHeader('x-request-id', requestId);
  
  req.logger = logger.child({ requestId });
  req.logger.info({
    message: 'Request received',
    method: req.method,
    url: req.url,
    ip: req.ip,
    userAgent: req.headers['user-agent'],
  });
  
  const start = Date.now();
  res.on('finish', () => {
    const duration = Date.now() - start;
    req.logger.info({
      message: 'Request completed',
      method: req.method,
      url: req.url,
      statusCode: res.statusCode,
      duration,
    });
  });
  
  next();
}
```

### Log Levels and Categories

Use consistent log levels across services:

| Level | Usage |
|-------|-------|
| error | Errors that prevent operation |
| warn | Anomalies that don't prevent operation |
| info | Significant events |
| debug | Detailed information for debugging |
| trace | Very detailed diagnostics |

### Sensitive Data Handling

Implement log scrubbing:

```javascript
// Log scrubbing for sensitive data
const sensitiveFields = ['password', 'token', 'key', 'secret', 'credit_card'];

function scrubSensitiveData(obj) {
  if (!obj) return obj;
  
  const scrubbed = { ...obj };
  
  for (const key in scrubbed) {
    if (sensitiveFields.some(field => key.toLowerCase().includes(field))) {
      scrubbed[key] = '********';
    } else if (typeof scrubbed[key] === 'object') {
      scrubbed[key] = scrubSensitiveData(scrubbed[key]);
    }
  }
  
  return scrubbed;
}

// Use in logging
logger.info('User data', scrubSensitiveData(userData));
```

## Distributed Tracing

### Jaeger Integration

Implement OpenTelemetry tracing:

```javascript
const { NodeTracerProvider } = require('@opentelemetry/sdk-trace-node');
const { SimpleSpanProcessor } = require('@opentelemetry/sdk-trace-base');
const { JaegerExporter } = require('@opentelemetry/exporter-jaeger');
const { registerInstrumentations } = require('@opentelemetry/instrumentation');
const { HttpInstrumentation } = require('@opentelemetry/instrumentation-http');
const { ExpressInstrumentation } = require('@opentelemetry/instrumentation-express');
const { Resource } = require('@opentelemetry/resources');
const { SemanticResourceAttributes } = require('@opentelemetry/semantic-conventions');

// Create a tracer provider
const provider = new NodeTracerProvider({
  resource: new Resource({
    [SemanticResourceAttributes.SERVICE_NAME]: 'bfapi',
    [SemanticResourceAttributes.SERVICE_VERSION]: '4.0.0',
    [SemanticResourceAttributes.DEPLOYMENT_ENVIRONMENT]: process.env.NODE_ENV || 'development',
  }),
});

// Configure span processor and exporter
const jaegerExporter = new JaegerExporter({
  endpoint: process.env.JAEGER_ENDPOINT || 'http://jaeger:14268/api/traces',
});

provider.addSpanProcessor(new SimpleSpanProcessor(jaegerExporter));
provider.register();

// Register instrumentations
registerInstrumentations({
  instrumentations: [
    new HttpInstrumentation(),
    new ExpressInstrumentation(),
  ],
});

// Get tracer
const tracer = provider.getTracer('bfapi-tracer');

// Custom span creation
function createCustomSpan(name, fn) {
  return tracer.startActiveSpan(name, async (span) => {
    try {
      const result = await fn();
      span.end();
      return result;
    } catch (error) {
      span.setStatus({ code: opentelemetry.SpanStatusCode.ERROR });
      span.recordException(error);
      span.end();
      throw error;
    }
  });
}
```

### Cross-Service Tracing

Propagate context between services:

```javascript
const { context, propagation } = require('@opentelemetry/api');

// HTTP client with context propagation
async function makeRequest(url, options = {}) {
  return new Promise((resolve, reject) => {
    const currentContext = context.active();
    const headers = { ...options.headers } || {};
    
    // Inject trace context into headers
    propagation.inject(currentContext, headers);
    
    const req = http.request(url, { ...options, headers }, (res) => {
      let data = '';
      res.on('data', (chunk) => {
        data += chunk;
      });
      res.on('end', () => {
        resolve({ statusCode: res.statusCode, data });
      });
    });
    
    req.on('error', reject);
    req.end(options.body);
  });
}
```

## Alert Management

### Prometheus Alerting Rules

```yaml
# prometheus/alert-rules.yml
groups:
  - name: service-availability
    rules:
      - alert: ServiceDown
        expr: up == 0
        for: 1m
        labels:
          severity: critical
        annotations:
          summary: "Service {{ $labels.instance }} is down"
          description: "Service {{ $labels.instance }} has been down for more than 1 minute."

      - alert: HighErrorRate
        expr: rate(http_request_errors_total[5m]) / rate(http_requests_total[5m]) > 0.05
        for: 2m
        labels:
          severity: warning
        annotations:
          summary: "High error rate for {{ $labels.instance }}"
          description: "Error rate is above 5% for service {{ $labels.instance }} for more than 2 minutes."

      - alert: HighLatency
        expr: histogram_quantile(0.95, rate(http_request_duration_seconds_bucket[5m])) > 0.5
        for: 5m
        labels:
          severity: warning
        annotations:
          summary: "High latency for {{ $labels.instance }}"
          description: "95th percentile latency is above 500ms for service {{ $labels.instance }} for more than 5 minutes."

      - alert: HighCpuUsage
        expr: avg by(instance) (system_cpu_usage) > 0.85
        for: 5m
        labels:
          severity: warning
        annotations:
          summary: "High CPU usage for {{ $labels.instance }}"
          description: "CPU usage is above 85% for service {{ $labels.instance }} for more than 5 minutes."

      - alert: HighMemoryUsage
        expr: avg by(instance) (system_memory_usage_bytes / node_memory_MemTotal_bytes) > 0.85
        for: 5m
        labels:
          severity: warning
        annotations:
          summary: "High memory usage for {{ $labels.instance }}"
          description: "Memory usage is above 85% for service {{ $labels.instance }} for more than 5 minutes."
```

### Service-Specific Alerts

#### bfllm Alerts

```yaml
# prometheus/bfllm-alerts.yml
groups:
  - name: bfllm-alerts
    rules:
      - alert: ModelLoadingError
        expr: increase(model_loading_errors_total[15m]) > 0
        for: 5m
        labels:
          severity: critical
          service: bfllm
        annotations:
          summary: "Model loading errors detected"
          description: "There have been model loading errors in the last 15 minutes."

      - alert: HighInferenceLatency
        expr: histogram_quantile(0.95, rate(inference_latency_seconds_bucket[5m])) > 2
        for: 5m
        labels:
          severity: warning
          service: bfllm
        annotations:
          summary: "High inference latency"
          description: "95th percentile inference latency is above 2 seconds for more than 5 minutes."
```

#### llm-mcp Alerts

```yaml
# prometheus/llm-mcp-alerts.yml
groups:
  - name: llm-mcp-alerts
    rules:
      - alert: HighToolExecutionErrors
        expr: increase(tool_execution_errors_total[15m]) / increase(tool_executions_total[15m]) > 0.1
        for: 5m
        labels:
          severity: warning
          service: llm-mcp
        annotations:
          summary: "High tool execution error rate"
          description: "Tool execution error rate is above 10% for more than 5 minutes."

      - alert: SlowToolDiscovery
        expr: histogram_quantile(0.95, rate(tool_discovery_latency_seconds_bucket[5m])) > 0.5
        for: 5m
        labels:
          severity: warning
          service: llm-mcp
        annotations:
          summary: "Slow tool discovery"
          description: "95th percentile tool discovery latency is above 500ms for more than 5 minutes."
```

### Alert Manager Configuration

```yaml
# alertmanager/config.yml
global:
  resolve_timeout: 5m

route:
  group_by: ['alertname', 'service']
  group_wait: 30s
  group_interval: 5m
  repeat_interval: 1h
  receiver: 'team-email'
  routes:
  - match:
      severity: critical
    receiver: 'team-pager'
    continue: true

receivers:
- name: 'team-email'
  email_configs:
  - to: 'team@example.com'
    from: 'alertmanager@example.com'
    smarthost: 'smtp.example.com:587'
    auth_username: 'alertmanager'
    auth_password: '{{ .SMTP_PASSWORD }}'
    require_tls: true

- name: 'team-pager'
  pagerduty_configs:
  - service_key: '{{ .PAGERDUTY_KEY }}'
```

## SLOs and SLIs

Define Service Level Objectives and Indicators:

### API Gateway SLOs

```yaml
# prometheus/slo-rules.yml
groups:
  - name: slo-rules
    rules:
      - record: slo:availability:ratio
        expr: sum(rate(http_requests_total{status_code=~"[2-3].."}[5m])) / sum(rate(http_requests_total[5m]))

      - record: slo:latency:ratio
        expr: sum(rate(http_request_duration_seconds_bucket{le="0.2"}[5m])) / sum(rate(http_request_duration_seconds_count[5m]))

      - alert: SLOAvailabilityBudgetBurning
        expr: slo:availability:ratio < 0.995
        for: 15m
        labels:
          severity: warning
        annotations:
          summary: "SLO availability budget burning fast"
          description: "Service availability is below target of 99.5% for more than 15 minutes."

      - alert: SLOLatencyBudgetBurning
        expr: slo:latency:ratio < 0.95
        for: 15m
        labels:
          severity: warning
        annotations:
          summary: "SLO latency budget burning fast"
          description: "Less than 95% of requests are completing within 200ms for more than 15 minutes."
```

## Dashboards

### Grafana Dashboard Examples

#### API Gateway Dashboard

```json
{
  "title": "API Gateway Dashboard",
  "panels": [
    {
      "title": "Requests per Second",
      "type": "graph",
      "datasource": "Prometheus",
      "targets": [
        {
          "expr": "sum(rate(http_requests_total[1m])) by (route)",
          "legendFormat": "{{route}}"
        }
      ]
    },
    {
      "title": "Request Duration (95th Percentile)",
      "type": "graph",
      "datasource": "Prometheus",
      "targets": [
        {
          "expr": "histogram_quantile(0.95, sum(rate(http_request_duration_seconds_bucket[1m])) by (le, route))",
          "legendFormat": "{{route}}"
        }
      ]
    },
    {
      "title": "Error Rate",
      "type": "graph",
      "datasource": "Prometheus",
      "targets": [
        {
          "expr": "sum(rate(http_request_errors_total[1m])) by (route) / sum(rate(http_requests_total[1m])) by (route)",
          "legendFormat": "{{route}}"
        }
      ]
    },
    {
      "title": "Active Requests",
      "type": "gauge",
      "datasource": "Prometheus",
      "targets": [
        {
          "expr": "sum(http_requests_active)",
          "legendFormat": "Active Requests"
        }
      ]
    }
  ]
}
```

#### LLM Inference Dashboard

```json
{
  "title": "LLM Inference Dashboard",
  "panels": [
    {
      "title": "Inference Requests per Second",
      "type": "graph",
      "datasource": "Prometheus",
      "targets": [
        {
          "expr": "sum(rate(inference_requests_total[1m])) by (model)",
          "legendFormat": "{{model}}"
        }
      ]
    },
    {
      "title": "Inference Latency (95th Percentile)",
      "type": "graph",
      "datasource": "Prometheus",
      "targets": [
        {
          "expr": "histogram_quantile(0.95, sum(rate(inference_latency_seconds_bucket[1m])) by (le, model))",
          "legendFormat": "{{model}}"
        }
      ]
    },
    {
      "title": "Tokens Processed per Second",
      "type": "graph",
      "datasource": "Prometheus",
      "targets": [
        {
          "expr": "sum(rate(inference_tokens_total[1m])) by (model)",
          "legendFormat": "{{model}}"
        }
      ]
    },
    {
      "title": "Model Loading Errors",
      "type": "stat",
      "datasource": "Prometheus",
      "targets": [
        {
          "expr": "sum(increase(model_loading_errors_total[24h])) by (model)",
          "legendFormat": "{{model}}"
        }
      ]
    }
  ]
}
```

## Health Checks

Implement comprehensive health checks for all services:

```javascript
// Health check endpoint
app.get('/health', async (req, res) => {
  try {
    // Basic health check
    const health = {
      status: 'UP',
      timestamp: new Date(),
      service: 'bfapi',
      version: '4.0.0',
      checks: []
    };
    
    // Database check
    try {
      await db.ping();
      health.checks.push({ component: 'database', status: 'UP' });
    } catch (error) {
      health.checks.push({ component: 'database', status: 'DOWN', error: error.message });
      health.status = 'DOWN';
    }
    
    // Redis check
    try {
      await redis.ping();
      health.checks.push({ component: 'redis', status: 'UP' });
    } catch (error) {
      health.checks.push({ component: 'redis', status: 'DOWN', error: error.message });
      health.status = 'DOWN';
    }
    
    // External service checks
    for (const service of ['bfllm', 'llm-mcp', 'bfrfp']) {
      try {
        const response = await axios.get(`http://${service}/health`);
        if (response.data.status === 'UP') {
          health.checks.push({ component: service, status: 'UP' });
        } else {
          health.checks.push({ component: service, status: 'DOWN' });
          health.status = 'DOWN';
        }
      } catch (error) {
        health.checks.push({ component: service, status: 'DOWN', error: error.message });
        health.status = 'DOWN';
      }
    }
    
    // Return health status
    res.status(health.status === 'UP' ? 200 : 503).json(health);
  } catch (error) {
    res.status(500).json({
      status: 'DOWN',
      timestamp: new Date(),
      service: 'bfapi',
      error: error.message
    });
  }
});
```

## Kubernetes Integration

### Prometheus Operator

Deploy using Prometheus Operator:

```yaml
# kubernetes/prometheus-operator.yaml
apiVersion: monitoring.coreos.com/v1
kind: Prometheus
metadata:
  name: bluefly-prometheus
spec:
  serviceAccountName: prometheus
  serviceMonitorSelector:
    matchLabels:
      app: bluefly
  ruleSelector:
    matchLabels:
      app: bluefly
  alerting:
    alertmanagers:
    - name: alertmanager
      namespace: monitoring
      port: web
  resources:
    requests:
      memory: 400Mi
    limits:
      memory: 2Gi
  retention: 15d
  storage:
    volumeClaimTemplate:
      spec:
        resources:
          requests:
            storage: 50Gi
```

### Service Monitors

Define service monitors for each component:

```yaml
# kubernetes/service-monitor.yaml
apiVersion: monitoring.coreos.com/v1
kind: ServiceMonitor
metadata:
  name: bfapi-monitor
  labels:
    app: bluefly
spec:
  selector:
    matchLabels:
      app: bfapi
  endpoints:
  - port: metrics
    interval: 15s
    path: /metrics
```

## Practical Implementation Steps

1. **Set Up Infrastructure**:
   - Deploy Prometheus, Grafana, Loki, and Jaeger
   - Configure service discovery
   - Set up alert manager

2. **Instrument Services**:
   - Add OpenTelemetry instrumentation to all services
   - Implement structured logging
   - Add health check endpoints

3. **Configure Dashboards**:
   - Create service-specific dashboards
   - Configure SLO dashboards
   - Set up user journey dashboards

4. **Set Up Alerts**:
   - Define alerting rules
   - Configure notification channels
   - Implement on-call rotation

5. **Establish Monitoring Practices**:
   - Regular review of metrics and logs
   - Incident response procedures
   - Continuous improvement of monitoring

## Observability Best Practices

1. **Instrument Everything**:
   - All services should expose metrics
   - Use structured logging with context
   - Implement distributed tracing

2. **Establish Baselines**:
   - Measure normal performance
   - Set up anomaly detection
   - Define clear SLOs

3. **Use the RED Method**:
   - Rate (requests per second)
   - Errors (failed requests per second)
   - Duration (distribution of request durations)

4. **Use the USE Method for Resources**:
   - Utilization (percent time the resource is busy)
   - Saturation (amount of work resource has to do)
   - Errors (count of error events)

5. **Focus on User Experience**:
   - Monitor end-to-end request flows
   - Track user-facing metrics
   - Define and monitor SLOs based on user experience

6. **Implement Cost Monitoring**:
   - Track resource usage costs
   - Monitor token usage for AI models
   - Implement cost optimization alerts
