---
id: operations-performance-optimization
---
# Performance Optimization Guide

This document provides strategies and best practices for optimizing the performance of the LLM Platform AI Platform.

## Performance Goals

| Service | Metric | Target |
|---------|--------|--------|
| API Gateway (bfapi) | Response Time (P95) | < 200ms |
| API Gateway (bfapi) | Throughput | > 100 RPS |
| LLM Inference (bfllm) | Inference Time (P95) | < 1000ms |
| LLM Inference (bfllm) | Tokens/Second | > 50 tokens/sec |
| MCP Protocol (llm-mcp) | Tool Discovery | < 100ms |
| MCP Protocol (llm-mcp) | Tool Execution | < 500ms |
| Content Engine (bfrfp) | Document Generation | < 3000ms |

## General Optimization Strategies

### 1. Caching

Implement caching at multiple levels:

#### Response Caching

```javascript
// Redis-based response caching
const cache = require('./cache');

async function getResource(id) {
  // Check cache first
  const cachedResource = await cache.get(`resource:${id}`);
  if (cachedResource) {
    return JSON.parse(cachedResource);
  }
  
  // If not in cache, fetch from database
  const resource = await db.getResource(id);
  
  // Store in cache for future requests
  await cache.set(`resource:${id}`, JSON.stringify(resource), 60 * 5); // 5 minutes TTL
  
  return resource;
}
```

#### In-Memory Caching

```javascript
// LRU cache for frequent computations
const LRU = require('lru-cache');
const cache = new LRU({
  max: 500, // Maximum items in cache
  ttl: 1000 * 60 * 5, // 5 minutes TTL
});

function expensiveComputation(input) {
  const cacheKey = JSON.stringify(input);
  
  // Check cache
  if (cache.has(cacheKey)) {
    return cache.get(cacheKey);
  }
  
  // Perform computation
  const result = /* expensive computation */;
  
  // Store in cache
  cache.set(cacheKey, result);
  
  return result;
}
```

#### Distributed Caching

Use Redis for distributed caching to share cache across multiple instances:

```javascript
const Redis = require('ioredis');
const redis = new Redis({
  host: process.env.REDIS_HOST,
  port: process.env.REDIS_PORT,
  password: process.env.REDIS_PASSWORD,
});

// Set cache with expiration
async function setCache(key, value, ttlSeconds) {
  await redis.set(key, JSON.stringify(value), 'EX', ttlSeconds);
}

// Get from cache
async function getCache(key) {
  const value = await redis.get(key);
  return value ? JSON.parse(value) : null;
}
```

### 2. Database Optimization

#### Connection Pooling

```javascript
const { Pool } = require('pg');

const pool = new Pool({
  host: process.env.DB_HOST,
  user: process.env.DB_USER,
  password: process.env.DB_PASSWORD,
  database: process.env.DB_NAME,
  min: 2, // Minimum pool size
  max: 10, // Maximum pool size
  idleTimeoutMillis: 30000, // Connection timeout
});
```

#### Query Optimization

- Use indexes for frequently queried fields
- Optimize complex queries
- Use query parameterization to enable query planning
- Implement query timeouts

```javascript
async function getUsers(filters) {
  // Use parameterized queries
  const query = {
    text: 'SELECT * FROM users WHERE role = $1 AND active = $2 LIMIT $3',
    values: [filters.role, true, filters.limit || 10],
  };
  
  // Set query timeout
  const result = await pool.query({
    ...query,
    timeout: 5000, // 5 second timeout
  });
  
  return result.rows;
}
```

#### Database Sharding

For high-volume data, consider implementing database sharding:

```javascript
function getDatabaseForUser(userId) {
  const shardId = userId % NUM_SHARDS;
  return dbPools[shardId];
}

async function getUserData(userId) {
  const db = getDatabaseForUser(userId);
  return db.query('SELECT * FROM users WHERE id = $1', [userId]);
}
```

### 3. Asynchronous Processing

Use message queues for non-blocking processing:

```javascript
// Using Bull queue with Redis
const Queue = require('bull');
const documentQueue = new Queue('document-processing', {
  redis: {
    host: process.env.REDIS_HOST,
    port: process.env.REDIS_PORT,
    password: process.env.REDIS_PASSWORD,
  }
});

// Add job to queue
async function generateDocument(data) {
  const job = await documentQueue.add(data, {
    attempts: 3,
    backoff: {
      type: 'exponential',
      delay: 1000,
    },
  });
  
  return { jobId: job.id };
}

// Process jobs in the queue
documentQueue.process(async (job) => {
  const { data } = job;
  // Process document
  const result = await processDocument(data);
  return result;
});
```

### 4. Code Optimization

#### Lazy Loading

```javascript
// Lazy loading expensive modules
let expensiveModule;

function getExpensiveModule() {
  if (!expensiveModule) {
    expensiveModule = require('expensive-module');
  }
  return expensiveModule;
}
```

#### Efficient Data Structures

Choose appropriate data structures for operations:

```javascript
// Use Set for fast lookups
const allowedTypes = new Set(['text', 'image', 'audio', 'video']);

function isValidType(type) {
  return allowedTypes.has(type);
}

// Use Map for key-value operations
const userCache = new Map();

function getUserFromCache(id) {
  return userCache.get(id);
}
```

#### Memory Management

```javascript
// Stream large files instead of loading into memory
function streamLargeFile(filePath, res) {
  const fileStream = fs.createReadStream(filePath);
  fileStream.pipe(res);
}

// Use buffer pooling for frequent operations
const bufferPool = Buffer.allocUnsafe(1024 * 1024); // 1MB buffer
```

## Service-Specific Optimizations

### API Gateway (bfapi)

#### Load Balancing

Implement intelligent load balancing:

```javascript
class IntelligentLoadBalancer {
  constructor(backends) {
    this.backends = backends;
    this.stats = backends.reduce((acc, backend) => {
      acc[backend.id] = { responseTime: 0, errorRate: 0, load: 0 };
      return acc;
    }, {});
  }
  
  selectBackend(request) {
    // Consider multiple factors for selection
    const scores = this.backends.map(backend => {
      const stats = this.stats[backend.id];
      const responseTimeScore = 1 / (stats.responseTime || 1);
      const errorRateScore = 1 - stats.errorRate;
      const loadScore = 1 / (stats.load || 1);
      
      // Weight factors based on importance
      return {
        backend,
        score: (responseTimeScore * 0.4) + (errorRateScore * 0.4) + (loadScore * 0.2)
      };
    });
    
    // Sort by score (highest first)
    scores.sort((a, b) => b.score - a.score);
    
    return scores[0].backend;
  }
  
  updateStats(backendId, metrics) {
    this.stats[backendId] = {
      ...this.stats[backendId],
      ...metrics,
    };
  }
}
```

#### Circuit Breaker

Implement circuit breaker pattern for resilience:

```javascript
class CircuitBreaker {
  constructor(service, options) {
    this.service = service;
    this.state = 'CLOSED';
    this.failureCount = 0;
    this.successCount = 0;
    this.lastFailureTime = null;
    this.options = {
      failureThreshold: 5,
      resetTimeout: 30000, // 30 seconds
      halfOpenSuccessThreshold: 3,
      ...options,
    };
  }
  
  async execute(request) {
    if (this.state === 'OPEN') {
      // Check if reset timeout has elapsed
      const now = Date.now();
      if (now - this.lastFailureTime > this.options.resetTimeout) {
        this.state = 'HALF_OPEN';
        this.successCount = 0;
      } else {
        throw new Error('Circuit breaker is open');
      }
    }
    
    try {
      const result = await this.service.execute(request);
      
      // On success in half-open state
      if (this.state === 'HALF_OPEN') {
        this.successCount++;
        if (this.successCount >= this.options.halfOpenSuccessThreshold) {
          this.state = 'CLOSED';
          this.failureCount = 0;
        }
      }
      
      return result;
    } catch (error) {
      // On failure
      this.failureCount++;
      this.lastFailureTime = Date.now();
      
      if (this.state === 'CLOSED' && this.failureCount >= this.options.failureThreshold) {
        this.state = 'OPEN';
      }
      
      throw error;
    }
  }
}
```

#### API Rate Limiting

```javascript
const rateLimit = require('express-rate-limit');
const RedisStore = require('rate-limit-redis');
const Redis = require('ioredis');

const redisClient = new Redis({
  host: process.env.REDIS_HOST,
  port: process.env.REDIS_PORT,
  password: process.env.REDIS_PASSWORD,
});

// Apply rate limiting middleware
app.use(
  rateLimit({
    store: new RedisStore({
      client: redisClient,
      prefix: 'rate-limit:',
    }),
    windowMs: 60 * 1000, // 1 minute
    max: 100, // 100 requests per minute
    standardHeaders: true,
    keyGenerator: (req) => {
      // Use API key or IP address
      return req.get('X-API-Key') || req.ip;
    },
  })
);
```

### LLM Inference (bfllm)

#### Model Quantization

Use quantized models for faster inference:

```javascript
async function loadQuantizedModel(modelPath) {
  // Load ONNX model with quantization
  const session = await ort.InferenceSession.create(modelPath, {
    executionProviders: ['CPUExecutionProvider'],
    graphOptimizationLevel: 'ORT_ENABLE_ALL',
  });
  
  return session;
}
```

#### Batching Requests

Batch multiple requests for efficient processing:

```javascript
class BatchProcessor {
  constructor(modelService, options) {
    this.modelService = modelService;
    this.queue = [];
    this.processing = false;
    this.options = {
      maxBatchSize: 8,
      maxWaitTime: 100, // ms
      ...options,
    };
  }
  
  async process(input) {
    return new Promise((resolve, reject) => {
      // Add to queue
      this.queue.push({
        input,
        resolve,
        reject,
        timestamp: Date.now(),
      });
      
      // Start processing if not already
      if (!this.processing) {
        this.processBatch();
      }
    });
  }
  
  async processBatch() {
    if (this.processing || this.queue.length === 0) {
      return;
    }
    
    this.processing = true;
    
    // Wait for more requests or max wait time
    if (this.queue.length < this.options.maxBatchSize) {
      const oldestRequest = this.queue[0].timestamp;
      const waitTime = Math.max(0, this.options.maxWaitTime - (Date.now() - oldestRequest));
      
      if (waitTime > 0) {
        await new Promise(resolve => setTimeout(resolve, waitTime));
      }
    }
    
    // Get batch
    const batch = this.queue.splice(0, this.options.maxBatchSize);
    const inputs = batch.map(item => item.input);
    
    try {
      // Process batch
      const results = await this.modelService.processBatch(inputs);
      
      // Resolve individual promises
      batch.forEach((item, index) => {
        item.resolve(results[index]);
      });
    } catch (error) {
      // Reject all promises in case of error
      batch.forEach(item => {
        item.reject(error);
      });
    }
    
    this.processing = false;
    
    // Process next batch if queue not empty
    if (this.queue.length > 0) {
      this.processBatch();
    }
  }
}
```

#### Model Caching

Cache model outputs for common inputs:

```javascript
const modelCache = new LRU({
  max: 1000, // Maximum items in cache
  ttl: 1000 * 60 * 60, // 1 hour TTL
  updateAgeOnGet: true, // Update age on access
});

async function getModelOutput(input, model) {
  const cacheKey = `${model}:${crypto.createHash('md5').update(JSON.stringify(input)).digest('hex')}`;
  
  // Check cache
  if (modelCache.has(cacheKey)) {
    return modelCache.get(cacheKey);
  }
  
  // Generate output
  const output = await generateModelOutput(input, model);
  
  // Cache output
  modelCache.set(cacheKey, output);
  
  return output;
}
```

### Drupal Performance (llm_platform_recipe)

#### Drupal Caching

Enable and configure Drupal's caching mechanisms:

```php
// services.yml configuration
services:
  cache.backend.redis:
    class: Drupal\redis\Cache\PhpRedis
    arguments: ['@redis.factory', '@cache_tags_provider.container', '@serialization.phpserialize']
    tags:
      - { name: cache.bin }
  
  cache.default:
    class: Drupal\Core\Cache\CacheBackendInterface
    tags:
      - { name: cache.bin }
    factory: ['@cache.backend.redis', 'get']
    arguments: ['default']
```

#### Database Query Optimization

Optimize Drupal database queries:

```php
// Use entity queries for efficiency
$query = \Drupal::entityQuery('node')
  ->condition('type', 'article')
  ->condition('status', 1)
  ->accessCheck(TRUE)
  ->range(0, 10)
  ->sort('created', 'DESC');

$nids = $query->execute();

// Load entities in bulk
$nodes = \Drupal\node\Entity\Node::loadMultiple($nids);
```

## Monitoring and Profiling

### Performance Monitoring

Implement comprehensive monitoring:

```javascript
const { MeterProvider } = require('@opentelemetry/sdk-metrics');
const { PrometheusExporter } = require('@opentelemetry/exporter-prometheus');

// Create Prometheus exporter
const exporter = new PrometheusExporter({
  port: 9464,
  startServer: true,
});

// Create meter provider
const meterProvider = new MeterProvider();
meterProvider.addMetricExporter(exporter);

// Create meters
const meter = meterProvider.getMeter('api-metrics');

// Create metrics
const requestDuration = meter.createHistogram('http_request_duration_ms', {
  description: 'Duration of HTTP requests in ms',
});

const activeRequests = meter.createUpDownCounter('http_requests_active', {
  description: 'Number of active HTTP requests',
});

// Use in middleware
function metricsMiddleware(req, res, next) {
  const startTime = performance.now();
  activeRequests.add(1);
  
  // Track response time
  res.on('finish', () => {
    const duration = performance.now() - startTime;
    requestDuration.record(duration, {
      method: req.method,
      route: req.route?.path || 'unknown',
      status_code: res.statusCode,
    });
    activeRequests.add(-1);
  });
  
  next();
}
```

### Profiling Tools

Use profiling tools to identify bottlenecks:

```bash
# CPU profiling with Node.js built-in profiler
node --prof app.js

# Convert profile to readable format
node --prof-process isolate-0xnnnnnnnnnnnn-v8.log > profile.txt

# Memory profiling with heapdump
npm install heapdump

# In your application:
const heapdump = require('heapdump');
heapdump.writeSnapshot('./heap-' + Date.now() + '.heapsnapshot');
```

## Load Testing and Benchmarking

### Load Testing Scripts

Use tools like Artillery for load testing:

```yaml
# artillery.yml
config:
  target: "http://localhost:3000"
  phases:
    - duration: 60
      arrivalRate: 5
      rampTo: 50
      name: "Ramp up load"
    - duration: 120
      arrivalRate: 50
      name: "Sustained load"
  defaults:
    headers:
      Authorization: "Bearer {{token}}"
      Content-Type: "application/json"
  
scenarios:
  - name: "API operations"
    flow:
      - get:
          url: "/api/v1/models"
          
      - post:
          url: "/api/v1/inference"
          json:
            model: "gpt-4"
            prompt: "Hello, world!"
            max_tokens: 100
            
      - get:
          url: "/api/v1/health"
```

Run with:

```bash
artillery run artillery.yml
```

### Benchmarking

Create benchmark scripts to measure performance changes:

```javascript
const Benchmark = require('benchmark');
const suite = new Benchmark.Suite;

// Add benchmark tests
suite
  .add('String concatenation', function() {
    let result = '';
    for (let i = 0; i < 1000; i++) {
      result += i.toString();
    }
  })
  .add('Array join', function() {
    const arr = [];
    for (let i = 0; i < 1000; i++) {
      arr.push(i.toString());
    }
    const result = arr.join('');
  })
  .on('cycle', function(event) {
    console.log(String(event.target));
  })
  .on('complete', function() {
    console.log('Fastest is ' + this.filter('fastest').map('name'));
  })
  .run({ 'async': true });
```

## Auto-Scaling Strategies

### Kubernetes HPA

Configure Horizontal Pod Autoscaler for automatic scaling:

```yaml
apiVersion: autoscaling/v2
kind: HorizontalPodAutoscaler
metadata:
  name: bfapi-hpa
spec:
  scaleTargetRef:
    apiVersion: apps/v1
    kind: Deployment
    name: bfapi
  minReplicas: 3
  maxReplicas: 10
  metrics:
  - type: Resource
    resource:
      name: cpu
      target:
        type: Utilization
        averageUtilization: 75
  - type: Resource
    resource:
      name: memory
      target:
        type: Utilization
        averageUtilization: 80
```

### Predictive Auto-Scaling

Implement predictive scaling based on historical patterns:

```javascript
class PredictiveScaler {
  constructor(kubeClient) {
    this.kubeClient = kubeClient;
    this.metricHistory = {};
  }
  
  // Record metrics for training
  recordMetrics(service, metrics) {
    if (!this.metricHistory[service]) {
      this.metricHistory[service] = [];
    }
    
    this.metricHistory[service].push({
      timestamp: Date.now(),
      ...metrics,
    });
    
    // Keep only last week of data
    const oneWeekAgo = Date.now() - (7 * 24 * 60 * 60 * 1000);
    this.metricHistory[service] = this.metricHistory[service].filter(
      record => record.timestamp >= oneWeekAgo
    );
  }
  
  // Predict future load
  predictLoad(service, timeHorizon) {
    if (!this.metricHistory[service] || this.metricHistory[service].length < 24) {
      return null; // Not enough data
    }
    
    // Get historical data for same day and time
    const targetTime = new Date();
    targetTime.setTime(targetTime.getTime() + timeHorizon);
    const dayOfWeek = targetTime.getDay();
    const hourOfDay = targetTime.getHours();
    
    // Find similar historical data points
    const similarPoints = this.metricHistory[service].filter(record => {
      const recordDate = new Date(record.timestamp);
      return recordDate.getDay() === dayOfWeek && 
             Math.abs(recordDate.getHours() - hourOfDay) <= 1;
    });
    
    if (similarPoints.length < 3) {
      return null; // Not enough similar data points
    }
    
    // Calculate average load
    const avgLoad = similarPoints.reduce((sum, point) => sum + point.load, 0) / similarPoints.length;
    
    // Calculate standard deviation for confidence
    const variance = similarPoints.reduce((sum, point) => sum + Math.pow(point.load - avgLoad, 2), 0) / similarPoints.length;
    const stdDev = Math.sqrt(variance);
    
    // Calculate confidence (higher stdDev = lower confidence)
    const confidence = Math.max(0, 1 - (stdDev / avgLoad));
    
    return {
      expectedLoad: avgLoad,
      confidence,
      timeHorizon,
    };
  }
  
  // Scale based on prediction
  async scalePredictively(service, deployment) {
    const prediction = this.predictLoad(service, 30 * 60 * 1000); // 30 minutes ahead
    
    if (!prediction || prediction.confidence < 0.7) {
      return; // Not confident enough
    }
    
    // Calculate required replicas
    const requiredReplicas = Math.ceil(prediction.expectedLoad / 100); // Assuming 100 RPS per replica
    
    // Apply scaling
    await this.kubeClient.apps.v1.deployments.patch(deployment, {
      spec: {
        replicas: requiredReplicas,
      },
    });
    
    console.log(`Predictively scaled ${service} to ${requiredReplicas} replicas`);
  }
}
```

## Conclusion

Performance optimization is an ongoing process. Regularly monitor system performance, identify bottlenecks, and apply appropriate optimization techniques. Always measure before and after optimizations to verify improvements.

Remember that optimizations often come with trade-offs. Consider the impact on code readability, maintainability, and development velocity before implementing complex optimizations.

## References

- [Node.js Performance Documentation](https://nodejs.org/en/docs/guides/dont-block-the-event-loop/)
- [Redis Caching Best Practices](https://redis.io/topics/lru-cache)
- [PostgreSQL Performance Tuning](https://www.postgresql.org/docs/current/performance-tips.html)
- [Kubernetes Horizontal Pod Autoscaler](https://kubernetes.io/docs/tasks/run-application/horizontal-pod-autoscale/)
- [OpenTelemetry Documentation](https://opentelemetry.io/docs/)
