# Apple Foundation Models Integration Guide

## Overview

The `@bluefly/apple-fm` package provides a bridge service for integrating Apple's Foundation Models framework with the LLM Platform ecosystem. This integration enables access to Apple's on-device and Private Cloud Compute language models through a Node.js interface.

## Architecture

### Foundation Models Framework (Apple)

Apple's Foundation Models are accessed exclusively through Swift applications using the Foundation Models framework introduced at WWDC 2025. The framework provides:

- **On-Device Model**: ~3 billion parameter language model running locally
- **Server Model**: Mixture-of-experts model via Apple Private Cloud Compute
- **Guided Generation**: Swift `@Generable` macro for structured output
- **Tool Calling**: Custom tool protocol for extending model capabilities

### Bridge Architecture

```
┌─────────────────┐    ┌─────────────────┐    ┌─────────────────┐
│   Drupal LLM    │───▶│  @bluefly/      │───▶│  Swift App      │
│   Platform      │    │  apple-fm       │    │  Foundation     │
│   (ai_provider_ │    │  Bridge Service │    │  Models API     │
│   apple module) │    │                 │    │  (@Generable)   │
└─────────────────┘    └─────────────────┘    └─────────────────┘
         │                       │                       │
         ▼                       ▼                       ▼
┌─────────────────┐    ┌─────────────────┐    ┌─────────────────┐
│ Provider Config │    │ Multiple Bridge │    │ Swift Framework │
│ & Management    │    │ Protocols       │    │ Integration     │
└─────────────────┘    └─────────────────┘    └─────────────────┘
```

## Bridge Communication Protocols

### 1. Swift App (IPC Socket) - **Recommended**

Direct communication with a Swift application that has Foundation Models access.

```typescript
import [AppleFMBridgeService] from '@bluefly/apple-fm';

const bridge = new AppleFMBridgeService({
  provider: 'swift-app',
  config: {
    swiftApp: {
      bundleId: 'com.yourcompany.foundation-models-bridge',
      socketPath: '/tmp/apple-fm-bridge.sock',
      timeout: 30000
    }
  }
});
```

### 2. WebSocket Bridge

Real-time bidirectional communication for dynamic applications.

```typescript
const bridge = new AppleFMBridgeService({
  provider: 'websocket',
  config: {
    websocket: {
      url: 'ws://localhost:8080/apple-fm',
      reconnectInterval: 5000
    }
  }
});
```

### 3. HTTP REST Bridge

Standard HTTP API for web-based integration.

```typescript
const bridge = new AppleFMBridgeService({
  provider: 'http-bridge',
  config: {
    httpBridge: {
      baseUrl: 'http://localhost:3000',
      apiKey: 'your-api-key'
    }
  }
});
```

### 4. File Bridge

File-system based communication for batch processing.

```typescript
const bridge = new AppleFMBridgeService({
  provider: 'file-bridge',
  config: {
    fileBridge: {
      inputDir: '/tmp/apple-fm-input',
      outputDir: '/tmp/apple-fm-output',
      pollInterval: 1000
    }
  }
});
```

### 5. Unix Socket Bridge

High-performance local communication.

```typescript
const bridge = new AppleFMBridgeService({
  provider: 'socket-bridge',
  config: {
    socketBridge: {
      socketPath: '/tmp/apple-fm.sock',
      timeout: 30000
    }
  }
});
```

## Usage Examples

### Basic Text Generation

```typescript
import [AppleFMBridgeService] from '@bluefly/apple-fm';

const bridge = new AppleFMBridgeService({
  provider: 'swift-app',
  config: { /* config */ }
});

await bridge.connect();

const text = await bridge.generateText(
  'Explain quantum computing in simple terms',
  {
    model: 'on-device',
    maxTokens: 500,
    temperature: 0.7
  }
);

console.log(text);
await bridge.disconnect();
```

### Structured Generation (Swift @Generable)

```typescript
// Define the structure that matches Swift @Generable struct
const recipeSchema = {
  type: 'object',
  properties: {
    name: { type: 'string' },
    ingredients: { type: 'array', items: { type: 'string' } },
    cookingTime: { type: 'number' }
  }
};

const recipe = await bridge.generateStructured(
  'Create a recipe for chocolate chip cookies',
  recipeSchema,
  'Recipe'
);

console.log(recipe);
// Output: { name: "Chocolate Chip Cookies", ingredients: [...], cookingTime: 25 }
```

### Stream Response

```typescript
bridge.on('response', (response) => {
  if (response.partial) {
    console.log('Partial response:', response.result?.text);
  } else {
    console.log('Final response:', response.result?.text);
  }
});

const request = {
  id: 'stream_123',
  type: 'stream',
  prompt: 'Write a story about AI',
  model: 'on-device'
};

await bridge.sendRequest(request);
```

## Apple Intelligence Availability

### Device Requirements

| Device | Processor | OS Version | Features |
|--------|-----------|------------|----------|
| iPhone | A17 Pro+ | iOS 18.1+ | Full Apple Intelligence |
| iPad | M1+ | iPadOS 18.1+ | Full Apple Intelligence |
| Mac | M1+ | macOS 15.1+ | Full Apple Intelligence |

### Model Capabilities

#### On-Device Model (~3B Parameters)

- **Privacy**: 100% on-device processing
- **Performance**: ~0.6ms first token latency, 30 tokens/sec
- **Capabilities**: Text generation, summarization, entity extraction
- **Context**: Up to 65K tokens
- **Languages**: 15 languages supported

#### Server Model (Private Cloud Compute)

- **Privacy**: End-to-end encrypted processing
- **Performance**: Higher throughput for complex tasks
- **Capabilities**: Advanced reasoning, multimodal understanding
- **Context**: Up to 65K tokens
- **Network**: Requires connection to Apple's Private Cloud Compute

## Swift App Requirements

To use the bridge service, you need a Swift application with Foundation Models access:

### Minimal Swift Bridge App

```swift
import Foundation
import FoundationModels

@main
struct AppleFMBridge [static func main() async {
        let bridge = FoundationModelsBridge()
        await bridge.startServer()]
}

class FoundationModelsBridge {
    private let session = LanguageModelSession()
    
    func startServer() async {
        // Start IPC socket server
        let server = IPCServer(socketPath: "/tmp/apple-fm-bridge.sock")
        
        server.onRequest { request in
            await self.handleRequest(request)
        }
        
        await server.start()
    }
    
    func handleRequest(_ request: BridgeRequest) async -> BridgeResponse {
        switch request.type {
        case "generate":
            let result = try await session.respond(to: request.prompt)
            return BridgeResponse(success: true, text: result)
            
        case "guided_generate":
            // Use @Generable for structured output
            let result = try await session.respond(
                to: request.prompt,
                generating: request.structureType
            )
            return BridgeResponse(success: true, structured: result)
            
        default:
            return BridgeResponse(success: false, error: "Unsupported request type")
        }
    }
}
```

## Privacy and Security

### Data Processing Modes

1. **On-Device (Default)**
   - All processing on user's device
   - No network requests
   - Perfect privacy for sensitive data
   - Lower computational capacity

2. **Private Cloud Compute**
   - Encrypted processing on Apple servers
   - No data retention
   - Cryptographic verification
   - Higher computational capacity

3. **Hybrid Mode**
   - Automatic selection based on request complexity
   - On-device for simple tasks
   - Private Cloud for complex reasoning

### Security Features

- **End-to-End Encryption**: All communications encrypted
- **No Data Logging**: Apple doesn't log user prompts or responses
- **Attestation**: Cryptographic verification of server integrity
- **Differential Privacy**: Statistical privacy guarantees

## Integration with Drupal LLM Platform

The bridge integrates seamlessly with the Drupal ai_provider_apple module:

### Configuration

1. Navigate to `/admin/config/ai/providers/apple`
2. Select bridge provider type (Swift App recommended)
3. Configure connection settings
4. Test connection

### Usage in Drupal

```php
// Get Apple Foundation Models provider
$provider = \Drupal::service('ai.provider')->get('apple_foundation_models');

// Generate text
$result = $provider->generateText('Summarize this article', [
  'model' => 'on-device',
  'max_tokens' => 200
]);

echo $result['text'];
```

## Development and Testing

### Running Tests

```bash
npm test
```

### Building the Package

```bash
npm run build
```

### Development Mode

```bash
npm run dev
```

## Troubleshooting

### Common Issues

1. **Bridge Connection Failed**
   - Ensure Swift app is running
   - Check socket path permissions
   - Verify firewall settings

2. **Model Not Available**
   - Check device compatibility
   - Verify iOS/macOS version
   - Ensure Apple Intelligence is enabled

3. **Permission Denied**
   - Check socket file permissions
   - Verify app sandboxing settings
   - Review security policies

### Debug Mode

```typescript
const bridge = new AppleFMBridgeService(config);

bridge.on('error', (error) => {
  console.error('Bridge error:', error);
});

bridge.on('disconnect', () => {
  console.log('Bridge disconnected');
});
```

## Limitations

### Current Implementation

- Requires native Swift app for actual model access
- Bridge service provides TypeScript interface only
- Limited to Apple device ecosystem
- Requires iOS 18.1+ / macOS 15.1+

### Production Considerations

- Swift app development required for real functionality
- App Store distribution may have additional requirements
- Enterprise deployment needs proper provisioning
- Performance varies by device generation

## Future Enhancements

- **Direct Swift Integration**: Native Swift package for iOS/macOS apps
- **Xcode Plugin**: Development tools for Foundation Models
- **Performance Optimization**: Caching and connection pooling
- **Enhanced Monitoring**: Detailed metrics and analytics

## Support

For issues and questions:

- **Repository**: https://gitlab.bluefly.io/llm/apple-fm
- **Documentation**: `/docs` directory
- **Issues**: GitLab issue tracker

## License

MIT License - see LICENSE file for details.