# Authentication Architecture for Apple Foundation Models

This document outlines the comprehensive authentication architecture for Apple Foundation Models, designed to integrate seamlessly with existing Drupal authentication modules without reinventing the wheel.

## 🏗️ Architecture Overview

The authentication system supports multiple providers commonly used in Drupal ecosystems:

- **Keycloak** - Enterprise SSO with OpenID Connect
- **Drupal OAuth2** - OAuth2 client authentication
- **Drupal OpenID Connect** - OpenID Connect integration
- **Simple OAuth** - Drupal as OAuth server
- **JWT** - JSON Web Token authentication
- **API Key** - Simple API key authentication
- **Basic Auth** - Username/password authentication

## 🔧 Components

### 1. AuthenticationService (TypeScript/JavaScript)

Core authentication service for Node.js/TypeScript applications:

```typescript
import [AuthenticationService] from '@llm-gateway-sdk/apple-fm';

const authService = new AuthenticationService({
  provider: 'keycloak',
  config: {
    keycloak: {
      realm: 'apple-fm',
      baseUrl: 'http://localhost:8080',
      clientId: 'apple-fm-client',
      clientSecret: 'your-client-secret'
    }
  }
});
```

### 2. AuthenticationIntegrationService (PHP/Drupal)

Drupal service that integrates with existing authentication modules:

```php
// In Drupal controller
$auth_integration = \Drupal::service('native_ai.authentication_integration');
$config = $auth_integration->getAuthConfig();
```

### 3. CLI Authentication Commands

Complete CLI interface for authentication management:

```bash
# Configure authentication
llmcli apple-fm config

# Login with different providers
llmcli apple-fm auth login --provider keycloak
llmcli apple-fm auth login --provider simple-oauth

# Check authentication status
llmcli apple-fm auth status

# Logout
llmcli apple-fm auth logout
```

## 🔒 Provider Configurations

### Keycloak Configuration

**Environment Variables:**
```bash
KEYCLOAK_URL=http://localhost:8080
KEYCLOAK_REALM=apple-fm
KEYCLOAK_CLIENT_ID=apple-fm-client
KEYCLOAK_CLIENT_SECRET=your-secret
```

**Drupal Integration:**
Requires the [Keycloak module](https://www.drupal.org/project/keycloak):
```bash
composer require drupal/keycloak
drush en keycloak -y
```

### Simple OAuth (Drupal as Server)

**Environment Variables:**
```bash
DRUPAL_URL=http://localhost
SIMPLE_OAUTH_CLIENT_ID=apple-fm
SIMPLE_OAUTH_CLIENT_SECRET=your-secret
```

**Drupal Integration:**
Requires the [Simple OAuth module](https://www.drupal.org/project/simple_oauth):
```bash
composer require drupal/simple_oauth
drush en simple_oauth -y
```

### OAuth2 Client Configuration

**Environment Variables:**
```bash
DRUPAL_URL=http://localhost
DRUPAL_OAUTH_CLIENT_ID=apple-fm
DRUPAL_OAUTH_CLIENT_SECRET=your-secret
```

**Drupal Integration:**
Can use the [OAuth & OpenID Connect Login module](https://www.drupal.org/project/miniorange_oauth_client):
```bash
composer require drupal/miniorange_oauth_client
drush en miniorange_oauth_client -y
```

### OpenID Connect Configuration

**Environment Variables:**
```bash
DRUPAL_URL=http://localhost
DRUPAL_OPENID_CLIENT_ID=apple-fm
DRUPAL_OPENID_CLIENT_SECRET=your-secret
DRUPAL_OPENID_ISSUER=http://localhost/openid
```

**Drupal Integration:**
Requires the [OpenID Connect module](https://www.drupal.org/project/openid_connect):
```bash
composer require drupal/openid_connect
drush en openid_connect -y
```

## 🔐 Security Features

### Token Management

- **Secure Storage**: Tokens stored in user-specific directories with proper permissions
- **Automatic Expiration**: Tokens automatically expire and refresh when possible
- **Encryption**: Sensitive data encrypted at rest
- **Audit Trails**: All authentication events logged for compliance

### Privacy Integration

Authentication integrates with Apple FM's privacy features:

```typescript
// Privacy-aware authentication
const user = await authService.getCurrentUser(token);
const hasPrivacyAccess = user.permissions.includes('process_sensitive_data');

if (hasPrivacyAccess) {
  // Allow access to strict privacy mode
  const response = await client.generateText({
    prompt: sensitiveData,
    privacy: 'strict'
  });
}
```

### Permission-Based Access Control

```typescript
// Check specific permissions
const canUseOnDevice = await authService.hasPermission(token, 'use_on_device_models');
const canAccessCloud = await authService.hasPermission(token, 'use_cloud_models');

// Select appropriate model based on permissions
const models = await client.getModels();
const allowedModels = models.filter(model => {
  if (model.location === 'on-device') return canUseOnDevice;
  if (model.location === 'cloud') return canAccessCloud;
  return true;
});
```

## 🌐 API Endpoints

The Drupal module provides REST API endpoints for authentication:

### GET /native-ai/auth/config
Get authentication configuration for clients.

### POST /native-ai/auth/token
Generate authentication token for current user.

### POST /native-ai/auth/validate
Validate authentication token.

### GET /native-ai/auth/user
Get current user information.

### GET /native-ai/auth/permissions
Get user permissions and roles.

### POST /native-ai/auth/logout
Logout and invalidate token.

### GET /native-ai/auth/health
Authentication service health check.

## 🔄 Integration Workflows

### 1. Client-Side Authentication Flow

```typescript
// 1. Configure authentication
const config = await fetch('/native-ai/auth/config').then(r => r.json());

// 2. Create authentication service
const authService = new AuthenticationService(config.auth_config);

// 3. Authenticate user
const result = await authService.authenticate({
  username: 'user@example.com',
  password: 'password'
});

// 4. Use token for API calls
const client = createAppleFmClient({
  apiKey: result.token,
  authToken: result.token
});
```

### 2. Server-Side Integration (Drupal)

```php
// 1. Get authentication service
$auth_service = \Drupal::service('native_ai.authentication_integration');

// 2. Generate token for current user
$token_data = $auth_service->generateTokenForCurrentUser();

// 3. Validate token on subsequent requests
$validation = $auth_service->validateToken($token);

if ($validation['valid']) {
  // Proceed with Apple FM operations
  $user_info = $validation['user_info'];
}
```

### 3. CLI Integration

```bash
# 1. Configure authentication provider
llmcli apple-fm config
# Select: keycloak

# 2. Login with credentials
llmcli apple-fm auth login
# Enter username and password

# 3. Use authenticated commands
llmcli apple-fm generate --prompt "Hello, world!"
# Uses stored authentication token

# 4. Check authentication status
llmcli apple-fm auth status
# Shows user info, token expiration, permissions
```

## 🚀 Getting Started

### Quick Setup for Keycloak

1. **Install Drupal Keycloak module:**
```bash
composer require drupal/keycloak
drush en keycloak -y
```

2. **Configure Keycloak settings in Drupal:**
```bash
drush config:set keycloak.settings base_url "http://localhost:8080"
drush config:set keycloak.settings realm "apple-fm"
drush config:set keycloak.settings client_id "apple-fm-client"
```

3. **Configure Apple FM authentication:**
```bash
llmcli apple-fm config
# Select provider: keycloak
# Enter Keycloak details
```

4. **Test authentication:**
```bash
llmcli apple-fm auth login
llmcli apple-fm auth status
```

### Quick Setup for Simple OAuth

1. **Install Simple OAuth module:**
```bash
composer require drupal/simple_oauth
drush en simple_oauth -y
```

2. **Generate OAuth keys:**
```bash
# Generate private key
openssl genrsa -out private.key 2048

# Generate public key
openssl rsa -in private.key -pubout -out public.key

# Configure Drupal
drush config:set simple_oauth.settings public_key "/path/to/public.key"
drush config:set simple_oauth.settings private_key "/path/to/private.key"
```

3. **Create OAuth client:**
```bash
# Create client via Drupal admin or drush
drush simple-oauth:create-client "Apple FM Client" apple-fm-client
```

4. **Configure and test:**
```bash
llmcli apple-fm config
# Select provider: simple-oauth
llmcli apple-fm auth login
```

## 🔍 Troubleshooting

### Common Issues

**1. Token Validation Fails**
```bash
# Check token status
llmcli apple-fm auth status

# Refresh token if expired
llmcli apple-fm auth refresh

# Re-login if refresh fails
llmcli apple-fm auth logout
llmcli apple-fm auth login
```

**2. Permission Denied**
```bash
# Check user permissions
llmcli apple-fm auth permissions

# Verify Drupal role assignments
drush user:role:list username@example.com
```

**3. Provider Configuration Issues**
```bash
# Test provider health
curl http://localhost/native-ai/auth/health

# Check provider configuration
curl http://localhost/native-ai/auth/config
```

### Debug Mode

Enable debug logging:

```bash
# Environment variable
export APPLE_FM_DEBUG=true

# Or in configuration
llmcli apple-fm config
# Enable debug logging
```

## 📚 Best Practices

### Security
1. **Use HTTPS in production**
2. **Rotate API keys regularly**
3. **Implement proper CORS policies**
4. **Monitor authentication logs**
5. **Use environment variables for secrets**

### Performance
1. **Cache authentication tokens**
2. **Implement token refresh logic**
3. **Use connection pooling**
4. **Monitor authentication latency**

### Compliance
1. **Log all authentication events**
2. **Implement audit trails**
3. **Regular security reviews**
4. **Follow GDPR/HIPAA guidelines**

## 🔗 Related Documentation

- [Apple FM Privacy Features](./USE_CASES.md#privacy-first-use-cases)
- [CLI Commands Reference](../README.md#cli-usage)
- [Drupal Integration Guide](../../DEMO/README.md)
- [API Reference](./API.md)

This authentication architecture provides enterprise-grade security while leveraging existing Drupal authentication infrastructure, ensuring seamless integration without reinventing established patterns.