# llm-ui Integration

This document outlines the integration of llm-ui (LLM Platform UI) components into the bfrfp project.

## 🎉 New in Version 1.1.0: Tailwind CSS Components

The bfrfp package now includes Tailwind CSS-based components as an alternative to the Chakra UI-based components from llm-ui. These new components provide a modern, responsive, and highly customizable UI experience.

## Overview

The bfrfp project now leverages standardized UI components from the llm-ui library, providing a consistent user experience across the LLM Platform ecosystem. This integration replaces custom implementations with llm-ui's well-tested, accessible, and feature-rich components.

## Components

### RFP Components

The following RFP-specific components are now available:

1. **RFPUploader**
   - Replaces custom upload components
   - Provides drag-and-drop file upload
   - Supports multiple file formats
   - Includes real-time status tracking
   - Maintains backward compatibility with existing code

2. **RFPContentGenerator**
   - Generates AI-powered RFP responses
   - Customizable generation parameters
   - Section-by-section content editing
   - Export capabilities (PDF, DOCX, Markdown)
   - Maintains backward compatibility with existing code

3. **RFPDashboard**
   - Comprehensive RFP project management
   - Metrics and analytics visualization
   - Project creation and management
   - Document tracking
   - Maintains backward compatibility with existing code

## Services

### RFP Service

The `RFPService` class provides methods for interacting with RFP-related functionality:

- `analyzeDocument()` - Analyze RFP documents
- `generateResponse()` - Generate RFP responses
- `getAnalysisStatus()` - Check analysis status
- `getRecentAnalyses()` - Get recent analyses
- `getGenerationStatus()` - Check generation status
- `exportAnalysis()` - Export analysis in various formats

This service leverages both llm-ui's service layer and LLM Platform's shared utilities for logging, error handling, and HTTP requests.

## Usage Examples

### Basic Component Usage

```tsx
import { RFPUploader, RFPContentGenerator, RFPDashboard } from '@bluefly/rfp';

// Upload component
<RFPUploader
  onUploadComplete=[handleUploadComplete]
  onParsingComplete=[handleParsingComplete]
  acceptedFileTypes={['pdf', 'docx']}
  maxFileSize=[10]
/>

// Content generator
<RFPContentGenerator
  opportunityData=[opportunityData]
  companyProfile=[companyProfile]
  onContentGenerated=[handleContentGenerated]
  onExportRequest=[handleExportRequest]
/>

// Dashboard
<RFPDashboard
  onCreateProject=[handleCreateProject]
  onOpenProject=[handleOpenProject]
/>
```

### Service Usage

```typescript
import [rfpService] from '@bluefly/rfp';

// Analyze a document
const analysisResult = await rfpService.analyzeDocument({
  documentId: 'doc-123',
  options: {
    analysisDepth: 'comprehensive',
    includeRecommendations: true,
    generateInsights: true
  }
});

// Generate content
const content = await rfpService.generateResponse({
  analysisId: 'analysis-123',
  sections: ['executiveSummary', 'approach', 'timeline'],
  options: {
    tone: 'professional',
    length: 'comprehensive'
  }
});

// Export as PDF
const exportResult = await rfpService.exportAnalysis({
  analysisId: 'analysis-123',
  format: 'pdf',
  options: {
    includeCoverPage: true,
    includeTableOfContents: true,
    includeExecutiveSummary: true
  }
});
```

## Shared Utilities Integration

The implementation leverages LLM Platform's shared utilities:

```typescript
// Logging
import [logger] from '@bluefly/cli/shared/logger';
logger.info('Analyzing document', [documentId]);

// HTTP Client
import [HttpClient] from '@bluefly/cli/shared/http';
const client = new HttpClient({
  baseURL: 'https://api.example.com',
  timeout: 5000,
  retries: 3
});

// Error Handling
import { createError, ERROR_CODES } from '@bluefly/cli/shared/errors';
throw createError('Resource not found', ERROR_CODES.NOT_FOUND, 404);
```

## Backward Compatibility

All new components maintain backward compatibility with existing code:

- Components accept both new props and legacy props
- Service methods support both new and legacy calling patterns
- Export patterns remain consistent with the prior implementation

## Migrating Existing Code

### From Custom Components to llm-ui Components

```diff
- import [RFPUpload] from '../../web-ui/src/components/rfp/RFPUpload';
+ import [RFPUploader] from '@bluefly/rfp';

- <RFPUpload
+ <RFPUploader
    onUploadComplete=[handleUploadComplete]
    onParsingComplete=[handleParsingComplete]
  />
```

### From Custom Services to RFP Service

```diff
- import [analyzeRFP] from './analyzer';
+ import [rfpService] from '@bluefly/rfp';

- const result = await analyzeRFP(documentPath);
+ const result = await rfpService.analyzeRFPDocument(documentId);
```

## Benefits

1. **Consistency**: Unified UI experience across LLM Platform ecosystem
2. **Maintainability**: Centralized component library reduces duplication
3. **Feature-rich**: Access to llm-ui's comprehensive component library
4. **Performance**: Optimized components with better rendering performance
5. **Accessibility**: llm-ui components follow accessibility best practices
6. **Testing**: Thoroughly tested components reduce bugs
7. **Documentation**: Comprehensive documentation and examples

## Tailwind CSS Components

The Tailwind CSS components provide a modern alternative to the llm-ui components with several advantages:

1. **Zero Dependencies**: Built with pure Tailwind CSS, eliminating the need for Chakra UI
2. **Smaller Bundle Size**: Significantly reduced package size for better performance
3. **Modern Design**: Follows the latest design trends and best practices
4. **Highly Customizable**: Easy to adapt to your brand and design system
5. **Responsive by Default**: Built with mobile-first approach

### Available Tailwind Components

1. **TailwindRFPUploader**
   - Modern drag-and-drop file uploader with real-time progress tracking
   - Support for multiple file formats and detailed status indicators
   - Comprehensive file validation and error handling

2. **TailwindRFPContentGenerator**
   - AI-powered content generation with configurable settings
   - Section-based editing with word count and confidence metrics
   - Real-time content enhancement recommendations

3. **TailwindRFPDashboard**
   - Project management dashboard with filtering and sorting
   - Visualized metrics for project status and progress
   - Interactive project list with detailed information

### Usage Example

```tsx
import { 
  TailwindRFPUploader, 
  TailwindRFPContentGenerator, 
  TailwindRFPDashboard 
} from '@bluefly/rfp';

// Uploader component
<TailwindRFPUploader
  onDocumentsProcessed=[handleDocumentsProcessed]
  onAnalysisComplete=[handleAnalysisComplete]
  maxFileSize=[10]
  acceptedFormats={['pdf', 'doc', 'docx']}
  className="my-8 mx-auto max-w-3xl"
/>

// Content generator
<TailwindRFPContentGenerator
  opportunityData=[opportunityData]
  companyProfile=[companyProfile]
  onContentGenerated=[handleContentGenerated]
  onExportRequest=[handleExportRequest]
  className="my-8 mx-auto max-w-6xl"
/>

// Dashboard
<TailwindRFPDashboard
  onCreateProject=[handleCreateProject]
  onOpenProject=[handleOpenProject]
  className="my-8 mx-auto max-w-7xl"
/>
```

### Configuration

To use the Tailwind components, you need to include Tailwind CSS in your project:

1. Install Tailwind CSS and its dependencies:
   ```bash
   npm install --save-dev tailwindcss postcss autoprefixer
   ```

2. Create a Tailwind configuration file:
   ```bash
   npx tailwindcss init -p
   ```

3. Update your Tailwind configuration to include the bfrfp components:
   ```js
   // tailwind.config.js
   module.exports = {
     content: [
       './src/**/*.{js,jsx,ts,tsx}',
       './node_modules/@bluefly/rfp/dist/**/*.{js,jsx,ts,tsx}',
     ],
     // ...rest of your configuration
   }
   ```

4. Import Tailwind CSS in your main CSS file:
   ```css
   @tailwind base;
   @tailwind components;
   @tailwind utilities;
   ```

## Future Enhancements

1. **Server-Side Rendering**: Enhanced SSR support for both component libraries
2. **Theme Customization**: Advanced theming capabilities for Tailwind components
3. **Animation Presets**: Additional animation options for enhanced user experience
4. **Mobile Optimization**: Further responsive design improvements
5. **Offline Support**: Progressive web app capabilities