# BFRFP Documentation Hub

RFP (Request for Proposal) content generation system using multiple LLM providers and document processing capabilities.

**Current Status**: Development/Testing - TypeScript implementation with Express.js API  
**Technology Stack**: Node.js, TypeScript, Express.js, Multiple LLM providers

## Technical Overview

BFRFP provides document processing and content generation workflows for RFP responses. The system integrates with multiple LLM providers (OpenAI, Anthropic, Ollama) and includes template-based content generation capabilities.

### Architecture & Data Flow

```
[RFP Document] → [Document Processor] → [Content Analysis]
      ↓                    ↓                    ↓
[PDF/Word Parser]    [Requirement Extract]  [AI Classification]
      ↓                    ↓                    ↓
[Text Extraction]    [Structure Analysis]   [Content Generation]
      ↓                    ↓                    ↓
[Template Engine]    [Content Assembly]     [Format Output]
      ↓                    ↓                    ↓
[Generated Content]  [Quality Review]       [Final Document]
```

### Component Interactions

1. **Document Processing Pipeline**:
   - PDF/Word document parsing and text extraction
   - Requirement identification and classification
   - Content structure analysis and mapping

2. **AI Content Generation**:
   - Multi-LLM integration (OpenAI, Anthropic, Ollama)
   - Template-based content generation
   - Context-aware proposal section creation

3. **Output Management**:
   - Multiple format support (Word, PDF, JSON)
   - Template customization and branding
   - Quality validation and review workflows

## Key Features

### 1. AI-Powered Response Generation
- **Smart Generation**: Uses advanced LLMs to generate contextually relevant responses
- **Learning Modes**: Basic, Advanced, and Expert modes for different complexity levels
- **Neural Memory**: Self-improving system that learns from successful responses
- **Model Registry**: Integration with GitLab Model Registry and MLflow for model versioning
- **Self-Healing**: Automated model retraining and validation pipeline

### 2. Team Collaboration
- **Real-time Chat**: Team communication with sentiment analysis
- **Review Workflow**: Structured review process with AI suggestions
- **Market Intelligence**: Real-time market trend analysis
- **Presence Awareness**: Team member status tracking
- **Document References**: URL-based learning from external resources

### 3. Sales Intelligence
- **Win Probability**: AI-driven deal scoring
- **Competitor Analysis**: Automated competitor tracking
- **Market Trends**: Real-time market analysis
- **Sentiment Analysis**: Content tone and impact assessment
- **Opportunity Tracking**: Deal pipeline management

### 4. Integration Capabilities
- **External Resources**: URL-based learning from:
  - Company websites (e.g., bluefly.io)
  - Google Sheets for estimates
  - Documentation repositories
  - Knowledge bases
- **LLM Platform Services**: Integration with:
  - LLM Services
  - MCP Server
  - Neural Memory
  - Model Registry
  - MLflow

## Architecture

### Core Components

1. **Web UI** (`/web-ui`)
   - React-based frontend
   - Real-time collaboration
   - Responsive design
   - Chakra UI components

2. **API Services** (`/api`)
   - OpenAPI 3.1 compliant
   - RESTful endpoints
   - WebSocket support
   - Authentication & authorization

3. **AI Engine** (`/ai`)
   - LLM integration
   - Neural memory system
   - Model training pipeline
   - Self-healing mechanisms

4. **Storage** (`/storage`)
   - Vector database
   - Document store
   - Model registry
   - MLflow integration

### Model Management

#### GitLab Model Registry
- Version control for ML models
- Automated model validation
- Model performance tracking
- A/B testing support

#### MLflow Integration
- Experiment tracking
- Model lifecycle management
- Performance metrics
- Model deployment

### Self-Healing System

1. **Model Monitoring**
   - Performance metrics tracking
   - Drift detection
   - Quality assessment
   - Usage analytics

2. **Automated Retraining**
   - Triggered by:
     - Performance degradation
     - New training data
     - User feedback
     - Scheduled intervals

3. **Validation Pipeline**
   - Automated testing
   - Quality checks
   - Performance validation
   - Security scanning

4. **Deployment Process**
   - Automated deployment
   - Canary releases
   - Rollback capability
   - Version management

## URL-Based Learning

### Supported Resources

1. **Company Websites**
   ```typescript
   // Example: Learning from bluefly.io
   const learnFromWebsite = async (url: string) => {
     const response = await apiClient.post('/api/v1/learn/website', {
       url: 'https://bluefly.io',
       context: 'company_services',
       depth: 2, // Crawl depth
       filters: ['services', 'team', 'case-studies']
     });
     return response.data;
   };
   ```

2. **Google Sheets**
   ```typescript
   // Example: Learning from estimate sheets
   const learnFromSheet = async (sheetId: string) => {
     const response = await apiClient.post('/api/v1/learn/sheet', {
       sheetId: 'your-sheet-id',
       range: 'Estimates!A1:Z1000',
       context: 'pricing_estimates'
     });
     return response.data;
   };
   ```

3. **Documentation**
   ```typescript
   // Example: Learning from documentation
   const learnFromDocs = async (url: string) => {
     const response = await apiClient.post('/api/v1/learn/docs', {
       url: 'https://docs.bluefly.io',
       context: 'technical_documentation',
       format: 'markdown'
     });
     return response.data;
   };
   ```

### Learning Process

1. **Content Extraction**
   - Web scraping
   - Document parsing
   - Data extraction
   - Content cleaning

2. **Knowledge Integration**
   - Vector embedding
   - Neural memory storage
   - Context association
   - Relevance scoring

3. **Model Training**
   - Incremental learning
   - Fine-tuning
   - Validation
   - Deployment

## Development

### Prerequisites
- Node.js 18.x
- GitLab access
- MLflow server
- Vector database

### Setup
```bash
# Clone repository
git clone https://gitlab.bluefly.io/llm/bfrfp.git
cd bfrfp

# Install dependencies
npm install

# Start development server
npm run dev

# Run tests
npm test

# Build documentation
npm run docs:build
```

### CI/CD Pipeline

1. **Build Stage**
   - Code compilation
   - Type checking
   - Unit tests
   - Documentation build

2. **Test Stage**
   - Integration tests
   - E2E tests
   - Performance tests
   - Security scans

3. **Model Stage**
   - Model training
   - Validation
   - Registry update
   - MLflow logging

4. **Deploy Stage**
   - GitLab Pages deployment
   - Model deployment
   - API deployment
   - Monitoring setup

## GitLab Pages Integration

### Documentation Site
- Built with VuePress
- Auto-deployed on merge
- Versioned documentation
- Search functionality

### Model Registry
- Model versioning
- Performance metrics
- Training history
- Deployment status

### MLflow Dashboard
- Experiment tracking
- Model metrics
- Artifact storage
- Run comparison

## Monitoring & Maintenance

### Performance Metrics
- Response generation time
- Model accuracy
- System resource usage
- User engagement

### Health Checks
- API availability
- Model performance
- Database status
- Cache efficiency

### Automated Maintenance
- Model retraining
- Cache cleanup
- Log rotation
- Backup management

## Security

### Authentication
- JWT tokens
- Role-based access
- Session management
- API key rotation

### Data Protection
- Encryption at rest
- Secure transmission
- Access logging
- Audit trails

## Support

- [Documentation](https://docs.bluefly.io/bfrfp)
- [Issue Queue](https://gitlab.bluefly.io/llm/bfrfp/-/issues)
- [Model Registry](https://gitlab.bluefly.io/llm/bfrfp/-/model-registry)
- [MLflow Dashboard](https://mlflow.bluefly.io)

## Contributing

See [CONTRIBUTING.md](CONTRIBUTING.md) for development guidelines.

## License

MIT License - see [LICENSE](LICENSE) for details.
