# CLAUDE.md for BFAPIClient

This document provides essential guidance for Claude Code when working with the BFAPIClient repository.

## Repository Overview

BFAPI (ARCHIVED - migrate to LangChain ecosystem)-grade applications.

## Ecosystem Context

BFAPIClient serves as the primary integration layer between frontend applications and the LLM Platform AI Platform backend services. It directly communicates with bfapi (the API gateway) to access services like bfllm, bfmcp, and bfrfp. Frontend applications, particularly bfui components, rely on BFAPIClient to handle all backend communication, providing consistent error handling, caching, and resilience patterns across the ecosystem.

### Key Features

- **Protocol Support**: Unified REST and GraphQL clients
- **Advanced Caching**: Multi-tier caching system with memory, Redis, and disk storage
- **Resilience Patterns**: Circuit breaking, exponential backoff with jitter, request batching
- **Observability**: OpenTelemetry integration for distributed tracing and metrics
- **TypeScript-First**: Full type safety with generated models from OpenAPI

## Project Structure

```
bfapiclient/
├── src/                  # Source code
│   ├── cache/            # Caching system (strategies, manager)
│   ├── core/             # Core client functionality
│   ├── errors/           # Error handling system
│   ├── generated/        # Auto-generated API models (from OpenAPI)
│   ├── graphql/          # GraphQL client implementation
│   ├── interceptors/     # Request/response interceptors
│   ├── mcp/              # Model Control Plane client
│   ├── native/           # Native AI client
│   ├── resilience/       # Resilience patterns (circuit breaker)
│   ├── rest/             # REST client implementation
│   ├── telemetry/        # Observability and metrics
│   ├── types/            # Type definitions
│   ├── unified/          # Unified API client
│   └── index.ts          # Main entry point and exports
├── test/                 # Test files
├── docs/                 # Documentation
├── examples/             # Usage examples
└── scripts/              # Build and utility scripts
```

## Development Commands

```bash
# Installation
npm install

# Build
npm run build             # Standard build
npm run build:prod        # Production build
npm run build:browser     # Browser-compatible build
npm run build:all         # Build all variants

# Development
npm run dev               # Development mode with watch

# Testing
npm run test              # Run tests
npm run test:watch        # Watch mode
npm run test:coverage     # Coverage report
npm run test:browser      # Browser compatibility tests

# Linting and formatting
npm run lint              # Run ESLint
npm run lint:fix          # Fix linting issues
npm run format            # Format code
npm run ts:check          # Type check

# Documentation
npm run docs              # Generate documentation
npm run serve:docs        # Serve documentation locally

# Publishing
npm run release           # Build and publish to npm
```

## Important Conventions

### Code Style

- **TypeScript Standards**: Follow strict TypeScript practices with proper typing
- **ESM Modules**: Package uses ESM modules (`"type": "module"` in package.json)
- **Tree-Shaking**: Designed for tree-shaking support (`"sideEffects": false"`)
- **Documentation**: Use JSDoc comments for all public APIs

### Architecture Patterns

1. **Client Initialization**:

   ```typescript
   import [RestClient] from '@bluefly/api-client';

   const client = new RestClient({
     baseUrl: 'https://api.example.com',
     timeout: 30000,
   });
   ```

2. **Interceptor Pattern**:

   ```typescript
   // Add authentication
   client.addRequestInterceptor(bearerTokenInterceptor('your-token'));

   // Add logging
   client.addRequestInterceptor(requestLoggingInterceptor({ logHeaders: true }));
   ```

3. **Advanced Caching**:

   ```typescript
   const cache = new HierarchicalCacheManager({
     memory: { maxSize: 1000, ttl: 300 },
     redis: { url: 'redis://localhost:6379', ttl: 3600 },
     disk: { basePath: './cache', ttl: 86400, compress: true },
   });
   ```

4. **Circuit Breaker**:

   ```typescript
   import [CircuitBreaker] from '@bluefly/api-client';

   const breaker = new CircuitBreaker({
     failureThreshold: 5,
     resetTimeout: 30000,
     // ...
   });
   ```

### Testing Approach

- Use Jest for unit and integration tests
- Test coverage should be maintained above 80%
- Browser compatibility testing through dedicated HTML pages

## Common Pitfalls

1. **ESM/CJS Compatibility**: The library uses ESM modules, but provides CJS compatibility via the `dist/index.js` output.
2. **Circular Dependencies**: Avoid circular imports, especially in type files.
3. **GraphQL Error Type**: Be aware of the `GraphQLError` export which was fixed to avoid duplication with errors/index.js export.
4. **TelemetryClient Implementation**: Ensure the TelemetryClient includes the required recordLog method.

## Current Development Focus

See the NEXT_STEPS.md file for current roadmap and planned features:

1. **Short-term**: TypeScript declaration maps, resilient request handling, browser compatibility, documentation
2. **Medium-term**: Enhanced caching, advanced telemetry, security enhancements, performance optimizations
3. **Long-term**: Ecosystem integration, advanced AI features, developer experience, enterprise features

## Integration With Other Projects

- **bfapi**: Primary backend API this client connects to
- **bfllm**: LLM services accessed through this client
- **bfmcp**: Model Control Plane integration
- **bfui**: Frontend components that consume this client
- **native_aijs**: JavaScript implementation of AI capabilities

## Build and Release Process

The project uses:

- **Rollup**: For bundling (see rollup.config.js and rollup.browser.config.js)
- **TypeDoc**: For documentation generation
- **Semantic Release**: For versioned releases to npm

## Security Considerations

- Never commit API tokens or credentials to the repository
- Use the provided authentication interceptors for secure API access
- Follow the error handling patterns for robust error management

## Project TODO List

### High Priority

#### Core Functionality

- [AI-402] Implement a clear versioning strategy for the client library
- [AI-404] Standardize error handling within the client
- [AI-405] Ensure authentication mechanisms are secure
- [AI-409] Allow for flexible configuration of the API client
- [AI-412] Set up a CI job that automatically regenerates bfapiclient

#### Cache System

- Fix Redis connection pooling
  - Implement proper connection management
  - Add connection health checks
  - Handle connection failures gracefully
  - Add connection metrics
- Improve Disk Cache
  - Implement better cleanup strategies
  - Add file locking for concurrent access
  - Improve error handling for file operations
  - Add disk space monitoring
- Enhance Memory Cache
  - Implement configurable eviction policies
  - Add memory usage monitoring
  - Improve garbage collection
  - Add cache size limits

#### Resilience & Performance

- Add built-in retry logic for transient errors
- Add jitter to retry delays to avoid thundering herd problems
- Support retry cancellation and abort signals

### Medium Priority

#### Cache Enhancements

- Implement Cache Warming
  - Add warmup strategies
  - Add warmup metrics
  - Add warmup configuration
  - Add warmup monitoring
- Add Cache Versioning
  - Implement version tracking
  - Add version migration
  - Add version validation
  - Add version rollback

#### GraphQL Client

- Improve operation name extraction to handle complex queries
- Add support for subscriptions and real-time updates
- Integrate caching strategies specific to GraphQL
- Enhance error handling with retry and fallback mechanisms

#### API Client

- [AI-401] Ensure the SDK generation script is robust
- [AI-403] Provide documentation for adding custom interceptors
- [AI-406] Verify that the client library supports tree shaking
- [AI-411] Design patterns for extending generated services
- [AI-413] Establish a process for handling deprecated API endpoints
- [AI-414] Consider utilities for mocking API responses
- [AI-415] Define integration with other services (bfllm/bfmcp)
- Add support for request cancellation

### Lower Priority

#### Testing & Documentation

- [AI-407] Auto-generate or supplement documentation for the client library
- [AI-408] Implement a strategy for testing the client library
- Add Redis Cluster Tests
- Add Cache Tests
- Add browser compatibility tests
- Add unit tests for interceptor registration
- Add integration tests for interceptor execution/flow
- Add OpenAPI documentation for interceptor usage

#### Telemetry

- Enhance Metrics
  - Add cache hit/miss ratios
  - Add operation latencies
  - Add error rates
  - Add resource usage
- Improve Monitoring
  - Add performance dashboards
  - Add alerting rules
  - Add health checks
  - Add status reporting

#### Security

- Add input sanitization
- Implement rate limiting
- Add access control
- Add security scanning
