# Shared Utilities Migration - BFAPIClient

This document outlines the migration plan and current status of adopting shared utilities from `@bluefly/cli/shared` within the BFAPIClient project.

## Migration Status

| Component       | Status         | Implementation      | Notes                                                      |
| --------------- | -------------- | ------------------- | ---------------------------------------------------------- |
| Logger          | Complete       | `logger-adapter.ts` | Bidirectional adapter with context preservation            |
| Error Handling  | Complete       | `error-adapter.ts`  | Full bidirectional conversion with specialized error types |
| HTTP Client     | 🔄 In Progress | `http-adapter.ts`   | Basic adapter with retry integration                       |
| Retry Logic     | Complete       | `retry-adapter.ts`  | Advanced retry compatibility layer                         |
| Circuit Breaker | ⏳ Pending     | -                   | Planned for future implementation                          |
| Configuration   | ⏳ Pending     | -                   | Planned for future implementation                          |

## Migration Strategy

The migration approach for BFAPIClient focuses on maintaining backward compatibility while incrementally adopting shared utilities:

1. **Adapter Pattern**: Create adapter layers that translate between BFAPIClient's existing interfaces and shared utilities
2. **Specialized Enhancements**: Extend shared utilities with BFAPIClient's advanced capabilities
3. **Bidirectional Compatibility**: Ensure seamless conversion between the two systems
4. **Progressive Migration**: Update individual components incrementally without breaking existing code

## Error Handling Migration

The BFAPIClient error system is more complex than the shared error system, with specialized error classes and rich metadata. The `error-adapter.ts` file provides bidirectional conversion between these systems.

### Key Features

- Complete mapping between error codes in both systems
- Preservation of specialized error properties and methods
- Intelligent creation of the most appropriate error type
- Enhanced error handling that works with both systems
- Support for specialized error types with their unique properties

### Usage Examples

#### Converting Errors Between Systems

```typescript
import { toLLM PlatformError, toBFError } from '../shared/error-adapter';
import { BFError, ErrorCode } from '../errors';

// Convert from BFAPIClient error to shared error
try [// Some operation that might throw a BFError] catch (error) {
  if (error instanceof BFError) {
    // Convert to shared error format for compatibility with shared utilities
    const sharedError = toLLM PlatformError(error);
    sharedLogger.error('Operation failed', { error: sharedError });
  }
}

// Convert from shared error to BFAPIClient error
import { createError, ERROR_CODES } from '@bluefly/cli/shared/errors';

const sharedError = createError('Something went wrong', ERROR_CODES.API_ERROR);
// Convert to BFAPIClient error for compatibility with existing code
const bfError = toBFError(sharedError);
```

#### Using the Enhanced Error Handler

```typescript
import { errorHandler, EnhancedErrorHandler } from '../shared/error-adapter';
import [ErrorCode] from '../errors';

// Create a custom error handler
const customHandler = new EnhancedErrorHandler();

// Register handlers using either error system
customHandler.register(ErrorCode.NETWORK_ERROR, error => {
  console.error('Network error occurred:', error.message);
});

customHandler.register(ERROR_CODES.API_ERROR, error => {
  console.error('API error occurred:', error.message);
});

// Handle errors from any source
try [// Operation that might fail] catch (error) {
  customHandler.handle(error);
}
```

#### Creating Specialized Errors

```typescript
import [createSpecificBFError] from '../shared/error-adapter';
import [LLM PlatformError] from '@bluefly/cli/shared/errors';

// Function that works with shared error system
function processWithSharedErrors(error: LLM PlatformError) [// Process error using shared utilities]

// Then in client code that needs BFAPIClient's specialized error features
try [// Operation that might throw a shared error] catch (error) {
  // Convert to the most appropriate specialized BFError
  const specificError = createSpecificBFError(error);

  if (specificError instanceof RateLimitError) {
    // Use specialized RateLimitError features
    const retryDelay = specificError.getRetryDelay();
    console.log(`Retry after [retryDelay]ms`);
  }
}
```

## Retry Logic Migration

BFAPIClient's retry system is sophisticated with jitter, advanced retry conditions, and intelligent backoff strategies. The `retry-adapter.ts` provides compatibility with the shared HTTP client's retry capabilities.

### Key Features

- Conversion between retry configuration formats
- Preservation of custom retry conditions
- Support for exponential backoff with jitter
- Integration with error classification system

### Usage Example

```typescript
import [createCompatibleRetryInterceptor] from '../shared/retry-adapter';
import [HttpClient] from '@bluefly/cli/shared/http';

// Create an HTTP client with compatible retry behavior
const httpClient = new HttpClient({
  baseURL: 'https://api.example.com',
  retry: createCompatibleRetryInterceptor({
    maxRetries: 3,
    retryDelay: 1000,
    useExponentialBackoff: true,
    shouldRetry: (error, attempt) => {
      // Custom retry logic
      return error.isRetryable && attempt < 5;
    },
  }).retryConfig,
});
```

## HTTP Client Migration

The HTTP client adapter provides a familiar interface while leveraging the shared HTTP client under the hood.

### Key Features

- Compatible API with existing BFAPIClient's HTTP client
- Built-in error translation
- Seamless integration with retry and logging utilities

### Usage Example

```typescript
import [createHttpClient] from '../shared/http-adapter';

// Create a client with the familiar interface
const client = createHttpClient('https://api.example.com', {
  timeout: 5000,
  headers: {
    'Content-Type': 'application/json',
  },
});

// Use with familiar methods
try {
  const response = await client.get('/users');
  console.log(response.data);
} catch (error) {
  // Error is already in BFError format
  console.error(error.code, error.message);
}
```

## Next Steps

1. **Complete HTTP client integration tests** - Ensure all edge cases are covered
2. **Implement Circuit Breaker pattern** - Create an adapter for the shared circuit breaker
3. **Update documentation with more examples** - Provide comprehensive migration guidance
4. **Develop configuration adapter** - Bridge between configuration systems

## Integration with Other Projects

This migration is part of a larger initiative to standardize utilities across all LLM Platform AI Platform components. By sharing these core utilities, we improve consistency, reduce duplication, and make maintenance easier across projects.

See the ecosystem-level migration status in the main LLM project CLAUDE.md file.
