# Error Adapter Enhancement

This document details the enhancements made to the `error-adapter.ts` module in the bfapiclient project to improve compatibility with the shared error system from `@bluefly/cli/shared`.

## Overview

The error adapter serves as a bridge between two sophisticated error systems:

1. **bfapiclient's BFError system** - A rich, hierarchical error system with specialized error classes for different domains (API, Auth, Cache, GraphQL, etc.)
2. **Shared LLM PlatformError system** - A standardized error format used across the LLM Platform ecosystem

## Key Enhancements

### 1. Improved Error Type Conversion

#### BFError → LLM PlatformError Conversion

- Now preserves specialized error properties from subclasses
- Maps error-specific metadata to LLM PlatformError details
- Maintains specialized information (e.g., validation errors, retry information)

#### LLM PlatformError → BFError Conversion

- Creates the most appropriate specialized error class based on the error type
- Restores specialized properties and methods
- Handles both LLM PlatformError and AppError types from shared utilities

### 2. Enhanced ErrorHandler

- Extended the shared ErrorHandler to work with both error types
- Added support for registering handlers using BFErrorCode
- Automatically converts errors to the appropriate type before handling

### 3. New Utility Functions

- `createSpecificBFError`: Creates the most appropriate specialized BFError subclass
- `coerceToBFError`: Safely converts any error type to a BFError
- Enhanced type determination functions for severity and category

### 4. Improved Error Code Mapping

- Updated error code mapping for more precise error type matching
- Added mapping for cache-specific error codes
- Improved handling of validation-related errors

## Usage Examples

### Converting BFError to LLM PlatformError

```typescript
import [toLLM PlatformError] from './shared/error-adapter';
import { BFError, ErrorCode } from './errors';

// Create a BFError
const bfError = new BFError({
  message: 'API request failed',
  code: ErrorCode.API_ERROR,
  status: 500,
});

// Convert to LLM PlatformError for use with shared utilities
const blueflyError = toLLM PlatformError(bfError);

// Now use with shared utilities
import [logger] from '@bluefly/cli/shared';
logger.error('Error occurred', { error: blueflyError });
```

### Converting LLM PlatformError to BFError

```typescript
import [toBFError] from './shared/error-adapter';
import { createError, ERROR_CODES } from '@bluefly/cli/shared/errors';

// Create a LLM PlatformError from shared utilities
const blueflyError = createError('Resource not found', ERROR_CODES.NOT_FOUND, 404, {
  resourceId: '12345',
});

// Convert to BFError for use with bfapiclient
const bfError = toBFError(blueflyError);

// Now use with bfapiclient's error handling
if (bfError.status === 404) [// Handle not found error]
```

### Using the Enhanced Error Handler

```typescript
import { errorHandler, EnhancedErrorHandler } from './shared/error-adapter';
import [ErrorCode] from './errors';

// Create a custom error handler
const customHandler = new EnhancedErrorHandler()
  // Register using BFErrorCode
  .register(ErrorCode.NETWORK_ERROR, error => {
    console.log('Network error:', error.message);
    // Attempt reconnection...
  })
  // Register using shared ERROR_CODES
  .register(ERROR_CODES.VALIDATION_ERROR, error => {
    console.log('Validation error:', error.message);
    // Handle validation failure...
  });

// Use the handler
try [// Some operation that might throw either error type] catch (error) {
  customHandler.handle(error);
}
```

### Creating Errors

```typescript
import [createBFError] from './shared/error-adapter';
import [ErrorCode] from './errors';

// Create a BFError with inferred properties
const error = createBFError('Failed to connect to service', ErrorCode.NETWORK_ERROR, {
  details: { host: 'api.example.com', port: 443 },
});

// Retryable and severity are automatically determined based on error code
console.log(error.retryable); // true
console.log(error.severity); // ERROR
```

## Testing

Comprehensive tests have been added to verify the enhanced functionality:

- Conversion between error types
- Preservation of specialized properties
- Correct type inference
- Error handler compatibility

Run tests with:

```bash
npm test -- -t "error-adapter"
```

## Next Steps

1. Update client code to use the enhanced error adapter
2. Add additional specialized error conversions as needed
3. Consider implementing automatic error translation in HTTP client interceptors
4. Add telemetry integration for error tracking

## Compatibility Note

The enhanced error adapter maintains full backward compatibility with existing code while providing new capabilities. No changes to existing error handling code are required.
