# Retry and Resilience Features

The LLM Platform API Client includes sophisticated retry and resilience capabilities to help your applications handle transient failures gracefully. These features are especially important in distributed systems, microservice architectures, and environments with unreliable networks.

## Basic Retry Interceptor

For simple retry needs, use the basic `retryInterceptor`:

```typescript
import { ApiClient, retryInterceptor } from '@bluefly/api-client';

const client = new ApiClient({
  baseUrl: 'https://api.example.com',
});

// Add a basic retry interceptor that retries failed requests up to 3 times
client.addErrorInterceptor(retryInterceptor);
```

## Advanced Retry Interceptor

For more complex scenarios, use the advanced retry interceptor which offers sophisticated exponential backoff, jitter, and detailed control:

```typescript
import { ApiClient, createAdvancedRetryInterceptor } from '@bluefly/api-client';

const client = new ApiClient({
  baseUrl: 'https://api.example.com',
});

// Create an advanced retry interceptor with custom configuration
const advancedRetry = createAdvancedRetryInterceptor({
  maxRetries: 5,
  baseDelayMs: 100,
  maxDelayMs: 30000,
  backoffFactor: 2,
  jitterFactor: 0.25,
  retryableStatusCodes: [408, 425, 429, 500, 502, 503, 504, 507, 509],
  timeout: 60000,
  retryNetworkErrors: true,

  // Optional hooks for monitoring and telemetry
  onRetry: (error, attempt, delay, state) => {
    console.log(`Retrying after error: [error.message] (attempt [attempt]/[maxRetries])`);
  },
  onRetryExhausted: (error, state) => {
    console.error(`Retry exhausted after [state.attempts] attempts`);
  },
});

// Add the advanced retry interceptor to the client
client.addErrorInterceptor(advancedRetry);
```

## Resilient Retry Interceptor

For distributed systems and high-reliability environments, use the resilient retry interceptor which is pre-configured with optimized settings:

```typescript
import { ApiClient, createResilientRetryInterceptor } from '@bluefly/api-client';

const client = new ApiClient({
  baseUrl: 'https://api.example.com',
});

// Create a resilient retry interceptor (pre-configured for distributed systems)
const resilientRetry = createResilientRetryInterceptor({
  // Optionally override any settings
  maxRetries: 8,
  timeout: 120000,
});

// Add the resilient retry interceptor to the client
client.addErrorInterceptor(resilientRetry);
```

## Advanced Retry Options

The advanced and resilient retry interceptors support the following configuration options:

| Option                 | Description                                                  | Default                                       |
| ---------------------- | ------------------------------------------------------------ | --------------------------------------------- |
| `maxRetries`           | Maximum number of retry attempts                             | 3                                             |
| `baseDelayMs`          | Base delay between retries in milliseconds                   | 100                                           |
| `maxDelayMs`           | Maximum delay between retries in milliseconds                | 30000                                         |
| `backoffFactor`        | Factor to apply to the delay for exponential backoff         | 2                                             |
| `jitterFactor`         | Random jitter factor to add to backoff (0-1)                 | 0.25                                          |
| `retryableStatusCodes` | HTTP status codes that should trigger a retry                | [408, 425, 429, 500, 502, 503, 504, 507, 509] |
| `timeout`              | Timeout for the entire retry sequence in milliseconds        | 60000                                         |
| `retryNetworkErrors`   | Whether to retry network errors                              | true                                          |
| `logger`               | Logger to use for retry logging                              | ConsoleLogger                                 |
| `shouldRetry`          | Custom function to determine if an error should be retried   | -                                             |
| `calculateDelay`       | Custom function to calculate the delay before the next retry | -                                             |
| `onRetry`              | Callback to execute before each retry                        | -                                             |
| `onRetryExhausted`     | Callback to execute when retries are exhausted               | -                                             |

## Best Practices

1. **Use exponential backoff**: Always use exponential backoff to prevent overwhelming services during recovery.

2. **Add jitter**: Include jitter to prevent the "thundering herd" problem where many clients retry simultaneously.

3. **Set appropriate timeouts**: Configure timeout values that make sense for your application's requirements.

4. **Respect rate limits**: Retry interceptors automatically respect the `Retry-After` header for rate-limited requests.

5. **Monitor retries**: Use the `onRetry` and `onRetryExhausted` callbacks to log and monitor retry behavior.

6. **Identify retriable errors**: Customize the `shouldRetry` function for your specific API's error patterns.

7. **Implement circuit breakers**: For complex distributed systems, combine retry interceptors with circuit breakers to prevent cascading failures.

## Examples

See the [examples directory](../examples/) for complete working examples:

- [Basic Retry Example](../examples/basic-retry.js)
- [Advanced Retry Example](../examples/advanced-retry.js)
- [Resilient Retry Example](../examples/resilient-retry.js)
