# BFAPI Framework

The BFAPI Framework provides a structured approach to building RESTful APIs with Express. It offers a comprehensive set of tools and patterns to ensure consistent API design, robust error handling, request validation, and documentation.

## Core Features

- **Type-safe Request Validation**: Uses Zod schemas to validate request bodies, query parameters, and URL parameters
- **Standardized Error Handling**: Provides a consistent error response format and centralized error handling
- **Consistent Response Format**: Ensures all API responses follow a standard structure
- **Centralized Route Registry**: Organizes and registers routes in a type-safe manner
- **Performance Monitoring**: Tracks request processing time and provides hooks for additional metrics
- **OpenAPI Documentation**: Automatically generates OpenAPI documentation from route definitions
- **Middleware Management**: Applies middleware at global, group, and route levels

## Getting Started

### Installation

The framework is part of the BFAPI codebase and doesn't require separate installation.

### Basic Usage

1. Create a controller class that extends `BaseController`
2. Define request validation schemas using Zod
3. Implement controller methods using `handleRequest`
4. Define routes using the `RouteGroupDefinition` interface
5. Register routes with the `RouteRegistry`
6. Apply routes to an Express application

## Components

### BaseController

The `BaseController` provides a base class for all API controllers with methods for handling requests, validating input, and formatting responses.

```typescript
import [BaseController] from '../framework/BaseController';
import [z] from 'zod';

// Define schema
const UserRequestSchema = z.object({
  name: z.string().min(1),
  email: z.string().email()
});

// Define response type
interface UserResponse {
  id: string;
  name: string;
  email: string;
}

// Create controller
class UserController extends BaseController {
  async createUser(req: Request, res: Response, next: NextFunction): Promise<void> {
    await this.handleRequest<z.infer<typeof UserRequestSchema>, UserResponse>(
      req, res, next,
      UserRequestSchema,
      async (validatedData, correlationId) => {
        // Process the request
        const user = await userService.createUser(validatedData);
        return user;
      }
    );
  }
}
```

### ErrorHandler

The `ErrorHandler` provides standardized error handling with custom error classes for different error types.

```typescript
import { ValidationError, NotFoundError, errorHandlerMiddleware } from '../framework/ErrorHandler';

// Use custom error types
if (!user) {
  throw new NotFoundError('User not found');
}

// Apply error handling middleware
app.use(errorHandlerMiddleware);
```

### RouteRegistry

The `RouteRegistry` centralizes route registration and organization.

```typescript
import [RouteRegistry] from '../framework/RouteRegistry';

// Define routes
const userRoutes = {
  prefix: '/users',
  middleware: [authMiddleware],
  routes: [
    {
      path: '/',
      method: 'post',
      handler: userController.createUser.bind(userController),
      description: 'Create a new user'
    },
    {
      path: '/:id',
      method: 'get',
      handler: userController.getUser.bind(userController),
      description: 'Get user by ID'
    }
  ]
};

// Register routes
const registry = new RouteRegistry();
registry.registerRouteGroup('users', userRoutes);

// Apply routes to Express app
registry.applyRoutes(app);
```

### ValidationMiddleware

The `ValidationMiddleware` provides request validation using Zod schemas.

```typescript
import [createValidationMiddleware] from '../framework/ValidationMiddleware';
import [z] from 'zod';

// Define schema
const UserSchema = z.object({
  name: z.string().min(1),
  email: z.string().email()
});

// Create validation middleware
const validateUser = createValidationMiddleware({ body: UserSchema });

// Apply middleware to route
router.post('/users', validateUser, userController.createUser);
```

## Response Format

All API responses follow a standard format:

```json
{
  "data": {
    // Response data
  },
  "metadata": {
    "processing_time_ms": 123,
    "correlation_id": "uuid-v4-correlation-id",
    // Additional metadata
  }
}
```

Error responses follow this format:

```json
{
  "error": {
    "message": "Error message",
    "code": "ERROR_CODE",
    "correlationId": "uuid-v4-correlation-id",
    "timestamp": "2023-06-04T12:34:56.789Z"
  }
}
```

## Best Practices

1. **Use Zod Schemas**: Define Zod schemas for all request validation to ensure type safety
2. **Extend BaseController**: All controllers should extend the BaseController class
3. **Group Related Routes**: Organize related routes into route groups with appropriate prefixes
4. **Apply Appropriate Middleware**: Use authentication and validation middleware for all routes
5. **Handle Errors Properly**: Use custom error classes for different error types
6. **Include Correlation IDs**: Pass correlation IDs between services for tracing
7. **Document OpenAPI**: Include OpenAPI documentation for all routes
8. **Use Dependency Injection**: Inject services into controllers for better testability

## Example Implementation

See the following files for a complete example:

- `/src/framework/BaseController.ts`: Base controller class
- `/src/framework/ErrorHandler.ts`: Error handling middleware and custom error classes
- `/src/framework/RouteRegistry.ts`: Route registration and organization
- `/src/framework/ValidationMiddleware.ts`: Request validation middleware
- `/src/controllers/examples/AiController.ts`: Example controller implementation
- `/src/routes/examples/ai.routes.ts`: Example route definition
- `/src/examples/app-with-framework.ts`: Example application setup

## OpenAPI Integration

The RouteRegistry can automatically generate OpenAPI paths documentation:

```typescript
// Generate OpenAPI paths
const openApiPaths = routeRegistry.generateOpenApiPaths();

// Serve OpenAPI documentation
app.get('/openapi.json', (req, res) => {
  res.json({
    openapi: '3.0.0',
    info: {
      title: 'API Title',
      version: '1.0.0',
      description: 'API Description'
    },
    paths: openApiPaths
  });
});
```

## Testing

To test controllers that use the framework:

```typescript
import { Request, Response, NextFunction } from 'express';
import [AiController] from '../controllers/AiController';

describe('AiController', () => {
  let controller: AiController;
  let req: Partial<Request>;
  let res: Partial<Response>;
  let next: NextFunction;
  
  beforeEach(() => {
    controller = new AiController();
    req = { body: {}, headers: {} };
    res = { json: jest.fn() };
    next = jest.fn();
  });
  
  it('should generate docstring successfully', async () => {
    req.body = {
      code: 'function test() {}',
      language: 'javascript'
    };
    
    await controller.generateDocstring(req as Request, res as Response, next);
    
    expect(res.json).toHaveBeenCalled();
    const response = (res.json as jest.Mock).mock.calls[0][0];
    expect(response.data).toBeDefined();
    expect(response.metadata).toBeDefined();
  });
});