# WebSocket Support

The API framework includes comprehensive WebSocket support, allowing you to build real-time applications with the same security, validation, and observability features as REST and GraphQL APIs.

## Features

- **Socket.IO Integration**: Built on Socket.IO with Express middleware
- **Zod Schema Validation**: Type-safe message handling with Zod schema validation
- **Room/Channel Support**: Built-in support for rooms, channels, and private messaging
- **Security Features**: Authentication, authorization, rate limiting, and connection limiting
- **Observability**: Metrics and tracing for WebSocket connections and messages
- **Real-time Features**: Typing indicators, presence awareness, and status updates
- **Full Framework Integration**: Compatible with all other framework features

## Usage

### Basic Setup

```typescript
import express from 'express';
import http from 'http';
import [WebSocketController] from '../framework/websocket';
import { PrometheusMetrics, OpenTelemetryTracing } from '../framework';

// Create Express app and HTTP server
const app = express();
const httpServer = http.createServer(app);

// Create metrics and tracing
const metrics = new PrometheusMetrics();
const tracing = new OpenTelemetryTracing();

// Initialize tracing
await tracing.init();

// Create WebSocket controller
const wsController = new WebSocketController(httpServer, {
  namespace: '/ws',
  cors: true,
  metrics,
  tracing
});

// Register message handlers
wsController.registerHandler('echo', z.string(), async (socket, data, context) => {
  socket.emit('echo_response', `Echo: [data]`);
});

// Start WebSocket server
wsController.start();

// Start HTTP server
httpServer.listen(3000, () => {
  console.log('Server running at http://localhost:3000');
  console.log('WebSocket endpoint: ws://localhost:3000/ws');
});
```

### Using Zod Schemas

You can validate WebSocket messages using Zod schemas:

```typescript
import [z] from 'zod';
import [WebSocketController] from '../framework/websocket';

// Define validation schema
const ChatMessageSchema = z.object({
  room: z.string().min(1).max(100),
  message: z.string().min(1).max(1000),
  type: z.enum(['text', 'image', 'system']).default('text')
});

// Create WebSocket controller
const chatController = new WebSocketController(httpServer, options);

// Register message handler with schema validation
chatController.registerHandler(
  'chat_message',
  ChatMessageSchema,
  async (socket, data, context) => {
    const { room, message, type } = data;
    const { connectionId, user } = context;
    
    // Broadcast message to room
    chatController.broadcastToRoom(room, 'chat_message', {
      id: context.correlationId,
      userId: user?.id || connectionId,
      room,
      message,
      type,
      timestamp: Date.now()
    });
  }
);
```

### Authentication and Authorization

The WebSocket controller supports authentication and authorization:

```typescript
// Create WebSocket security
const wsSecurity = new WebSocketSecurity({
  jwt: {
    secret: process.env.JWT_SECRET,
    algorithm: 'HS256'
  },
  apiKey: {
    header: 'x-api-key',
    keys: [process.env.API_KEY]
  },
  roles: {
    default: 'user',
    admin: 'admin',
    eventPermissions: {
      'admin_action': ['admin']
    }
  }
});

// Create WebSocket controller with authentication and authorization
const wsController = new WebSocketController(httpServer, {
  // ...
  authenticate: async (socket) => {
    const authResult = await wsSecurity.authenticate(socket);
    return authResult.user;
  },
  authorize: async (socket, user, event) => {
    return await wsSecurity.authorize(socket, event);
  }
});
```

### Room Management

The WebSocket controller provides built-in support for rooms:

```typescript
// Join a room
wsController.joinRoom(connectionId, 'room-name');

// Leave a room
wsController.leaveRoom(connectionId, 'room-name');

// Broadcast to a room
wsController.broadcastToRoom('room-name', 'event-name', {
  message: 'Hello room!'
});

// Get all rooms
const rooms = wsController.getRooms();

// Get connections in a room
const connections = wsController.getConnectionsInRoom('room-name');
```

## Advanced Features

### Typing Indicators

Implement typing indicators for chat applications:

```typescript
// Define typing schema
const TypingSchema = z.object({
  room: z.string().min(1),
  isTyping: z.boolean()
});

// Register typing handler
wsController.registerHandler('typing', TypingSchema, async (socket, data, context) => {
  const { room, isTyping } = data;
  const { connectionId, user } = context;
  
  // Broadcast typing status to room
  wsController.broadcastToRoom(room, 'typing', {
    userId: user?.id || connectionId,
    room,
    isTyping,
    timestamp: Date.now()
  });
  
  // Set timeout to clear typing indicator
  if (isTyping) {
    setTimeout(() => {
      wsController.broadcastToRoom(room, 'typing', {
        userId: user?.id || connectionId,
        room,
        isTyping: false,
        timestamp: Date.now()
      });
    }, 5000);
  }
});
```

### Private Messaging

Implement private messaging between users:

```typescript
// Define private message schema
const PrivateMessageSchema = z.object({
  to: z.string().min(1),
  message: z.string().min(1).max(1000)
});

// Register private message handler
wsController.registerHandler(
  'private_message', 
  PrivateMessageSchema, 
  async (socket, data, context) => {
    const { to, message } = data;
    const { connectionId, user, correlationId } = context;
    
    // Find recipient connections
    const recipientSockets = wsController.getConnectionsByUser(to);
    
    // Send message to all recipient connections
    const messageObj = {
      id: correlationId,
      from: user?.id || connectionId,
      to,
      message,
      timestamp: Date.now()
    };
    
    let delivered = false;
    for (const recipientSocket of recipientSockets) {
      recipientSocket.emit('private_message', messageObj);
      delivered = true;
    }
    
    // Send delivery status to sender
    socket.emit('message_status', {
      id: correlationId,
      delivered,
      timestamp: Date.now()
    });
  }
);
```

### Presence and Status

Implement user presence and status updates:

```typescript
// Define status schema
const UserStatusSchema = z.object({
  status: z.enum(['online', 'away', 'busy', 'offline']),
  statusMessage: z.string().max(100).optional()
});

// User status tracking
const userStatus = new Map();

// Register status handler
wsController.registerHandler(
  'user_status',
  UserStatusSchema,
  async (socket, data, context) => {
    const { status, statusMessage } = data;
    const { connectionId, user } = context;
    const userId = user?.id || connectionId;
    
    // Update status
    userStatus.set(userId, {
      status,
      statusMessage,
      lastActivity: Date.now()
    });
    
    // Broadcast status update to interested parties
    // (e.g., friends, room members, etc.)
  }
);
```

## Security Considerations

### Connection Limiting

Protect against DoS attacks by limiting connections:

```typescript
const wsController = new WebSocketController(httpServer, {
  // ...
  maxConnectionsPerIp: 10,
  messageRateLimit: {
    maxRequests: 100,
    windowMs: 60000 // 1 minute
  }
});
```

### IP Blocking

Block or allow specific IP addresses:

```typescript
const wsSecurity = new WebSocketSecurity({
  // ...
  ipBlocking: {
    enabled: true,
    blockedIps: ['192.168.1.100', '10.0.0.5'],
    allowedIps: ['192.168.1.200']
  }
});
```

### JWT Authentication

Secure connections with JWT authentication:

```typescript
const wsSecurity = new WebSocketSecurity({
  jwt: {
    secret: process.env.JWT_SECRET,
    algorithm: 'HS256',
    expiresIn: '1h',
    issuer: 'your-app',
    audience: 'your-api'
  }
});

// Generate a token
const token = wsSecurity.generateToken({
  id: 'user-123',
  role: 'user'
});

// Client connection with token
const socket = io('ws://localhost:3000/ws', {
  auth: {
    token: token
  }
});
```

## Performance Optimization

### Connection Pooling

Optimize WebSocket connections for high traffic:

```typescript
const wsController = new WebSocketController(httpServer, {
  // ...
  socketOptions: {
    pingInterval: 25000,
    pingTimeout: 20000,
    connectTimeout: 30000,
    maxHttpBufferSize: 1e6 // 1MB
  }
});
```

### Message Batching

Reduce network overhead by batching messages:

```typescript
// Client-side batching
const batchedMessages = [];

function sendMessage(message) {
  batchedMessages.push(message);
  
  if (batchedMessages.length >= 10) {
    flushMessages();
  } else if (batchedMessages.length === 1) {
    setTimeout(flushMessages, 100);
  }
}

function flushMessages() {
  if (batchedMessages.length === 0) return;
  
  socket.emit('batch_messages', batchedMessages);
  batchedMessages.length = 0;
}

// Server-side handling
wsController.registerHandler(
  'batch_messages',
  z.array(MessageSchema),
  async (socket, messages, context) => [for (const message of messages) {
      // Process each message]
  }
);
```

### Scaling Considerations

For scaling WebSocket applications:

- Use Redis adapter for multi-server deployments
- Implement sticky sessions for load balancing
- Consider horizontal scaling with shared state

## Integration with Other Framework Components

### REST API Integration

You can run both REST and WebSocket APIs in the same application:

```typescript
// Create REST routes
const routeRegistry = new RouteRegistry();
routeRegistry.registerRouteGroup('users', [// REST route configuration]);

// Apply REST routes
routeRegistry.applyRoutes(app);

// Create WebSocket controller
const wsController = new WebSocketController(httpServer, [// WebSocket configuration]);

// Register WebSocket handlers
wsController.registerHandler('ping', z.any(), async (socket) => {
  socket.emit('pong', { timestamp: Date.now() });
});

// Start WebSocket server
wsController.start();
```

### Event-Driven Architecture

Integrate WebSockets with the event system:

```typescript
import [EventEmitter] from '../framework/events';

const eventEmitter = new EventEmitter();
const wsController = new WebSocketController(httpServer, options);

// Register event listener to broadcast events to WebSocket clients
eventEmitter.on('user:created', (user) => {
  wsController.broadcast('user_created', {
    id: user.id,
    name: user.name,
    timestamp: Date.now()
  });
});

// Register WebSocket handler that emits events
wsController.registerHandler(
  'create_post',
  PostSchema,
  async (socket, data, context) => {
    // Create post in database
    const post = await createPost(data);
    
    // Emit event
    eventEmitter.emit('post:created', post);
    
    // Respond to client
    socket.emit('post_created', {
      id: post.id,
      success: true
    });
  }
);
```

## Metrics and Monitoring

The WebSocket controller provides built-in metrics for monitoring:

```typescript
// Create WebSocket metrics
const wsMetrics = new WebSocketMetrics(metrics, {
  namespace: 'websocket',
  trackRooms: true,
  trackUsers: true,
  trackMessageSizes: true,
  customMetrics: {
    counters: [
      {
        name: 'custom_events_total',
        help: 'Total number of custom events',
        labelNames: ['event_type']
      }
    ]
  }
});

// Track connections
wsMetrics.trackConnection(socket, true);

// Track messages
wsMetrics.trackReceivedMessage(socket, 'chat_message', data);
wsMetrics.trackSentMessage(socket, 'chat_message', data);

// Track room activity
wsMetrics.trackRoomJoin('room-name');
wsMetrics.trackRoomLeave('room-name');

// Track custom metrics
wsMetrics.getCounter('custom_events_total').inc({ event_type: 'login' });
```

The following metrics are available:

- **connections_total**: Total number of WebSocket connections
- **connections_active**: Number of active WebSocket connections
- **connection_duration_seconds**: WebSocket connection duration
- **messages_received_total**: Total number of messages received
- **messages_sent_total**: Total number of messages sent
- **messages_error_total**: Total number of message errors
- **message_processing_time_seconds**: Message processing time
- **room_connections**: Number of connections in a room
- **room_messages_total**: Total number of messages sent to a room

## Example Implementation

See the `/examples/websocket-api` directory for a complete example of a WebSocket API using the framework, including:

- Real-time chat functionality
- Authentication and authorization
- Room management
- Private messaging
- Typing indicators
- User presence and status
- Metrics and tracing