# API Reference

The LLM Platform API provides a comprehensive set of endpoints for interacting with the LLM Platform. This document serves as a guide to using the API effectively.

## Interactive Documentation

For interactive API documentation with the ability to try out endpoints:

[Redoc API Docs](../public/api-docs.html)

## Base URLs

| Environment | Base URL                       |
|-------------|--------------------------------|
| Production  | `https://api.bluefly.io/api/v1`|
| Staging     | `https://staging-api.bluefly.io/api/v1` |
| Local       | `http://localhost:3000/v1`     |

## Authentication

All API requests (except explicitly noted public endpoints) require authentication using one of the following methods:

### Bearer Token Authentication

```
Authorization: Bearer eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9...
```

JWT tokens are valid for 15 minutes and can be refreshed using the refresh token endpoint.

### API Key Authentication

```
X-API-Key: api_key_123456789
```

API keys are long-lived credentials for service-to-service communication.

### Public Endpoints

The following endpoints do not require authentication:

- **POST** `/auth/login` - Authenticate and get access tokens
- **POST** `/auth/refresh` - Refresh an expired access token
- **GET** `/health` - Check API health status
- **GET** `/version` - Get API version information

## Request & Response Format

### Content Types

The API accepts and returns JSON data:

```
Content-Type: application/json
Accept: application/json
```

### Standard Response Format

All API responses follow a consistent format:

```json
{
  "data": { ... },       // The response payload (null for errors)
  "meta": {              // Optional metadata
    "pagination": {      // Present for paginated results
      "total": 100,
      "page": 1,
      "limit": 10,
      "pages": 10
    }
  },
  "error": {             // Present only for error responses
    "code": "string",    // Machine-readable error code
    "message": "string", // Human-readable error message
    "details": {},       // Additional context (optional)
    "request_id": "uuid" // Unique request identifier for troubleshooting
  }
}
```

### Pagination

Endpoints that return collections support pagination:

| Parameter | Description                           | Default |
|-----------|---------------------------------------|---------|
| `page`    | Page number (1-based)                 | 1       |
| `limit`   | Number of items per page (max 100)    | 20      |
| `sort`    | Field to sort by                      | id      |
| `order`   | Sort order (`asc` or `desc`)          | asc     |

Example:
```
GET /models?page=2&limit=50&sort=created_at&order=desc
```

## Rate Limiting

The API implements rate limiting to ensure fair usage:

| Plan      | Rate Limit                     | Header                     |
|-----------|--------------------------------|----------------------------|
| Free      | 100 requests per minute        | `X-RateLimit-Limit`        |
| Pro       | 1,000 requests per minute      | `X-RateLimit-Remaining`    |
| Enterprise| Custom                         | `X-RateLimit-Reset`        |

Rate limit headers are included in all responses:

```
X-RateLimit-Limit: 100
X-RateLimit-Remaining: 95
X-RateLimit-Reset: 1623456789
```

## Error Handling

### HTTP Status Codes

The API uses standard HTTP status codes:

| Status Code | Description                                                   |
|-------------|---------------------------------------------------------------|
| 200         | Success                                                       |
| 201         | Resource created                                              |
| 204         | Success (no content returned)                                 |
| 400         | Bad request (validation error)                                |
| 401         | Unauthorized (invalid or missing authentication)              |
| 403         | Forbidden (insufficient permissions)                          |
| 404         | Resource not found                                            |
| 422         | Unprocessable entity (valid request but business logic error) |
| 429         | Too many requests (rate limit exceeded)                       |
| 500         | Internal server error                                         |
| 503         | Service unavailable (maintenance or overload)                 |

### Error Response Format

Error responses include a detailed error object:

```json
{
  "data": null,
  "error": {
    "code": "VALIDATION_ERROR",
    "message": "Invalid input parameters",
    "details": {
      "name": "Name is required",
      "email": "Invalid email format"
    },
    "request_id": "req_1234567890"
  }
}
```

### Common Error Codes

| Error Code          | Description                                   |
|---------------------|-----------------------------------------------|
| AUTHENTICATION      | Authentication failed                         |
| AUTHORIZATION       | Insufficient permissions                      |
| VALIDATION_ERROR    | Invalid input parameters                      |
| RESOURCE_NOT_FOUND  | Requested resource does not exist             |
| RATE_LIMIT_EXCEEDED | Too many requests                             |
| INTERNAL_ERROR      | Server-side error                             |
| SERVICE_UNAVAILABLE | Service temporarily unavailable               |

## API Endpoints

### Models API

Endpoints for managing LLM models and performing inference.

#### List Models

```
GET /models
```

Returns a list of available models.

**Query Parameters:**

| Parameter   | Type   | Description                                       | Default |
|-------------|--------|---------------------------------------------------|---------|
| `provider`  | string | Filter by provider (e.g., "bfllm", "openai")      | null    |
| `type`      | string | Filter by model type (e.g., "text", "embedding")  | null    |
| `status`    | string | Filter by status ("active", "inactive", "hidden") | null    |

**Response Example:**

```json
{
  "data": [
    {
      "id": "model_123",
      "name": "BlueGPT-1",
      "provider": "bfllm",
      "type": "text",
      "status": "active",
      "description": "General purpose text generation model",
      "created_at": "2023-01-15T14:22:31Z",
      "updated_at": "2023-06-01T09:12:44Z",
      "capabilities": ["text-generation", "embedding", "classification"],
      "contextWindow": 8192,
      "maxTokens": 4096
    },
    {
      "id": "model_456",
      "name": "BlueEmbed-1",
      "provider": "bfllm",
      "type": "embedding",
      "status": "active",
      "description": "Text embedding model",
      "created_at": "2023-02-10T11:15:22Z",
      "updated_at": "2023-06-01T09:13:12Z",
      "capabilities": ["embedding"],
      "dimensions": 768
    }
  ],
  "meta": {
    "pagination": {
      "total": 8,
      "page": 1,
      "limit": 10,
      "pages": 1
    }
  }
}
```

#### Get Model Details

```
GET /models/[modelId]
```

Returns detailed information about a specific model.

**Path Parameters:**

| Parameter | Type   | Description              |
|-----------|--------|--------------------------|
| `modelId` | string | Unique model identifier  |

**Response Example:**

```json
{
  "data": {
    "id": "model_123",
    "name": "BlueGPT-1",
    "provider": "bfllm",
    "type": "text",
    "status": "active",
    "description": "General purpose text generation model",
    "created_at": "2023-01-15T14:22:31Z",
    "updated_at": "2023-06-01T09:12:44Z",
    "capabilities": ["text-generation", "embedding", "classification"],
    "contextWindow": 8192,
    "maxTokens": 4096,
    "parameters": {
      "temperature": {
        "default": 0.7,
        "min": 0.0,
        "max": 1.0
      },
      "top_p": {
        "default": 0.9,
        "min": 0.0,
        "max": 1.0
      }
    },
    "versions": [
      {
        "id": "v1.0",
        "name": "Version 1.0",
        "status": "active",
        "created_at": "2023-01-15T14:22:31Z"
      },
      {
        "id": "v1.1",
        "name": "Version 1.1",
        "status": "active",
        "created_at": "2023-03-20T10:45:22Z"
      }
    ]
  }
}
```

#### Text Completion

```
POST /models/[modelId]/predict
```

Generate text completion using a specific model.

**Path Parameters:**

| Parameter | Type   | Description              |
|-----------|--------|--------------------------|
| `modelId` | string | Unique model identifier  |

**Request Body:**

```json
{
  "prompt": "Once upon a time in a land far away",
  "max_tokens": 100,
  "temperature": 0.7,
  "top_p": 0.9,
  "stop": ["\n\n"],
  "frequency_penalty": 0.0,
  "presence_penalty": 0.0,
  "stream": false
}
```

| Parameter          | Type    | Description                                     | Default |
|--------------------|---------|-------------------------------------------------|---------|
| `prompt`           | string  | Input text to complete                          | Required|
| `max_tokens`       | integer | Maximum number of tokens to generate            | 16      |
| `temperature`      | float   | Randomness of generation (0.0-1.0)              | 0.7     |
| `top_p`            | float   | Nucleus sampling parameter (0.0-1.0)            | 0.9     |
| `stop`             | array   | Sequences where generation should stop          | []      |
| `frequency_penalty`| float   | Penalty for token frequency (-2.0 to 2.0)       | 0.0     |
| `presence_penalty` | float   | Penalty for token presence (-2.0 to 2.0)        | 0.0     |
| `stream`           | boolean | Whether to stream the response                  | false   |

**Response Example:**

```json
{
  "data": {
    "id": "cmpl_123456",
    "model": "model_123",
    "created_at": "2023-06-15T12:34:56Z",
    "text": "Once upon a time in a land far away, there lived a wise old wizard who had a magical library filled with ancient scrolls and mystical artifacts. The wizard was known throughout the kingdom for his ability to solve the most perplexing problems with his vast knowledge.",
    "tokens_used": {
      "prompt": 9,
      "completion": 42,
      "total": 51
    },
    "finish_reason": "length"
  }
}
```

#### Chat Completion

```
POST /v1/llm/inference/chat
```

Generate a chat completion with multiple messages.

**Request Body:**

```json
{
  "model": "model_123",
  "messages": [
    {"role": "system", "content": "You are a helpful assistant."},
    {"role": "user", "content": "Hello, who are you?"},
    {"role": "assistant", "content": "I'm BlueFly's AI assistant. How can I help you today?"},
    {"role": "user", "content": "What can you do?"}
  ],
  "temperature": 0.7,
  "max_tokens": 100,
  "stream": false
}
```

| Parameter    | Type    | Description                                     | Default |
|--------------|---------|-------------------------------------------------|---------|
| `model`      | string  | Model ID to use                                 | Required|
| `messages`   | array   | Array of message objects                        | Required|
| `temperature`| float   | Randomness of generation (0.0-1.0)              | 0.7     |
| `max_tokens` | integer | Maximum number of tokens to generate            | 100     |
| `stream`     | boolean | Whether to stream the response                  | false   |

**Response Example:**

```json
{
  "data": {
    "id": "chat_123456",
    "model": "model_123",
    "created_at": "2023-06-15T12:34:56Z",
    "message": {
      "role": "assistant",
      "content": "I can help you with a variety of tasks including answering questions, generating creative content, summarizing information, and providing assistance with various topics. Just let me know what you need help with!"
    },
    "tokens_used": {
      "prompt": 63,
      "completion": 37,
      "total": 100
    },
    "finish_reason": "stop"
  }
}
```

#### Streaming Responses

For streaming responses, set `stream: true` in your request. The API will return a stream of server-sent events:

```
event: completion
data: {"text": " Once", "index": 0}

event: completion
data: {"text": " upon", "index": 1}

event: completion
data: {"text": " a", "index": 2}

...

event: done
data: {"finish_reason": "stop", "tokens_used": {"prompt": 9, "completion": 42, "total": 51}}
```

### Agents API

Endpoints for working with intelligent agents.

#### Run Agent

```
POST /agents/run
```

Execute an agent with specified input and options.

**Request Body:**

```json
{
  "input": "What's the weather like in New York today?",
  "chatHistory": [
    {"role": "user", "content": "Hello agent"},
    {"role": "assistant", "content": "Hello! How can I help you today?"}
  ],
  "options": {
    "tools": ["weather", "search"],
    "max_iterations": 5,
    "verbose": true
  }
}
```

| Parameter     | Type    | Description                                     | Default |
|---------------|---------|-------------------------------------------------|---------|
| `input`       | string  | Input query for the agent                       | Required|
| `chatHistory` | array   | Previous conversation messages                  | []      |
| `options`     | object  | Additional options for agent execution          | {}      |

**Response Example:**

```json
{
  "data": {
    "response": "The weather in New York today is partly cloudy with a high of 76°F (24°C) and a low of 62°F (17°C). There's a 20% chance of rain in the afternoon.",
    "steps": [
      {
        "thought": "I need to check the weather in New York",
        "action": "weather",
        "action_input": {"location": "New York, NY"},
        "observation": "Weather data for New York: partly cloudy, high 76°F, low 62°F, 20% chance of rain in afternoon"
      }
    ],
    "toolCalls": [
      {
        "tool": "weather",
        "input": {"location": "New York, NY"},
        "output": "Weather data for New York: partly cloudy, high 76°F, low 62°F, 20% chance of rain in afternoon"
      }
    ]
  }
}
```

## Common Use Cases

### Authentication Flow

1. **Login and obtain tokens**:

```
POST /auth/login
```
```json
{
  "username": "user@example.com",
  "password": "securepassword"
}
```

2. **Use the access token for API requests**:

```
GET /models
Authorization: Bearer eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9...
```

3. **Refresh an expired token**:

```
POST /auth/refresh
```
```json
{
  "refresh_token": "eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9..."
}
```

### Text Generation Flow

1. **List available models**:

```
GET /models?type=text
```

2. **Generate text with a selected model**:

```
POST /models/model_123/predict
```
```json
{
  "prompt": "Write a poem about artificial intelligence",
  "max_tokens": 200,
  "temperature": 0.8
}
```

## SDKs and Client Libraries

LLM Platform provides official client libraries for the following languages:

- [JavaScript/TypeScript](https://github.com/bluefly/bfapi-js)
- [Python](https://github.com/bluefly/bfapi-python)
- [PHP](https://github.com/bluefly/bfapi-php)

Example using the JavaScript SDK:

```javascript
import [LLM PlatformAPI] from '@bluefly/bfapi-js';

const bfapi = new LLM PlatformAPI({
  apiKey: 'your_api_key',
  baseUrl: 'https://api.bluefly.io/api/v1'
});

const response = await bfapi.models.predict('model_123', {
  prompt: 'Hello, world!',
  max_tokens: 50
});

console.log(response.text);
```

## Best Practices

### Efficient API Usage

1. **Use pagination** for endpoints that return collections
2. **Minimize token usage** by sending concise prompts
3. **Implement caching** for frequently used model responses
4. **Use streaming** for real-time UI feedback on long generations
5. **Handle rate limits** gracefully with exponential backoff

### Security Recommendations

1. **Store API keys securely** and never expose them in client-side code
2. **Rotate API keys** periodically and after staff changes
3. **Use the minimum required scopes** for API keys
4. **Implement content filtering** before sending sensitive data
5. **Validate and sanitize** all user inputs before sending to the API

## API Versioning

The API uses a date-based versioning scheme. The current version is `v1`.

Major changes will be introduced in new API versions. We maintain backward compatibility within a version.

## Support

For API support, contact [api-support@bluefly.io](mailto:api-support@bluefly.io) or visit our [Developer Forum](https://community.bluefly.io).

## Rate Limits and Quotas

Each API key has associated quotas for requests and token usage. View your current usage and limits in the [Developer Dashboard](https://dashboard.bluefly.io).

## Webhooks

The API supports webhooks for asynchronous notifications about events:

1. **Register a webhook endpoint**:

```
POST /integrations/webhooks
```
```json
{
  "url": "https://example.com/webhook",
  "events": ["model.training.completed", "job.status.changed"],
  "secret": "webhook_signing_secret"
}
```

2. **Verify webhook signatures** using the `X-LLM Platform-Signature` header

## Changelog

| Date       | Changes                                           |
|------------|---------------------------------------------------|
| 2023-06-01 | Added streaming support for chat completions      |
| 2023-05-15 | Added agent workflow endpoints                    |
| 2023-04-10 | Enhanced model management API                     |
| 2023-03-01 | Initial API release                               |