# OpenAPI-First Development for AI Agents

This document serves as a comprehensive guide for AI agents on implementing OpenAPI-first development and validation practices across the LLM Platform AI Platform projects.

## 🚦 The OpenAPI-First Approach

### Core Principle

In the LLM Platform AI Platform, all API development **MUST** follow an OpenAPI-first approach, where:

1. API contracts are defined in OpenAPI 3.1 specification **BEFORE** any implementation
2. All API implementations are validated against their specifications
3. Documentation, SDKs, and client libraries are generated from OpenAPI specifications
4. Breaking changes are carefully managed through proper versioning

### Why This Matters

- **Consistency**: Ensures all services provide predictable, well-documented interfaces
- **Interoperability**: Enables seamless integration between services
- **Quality**: Validates requests and responses against defined schemas
- **Developer Experience**: Provides clear, accurate documentation and type-safe clients
- **Automation**: Facilitates automated testing, validation, and code generation

## 📋 AI Agent OpenAPI Workflow

### 1. Locating the OpenAPI Specification

When working with any service in the platform, first locate its OpenAPI specification:

| Service | Primary OpenAPI Location | Purpose |
|---------|--------------------------|---------|
| bfapi | `/bfapi/openapi.yaml` | API Gateway definitions |
| llm-mcp | `/llm-mcp/openapi.yaml` | Model Context Protocol definitions |
| bfllm | `/bfllm/openapi.yaml` | LLM service definitions |
| bfrfp | `/bfrfp/openapi.yaml` | Content Engine definitions |
| llm_platform_recipe | `/llm_platform_recipe/openapi/llm_platform_recipe.openapi.yml` | Drupal module API definitions |

### 2. Understanding the Contract

Before modifying endpoints or adding new features:

1. Review the existing API contract to understand:
   - Available endpoints and their purpose
   - Request and response schemas
   - Authentication requirements
   - Common patterns and conventions

2. Study how the specification is organized:
   - Components and shared schemas
   - Security definitions
   - Parameter definitions
   - Response patterns

### 3. Modifying or Extending the API

When implementing new features or modifying existing ones:

1. First update the OpenAPI specification to define the change
2. Validate the updated specification using validation tools
3. Only then implement the code changes to match the specification
4. Write tests that validate against the specification

### 4. Validation and Quality Assurance

Always verify your changes using the appropriate tools:

```bash
# Validate OpenAPI syntax and standards compliance
npx @stoplight/spectral-cli lint ./openapi.yaml

# Check for breaking changes (service-specific)
node scripts/validate-api-compatibility.js openapi.yaml

# Test against the specification
npx dredd openapi.yaml http://localhost:3000
```

## 🛠️ Tools for OpenAPI Development

### Required Tools

The following tools are essential for OpenAPI development in this platform:

1. **Spectral**: Linting and validation
   ```bash
   npm install -g @stoplight/spectral-cli
   spectral lint openapi.yaml
   ```

2. **Express OpenAPI Validator**: Runtime validation
   ```javascript
   const OpenApiValidator = require('express-openapi-validator');
   app.use(OpenApiValidator.middleware({
     apiSpec: './openapi.yaml',
     validateRequests: true,
     validateResponses: true,
   }));
   ```

3. **OpenAPI Generator**: SDK generation
   ```bash
   npm install @openapitools/openapi-generator-cli -g
   openapi-generator-cli generate -i openapi.yaml -g typescript-axios -o ./client-sdk
   ```

4. **ReDoc/Swagger UI**: Documentation generation
   ```bash
   npm install -g redoc-cli
   redoc-cli bundle openapi.yaml -o api-docs.html
   ```

### Project-Specific Tools

Each project may have custom scripts and tools:

- `/bfapi/scripts/validate-openapi.js`: Validates OpenAPI specifications
- `/bfapi/scripts/generate-sdk.js`: Generates client SDKs
- `/bfapi/scripts/generate-api-docs.js`: Generates API documentation

### CI/CD Integration

OpenAPI validation is integrated into the CI/CD pipeline:

```yaml
# Example GitLab CI job
openapi-validate:
  stage: validate
  script:
    - npm install -g @stoplight/spectral-cli
    - spectral lint openapi.yaml
  only:
    changes:
      - "openapi.yaml"
      - "**/*openapi*.yaml"
```

## 📑 Schema Design Guidelines

### Common Schema Patterns

The platform uses consistent schema patterns that you should follow:

#### Response Envelope

```yaml
ApiResponse:
  type: object
  required:
    - status
    - data
  properties:
    status:
      type: string
      enum: [success, error]
    data:
      type: object
      description: The actual response data
    error:
      type: object
      properties:
        code:
          type: string
        message:
          type: string
```

#### Pagination Pattern

```yaml
PaginatedResponse:
  type: object
  required:
    - items
    - pagination
  properties:
    items:
      type: array
      items:
        $ref: '#/components/schemas/Item'
    pagination:
      type: object
      required:
        - total
        - page
        - limit
      properties:
        total:
          type: integer
        page:
          type: integer
        limit:
          type: integer
```

### Naming Conventions

- **Paths**: Use kebab-case for paths (`/model-registry/models`)
- **Parameters**: Use camelCase for parameters
- **Properties**: Use camelCase for properties in request/response bodies
- **Schemas**: Use PascalCase for schema names
- **Operations**: Use descriptive verbs (`createModel`, not `postModel`)

### Best Practices

1. **Use References**: Leverage `$ref` to maintain consistent schemas
   ```yaml
   $ref: '#/components/schemas/ErrorResponse'
   ```

2. **Provide Examples**: Include examples for complex objects
   ```yaml
   examples:
     success:
       value:
         id: "model-123"
         name: "gpt-4"
         status: "active"
   ```

3. **Document Everything**: Add clear descriptions for all components
   ```yaml
   description: "The unique identifier for the model"
   ```

4. **Define Required Fields**: Explicitly mark required properties
   ```yaml
   required:
     - id
     - name
   ```

5. **Use Appropriate Types**: Match types to their actual usage
   ```yaml
   type: integer
   format: int64  # For larger numbers
   ```

## 🔄 Integration with TDD

The OpenAPI-first approach integrates with the platform's mandatory TDD methodology:

### 1. Write Contract Tests First

Before implementing any API endpoint:

1. Define the API contract in the OpenAPI specification
2. Write tests that validate requests and responses against this contract
3. Ensure the tests fail initially (RED phase)
4. Implement the API to make the tests pass (GREEN phase)
5. Refactor while maintaining test compliance (REFACTOR phase)

### 2. Test Examples

```typescript
// Example contract test for an API endpoint
describe('GET /models/[modelId]', () => {
  beforeAll(() => {
    // Load and validate the OpenAPI spec
    openApiValidator.validateSpec('./openapi.yaml');
  });

  it('should return a response that matches the contract', async () => {
    // Arrange
    const modelId = 'test-model';
    
    // Act
    const response = await request(app).get(`/models/[modelId]`);
    
    // Assert
    expect(response.status).toBe(200);
    
    // Validate against OpenAPI schema
    const validationResult = openApiValidator.validateResponse(
      response.body,
      '/models/{modelId}',
      'get',
      '200'
    );
    
    expect(validationResult.valid).toBe(true);
  });
});
```

## 📊 Versioning and Compatibility

### API Versioning Strategy

The platform uses a clear versioning strategy:

1. **Path-Based Versioning**: Major versions are in the URL path (`/v1/models`, `/v2/models`)
2. **Semantic Versioning**: Following semver (MAJOR.MINOR.PATCH)
   - MAJOR: Breaking changes
   - MINOR: New features, backward compatible
   - PATCH: Bug fixes, backward compatible

### Breaking vs. Non-Breaking Changes

Understanding what constitutes a breaking change is critical:

#### Breaking Changes ❌
- Removing an endpoint
- Removing a required parameter
- Making an optional parameter required
- Changing a parameter type
- Removing a field from response
- Changing a field type in response
- Changing security requirements

#### Non-Breaking Changes ✅
- Adding a new endpoint
- Adding a new optional parameter
- Adding a new response field
- Extending enum values (with caution)
- Adding a new response type
- Improving documentation

### Compatibility Checking

Use built-in compatibility checking tools:

```bash
# Check for breaking changes
node scripts/validate-api-compatibility.js openapi.yaml main
```

## 🚀 Project-Specific Implementation

### bfapi

```yaml
# Key components in bfapi
paths:
  /models:
    get:
      summary: List all models
      # ...
  /models/[modelId]:
    get:
      summary: Get model details
      # ...
```

Implementation approach:
- Express routes match OpenAPI paths
- Request validation middleware ensures compliance
- Response validation ensures contract adherence
- Automated SDK generation for client libraries

### llm-mcp

```yaml
# Key components in llm-mcp
paths:
  /tools:
    get:
      summary: List available tools
      # ...
  /resources:
    get:
      summary: List available resources
      # ...
```

Implementation approach:
- Protocol definition using OpenAPI
- Tool registry management
- Resource orchestration
- Strong security definitions

### bfllm

```yaml
# Key components in bfllm
paths:
  /inference:
    post:
      summary: Perform model inference
      # ...
  /training:
    post:
      summary: Start model training
      # ...
```

Implementation approach:
- Model management API
- Inference endpoints
- Training and fine-tuning
- Agent configuration

## 📝 Documentation Best Practices

When working with OpenAPI specifications, ensure documentation is comprehensive:

1. **Clear Summary and Description**:
   ```yaml
   summary: "Get model details"
   description: "Retrieves detailed information about a specific model by its unique identifier."
   ```

2. **Parameter Documentation**:
   ```yaml
   parameters:
     - name: modelId
       in: path
       required: true
       description: "The unique identifier of the model to retrieve."
       schema:
         type: string
   ```

3. **Response Documentation**:
   ```yaml
   responses:
     '200':
       description: "Successfully retrieved model details."
       content:
         application/json:
           schema:
             $ref: '#/components/schemas/ModelDetail'
   ```

4. **Schema Property Documentation**:
   ```yaml
   ModelDetail:
     type: object
     properties:
       id:
         type: string
         description: "Unique identifier for the model."
       name:
         type: string
         description: "Human-readable name of the model."
   ```

## 🛡️ Security Definition Best Practices

All APIs must define security requirements consistently:

```yaml
# Security Schemes
components:
  securitySchemes:
    BearerAuth:
      type: http
      scheme: bearer
      bearerFormat: JWT
      description: "JWT token for authentication"
    ApiKeyAuth:
      type: apiKey
      in: header
      name: X-API-Key
      description: "API key for service-to-service authentication"

# Applied Security
security:
  - BearerAuth: []
```

## 🧩 AI Agent Checklist for OpenAPI Work

When working with APIs in this repository, always:

1. **✅ Find the OpenAPI Specification**: Locate the specification before making changes
2. **✅ Understand the Contract**: Review the existing API contract thoroughly
3. **✅ Validate Changes**: Use validation tools on any specification changes
4. **✅ Follow the TDD Process**: Write contract tests before implementation
5. **✅ Check for Breaking Changes**: Validate backward compatibility
6. **✅ Generate Documentation**: Update API documentation with changes
7. **✅ Generate SDKs**: Update client libraries when needed
8. **✅ Follow Security Standards**: Maintain consistent security requirements
9. **✅ Use Consistent Patterns**: Follow established schema and naming patterns
10. **✅ Document Thoroughly**: Provide clear descriptions for all components

## 📚 Additional Resources

- [OpenAPI 3.1 Specification](https://spec.openapis.org/oas/v3.1.0)
- [Spectral Rules](https://meta.stoplight.io/docs/spectral/docs/reference/openapi-rules.md)
- [OpenAPI Generator](https://openapi-generator.tech/docs/generators/)
- [ReDoc Documentation](https://redoc.ly/docs/redoc/quickstart/)
- [OpenAPI Map](https://openapi-map.apihandyman.io/) - Visual guide to OpenAPI spec

---

By following this OpenAPI-first development guide, AI agents will ensure consistent, high-quality API design and implementation across all services in the LLM Platform AI Platform.