# OpenAPI-First Development Guidelines

This document outlines the OpenAPI-first development and validation practices for AI agents working in the LLM Platform AI Platform codebase.

## 🚦 Why OpenAPI-First?

The LLM Platform AI Platform uses OpenAPI 3.1 as the cornerstone of API development to ensure:

- **Standardized Interfaces**: Consistent API structure across services
- **Contract Validation**: Automated testing against API contracts
- **Documentation Generation**: Auto-generated, always up-to-date documentation
- **SDK Generation**: Automated client library generation for multiple languages
- **Interoperability**: Seamless integration between services

## 📋 Core OpenAPI Principles

### 1. API Contract Is the Source of Truth

- **API-First Development**: Design the API contract *before* implementing code
- **Contract Enforcement**: All API endpoints must adhere to their OpenAPI specification
- **Breaking Changes**: API contracts should be versioned to avoid breaking clients
- **Schema Validation**: Request and response payloads must be validated against their schemas

### 2. OpenAPI 3.1 Standards

- **Use OpenAPI 3.1**: All API specifications must use OpenAPI 3.1 format
- **Full Schema Definitions**: Complete schema definitions for all objects
- **Proper Typing**: Use appropriate types (string, number, boolean, etc.)
- **Required Properties**: Clearly mark which properties are required
- **Descriptive Documentation**: Include descriptions for all endpoints, parameters, and properties

## 🛠️ OpenAPI Workflow

### API Design Phase

1. **Create/Update OpenAPI Spec**: Define endpoints, parameters, request/response schemas
2. **Validate OpenAPI Document**: Use OpenAPI validation tools to ensure correctness
3. **Review API Design**: Have team members review the API contract

### Development Phase

1. **Generate Server Stubs**: Generate server code from OpenAPI specs
2. **Implement Business Logic**: Fill in the generated stubs with actual implementation
3. **Generate Client SDK**: Generate client libraries for service consumers

### Testing Phase

1. **Contract Testing**: Validate all requests/responses against the API spec
2. **Automated Testing**: Use generated clients for integration tests
3. **API Mocking**: Use API specs to generate mock services for testing

## 🔍 OpenAPI Validation Tools

The following tools are used throughout the platform for OpenAPI validation:

- **Spectral**: Linting and validation of OpenAPI documents
- **42Crunch**: Security analysis of API specifications
- **OpenAPI Validator**: Runtime validation of requests/responses
- **Swagger Editor**: Visual editing and validation of OpenAPI specs
- **Redoc/Swagger UI**: API documentation generation

Example of validation command:
```bash
# Validate OpenAPI specification
npx @stoplight/spectral-cli lint ./openapi.yaml

# Validate request/response against API spec at runtime
npm run openapi:validate
```

## 🚀 Service-Specific OpenAPI Implementations

### bfapi

- **File**: `/bfapi/openapi.yaml`
- **Validation**: CI pipeline includes openapi-validation-tests.js
- **Documentation**: Auto-generates Swagger UI at /api-docs.html
- **Client Generation**: Generates TypeScript client with `npm run generate-sdk`

### llm-mcp

- **File**: `/llm-mcp/openapi.yaml`
- **Integration**: Implements Model Context Protocol via OpenAPI
- **Tool Registry**: Exposes available tools in standardized format
- **Resource Management**: Documents resource access APIs

### bfllm

- **File**: `/bfllm/openapi.yaml`
- **MCP Integration**: `/bfllm/openapi-mcp-integration.yaml`
- **Model Registry**: Documents model registration and inference APIs
- **Training APIs**: Specifies distributed training interfaces

## 📊 OpenAPI Components

### Common Components to Reference

The platform uses shared components across services:

#### Common Schemas
- `Error`: Standard error response format
- `HealthStatus`: Health check response format
- `ServiceStatus`: Individual service status format

#### Common Parameters
- `modelId`: Standard parameter for model identification
- `versionId`: Standard parameter for version identification

#### Common Responses
- `BadRequest`: Invalid request parameters (400)
- `Unauthorized`: Authentication required (401)
- `Forbidden`: Insufficient permissions (403)
- `NotFound`: Resource not found (404)
- `InternalServerError`: Server error (500)

## 🔒 Security Definitions

All services implement consistent security schemes:

```yaml
securitySchemes:
  ApiKeyAuth:
    type: apiKey
    in: header
    name: X-API-Key
    description: API key for authentication
  BearerAuth:
    type: http
    scheme: bearer
    bearerFormat: JWT
    description: JWT token for authentication
```

## 🌟 Best Practices

1. **Versioning**: Include API version in the URL path (e.g., `/v1/resource`)
2. **Consistent Naming**: Use `camelCase` for properties, `snake_case` for query parameters
3. **Descriptive Operations**: Use clear `operationId` values for endpoints
4. **Response Codes**: Include all possible response codes with descriptions
5. **Examples**: Provide example request/response payloads
6. **Tags**: Group endpoints by functional area using tags
7. **Extensions**: Use vendor extensions for additional metadata
8. **Security**: Always define security requirements

## 📚 Examples

### Endpoint Definition Example

```yaml
/models/[modelId]:
  get:
    tags:
      - Models
    summary: Get model details
    operationId: getModelDetails
    parameters:
      - $ref: '#/components/parameters/modelId'
    responses:
      '200':
        description: Model details
        content:
          application/json:
            schema:
              $ref: '#/components/schemas/ModelDetail'
      '404':
        $ref: '#/components/responses/NotFound'
      '500':
        $ref: '#/components/responses/InternalServerError'
```

### Schema Definition Example

```yaml
ModelDetail:
  type: object
  properties:
    id:
      type: string
    name:
      type: string
    description:
      type: string
    type:
      type: string
    status:
      type: string
    metadata:
      type: object
    versions:
      type: array
      items:
        $ref: '#/components/schemas/ModelVersion'
```

## 🔄 Integration with TDD

The OpenAPI-first approach integrates with Test-Driven Development through:

1. **API Contract Tests**: Write tests against the OpenAPI specification first
2. **Response Validation**: Validate all responses against schemas
3. **Request Validation**: Validate all requests against schemas
4. **Coverage Validation**: Ensure all API endpoints have contract tests

Example TDD test for an API endpoint:

```typescript
describe('GET /models/[modelId]', () => {
  it('should return a valid model according to the schema', async () => {
    const response = await request(app).get('/models/test-model-id');
    
    // Verify response status
    expect(response.status).toBe(200);
    
    // Validate response against OpenAPI schema
    const validationResult = validateAgainstSchema(
      response.body, 
      '#/components/schemas/ModelDetail'
    );
    
    expect(validationResult.valid).toBe(true);
  });
});
```

## 📋 AI Agent OpenAPI Checklist

When working with APIs in this repository:

- [ ] **Find OpenAPI Spec**: Locate the OpenAPI specification for the service
- [ ] **Understand Schema**: Review the available endpoints and schemas
- [ ] **Validate Changes**: Run validation on any API contract changes
- [ ] **Update Tests**: Update contract tests when modifying APIs
- [ ] **Maintain Backward Compatibility**: Avoid breaking changes to existing clients
- [ ] **Follow Security Standards**: Implement proper authentication and authorization
- [ ] **Document Changes**: Update OpenAPI spec with detailed descriptions

## 🔍 Useful Commands

```bash
# Generate client from OpenAPI spec
npm run generate-sdk

# Validate OpenAPI spec
npm run openapi:validate

# Generate API documentation
npm run generate-api-docs

# Run contract tests
npm run test:contract

# View OpenAPI UI
npm run openapi:ui
```

---

By following these OpenAPI-first development guidelines, all services in the LLM Platform AI Platform maintain consistent, well-documented, and validated APIs that enable seamless integration and interoperability.