# OpenAPI Contract Validation in CI Pipeline

This guide provides detailed instructions for implementing robust OpenAPI contract validation in your CI pipeline for the LLM Platform. Following the OpenAPI-first development approach, all services must validate their API contracts as part of the CI process.

## 🎯 Benefits of CI Contract Validation

- **Enforces API Contract Compliance**: Ensures all services adhere to their defined API contracts
- **Prevents Breaking Changes**: Catches breaking API changes before they reach production
- **Maintains Documentation Accuracy**: Keeps OpenAPI specs and documentation in sync with implementation
- **Enables Cross-Service Compatibility**: Verifies services can communicate as expected
- **Automates Quality Control**: Reduces manual review burden for API changes

## 🔧 Implementation Steps

### 1. OpenAPI Spec Organization

```
project_directory/
├── openapi/
│   ├── main.yaml           # Main OpenAPI document (can import others)
│   ├── components/         # Reusable components (schemas, parameters, etc.)
│   │   ├── schemas/        # Data models
│   │   ├── parameters/     # API parameters
│   │   ├── responses/      # Response definitions
│   │   └── security/       # Security schemes
│   ├── paths/              # API endpoints grouped by resource
│   │   ├── users.yaml      # User-related endpoints
│   │   ├── models.yaml     # Model-related endpoints
│   │   └── agents.yaml     # Agent-related endpoints
│   └── versions/           # Version-specific specifications
│       ├── v1/             # v1 API specs
│       └── v2/             # v2 API specs
```

### 2. Setting Up Validation Tools

#### Core Tools to Include in CI

1. **Spectral**: OpenAPI linter for style and best practices
2. **OpenAPI-Validator**: Structural validation against the OpenAPI 3.1 spec
3. **Prism**: API mocking and contract testing
4. **Dredd**: API implementation testing against OpenAPI specs
5. **OpenAPI-Diff**: Breaking change detection between versions

#### Installation in CI Environment

```yaml
# In your GitLab CI or GitHub Actions workflow
setup_validation_tools:
  script:
    - npm install -g @stoplight/spectral-cli
    - npm install -g openapi-validator
    - npm install -g @stoplight/prism-cli
    - npm install -g dredd
    - npm install -g openapi-diff
```

### 3. CI Pipeline Configuration

#### GitLab CI Example

```yaml
# .gitlab-ci.yml
stages:
  - validate
  - test
  - build
  - deploy

# OpenAPI Validation Stage
openapi_validation:
  stage: validate
  script:
    # Lint the OpenAPI spec for style and best practices
    - spectral lint openapi/main.yaml --ruleset .spectral.yaml
    
    # Validate structural correctness against OpenAPI 3.1 spec
    - openapi-validator --strict openapi/main.yaml
    
    # Check for breaking changes compared to production
    - openapi-diff production-openapi.yaml openapi/main.yaml --fail-on-breaking-changes
    
    # Run mock server for contract testing
    - prism mock openapi/main.yaml &
    - sleep 5 # Wait for server to start
    
    # Run contract tests against the spec
    - dredd openapi/main.yaml http://localhost:4010 --hookfiles=./dredd-hooks.js
```

#### GitHub Actions Example

```yaml
# .github/workflows/openapi-validation.yml
name: OpenAPI Validation

on:
  push:
    branches: [ main, develop ]
    paths:
      - 'openapi/**'
  pull_request:
    branches: [ main, develop ]
    paths:
      - 'openapi/**'

jobs:
  validate:
    runs-on: ubuntu-latest
    steps:
      - uses: actions/checkout@v3
      - uses: actions/setup-node@v3
        with:
          node-version: '18'
      
      - name: Install OpenAPI tools
        run: |
          npm install -g @stoplight/spectral-cli
          npm install -g openapi-validator
          npm install -g @stoplight/prism-cli
          npm install -g dredd
          npm install -g openapi-diff
      
      - name: Lint OpenAPI Specification
        run: spectral lint openapi/main.yaml --ruleset .spectral.yaml
      
      - name: Validate OpenAPI Structure
        run: openapi-validator --strict openapi/main.yaml
      
      - name: Check for Breaking Changes
        run: |
          git fetch origin main
          git show origin/main:openapi/main.yaml > previous.yaml
          openapi-diff previous.yaml openapi/main.yaml --fail-on-breaking-changes
      
      - name: Contract Testing
        run: |
          prism mock openapi/main.yaml &
          sleep 5
          dredd openapi/main.yaml http://localhost:4010 --hookfiles=./dredd-hooks.js
```

### 4. Custom Validation Rules

Create a `.spectral.yaml` ruleset file in your project root:

```yaml
# .spectral.yaml
extends: ["spectral:oas", "spectral:asyncapi"]
rules:
  # Enforce semantic versioning in API paths
  path-versioning:
    description: "API paths should include version (e.g., /v1/resources)"
    given: "$.paths[*]~"
    severity: error
    then:
      function: pattern
      functionOptions:
        match: "^/v[0-9]+/"
  
  # Require descriptions for all schemas
  schema-description-required:
    description: "All schemas must have descriptions"
    given: "$.components.schemas[*]"
    severity: error
    then:
      field: description
      function: truthy
  
  # Enforce consistent response structure
  response-structure:
    description: "Responses should follow platform standard structure"
    given: "$.paths[*][*].responses[*].content[*].schema"
    severity: error
    then:
      function: schema
      functionOptions:
        schema:
          type: object
          required: ["data", "meta"]
          properties:
            data: {}
            meta:
              type: object
              properties:
                requestId:
                  type: string
```

### 5. Breaking Change Detection

Configure OpenAPI-Diff to detect breaking changes:

```json
// openapi-diff-config.json
{
  "breakingChangesRules": {
    "added": {
      "required": true
    },
    "removed": {
      "deprecated": false,
      "description": false,
      "enum": true,
      "default": false,
      "nullable": true,
      "pattern": true,
      "properties": true,
      "summary": false,
      "type": true
    },
    "modified": {
      "maxToRequired": true,
      "minToRequired": true,
      "patternStringToPatternRegExp": false,
      "typeToEnum": false
    }
  },
  "pathsToIgnore": [
    "/deprecated/**"
  ]
}
```

### 6. Pre-Commit Hooks (Local Development)

Set up pre-commit hooks to catch issues before pushing:

```bash
# .git/hooks/pre-commit
#!/bin/sh

# Check if OpenAPI specs have changed
OPENAPI_FILES=$(git diff --cached --name-only --diff-filter=ACMR | grep -E '\.(yaml|json)$' | grep -E 'openapi/')

if [ -n "$OPENAPI_FILES" ]; then
  echo "OpenAPI files changed, running validation..."
  
  # Run Spectral linting
  npx spectral lint openapi/main.yaml --ruleset .spectral.yaml
  
  # Exit with error if linting fails
  if [ $? -ne 0 ]; then
    echo "OpenAPI linting failed. Please fix the issues before committing."
    exit 1
  fi
  
  # Run structure validation
  npx openapi-validator --strict openapi/main.yaml
  
  if [ $? -ne 0 ]; then
    echo "OpenAPI validation failed. Please fix the issues before committing."
    exit 1
  fi
fi

exit 0
```

### 7. Automated Schema Synchronization

Set up a job to auto-generate TypeScript types from OpenAPI schemas:

```yaml
# In GitLab CI or GitHub Actions
generate_types:
  stage: build
  script:
    - npx openapi-typescript openapi/main.yaml -o src/types/api.ts
    - git diff --exit-code || (git add src/types/api.ts && git commit -m "Update API types from OpenAPI spec" --no-verify)
```

## 🧪 Contract Testing Approach

### 1. Setting Up Dredd Hooks

Create a `dredd-hooks.js` file to handle dynamic values and authentication:

```javascript
const hooks = require('hooks');
const jwt = require('jsonwebtoken');

// Generate JWT token for authenticated endpoints
hooks.beforeEach((transaction) => {
  if (transaction.request.headers.Authorization) {
    const token = jwt.sign(
      { sub: 'test-user', role: 'admin' },
      'test-secret',
      { expiresIn: '1h' }
    );
    transaction.request.headers.Authorization = `Bearer auth-token`;
  }
});

// Handle dynamic values in responses
hooks.afterEach((transaction) => {
  // Ignore dynamic fields like timestamps, IDs, etc.
  if (transaction.expected.body) {
    const expected = JSON.parse(transaction.expected.body);
    const actual = JSON.parse(transaction.real.body);
    
    // Replace dynamic values with patterns for comparison
    if (expected.data && expected.data.id && actual.data && actual.data.id) {
      expected.data.id = '<dynamic>';
      actual.data.id = '<dynamic>';
    }
    
    if (expected.meta && expected.meta.timestamp && actual.meta && actual.meta.timestamp) {
      expected.meta.timestamp = '<dynamic>';
      actual.meta.timestamp = '<dynamic>';
    }
    
    transaction.expected.body = JSON.stringify(expected);
    transaction.real.body = JSON.stringify(actual);
  }
});
```

### 2. Mock Response Examples

Ensure your OpenAPI spec includes realistic examples:

```yaml
# In your OpenAPI paths
/models/[modelId]:
  get:
    responses:
      '200':
        description: Model details
        content:
          application/json:
            schema:
              $ref: '#/components/schemas/ModelResponse'
            examples:
              model7B:
                value:
                  data:
                    id: "model-7b-1"
                    name: "LLaMA-7B"
                    parameters: 7000000000
                    contextSize: 4096
                    supportsFunctions: true
                  meta:
                    requestId: "req-123456"
                    timestamp: "2025-05-31T12:34:56Z"
```

## 📊 CI Results & Reporting

### 1. Generating HTML Reports

```yaml
# In GitLab CI
openapi_documentation:
  stage: build
  script:
    # Generate HTML documentation
    - npx redoc-cli bundle openapi/main.yaml -o public/api-docs.html
    - npx widdershins openapi/main.yaml -o docs/api.md
    
    # Generate validation report
    - npx spectral lint openapi/main.yaml --format html > public/validation-report.html
    
  artifacts:
    paths:
      - public/api-docs.html
      - public/validation-report.html
      - docs/api.md
```

### 2. Creating Validation Badges

Generate status badges based on validation results:

```javascript
// scripts/generate-badges.js
const fs = require('fs');
const path = require('path');
const [execSync] = require('child_process');

// Run validation
try {
  execSync('npx spectral lint openapi/main.yaml');
  fs.writeFileSync(
    path.join(__dirname, '../public/openapi-badge.svg'),
    '<svg>...OpenAPI Valid...</svg>'
  );
} catch (e) {
  fs.writeFileSync(
    path.join(__dirname, '../public/openapi-badge.svg'),
    '<svg>...OpenAPI Invalid...</svg>'
  );
}
```

## 🔐 Security Validation

### 1. Security Scanning Setup

```yaml
# In GitLab CI or GitHub Actions
security_scan:
  stage: validate
  script:
    # Scan OpenAPI spec for security issues
    - npx openapi-security-scanner openapi/main.yaml
    
    # Check for exposed secrets
    - npx detect-secrets-hook openapi/
    
    # Validate authentication schemes
    - npx spectral lint openapi/main.yaml --ruleset security-ruleset.yaml
```

### 2. Custom Security Rules

```yaml
# security-ruleset.yaml
extends: ["spectral:oas", "spectral:asyncapi"]
rules:
  # Require authentication for all non-public endpoints
  endpoints-require-auth:
    description: "All non-public endpoints must require authentication"
    given: "$.paths[*][*]"
    severity: error
    then:
      function: schema
      functionOptions:
        schema:
          type: object
          properties:
            security:
              type: array
              minItems: 1
          required: ["security"]
    # Exclude public endpoints
    exclude:
      - "$.paths['/health'][*]"
      - "$.paths['/version'][*]"
      - "$.paths['/docs'][*]"
```

## 🔄 Version Compatibility Testing

### 1. Testing Against Multiple Versions

```yaml
# In GitLab CI
version_compatibility:
  stage: test
  script:
    # Test current spec against v1 implementation
    - dredd openapi/versions/v1/openapi.yaml http://v1-api.example.com
    
    # Test current spec against v2 implementation
    - dredd openapi/versions/v2/openapi.yaml http://v2-api.example.com
    
    # Check backward compatibility
    - openapi-diff openapi/versions/v1/openapi.yaml openapi/main.yaml --fail-on-breaking-changes
```

### 2. Semantic Versioning Enforcement

Create a script to enforce semantic versioning based on changes:

```javascript
// scripts/enforce-semver.js
const openapiDiff = require('openapi-diff');
const semver = require('semver');
const fs = require('fs');

// Read current version
const packageJson = JSON.parse(fs.readFileSync('./package.json', 'utf8'));
const currentVersion = packageJson.version;

// Compare current spec with previous version
openapiDiff.diffSpecs({
  sourceSpec: fs.readFileSync('./previous-openapi.yaml', 'utf8'),
  destinationSpec: fs.readFileSync('./openapi/main.yaml', 'utf8'),
}).then(diff => {
  let nextVersion;
  
  if (diff.breakingDifferences.length > 0) {
    // Major version bump for breaking changes
    nextVersion = semver.inc(currentVersion, 'major');
  } else if (diff.nonBreakingDifferences.length > 0) {
    // Minor version bump for new features
    nextVersion = semver.inc(currentVersion, 'minor');
  } else {
    // Patch version bump for fixes
    nextVersion = semver.inc(currentVersion, 'patch');
  }
  
  console.log(`Suggested next version: [nextVersion]`);
  
  // Update version in package.json
  packageJson.version = nextVersion;
  fs.writeFileSync('./package.json', JSON.stringify(packageJson, null, 2));
});
```

## 📝 Best Practices & Guidelines

### 1. OpenAPI Spec Versioning

- Store OpenAPI specs in version control
- Tag releases with API versions
- Keep a history of all published specs
- Include version information in the OpenAPI document itself

### 2. CI Integration Points

- Run validation on every commit and PR
- Block merges if validation fails
- Generate documentation from validated specs
- Publish documentation as part of CI pipeline
- Archive validated specs as artifacts

### 3. Validation Severity Levels

Configure appropriate severity levels for different validation rules:

```yaml
# In .spectral.yaml
rules:
  # Critical rules that should block the pipeline
  path-versioning:
    severity: error
  
  # Important rules that should be fixed but may not block
  schema-description-required:
    severity: warning
  
  # Suggestions for improvement
  response-example-required:
    severity: info
```

### 4. Handling Legacy APIs

- Use feature flags to maintain multiple API versions
- Set up separate validation pipelines for legacy versions
- Apply different rule sets based on API version
- Document deprecation timelines in OpenAPI spec

## 🚀 Example Implementation

### Project Structure

```
project_directory/
├── .github/
│   └── workflows/
│       └── openapi-validation.yml  # GitHub Actions workflow
├── .gitlab/
│   └── ci/
│       └── openapi-validation.yml  # GitLab CI job definitions
├── openapi/
│   ├── main.yaml                   # Main OpenAPI document
│   ├── components/                 # Reusable components
│   └── paths/                      # API endpoints
├── scripts/
│   ├── validate-openapi.js         # Validation script
│   ├── generate-types.js           # Type generation script
│   └── enforce-semver.js           # Version enforcement
├── .spectral.yaml                  # Spectral ruleset
├── dredd-hooks.js                  # Dredd hooks for testing
├── openapi-diff-config.json        # Diff configuration
└── security-ruleset.yaml           # Security validation rules
```

### Complete CI Pipeline

```yaml
# Full CI pipeline example (GitLab CI)
stages:
  - validate
  - test
  - build
  - deploy

# Validate OpenAPI specs
openapi_validation:
  stage: validate
  script:
    # Lint for style and best practices
    - spectral lint openapi/main.yaml --ruleset .spectral.yaml
    # Validate structure
    - openapi-validator --strict openapi/main.yaml
    # Security scanning
    - openapi-security-scanner openapi/main.yaml
  artifacts:
    paths:
      - validation-results.json

# Test API implementation against specs
contract_testing:
  stage: test
  script:
    # Run mock server
    - prism mock openapi/main.yaml &
    - sleep 5
    # Run contract tests
    - dredd openapi/main.yaml http://localhost:4010 --hookfiles=./dredd-hooks.js
  dependencies:
    - openapi_validation

# Check for breaking changes
breaking_changes:
  stage: test
  script:
    # Get previous version
    - git fetch origin main
    - git show origin/main:openapi/main.yaml > previous.yaml
    # Check for breaking changes
    - openapi-diff previous.yaml openapi/main.yaml --config openapi-diff-config.json
    # Enforce semantic versioning
    - node scripts/enforce-semver.js
  dependencies:
    - openapi_validation
  only:
    - merge_requests

# Generate documentation and types
generate_artifacts:
  stage: build
  script:
    # Generate HTML documentation
    - redoc-cli bundle openapi/main.yaml -o public/api-docs.html
    # Generate Markdown documentation
    - widdershins openapi/main.yaml -o docs/api.md
    # Generate TypeScript types
    - openapi-typescript openapi/main.yaml -o src/types/api.ts
  artifacts:
    paths:
      - public/api-docs.html
      - docs/api.md
      - src/types/api.ts
  dependencies:
    - openapi_validation

# Deploy documentation
deploy_docs:
  stage: deploy
  script:
    - deploy-script public/api-docs.html
  dependencies:
    - generate_artifacts
  only:
    - main
```

## 🧩 Advanced Integration

### 1. Multi-Repository Coordination

For microservices architectures with multiple repositories:

```javascript
// scripts/fetch-dependent-specs.js
const axios = require('axios');
const fs = require('fs');
const path = require('path');

// Fetch OpenAPI specs from other services
async function fetchSpecs() {
  const services = [
    { name: 'users-service', url: 'https://raw.githubusercontent.com/org/users-service/main/openapi/main.yaml' },
    { name: 'models-service', url: 'https://raw.githubusercontent.com/org/models-service/main/openapi/main.yaml' },
    { name: 'agents-service', url: 'https://raw.githubusercontent.com/org/agents-service/main/openapi/main.yaml' },
  ];
  
  for (const service of services) {
    const response = await axios.get(service.url);
    fs.writeFileSync(
      path.join(__dirname, `../openapi/external/[service.name].yaml`),
      response.data
    );
  }
}

fetchSpecs();
```

### 2. API Gateway Integration Testing

Test combined API specs through an API gateway:

```yaml
# In GitLab CI
gateway_testing:
  stage: test
  script:
    # Combine all service specs
    - node scripts/combine-specs.js
    
    # Run contract tests against gateway
    - dredd openapi/gateway.yaml https://api-gateway.example.com --hookfiles=./gateway-hooks.js
```

### 3. Consumer-Driven Contract Testing

Implement consumer-driven contract testing:

```javascript
// scripts/consumer-contracts.js
const fs = require('fs');
const yaml = require('js-yaml');
const path = require('path');

// Load consumer contracts
const consumerContracts = fs.readdirSync(path.join(__dirname, '../contracts'))
  .filter(file => file.endsWith('.yaml'))
  .map(file => {
    const content = fs.readFileSync(path.join(__dirname, '../contracts', file), 'utf8');
    return yaml.load(content);
  });

// Load provider OpenAPI spec
const providerSpec = yaml.load(
  fs.readFileSync(path.join(__dirname, '../openapi/main.yaml'), 'utf8')
);

// Validate that provider meets all consumer requirements
for (const contract of consumerContracts) {
  // Check each path and operation in the contract
  for (const [path, operations] of Object.entries(contract.paths)) {
    if (!providerSpec.paths[path]) {
      console.error(`Path /path/to/resource required by consumer not found in provider spec`);
      process.exit(1);
    }
    
    for (const [method, operation] of Object.entries(operations)) {
      if (!providerSpec.paths[path][method]) {
        console.error(`Operation [method.toUpperCase()] /path/to/resource required by consumer not found in provider spec`);
        process.exit(1);
      }
      
      // Check response schemas
      for (const [status, response] of Object.entries(operation.responses)) {
        if (!providerSpec.paths[path][method].responses[status]) {
          console.error(`Response Status for [method.toUpperCase()] /path/to/resource required by consumer not found in provider spec`);
          process.exit(1);
        }
      }
    }
  }
}

console.log('All consumer contracts are satisfied by the provider');
```

## 📚 Resources and Tools

- [Spectral](https://stoplight.io/open-source/spectral) - OpenAPI linter
- [OpenAPI-Validator](https://github.com/IBM/openapi-validator) - Structural validation
- [Prism](https://stoplight.io/open-source/prism) - API mocking
- [Dredd](https://github.com/apiaryio/dredd) - API implementation testing
- [OpenAPI-Diff](https://github.com/OpenAPITools/openapi-diff) - Breaking change detection
- [Redoc](https://github.com/Redocly/redoc) - Documentation generation
- [openapi-typescript](https://github.com/drwpow/openapi-typescript) - TypeScript type generation

## 🏆 Success Metrics

### 1. Validation Pass Rate

- Track percentage of validation runs that pass without errors
- Monitor trends over time to ensure continuous improvement
- Set targets for validation success (e.g., 100% for critical rules)

### 2. Breaking Change Detection

- Monitor frequency of breaking changes
- Track adherence to semantic versioning
- Measure time to resolve breaking changes

### 3. Documentation Quality

- Measure completeness of API documentation
- Track usage of examples and descriptions
- Monitor documentation freshness

### 4. Contract Test Coverage

- Measure percentage of endpoints covered by contract tests
- Track contract test success rate
- Monitor integration with dependent services

---

**Version**: 1.0.0  
**Last Updated**: May 31, 2025  
**Author**: Platform Engineering Team