# Connection Pooling in LLM-MCP

This document describes the connection pooling system implemented in the LLM-MCP server. The connection pooling system optimizes performance for high-throughput operations and provides resilience against transient connectivity issues.

## Overview

The connection pooling system includes:

1. **Managed Connections**: Wraps individual HTTP connections with lifecycle management, health monitoring, and metrics.

2. **Connection Pool**: Manages a set of connections to a specific service, including connection creation, acquisition, and recycling.

3. **Connection Pool Manager**: Coordinates multiple connection pools for different services.

4. **Pooled HTTP Client**: Client interface that uses the connection pool for making HTTP requests.

5. **Pooled HTTP Client Factory**: Factory for creating and retrieving clients for different services.

## Architecture

```
┌─────────────────────────────────────────────────────────┐
│                   Application Code                       │
└───────────────────────────┬─────────────────────────────┘
                            │
                            ▼
┌─────────────────────────────────────────────────────────┐
│              PooledHttpClientFactory                     │
│                                                         │
│  ┌─────────────┐   ┌─────────────┐   ┌─────────────┐    │
│  │  llm-gateway-sdk │   │  bfllmClient │   │ drupalClient│    │
│  └──────┬──────┘   └──────┬──────┘   └──────┬──────┘    │
└─────────┼───────────────┬─┼───────────────┬─┼───────────┘
          │               │ │               │ │
          ▼               │ ▼               │ ▼
┌─────────────────┐       │ ┌─────────────┐ │ ┌─────────────┐
│ PooledHttpClient│       │ │ PooledHttp- │ │ │ PooledHttp- │
│    (bfapi)      │       │ │Client(bfllm)│ │ │Client(drupal│
└────────┬────────┘       │ └──────┬─────┘ │ └──────┬──────┘
         │                │        │       │        │
         ▼                │        ▼       │        ▼
┌────────────────┐        │ ┌────────────┐ │ ┌─────────────┐
│ ConnectionPool │        │ │Connection- │ │ │Connection-  │
│    (bfapi)     │        │ │Pool (bfllm)│ │ │Pool (drupal)│
└───────┬────────┘        │ └─────┬─────┘ │ └──────┬──────┘
        │                 │       │       │        │
        ▼                 │       ▼       │        ▼
┌───────────────┐         │┌───────────┐  │ ┌─────────────┐
│ ManagedConn 1 │         ││ManagedConn│  │ │ ManagedConn │
└───────────────┘         ││    1      │  │ │     1       │
┌───────────────┐         │└───────────┘  │ └─────────────┘
│ ManagedConn 2 │         │┌───────────┐  │ ┌─────────────┐
└───────────────┘         ││ManagedConn│  │ │ ManagedConn │
┌───────────────┐         ││    2      │  │ │     2       │
│ ManagedConn 3 │         │└───────────┘  │ └─────────────┘
└───────────────┘         └──────────────┘ └──────────────┘
```

## Configuration

Connection pooling can be configured using environment variables or the configuration object:

| Environment Variable | Configuration Key | Description | Default |
|---------------------|-------------------|-------------|---------|
| `CONNECTION_POOL_ENABLED` | `performance.connectionPool.enabled` | Enable/disable connection pooling | `true` |
| `POOL_MAX_CONNECTIONS` | `performance.connectionPool.maxConnections` | Maximum connections per pool | `10` |
| `POOL_MIN_CONNECTIONS` | `performance.connectionPool.minConnections` | Minimum connections per pool | `2` |
| `POOL_IDLE_TIMEOUT` | `performance.connectionPool.idleTimeout` | Idle timeout in milliseconds | `60000` (1 minute) |
| `POOL_CONNECTION_TTL` | `performance.connectionPool.connectionTTL` | Connection time-to-live in milliseconds | `600000` (10 minutes) |
| `POOL_VALIDATE_ON_BORROW` | `performance.connectionPool.validateOnBorrow` | Validate connections before use | `true` |
| `POOL_HEALTH_CHECK_INTERVAL` | `performance.connectionPool.healthCheckInterval` | Health check interval in milliseconds | `30000` (30 seconds) |
| `POOL_ACQUIRE_TIMEOUT` | `performance.connectionPool.acquireTimeout` | Timeout for acquiring connections | `30000` (30 seconds) |
| `POOL_MAX_RETRIES` | `performance.connectionPool.retryOptions.maxRetries` | Maximum retry attempts | `3` |
| `POOL_RETRY_DELAY` | `performance.connectionPool.retryOptions.retryDelay` | Base delay between retries in milliseconds | `300` |
| `POOL_EXPONENTIAL_BACKOFF` | `performance.connectionPool.retryOptions.exponentialBackoff` | Use exponential backoff for retries | `true` |

## Usage

### Basic Usage

```typescript
// Get a client for a specific service
import [PooledHttpClientFactory] from '../utils/pooled-http-client.js';

// Get a client for a specific service
const llm-gateway-sdk = PooledHttpClientFactory.getClient('bfapi');

// Make a request
const response = await llm-gateway-sdk.get('/health');
```

### Custom Client Creation

```typescript
// Create a custom client
const customClient = PooledHttpClientFactory.createClient({
  serviceName: 'custom-service',
  baseURL: 'http://custom-service.example.com',
  timeout: 60000,
  pool: {
    minConnections: 3,
    maxConnections: 10,
    idleTimeout: 120000
  }
});

// Make a request
const response = await customClient.post('/data', { key: 'value' });
```

### Using Retry Policies

The connection pooling system includes sophisticated retry policies with exponential backoff:

```typescript
// Create a client with custom retry options
const clientWithRetries = PooledHttpClientFactory.createClient({
  serviceName: 'resilient-service',
  baseURL: 'http://api.example.com',
  retries: 5, // Maximum retry attempts
  retryDelay: 500, // Base delay between retries (ms)
  pool: {
    minConnections: 2,
    maxConnections: 8
  }
});

// Requests will automatically retry with exponential backoff
try {
  const response = await clientWithRetries.get('/flaky-endpoint');
} catch (error) [// All retry attempts failed]
```

## Monitoring and Management

The connection pooling system provides comprehensive monitoring and management capabilities:

### Health Check Endpoints

- **GET /api/v1/connection-pools/health**: Get overall health status of all connection pools
- **GET /api/v1/connection-pools/stats**: Get detailed statistics for all connection pools
- **GET /api/v1/connection-pools/service/:serviceName**: Get metrics for a specific service
- **POST /api/v1/connection-pools/reset**: Reset all connection pools

### Metrics

The connection pooling system collects detailed metrics for each connection:

- **Request count**: Total number of requests made through the connection
- **Error count**: Number of errors encountered
- **Consecutive errors**: Number of consecutive errors (used for health detection)
- **Average latency**: Average response time in milliseconds
- **Connection state**: Current state of the connection (idle, active, unhealthy, closed)
- **Creation time**: When the connection was created
- **Last used time**: When the connection was last used

## Key Features

### Connection Health Monitoring

- **Automatic health checks**: Regularly checks connection health
- **Consecutive error detection**: Marks connections as unhealthy after consecutive errors
- **Auto-healing**: Automatically recycles unhealthy connections
- **Validation before use**: Optionally validates connections before use

### Connection Lifecycle Management

- **Minimum connections**: Maintains a minimum number of connections
- **Maximum connections**: Limits the maximum number of connections
- **Idle timeout**: Closes idle connections after a timeout
- **Connection TTL**: Limits the lifetime of connections
- **Graceful shutdown**: Properly closes connections on shutdown

### Load Balancing

- **Round-robin selection**: Distributes requests across available connections
- **Health-aware routing**: Only routes requests to healthy connections
- **Dynamic scaling**: Creates new connections as needed, up to the maximum

### Retry Policies

- **Configurable retries**: Set maximum retry attempts
- **Exponential backoff**: Increases delay between retries
- **Intelligent retry logic**: Only retries retryable errors
- **Status code filtering**: Configurable status codes to retry

## Performance Benefits

The connection pooling system provides significant performance benefits:

- **Reduced connection overhead**: Reuses connections instead of creating new ones
- **Improved throughput**: Maintains persistent connections for higher throughput
- **Reduced latency**: Eliminates connection establishment time
- **Better resource utilization**: Efficiently manages connections
- **Enhanced resilience**: Automatically handles transient connectivity issues

## Implementation Details

### Managed Connection

Each managed connection:
- Wraps an Axios HTTP client
- Tracks metrics (requests, errors, latency)
- Monitors its own health
- Implements lifecycle events

### Connection Pool

Each connection pool:
- Manages a set of connections to a specific service
- Handles connection acquisition, creation, and recycling
- Implements connection selection logic
- Provides retry capabilities
- Collects aggregate metrics

### Connection Pool Manager

The connection pool manager:
- Coordinates multiple pools
- Provides a singleton interface
- Handles global settings
- Collects system-wide metrics

## Best Practices

1. **Use predefined clients** when possible
2. **Configure pool sizes** based on expected load
3. **Monitor pool health** using the health check endpoints
4. **Adjust retry settings** based on service reliability
5. **Properly shut down** pools when no longer needed