# Examples

This page provides practical examples of using the LLM PlatformAI Coding Assistant in various scenarios.

## Basic Examples

### Code Search

This example demonstrates how to search for code related to authentication:

```typescript
import [CodingAssistant] from '@bluefly/native_aijs/core/neurosymbolic/CodingAssistant';

// Initialize CodingAssistant (initialization code omitted for brevity)

// Define context
const context = {
  currentFile: {
    path: '/path/to/file.ts',
    content: '/* file content */',
    language: 'typescript'
  },
  language: 'typescript',
  framework: 'express'
};

// Define search query
const query = {
  text: "How is user authentication implemented?",
  maxResults: 5,
  includeComments: true
};

// Execute search
async function searchAuthenticationCode() {
  try {
    const results = await codingAssistant.searchCode(query, context);
    
    console.log(`Found [results.length] results:`);
    results.forEach((result, index) => {
      console.log(`\nResult [index + 1]:`);
      console.log(`File: [result.path]`);
      console.log(`Relevance: [result.relevance]`);
      console.log(`Snippet:\n[result.snippet]`);
      console.log(`Related concepts: [result.concepts.join(', ')]`);
    });
    
    return results;
  } catch (error) {
    console.error(`Error searching code: [error.message]`);
    return [];
  }
}

searchAuthenticationCode();
```

### Code Generation

This example shows how to generate a function to validate email addresses:

```typescript
import [AICodingAssistant] from '@bluefly/llm-mcp/services/ai-coding-assistant';

// Initialize AICodingAssistant (initialization code omitted for brevity)

// Define context
const context = {
  currentFile: {
    path: '/path/to/validators.ts',
    content: `/**
 * Validators for common form inputs
 */

/**
 * Validates that a string is a non-empty value
 */
export function validateRequired(value: string): boolean {
  return value !== undefined && value !== null && value.trim() !== '';
}

// TODO: Add more validators
`,
    language: 'typescript'
  },
  language: 'typescript'
};

// Define task
const task = {
  description: "Add an email validator function",
  type: "generate",
  parameters: {
    functionName: "validateEmail",
    returnType: "boolean",
    testCases: true
  }
};

// Execute code generation
async function generateEmailValidator() {
  try {
    const result = await codingAssistant.processCodingTask(task, context);
    
    console.log("Generated code:");
    console.log(result.code);
    
    console.log("\nTest cases:");
    console.log(result.testCases);
    
    return result;
  } catch (error) {
    console.error(`Error generating code: [error.message]`);
    return null;
  }
}

generateEmailValidator();
```

### Code Editing

This example demonstrates how to refactor a function to use modern JavaScript features:

```typescript
import [CodingAssistant] from '@bluefly/native_aijs/core/neurosymbolic/CodingAssistant';

// Initialize CodingAssistant (initialization code omitted for brevity)

// Define context with a function that needs refactoring
const context = {
  currentFile: {
    path: '/path/to/utils.js',
    content: `
function fetchUserData(userId, callback) {
  var xhr = new XMLHttpRequest();
  xhr.open('GET', '/api/users/' + userId, true);
  xhr.onreadystatechange = function() {
    if (xhr.readyState === 4) {
      if (xhr.status === 200) {
        var data = JSON.parse(xhr.responseText);
        callback(null, data);
      } else {
        callback(new Error('Failed to fetch user data: ' + xhr.status));
      }
    }
  };
  xhr.send();
}
`,
    language: 'javascript'
  },
  language: 'javascript'
};

// Define edit operations
const operations = [
  {
    type: 'replace',
    position: {
      start: 0,
      end: context.currentFile.content.length
    },
    text: '', // Will be determined by the assistant
    preserveSemantics: true
  }
];

// Execute code refactoring
async function refactorFetchFunction() {
  try {
    // First, analyze the code to understand it
    const analysis = await codingAssistant.analyzeCode(context.currentFile.content);
    
    // Process a natural language request to modernize the code
    const request = "Refactor this function to use fetch API, async/await, and ES6+ features";
    const refactorResult = await codingAssistant.processRequest(request, context);
    
    console.log("Original code:");
    console.log(context.currentFile.content);
    
    console.log("\nRefactored code:");
    console.log(refactorResult.code);
    
    console.log("\nExplanation of changes:");
    console.log(refactorResult.explanation);
    
    return refactorResult;
  } catch (error) {
    console.error(`Error refactoring code: [error.message]`);
    return null;
  }
}

refactorFetchFunction();
```

### Code Visualization

This example shows how to generate a visualization of code dependencies:

```typescript
import [AICodingAssistant] from '@bluefly/llm-mcp/services/ai-coding-assistant';
import * as fs from 'fs';

// Initialize AICodingAssistant (initialization code omitted for brevity)

// Define context
const context = {
  currentFile: {
    path: '/path/to/app.js',
    content: fs.readFileSync('/path/to/app.js', 'utf8'),
    language: 'javascript'
  },
  language: 'javascript',
  projectStructure: {
    rootDir: '/path/to/project',
    srcDir: '/path/to/project/src'
  }
};

// Define visualization config
const config = {
  type: 'dependency',
  format: 'svg',
  includePatterns: true,
  includeRelationships: true
};

// Generate visualization
async function visualizeCodeDependencies() {
  try {
    const visualization = await codingAssistant.handleArchitectureVisualization({
      scope: 'module',
      viewType: 'dependency',
      outputFormat: 'svg'
    }, context);
    
    console.log("Visualization generated:");
    console.log(`Output URL: [visualization.outputUrl]`);
    console.log(`Entities: [visualization.entities.length]`);
    console.log(`Relationships: [visualization.relationships.length]`);
    
    // Save visualization to file
    fs.writeFileSync('dependency-graph.svg', visualization.data);
    
    return visualization;
  } catch (error) {
    console.error(`Error generating visualization: [error.message]`);
    return null;
  }
}

visualizeCodeDependencies();
```

## Advanced Examples

### Multi-file Refactoring

This example demonstrates refactoring a pattern across multiple files:

```typescript
import [AICodingAssistant] from '@bluefly/llm-mcp/services/ai-coding-assistant';
import * as fs from 'fs';
import * as path from 'path';

// Initialize AICodingAssistant (initialization code omitted for brevity)

// Define context with multiple files
const context = {
  currentFile: {
    path: '/path/to/project/src/services/userService.js',
    content: fs.readFileSync('/path/to/project/src/services/userService.js', 'utf8'),
    language: 'javascript'
  },
  language: 'javascript',
  openFiles: {
    '/path/to/project/src/services/userService.js': fs.readFileSync('/path/to/project/src/services/userService.js', 'utf8'),
    '/path/to/project/src/services/productService.js': fs.readFileSync('/path/to/project/src/services/productService.js', 'utf8'),
    '/path/to/project/src/services/orderService.js': fs.readFileSync('/path/to/project/src/services/orderService.js', 'utf8')
  }
};

// Define task
const task = {
  description: "Refactor all service files to use dependency injection pattern",
  type: "refactor",
  parameters: {
    pattern: "dependency-injection",
    files: Object.keys(context.openFiles),
    preserveFunctionality: true
  }
};

// Execute multi-file refactoring
async function refactorServiceFiles() {
  try {
    const result = await codingAssistant.processCodingTask(task, context);
    
    console.log("Refactoring summary:");
    console.log(`Files modified: [result.modifiedFiles.length]`);
    console.log(`Changes made: [result.totalChanges]`);
    
    // Log detailed changes for each file
    result.fileChanges.forEach(fileChange => {
      console.log(`\nFile: [fileChange.path]`);
      console.log(`Changes: [fileChange.changes.length]`);
      console.log("Preview of changes:");
      console.log(fileChange.preview);
    });
    
    return result;
  } catch (error) {
    console.error(`Error refactoring files: [error.message]`);
    return null;
  }
}

refactorServiceFiles();
```

### Test Generation

This example shows how to generate tests for an existing function:

```typescript
import [CodingAssistant] from '@bluefly/native_aijs/core/neurosymbolic/CodingAssistant';
import * as fs from 'fs';

// Initialize CodingAssistant (initialization code omitted for brevity)

// Define context with a function that needs tests
const context = {
  currentFile: {
    path: '/path/to/project/src/utils/formatters.js',
    content: `
/**
 * Formats a price value according to the provided locale and currency
 * @param {number} price - The price value to format
 * @param [string] locale - The locale to use for formatting (e.g., 'en-US')
 * @param [string] currency - The currency code (e.g., 'USD')
 * @returns [string] The formatted price string
 */
export function formatPrice(price, locale = 'en-US', currency = 'USD') {
  if (typeof price !== 'number') {
    throw new TypeError('Price must be a number');
  }
  
  return new Intl.NumberFormat(locale, {
    style: 'currency',
    currency: currency
  }).format(price);
}
`,
    language: 'javascript'
  },
  language: 'javascript',
  preferences: {
    testFramework: 'jest'
  }
};

// Define request
const request = "Generate comprehensive unit tests for the formatPrice function using Jest";

// Generate tests
async function generateTests() {
  try {
    const result = await codingAssistant.processRequest(request, context);
    
    console.log("Generated tests:");
    console.log(result.code);
    
    // Save tests to file
    const testFilePath = '/path/to/project/tests/utils/formatters.test.js';
    fs.writeFileSync(testFilePath, result.code);
    
    console.log(`\nTests written to: [testFilePath]`);
    
    return result;
  } catch (error) {
    console.error(`Error generating tests: [error.message]`);
    return null;
  }
}

generateTests();
```

### Git Operations

This example demonstrates how to perform git operations using natural language:

```typescript
import [AICodingAssistant] from '@bluefly/llm-mcp/services/ai-coding-assistant';

// Initialize AICodingAssistant (initialization code omitted for brevity)

// Define context
const context = {
  currentFile: {
    path: '/path/to/project/src/components/UserProfile.jsx',
    content: '// User profile component',
    language: 'jsx'
  },
  language: 'javascript'
};

// Perform git operations
async function performGitOperations() {
  try {
    // First, get the status of the repository
    const statusResult = await codingAssistant.handleGitOperations({
      operation: "status",
      options: {}
    }, context);
    
    console.log("Git status:");
    console.log(statusResult.details);
    
    // Stage changes in specific files
    const stageResult = await codingAssistant.handleGitOperations({
      operation: "add",
      options: {
        files: [
          'src/components/UserProfile.jsx',
          'src/styles/UserProfile.css'
        ]
      }
    }, context);
    
    console.log("\nStaged files:");
    console.log(stageResult.details);
    
    // Create a commit
    const commitResult = await codingAssistant.handleGitOperations({
      operation: "commit",
      message: "Add user profile component with responsive styling",
      options: {
        push: false
      }
    }, context);
    
    console.log("\nCommit created:");
    console.log(commitResult.details);
    
    return {
      status: statusResult,
      stage: stageResult,
      commit: commitResult
    };
  } catch (error) {
    console.error(`Error performing git operations: [error.message]`);
    return null;
  }
}

performGitOperations();
```

### Task Planning and Execution

This example shows how to plan and execute a complex task:

```typescript
import [AICodingAssistant] from '@bluefly/llm-mcp/services/ai-coding-assistant';

// Initialize AICodingAssistant (initialization code omitted for brevity)

// Define context
const context = {
  currentFile: {
    path: '/path/to/project/src/App.js',
    content: '// React application entry point',
    language: 'javascript'
  },
  language: 'javascript',
  framework: 'react'
};

// Define task
const task = {
  description: "Implement a React form with validation, state management using hooks, and API submission",
  parameters: {
    formName: "ContactForm",
    fields: [
      { name: "name", type: "text", required: true },
      { name: "email", type: "email", required: true },
      { name: "message", type: "textarea", required: true }
    ],
    apiEndpoint: "/api/contact",
    stateManagement: "useState"
  }
};

// Plan and execute task
async function planAndExecuteTask() {
  try {
    // First, create a plan for the task
    const planResult = await codingAssistant.handleTaskPlanning({
      task: task.description,
      context: task.parameters,
      preferences: {
        componentStyle: 'functional',
        testingStrategy: 'component'
      }
    }, context);
    
    console.log("Task plan:");
    console.log(`Task: [planResult.task]`);
    
    // Display the plan steps
    planResult.plan.steps.forEach((step, index) => {
      console.log(`\nStep [index + 1]: [step.description]`);
      console.log(`Tool: [step.tool]`);
      console.log(`Parameters: [JSON.stringify(step.parameters, null, 2)]`);
    });
    
    // Execute the task plan
    const executionResult = await codingAssistant.executeTaskPlan(planResult.plan, context);
    
    console.log("\nTask execution results:");
    console.log(`Task ID: [executionResult.taskId]`);
    console.log(`Results: [executionResult.results.length]`);
    
    return {
      plan: planResult,
      execution: executionResult
    };
  } catch (error) {
    console.error(`Error planning and executing task: [error.message]`);
    return null;
  }
}

planAndExecuteTask();
```

## Integration Examples

### Integration with VSCode Extension

This example demonstrates how to integrate the AI Coding Assistant with a VSCode extension:

```typescript
import * as vscode from 'vscode';
import [AICodingAssistant] from '@bluefly/llm-mcp/services/ai-coding-assistant';

// Initialize AICodingAssistant (initialization code omitted for brevity)

// Register commands
export function activate(context: vscode.ExtensionContext) {
  // Register command to search code
  context.subscriptions.push(
    vscode.commands.registerCommand('bfcode.searchCode', async () => {
      // Get user input for search query
      const query = await vscode.window.showInputBox({
        prompt: 'Enter search query',
        placeHolder: 'How does authentication work?'
      });
      
      if (!query) return;
      
      // Show progress indicator
      vscode.window.withProgress({
        location: vscode.ProgressLocation.Notification,
        title: 'Searching code...',
        cancellable: true
      }, async (progress) => {
        try {
          // Get current file information
          const editor = vscode.window.activeTextEditor;
          if (!editor) {
            throw new Error('No active editor');
          }
          
          const document = editor.document;
          const context = {
            currentFile: {
              path: document.fileName,
              content: document.getText(),
              language: document.languageId
            },
            language: document.languageId
          };
          
          // Execute search
          const results = await codingAssistant.searchCode({
            text: query,
            maxResults: 10
          }, context);
          
          // Display results
          const resultsPanel = vscode.window.createWebviewPanel(
            'searchResults',
            'Code Search Results',
            vscode.ViewColumn.Beside,
            { enableScripts: true }
          );
          
          resultsPanel.webview.html = generateResultsHtml(query, results);
          
          return results;
        } catch (error) {
          vscode.window.showErrorMessage(`Error searching code: [error.message]`);
          return null;
        }
      });
    })
  );
  
  // Register more commands here...
}

// Generate HTML for search results
function generateResultsHtml(query: string, results: any[]): string {
  // HTML generation code omitted for brevity
  return `<!DOCTYPE html>
    <html>
      <head>
        <title>Search Results</title>
        <style>/* Styles omitted */</style>
      </head>
      <body>
        <h1>Search Results for "[query]"</h1>
        <div class="results">
          [results.map(result => `
            <div class="result">
              <h3>${result.path]</h3>
              <pre>[escapeHtml(result.snippet)]</pre>
              <div class="meta">
                <span>Relevance: [result.relevance]</span>
                <span>Concepts: [result.concepts.join(', ')]</span>
              </div>
            </div>
          `).join('')}
        </div>
      </body>
    </html>`;
}

function escapeHtml(text: string): string {
  return text
    .replace(/&/g, '&amp;')
    .replace(/</g, '&lt;')
    .replace(/>/g, '&gt;')
    .replace(/"/g, '&quot;')
    .replace(/'/g, '&#039;');
}
```

### Integration with Command Line Interface

This example shows how to integrate the AI Coding Assistant with a CLI:

```typescript
#!/usr/bin/env node
import [program] from 'commander';
import [AICodingAssistant] from '@bluefly/llm-mcp/services/ai-coding-assistant';
import * as fs from 'fs';
import * as path from 'path';

// Initialize AICodingAssistant (initialization code omitted for brevity)

// Setup CLI program
program
  .name('bf-code')
  .description('LLM PlatformAI Coding Assistant CLI')
  .version('1.0.0');

// Add search command
program
  .command('search <query>')
  .description('Search code using natural language')
  .option('-m, --max-results <number>', 'Maximum number of results', '5')
  .option('-f, --file <path>', 'File to use as context')
  .option('-l, --language <language>', 'Programming language')
  .action(async (query, options) => {
    try {
      // Build context
      const context = buildContext(options);
      
      // Execute search
      const results = await codingAssistant.searchCode({
        text: query,
        maxResults: parseInt(options.maxResults)
      }, context);
      
      // Display results
      console.log(`\nSearch results for: "[query]"`);
      console.log('----------------------------------------');
      
      results.forEach((result, index) => {
        console.log(`\nResult [index + 1]: [result.path]`);
        console.log(`Relevance: [result.relevance]`);
        console.log('----------------------------------------');
        console.log(result.snippet);
        console.log('----------------------------------------');
      });
      
    } catch (error) {
      console.error(`Error: [error.message]`);
      process.exit(1);
    }
  });

// Add generate command
program
  .command('generate <description>')
  .description('Generate code from natural language description')
  .option('-f, --file <path>', 'File to use as context')
  .option('-l, --language <language>', 'Programming language')
  .option('-o, --output <path>', 'Output file path')
  .action(async (description, options) => {
    try {
      // Build context
      const context = buildContext(options);
      
      // Execute generation
      const result = await codingAssistant.processRequest(description, context);
      
      if (options.output) {
        // Write to file
        fs.writeFileSync(options.output, result.code);
        console.log(`Code written to: [options.output]`);
      } else {
        // Display result
        console.log('\nGenerated code:');
        console.log('----------------------------------------');
        console.log(result.code);
        console.log('----------------------------------------');
      }
      
    } catch (error) {
      console.error(`Error: [error.message]`);
      process.exit(1);
    }
  });

// Helper function to build context
function buildContext(options) {
  const context = {
    language: options.language || 'javascript',
    currentFile: {
      path: '',
      content: '',
      language: options.language || 'javascript'
    }
  };
  
  if (options.file) {
    const filePath = path.resolve(options.file);
    if (fs.existsSync(filePath)) {
      const content = fs.readFileSync(filePath, 'utf8');
      context.currentFile = {
        path: filePath,
        content: content,
        language: options.language || getLanguageFromPath(filePath)
      };
    }
  }
  
  return context;
}

// Helper function to get language from file path
function getLanguageFromPath(filePath) {
  const ext = path.extname(filePath).toLowerCase();
  const languageMap = {
    '.js': 'javascript',
    '.ts': 'typescript',
    '.jsx': 'javascriptreact',
    '.tsx': 'typescriptreact',
    '.py': 'python',
    '.java': 'java',
    '.html': 'html',
    '.css': 'css',
    '.php': 'php'
  };
  
  return languageMap[ext] || 'plaintext';
}

// Parse command line arguments
program.parse(process.argv);
```

## See Also

- [Installation Guide](./installation.md)
- [Usage Guide](./usage.md)
- [API Reference](./api-reference.md)
- [Integration Guide](./integration.md)