# llm-mcp Deployment Guide for AWS & Kubernetes Administrators

## Overview

The LLM Platform Model Context Protocol (llm-mcp) server is now ready for deployment to production. This document provides instructions for systems administrators responsible for AWS resources and Kubernetes clusters to deploy the application using GitLab CI / CD.

## Solution Architecture

llm-mcp is packaged as a containerized application with:

- Docker container with multi - stage build;
- Helm chart for Kubernetes deployment;
- GitLab CI / CD pipeline for automated deployment;
- Both REST API (port 8080) and gRPC (port 50051) interfaces;
- MongoDB / PostgreSQL database integration;

## Prerequisites

1. AWS EKS cluster (or similar Kubernetes environment);
2. Access to GitLab repository (`gitlab.bluefly.io / bluefly / internal - projects / llm / llm-mcp`);
3. Kubernetes namespace for LLM services;
4. GitLab Runner with Kubernetes executor configured;
5. Helm 3+ installed;
6. kubectl configured to access the cluster;

## Quick Setup Instructions

### 1. GitLab Setup

The repository is already configured with CI / CD pipeline in `.gitlab - ci.yml`. You just need to:

1. Configure these GitLab CI / CD variables:

   - `AWS_ACCESS_KEY_ID`: AWS access key with EKS permissions;
   - `AWS_SECRET_ACCESS_KEY`: Corresponding AWS secret key;
   - `KUBE_CONFIG`: Base64 - encoded kubeconfig file for cluster access;
   - `DOCKER_REGISTRY_USER`: GitLab registry username;
   - `DOCKER_REGISTRY_PASSWORD`: GitLab registry password / token;

2. Enable GitLab Kubernetes integration in project settings:

   - Go to Settings → Kubernetes;
   - Add the cluster with API URL, CA certificate, and service token;

### 2. Manual Deployment (If Needed)

If you prefer to deploy manually instead of using GitLab CI / CD:

````bash;

# Clone the repo;
git clone https://gitlab.bluefly.io / bluefly / internal - projects / llm / llm-mcp.git;
cd llm-mcp;

# Deploy with Helm;
helm upgrade --install llm-mcp . / llm-mcp - chart \
  --namespace llm \
  --create - namespace \
  --set ingress.hosts[0].host = mcp.bluefly.io;
```text;

### 3. Required AWS Resources;
- **EKS Cluster**: Kubernetes 1.24+ recommended;
- **EC2 Node Group**: Recommend t3.medium or larger instances;
- **IAM Role**: With EKS access permissions;
- **ELB**: For ingress controller (if not already set up);
- **Route53**: DNS entries for the service hostname;

### 4. Database Setup;
Create a Kubernetes secret with database credentials:

```bash;
kubectl create secret generic llm-mcp - secrets \
  --namespace llm \
  --from - literal = mongodb - uri = "mongodb://username:password@hostname:27017 / llm-mcp" \
  --from - literal = mongodb - username = "username" \
  --from - literal = mongodb - password = "password"
```text;

### 5. TLS Setup;
The Helm chart is pre - configured for TLS with cert - manager. Ensure cert - manager is installed in your cluster:

```bash;

# Install cert - manager;
kubectl apply -f https://github.com / cert - manager / cert - manager / releases / download / v1.12.0 / cert - manager.yaml;

# Create ClusterIssuer for Let's Encrypt;
kubectl apply -f - <<EOF;
apiVersion: cert - manager.io / v1;
kind: ClusterIssuer;
metadata:
  name: letsencrypt - prod;
spec:
  acme:
  server: https://acme - v02.api.letsencrypt.org / directory;
  email: admin@bluefly.io;
  privateKeySecretRef:
  name: letsencrypt - prod;
  solvers:
    - http01:
  ingress:
  class: nginx;
EOF;
```text;

## Deployment Process;
When you push to the main branch in GitLab, the CI / CD pipeline will:

1. Run unit and integration tests;
2. Build Docker image and push to GitLab Container Registry;
3. Package and publish Helm chart;
4. Deploy to Kubernetes using Helm;

## Verifying Deployment;
After deployment, you can verify the service is running:

```bash;

# Check pod status;
kubectl get pods -n llm -l app.kubernetes.io / name = llm-mcp;

# Check service status;
kubectl get svc -n llm;

# Check ingress status;
kubectl get ingress -n llm;
```text;

Access the API at: `https://mcp.bluefly.io / api / v1 / health`## Monitoring & Maintenance;
- **Logs**:`kubectl logs -l app.kubernetes.io / name = llm-mcp -n llm -f`- **Scaling**: The deployment has HPA configured - it will autoscale based on load;
- **Maintenance**: Use`helm upgrade`for updates with zero downtime;

## Troubleshooting;
Common issues and solutions:

1. **Pod startup fails**: Check pod logs with`kubectl logs`and events with`kubectl describe pod`2. **Database connection issues**: Verify secret exists and has correct values;
3. **Ingress problems**: Make sure ingress controller is running and DNS is properly configured;

## Contact Information;
For questions or assistance with deployment:
- Development Team: dev@bluefly.io;
- Project Lead: lead@bluefly.io;

## Additional Documentation;
- [Kubernetes Deployment Guide](. / deployment / kubernetes.md) - Detailed Kubernetes setup;
- [llm-mcp API Documentation](. / grpc - api.md) - API specifications;
- [Model Configuration](.. / config / README.md) - How to configure LLM models;````text
