# Tool Registration Guide

This guide provides a detailed walkthrough for registering, managing, and executing tools with the LLM Platform Model Context Protocol (LLM-MCP) platform. Tools are the fundamental building blocks that enable AI agents to perform specific actions within your application ecosystem.

## What Are LLM-MCP Tools?

In the LLM-MCP platform, a tool is a well-defined function that:

- Has a unique identifier
- Accepts structured input parameters
- Performs a specific action
- Returns structured output
- Is discoverable by AI agents and systems

Tools enable AI systems to interact with external services, databases, APIs, and business logic in a controlled, standardized way.

## Tool Registration Process

### Prerequisites

Before registering a tool, you need:

1. A running LLM-MCP server
2. Authentication credentials for the LLM-MCP API
3. A clearly defined tool function with inputs and outputs

### Basic Tool Registration

The simplest way to register a tool is using the LLM-MCP client:

```javascript
const [BfmcpClient] = require('@bluefly/llm-mcp/client');

// Initialize client
const client = new BfmcpClient({
  serverUrl: 'http://localhost:3001',
  apiKey: 'your-api-key'
});

// Define a tool
const simpleTool = {
  id: "hello_world",
  name: "Hello World",
  description: "A simple greeting tool",
  version: "1.0.0",
  category: "demo",
  input_schema: {
    type: "object",
    properties: {
      name: {
        type: "string",
        description: "The name to greet"
      }
    },
    required: ["name"]
  },
  output_schema: {
    type: "object",
    properties: {
      greeting: {
        type: "string",
        description: "The greeting message"
      }
    }
  },
  function: async (Name) => {
    return {
      greeting: `Hello, Name!`
    };
  }
};

// Register the tool
async function registerTool() {
  try {
    await client.registerTool(simpleTool);
    console.log(`Tool [simpleTool.id] registered successfully`);
  } catch (error) {
    console.error(`Tool registration failed: [error.message]`);
  }
}

registerTool();
```

### Tool Definition Schema

A complete tool definition includes:

| Field | Type | Required | Description |
|-------|------|----------|-------------|
| `id` | string | Yes | Unique identifier for the tool |
| `name` | string | Yes | Human-readable name |
| `description` | string | Yes | Detailed description of what the tool does |
| `version` | string | Yes | Semantic version (e.g., "1.0.0") |
| `category` | string | No | Category for grouping related tools |
| `tags` | string[] | No | Tags for filtering and discovery |
| `input_schema` | object | Yes | JSON Schema for input validation |
| `output_schema` | object | Yes | JSON Schema for output validation |
| `function` | function | Yes* | Implementation function (for local tools) |
| `endpoint` | string | Yes* | API endpoint URL (for remote tools) |
| `auth_config` | object | No | Authentication configuration for remote tools |
| `rate_limit` | object | No | Rate limiting configuration |
| `timeout` | number | No | Execution timeout in milliseconds |
| `cache_config` | object | No | Caching configuration |
| `metadata` | object | No | Additional tool metadata |

*Either `function` or `endpoint` is required, depending on whether the tool is local or remote.

### Input and Output Schemas

Input and output schemas use JSON Schema format to define the structure and validation rules:

```javascript
// Example of a more complex input schema
const inputSchema = {
  type: "object",
  properties: {
    query: {
      type: "string",
      description: "Search query string",
      minLength: 3
    },
    filters: {
      type: "object",
      properties: {
        category: {
          type: "string",
          enum: ["books", "electronics", "clothing"]
        },
        price_range: {
          type: "object",
          properties: {
            min: { type: "number", minimum: 0 },
            max: { type: "number" }
          }
        }
      }
    },
    limit: {
      type: "integer",
      description: "Maximum number of results",
      default: 10,
      minimum: 1,
      maximum: 100
    },
    sort: {
      type: "string",
      enum: ["relevance", "price_asc", "price_desc", "newest"],
      default: "relevance"
    }
  },
  required: ["query"]
};
```

Input schemas should:
- Clearly define all parameters
- Include descriptive information for each field
- Specify which fields are required
- Set appropriate defaults where applicable
- Include validation rules (min/max values, patterns, etc.)

Output schemas should:
- Define the structure of successful responses
- Include all possible return fields
- Document field types and formats

## Advanced Tool Registration

### Remote Tools

For tools that call external APIs or services:

```javascript
const remoteTool = {
  id: "weather_api",
  name: "Weather API",
  description: "Get current weather for a location using an external API",
  version: "1.0.0",
  category: "weather",
  tags: ["api", "weather", "external"],
  input_schema: {
    type: "object",
    properties: {
      location: {
        type: "string",
        description: "City and country/state"
      },
      units: {
        type: "string",
        enum: ["metric", "imperial"],
        default: "metric",
        description: "Temperature units"
      }
    },
    required: ["location"]
  },
  output_schema: {
    type: "object",
    properties: {
      temperature: {
        type: "number",
        description: "Current temperature"
      },
      conditions: {
        type: "string",
        description: "Weather conditions"
      },
      humidity: {
        type: "number",
        description: "Humidity percentage"
      },
      wind_speed: {
        type: "number",
        description: "Wind speed"
      }
    }
  },
  // Instead of function, specify an endpoint
  endpoint: "https://api.weatherservice.com/current",
  // Authentication for the remote API
  auth_config: {
    type: "api_key",
    header_name: "X-API-Key",
    key: process.env.WEATHER_API_KEY
  },
  // Rate limiting
  rate_limit: {
    max_requests: 100,
    period: "hour"
  },
  // Timeout
  timeout: 5000,
  // Caching
  cache_config: {
    enabled: true,
    ttl: 600 // 10 minutes
  }
};

// Register the remote tool
await client.registerTool(remoteTool);
```

### Tool with Authentication

For tools that require authentication:

```javascript
const authenticatedTool = {
  id: "private_database_query",
  name: "Private Database Query",
  description: "Query private database with authentication",
  version: "1.0.0",
  // Tool definition...
  
  // Authentication configuration
  auth_config: {
    // API Key authentication
    type: "api_key",
    header_name: "Authorization",
    key: "Bearer YOUR_SECRET_API_KEY"
    
    // Or OAuth2 authentication
    /*
    type: "oauth2",
    token_url: "https://auth.example.com/oauth/token",
    client_id: "your-client-id",
    client_secret: "your-client-secret",
    scopes: ["read", "write"]
    */
    
    // Or Basic authentication
    /*
    type: "basic",
    username: "username",
    password: "password"
    */
  }
};
```

### Tool with Rate Limiting

To prevent abuse and control resource usage:

```javascript
const rateLimitedTool = {
  id: "rate_limited_tool",
  name: "Rate Limited Tool",
  description: "Tool with rate limiting",
  version: "1.0.0",
  // Tool definition...
  
  // Rate limiting configuration
  rate_limit: {
    max_requests: 5,    // Maximum number of requests
    period: "minute",   // Time period ("second", "minute", "hour", "day")
    strategy: "fixed",  // Rate limiting strategy ("fixed", "sliding", "token_bucket")
    
    // Optional: per-user rate limits
    scope: "user",      // Apply limits per user ("global", "user", "ip")
    
    // Optional: queueing behavior
    queue_excess: true, // Queue excess requests instead of rejecting
    queue_timeout: 30   // Timeout for queued requests (seconds)
  }
};
```

### Tool with Caching

For improved performance and reduced resource usage:

```javascript
const cachedTool = {
  id: "cached_tool",
  name: "Cached Tool",
  description: "Tool with response caching",
  version: "1.0.0",
  // Tool definition...
  
  // Caching configuration
  cache_config: {
    enabled: true,            // Enable caching
    ttl: 300,                 // Cache TTL in seconds (5 minutes)
    max_entries: 1000,        // Maximum cache entries
    strategy: "lru",          // Cache eviction strategy ("lru", "lfu", "fifo")
    
    // Optional: cache key generation
    key_fields: ["query"],    // Fields to include in cache key
    
    // Optional: conditional caching
    cache_if: (result) => result.status === "success",
    
    // Optional: cache invalidation tags
    tags: ["products", "search"]
  }
};
```

## Tool Categories and Organization

Organizing tools into categories helps with discovery and management:

```javascript
// Define a tool with categories and tags
const categorizedTool = {
  id: "product_search",
  name: "Product Search",
  description: "Search for products in catalog",
  version: "1.0.0",
  
  // Categorization
  category: "e-commerce",  // Primary category
  subcategory: "search",   // Optional subcategory
  
  // Tags for filtering and discovery
  tags: ["products", "search", "catalog", "inventory"],
  
  // Additional metadata
  metadata: {
    owner: "inventory-team",
    documentation_url: "https://docs.example.com/tools/product-search",
    created_at: "2023-09-15T00:00:00Z",
    updated_at: "2023-10-20T00:00:00Z",
    deprecated: false,
    display_order: 5
  },
  
  // Tool definition...
};
```

## Tool Registration Management

### Listing Registered Tools

```javascript
// List all registered tools
const tools = await client.listTools();
console.log(`Found [tools.length] registered tools`);

// List tools by category
const searchTools = await client.listTools({ category: "search" });
console.log(`Found [searchTools.length] search tools`);

// List tools with filtering
const filteredTools = await client.listTools({
  tags: ["api"],
  query: "weather"
});
console.log(`Found [filteredTools.length] matching tools`);
```

### Retrieving Tool Details

```javascript
// Get details for a specific tool
const toolDetails = await client.getTool("weather_api");
console.log(`Tool details for [toolDetails.name]:`);
console.log(`- Description: [toolDetails.description]`);
console.log(`- Version: [toolDetails.version]`);
console.log(`- Category: [toolDetails.category]`);
```

### Updating Tools

```javascript
// Update an existing tool
const toolUpdate = {
  id: "weather_api",
  // Only include fields that need updating
  description: "Updated weather API description",
  version: "1.1.0",
  // New fields to add
  tags: ["weather", "api", "updated"],
  // Updated schema
  input_schema: {
    // Updated schema definition
  }
};

await client.updateTool(toolUpdate);
console.log(`Tool [toolUpdate.id] updated successfully`);
```

### Deactivating and Reactivating Tools

```javascript
// Deactivate a tool (without deleting it)
await client.deactivateTool("weather_api");
console.log(`Tool weather_api deactivated`);

// Reactivate a tool
await client.activateTool("weather_api");
console.log(`Tool weather_api reactivated`);
```

### Deleting Tools

```javascript
// Delete a tool permanently
await client.deleteTool("deprecated_tool");
console.log(`Tool deprecated_tool deleted`);
```

## Tool Execution

### Basic Tool Execution

```javascript
// Execute a tool
const result = await client.executeTool("hello_world", {
  name: "John"
});

console.log(`Execution result: [result.greeting]`);
// Output: Execution result: Hello, John!
```

### Tool Execution with Options

```javascript
// Execute with additional options
const result = await client.executeTool("weather_api", 
  // Tool input
  {
    location: "New York, NY",
    units: "metric"
  },
  // Execution options
  {
    timeout: 10000,            // Override timeout (10 seconds)
    cache: false,              // Bypass cache for this request
    context: {                 // Provide execution context
      user_id: "user-123",
      session_id: "session-456",
      request_id: "req-789"
    },
    execution_mode: "async"    // Execute asynchronously
  }
);

console.log("Weather:", result);
```

### Asynchronous Tool Execution

For long-running operations:

```javascript
// Start asynchronous execution
const executionId = await client.executeToolAsync("data_processing_task", {
  dataset_id: "large-dataset-123",
  operation: "aggregate"
});

console.log(`Execution started with ID: [executionId]`);

// Check execution status
const status = await client.getToolExecutionStatus(executionId);
console.log(`Execution status: [status.state]`);

// Get execution result when completed
if (status.state === "completed") {
  const result = await client.getToolExecutionResult(executionId);
  console.log("Execution result:", result);
}
```

### Batch Tool Execution

```javascript
// Execute multiple tools in a batch
const batchResults = await client.executeToolBatch([
  {
    tool_id: "weather_api",
    input: { location: "New York, NY" }
  },
  {
    tool_id: "currency_converter",
    input: { amount: 100, from: "USD", to: "EUR" }
  },
  {
    tool_id: "news_headlines",
    input: { category: "technology", limit: 5 }
  }
]);

// Process batch results
batchResults.forEach((result, index) => {
  console.log(`Result [index + 1]:`, result);
});
```

## Tool Development Best Practices

### Input Validation

Always validate inputs thoroughly:

```javascript
// Example of input validation in a tool function
function: async (input) => {
  // Validate required fields
  if (!input.query || typeof input.query !== 'string') {
    throw new Error("Invalid query: must be a non-empty string");
  }
  
  // Validate numeric ranges
  if (input.limit !== undefined) {
    if (typeof input.limit !== 'number' || input.limit < 1 || input.limit > 100) {
      throw new Error("Invalid limit: must be a number between 1 and 100");
    }
  }
  
  // Perform the tool operation
  // ...
  
  return result;
}
```

### Error Handling

Handle errors gracefully and provide informative error messages:

```javascript
function: async (input) => {
  try {
    // Attempt to call external API
    const response = await fetch(`https://api.example.com/data?q=[encodeURIComponent(input.query)]`);
    
    // Handle HTTP errors
    if (!response.ok) {
      if (response.status === 404) {
        return {
          status: "not_found",
          message: "No data found for the given query"
        };
      } else if (response.status === 429) {
        throw new Error("RATE_LIMIT_EXCEEDED: Too many requests to external API");
      } else {
        throw new Error(`API Error: [response.status] [response.statusText]`);
      }
    }
    
    // Process successful response
    const data = await response.json();
    return {
      status: "success",
      data: data.results
    };
  } catch (error) {
    // Log the error for monitoring
    console.error("Tool execution error:", error);
    
    // Return structured error response
    return {
      status: "error",
      error_code: error.code || "EXECUTION_FAILED",
      message: error.message || "An unexpected error occurred during tool execution"
    };
  }
}
```

### Performance Optimization

Optimize your tools for performance:

```javascript
// Example of a performance-optimized tool
function: async (input) => {
  // Use memoization for expensive operations
  const result = await memoizedExpensiveOperation(input);
  
  // Process only necessary fields
  const processedResult = {
    id: result.id,
    name: result.name,
    // Only include required fields, not the entire result object
  };
  
  // Limit result size
  if (result.items && result.items.length > input.limit) {
    processedResult.items = result.items.slice(0, input.limit);
  }
  
  return processedResult;
}
```

### Security Considerations

Follow security best practices:

```javascript
// Example of security-conscious tool
function: async (input) => {
  // Sanitize inputs
  const sanitizedQuery = sanitizeInput(input.query);
  
  // Validate permissions
  await validateUserPermissions(input.context.user_id, 'read_data');
  
  // Use secure API calls
  const result = await secureApiCall(sanitizedQuery);
  
  // Audit logging
  await logAuditEvent({
    user_id: input.context.user_id,
    action: 'tool_execution',
    tool_id: 'secure_tool',
    input_summary: { query: maskSensitiveData(input.query) }
  });
  
  return result;
}
```

## Tool Registration with gRPC API

For high-performance tool registration, use the gRPC API:

```javascript
const [BfmcpGrpcClient] = require('@bluefly/llm-mcp/grpc');

// Create gRPC client
const grpcClient = new BfmcpGrpcClient('localhost:3002');

// Register tool using gRPC
await grpcClient.registerTool({
  id: "grpc_tool",
  name: "gRPC Tool",
  description: "Tool registered via gRPC",
  version: "1.0.0",
  // Tool definition...
});

// Execute tool using gRPC
const result = await grpcClient.executeTool("grpc_tool", {
  parameter: "value"
});
```

## Tool Provider Registration

For services that provide multiple tools:

```javascript
// Register a tool provider
await client.registerProvider({
  id: "weather_service_provider",
  name: "Weather Service Provider",
  description: "Provider for weather-related tools",
  endpoint: "https://weather-service.example.com/tools",
  auth_config: {
    type: "api_key",
    header_name: "X-API-Key",
    key: process.env.WEATHER_SERVICE_API_KEY
  }
});

// List all tools from a provider
const providerTools = await client.listProviderTools("weather_service_provider");
console.log(`Provider has [providerTools.length] tools`);
```

## Tool Registration from OpenAPI Specifications

You can automatically generate tools from OpenAPI specifications:

```javascript
// Register tools from OpenAPI specification
await client.registerToolsFromOpenApi({
  specUrl: "https://api.example.com/openapi.json",
  // Optional filtering to specific endpoints
  pathPatterns: ["/v1/products/*", "/v1/orders/*"],
  // Optional tool naming convention
  toolIdPrefix: "api_",
  // Optional authentication for all generated tools
  auth_config: {
    type: "oauth2",
    token_url: "https://api.example.com/oauth/token",
    client_id: process.env.API_CLIENT_ID,
    client_secret: process.env.API_CLIENT_SECRET
  }
});
```

## Integration with Agent Systems

LLM-MCP tools can be integrated with various agent systems:

```javascript
// Example: Registering LLM-MCP tools with LangChain
const [Tool] = require('langchain/tools');
const [BfmcpClient] = require('@bluefly/llm-mcp/client');

// Initialize LLM-MCP client
const llm-mcpClient = new BfmcpClient({
  serverUrl: 'http://localhost:3001'
});

// Get all available tools
const llm-mcpTools = await llm-mcpClient.listTools();

// Create LangChain tools from LLM-MCP tools
const langchainTools = llm-mcpTools.map(tool => {
  return new Tool({
    name: tool.id,
    description: tool.description,
    func: async (input) => {
      // Parse input string to JSON if needed
      let parsedInput;
      try {
        parsedInput = typeof input === 'string' ? JSON.parse(input) : input;
      } catch (e) {
        // Handle simple string inputs by mapping to first required parameter
        const firstRequired = tool.input_schema.required?.[0];
        parsedInput = firstRequired ? { [firstRequired]: input } : [input];
      }
      
      // Execute the LLM-MCP tool
      const result = await llm-mcpClient.executeTool(tool.id, parsedInput);
      
      // Return result as string for LangChain
      return typeof result === 'object' ? JSON.stringify(result) : result;
    }
  });
});

// Use the tools with LangChain
const agent = AgentExecutor.fromAgentAndTools({
  agent,
  tools: langchainTools
});
```

## Monitoring and Debugging Tools

### Monitoring Tool Usage

```javascript
// Get tool usage statistics
const usageStats = await client.getToolUsageStats("weather_api", {
  timeRange: {
    start: "2023-10-01T00:00:00Z",
    end: "2023-10-31T23:59:59Z"
  },
  interval: "day"
});

console.log("Usage statistics:", usageStats);
```

### Tool Execution Logs

```javascript
// Get execution logs for a tool
const executionLogs = await client.getToolExecutionLogs("weather_api", {
  limit: 10,
  offset: 0,
  status: "error" // Filter to only error logs
});

console.log("Execution logs:", executionLogs);
```

### Testing Tools

```javascript
// Test a tool without registering it
const testResult = await client.testTool({
  id: "test_tool",
  name: "Test Tool",
  description: "Tool for testing",
  version: "1.0.0",
  function: async (input) => {
    return { message: `Test successful with input: [JSON.stringify(input)]` };
  },
  input_schema: {
    type: "object",
    properties: {
      test_param: { type: "string" }
    }
  }
}, {
  test_param: "test_value"
});

console.log("Test result:", testResult);
```

## Conclusion

Tool registration is a core functionality of the LLM-MCP platform, enabling AI systems to interact with external services and business logic in a standardized, secure way. By following the best practices outlined in this guide, you can create robust, performant, and secure tools that enhance your AI applications.

For more information:

- See the [API Reference](../api/api-reference.md) for detailed endpoint documentation
- Review the [Architecture Overview](../architecture/architecture-overview.md) for system design
- Check the [Configuration Guide](../configuration.md) for tool registry configuration
- Explore [Example Code](../examples/tool-examples.md) for more tool examples

If you encounter issues with tool registration or execution, refer to our [Troubleshooting Guide](../troubleshooting/troubleshooting.md).