# llm-mcp Integration with bfapi and bfllm

This document details how llm-mcp (LLM Platform Model Context Protocol) integrates with bfapi and bfllm (LLM Platform Language Learning Models) to provide a comprehensive AI model management and inference system.

## Overview

llm-mcp serves as the middleware that enables AI assistants like Claude to access bfllm models and other tools. It provides a standardized protocol for tool discovery, registration, and execution, allowing bfapi to use AI tools through the LangChain Agent and bfllm to expose its models as tools and resources.

```mermaid
graph TD
    subgraph llm-mcp
        MCPServer[MCP Server]
        ToolRegistry[Tool Registry]
        ResourceProviders[Resource Providers]
        ClaudeAdapter[Claude Adapter]
        LLMPlugin[LLM Plugin]
    end

    subgraph bfapi
        APIGateway[API Gateway]
        LangChainAgent[LangChain Agent]
    end

    subgraph bfllm
        ModelRegistry[Model Registry]
        InferenceEngine[Inference Engine]
        MCPPlugin[MCP Plugin]
    end

    subgraph External
        Claude[Claude]
    end

    LangChainAgent -->|Discover Execute| ToolRegistry
    MCPPlugin -->|Register| ToolRegistry
    MCPPlugin -->|Register| ResourceProviders
    
    LLMPlugin -->|Forward| InferenceEngine
    
    Claude -->|Connect| ClaudeAdapter
    ClaudeAdapter -->|Register| MCPServer
    Claude -->|Use Tools| ToolRegistry
```

## Integration Components

### 1. LLM Plugin

The `llm-plugin.js` in llm-mcp integrates with bfllm to provide models as tools and resources:

```javascript
// src/plugins/llm-plugin.js
/**
 * LLM Plugin for OpenAPI MCP Server
 *
 * This plugin registers LLM models as MCP resources and tools,
 * enabling Claude and other assistants to interact with the models.
 */

import [logger] from '../utils/logger.js';
import [llmService] from '../services/llm-service.js';

/**
 * LLM MCP Plugin
 */
class LLMPlugin {
  constructor() {
    this.initialized = false;
  }

  /**
   * Initialize the plugin
   */
  async initialize(mcpServer) {
    if (this.initialized) return;

    logger.info('Initializing LLM MCP Plugin');
    this.mcpServer = mcpServer;

    try {
      // Initialize LLM service
      await llmService.initialize();

      // Register tools and resources
      this.registerTools();
      this.registerResources();

      logger.info('LLM MCP Plugin initialized successfully');
      this.initialized = true;
    } catch (error) {
      logger.error(`Failed to initialize LLM MCP Plugin: [error.message]`);
      throw error;
    }
  }

  /**
   * Register LLM tools with MCP
   */
  registerTools() {
    logger.info('Registering LLM tools with MCP');

    try {
      // Get tool definitions from LLM service
      const tools = llmService.generateMCPTools();

      // Register each tool
      for (const tool of tools) {
        this.mcpServer.registerTool(tool.name, tool, async params => {
          logger.info(`Tool [tool.name] called with params: [JSON.stringify(params)]`);

          // Handle generate_text tool
          if (tool.name === 'generate_text') {
            return await llmService.generateText(params);
          }

          // Handle model-specific generate tools
          if (tool.name.endsWith('_generate')) {
            const modelId = tool.name.replace('_generate', '');
            return await llmService.generateText({
              ...params,
              model: modelId,
            });
          }

          // Unsupported tool
          return {
            content: [
              {
                type: 'error',
                text: `Tool [tool.name] not implemented`,
              },
            ],
            isError: true,
          };
        });

        logger.info(`Registered tool: [tool.name]`);
      }
    } catch (error) {
      logger.error(`Error registering LLM tools: [error.message]`);
      throw error;
    }
  }

  /**
   * Register LLM resources with MCP
   */
  registerResources() {
    logger.info('Registering LLM resources with MCP');

    try {
      // Get resource definitions from LLM service
      const resources = llmService.generateMCPResources();

      // Register each resource
      for (const resource of resources) {
        this.mcpServer.registerResource(resource.uri, resource, async uri => {
          logger.info(`Resource [uri] requested`);

          // Handle models list resource
          if (uri === 'llm://models') {
            const content = await llmService.getModelsResource();
            return {
              uri,
              mimeType: 'application/json',
              text: content,
            };
          }

          // Handle model-specific resources
          if (uri.startsWith('llm://models/')) {
            const modelId = uri.replace('llm://models/', '');
            const content = await llmService.getModelResource(modelId);
            return {
              uri,
              mimeType: 'application/json',
              text: content,
            };
          }

          // Unsupported resource
          throw new Error(`Resource [uri] not found`);
        });

        logger.info(`Registered resource: [resource.uri]`);
      }
    } catch (error) {
      logger.error(`Error registering LLM resources: [error.message]`);
      throw error;
    }
  }
}
```

### 2. Claude Adapter

The `claude-adapter.js` in llm-mcp integrates with Claude to provide access to bfllm models:

```javascript
// src/adapters/claude-adapter.js
import fs from 'fs';
import path from 'path';
import os from 'os';
import [createLogger] from '../utils/logger.js';

const logger = createLogger('claude-adapter');

/**
 * Adapter for connecting local Claude instances to the MCP server
 */
export class ClaudeAdapter [constructor(config = {]) {
    // Check for container environment first
    const containerConfigPath = 
      '/home/.docker.host-files/Library/Application Support/Claude/claude_desktop_config.json';
    const defaultConfigPath = path.join(
      os.homedir(),
      'Library/Application Support/Claude/claude_desktop_config.json'
    );

    // Determine which path to use based on what exists
    let detectedPath = config.configPath;
    if (!detectedPath) {
      if (fs.existsSync(containerConfigPath)) {
        detectedPath = containerConfigPath;
        logger.info('Using container Claude config path');
      } else {
        detectedPath = defaultConfigPath;
        logger.info('Using default Claude config path');
      }
    }

    this.config = {
      configPath: detectedPath,
      pollIntervalMs: config.pollIntervalMs || 60000, // 1 minute
      enabled: config.enabled !== false,
      ...config,
    };

    this.claudeConfig = null;
    this.isInitialized = false;
    this.pollInterval = null;

    logger.info('Claude adapter initialized with config path:', this.config.configPath);
  }

  /**
   * Initialize the adapter
   * @param [Object] mcpService - The MCP service instance
   * @returns [Promise<boolean>] Initialization result
   */
  async initialize(mcpService) {
    if (!this.config.enabled) {
      logger.info('Claude adapter is disabled');
      return false;
    }

    try {
      this.mcpService = mcpService;

      // Load Claude config
      await this.loadClaudeConfig();

      // Register the Claude service with MCP
      await this.registerWithMCP();

      // Start polling for config changes
      this.startPolling();

      this.isInitialized = true;
      logger.info('Claude adapter initialized successfully');
      return true;
    } catch (error) {
      logger.error('Failed to initialize Claude adapter:', error);
      return false;
    }
  }

  /**
   * Register the Claude service with the MCP
   * @returns [Promise<void>]
   */
  async registerWithMCP() {
    if (!this.mcpService) {
      logger.error('Cannot register with MCP: MCP service not provided');
      return;
    }

    try {
      // Extract Claude API details from config
      const apiKey = this.claudeConfig?.api_key;
      const model = this.claudeConfig?.default_model || 'claude-3-haiku';

      if (!apiKey) {
        logger.warn('Cannot register with MCP: No API key found in Claude config');
        return;
      }

      // Register Claude as a model provider
      await this.mcpService.registerComponent({
        id: 'claude-local',
        type: 'model-provider',
        url: 'local://claude',
        description: 'Local Claude Instance',
        metadata: {
          model,
          provider: 'anthropic',
          local: true,
          configPath: this.config.configPath,
        },
      });

      logger.info('Registered Claude service with MCP');
    } catch (error) {
      logger.error('Error registering Claude with MCP:', error);
      throw error;
    }
  }
}
```

## Integration Flow Diagrams

### 1. Tool Registration Flow

```mermaid
sequenceDiagram
    participant bfllm
    participant MCPPlugin
    participant llm-mcp
    participant ToolRegistry
    
    bfllm->>MCPPlugin: Initialize plugin
    activate MCPPlugin
    MCPPlugin->>llm-mcp: Connect to MCP server
    llm-mcp->>ToolRegistry: Create tool registry
    MCPPlugin->>ToolRegistry: Register generate_text tool
    MCPPlugin->>ToolRegistry: Register model-specific tools
    deactivate MCPPlugin
    
    Note over llm-mcp,ToolRegistry: Tools are now available for discovery
```

### 2. Tool Discovery and Execution Flow

```mermaid
sequenceDiagram
    participant bfapi
    participant LangChainAgent
    participant llm-mcp
    participant ToolRegistry
    participant LLMPlugin
    participant bfllm
    
    bfapi->>LangChainAgent: Create agent with MCP tools
    LangChainAgent->>llm-mcp: Discover available tools
    llm-mcp->>ToolRegistry: Get registered tools
    ToolRegistry-->>llm-mcp: Tool definitions
    llm-mcp-->>LangChainAgent: Tool definitions
    
    LangChainAgent->>llm-mcp: Execute generate_text tool
    llm-mcp->>ToolRegistry: Find tool handler
    ToolRegistry->>LLMPlugin: Execute generate_text
    LLMPlugin->>bfllm: Generate text with model
    bfllm-->>LLMPlugin: Generated text
    LLMPlugin-->>ToolRegistry: Tool result
    ToolRegistry-->>llm-mcp: Tool result
    llm-mcp-->>LangChainAgent: Tool execution result
    LangChainAgent-->>bfapi: Agent execution result
```

### 3. Claude Integration Flow

```mermaid
sequenceDiagram
    participant Claude
    participant ClaudeAdapter
    participant llm-mcp
    participant ToolRegistry
    participant LLMPlugin
    participant bfllm
    
    Claude->>ClaudeAdapter: Connect to adapter
    ClaudeAdapter->>llm-mcp: Register with MCP
    
    Claude->>llm-mcp: Discover available tools
    llm-mcp->>ToolRegistry: Get registered tools
    ToolRegistry-->>llm-mcp: Tool definitions
    llm-mcp-->>Claude: Tool definitions
    
    Claude->>llm-mcp: Execute generate_text tool
    llm-mcp->>ToolRegistry: Find tool handler
    ToolRegistry->>LLMPlugin: Execute generate_text
    LLMPlugin->>bfllm: Generate text with model
    bfllm-->>LLMPlugin: Generated text
    LLMPlugin-->>ToolRegistry: Tool result
    ToolRegistry-->>llm-mcp: Tool result
    llm-mcp-->>Claude: Tool execution result
```

## Configuration

llm-mcp requires specific configuration to connect to bfapi and bfllm:

```javascript
// config/index.js
export const config = {
  // Server configuration
  server: {
    port: process.env.PORT || 8000,
    host: process.env.HOST || 'localhost',
  },
  
  // Plugin configuration
  plugins: {
    llm: {
      enabled: process.env.ENABLE_LLM_PLUGIN !== 'false',
      path: process.env.LLM_PLUGIN_PATH || './src/plugins/llm-plugin.js',
    },
    claude: {
      enabled: process.env.ENABLE_CLAUDE_ADAPTER !== 'false',
      configPath: process.env.CLAUDE_CONFIG_PATH,
    }
  },
  
  // bfllm configuration
  llm: {
    url: process.env.bfllm_URL || 'http://localhost:5000',
    apiKey: process.env.bfllm_API_KEY || 'development-key',
  },
  
  // bfapi configuration
  api: {
    url: process.env.bfapi_URL || 'http://localhost:3000/api/v1',
    apiKey: process.env.bfapi_API_KEY || 'development-key',
  }
};
```

## Environment Variables

Set these environment variables to configure the integration:

```bash
# .env file
PORT=8000
HOST=localhost
ENABLE_LLM_PLUGIN=true
ENABLE_CLAUDE_ADAPTER=true
CLAUDE_CONFIG_PATH=/path/to/claude_desktop_config.json
bfllm_URL=http://localhost:5000
bfllm_API_KEY=your_development_key
bfapi_URL=http://localhost:3000/api/v1
bfapi_API_KEY=your_development_key
```

## Component Diagram

```mermaid
classDiagram
    class MCPServer {
        +registerTool(name, definition, handler)
        +registerResource(uri, definition, handler)
        +registerComponent(component)
        +start()
        +stop()
    }
    
    class ToolRegistry {
        +register(tool)
        +get(name)
        +list()
        +execute(name, params)
    }
    
    class ResourceProviders [+register(provider)
        +get(uri)
        +list()
        +read(uri)]
    
    class LLMPlugin [+initialize(server)
        +registerTools()
        +registerResources()]
    
    class ClaudeAdapter [+initialize(mcpService)
        +loadClaudeConfig()
        +registerWithMCP()
        +startPolling()
        +stop()]
    
    class LLMService [+initialize()
        +scanModels()
        +loadModel(modelName)
        +getModels()
        +getModel(modelName)
        +generateMCPTools()
        +generateMCPResources()
        +generateText(params)]
    
    MCPServer --> ToolRegistry : uses
    MCPServer --> ResourceProviders : uses
    LLMPlugin --> LLMService : uses
    LLMPlugin --> MCPServer : registers with
    ClaudeAdapter --> MCPServer : registers with
```

## Error Handling

llm-mcp implements error handling for both bfapi and bfllm integrations:

```javascript
// Example error handling in llm-plugin.js
try {
  return await llmService.generateText(params);
} catch (error) {
  logger.error(`Error generating text: [error.message]`);
  return {
    content: [
      {
        type: 'error',
        text: `Error generating text: [error.message]`,
      },
    ],
    isError: true,
  };
}

// Example error handling in claude-adapter.js
try {
  await this.registerWithMCP();
} catch (error) {
  logger.error('Error registering Claude with MCP:', error);
  throw error;
}
```

## Testing the Integration

### Testing LLM Plugin

```javascript
// __tests__/llm-plugin.test.js
describe('LLM Plugin', () => {
  it('should register tools with MCP server', async () => {
    // Mock MCP server
    const mockServer = {
      registerTool: jest.fn(),
      registerResource: jest.fn(),
    };
    
    // Mock LLM service
    jest.spyOn(llmService, 'generateMCPTools').mockReturnValue([
      { name: 'generate_text', description: 'Generate text' }
    ]);
    
    // Initialize plugin
    const plugin = new LLMPlugin();
    await plugin.initialize(mockServer);
    
    // Verify tool registration
    expect(mockServer.registerTool).toHaveBeenCalledWith(
      'generate_text',
      expect.any(Object),
      expect.any(Function)
    );
  });
});
```

### Testing Claude Adapter

```javascript
// __tests__/claude-adapter.test.js
describe('Claude Adapter', () => {
  it('should register with MCP service', async () => {
    // Mock MCP service
    const mockMCPService = {
      registerComponent: jest.fn(),
    };
    
    // Mock Claude config
    jest.spyOn(fs.promises, 'readFile').mockResolvedValue(JSON.stringify({
      api_key: 'test-key',
      default_model: 'claude-3-haiku',
    }));
    
    // Initialize adapter
    const adapter = new ClaudeAdapter();
    await adapter.initialize(mockMCPService);
    
    // Verify registration
    expect(mockMCPService.registerComponent).toHaveBeenCalledWith(
      expect.objectContaining({
        id: 'claude-local',
        type: 'model-provider',
      })
    );
  });
});
```

## Troubleshooting

Common integration issues and their solutions:

| Issue | Possible Cause | Solution |
|-------|----------------|----------|
| Cannot connect to bfllm | Incorrect bfllm_URL | Check the bfllm_URL environment variable |
| Cannot connect to bfapi | Incorrect bfapi_URL | Check the bfapi_URL environment variable |
| Authentication errors | Incorrect API keys | Verify bfllm_API_KEY and bfapi_API_KEY |
| LLM plugin not loading | Incorrect plugin path | Check LLM_PLUGIN_PATH |
| Claude adapter not working | Incorrect Claude config | Check CLAUDE_CONFIG_PATH |

## References

- [llm-mcp Documentation](/docs/index.md)
- [bfapi Documentation](../../bfapi/docs/index.md)
- [bfllm Documentation](../../bfllm/docs/index.md)
- [Central Integration Documentation](../../BFDOCS/docs/integration/bfapi-bfllm-llm-mcp-integration.md)
- [MCP Integration in bfllm](../../bfllm/docs/mcp-integration.md)

## Drupal Integration: Vector Storage & Search

The MCP Vector API can be used from Drupal for semantic search, recommendations, and content enrichment. Below are example PHP code snippets for common operations using Guzzle HTTP client. For production, generate a typed client from the OpenAPI spec.

### Store a Vector from Drupal

```php
use GuzzleHttp\Client;
$client = new Client(['base_uri' => 'http://mcp-server-host']);
$response = $client->post('/api/v1/vector/store', [
    'json' => [
        'id' => 'drupal-node-123',
        'vector' => [0.1, 0.2, 0.3, 0.4],
        'metadata' => ['entity_type' => 'node', 'bundle' => 'article']
    ],
    'headers' => [
        'X-API-Key' => 'your-api-key'
    ]
]);
$data = json_decode($response->getBody(), true);
```

### Search for Similar Content

```php
$response = $client->post('/api/v1/vector/search', [
    'json' => [
        'query' => [0.1, 0.2, 0.3, 0.4],
        'topK' => 5
    ],
    'headers' => [
        'X-API-Key' => 'your-api-key'
    ]
]);
$results = json_decode($response->getBody(), true);
foreach ($results as $result) {
    // $result['id'], $result['score'], $result['metadata']
}
```

### Retrieve a Vector by ID

```php
$response = $client->get('/api/v1/vector/get', [
    'query' => ['id' => 'drupal-node-123'],
    'headers' => [
        'X-API-Key' => 'your-api-key'
    ]
]);
$vector = json_decode($response->getBody(), true);
```

### Delete a Vector

```php
$response = $client->delete('/api/v1/vector/delete', [
    'query' => ['id' => 'drupal-node-123'],
    'headers' => [
        'X-API-Key' => 'your-api-key'
    ]
]);
$data = json_decode($response->getBody(), true);
```

### List All Vectors

```php
$response = $client->get('/api/v1/vector/list', [
    'headers' => [
        'X-API-Key' => 'your-api-key'
    ]
]);
$list = json_decode($response->getBody(), true);
```

**Tip:** For strict typing and easier integration, generate a PHP client from the OpenAPI spec using [OpenAPI Generator](https://openapi-generator.tech/).
