# LLM Platform MCP Integration

This document describes the integration between the LLM Platform Recipe and the Model Control Plane (MCP), enabling seamless installation, configuration, and management of LLM Platform environments through the MCP API.

## Overview

The LLM Platform MCP Integration provides a robust solution for:

1. **Environment-Aware Installation**: Intelligently detects and adapts to the environment (local, DDEV, staging, production)
2. **DDEV Integration**: Seamlessly integrates with DDEV for local development environments
3. **API-Driven Management**: Complete REST API for managing LLM Platform installations
4. **MCP Tool Integration**: Register LLM Platform installer as an MCP tool for workflow automation
5. **Cross-Environment Orchestration**: Manage installations across multiple environments

## Architecture

The integration consists of several components:

- **LLM Platform Installer Tool**: A registered MCP tool for installing and configuring LLM Platform
- **LLM Platform Service**: A service that manages LLM Platform installations and their configurations
- **REST API Endpoints**: HTTP endpoints for managing installations
- **Integration Module**: Registers the service and tool with the MCP server

```
┌───────────────────────────────────────────────────────┐
│                   MCP Server                          │
│                                                       │
│  ┌───────────────┐      ┌───────────────────────┐    │
│  │ Tool Registry  │◄────►│ LLM Platform Tool    │    │
│  └───────────────┘      └───────────────────────┘    │
│         ▲                         ▲                  │
│         │                         │                  │
│         ▼                         ▼                  │
│  ┌───────────────┐      ┌───────────────────────┐    │
│  │ Service Mgr   │◄────►│ LLM Platform Service  │    │
│  └───────────────┘      └───────────────────────┘    │
│         ▲                         ▲                  │
│         │                         │                  │
│         ▼                         ▼                  │
│  ┌───────────────┐      ┌───────────────────────┐    │
│  │ REST API      │◄────►│ LLM Platform API      │    │
│  └───────────────┘      └───────────────────────┘    │
└───────────────────────────────────────────────────────┘
                            ▲
                            │
                            ▼
┌─────────────────────────────────────────────────────┐
│                                                     │
│             LLM Platform Installations              │
│                                                     │
│  ┌──────────────┐  ┌───────────────┐  ┌──────────┐  │
│  │ Local (DDEV) │  │ Staging       │  │ Production│  │
│  └──────────────┘  └───────────────┘  └──────────┘  │
└─────────────────────────────────────────────────────┘
```

## Features

### Environment Detection

The integration automatically detects the environment and adapts the installation process:

- **Local Development**: Configures with DDEV integration
- **DDEV Environment**: Optimizes for existing DDEV setup
- **Staging**: Balances between performance and debugging
- **Production**: Optimizes for performance, security, and stability

### DDEV Integration

For local development with DDEV:

- Detects existing DDEV configurations
- Creates optimal DDEV configuration if needed
- Sets up Docker services for vector databases
- Configures environment-specific settings
- Provides service management (start, stop)

### REST API

Complete REST API for managing LLM Platform installations:

- `POST /api/llm-platform/install`: Install LLM Platform
- `GET /api/llm-platform/status/:path`: Get installation status
- `GET /api/llm-platform/list`: List all managed installations
- `POST /api/llm-platform/ddev/start`: Start DDEV services
- `POST /api/llm-platform/ddev/stop`: Stop DDEV services
- `POST /api/llm-platform/config/provider`: Update AI provider

## Using the Integration

### Via the REST API

Install LLM Platform in a specific directory:

```bash
curl -X POST http://localhost:3001/api/llm-platform/install \
  -H "Content-Type: application/json" \
  -d '{
    "targetDir": "/path/to/installation",
    "siteName": "My LLM Platform",
    "detectEnvironment": true,
    "useDdev": true,
    "modules": ["langchain", "vector_database", "ai_swarm_intelligence"],
    "aiProvider": "gemini"
  }'
```

Check installation status:

```bash
curl -X GET http://localhost:3001/api/llm-platform/status/path/to/installation
```

List all managed installations:

```bash
curl -X GET http://localhost:3001/api/llm-platform/list
```

### Via the MCP Tool

You can execute the LLM Platform Installer tool using the MCP protocol:

```json
{
  "jsonrpc": "2.0",
  "id": 1,
  "method": "call-tool",
  "params": {
    "name": "llm-platform-installer",
    "arguments": {
      "targetDir": "/path/to/installation",
      "siteName": "My LLM Platform",
      "detectEnvironment": true,
      "useDdev": true,
      "modules": ["langchain", "vector_database", "ai_swarm_intelligence"],
      "aiProvider": "gemini"
    }
  }
}
```

### Via the JavaScript/TypeScript API

Using the MCP client library:

```typescript
import [McpClient] from '@bluefly/mcp-client';

// Create MCP client
const client = new McpClient('http://localhost:3001');

// Install LLM Platform
const result = await client.callTool('llm-platform-installer', {
  targetDir: '/path/to/installation',
  siteName: 'My LLM Platform',
  detectEnvironment: true,
  useDdev: true,
  modules: ['langchain', 'vector_database', 'ai_swarm_intelligence'],
  aiProvider: 'gemini'
});

console.log(result);
```

## Configuration Options

The LLM Platform installer accepts the following parameters:

| Parameter | Type | Description | Default |
|-----------|------|-------------|---------|
| targetDir | string | Target directory for installation | (required) |
| siteName | string | Site name for the installation | "LLM Platform" |
| detectEnvironment | boolean | Auto-detect environment settings | true |
| useDdev | boolean | Use DDEV for local development | true |
| ddevName | string | DDEV project name if using DDEV | "llm-platform" |
| modules | string[] | Modules to install | ["langchain", "vector_database", "ai_swarm_intelligence"] |
| aiProvider | string | AI provider to use (gemini, openai, anthropic, local) | "gemini" |
| apiKey | string | API key for the selected provider | "" |
| isDemo | boolean | Set up as a demo environment | false |
| setupVectorServices | boolean | Set up vector database services | true |

## Extending the Integration

You can extend the integration with additional capabilities:

1. **Additional Tools**: Create new MCP tools for specific LLM Platform operations
2. **Custom Service Logic**: Extend the LLM Platform service with additional features
3. **New API Endpoints**: Add custom API endpoints for specialized operations
4. **Environment Customization**: Add support for additional environments

## Troubleshooting

Common issues and solutions:

1. **Tool Registration Failures**: Ensure the MCP server is running and the tool definition is valid
2. **Installation Errors**: Check file permissions in the target directory
3. **DDEV Integration Issues**: Verify DDEV is installed and available in PATH
4. **API Communication Errors**: Check network connectivity and API endpoint URLs
5. **Environment Detection Problems**: Manually specify environment parameters

## Deployment

When deploying the integrated MCP server:

1. Ensure the LLM Platform installer tool is properly registered
2. Configure appropriate permission levels for the API endpoints
3. Set up security for API key storage and management
4. Configure environment-specific settings for optimal performance
5. Implement monitoring for installation status and health