# Project Integration Guide

This document outlines how AI agents should handle new project types, plugins, and integrations within the LLM Platform AI Platform ecosystem.

## 🏗️ Understanding Project Types

The LLM Platform AI Platform consists of several distinct project types, each with specific purposes and integration patterns:

### Core Service Projects

- **API Gateway (bfapi)**: Central API entry point with JWT auth, load balancing
- **LLM Engine (bfllm)**: Inference engine for language models
- **MCP Server (llm-mcp)**: Model Context Protocol orchestration
- **Content Engine (bfrfp)**: Document generation and analysis

### Supporting Projects

- **UI Components (llm-ui)**: React component library
- **CLI Tools (llmcli)**: Command-line interface for platform operations
- **CI Components (bfcicomponents)**: GitLab CI integration components
- **Documentation (bfdocs)**: Consolidated documentation system

### Drupal Integration

- **Platform Recipe (llm_platform_recipe)**: Complete Drupal integration recipe
- **Native AI JS (native_aijs)**: JavaScript framework for AI integration

## 🔄 Integration Patterns

When handling new integrations, follow these established patterns:

### 1. Service-to-Service Integration

Services communicate via standardized REST APIs defined with OpenAPI:

```javascript
// Example: Calling bfllm from bfapi
const response = await axios.post(
  `[process.env.BFLLM_URL]/api/v1/models/[modelId]/predict`,
  requestPayload,
  {
    headers: {
      'Authorization': `Bearer [serviceToken]`,
      'Content-Type': 'application/json'
    }
  }
);
```

### 2. MCP Integration

Services integrate with the Model Context Protocol (MCP) for tool orchestration:

```javascript
// Registering a tool with MCP
await mcpClient.registerTool({
  name: 'content_search',
  description: 'Search for content across the platform',
  input_schema: {
    type: 'object',
    properties: {
      query: { type: 'string' },
      limit: { type: 'number', default: 10 }
    },
    required: ['query']
  },
  output_schema: {
    type: 'object',
    properties: {
      results: {
        type: 'array',
        items: { /* ... */ }
      }
    }
  },
  execute: async (input) => [// Implementation]
});
```

### 3. Drupal Module Integration

Drupal modules integrate via service providers and module hooks:

```php
// Example: services.yml definition
services:
  llm_platform_recipe.ai_service:
    class: Drupal\llm_platform_recipe\Service\AIService
    arguments: ['@http_client', '@config.factory', '@logger.channel.llm_platform']
```

```php
// Example: Hook implementation
function llm_platform_recipe_node_presave(EntityInterface $entity) [// Integration logic]
```

##  Adding New Project Types

When adding a new project type, follow these steps:

### 1. Project Structure Setup

```bash
# Create new project directory
mkdir new_project_name

# Initialize package.json
cd new_project_name
npm init -y

# Set up TypeScript
npm install --save-dev typescript @types/node
npx tsc --init

# Set up testing
npm install --save-dev jest @types/jest ts-jest
npx ts-jest config:init

# Create basic structure
mkdir -p src/__tests__ config docs
```

### 2. Configuration Integration

Each project should use the shared configuration system:

```javascript
// Import shared configuration
const [getConfig] = require('../config/shared-config');

// Get project-specific configuration
const config = getConfig('new_project_name');
```

### 3. OpenAPI Integration

For service projects, define an OpenAPI specification:

```bash
# Create OpenAPI specification
touch openapi.yaml

# Set up validation
npm install --save-dev @stoplight/spectral-cli
```

### 4. Documentation Integration

Add documentation that will be aggregated by bfdocs:

```markdown
# New Project Name

**Description**: Brief description of the project's purpose

## Features

- Feature 1
- Feature 2

## Integration

How this project integrates with other components...
```

## 🔌 Plugin Architecture

The platform uses several plugin architectures:

### 1. llmcli Command Plugins

```javascript
// Define a new llmcli command plugin
module.exports = {
  name: 'new-command',
  description: 'Description of the new command',
  execute: async (args, options) => {
    // Command implementation
  }
};
```

### 2. bfllm Model Plugins

```typescript
// Define a new model plugin for bfllm
import [ModelPlugin] from '@bluefly/llm-core';

export class NewModelPlugin implements ModelPlugin {
  id = 'new-model-plugin';
  name = 'New Model Plugin';
  
  async initialize(): Promise<void> {
    // Initialization logic
  }
  
  async generateText(prompt: string, options: any): Promise<string> [// Text generation implementation]
}
```

### 3. llm-mcp Tool Plugins

```typescript
// Define a new tool plugin for llm-mcp
import [ToolPlugin] from '@bluefly/mcp-core';

export class NewToolPlugin implements ToolPlugin {
  id = 'new-tool';
  name = 'New Tool';
  description = 'Description of the new tool';
  
  inputSchema = {
    type: 'object',
    properties: {
      // Input parameters
    }
  };
  
  outputSchema = {
    type: 'object',
    properties: {
      // Output format
    }
  };
  
  async execute(input: any): Promise<any> [// Tool implementation]
}
```

### 4. Drupal Plugin Integration

```php
/**
 * Implements hook_plugin_type_info().
 */
function llm_platform_recipe_plugin_type_info() {
  return [
    'llm_processor' => [
      'label' => t('LLM Processor'),
      'provider' => 'llm_platform_recipe',
      'plugin_manager_service_id' => 'plugin.manager.llm_processor',
    ],
  ];
}
```

## 🔗 Extension Points

The platform provides several extension points for agents:

### 1. Service Extension Points

| Service | Extension Point | Purpose |
|---------|----------------|---------|
| bfapi | Middleware | Add request/response processing logic |
| bfapi | Routes | Add new API endpoints |
| bfllm | Models | Add new model implementations |
| bfllm | Agents | Add new agent capabilities |
| llm-mcp | Tools | Add new tool implementations |
| llm-mcp | Resources | Add new resource types |
| bfrfp | Templates | Add new document templates |
| bfrfp | Processors | Add new content processors |

### 2. Client Extension Points

| Client | Extension Point | Purpose |
|--------|----------------|---------|
| llm-ui | Components | Add new React components |
| llm-ui | Hooks | Add new React hooks |
| llmcli | Commands | Add new CLI commands |
| llmcli | Plugins | Add new CLI plugins |

### 3. Drupal Extension Points

| Module | Extension Point | Purpose |
|--------|----------------|---------|
| llm_platform_recipe | Hooks | Add new Drupal hook implementations |
| llm_platform_recipe | Plugins | Add new plugin types |
| llm_platform_recipe | Services | Add new service implementations |
| llm_platform_recipe | Forms | Add new configuration forms |

## 📋 Integration Checklist

When integrating new components, follow this checklist:

- [ ] **Configuration**: Add configuration to the shared configuration system
- [ ] **OpenAPI**: Define OpenAPI specification for any new APIs
- [ ] **Testing**: Add comprehensive tests for the new component
- [ ] **Documentation**: Add documentation for the new component
- [ ] **CI/CD**: Update CI/CD configuration for the new component
- [ ] **Security**: Ensure proper authentication and authorization
- [ ] **Monitoring**: Add appropriate monitoring and logging
- [ ] **Dependencies**: Update dependencies in package.json/composer.json

## 🧪 Testing Integration

Each project type has specific testing requirements:

### Service Projects

```typescript
// Example: Service integration test
describe('Service Integration', () => {
  it('should integrate with dependent service', async () => {
    // Test implementation
  });
});
```

### Drupal Integration

```php
// Example: Drupal module integration test
namespace Drupal\Tests\llm_platform_recipe\Kernel;

class IntegrationTest extends KernelTestBase {
  protected static $modules = ['llm_platform_recipe', 'dependent_module'];
  
  public function testIntegration() {
    // Test implementation
  }
}
```

##  Example Integrations

### 1. Adding a New LLM Model

```typescript
// Import base classes
import [BaseModel] from '@bluefly/llm-core';

// Create new model class
export class NewModel extends BaseModel {
  constructor(config) {
    super(config);
    // Initialization
  }
  
  async initialize() [// Load model resources]
  
  async generate(prompt, options) [// Generate text using the model]
}

// Register the model
export function registerModel(registry) {
  registry.register('new-model', (config) => new NewModel(config));
}
```

### 2. Adding a New API Endpoint

```typescript
// Import express and middleware
import express from 'express';
import { authenticate, validate } from '../middleware';

// Create router
const router = express.Router();

// Define endpoint with OpenAPI validation
router.post('/api/v1/new-feature',
  authenticate,
  validate('NewFeatureRequest'),
  async (req, res) => {
    try {
      // Implementation
      res.json({ success: true, data: result });
    } catch (error) {
      res.status(500).json({ success: false, error: error.message });
    }
  }
);

// Export router
export default router;
```

### 3. Adding a New Drupal Plugin

```php
<?php

namespace Drupal\llm_platform_recipe\Plugin\LLMProcessor;

use Drupal\llm_platform_recipe\Plugin\LLMProcessorBase;

/**
 * @LLMProcessor(
 *   id = "new_processor",
 *   label = @Translation("New Processor"),
 *   description = @Translation("Description of the new processor")
 * )
 */
class NewProcessor extends LLMProcessorBase [/**
   * {@inheritdoc]
   */
  public function process($content, array $options = []) {
    // Implementation
    return $processed_content;
  }

}
```

## 🚧 Integration Troubleshooting

When troubleshooting integration issues:

1. **Check Configuration**: Verify shared configuration is loaded correctly
2. **Validate APIs**: Ensure OpenAPI contracts are properly implemented
3. **Review Logs**: Check service logs for detailed error information
4. **Test Dependencies**: Verify all dependent services are running
5. **Check Authentication**: Verify service-to-service authentication is working
6. **Validate Plugins**: Ensure plugins are properly registered and loaded

##  Further Reading

- [Service Architecture Documentation](docs/architecture.md)
- [Plugin Development Guide](docs/plugin-development.md)
- [Drupal Integration Guide](docs/drupal-integration.md)
- [OpenAPI Guidelines](OPENAPI_GUIDELINES.md)
- [TDD Implementation Plan](TDD_IMPLEMENTATION_PLAN.md)

---

This guide provides an overview of how to handle new project types, plugins, and integrations within the LLM Platform AI Platform ecosystem. For specific implementation details, refer to the individual project documentation.