# WebSocket Client

A TypeScript package for WebSocket connections with advanced features and HTTP polling fallback.

## Features

- Robust WebSocket connection management
- Automatic reconnection with exponential backoff
- HTTP polling fallback for environments without WebSocket support
- Channel-based messaging system
- Message queuing for disconnected periods
- Event-based API
- Connection status tracking and metrics
- OpenTelemetry integration for monitoring
- TypeScript support with comprehensive type definitions
- Framework-agnostic design

## Installation

```bash
npm install @llm-mcp/websocket-client
```

## Basic Usage

```typescript
import [createConnection] from '@llm-mcp/websocket-client';

// Create a connection
const connection = createConnection({
  url: 'wss://example.com/websocket',
  token: 'your-auth-token', // Optional
  autoReconnect: true,
  maxReconnectAttempts: 5
});

// Listen for connection events
connection.on('connected', () => {
  console.log('Connected to WebSocket server!');
});

connection.on('disconnected', () => {
  console.log('Disconnected from WebSocket server');
});

// Send a message
connection.send('hello', { message: 'Hello, server!' })
  .then(response => {
    console.log('Message sent successfully');
  })
  .catch(error => {
    console.error('Failed to send message', error);
  });

// Listen for messages
connection.on('message:welcome', (data) => {
  console.log('Received welcome message:', data);
});
```

## Connection Manager

For more advanced usage, the package provides a ConnectionManager that can handle multiple connections:

```typescript
import [createManager] from '@llm-mcp/websocket-client';

// Create a connection manager
const manager = createManager({
  url: 'wss://example.com/websocket',
  token: 'your-auth-token',
  clientType: 'web-client',
  capabilities: ['chat', 'notifications']
});

// Connect to multiple servers
manager.connect('wss://server1.example.com/websocket');
manager.connect('wss://server2.example.com/websocket');

// Subscribe to channels
manager.subscribeToChannel('chat');
manager.subscribeToChannel('notifications');

// Listen for channel-specific events
manager.on('message', (data) => {
  console.log('Received chat message:', data);
}, 'chat');

manager.on('notification', (data) => {
  console.log('Received notification:', data);
}, 'notifications');

// Send a message on a specific channel
manager.send('message', { text: 'Hello, everyone!' }, 'chat');
```

## HTTP Polling Fallback

The package automatically falls back to HTTP polling when WebSocket is not available:

```typescript
import [createConnection] from '@llm-mcp/websocket-client';

// Create a connection with polling fallback
const connection = createConnection({
  url: 'https://example.com',
  pollingEndpoint: '/api/polling',
  pollingInterval: 5000, // 5 seconds
  enablePolling: true // Force polling even if WebSocket is available
});

// The API is the same regardless of the connection type
connection.on('message:update', (data) => {
  console.log('Received update:', data);
});

connection.send('request', { type: 'data' });
```

## Advanced Configuration

The package supports many configuration options:

```typescript
import [createManager] from '@llm-mcp/websocket-client';

const manager = createManager({
  // Connection options
  url: 'wss://example.com/websocket',
  token: 'your-auth-token',
  protocol: 'custom-protocol',
  
  // Reconnection options
  autoReconnect: true,
  maxReconnectAttempts: 5,
  reconnectDelay: 1000,
  
  // Polling options
  enablePolling: true,
  pollingEndpoint: '/api/polling',
  pollingInterval: 5000,
  
  // Client identification
  clientType: 'web-client',
  capabilities: ['chat', 'notifications'],
  channelPrefix: 'app',
  
  // Debugging
  logLevel: 'debug',
  showStatus: true,
  
  // Performance
  enableCompression: true,
  
  // Socket options
  socketOptions: {
    // Any options to pass to the WebSocket constructor
  }
});
```

## API Reference

### Connection Classes

- `WebSocketConnection`: Core WebSocket connection implementation
- `PollingConnection`: HTTP polling fallback implementation
- `ConnectionManager`: Manages multiple connections

### Helper Functions

- `createConnection`: Creates a connection with automatic fallback
- `createManager`: Creates a connection manager

### Events

- `connected`: Emitted when connection is established
- `disconnected`: Emitted when connection is closed
- `status_change`: Emitted when connection status changes
- `message`: Emitted for all messages
- `message:[event]`: Emitted for specific event types
- `message:[channel]:[event]`: Emitted for channel-specific events
- `error`: Emitted on connection errors
- `registered`: Emitted when registration with server is complete
- `server_info`: Emitted when server info is received

## Utilities

### Logger

```typescript
import [createLogger] from '@llm-mcp/websocket-client';

const logger = createLogger('info');

logger.info('Connection established');
logger.warn('Connection unstable');
logger.error('Connection failed', { reason: 'timeout' });
```

### Telemetry

```typescript
import [Telemetry] from '@llm-mcp/websocket-client';

// Record metrics
Telemetry.recordMessageSent('chat', 'default', 'client-123');
Telemetry.recordConnectionDuration(5000, 'wss://example.com', 'client-123');

// Create spans
const span = Telemetry.startSpan('message-processing');
try {
  // Process message
  Telemetry.endSpan(span);
} catch (error) {
  Telemetry.endSpanWithError(span, error);
}
```

## TypeScript Support

The package is written in TypeScript and includes comprehensive type definitions:

```typescript
import { 
  ConnectionOptions, 
  ConnectionState,
  WebSocketMessage 
} from '@llm-mcp/websocket-client';

// Use types in your code
const options: ConnectionOptions = {
  url: 'wss://example.com/websocket',
  token: 'your-auth-token'
};

// Check connection state
if (connection.getState() === ConnectionState.CONNECTED) [// Connection is ready]

// Handle message with typed data
connection.on<{ id: string; text: string }>('chat', (data) => {
  console.log(`Message [data.id]: [data.text]`);
});
```

## License

MIT