# API Endpoint Name

Brief description of the endpoint and its purpose. This section should clearly explain what the endpoint does and when to use it.

## Endpoint

`METHOD /path/to/endpoint`

## Description

Detailed description of what this endpoint does and when to use it. Include information about:

- What the endpoint is used for
- When to use this endpoint
- Any important considerations or limitations
- Authentication requirements
- Rate limiting information
- Versioning details

## Request

### Headers

| Header | Description | Required |
|--------|-------------|----------|
| `Authorization` | API key or token in format `Bearer YOUR_API_KEY` | Yes |
| `Content-Type` | Must be `application/json` | Yes |
| `Accept` | Response format, defaults to `application/json` | No |

### URL Parameters

| Parameter | Type | Description | Required |
|-----------|------|-------------|----------|
| `param1` | string | Description of param1 | Yes |
| `param2` | number | Description of param2 | No |

### Query Parameters

| Parameter | Type | Description | Default | Required |
|-----------|------|-------------|---------|----------|
| `filter` | string | Filter results by criteria | none | No |
| `limit` | number | Maximum number of results | 20 | No |
| `offset` | number | Result offset for pagination | 0 | No |

### Request Body

```json
{
  "field1": "value1",
  "field2": 123,
  "field3": {
    "nestedField": "value"
  },
  "field4": [
    "array",
    "of",
    "values"
  ]
}
```

#### Request Body Fields

| Field | Type | Description | Required |
|-------|------|-------------|----------|
| `field1` | string | Description of field1 | Yes |
| `field2` | number | Description of field2 | Yes |
| `field3` | object | Nested object with additional data | No |
| `field3.nestedField` | string | Description of nestedField | Yes (if field3 is provided) |
| `field4` | array | Array of string values | No |

## Response

### Success Response

**Status Code**: `200 OK`

```json
{
  "id": "example-id",
  "status": "success",
  "data": {
    "field1": "value1",
    "field2": 123,
    "createdAt": "2025-05-30T12:34:56.789Z",
    "items": [
      {
        "id": "item-1",
        "name": "First Item"
      },
      {
        "id": "item-2",
        "name": "Second Item"
      }
    ]
  }
}
```

#### Response Fields

| Field | Type | Description |
|-------|------|-------------|
| `id` | string | Unique identifier for the request |
| `status` | string | Status of the request (`success` or `error`) |
| `data` | object | Response data object |
| `data.field1` | string | Description of field1 |
| `data.field2` | number | Description of field2 |
| `data.createdAt` | string | ISO 8601 timestamp of creation |
| `data.items` | array | Array of item objects |
| `data.items[].id` | string | Unique identifier for the item |
| `data.items[].name` | string | Name of the item |

### Error Responses

#### Bad Request

**Status Code**: `400 Bad Request`

```json
{
  "error": "InvalidParameters",
  "message": "Invalid value for field1",
  "details": {
    "field": "field1",
    "issue": "Must be a string with at least 3 characters"
  }
}
```

#### Unauthorized

**Status Code**: `401 Unauthorized`

```json
{
  "error": "Unauthorized",
  "message": "Invalid or missing API key"
}
```

#### Forbidden

**Status Code**: `403 Forbidden`

```json
{
  "error": "Forbidden",
  "message": "You do not have permission to access this resource"
}
```

#### Not Found

**Status Code**: `404 Not Found`

```json
{
  "error": "NotFound",
  "message": "The requested resource could not be found"
}
```

#### Too Many Requests

**Status Code**: `429 Too Many Requests`

```json
{
  "error": "RateLimitExceeded",
  "message": "Rate limit exceeded. Try again in 30 seconds",
  "retryAfter": 30
}
```

#### Server Error

**Status Code**: `500 Internal Server Error`

```json
{
  "error": "ServerError",
  "message": "An unexpected error occurred",
  "requestId": "req-123456"
}
```

## Examples

### Example Request

```bash
curl -X POST "https://api.example.com/v1/endpoint" \
  -H "Authorization: Bearer YOUR_API_KEY" \
  -H "Content-Type: application/json" \
  -d '{
    "field1": "example",
    "field2": 123,
    "field3": {
      "nestedField": "value"
    }
  }'
```

### Example Response

```json
{
  "id": "example-id",
  "status": "success",
  "data": {
    "field1": "example",
    "field2": 123,
    "createdAt": "2025-05-30T12:34:56.789Z",
    "items": [
      {
        "id": "item-1",
        "name": "First Item"
      }
    ]
  }
}
```

### Node.js Example

```javascript
const axios = require('axios');

async function callEndpoint() {
  try {
    const response = await axios({
      method: 'POST',
      url: 'https://api.example.com/v1/endpoint',
      headers: {
        'Authorization': 'Bearer YOUR_API_KEY',
        'Content-Type': 'application/json'
      },
      data: {
        field1: 'example',
        field2: 123,
        field3: {
          nestedField: 'value'
        }
      }
    });
    
    console.log('Response:', response.data);
    return response.data;
  } catch (error) {
    console.error('Error:', error.response ? error.response.data : error.message);
    throw error;
  }
}
```

## Rate Limits

This endpoint is subject to the following rate limits:

- 100 requests per minute per API key
- 1000 requests per day per API key

## See Also

- [Related Endpoint 1](path/to/related-endpoint-1.md) - Brief description of related endpoint
- [Related Endpoint 2](path/to/related-endpoint-2.md) - Brief description of related endpoint
- [Guide](path/to/guide.md) - Guide that explains how to use this endpoint