# Example: Title of Example

This document demonstrates how to [brief description of what the example shows]. It provides practical code examples that you can use as a reference for your own implementation.

## Table of Contents

- [Overview](#overview)
- [Prerequisites](#prerequisites)
- [Example 1: Basic Implementation](#example-1-basic-implementation)
- [Example 2: Advanced Implementation](#example-2-advanced-implementation)
- [Example 3: Common Use Case](#example-3-common-use-case)
- [Complete Working Example](#complete-working-example)
- [Best Practices](#best-practices)
- [See Also](#see-also)

## Overview

Brief overview of what this example covers and why it's useful. Explain the specific problem or use case that these examples address. This section should be 1-2 paragraphs.

## Prerequisites

Before using these examples, ensure you have:

- LLM-MCP server running (version X.Y.Z or higher)
- Node.js (version X or higher)
- @llm-mcp/client package installed
- Appropriate API credentials/keys

## Example 1: Basic Implementation

This example shows the simplest way to implement [feature/functionality].

### Code Example

```typescript
// Import required modules
import [McpClient] from '@llm-mcp/client';

// Initialize client
const client = new McpClient({
  url: 'http://localhost:3001',
  apiKey: 'your-api-key'
});

// Basic implementation
async function basicExample() {
  try {
    // Example code demonstrating the basic functionality
    const result = await client.someMethod({
      param1: 'value1',
      param2: 'value2'
    });
    
    console.log('Result:', result);
    return result;
  } catch (error) {
    console.error('Error:', error);
    throw error;
  }
}

// Execute the example
basicExample();
```

### Explanation

This example demonstrates:
1. **Client Initialization**: Creating a basic LLM-MCP client
2. **Method Call**: Calling a specific method with parameters
3. **Result Handling**: Processing the result and handling errors

## Example 2: Advanced Implementation

This example shows a more advanced implementation with additional features.

### Code Example

```typescript
// Import required modules
import { McpClient, AdvancedOptions } from '@llm-mcp/client';

// Advanced configuration
const advancedOptions: AdvancedOptions = {
  timeout: 5000,
  retries: 3,
  logging: true,
  cache: {
    enabled: true,
    ttl: 300 // 5 minutes
  }
};

// Initialize client with advanced options
const client = new McpClient({
  url: 'http://localhost:3001',
  apiKey: 'your-api-key',
  options: advancedOptions
});

// Advanced implementation
async function advancedExample() {
  try {
    // Example code demonstrating advanced functionality
    const result = await client.advancedMethod({
      complexParam: {
        nestedField1: 'value1',
        nestedField2: 42,
        nestedArray: ['item1', 'item2']
      },
      callback: (data) => {
        console.log('Callback received:', data);
      }
    });
    
    console.log('Advanced result:', result);
    return result;
  } catch (error) {
    console.error('Advanced error:', error);
    throw error;
  }
}
```

### Explanation

This example demonstrates:
1. **Advanced Configuration**: Setting up the client with advanced options
2. **Complex Parameters**: Passing nested objects and arrays
3. **Callbacks**: Using callback functions for event handling
4. **Enhanced Error Handling**: More detailed error management

## Example 3: Common Use Case

This example demonstrates a common real-world use case: [description of use case].

### Code Example

```typescript
// Import required modules
import [McpClient] from '@llm-mcp/client';

// Initialize client
const client = new McpClient({
  url: 'http://localhost:3001',
  apiKey: 'your-api-key'
});

// Implementation for common use case
async function realWorldExample() {
  try {
    // Step 1: First operation
    const firstResult = await client.firstOperation({
      // Parameters
    });
    
    // Step 2: Process intermediate results
    const processedData = processResults(firstResult);
    
    // Step 3: Second operation using processed data
    const finalResult = await client.secondOperation({
      data: processedData,
      options: {
        // Additional options
      }
    });
    
    console.log('Final result:', finalResult);
    return finalResult;
  } catch (error) {
    console.error('Error in real-world example:', error);
    throw error;
  }
}

// Helper function to process results
function processResults(data) {
  // Process the data
  return {
    // Processed data
  };
}
```

### Explanation

This example demonstrates:
1. **Multi-step Process**: Performing multiple operations in sequence
2. **Data Processing**: Processing intermediate results
3. **Real-world Application**: Solving a common use case

## Complete Working Example

This is a complete, self-contained example that you can copy and run directly.

```typescript
import [McpClient] from '@llm-mcp/client';

async function completeExample() {
  // Initialize client
  const client = new McpClient({
    url: process.env.LLM-MCP_URL || 'http://localhost:3001',
    apiKey: process.env.LLM-MCP_API_KEY || 'your-api-key'
  });
  
  try {
    // Step 1: Initialize resources
    console.log('Initializing resources...');
    
    // Step 2: Perform main operation
    console.log('Performing operation...');
    const result = await client.performOperation({
      // Parameters
    });
    
    // Step 3: Process and display results
    console.log('Operation successful!');
    console.log('Result:', JSON.stringify(result, null, 2));
    
    return result;
  } catch (error) {
    console.error('Error in complete example:', error);
    
    // Handle specific error types
    if (error.code === 'AUTHENTICATION_ERROR') {
      console.error('Please check your API key');
    } else if (error.code === 'CONNECTION_ERROR') {
      console.error('Please check your connection to the LLM-MCP server');
    }
    
    throw error;
  } finally {
    // Cleanup if necessary
    console.log('Example completed');
  }
}

// Run the example if this file is executed directly
if (require.main === module) {
  completeExample()
    .then(() => process.exit(0))
    .catch(() => process.exit(1));
}

// Export for use in other modules
export [completeExample];
```

## Best Practices

When implementing similar functionality, follow these best practices:

1. **Error Handling**: Always implement proper error handling
2. **Configuration Management**: Use environment variables for configuration
3. **Resource Cleanup**: Properly clean up resources when done
4. **Logging**: Implement appropriate logging for debugging
5. **Validation**: Validate inputs before making API calls
6. **Retries**: Implement retry logic for transient failures
7. **Security**: Never hardcode API keys or sensitive information

## See Also

- [API Reference for someMethod](../api/api-reference.md#somemethod) - Detailed API reference
- [Guide for This Feature](../guides/feature-guide.md) - Comprehensive guide on this feature
- [Troubleshooting](../troubleshooting/common-issues.md) - Help with common issues