# BFUI Integration Master Plan

## Overview

This document outlines a comprehensive approach to integrate the React-based `bfui` component library with Drupal projects using the UI Patterns module. The goal is to create a unified design system that can be used across both React applications and Drupal sites, ensuring visual consistency and developer efficiency.

## Current State Analysis

### bfui Component Library
- Modern React component library built with Chakra UI
- Extensive set of components for agent interfaces, dashboards, forms, and data visualization
- Currently isolated from Drupal ecosystem
- Uses TypeScript for type safety
- Storybook for component documentation

### Drupal Projects
- Multiple independent Drupal modules for LLM platform
- No standardized UI component system
- Potential for inconsistent UI across different modules
- Limited reuse of UI components

## Integration Strategy

We will use a four-phase approach to integrate bfui with Drupal UI Patterns:

### Phase 1: Architecture & Foundation
1. Create a bridge module called `bfui_patterns` to house UI Pattern definitions
2. Define a conversion system to transform React component props to UI Pattern fields
3. Establish shared design tokens (colors, spacing, typography) as variables
4. Develop pattern library organization structure
5. Create build pipeline for assets (CSS, JS)

### Phase 2: Core Components
1. Implement foundational UI Patterns from bfui (Alert, Button, Card, Form elements)
2. Create component documentation and examples
3. Develop Twig templates that mirror React component structure
4. Implement shared CSS classes and style system
5. Test components in Drupal context

### Phase 3: Advanced Components
1. Implement complex patterns (Dashboard, AgentStatus, Charts)
2. Develop JS-driven interactive components
3. Create component variants for different contexts
4. Build integration examples for Drupal entities and views
5. Implement responsive behavior patterns

### Phase 4: Optimization & Distribution
1. Optimize assets for production use
2. Create comprehensive documentation
3. Develop pattern usage guidelines
4. Implement automated testing
5. Create distribution mechanism for pattern updates

## Technical Architecture

### 1. Component Bridge Pattern

We'll use a "bridge pattern" architecture with these key elements:

```
bfui (React Components)
     │
     ▼
bfui_patterns_generator (Conversion Layer)
     │
     ▼
bfui_patterns (Drupal UI Patterns)
     │
     ▼
Drupal Applications
```

### 2. File Structure

```
bfui_patterns/
├── bfui_patterns.info.yml
├── bfui_patterns.module
├── bfui_patterns.libraries.yml
├── bfui_patterns.patterns.yml (Main patterns file)
├── components/
│   ├── alert/
│   │   ├── alert.patterns.yml
│   │   ├── pattern-alert.html.twig
│   │   └── css/
│   │       └── alert.css
│   ├── button/
│   │   ├── button.patterns.yml
│   │   ├── pattern-button.html.twig
│   │   └── css/
│   │       └── button.css
│   └── ...
├── js/
│   ├── bfui-core.js
│   └── components/
│       ├── interactive-dashboard.js
│       └── ...
└── examples/
    ├── views/
    ├── blocks/
    └── layouts/
```

### 3. Pattern Definition Approach

Each UI Pattern will be defined with:

1. **Pattern Identifier**: Matching the React component name
2. **Fields**: Corresponding to React props
3. **Variants**: Matching component variants
4. **Libraries**: Required CSS/JS assets
5. **Examples**: Usage examples in Drupal contexts

Example:

```yaml
# components/alert/alert.patterns.yml
alert:
  label: Alert
  description: Displays an alert/message to the user
  variants:
    info:
      label: Info
      description: Information alert
    warning:
      label: Warning
      description: Warning alert
    error:
      label: Error
      description: Error alert
    success:
      label: Success
      description: Success alert
  fields:
    status:
      type: string
      label: Status
      description: Alert status/type
      preview: info
    title:
      type: text
      label: Title
      description: Alert title text
      preview: Information
    description:
      type: text
      label: Description
      description: Alert description text
      preview: This is an informational alert.
    variant:
      type: string
      label: Variant
      description: Visual variant
      preview: subtle
  libraries:
    - bfui_patterns/alert
```

### 4. Pattern Template Approach

Templates will mirror the React component structure but use Twig syntax:

```twig
[# components/alert/pattern-alert.html.twig #]
<div class="bfui-alert bfui-alert--[status|default('info')] bfui-alert--[variant|default('subtle')]">
  <div class="bfui-alert__icon">
    [% if status == 'info' %]
      <svg class="bfui-icon">...</svg>
    [% elseif status == 'warning' %]
      <svg class="bfui-icon">...</svg>
    [% elseif status == 'error' %]
      <svg class="bfui-icon">...</svg>
    [% elseif status == 'success' %]
      <svg class="bfui-icon">...</svg>
    [% endif %]
  </div>
  
  [% if title %]
    <div class="bfui-alert__title">Documentation</div>
  [% endif %]
  
  [% if description %]
    <div class="bfui-alert__description">Project Description</div>
  [% endif %]
  
  [% if children %]
    <div class="bfui-alert__content">[children]</div>
  [% endif %]
</div>
```

### 5. CSS Architecture

We'll use a BEM-style naming convention for CSS classes:

```scss
.bfui-[component] [// Component base styles
  
  &--[variant] {
    // Variant styles]
  
  &__[element] [// Element styles]
}
```

CSS will be organized by component and compiled into a cohesive library.

### 6. JavaScript Integration

For interactive components, we'll use a JavaScript bridge:

```js
// js/components/interactive-dashboard.js
(function ($, Drupal) {
  'use strict';
  
  Drupal.behaviors.bfuiInteractiveDashboard = {
    attach: function (context, settings) {
      once('bfui-dashboard', '.bfui-dashboard', context).forEach(function (element) {
        // Initialize dashboard behavior
        const config = JSON.parse(element.dataset.config || '{}');
        // Apply interactive behaviors
      });
    }
  };
})(jQuery, Drupal);
```

## Implementation Roadmap

### Immediate Actions (1-2 weeks)

1. **Create base module structure**
   - Set up bfui_patterns module with core files
   - Create component directory structure
   - Establish build pipeline

2. **Implement foundational components**
   - Alert component
   - Button component
   - Form elements (Input, Select)
   - Typography components

3. **Develop CSS framework**
   - Define design tokens (colors, spacing, typography)
   - Create base CSS architecture
   - Implement responsive grid system

### Short-term (1 month)

1. **Expand component library**
   - Cards and layout components
   - Navigation elements
   - Data display components
   - Status indicators

2. **Create Drupal integration examples**
   - Field formatters
   - View modes
   - Block configurations
   - Layout builder examples

3. **Developer documentation**
   - Usage guides
   - Integration examples
   - Component API documentation

### Medium-term (2-3 months)

1. **Advanced component integration**
   - Dashboard components
   - Chart and data visualization
   - Agent components
   - Interactive forms

2. **Tooling improvements**
   - Component generator tool
   - Pattern validation
   - Visual regression testing

3. **Extended Drupal integration**
   - Paragraphs integration
   - Views integration
   - Custom block types
   - Field formatters

### Long-term (4-6 months)

1. **Ecosystem expansion**
   - Custom admin themes using patterns
   - Distribution packaging
   - Performance optimization
   - Accessibility improvements

2. **Maintenance and governance**
   - Version control strategy
   - Update mechanism
   - Community documentation

## Challenges and Considerations

1. **Styling consistency**
   - Drupal's default styles may conflict with component styles
   - Need for CSS isolation and specificity management

2. **JavaScript dependencies**
   - React components may rely on React's virtual DOM
   - Need for pure JS/jQuery alternatives for some functionality

3. **Build process integration**
   - Coordinating builds between React library and Drupal module
   - Managing versioning and dependencies

4. **Performance optimization**
   - Ensuring components load efficiently in Drupal context
   - Avoiding duplication of assets

5. **Maintenance overhead**
   - Keeping patterns in sync with React components
   - Managing updates across multiple projects

## Conclusion

This master plan provides a comprehensive approach to integrating the bfui React component library with Drupal using UI Patterns. By implementing this plan, we'll create a unified design system that can be used across both React and Drupal applications, ensuring visual consistency and improving developer efficiency.

The phased approach allows for incremental implementation and testing, while the technical architecture ensures a clean separation of concerns and maintainability over time.