# BFUI Integration Revised Plan

## Overview

This document outlines a revised approach to integrate the React-based `bfui` component library with Drupal projects using the UI Patterns module. The key difference from the initial plan is that this approach respects the isolation between projects and uses `llmcli` as the integration point.

## Current State Analysis

### Project Structure
- Multiple independent projects with no common files except llmcli
- Each project has its own build process and dependencies
- No direct code sharing mechanism between projects

### bfui Component Library
- Modern React component library built with Chakra UI
- Extensive set of components for agent interfaces, dashboards, forms, and data visualization
- Uses TypeScript for type safety
- Storybook for component documentation

### Drupal Projects
- Multiple independent Drupal modules for LLM platform
- No standardized UI component system
- Potential for inconsistent UI across different modules

## Integration Strategy

We will use llmcli as the integration point between the bfui React components and Drupal UI Patterns. The revised approach involves:

1. Creating a new llmcli command to generate UI Patterns from React components
2. Defining a pattern translation mechanism to convert React props to UI Pattern fields
3. Establishing a shared design token system via CSS variables
4. Implementing a documentation generation system

## Technical Architecture

### 1. Component Bridge via llmcli

```
bfui (React Components)
     │
     ▼
llmcli component-to-pattern converter
     │
     ▼
UI Patterns in individual Drupal projects
```

The key advantage of this approach is that we don't create new dependencies between projects. Instead, we use llmcli as a tool to generate UI Patterns that can be copied into the appropriate Drupal projects.

### 2. llmcli Command Structure

We'll create a new llmcli command:

```
llmcli ui-patterns generate [component] [options]
```

Options:
- `--output` - Directory to output the generated patterns
- `--theme` - Theme to use for the patterns (defaults to 'bfui')
- `--format` - Output format (yaml, yml, json)
- `--include-css` - Whether to include CSS files
- `--include-js` - Whether to include JS files

### 3. Translation Mechanism

The component-to-pattern translator will:

1. Parse React component TypeScript definitions
2. Extract prop types and JSDoc comments
3. Generate UI Pattern field definitions
4. Create Twig templates that mirror the React component structure
5. Extract CSS variables and styles
6. Generate appropriate library definitions

### 4. Pattern Generation Process

For each React component:

1. Analyze the TypeScript interface/type definition
2. Extract prop names, types, and documentation
3. Map React prop types to UI Pattern field types
4. Generate a YAML pattern definition
5. Generate a Twig template with equivalent markup
6. Extract CSS from the React component (if available)
7. Generate a CSS file with equivalent styles
8. Create a library definition for the pattern

### 5. Integration in Drupal Projects

For each Drupal project that wants to use UI Patterns:

1. Add UI Patterns as a dependency
2. Run the llmcli command to generate patterns for desired components
3. Copy the generated patterns into the project
4. Adjust as needed for project-specific requirements

## Implementation Plan

### Phase 1: llmcli Extension (1-2 weeks)

1. **Create llmcli command structure**
   - Define command interface and options
   - Implement command execution logic
   - Add help documentation

2. **Implement TypeScript parser**
   - Create parser for React component prop types
   - Extract JSDoc comments and type information
   - Handle complex types (unions, intersections, etc.)

3. **Develop UI Pattern generator**
   - Implement YAML pattern definition generator
   - Create Twig template generator
   - Develop CSS extractor and transformer

### Phase 2: Core Components (2-3 weeks)

1. **Implement generators for core components**
   - Alert component
   - Button component
   - Form elements
   - Typography components

2. **Create shared design token system**
   - Define CSS variables for colors, spacing, typography
   - Create transformer for Chakra UI theme to CSS variables
   - Implement responsive utility classes

3. **Develop documentation generator**
   - Create markdown documentation for each pattern
   - Generate examples and usage guidelines
   - Implement component showcase

### Phase 3: Integration Examples (1-2 weeks)

1. **Create integration examples for Drupal**
   - Example module with UI Patterns
   - Integration with Views
   - Integration with Field formatters
   - Integration with Block system

2. **Develop testing utilities**
   - Pattern validation tests
   - Visual regression testing
   - Accessibility testing

### Phase 4: Advanced Features (2-3 weeks)

1. **Implement advanced component generators**
   - Dashboard components
   - Agent components
   - Chart components
   - Interactive components

2. **Create asset optimization tools**
   - CSS optimization
   - JavaScript bundling
   - Image optimization

3. **Develop pattern synchronization**
   - Track changes in React components
   - Update UI Patterns based on changes
   - Version management

## Example Implementation: llmcli Command

Here's a sketch of the llmcli command implementation:

```typescript
// In llmcli/src/commands/ui-patterns.ts

import [Command] from '@oclif/command';
import [analyzeComponent] from '../lib/component-analyzer';
import [generatePattern] from '../lib/pattern-generator';
import [generateTwig] from '../lib/twig-generator';
import [extractStyles] from '../lib/style-extractor';

export default class GenerateUIPattern extends Command {
  static description = 'Generate UI Patterns from React components';

  static args = [
    { name: 'component', required: true, description: 'Component name or path' },
  ];

  static flags = {
    output: flags.string({ char: 'o', description: 'Output directory' }),
    theme: flags.string({ char: 't', description: 'Theme name', default: 'bfui' }),
    format: flags.string({ char: 'f', description: 'Output format', default: 'yml' }),
    includeCss: flags.boolean({ description: 'Include CSS files', default: true }),
    includeJs: flags.boolean({ description: 'Include JS files', default: false }),
  };

  async run() {
    const { args, flags } = this.parse(GenerateUIPattern);
    
    // 1. Analyze component
    const componentInfo = await analyzeComponent(args.component);
    
    // 2. Generate pattern definition
    const patternDefinition = generatePattern(componentInfo, {
      theme: flags.theme,
      format: flags.format,
    });
    
    // 3. Generate Twig template
    const twigTemplate = generateTwig(componentInfo);
    
    // 4. Extract styles
    const styles = flags.includeCss ? extractStyles(componentInfo) : null;
    
    // 5. Write files
    const outputDir = flags.output || './patterns';
    await writeFiles(outputDir, componentInfo.name, {
      pattern: patternDefinition,
      twig: twigTemplate,
      styles,
    });
    
    this.log(`Generated UI Pattern for [componentInfo.name] in [outputDir]`);
  }
}
```

## Example Pattern Definition

Here's an example of how the Alert component might be translated:

### React Component (TypeScript)

```tsx
// In bfui/src/components/Alert.tsx

export interface AlertProps {
  /** Alert status/type */
  status?: 'info' | 'warning' | 'success' | 'error';
  /** Alert title */
  title?: string;
  /** Alert description */
  description?: string;
  /** Visual variant */
  variant?: 'subtle' | 'solid' | 'left-accent' | 'top-accent';
  /** Whether to show an icon */
  showIcon?: boolean;
  /** Children elements */
  children?: React.ReactNode;
}

export const Alert: React.FC<AlertProps> = ({
  status = 'info',
  variant = 'subtle',
  title,
  description,
  showIcon = true,
  children,
}) => (
  <div className=[`alert alert--Status alert--[variant]`]>
    [showIcon && <div className="alert__icon">...</div>]
    <div className="alert__content">
      [title && <div className="alert__title">{title]</div>}
      [description && <div className="alert__description">{description]</div>}
      [children]
    </div>
  </div>
);
```

### Generated UI Pattern (YAML)

```yaml
# In <drupal-project>/patterns/alert/alert.ui_patterns.yml

alert:
  label: Alert
  description: Displays important messages to users with different severity levels.
  fields:
    status:
      type: string
      label: Status
      description: Alert status/type (info, warning, success, error)
      preview: info
    title:
      type: text
      label: Title
      description: Alert title text
      preview: Information
    description:
      type: text
      label: Description
      description: Alert description text
      preview: This is an informational alert.
    variant:
      type: string
      label: Variant
      description: Visual variant (subtle, solid, left-accent, top-accent)
      preview: subtle
    show_icon:
      type: boolean
      label: Show Icon
      description: Whether to display the status icon
      preview: true
  libraries:
    - <theme>/alert
  variants:
    info:
      label: Info
      description: Informational alert
    warning:
      label: Warning
      description: Warning alert
    error:
      label: Error
      description: Error alert
    success:
      label: Success
      description: Success alert
```

### Generated Twig Template

```twig
[# In <drupal-project>/patterns/alert/pattern-alert.html.twig #]

<div class="alert alert--[status|default('info')] alert--[variant|default('subtle')]">
  [% if show_icon %]
    <div class="alert__icon">
      [# Icon markup based on status #]
    </div>
  [% endif %]
  
  <div class="alert__content">
    [% if title %]
      <div class="alert__title">Documentation</div>
    [% endif %]
    
    [% if description %]
      <div class="alert__description">Project Description</div>
    [% endif %]
    
    [% if content %]
      <div class="alert__children">[content]</div>
    [% endif %]
  </div>
</div>
```

## Benefits of This Approach

1. **Respects Project Isolation**: No new dependencies between projects
2. **Flexible Integration**: Each project can choose which components to use
3. **Maintainable**: Changes to React components can be propagated via the tool
4. **Consistent**: Ensures visual consistency across platforms
5. **Extensible**: New components can be added easily
6. **Documented**: Generates documentation alongside the patterns

## Challenges and Considerations

1. **Manual Synchronization**: Changes in React components require manual regeneration
2. **Complex Props**: React components with complex props (functions, ReactNode) need special handling
3. **JavaScript Interactivity**: Some React components rely heavily on JavaScript interactivity
4. **Asset Management**: Managing CSS and JS assets across platforms
5. **Testing**: Ensuring patterns work correctly in Drupal context

## Next Steps

1. Implement the llmcli command structure
2. Create the component analyzer
3. Develop the pattern generator
4. Build the Twig template generator
5. Implement the style extractor
6. Test with a simple component
7. Refine and iterate
8. Expand to cover more complex components