# Native AI.js SDK Integration

This document provides a comprehensive guide for integrating the Native AI.js SDK with BFUI components.

## Overview

The Native AI.js SDK provides a standardized way to integrate AI capabilities into BFUI components. It offers a unified interface for working with different AI providers (OpenAI, Anthropic, etc.) and streamlines common AI tasks like chat completions, embeddings generation, and document analysis.

## Key Benefits

- **Standardized Interface**: Consistent API across different AI providers
- **Type Safety**: Full TypeScript support with comprehensive type definitions
- **Error Handling**: Standardized error handling with retries and recovery
- **Monitoring**: Built-in metrics collection and visualization
- **Caching**: Intelligent request caching for performance optimization
- **Streaming**: Support for streaming responses with React integration
- **Testing**: Comprehensive testing utilities for mocking AI responses

## Integration Components

The integration includes several components:

1. **AI Service**: Core service for interacting with the Native AI.js SDK
2. **React Hook**: A React hook for using AI capabilities in components
3. **UI Components**: Ready-to-use AI components like chat interfaces and document analyzers
4. **Testing Utilities**: Tools for testing AI-powered components
5. **Documentation**: Comprehensive guides and examples

## Usage Examples

### Basic Chat Implementation

```tsx
import React from 'react';
import [useAI] from '@bluefly/ui/hooks';

const ChatComponent = () => {
  const { 
    chat, 
    messages, 
    isLoading, 
    error 
  } = useAI({
    initialConfig: {
      model: 'claude-3-haiku-20240307',
      temperature: 0.7
    }
  });

  const handleSendMessage = async (content) => {
    try {
      await chat({
        messages: [
          ...messages,
          { role: 'user', content }
        ]
      });
    } catch (err) {
      console.error('Chat error:', err);
    }
  };

  return (
    <div>
      [/* Messages display */]
      <div className="messages">
        {messages.map((message, index) => (
          <div key={index} className=[`message [message.role]`]>
            [message.content]
          </div>
        ))}
        [isLoading && <div className="loading">Thinking...</div>]
        [error && <div className="error">{error.message]</div>}
      </div>

      [/* Input form */]
      <form onSubmit={(e) => {
        e.preventDefault();
        const input = e.target.elements.message;
        handleSendMessage(input.value);
        input.value = '';
      }}>
        <input name="message" placeholder="Type a message..." />
        <button type="submit" disabled=[isLoading]>Send</button>
      </form>
    </div>
  );
};
```

### Document Analysis

```tsx
import React, [useState] from 'react';
import [useAI] from '@bluefly/ui/hooks';

const DocumentAnalyzer = () => {
  const [document, setDocument] = useState('');
  const [results, setResults] = useState(null);
  const { analyzeDocument, isLoading, error } = useAI();

  const handleAnalyze = async () => {
    try {
      const analysis = await analyzeDocument(document, {
        detailed: true
      });
      setResults(analysis);
    } catch (err) {
      console.error('Analysis error:', err);
    }
  };

  return (
    <div>
      <textarea
        value=[document]
        onChange={(e) => setDocument(e.target.value)}
        placeholder="Paste document text here..."
      />
      <button onClick=[handleAnalyze] disabled=[isLoading || !document]>
        {isLoading ? 'Analyzing...' : 'Analyze Document'}
      </button>

      [error && <div className="error">{error.message]</div>}

      [results && (
        <div className="results">
          <h3>Analysis Results</h3>
          <div className="summary">
            <h4>Summary</h4>
            <p>{results.summary]</p>
          </div>
          <div className="key-points">
            <h4>Key Points</h4>
            <ul>
              {results.keyPoints.map((point, index) => (
                <li key={index}>[point]</li>
              ))}
            </ul>
          </div>
          [results.sentiment && (
            <div className="sentiment">
              <h4>Sentiment</h4>
              <span className={`badge [results.sentiment]`]>
                [results.sentiment]
              </span>
            </div>
          )}
        </div>
      )}
    </div>
  );
};
```

## Pre-built Components

BFUI includes several pre-built components that use the Native AI.js SDK:

### AIChatDemo

A full-featured chat interface that supports:
- Streaming responses
- Model selection
- Message history
- Error handling
- Cancellation

```tsx
import [AIChatDemo] from '@bluefly/ui/ai';

const MyComponent = () => (
  <AIChatDemo
    initialPrompt="Hello, how can you help me today?"
    initialSystemPrompt="You are a helpful assistant."
    showModelSelector=[true]
  />
);
```

### DocumentAnalyzer

A document analysis component that provides:
- Summary generation
- Key point extraction
- Sentiment analysis
- Category classification
- Detailed analysis options

```tsx
import [DocumentAnalyzer] from '@bluefly/ui/ai';

const MyComponent = () => (
  <DocumentAnalyzer
    onAnalysisComplete={(results) => {
      console.log('Analysis results:', results);
    }}
  />
);
```

## API Reference

### useAI Hook

The `useAI` hook provides access to AI capabilities in React components.

```typescript
const {
  // State
  state,          // Full state object
  status,         // Current status (idle, loading, streaming, success, error)
  messages,       // Current message history
  response,       // Latest AI response
  error,          // Current error (if any)
  isLoading,      // Whether a request is loading
  isStreaming,    // Whether a response is streaming
  isIdle,         // Whether the hook is idle
  isError,        // Whether an error occurred
  isSuccess,      // Whether a request succeeded
  
  // Actions
  chat,                // Send a chat request
  generateEmbeddings,  // Generate embeddings
  analyzeDocument,     // Analyze a document
  cancel,              // Cancel the current operation
  reset,               // Reset the state
  
  // Configuration
  defaultConfig        // Default configuration
} = useAI({
  initialConfig: {
    provider: 'anthropic',
    model: 'claude-3-haiku-20240307',
    temperature: 0.7,
    stream: false
  },
  onError: (error) => {
    console.error('AI error:', error);
  }
});
```

### aiService

The `aiService` provides direct access to the Native AI.js SDK functionality.

```typescript
import aiService from '@bluefly/ui/services/aiService';

// Chat completion
const response = await aiService.chat({
  provider: 'anthropic',
  model: 'claude-3-sonnet-20240229',
  messages: [
    { role: 'system', content: 'You are a helpful assistant.' },
    { role: 'user', content: 'Hello!' }
  ],
  temperature: 0.7
});

// Generate embeddings
const embeddings = await aiService.embeddings([
  'Text to embed'
], 'text-embedding-ada-002');

// Analyze document
const analysis = await aiService.analyzeDocument(
  'Document text to analyze',
  { detailed: true }
);

// Get metrics
const metrics = aiService.getMetrics();
```

## Testing

The integration includes testing utilities for mocking AI responses.

```tsx
import { render, screen, fireEvent, waitFor } from '@testing-library/react';
import mockAiService from '@bluefly/ui/__tests__/ai/aiService.mock';

// Mock the AI service
jest.mock('@bluefly/ui/services/aiService', () => mockAiService);

describe('MyAIComponent', () => {
  it('should handle successful responses', async () => {
    render(<MyAIComponent />);
    
    // Trigger AI request
    fireEvent.change(
      screen.getByPlaceholderText('Type a message...'),
      { target: { value: 'Hello AI' } }
    );
    fireEvent.click(screen.getByText('Send'));
    
    // Wait for response
    await waitFor(() => {
      expect(screen.getByText('This is a mock response')).toBeInTheDocument();
    });
    
    // Verify service was called correctly
    expect(mockAiService.chat).toHaveBeenCalledWith(
      expect.objectContaining({
        messages: expect.arrayContaining([
          expect.objectContaining({
            role: 'user',
            content: 'Hello AI'
          })
        ])
      }),
      expect.anything()
    );
  });
});
```

## Advanced Configuration

### Customizing Providers

```typescript
import [NativeAISDK] from '@bluefly/native_aijs/sdk';

const sdk = new NativeAISDK({
  providers: {
    anthropic: {
      apiKey: process.env.ANTHROPIC_API_KEY,
      baseUrl: 'https://api.anthropic.com',
      models: ['claude-3-opus', 'claude-3-sonnet', 'claude-3-haiku']
    },
    openai: {
      apiKey: process.env.OPENAI_API_KEY,
      baseUrl: 'https://api.openai.com/v1',
      models: ['gpt-4', 'gpt-3.5-turbo']
    },
    mistral: {
      apiKey: process.env.MISTRAL_API_KEY,
      baseUrl: 'https://api.mistral.ai/v1',
      models: ['mistral-medium', 'mistral-small']
    }
  },
  defaultProvider: 'anthropic'
});
```

### Monitoring Configuration

```typescript
import [MonitoringService] from '@bluefly/native_aijs/sdk';

const monitoring = MonitoringService.getInstance();

// Start metrics server
monitoring.startServer(3010);

// Configure alerts
monitoring.setAlertThresholds({
  errorRate: 0.05,        // 5% error rate
  latency: 2000,          // 2000ms latency
  tokenUsage: 1000000     // 1M tokens per day
});

// Subscribe to alerts
monitoring.on('alert', (alert) => {
  console.error(`Alert: [alert.type] - [alert.message]`);
});

// Get current metrics
const metrics = monitoring.getMetrics();
console.log('Current latency:', metrics.latency.avg);
```

## Performance Considerations

1. **Caching**: Configure appropriate TTL for responses that can be cached
2. **Streaming**: Use streaming for better user experience with long responses
3. **Abort Signals**: Always implement cancellation for long-running requests
4. **Retry Strategies**: Configure retry options based on your use case
5. **Monitoring**: Monitor token usage and latency to optimize costs
6. **Model Selection**: Use smaller models for less complex tasks

## Error Handling

The SDK provides standardized error handling with detailed error codes:

```typescript
try [await aiService.chat({
    // request options]);
} catch (error) {
  switch (error.code) {
    case 'authentication_error':
      // Handle authentication issues
      break;
    case 'rate_limit_error':
      // Handle rate limiting
      break;
    case 'invalid_request_error':
      // Handle invalid request parameters
      break;
    case 'model_not_available_error':
      // Handle model availability issues
      break;
    case 'context_length_exceeded':
      // Handle context length issues
      break;
    case 'network_error':
      // Handle network issues
      break;
    default:
      // Handle unknown errors
      console.error('Unknown error:', error);
  }
}
```

## Security Considerations

1. **API Keys**: Never expose API keys in client-side code
2. **Content Filtering**: Implement content filtering for user inputs
3. **Rate Limiting**: Implement rate limiting to prevent abuse
4. **Input Validation**: Validate all user inputs before sending to AI services
5. **Output Sanitization**: Sanitize AI outputs before rendering
6. **Cost Management**: Implement token budget controls to manage costs

## Further Resources

- [Native AI.js SDK Documentation](https://docs.bluefly.io/native-aijs/)
- [AI Provider Documentation](https://docs.bluefly.io/ai-providers/)
- [BFUI Component Library](https://docs.bluefly.io/ui-library/)
- [Testing AI Components](https://docs.bluefly.io/testing-ai-components/)
- [AI Integration Patterns](https://docs.bluefly.io/ai-integration-patterns/)