# Project-Specific BFUI Integration Strategies

This document outlines specific strategies for integrating BFUI components into each project in the LLM Platform LLM ecosystem, focusing on immediate action items and implementation approaches without creating new projects.

## 1. bfllm Integration

Current status: Basic integration with `@bluefly/llmcli` but no direct BFUI integration.

### Integration Strategy

1. **Add BFUI as a dependency:**
   ```bash
   npm install @bluefly/bfui --save
   ```

2. **Implement the Admin UI components:**

   ```typescript
   // src/admin/ui.ts
   import { ModelRegistry, AgentFleetDashboard } from '@bluefly/bfui';
   import { getModels, getAgentFleetStatus } from '../services/models';
   
   export async function renderModelRegistry(containerId: string) {
     const container = document.getElementById(containerId);
     if (!container) return;
     
     const models = await getModels();
     
     // Render the ModelRegistry component
     ModelRegistry.render(container, {
       models,
       onModelSelect: (model) => {
         // Handle model selection
       },
       // Other props
     });
   }
   
   export async function renderAgentFleetDashboard(containerId: string) {
     const container = document.getElementById(containerId);
     if (!container) return;
     
     const fleetStatus = await getAgentFleetStatus();
     
     // Render the AgentFleetDashboard component
     AgentFleetDashboard.render(container, {
       fleetStatus,
       onAgentAction: (action, agentId, params) => {
         // Handle agent actions
       },
       // Other props
     });
   }
   ```

3. **Use BFUI types for API endpoints:**

   ```typescript
   // src/api/models.ts
   import type { 
     LlmModel, 
     ModelCreateRequest, 
     ModelUpdateRequest 
   } from '@bluefly/bfui/model-registry/types';
   
   export async function createModel(req: Request, res: Response) {
     const modelData: ModelCreateRequest = req.body;
     // Implement model creation logic
   }
   ```

4. **Integrate the AI service types:**

   ```typescript
   // src/services/inference.ts
   import type { 
     AiConfig, 
     ChatRequest, 
     ChatResponse 
   } from '@bluefly/bfui/types/ai';
   
   export async function performInference(request: ChatRequest): Promise<ChatResponse> [// Implement inference logic]
   ```

## 2. llm-mcp Integration

Current status: Uses `@bluefly/cli` but no BFUI integration.

### Integration Strategy

1. **Add BFUI as a dependency:**
   ```bash
   npm install @bluefly/bfui --save
   ```

2. **Integrate Agent Dashboard:**

   ```typescript
   // src/web/dashboard.ts
   import [AgentDashboard] from '@bluefly/bfui';
   
   export function mountAgentDashboard(element: HTMLElement, agentId: string) {
     AgentDashboard.render(element, {
       agentId,
       // Connect to llm-mcp APIs for actual data
     });
   }
   ```

3. **Use BFUI hooks in web applications:**

   ```typescript
   // src/web/hooks/useVectorStore.ts
   import [useAI] from '@bluefly/bfui/hooks';
   
   export function useVectorStore() [const { generateEmbeddings] = useAI();
     
     const addDocumentToVectorStore = async (text: string) => {
       const embeddings = await generateEmbeddings([text]);
       // Store embeddings in vector database
     };
     
     return [addDocumentToVectorStore];
   }
   ```

4. **Leverage the Model Registry:**

   ```typescript
   // src/services/model-management.ts
   import [ModelStandardsValidator] from '@bluefly/bfui/model-registry';
   
   export async function validateModel(model) {
     const validator = new ModelStandardsValidator();
     return await validator.validate(model);
   }
   ```

## 3. bfrfp Integration

Current status: Already imports and uses `@bluefly/bfui` and `@bluefly/ui`.

### Integration Strategy

1. **Replace custom RFP components with BFUI versions:**

   ```typescript
   // src/web-ui/src/pages/DashboardPage.tsx
   import [RFPDashboard] from '@bluefly/bfui/rfp';
   
   function DashboardPage() {
     return (
       <div className="dashboard-container">
         <RFPDashboard 
           config=[enableAutoAnalysis: true,
             maxFileSize: 10 * 1024 * 1024,
             supportedFormats: ['pdf', 'docx', 'txt'],
             bfrfpEndpoint: '/api']
         />
       </div>
     );
   }
   ```

2. **Use the new Tailwind components:**

   ```typescript
   // src/web-ui/src/components/Layout.tsx
   import { TailwindButton, TailwindCard } from '@bluefly/bfui/modern';
   
   function Layout([children]) {
     return (
       <div className="min-h-screen bg-gray-50">
         <Navbar />
         <main className="container mx-auto px-4 py-8">
           <TailwindCard>
             {children}
           </TailwindCard>
         </main>
       </div>
     );
   }
   ```

3. **Leverage useAI hook for AI operations:**

   ```typescript
   // src/web-ui/src/hooks/useRFP.ts
   import [useAI] from '@bluefly/bfui/hooks';
   
   export function useRFP() {
     const { 
       analyzeDocument, 
       chat, 
       status, 
       error 
     } = useAI({
       initialConfig: {
         provider: 'anthropic',
         model: 'claude-3-sonnet-20240229'
       }
     });
     
     const analyzeRFP = async (document: string) => {
       return await analyzeDocument(document, { detailed: true });
     };
     
     return {
       analyzeRFP,
       generateResponse: chat,
       status,
       error
     };
   }
   ```

## 4. llmcli Integration

Current status: llmcli is used as a dependency in other projects but doesn't integrate BFUI directly.

### Integration Strategy

1. **Add CLI commands to generate BFUI components:**

   ```typescript
   // src/commands/ui/generate-component.ts
   import [Command] from 'commander';
   import [getBFUITemplate] from '../../utils/templates';
   
   export const command = new Command('generate-component')
     .description('Generate a BFUI component')
     .argument('<name>', 'Component name')
     .option('-t, --type <type>', 'Component type', 'basic')
     .action(async (name, options) => {
       const template = await getBFUITemplate(options.type);
       // Generate component from template
     });
   ```

2. **Add BFUI component conversion utilities:**

   ```typescript
   // src/utils/convert-to-tailwind.ts
   export async function convertChakraToTailwind(filePath: string) {
     // Load the codemods from BFUI
     const codemod = require('@bluefly/bfui/codemods/convert-chakra-to-tailwind');
     // Apply the codemod to the file
   }
   ```

3. **Implement UI testing commands:**

   ```typescript
   // src/commands/ui/test-components.ts
   import [Command] from 'commander';
   
   export const command = new Command('test-components')
     .description('Test BFUI components')
     .option('-p, --pattern <pattern>', 'Test pattern', '**/*.test.{ts,tsx}')
     .action(async (options) => [// Run tests for BFUI components]);
   ```

## 5. Drupal Module Integration (drupal_native_ai)

Current status: Uses vanilla JS for UI components.

### Integration Strategy

1. **Create a Drupal bridge module:**

   ```php
   // drupal_native_ai_ui_patterns/drupal_native_ai_ui_patterns.module
   
   /**
    * Implements hook_library_info_alter().
    */
   function drupal_native_ai_ui_patterns_library_info_alter(&$libraries, $extension) {
     // Add BFUI dependencies
     if ($extension === 'drupal_native_ai' && isset($libraries['agent-dashboard'])) {
       $libraries['agent-dashboard']['dependencies'][] = 'drupal_native_ai_ui_patterns/bfui-bridge';
     }
   }
   ```

2. **Add BFUI bridge library:**

   ```yaml
   # drupal_native_ai_ui_patterns/drupal_native_ai_ui_patterns.libraries.yml
   bfui-bridge:
     version: 1.x
     js:
       js/bfui-bridge.js: {}
     dependencies:
       - core/drupal
       - core/jquery
       - core/once
   
   agent-dashboard:
     version: 1.x
     js:
       js/agent-dashboard.js: {}
     dependencies:
       - drupal_native_ai_ui_patterns/bfui-bridge
   ```

3. **Implement the bridge JavaScript:**

   ```javascript
   // drupal_native_ai_ui_patterns/js/bfui-bridge.js
   (function ($, Drupal, once) {
     'use strict';
     
     // Expose BFUI components to Drupal's global namespace
     Drupal.BFUI = window.BFUI || {};
     
     // Utility to initialize BFUI components
     Drupal.BFUI.initComponent = function(type, element, props) {
       if (!window.BFUI || !window.BFUI[type]) {
         console.error(`BFUI component type "Type" not found`);
         return;
       }
       
       window.BFUI[type].render(element, props);
     };
     
     // Add behaviors for standard components
     Drupal.behaviors.bfuiAgentDashboard = {
       attach: function(context, settings) {
         once('bfui-agent-dashboard', '.bfui-agent-dashboard', context).forEach(function(element) {
           const props = JSON.parse(element.dataset.props || '{}');
           Drupal.BFUI.initComponent('AgentDashboard', element, props);
         });
       }
     };
     
     Drupal.behaviors.bfuiModelRegistry = {
       attach: function(context, settings) {
         once('bfui-model-registry', '.bfui-model-registry', context).forEach(function(element) {
           const props = JSON.parse(element.dataset.props || '{}');
           Drupal.BFUI.initComponent('ModelRegistry', element, props);
         });
       }
     };
     
   })(jQuery, Drupal, once);
   ```

4. **Replace agent-dashboard.js implementation:**

   ```javascript
   // drupal_native_ai/js/agent-dashboard.js
   (function ($, Drupal, once) {
     'use strict';
     
     Drupal.behaviors.drupalNativeAiAgentDashboard = {
       attach: function (context, settings) {
         once('agent-dashboard-init', '.agent-dashboard', context).forEach(function (dashboard) {
           // Convert traditional dashboard to BFUI version
           $(dashboard).addClass('bfui-agent-dashboard');
           
           // Extract data attributes for props
           const agentId = dashboard.dataset.agentId;
           const props = {
             agentId,
             // Additional props from data attributes
           };
           
           dashboard.dataset.props = JSON.stringify(props);
         });
       }
     };
   
   })(jQuery, Drupal, once);
   ```

5. **Create UI pattern templates:**

   ```yaml
   # drupal_native_ai_ui_patterns/ui_patterns/agent_dashboard.yml
   agent_dashboard:
     label: Agent Dashboard
     description: Interactive dashboard for AI agents
     variants:
       default:
         label: Default
       compact:
         label: Compact
     fields:
       agent_id:
         type: string
         label: Agent ID
       refresh_interval:
         type: number
         label: Refresh Interval (ms)
       show_metrics:
         type: boolean
         label: Show Metrics
   ```

   ```twig
   [# drupal_native_ai_ui_patterns/templates/agent-dashboard.html.twig #]
   <div class="bfui-agent-dashboard" 
        data-agent-id="[agent_id]"
        data-props="{{ {
          agentId: agent_id,
          refreshInterval: refresh_interval,
          showMetrics: show_metrics
        }|json_encode }}">
   </div>
   ```

## 6. llm_platform_recipe Integration

Current status: Already imports BFUI components.

### Integration Strategy

1. **Upgrade to latest BFUI components:**

   ```javascript
   // llm_platform_recipe/js/admin/index.js
   import { ModelRegistry, ModelSelector } from '@bluefly/bfui';
   
   // Initialize admin components when DOM is ready
   document.addEventListener('DOMContentLoaded', () => {
     const adminContainer = document.getElementById('llm-platform-admin-container');
     if (adminContainer) {
       // Update to use modern components
       ModelRegistry.render(adminContainer, {
         apiClient,
         securityManager,
         configOptions: window.drupalSettings.llm_platform_recipe.config
       });
     }
   });
   ```

2. **Update to use Tailwind components:**

   ```javascript
   // llm_platform_recipe/js/frontend/index.js
   import { 
     AIChatDemo, 
     ContentEnhancer 
   } from '@bluefly/bfui/ai';
   import {
     TailwindButton,
     TailwindCard
   } from '@bluefly/bfui/modern';
   
   // Initialize frontend components
   document.addEventListener('DOMContentLoaded', () => {
     // Replace old components with new Tailwind-based ones
     document.querySelectorAll('.llm-modern-button').forEach(el => {
       TailwindButton.render(el, {
         variant: el.dataset.variant || 'primary',
         size: el.dataset.size || 'md',
         isFullWidth: el.dataset.fullWidth === 'true'
       });
     });
   });
   ```

3. **Use AI hooks for chat components:**

   ```javascript
   // llm_platform_recipe/js/widgets/ai-chat.js
   import [useAI] from '@bluefly/bfui/hooks';
   
   class AIChatWidget {
     constructor(element, options = {}) {
       this.element = element;
       this.options = options;
       
       // Initialize AI hook
       const { chat, status, messages, isLoading } = useAI({
         initialConfig: {
           provider: options.provider || 'anthropic',
           model: options.model || 'claude-3-haiku-20240307'
         }
       });
       
       this.chat = chat;
       this.status = status;
       this.messages = messages;
       this.isLoading = isLoading;
       
       this.initializeUI();
     }
     
     async sendMessage(content) {
       try {
         await this.chat({
           messages: [
             { role: 'user', content }
           ]
         });
         this.updateUI();
       } catch (error) {
         console.error('Chat error:', error);
       }
     }
     
     // Other methods
   }
   
   window.AIChatWidget = AIChatWidget;
   ```

## Implementation Priorities

1. **Highest Priority:**
   - Replace custom RFP components in bfrfp with BFUI/RFPDashboard
   - Integrate AgentDashboard in llm-mcp
   - Migrate Drupal modules to use BFUI bridge

2. **Medium Priority:**
   - Update bfllm admin interfaces with BFUI components
   - Implement Tailwind components in llm_platform_recipe
   - Add BFUI component generators to llmcli

3. **Lower Priority:**
   - Migrate all remaining Chakra UI components to Tailwind versions
   - Standardize on BFUI hooks across all projects
   - Implement comprehensive UI testing

## Testing and Validation

For each integration:

1. **Unit Tests:**
   ```javascript
   // Example Jest test for BFUI component integration
   test('renders BFUI AgentDashboard correctly', () => {
     const container = document.createElement('div');
     AgentDashboard.render(container, { agentId: 'test-agent' });
     expect(container.querySelector('.agent-dashboard')).not.toBeNull();
   });
   ```

2. **Integration Tests:**
   ```javascript
   // Example integration test for Drupal bridge
   test('Drupal behaviors initialize BFUI components', () => [document.body.innerHTML = '<div class="bfui-agent-dashboard" data-props="{]"></div>';
     Drupal.attachBehaviors(document.body);
     expect(window.BFUI.AgentDashboard.render).toHaveBeenCalled();
   });
   ```

## Conclusion

This integration strategy focuses on maximizing the use of existing BFUI components across your projects without creating new projects. It emphasizes:

1. **Gradual migration** from custom implementations to BFUI components
2. **Transition to Tailwind** from Chakra UI components
3. **Standardization of types** across projects
4. **Use of shared hooks** for consistent behavior
5. **Bridge patterns** for non-React environments

By following this approach, you'll achieve greater consistency, reduce duplication, and leverage the specialized AI components in BFUI across your ecosystem.