# Button Component

## Overview

The Button component is a versatile interactive element for triggering actions or navigation events. It supports various visual styles, sizes, and states.

## Import

```tsx
// Chakra UI version (legacy)
import [Button] from '@bluefly/bfui';

// Tailwind CSS version (modern)
import [TailwindButton] from '@bluefly/bfui/modern';
```

## Usage

### Basic Button

```tsx
<Button>Click Me</Button>
```

### Button Variants

```tsx
// Chakra UI variants
<Button variant="solid">Solid Button</Button>
<Button variant="outline">Outline Button</Button>
<Button variant="ghost">Ghost Button</Button>
<Button variant="link">Link Button</Button>

// Modern variants
<TailwindButton variant="primary">Primary Button</TailwindButton>
<TailwindButton variant="secondary">Secondary Button</TailwindButton>
<TailwindButton variant="danger">Danger Button</TailwindButton>
```

### Button Sizes

```tsx
<Button size="xs">Extra Small</Button>
<Button size="sm">Small</Button>
<Button size="md">Medium</Button>
<Button size="lg">Large</Button>
```

### Button States

```tsx
// Loading state
<Button isLoading>Loading</Button>
<Button isLoading loadingText="Saving...">Save</Button>

// Disabled state
<Button isDisabled>Disabled</Button>
```

### With Icons

```tsx
<Button leftIcon=[<Icon name="edit" />]>Edit</Button>
<Button rightIcon=[<Icon name="arrow-right" />]>Next</Button>
```

### Full Width

```tsx
<Button isFullWidth>Full Width Button</Button>
```

## Props

### Common Props

| Prop | Type | Default | Description |
|------|------|---------|-------------|
| children | ReactNode | - | Button content |
| size | 'xs' \| 'sm' \| 'md' \| 'lg' | 'md' | Button size |
| isLoading | boolean | false | Whether the button is in a loading state |
| isDisabled | boolean | false | Whether the button is disabled |
| leftIcon | ReactElement | - | Icon element to display before button text |
| rightIcon | ReactElement | - | Icon element to display after button text |
| loadingText | string | - | Text to display when button is loading |
| onClick | function | - | Click handler for the button |
| className | string | - | Additional CSS classes |

### Chakra UI Button Props

| Prop | Type | Default | Description |
|------|------|---------|-------------|
| variant | 'solid' \| 'outline' \| 'ghost' \| 'link' \| 'unstyled' | 'solid' | Visual style variant |
| colorScheme | string | 'blue' | Color scheme for the button |
| spinnerPlacement | 'start' \| 'end' | 'start' | Placement of spinner when loading |

### Tailwind Button Props

| Prop | Type | Default | Description |
|------|------|---------|-------------|
| variant | 'solid' \| 'outline' \| 'ghost' \| 'link' \| 'unstyled' \| 'primary' \| 'secondary' \| 'danger' | 'solid' | Visual style variant |
| isFullWidth | boolean | false | Whether button takes full width of container |

## Theming

### Chakra UI Theme Customization

```tsx
// In your Chakra theme file
export const theme = {
  components: {
    Button: {
      baseStyle: {
        fontWeight: 'semibold',
        borderRadius: 'md',
      },
      variants: {
        custom: {
          bg: 'brand.500',
          color: 'white',
          _hover: {
            bg: 'brand.600',
          },
        },
      },
    },
  },
};
```

### Tailwind CSS Customization

```tsx
// Using the className prop
<TailwindButton 
  className="bg-indigo-600 hover:bg-indigo-700 text-white"
>
  Custom Style
</TailwindButton>

// Using Tailwind CSS variables
// In your CSS:
:root {
  --primary: 235 100% 50%;
  --primary-foreground: 0 0% 100%;
}
```

## Accessibility

- Buttons use the native `<button>` element when possible
- `aria-disabled` is set when a button is disabled
- Loading buttons include `aria-busy` attribute
- Focus states are visually apparent
- Icon-only buttons require an `aria-label`

## Examples

### Form Submit Button

```tsx
function SubmitForm() {
  const [isLoading, setIsLoading] = useState(false);
  
  const handleSubmit = async () => {
    setIsLoading(true);
    try {
      await submitData();
      // Success handling
    } catch (error) [// Error handling] finally {
      setIsLoading(false);
    }
  };
  
  return (
    <Button 
      onClick=[handleSubmit]
      isLoading=[isLoading]
      loadingText="Submitting..."
      colorScheme="green"
    >
      Submit
    </Button>
  );
}
```

### Navigation Button

```tsx
import [useNavigate] from 'react-router-dom';

function NavigationButton() {
  const navigate = useNavigate();
  
  return (
    <Button 
      onClick={() => navigate('/dashboard')}
      rightIcon=[<ArrowRightIcon />]
      variant="outline"
    >
      Go to Dashboard
    </Button>
  );
}
```

### Delete Confirmation Button

```tsx
function DeleteButton({ itemId, onDelete }) {
  const [isDeleting, setIsDeleting] = useState(false);
  
  const handleDelete = async () => {
    setIsDeleting(true);
    try {
      await deleteItem(itemId);
      onDelete();
    } catch (error) {
      console.error('Delete failed:', error);
    } finally {
      setIsDeleting(false);
    }
  };
  
  return (
    <Button 
      onClick=[handleDelete]
      isLoading=[isDeleting]
      loadingText="Deleting..."
      variant="danger"
      leftIcon=[<TrashIcon />]
    >
      Delete
    </Button>
  );
}
```

### Button Group

```tsx
function ButtonGroup() {
  return (
    <div className="flex space-x-2">
      <Button variant="outline">Cancel</Button>
      <Button variant="solid" colorScheme="green">Save</Button>
    </div>
  );
}
```

## Best Practices

1. **Use descriptive labels**: Button text should clearly describe the action that will occur
2. **Choose appropriate variants**: Use variants that match the semantic meaning of the action
3. **Provide feedback**: Use loading states to indicate when an action is in progress
4. **Maintain hierarchy**: Use visual styling to indicate the importance of different buttons
5. **Ensure accessibility**: Always include proper ARIA attributes for screen readers

## Migration Guide

### From Chakra UI to Tailwind CSS

```tsx
// Chakra UI Button
<Button
  variant="solid"
  colorScheme="blue"
  size="md"
  isLoading=[isLoading]
  loadingText="Saving..."
  leftIcon=[<SaveIcon />]
  onClick=[handleSave]
>
  Save Changes
</Button>

// Equivalent Tailwind Button
<TailwindButton
  variant="primary" // Maps to solid variant with blue color scheme
  size="md"
  isLoading=[isLoading]
  loadingText="Saving..."
  leftIcon=[<SaveIcon />]
  onClick=[handleSave]
>
  Save Changes
</TailwindButton>
```

## Troubleshooting

### Common Issues

1. **Button doesn't trigger onClick handler**
   - Check if the button is disabled or in loading state
   - Verify event propagation isn't being stopped

2. **Icons not aligning properly**
   - Ensure icon components are properly sized
   - Check for CSS conflicts

3. **Button text wrapping unexpectedly**
   - Add `whitespace-nowrap` class to prevent wrapping
   - Set explicit width or use flex containers

## Related Components

- [IconButton](./icon-button.md): Button with only an icon and no text
- [ButtonGroup](./button-group.md): Group of related buttons
- [LinkButton](./link-button.md): Button that functions as a link