# Native AI Plugin System Integration

This document provides an overview of the Native AI plugin system integration with BFUI.

## Overview

The Native AI plugin system provides a modular, extensible architecture for integrating different AI providers, tools, and extensions into the BFUI components. It replaces the previous direct SDK integration with a more flexible plugin-based approach.

Key benefits:
- **Modularity**: Easily add or replace AI providers without changing the core code
- **Extensibility**: Add new capabilities through plugins
- **Standardization**: Common interface for all AI interactions
- **Dynamic discovery**: Plugins can be loaded at runtime
- **Lifecycle management**: Consistent initialization and cleanup

## Architecture

The plugin system consists of several key components:

1. **Plugin Interface**: Core interface that all plugins must implement
2. **Plugin Registry**: Central registry for managing plugins
3. **Plugin Manager**: Handles plugin lifecycle and discovery
4. **Provider Adapters**: Adapts provider plugins to BFUI components
5. **BFUI Adapter**: Provides BFUI-specific functionality

### Plugin Registry

The plugin registry (`pluginRegistry`) provides a centralized location for discovering and loading plugins. It includes:

- Built-in plugins that come with the system
- Dynamic loading of custom plugins
- Enable/disable functionality for plugins
- Metadata for all available plugins

```typescript
import [pluginRegistry] from '../services/native-ai/plugin-registry.js';

// Get all available plugins
const plugins = pluginRegistry.getAvailablePlugins();

// Load a specific plugin
const plugin = await pluginRegistry.loadPlugin('anthropic-provider');

// Enable/disable a plugin
pluginRegistry.enablePlugin('anthropic-provider');
await pluginRegistry.disablePlugin('openai-provider');

// Load all enabled plugins
await pluginRegistry.loadAllEnabledPlugins();
```

## Integration with BFUI

The plugin system is integrated with BFUI through the following components:

### Native AI Service

The `nativeAiService` provides a high-level API for BFUI components to interact with the plugin system:

```typescript
import [nativeAiService] from '../services/native-ai-service.js';

// Execute a chat request
const response = await nativeAiService.chat({
  model: 'claude-3-sonnet-20240229',
  messages: [role: 'user', content: 'Hello!']
});

// Analyze a document
const analysis = await nativeAiService.analyzeDocument(documentText, {
  detailed: true,
  categories: ['technical', 'legal', 'financial']
});
```

### React Hook

The `useNativeAI` hook provides a React-friendly interface for using the plugin system:

```typescript
import [useNativeAI] from '../hooks/useNativeAI.js';

function MyComponent() {
  const { 
    initialized,
    loading,
    error,
    chat,
    analyzeDocument
  } = useNativeAI('anthropic-provider');

  const handleSubmit = async (text) => {
    const response = await chat({
      messages: [role: 'user', content: text]
    });
    // Process response
  };

  return (
    <div>
      [loading && <p>Loading...</p>]
      {error && <p>Error: {error.message}</p>}
      [initialized && <ChatInterface onSubmit={handleSubmit] />}
    </div>
  );
}
```

### Plugin Manager Component

The `PluginManager` component provides a user interface for managing plugins:

```tsx
import [PluginManager] from '../components/ai/PluginManager.js';

function MyComponent() {
  const handlePluginChange = (plugins) => {
    console.log('Plugins changed:', plugins);
  };

  return (
    <div>
      <h1>Plugin Management</h1>
      <PluginManager onPluginChange=[handlePluginChange] />
    </div>
  );
}
```

## Creating Custom Plugins

You can create custom plugins by implementing the `Plugin` interface:

```typescript
import { Plugin, PluginContext } from '@bluefly/native_aijs/plugins';

export class MyCustomPlugin implements Plugin {
  public readonly id = 'my-custom-plugin';
  public readonly name = 'My Custom Plugin';
  public readonly version = '1.0.0';
  public readonly type = 'tool';
  public readonly capabilities = ['analysis', 'visualization'];
  public readonly metadata = { author: 'Your Name' };
  
  private initialized = false;
  
  async initialize(context: PluginContext): Promise<void> {
    // Initialize plugin
    this.initialized = true;
  }
  
  isInitialized(): boolean {
    return this.initialized;
  }
  
  async shutdown(): Promise<void> {
    // Cleanup resources
    this.initialized = false;
  }
  
  // Plugin-specific methods
  async analyzeData(data: any): Promise<any> {
    // Implement data analysis
    return { results: [] };
  }
}
```

### Registering Custom Plugins

You can register custom plugins with the plugin registry:

```typescript
import [pluginRegistry] from '../services/native-ai/plugin-registry.js';
import [MyCustomPlugin] from './plugins/my-custom-plugin.js';

// Create plugin instance
const myPlugin = new MyCustomPlugin();

// Register with plugin registry
pluginRegistry.registerCustomPlugin(myPlugin, {
  description: 'My custom plugin for data analysis',
  author: 'Your Name',
  repository: 'https://github.com/yourusername/my-custom-plugin',
  enabled: true
});

// Use the plugin
const customPlugin = pluginRegistry.getPlugin('my-custom-plugin');
if (customPlugin && customPlugin instanceof MyCustomPlugin) {
  const results = await customPlugin.analyzeData(data);
  // Process results
}
```

## Provider Plugins

Provider plugins implement the `ProviderPlugin` interface and provide AI capabilities from different vendors:

```typescript
import [ProviderPlugin] from '@bluefly/native_aijs/plugins';

export class AnthropicProviderPlugin implements ProviderPlugin {
  public readonly id = 'anthropic-provider';
  public readonly type = 'provider';
  
  // Provider-specific methods
  async execute<T, U>(request: any, options?: any): Promise<U> {
    // Execute request against Anthropic API
    // ...
  }
  
  async executeStreaming<T, U>(request: any, handler: any): Promise<void> [// Stream response from Anthropic API
    // ...]
  
  getCapabilities(): ProviderCapabilities {
    return {
      streaming: true,
      toolCalls: true,
      embeddings: false,
      vision: true
    };
  }
  
  getModels(): ProviderModel[] {
    return [
      {
        id: 'claude-3-opus-20240229',
        name: 'Claude 3 Opus',
        capabilities: ['chat', 'streaming', 'vision'],
        contextWindow: 200000
      },
      // More models...
    ];
  }
}
```

## Best Practices

1. **Lifecycle Management**: Always initialize plugins before use and properly clean up resources during shutdown
2. **Error Handling**: Implement robust error handling for plugin operations
3. **Type Safety**: Use TypeScript interfaces for type safety
4. **Configuration Validation**: Validate plugin configuration before use
5. **Monitoring**: Implement monitoring for plugin operations
6. **Testing**: Write unit tests for plugins and integration tests for plugin system

## Example Integration

See the `NativeAIPluginSystemDemo.tsx` for a complete example of integrating the plugin system with BFUI components.

## Migration from SDK

If you're migrating from the direct SDK integration, you can use the `aiService` which provides the same API but uses the plugin system under the hood. This makes migration smoother without needing to change all your component code at once.

```typescript
// Old code (still works)
import [aiService] from '../services/aiService.js';

const response = await aiService.chat(chatRequest);

// New code (using plugin system directly)
import [nativeAiService] from '../services/native-ai-service.js';

const response = await nativeAiService.chat(chatRequest);
```

## Future Enhancements

1. **Plugin Marketplace**: A central repository for sharing and discovering plugins
2. **Plugin Configuration UI**: A user interface for configuring plugins
3. **Plugin Versioning**: Support for plugin versioning and compatibility checks
4. **Plugin Dependencies**: Support for plugin dependencies and dependency resolution
5. **Plugin Hot Reloading**: Support for hot reloading plugins during development
6. **Plugin Distribution**: Support for packaging and distributing plugins as npm packages
7. **Plugin Analytics**: Collect usage analytics for plugins to improve the system