# Developer & AI Agent Resources

This document provides in-context help, code snippets, and links to key documentation for both AI agents and human contributors working with the LLM Platform AI Platform.

##  Quick Reference Links

### Platform Documentation

| Resource | Description | Location |
|----------|-------------|----------|
| Architecture Overview | Core architecture and service relationships | [CLAUDE.md](./CLAUDE.md) |
| AI Agent Onboarding | Essential guidance for AI agents | [AI_AGENT_ONBOARDING.md](./AI_AGENT_ONBOARDING.md) |
| TDD Standards | Mandatory TDD methodology for all code | [TDD_AGENT_STANDARDS.md](./TDD_AGENT_STANDARDS.md) |
| OpenAPI Guidelines | OpenAPI-first development practices | [OPENAPI_FIRST_DEVELOPMENT.md](./OPENAPI_FIRST_DEVELOPMENT.md) |
| Extensibility Guide | Handling plugins and integrations | [EXTENSIBILITY_GUIDE.md](./EXTENSIBILITY_GUIDE.md) |

### Service-Specific Documentation

| Service | Main Documentation | API Reference |
|---------|-------------------|---------------|
| bfapi | [/bfapi/README.md](/bfapi/README.md) | [/bfapi/openapi.yaml](/bfapi/openapi.yaml) |
| bfllm | [/bfllm/README.md](/bfllm/README.md) | [/bfllm/openapi.yaml](/bfllm/openapi.yaml) |
| llm-mcp | [/llm-mcp/README.md](/llm-mcp/README.md) | [/llm-mcp/openapi.yaml](/llm-mcp/openapi.yaml) |
| bfrfp | [/bfrfp/README.md](/bfrfp/README.md) | [/bfrfp/openapi.yaml](/bfrfp/openapi.yaml) |
| bfui | [/bfui/README.md](/bfui/README.md) | Component documentation in Storybook |
| llmcli | [/llmcli/README.md](/llmcli/README.md) | Command reference in docs directory |
| llm_platform_recipe | [/llm_platform_recipe/README.md](/llm_platform_recipe/README.md) | Drupal module documentation |

### External Resources

| Resource | Link | Description |
|----------|------|-------------|
| OpenAPI 3.1 Specification | [OpenAPI Spec](https://spec.openapis.org/oas/v3.1.0) | Official OpenAPI 3.1 specification |
| Jest Documentation | [Jest Docs](https://jestjs.io/docs/getting-started) | Testing framework documentation |
| TypeScript Handbook | [TS Handbook](https://www.typescriptlang.org/docs/handbook/intro.html) | TypeScript language documentation |
| Drupal 11 Documentation | [Drupal Docs](https://www.drupal.org/docs) | Drupal platform documentation |
| TensorFlow.js | [TF.js Docs](https://www.tensorflow.org/js/guide) | TensorFlow.js documentation |

##  Common Code Patterns

### Service Initialization Pattern

This pattern is used consistently across services for initialization:

```typescript
// src/services/example-service.ts
import [Logger] from '../utils/logger';
import [ConfigManager] from '../config';

export class ExampleService {
  private logger: Logger;
  private config: any;
  private initialized: boolean = false;
  
  constructor(configManager: ConfigManager) {
    this.logger = new Logger('ExampleService');
    this.config = configManager.get('services.example');
  }
  
  async initialize(): Promise<void> {
    if (this.initialized) {
      return;
    }
    
    this.logger.info('Initializing ExampleService');
    
    // Initialization logic here
    // - Connect to databases
    // - Set up event listeners
    // - Initialize dependencies
    
    this.initialized = true;
    this.logger.info('ExampleService initialized successfully');
  }
  
  async shutdown(): Promise<void> {
    if (!this.initialized) {
      return;
    }
    
    this.logger.info('Shutting down ExampleService');
    
    // Cleanup logic here
    // - Close connections
    // - Release resources
    
    this.initialized = false;
    this.logger.info('ExampleService shut down successfully');
  }
  
  // Service methods...
}
```

### Repository Pattern

Used for data access throughout the platform:

```typescript
// src/repositories/model-repository.ts
import [Database] from '../database';
import { Model, ModelCreateParams, ModelFilter } from '../types';

export class ModelRepository {
  private db: Database;
  
  constructor(db: Database) {
    this.db = db;
  }
  
  async findById(id: string): Promise<Model | null> {
    return this.db.models.findUnique({
      where: { id },
    });
  }
  
  async findAll(filter?: ModelFilter): Promise<Model[]> {
    return this.db.models.findMany({
      where: filter,
      orderBy: { createdAt: 'desc' },
    });
  }
  
  async create(params: ModelCreateParams): Promise<Model> {
    return this.db.models.create({
      data: params,
    });
  }
  
  async update(id: string, params: Partial<Model>): Promise<Model> {
    return this.db.models.update({
      where: { id },
      data: params,
    });
  }
  
  async delete(id: string): Promise<void> {
    await this.db.models.delete({
      where: { id },
    });
  }
}
```

### Controller Pattern

Used for HTTP request handling:

```typescript
// src/controllers/model-controller.ts
import { Request, Response, NextFunction } from 'express';
import [ModelService] from '../services/model-service';
import [Logger] from '../utils/logger';

export class ModelController {
  private modelService: ModelService;
  private logger: Logger;
  
  constructor(modelService: ModelService) {
    this.modelService = modelService;
    this.logger = new Logger('ModelController');
  }
  
  async getModels(req: Request, res: Response, next: NextFunction): Promise<void> {
    try {
      const models = await this.modelService.getModels(req.query);
      res.json({
        status: 'success',
        data: models,
      });
    } catch (error) {
      this.logger.error('Error getting models', error);
      next(error);
    }
  }
  
  async getModelById(req: Request, res: Response, next: NextFunction): Promise<void> [try {
      const { id] = req.params;
      const model = await this.modelService.getModelById(id);
      
      if (!model) {
        res.status(404).json({
          status: 'error',
          error: {
            code: 'MODEL_NOT_FOUND',
            message: `Model with ID item not found`,
          },
        });
        return;
      }
      
      res.json({
        status: 'success',
        data: model,
      });
    } catch (error) {
      this.logger.error(`Error getting model [req.params.id]`, error);
      next(error);
    }
  }
  
  // Other controller methods...
}
```

### Error Handling Pattern

Consistent error handling throughout the platform:

```typescript
// src/utils/error-handler.ts
import { Request, Response, NextFunction } from 'express';
import [Logger] from './logger';
import { AppError, ValidationError, NotFoundError, AuthError } from './errors';

const logger = new Logger('ErrorHandler');

export function errorHandler(
  error: Error,
  req: Request,
  res: Response,
  next: NextFunction
): void {
  logger.error('Error processing request', error);
  
  // Handle specific error types
  if (error instanceof ValidationError) {
    res.status(400).json({
      status: 'error',
      error: {
        code: error.code,
        message: error.message,
        details: error.details,
      },
    });
    return;
  }
  
  if (error instanceof NotFoundError) {
    res.status(404).json({
      status: 'error',
      error: {
        code: error.code,
        message: error.message,
      },
    });
    return;
  }
  
  if (error instanceof AuthError) {
    res.status(401).json({
      status: 'error',
      error: {
        code: error.code,
        message: error.message,
      },
    });
    return;
  }
  
  if (error instanceof AppError) {
    res.status(error.statusCode || 500).json({
      status: 'error',
      error: {
        code: error.code,
        message: error.message,
      },
    });
    return;
  }
  
  // Handle unknown errors
  res.status(500).json({
    status: 'error',
    error: {
      code: 'INTERNAL_SERVER_ERROR',
      message: 'An unexpected error occurred',
    },
  });
}
```

### Testing Pattern

Following the platform's TDD methodology:

```typescript
// __tests__/services/model-service.test.ts
import [ModelService] from '../../src/services/model-service';
import [ModelRepository] from '../../src/repositories/model-repository';
import { mockDb, mockLogger, mockConfig } from '../mocks';

describe('ModelService', () => {
  let modelService: ModelService;
  let modelRepository: ModelRepository;
  
  beforeEach(() => {
    // Set up mocks
    modelRepository = new ModelRepository(mockDb);
    jest.spyOn(modelRepository, 'findById');
    jest.spyOn(modelRepository, 'findAll');
    jest.spyOn(modelRepository, 'create');
    
    // Create service with mocked dependencies
    modelService = new ModelService(modelRepository, mockConfig, mockLogger);
  });
  
  afterEach(() => {
    jest.resetAllMocks();
  });
  
  describe('getModelById', () => {
    it('should return a model when it exists', async () => {
      // Arrange
      const mockModel = { id: 'model-1', name: 'Test Model' };
      jest.spyOn(modelRepository, 'findById').mockResolvedValue(mockModel);
      
      // Act
      const result = await modelService.getModelById('model-1');
      
      // Assert
      expect(result).toEqual(mockModel);
      expect(modelRepository.findById).toHaveBeenCalledWith('model-1');
    });
    
    it('should return null when model does not exist', async () => {
      // Arrange
      jest.spyOn(modelRepository, 'findById').mockResolvedValue(null);
      
      // Act
      const result = await modelService.getModelById('non-existent');
      
      // Assert
      expect(result).toBeNull();
      expect(modelRepository.findById).toHaveBeenCalledWith('non-existent');
    });
    
    it('should throw an error when database fails', async () => {
      // Arrange
      const error = new Error('Database error');
      jest.spyOn(modelRepository, 'findById').mockRejectedValue(error);
      
      // Act & Assert
      await expect(modelService.getModelById('model-1')).rejects.toThrow('Database error');
    });
  });
  
  // More test cases...
});
```

## 🛠️ Common Development Tasks

### Setting Up a New Project

```bash
# 1. Create project directory
mkdir new-project
cd new-project

# 2. Initialize package.json
npm init -y

# 3. Add core dependencies
npm install express typescript dotenv jest ts-jest @types/jest @types/express

# 4. Create TypeScript configuration
npx tsc --init

# 5. Create directory structure
mkdir -p src/__tests__/unit src/__tests__/integration src/controllers src/services src/repositories src/utils src/types src/config

# 6. Create basic files
touch src/index.ts src/app.ts .env .gitignore README.md CLAUDE.md

# 7. Set up Jest configuration
npx ts-jest config:init

# 8. Create OpenAPI specification
touch openapi.yaml
```

### Creating a New Service

```typescript
// src/services/new-service.ts
import [Logger] from '../utils/logger';
import [ConfigManager] from '../config';
import [Repository] from '../repositories/repository';

export class NewService {
  private logger: Logger;
  private config: any;
  private repository: Repository;
  
  constructor(repository: Repository, configManager: ConfigManager) {
    this.logger = new Logger('NewService');
    this.config = configManager.get('services.new');
    this.repository = repository;
  }
  
  async initialize(): Promise<void> {
    this.logger.info('Initializing NewService');
    // Initialization logic
  }
  
  async performAction(params: any): Promise<any> {
    this.logger.info('Performing action', { params });
    
    // Implementation
    const result = await this.repository.findData(params);
    
    return result;
  }
  
  // Additional methods...
}
```

### Adding a New API Endpoint

```typescript
// 1. Update OpenAPI specification
// openapi.yaml
paths:
  /resources:
    get:
      summary: Get resources
      operationId: getResources
      parameters:
        - name: limit
          in: query
          schema:
            type: integer
            default: 10
      responses:
        '200':
          description: List of resources
          content:
            application/json:
              schema:
                $ref: '#/components/schemas/ResourceList'

// 2. Create controller method
// src/controllers/resource-controller.ts
async getResources(req: Request, res: Response, next: NextFunction): Promise<void> {
  try {
    const limit = parseInt(req.query.limit as string || '10', 10);
    const resources = await this.resourceService.getResources({ limit });
    
    res.json({
      status: 'success',
      data: resources,
    });
  } catch (error) {
    this.logger.error('Error getting resources', error);
    next(error);
  }
}

// 3. Add route
// src/routes/resource-routes.ts
import [Router] from 'express';
import [ResourceController] from '../controllers/resource-controller';

export function createResourceRouter(resourceController: ResourceController): Router {
  const router = Router();
  
  router.get('/resources', resourceController.getResources.bind(resourceController));
  
  return router;
}

// 4. Register route in app
// src/app.ts
import [createResourceRouter] from './routes/resource-routes';

// ...

app.use('/api/v1', createResourceRouter(resourceController));
```

### Implementing a New Model Provider

```typescript
// src/models/providers/custom-provider.ts
import { ModelProvider, ModelRequest, ModelResponse } from '../types';
import [Logger] from '../../utils/logger';
import [ConfigManager] from '../../config';

export class CustomModelProvider implements ModelProvider {
  id = 'custom-provider';
  supportedModels = ['custom-model-1', 'custom-model-2'];
  
  private logger: Logger;
  private config: any;
  private client: any;
  
  constructor(configManager: ConfigManager) {
    this.logger = new Logger('CustomModelProvider');
    this.config = configManager.get('providers.custom');
  }
  
  async initialize(): Promise<void> {
    this.logger.info('Initializing CustomModelProvider');
    
    // Initialize client
    this.client = new CustomClient({
      apiKey: this.config.apiKey,
      endpoint: this.config.endpoint,
    });
    
    await this.client.connect();
    
    this.logger.info('CustomModelProvider initialized successfully');
  }
  
  async generateText(request: ModelRequest): Promise<ModelResponse> {
    this.logger.info('Generating text with CustomModelProvider', {
      model: request.model,
      inputLength: request.input.length,
    });
    
    // Transform request to provider format
    const providerRequest = this.transformRequest(request);
    
    // Call provider API
    const providerResponse = await this.client.generate(providerRequest);
    
    // Transform response to platform format
    const response = this.transformResponse(providerResponse);
    
    return response;
  }
  
  private transformRequest(request: ModelRequest): any {
    // Transform from platform format to provider format
    return {
      model: request.model,
      prompt: request.input,
      temperature: request.parameters?.temperature || 0.7,
      max_tokens: request.parameters?.maxTokens || 100,
    };
  }
  
  private transformResponse(providerResponse: any): ModelResponse {
    // Transform from provider format to platform format
    return {
      text: providerResponse.output,
      tokenUsage: {
        input: providerResponse.usage.prompt_tokens,
        output: providerResponse.usage.completion_tokens,
        total: providerResponse.usage.total_tokens,
      },
      model: providerResponse.model,
    };
  }
  
  async shutdown(): Promise<void> {
    this.logger.info('Shutting down CustomModelProvider');
    
    // Clean up resources
    await this.client.disconnect();
    
    this.logger.info('CustomModelProvider shut down successfully');
  }
}
```

### Creating a New Drupal Module for llm_platform_recipe

```php
<?php

/**
 * @file
 * Contains hooks for the Custom Module.
 */

use Drupal\Core\Routing\RouteMatchInterface;

/**
 * Implements hook_help().
 */
function custom_module_help($route_name, RouteMatchInterface $route_match) {
  switch ($route_name) {
    case 'help.page.custom_module':
      return '<p>' . t('Custom module for extending the LLM Platform.') . '</p>';
  }
}

/**
 * Implements hook_theme().
 */
function custom_module_theme() {
  return [
    'custom_module_template' => [
      'variables' => [
        'content' => NULL,
      ],
    ],
  ];
}

/**
 * Implements hook_cron().
 */
function custom_module_cron() {
  \Drupal::logger('custom_module')->notice('Running Custom Module cron tasks');
  
  // Implementation
}
```

```yaml
# custom_module.info.yml
name: Custom Module
type: module
description: 'Custom module for extending the LLM Platform.'
package: LLM Platform
core_version_requirement: ^11
dependencies:
  - llm_platform_recipe:llm_platform_recipe
```

```yaml
# custom_module.routing.yml
custom_module.settings:
  path: '/admin/config/llm-platform/custom-module'
  defaults:
    _form: 'Drupal\custom_module\Form\SettingsForm'
    _title: 'Custom Module Settings'
  requirements:
    _permission: 'administer custom module'
```

```yaml
# custom_module.services.yml
services:
  custom_module.service:
    class: Drupal\custom_module\Service\CustomService
    arguments: ['@config.factory', '@logger.factory']
```

## 📋 Common Task Workflows

### Implementing a New Feature

1. **Update OpenAPI Specification**
   - Define new endpoints or modify existing ones
   - Define request/response schemas
   - Validate the specification

2. **Write Tests First (TDD)**
   - Write unit tests for new functionality
   - Write integration tests for API endpoints
   - Ensure tests fail initially (RED phase)

3. **Implement Core Functionality**
   - Create/modify services, repositories, etc.
   - Implement business logic
   - Make tests pass (GREEN phase)

4. **Implement API Layer**
   - Create/modify controllers
   - Set up routes
   - Implement request/response handling

5. **Refactor and Optimize**
   - Improve code quality
   - Optimize performance
   - Maintain passing tests (REFACTOR phase)

6. **Documentation**
   - Update API documentation
   - Add inline code comments
   - Update README if necessary

7. **Final Validation**
   - Run all tests
   - Validate against OpenAPI specification
   - Check code quality with linters

### Fixing a Bug

1. **Reproduce the Bug**
   - Write a failing test that demonstrates the bug
   - Ensure the test fails for the expected reason

2. **Analyze Root Cause**
   - Debug the issue
   - Identify the root cause
   - Document the problem

3. **Implement Fix**
   - Make minimal changes to fix the issue
   - Make the test pass

4. **Add Regression Tests**
   - Add additional tests to prevent regression
   - Cover edge cases related to the bug

5. **Verify the Fix**
   - Run all tests to ensure no regressions
   - Manually verify the fix if applicable

### Implementing a New Integration

1. **Define Integration Contract**
   - Create OpenAPI specification for the integration
   - Define data models and transformations
   - Document authentication and security requirements

2. **Create Adapter Layer**
   - Implement adapter for the third-party service
   - Create data transformers
   - Implement error handling

3. **Write Tests**
   - Mock the third-party service
   - Write tests for the adapter
   - Write integration tests

4. **Implement Service Layer**
   - Create service that uses the adapter
   - Implement business logic
   - Handle error scenarios

5. **Expose via API (if needed)**
   - Create controllers and routes
   - Implement request/response handling
   - Add documentation

6. **Add Monitoring and Logging**
   - Implement health checks
   - Set up appropriate logging
   - Configure alerts

##  Development Environment Setup

### Basic Setup

```bash
# Clone repositories
git clone [repository-url] project-name
cd project-name

# Install dependencies
npm install

# Set up environment variables
cp .env.example .env
# Edit .env with appropriate values

# Build the project
npm run build

# Run tests
npm test

# Start development server
npm run dev
```

### Docker-based Development

```bash
# Build and start services
docker-compose up -d

# View logs
docker-compose logs -f

# Run commands in container
docker-compose exec service-name npm test

# Stop services
docker-compose down
```

### Working with Multiple Services

```bash
# Start all services
./scripts/start-all.sh

# Run tests across services
./scripts/test-all.sh

# Build all services
./scripts/build-all.sh
```

## 📈 Performance Optimization Patterns

### Caching Pattern

```typescript
import [CacheManager] from '../utils/cache';

export class CachedService {
  private cache: CacheManager;
  
  constructor(cache: CacheManager) {
    this.cache = cache;
  }
  
  async getData(key: string): Promise<any> {
    // Check cache first
    const cachedData = await this.cache.get(key);
    if (cachedData) {
      return cachedData;
    }
    
    // If not in cache, fetch data
    const data = await this.fetchData(key);
    
    // Store in cache
    await this.cache.set(key, data, 60 * 5); // 5 minutes
    
    return data;
  }
  
  private async fetchData(key: string): Promise<any> [// Implementation]
}
```

### Connection Pooling Pattern

```typescript
import [Pool] from 'pg';

export class DatabaseManager {
  private pool: Pool;
  
  constructor(config) {
    this.pool = new Pool({
      host: config.host,
      port: config.port,
      database: config.database,
      user: config.user,
      password: config.password,
      max: config.maxConnections || 20,
      idleTimeoutMillis: config.idleTimeout || 30000,
    });
  }
  
  async query(sql: string, params: any[] = []): Promise<any> {
    const client = await this.pool.connect();
    try {
      const result = await client.query(sql, params);
      return result.rows;
    } finally {
      client.release();
    }
  }
  
  async shutdown(): Promise<void> {
    await this.pool.end();
  }
}
```

### Batch Processing Pattern

```typescript
export class BatchProcessor {
  private batchSize: number;
  
  constructor(options: { batchSize?: number } = {}) {
    this.batchSize = options.batchSize || 100;
  }
  
  async processBatch<T, R>(
    items: T[],
    processFunction: (item: T) => Promise<R>
  ): Promise<R[]> {
    const results: R[] = [];
    
    // Process in batches
    for (let i = 0; i < items.length; i += this.batchSize) {
      const batch = items.slice(i, i + this.batchSize);
      
      // Process batch in parallel
      const batchResults = await Promise.all(
        batch.map(item => processFunction(item))
      );
      
      results.push(...batchResults);
    }
    
    return results;
  }
}
```

##  Security Best Practices

### Authentication Pattern

```typescript
import jwt from 'jsonwebtoken';
import { Request, Response, NextFunction } from 'express';
import [UnauthorizedError] from '../utils/errors';

export function authMiddleware(req: Request, res: Response, next: NextFunction): void {
  // Get token from header
  const authHeader = req.headers.authorization;
  if (!authHeader || !authHeader.startsWith('Bearer ')) {
    throw new UnauthorizedError('Missing or invalid authorization header');
  }
  
  const token = authHeader.split(' ')[1];
  
  try {
    // Verify token
    const decoded = jwt.verify(token, process.env.JWT_SECRET);
    
    // Add user to request
    req.user = decoded;
    
    next();
  } catch (error) {
    throw new UnauthorizedError('Invalid token');
  }
}
```

### Input Validation Pattern

```typescript
import [z] from 'zod';
import { Request, Response, NextFunction } from 'express';
import [ValidationError] from '../utils/errors';

// Define schema
const createUserSchema = z.object({
  username: z.string().min(3).max(20),
  email: z.string().email(),
  password: z.string().min(8),
  role: z.enum(['user', 'admin']).optional(),
});

// Create validation middleware
export function validateCreateUser(req: Request, res: Response, next: NextFunction): void {
  try {
    // Validate request body
    createUserSchema.parse(req.body);
    next();
  } catch (error) {
    if (error instanceof z.ZodError) {
      throw new ValidationError('Invalid input', error.errors);
    }
    next(error);
  }
}
```

### Rate Limiting Pattern

```typescript
import rateLimit from 'express-rate-limit';

export const apiRateLimiter = rateLimit({
  windowMs: 15 * 60 * 1000, // 15 minutes
  max: 100, // limit each IP to 100 requests per windowMs
  standardHeaders: true, // Return rate limit info in the `RateLimit-*` headers
  legacyHeaders: false, // Disable the `X-RateLimit-*` headers
  message: {
    status: 'error',
    error: {
      code: 'RATE_LIMIT_EXCEEDED',
      message: 'Too many requests, please try again later.',
    },
  },
});
```

## 🧪 Testing Patterns

### Mock Pattern

```typescript
// __tests__/mocks/repository-mock.ts
export class MockRepository {
  private data: Record<string, any[]> = {
    users: [],
    models: [],
  };
  
  // Mock implementation methods
  async findById(collection: string, id: string): Promise<any> {
    return this.data[collection].find(item => item.id === id) || null;
  }
  
  async findAll(collection: string, filter?: any): Promise<any[]> {
    return this.data[collection].filter(item => {
      if (!filter) return true;
      
      // Simple filter implementation
      return Object.entries(filter).every(([key, value]) => item[key] === value);
    });
  }
  
  async create(collection: string, data: any): Promise<any> {
    const item = { ...data, id: `[collection]-[Date.now()]` };
    this.data[collection].push(item);
    return item;
  }
  
  // Add data for testing
  addTestData(collection: string, data: any[]): void {
    this.data[collection] = [...data];
  }
  
  // Clear test data
  clear(): void {
    Object.keys(this.data).forEach(key => {
      this.data[key] = [];
    });
  }
}
```

### Integration Test Pattern

```typescript
// __tests__/integration/api.test.ts
import request from 'supertest';
import [createApp] from '../../src/app';
import { setupTestDatabase, cleanupTestDatabase } from '../utils/test-db';

describe('API Integration Tests', () => {
  let app;
  
  beforeAll(async () => {
    // Set up test database
    await setupTestDatabase();
    
    // Create app with test configuration
    app = await createApp({
      testing: true,
      database: {
        url: process.env.TEST_DATABASE_URL,
      },
    });
  });
  
  afterAll(async () => {
    // Clean up test database
    await cleanupTestDatabase();
  });
  
  describe('GET /api/v1/models', () => {
    it('should return list of models', async () => {
      // Act
      const response = await request(app)
        .get('/api/v1/models')
        .set('Authorization', `Bearer [process.env.TEST_TOKEN]`);
      
      // Assert
      expect(response.status).toBe(200);
      expect(response.body.status).toBe('success');
      expect(Array.isArray(response.body.data)).toBe(true);
    });
    
    it('should filter models by type', async () => {
      // Act
      const response = await request(app)
        .get('/api/v1/models?type=embedding')
        .set('Authorization', `Bearer [process.env.TEST_TOKEN]`);
      
      // Assert
      expect(response.status).toBe(200);
      expect(response.body.status).toBe('success');
      expect(response.body.data.every(model => model.type === 'embedding')).toBe(true);
    });
  });
  
  // More tests...
});
```

### Contract Test Pattern

```typescript
// __tests__/contract/api.test.ts
import request from 'supertest';
import [createApp] from '../../src/app';
import [validateAgainstSchema] from '../utils/schema-validator';
import [loadOpenApiSpec] from '../utils/openapi-loader';

describe('API Contract Tests', () => {
  let app;
  let openApiSpec;
  
  beforeAll(async () => {
    // Create app with test configuration
    app = await createApp({
      testing: true,
    });
    
    // Load OpenAPI specification
    openApiSpec = await loadOpenApiSpec('./openapi.yaml');
  });
  
  describe('GET /api/v1/models', () => {
    it('should return response that matches the contract', async () => {
      // Act
      const response = await request(app)
        .get('/api/v1/models')
        .set('Authorization', `Bearer [process.env.TEST_TOKEN]`);
      
      // Assert
      expect(response.status).toBe(200);
      
      // Validate response against OpenAPI schema
      const schemaPath = '#/components/schemas/ModelList';
      const validationResult = validateAgainstSchema(
        response.body,
        schemaPath,
        openApiSpec
      );
      
      expect(validationResult.valid).toBe(true);
      if (!validationResult.valid) {
        console.error('Validation errors:', validationResult.errors);
      }
    });
  });
  
  // More contract tests...
});
```

---

This document provides essential resources for both AI agents and human contributors working with the LLM Platform AI Platform. By following these patterns and practices, you can ensure your code is consistent with the platform's architecture and standards.