# Model Registry Component with ReelCode Integration

This module provides a comprehensive UI for managing LLM models with built-in standards validation powered by ReelCode.

## Features

- **Model Management**: Full CRUD operations for LLM models
- **Deployment Controls**: Deploy models to different environments with configurable strategies
- **Version Management**: Track and roll back to previous versions
- **Standards Validation**: Integrated validation of model configurations against predefined standards
- **Responsive UI**: Works across desktop and mobile viewports

## Integration with ReelCode

The Model Registry components have been enhanced with ReelCode's standards validation capabilities:

1. **ModelStandardsValidator**: A React component that validates model configurations against standards
2. **ModelStandardsUtility**: Utility functions for validation, auto-fixing, and standards configuration
3. **Enhanced ModelForm**: Form component with built-in standards validation

## Installation

The integration uses the published `@bluefly/reelcode` package:

```bash
npm install @bluefly/reelcode
```

## Usage

```tsx
import { 
  ModelRegistry, 
  ModelRegistryProvider,
  validateModelStandards
} from './components/model-registry';

// Basic usage
const MyModelRegistryPage = () => {
  return (
    <ModelRegistryProvider>
      <ModelRegistry />
    </ModelRegistryProvider>
  );
};

// With custom handlers
const MyCustomModelRegistry = () => {
  const handleModelSelect = (model) => {
    console.log('Selected model:', model);
  };
  
  const handleModelCreate = (model) => {
    console.log('Created model:', model);
  };
  
  return (
    <ModelRegistryProvider>
      <ModelRegistry
        onModelSelect=[handleModelSelect]
        onModelCreate=[handleModelCreate]
        showActions=[true]
        searchable=[true]
        filterable=[true]
      />
    </ModelRegistryProvider>
  );
};

// Using the validation utility directly
const validateMyModel = (model) => {
  const result = validateModelStandards(model);
  
  if (result.valid) {
    console.log('Model is valid!');
  } else {
    console.error('Validation errors:', result.errors);
    console.warn('Validation warnings:', result.warnings);
  }
};
```

## Implementation Details

### Model Forms with Standards Validation

The `ModelForm` component now includes standards validation with auto-fix capability:

```tsx
<ModelForm 
  model=[selectedModel]
  onSubmit=[handleUpdateModel]
  onCancel={() => setShowEditModal(false)}
  loading=[loading]
  error=[error]
/>
```

This form automatically validates the model configuration against coding standards, showing errors and warnings in real-time. When validation errors are found, it offers to automatically fix the issues.

### Standalone Validation Component

You can also use the validation component independently:

```tsx
<ModelStandardsValidator
  model=[myModel]
  onChange={(isValid, results) => {
    console.log('Validation results:', results);
  }}
  showWarnings=[true]
/>
```

## Standards Configuration

The standards validation is based on the rules from the ReelCode package, which validate:

1. Model naming conventions
2. Semantic versioning
3. Parameter ranges and best practices
4. Deployment configuration requirements
5. Production-specific requirements

## Benefits of ReelCode Integration

- **Standardized Configuration**: Ensures all model configurations follow best practices
- **Reduced Errors**: Catches configuration issues before deployment
- **Automated Fixes**: Suggests and applies fixes for common issues
- **Enhanced UX**: Provides immediate feedback to users on configuration quality
- **Consistency**: Maintains consistent model configurations across teams

## Future Enhancements

- Add support for custom standards definitions
- Implement integration with CI/CD pipelines for automated validation
- Add AI-assisted model configuration generation
- Deeper integration with the ReelCode package's AI capabilities