# Tailwind CSS Migration Guide

## Overview

This document outlines the migration process from Chakra UI to Tailwind CSS for the BFUI component library. The migration is designed to be incremental, allowing both systems to coexist during the transition period.

## Why Migrate to Tailwind CSS?

1. **Performance**: Tailwind's utility-first approach results in smaller bundle sizes
2. **Flexibility**: More direct control over styling without abstraction layers
3. **Ecosystem**: Larger community and broader adoption
4. **Maintainability**: Simpler mental model for styling components
5. **Design System Integration**: Better alignment with our design tokens

## Migration Strategy

### Phase 1: Parallel Implementation (Current)

- New Tailwind-based components are created in the `modern/` directory
- Legacy Chakra UI components remain functional
- Applications can adopt new components incrementally

### Phase 2: Gradual Adoption

- Update applications to use new Tailwind components
- Maintain backwards compatibility for legacy components
- Collect feedback and refine implementation

### Phase 3: Complete Migration

- Remove Chakra UI dependencies
- Consolidate component API
- Finalize documentation

## Using Modern Tailwind Components

### Installation

The modern components are already included in the BFUI package. To use them:

```tsx
// Import from the modern path
import { TailwindButton, TailwindCard } from '@bluefly/bfui/modern';

// Use in your component
function MyComponent() {
  return (
    <TailwindCard title="Modern Card">
      <TailwindButton variant="primary">
        Click Me
      </TailwindButton>
    </TailwindCard>
  );
}
```

### Available Components

| Legacy (Chakra UI) | Modern (Tailwind CSS) | Status |
|--------------------|-----------------------|--------|
| Button | TailwindButton | Complete |
| Card | TailwindCard | Complete |
| Input | TailwindInput | Planned |
| Select | TailwindSelect | Planned |
| Checkbox | TailwindCheckbox | Planned |
| Modal | TailwindModal | Planned |
| ... | ... | ... |

## Component Comparison

### Button Component

#### Chakra UI Button (Legacy)

```tsx
import [Button] from '@bluefly/bfui';

// Usage
<Button
  variant="solid" 
  colorScheme="blue"
  size="md"
  isLoading=[isLoading]
  leftIcon=[<Icon name="edit" />]
>
  Edit Profile
</Button>
```

#### Tailwind CSS Button (Modern)

```tsx
import [TailwindButton] from '@bluefly/bfui/modern';

// Usage
<TailwindButton
  variant="primary"
  size="md"
  isLoading=[isLoading]
  leftIcon=[<Icon name="edit" />]
>
  Edit Profile
</TailwindButton>
```

### API Differences

#### Props Comparison

| Chakra UI Prop | Tailwind CSS Prop | Notes |
|----------------|-------------------|-------|
| colorScheme | N/A | Consolidated into variant |
| variant | variant | Enhanced with additional options |
| size | size | Same API |
| isLoading | isLoading | Same API |
| isDisabled | isDisabled | Same API |
| leftIcon | leftIcon | Same API |
| rightIcon | rightIcon | Same API |
| loadingText | loadingText | Same API |

## Styling Customization

### Theme Extension

The Tailwind components use CSS variables for theming. You can customize the theme by extending the Tailwind configuration:

```js
// tailwind.config.js
module.exports = {
  theme: {
    extend: {
      colors: {
        primary: {
          DEFAULT: 'var(--color-primary)',
          foreground: 'var(--color-primary-foreground)',
        },
        // ... other colors
      },
      // ... other theme extensions
    },
  },
  // ... other config
};
```

### Component Styling

Each component accepts a `className` prop for additional styling:

```tsx
<TailwindButton 
  className="bg-gradient-to-r from-purple-500 to-pink-500 hover:from-pink-500 hover:to-purple-500"
>
  Custom Styled Button
</TailwindButton>
```

## Best Practices

1. **Prefer Modern Components**: Use Tailwind components for new development
2. **Consistent API**: Follow the established component API patterns
3. **Responsive Design**: Use Tailwind's responsive utilities
4. **Dark Mode**: Leverage Tailwind's dark mode support
5. **Accessibility**: Maintain WCAG compliance in all components

## Migration Helpers

### Utility Functions

The `cn()` utility helps with conditional class names:

```tsx
import [cn] from '@bluefly/bfui/utils';

const className = cn(
  'base-class', 
  isActive && 'active-class',
  variant === 'primary' ? 'primary-class' : 'secondary-class'
);
```

### Component Converter

For cases where you need to convert many components, use the provided codemod:

```bash
# Run the converter script
node ./codemods/convert-chakra-to-tailwind.js ./src/components/MyComponent.tsx
```

## Troubleshooting

### Common Issues

1. **Class Conflicts**: Use `cn()` to manage class merging
2. **Style Specificity**: Tailwind uses utility classes that may need `!important` in some cases
3. **Missing Features**: Some Chakra features may need custom implementation

### Getting Help

- Check the [Modern Components Documentation](./components-modern.md)
- Reference the [Tailwind CSS Documentation](https://tailwindcss.com/docs)
- Contact the BFUI team for migration assistance

## Conclusion

The migration to Tailwind CSS represents a significant improvement to our component library's flexibility, performance, and maintainability. By following this incremental approach, we can ensure a smooth transition while maintaining application stability.