# BFUI Build Troubleshooting Guide

This guide provides solutions to common issues encountered when building the BFUI library, particularly focusing on the full build process.

## Understanding Build Configurations

The BFUI project uses two separate build configurations:

1. **Minimal Build** (`npm run build`): Uses `tsup.minimal.config.ts`, only includes essential components and is more stable.
2. **Full Build** (`npm run build:full`): Uses `tsup.config.ts`, includes all components but may have issues with some dependencies.

## Common Issues with Full Build

### 1. Missing Dependencies

**Symptoms:**
- Error messages about missing modules
- Import statements failing to resolve

**Solutions:**
- Check that all dependencies are installed: `npm install`
- Verify peer dependencies are correctly specified in package.json
- Ensure all imports use correct paths and extensions (.js)

### 2. Circular Dependencies

**Symptoms:**
- Error messages about circular dependencies
- Build process hangs or fails

**Solutions:**
- Identify circular imports using a tool like `madge`: `npx madge --circular src/index.ts`
- Break circular dependencies by:
  - Creating intermediate modules
  - Using interface segregation
  - Restructuring component hierarchy

### 3. TypeScript Errors

**Symptoms:**
- Type errors in the build process
- Missing type definitions

**Solutions:**
- Check that all JavaScript files have corresponding `.d.ts` files
- Ensure TypeScript configurations in `tsconfig.json` are correct
- Add missing type declarations for third-party libraries

### 4. Import Path Issues

**Symptoms:**
- Errors about incorrect import paths
- Cannot find module errors

**Solutions:**
- Ensure all imports use the `.js` extension for TypeScript files when imported in other files
- Use consistent path formats (relative vs. absolute)
- Check the `baseUrl` and `paths` in `tsconfig.json`

### 5. Component-Specific Issues

#### Layout Components

```typescript
// Example fix for circular dependency in Layout components
// Before:
import [ThemeProvider] from '../components/ThemeProvider';
import [useMediaQuery] from '../hooks/useMediaQuery';

// After:
import [ThemeProvider] from '../components/ThemeProvider';
// Create a new hook that doesn't import from Layout
import [useResponsive] from '../../hooks/useResponsive';
```

#### DataTable Components

```typescript
// Example fix for DataTable component
// Before:
import [VirtualScrollContainer] from './VirtualScrollContainer';

// After - ensure the path is correct:
import [VirtualScrollContainer] from './VirtualScrollContainer.js';
```

#### Workflow Builder Components

```typescript
// Example fix for Workflow Builder component
// Check for missing exports in index.ts
export [default as ContextMenu] from './components/ContextMenu.js';
export [default as MiniMap] from './components/MiniMap.js';
export [default as Toolbar] from './components/Toolbar.js';
```

#### Notification Components

```typescript
// Example fix for Notification component
// Ensure components are properly exported
export [default as ModalNotification] from './components/ModalNotification.js';
export [default as ProgressNotification] from './components/ProgressNotification.js';
```

## Step-by-Step Debugging Process

1. **Run the full build with verbose logging**:
   ```bash
   npm run build:full -- --verbose
   ```

2. **Identify the first error**:
   Focus on the first error message, as subsequent errors may be cascading effects.

3. **Check file existence**:
   Verify that the file being imported actually exists at the specified path.

4. **Inspect import statements**:
   Ensure all import statements use the `.js` extension for TypeScript files.

5. **Look for circular dependencies**:
   Use the `madge` tool to identify circular dependencies.

6. **Fix one issue at a time**:
   Make a single change, then run the build again to see if the error is resolved.

7. **Add module mocks if needed**:
   For problematic dependencies, consider creating mock implementations.

## Using the Minimal Build

If you're actively developing and don't need all components, use the minimal build:

```bash
npm run build
# or for watch mode
npm run dev
```

This will only include essential components and is less likely to encounter errors.

## Advanced Troubleshooting

### Module Resolution

If you're experiencing module resolution issues, check:

1. The `moduleResolution` setting in `tsconfig.json`
2. The `baseUrl` and `paths` configurations
3. Whether you're using correct import path formats

### Bundle Analysis

To analyze the bundle size and identify problematic dependencies:

```bash
# Install source-map-explorer
npm install --save-dev source-map-explorer

# Analyze the bundle
npx source-map-explorer dist/index.js
```

### Incremental Building

For complex components, try building them separately:

1. Create a separate entry point for problematic components
2. Build them in isolation
3. Integrate them into the main build once they're working correctly

## When to Ask for Help

If you've tried the solutions above and still encounter issues:

1. Document the exact error message
2. Create a minimal reproduction of the issue
3. Share the steps you've already taken to troubleshoot
4. Reach out to the team through the issue tracker

## Conclusion

Building a complex component library can be challenging, but most issues can be resolved by carefully checking dependencies, import paths, and component structure. Use the minimal build for development and troubleshoot the full build incrementally.

Remember that the most common issues are:
- Missing or incorrect import paths
- Circular dependencies
- TypeScript configuration issues
- Missing type definitions

By systematically addressing these issues, you can successfully build the entire BFUI library.