# LLM Platform Installation Guide

Complete reference for the `llmcli llm platform:install` command.

## Overview

The `llm platform:install` command provides comprehensive LLM platform setup with Drupal integration. It supports multiple AI providers, vector databases, security configurations, and development workflows.

## Command Syntax

```bash
llmcli llm platform:install [options]
```

## Required Options

- `--working-dir <path>` - Installation directory (required for actual installation)

## Basic Configuration Options

| Option | Description | Default | Example |
|--------|-------------|---------|---------|
| `--project-name <name>` | Project name | `llm-platform` | `--project-name my-app` |
| `--working-dir <path>` | Installation directory | Current directory | `--working-dir ./my-project` |
| `--site-name <name>` | Drupal site name | `LLM Platform` | `--site-name "My AI Site"` |
| `--admin-user <user>` | Admin username | `admin` | `--admin-user administrator` |
| `--admin-pass <pass>` | Admin password | `admin` | `--admin-pass secure123` |
| `--profile <profile>` | Installation profile | `standard` | `--profile minimal` |
| `--drupal-version <version>` | Drupal version | `10` | `--drupal-version 11` |

## AI Provider Configuration

### Primary Provider
```bash
--provider <provider>
```

Supported providers:
- `openai` - OpenAI GPT models (GPT-4, GPT-3.5-turbo)
- `anthropic` - Anthropic Claude models (Claude-3, Claude-2)
- `google-gemini` - Google Gemini models
- `mistral` - Mistral AI models
- `fireworks` - Fireworks AI models
- `huggingface` - Hugging Face models
- `ollama` - Local Ollama models
- `lmstudio` - LM Studio models

### Additional Configuration
```bash
--suggest-providers "openai,anthropic,ollama"    # Multiple providers
--ai-models "gpt-4,claude-3-opus,llama2"         # Specific models
--model-fallback enabled                         # Enable fallback
--model-routing "round-robin"                    # Routing strategy
--model-caching "standard"                       # Caching level
```

## Vector Database Configuration

### Enable Vector Database
```bash
--with-vector-db
```

### Vector Database Types
```bash
--vector-db <type>
```

| Type | Description | Service Type | Configuration |
|------|-------------|--------------|---------------|
| `weaviate` | Weaviate vector DB | DDEV service | Port 8080, gRPC 50051 |
| `milvus` | Milvus vector DB | DDEV service | Port 19530, gRPC 19531 |
| `qdrant` | Qdrant vector DB | DDEV service | Port 6333, gRPC 6334 |
| `pinecone` | Pinecone cloud | Cloud service | Requires API key |
| `postgres` | PostgreSQL + pgvector | DDEV service | Port 5432, pgvector extension |
| `azure-ai-search` | Azure AI Search | Cloud service | Requires API key + endpoint |
| `sqlite` | SQLite vector support | Embedded | Local file, no service |

### Embedding Configuration
```bash
--embedding-models "text-embedding-ada-002,text-embedding-3-small"
--vector-dimensions 1536
--similarity-threshold 0.8
```

## Feature Flags

| Flag | Description |
|------|-------------|
| `--with-ai-services` | Include AI services integration |
| `--with-ai-swarm` | Include AI swarm orchestration |
| `--with-secure` | Include security hardening |
| `--with-alternative-services` | Include Alternative Services integration |
| `--with-recipe-onboarding` | Include Recipe Onboarding framework |
| `--with-ddev-orchestration` | Include DDEV service orchestration |
| `--with-automated-testing` | Include automated testing setup |

## DDEV Service Configuration

### Available Services
```bash
--ddev-services "service1,service2,service3"
```

**Database Services:**
- `mariadb` - MariaDB database
- `mysql` - MySQL database  
- `postgres` - PostgreSQL database

**Caching Services:**
- `redis` - Redis cache
- `memcached` - Memcached

**Search Services:**
- `elasticsearch` - Elasticsearch
- `opensearch` - OpenSearch

**Email Testing:**
- `mailpit` - Modern mail testing
- `mailhog` - Mail capture service

**Storage:**
- `minio` - S3-compatible object storage

**Vector Databases:**
- `weaviate` - Weaviate vector DB
- `milvus` - Milvus vector DB
- `qdrant` - Qdrant vector DB

### Custom DDEV Project Name
```bash
--ddev-name custom-project-name
```

## Template Presets

### Enterprise Template
```bash
--template enterprise
```
**Includes:**
- Vector database (Weaviate)
- AI swarm orchestration
- Maximum security
- Production performance mode
- Aggressive caching
- 95% test coverage
- Comprehensive logging

### Development Template
```bash
--template development
```
**Includes:**
- Guided recipe onboarding
- Standard security
- Alternative services
- 80% test coverage
- Development-friendly defaults

### Minimal Template
```bash
--template minimal
```
**Includes:**
- Basic features only
- No vector database
- No AI swarm
- Basic security
- Minimal resources

## Security Configuration

### Security Levels
```bash
--security-level <level>
```
- `basic` - Basic security features
- `standard` - Standard security configuration
- `maximum` - Maximum security hardening

### Advanced Security Options
```bash
--encryption-at-rest enabled
--security-headers "csp,hsts,xss_protection"
```

**Security Headers:**
- `csp` - Content Security Policy
- `hsts` - HTTP Strict Transport Security
- `xss_protection` - XSS Protection headers

## Performance Configuration

### Performance Modes
```bash
--performance-mode <mode>
```
- `development` - Development optimizations
- `balanced` - Balanced performance
- `production` - Production optimizations

### Caching Strategies
```bash
--caching-strategy <strategy>
```
- `minimal` - Minimal caching
- `standard` - Standard caching
- `aggressive` - Aggressive caching

## Development Options

### Interactive Setup
```bash
--ui-wizard                          # Run interactive wizard
--wizard-suggest-additional          # Suggest additional features
--development-mode true              # Enable development mode
--interactive                        # Enable interactive setup
--env-detect                         # Auto-detect environment
```

### Recipe Configuration
```bash
--use-recipe "drupal_cms,ai_starter"     # Use specific recipes
--recipe-mode guided                     # guided | expert
--recipe-development true                # Recipe development mode
--recipe-validation strict               # strict | loose
```

## Testing Configuration

```bash
--test-runner jest                       # jest | phpunit
--coverage-threshold 85                  # Coverage percentage
--with-automated-testing                 # Include testing setup
```

## Path Configuration

```bash
--token-location /path/to/tokens        # Token storage location
--common-npm-path /path/to/npm           # Common NPM packages
--workspace-root /path/to/workspace      # Workspace root
--recipes-path /path/to/recipes          # Recipes directory
```

## Build & Package Management

```bash
--npm-link true                          # Use npm link
--with-local-packages                    # Use local packages
--auto-npm-install                       # Auto install npm packages
--auto-composer-install                  # Auto install composer packages
```

## Documentation Generation

```bash
--generate-docs "api,admin,developer"    # Generate documentation types
--docs-format "markdown,html"            # Documentation formats
```

## Suggestion Options

```bash
--suggest-integrations "cms,ecommerce"   # Suggested integrations
--suggest-security "2fa,audit"           # Suggested security features
--suggest-performance "cdn,cache"        # Suggested performance features
--suggest-content "multilingual,seo"     # Suggested content features
--suggest-development "testing,ci"       # Suggested development tools
```

## Configuration Examples

### Complete Enterprise Setup
```bash
llmcli llm platform:install \
  --template enterprise \
  --working-dir ./enterprise-llm \
  --project-name enterprise-ai \
  --site-name "Enterprise AI Platform" \
  --admin-user administrator \
  --admin-pass secure_password_123 \
  --drupal-version 10 \
  --provider openai \
  --suggest-providers "anthropic,google-gemini" \
  --ai-models "gpt-4,gpt-3.5-turbo,claude-3-opus" \
  --model-fallback enabled \
  --model-routing "load-balance" \
  --with-vector-db \
  --vector-db weaviate \
  --embedding-models "text-embedding-ada-002" \
  --vector-dimensions 1536 \
  --similarity-threshold 0.85 \
  --with-ai-swarm \
  --with-secure \
  --security-level maximum \
  --encryption-at-rest enabled \
  --security-headers "csp,hsts,xss_protection" \
  --performance-mode production \
  --caching-strategy aggressive \
  --with-ddev-orchestration \
  --ddev-services "postgres,redis,elasticsearch,weaviate" \
  --ddev-name enterprise-ai-platform \
  --with-automated-testing \
  --test-runner jest \
  --coverage-threshold 95 \
  --generate-docs "api,admin,developer" \
  --auto-npm-install \
  --auto-composer-install
```

### Development Setup with Local AI
```bash
llmcli llm platform:install \
  --template development \
  --working-dir ./dev-ai \
  --project-name dev-ai-platform \
  --site-name "Development AI Platform" \
  --provider ollama \
  --ai-models "llama2,mistral,codellama" \
  --vector-db sqlite \
  --embedding-models "BAAI/bge-small-en-v1.5" \
  --security-level standard \
  --performance-mode development \
  --development-mode true \
  --ui-wizard \
  --with-recipe-onboarding \
  --recipe-mode guided \
  --use-recipe "drupal_cms" \
  --with-automated-testing \
  --coverage-threshold 80 \
  --env-detect
```

### Minimal Cloud Setup
```bash
llmcli llm platform:install \
  --template minimal \
  --working-dir ./minimal-ai \
  --project-name minimal-platform \
  --provider openai \
  --ai-models "gpt-3.5-turbo" \
  --vector-db pinecone \
  --security-level basic \
  --performance-mode balanced
```

### Multi-Provider Research Setup
```bash
llmcli llm platform:install \
  --working-dir ./research-platform \
  --project-name ai-research \
  --provider openai \
  --suggest-providers "anthropic,google-gemini,huggingface,ollama" \
  --ai-models "gpt-4,claude-3-opus,gemini-pro,llama2" \
  --model-routing "round-robin" \
  --model-fallback enabled \
  --with-vector-db \
  --vector-db postgres \
  --embedding-models "text-embedding-ada-002,BAAI/bge-large-en-v1.5" \
  --vector-dimensions 1536 \
  --ddev-services "postgres,redis" \
  --security-level standard \
  --with-automated-testing \
  --coverage-threshold 85
```

## Generated Project Structure

After installation, your project will have this structure:

```
project-name/
├── .ddev/                    # DDEV configuration
│   ├── config.yaml
│   └── docker-compose.*.yaml
├── config/
│   ├── sync/                 # Drupal configuration sync
│   ├── security/             # Security configuration
│   │   └── settings.json
│   ├── ai/                   # AI service configuration
│   │   └── services.json
│   └── performance.json      # Performance configuration
├── web/                      # Drupal web root
│   ├── modules/custom/       # Custom Drupal modules
│   ├── themes/custom/        # Custom themes
│   ├── sites/default/
│   │   └── settings.php
│   └── .htaccess            # Security headers (if enabled)
├── scripts/                  # Build and deployment scripts
├── tests/                    # Test files
│   ├── phpunit/             # PHP unit tests
│   └── jest/                # JavaScript tests
├── docs/                     # Generated documentation
│   ├── README.md
│   ├── API.md               # API documentation (if generated)
│   └── ADMIN.md            # Admin guide (if generated)
├── recipes/                  # Drupal recipes
├── vendor/                   # Composer dependencies
├── node_modules/            # Node.js dependencies (if enabled)
├── composer.json            # PHP dependencies
├── package.json             # Node.js dependencies (if enabled)
├── .env                     # Environment configuration
├── .gitignore               # Git ignore file
└── README.md               # Project documentation
```

## Environment Configuration

The `.env` file contains:

```bash
# Site Configuration
SITE_NAME="Your Site Name"
ENVIRONMENT=development|production

# Database
DATABASE_URL=mysql://db:db@db:3306/db

# AI Providers (based on selection)
OPENAI_API_KEY=your_openai_api_key_here
ANTHROPIC_API_KEY=your_anthropic_api_key_here
GOOGLE_GEMINI_API_KEY=your_gemini_api_key_here

# Vector Database (if configured)
VECTOR_DB_TYPE=weaviate
VECTOR_DB_HOST=localhost
VECTOR_DB_PORT=8080

# Security (if configured)
ENCRYPTION_KEY=generated_encryption_key
```

## Troubleshooting

### Common Issues

**Missing working directory:**
```bash
error: --working-dir is required for installation
```
Solution: Always specify `--working-dir` for actual installations.

**DDEV service conflicts:**
```bash
Service 'postgres' already exists
```
Solution: Use `--ddev-name` to specify a unique project name.

**Vector database connection issues:**
```bash
Failed to connect to vector database
```
Solution: Ensure the vector database service is running and ports are available.

### Debug Mode

Use `--dry-run` to test configurations without making changes:

```bash
llmcli llm platform:install [your-options] --dry-run
```

This shows exactly what would be configured without performing the installation.

## Integration Notes

### Drupal AI Module
The installation is designed to work with the [Drupal AI module](https://www.drupal.org/project/ai):
- Automatic provider configuration
- Vector database integration
- API key management
- Module activation

### DDEV Integration
- Automatic service configuration
- Docker container management
- Port management
- Environment setup

### TDDAI Workers
- Alternative Services integration
- Recipe Onboarding framework
- Test automation
- Interactive setup

## Best Practices

1. **Use templates** for consistent setups
2. **Test with dry-run** before actual installation
3. **Use version control** for configuration tracking
4. **Set proper security levels** for your environment
5. **Configure monitoring** for production setups
6. **Use local AI** for development to reduce API costs
7. **Set appropriate coverage thresholds** for your team

## Support

For issues or questions:
1. Check the troubleshooting section
2. Use `--dry-run` to verify configuration
3. Review generated documentation
4. Check DDEV service status
5. Verify API keys and credentials