# Secure Project API Reference

## Table of Contents

1. [Core Services](#core-services)
   - [AuditLogger](#auditlogger)
   - [PolicyEngine](#policyengine)
   - [DataSeparationAuditor](#dataseparationauditor)
   - [SecurityAlertManager](#securityalertmanager)
   - [ComplianceFrameworkEngine](#complianceframeworkengine)

2. [Security Services](#security-services)
   - [IncidentResponseService](#incidentresponseservice)
   - [VulnerabilityScanner](#vulnerabilityscanner)
   - [ThreatIntelligenceService](#threatintelligenceservice)
   - [ComplianceReportGenerator](#compliancereportgenerator)
   - [RiskAssessmentEngine](#riskassessmentengine)

3. [Framework Adapters](#framework-adapters)
   - [ExpressAdapter](#expressadapter)
   - [FastifyAdapter](#fastifyadapter)
   - [KoaAdapter](#koaadapter)
   - [DrupalAdapter](#drupaladapter)

## Security Services

### IncidentResponseService

The IncidentResponseService provides comprehensive incident response management following the NIST incident response framework.

#### Import

```typescript
import [IncidentResponseService] from '@bluefly/secure-project';
```

#### Constructor

```typescript
const incidentService = new IncidentResponseService(config?: Partial<IncidentResponseConfig>);
```

**Configuration Options:**

```typescript
interface IncidentResponseConfig {
  storage: 'memory' | 'database';
  enableNotifications: boolean;
  enableAutomation: boolean;
  
  // SLA configuration (in minutes)
  sla: {
    critical: number;  // default: 15
    high: number;      // default: 60
    medium: number;    // default: 240
    low: number;       // default: 1440
  };
  
  // Notification channels
  notifications: {
    email?: {
      smtp: { host: string; port: number; secure: boolean };
      from: string;
    };
    slack?: {
      webhook: string;
    };
    webhook?: {
      url: string;
      headers?: Record<string, string>;
    };
  };
}
```

#### Methods

##### createIncident

Creates a new security incident.

```typescript
async createIncident(params: {
  title: string;
  description: string;
  severity: IncidentSeverity;
  type: IncidentType;
  source: IncidentSource;
  affectedSystems?: string[];
  affectedUsers?: string[];
  indicators?: string[];
  initialFindings?: string;
}): Promise<Incident>
```

**Example:**

```typescript
const incident = await incidentService.createIncident({
  title: 'Suspicious Login Activity Detected',
  description: 'Multiple failed login attempts from unknown IP addresses',
  severity: 'high',
  type: 'security_breach',
  source: 'automated',
  affectedSystems: ['auth-service'],
  affectedUsers: ['user123', 'user456'],
  indicators: ['192.168.1.100', 'failed_login_threshold_exceeded']
});
```

##### updateIncident

Updates an existing incident's status or details.

```typescript
async updateIncident(
  incidentId: string,
  update: Partial<Incident>
): Promise<Incident>
```

##### assignResponder

Assigns a responder to an incident.

```typescript
async assignResponder(
  incidentId: string,
  responder: {
    id: string;
    name: string;
    email: string;
    role: 'lead' | 'member' | 'observer';
  }
): Promise<void>
```

##### addTimelineEvent

Adds an event to the incident timeline.

```typescript
async addTimelineEvent(
  incidentId: string,
  event: {
    description: string;
    type: 'detection' | 'containment' | 'eradication' | 'recovery' | 'lessons_learned' | 'communication' | 'escalation';
    actor: string;
    evidence?: any;
  }
): Promise<void>
```

##### collectEvidence

Collects and stores evidence related to an incident.

```typescript
async collectEvidence(
  incidentId: string,
  evidence: {
    type: 'log' | 'screenshot' | 'memory_dump' | 'network_capture' | 'file' | 'other';
    description: string;
    data: any;
    hash?: string;
    collectedBy: string;
  }
): Promise<void>
```

##### executePlaybook

Executes a predefined response playbook.

```typescript
async executePlaybook(
  incidentId: string,
  playbookId: string
): Promise<void>
```

##### generatePostMortem

Generates a post-mortem report for a closed incident.

```typescript
async generatePostMortem(incidentId: string): Promise<{
  incident: Incident;
  duration: number;
  timeToDetect: number;
  timeToContain: number;
  timeToResolve: number;
  affectedAssets: { systems: number; users: number; data: number };
  rootCause?: string;
  lessonsLearned: string[];
  recommendations: string[];
}>
```

#### Events

The service emits the following events:

- `incident:created` - When a new incident is created
- `incident:updated` - When an incident is updated
- `incident:assigned` - When a responder is assigned
- `incident:escalated` - When an incident is escalated
- `incident:resolved` - When an incident is resolved
- `incident:sla_breach` - When an SLA is breached

**Example:**

```typescript
incidentService.on('incident:sla_breach', (data) => {
  console.log(`SLA breached for incident [data.incident.id]`);
});
```

### VulnerabilityScanner

The VulnerabilityScanner provides automated vulnerability detection and management across multiple layers.

#### Import

```typescript
import [VulnerabilityScanner] from '@bluefly/secure-project';
```

#### Constructor

```typescript
const scanner = new VulnerabilityScanner(config?: Partial<VulnerabilityScannerConfig>);
```

**Configuration Options:**

```typescript
interface VulnerabilityScannerConfig {
  storage: 'memory' | 'database';
  enableAutomation: boolean;
  enableScheduling: boolean;
  
  // Scanning configuration
  scanning: {
    parallel: boolean;
    maxConcurrent: number;
    timeout: number;
  };
  
  // CVE database settings
  cveDatabase: {
    updateInterval: number;  // hours
    sources: string[];
  };
  
  // External scanner integrations
  scanners?: {
    dependency?: { 
      enabled: boolean; 
      tool: 'npm-audit' | 'snyk' | 'owasp-dependency-check'; 
    };
    code?: { 
      enabled: boolean; 
      tool: 'sonarqube' | 'checkmarx' | 'veracode'; 
    };
    infrastructure?: { 
      enabled: boolean; 
      tool: 'nessus' | 'qualys' | 'openvas'; 
    };
  };
}
```

#### Methods

##### scan

Performs a vulnerability scan on a target.

```typescript
async scan(
  target: ScanTarget,
  options?: Partial<ScanOptions>
): Promise<ScanResult>
```

**ScanTarget Types:**

```typescript
type ScanTarget = 
  | { type: 'package'; name: string; version?: string; ecosystem: 'npm' | 'maven' | 'pypi' | 'rubygems'; }
  | { type: 'repository'; url: string; branch?: string; }
  | { type: 'container'; image: string; tag?: string; registry?: string; }
  | { type: 'host'; hostname: string; ip?: string; ports?: number[]; }
  | { type: 'application'; url: string; authentication?: any; };
```

**Example:**

```typescript
const result = await scanner.scan(
  { type: 'package', name: 'express', version: '4.17.1', ecosystem: 'npm' },
  { 
    scanTypes: ['dependency', 'known_vulnerabilities'],
    deep: true
  }
);

console.log(`Found [result.vulnerabilities.length] vulnerabilities`);
```

##### scanMultiple

Scans multiple targets in parallel.

```typescript
async scanMultiple(
  targets: ScanTarget[],
  options?: Partial<ScanOptions>
): Promise<ScanResult[]>
```

##### getVulnerability

Retrieves details about a specific vulnerability.

```typescript
async getVulnerability(id: string): Promise<Vulnerability | undefined>
```

##### searchVulnerabilities

Searches for vulnerabilities based on criteria.

```typescript
searchVulnerabilities(criteria: {
  severity?: VulnerabilitySeverity;
  status?: VulnerabilityStatus;
  component?: string;
  cve?: string;
  dateRange?: { start: Date; end: Date };
}): Vulnerability[]
```

##### assessVulnerability

Updates the assessment status of a vulnerability.

```typescript
async assessVulnerability(
  vulnerabilityId: string,
  assessment: {
    exploitability: 'high' | 'medium' | 'low' | 'none';
    impact: 'critical' | 'high' | 'medium' | 'low';
    riskAccepted?: boolean;
    notes?: string;
  }
): Promise<void>
```

##### createRemediationPlan

Creates a remediation plan for vulnerabilities.

```typescript
async createRemediationPlan(
  vulnerabilityIds: string[],
  plan: {
    strategy: 'patch' | 'upgrade' | 'replace' | 'mitigate' | 'accept';
    timeline: string;
    assignee?: string;
    steps?: string[];
  }
): Promise<RemediationPlan>
```

##### scheduleScan

Schedules recurring vulnerability scans.

```typescript
scheduleScan(schedule: {
  name: string;
  targets: ScanTarget[];
  scanOptions: ScanOptions;
  frequency: 'hourly' | 'daily' | 'weekly' | 'monthly';
  enabled: boolean;
}): ScanSchedule
```

#### Events

- `vulnerability:detected` - When a new vulnerability is found
- `vulnerability:assessed` - When a vulnerability is assessed
- `scan:started` - When a scan begins
- `scan:completed` - When a scan completes
- `scan:failed` - When a scan fails
- `remediation:created` - When a remediation plan is created

### ThreatIntelligenceService

The ThreatIntelligenceService manages threat intelligence, indicators of compromise (IoCs), and threat actor information.

#### Import

```typescript
import [ThreatIntelligenceService] from '@bluefly/secure-project';
```

#### Constructor

```typescript
const threatService = new ThreatIntelligenceService(config?: Partial<ThreatIntelConfig>);
```

#### Methods

##### addIndicator

Adds a new indicator of compromise (IoC).

```typescript
async addIndicator(indicator: {
  type: IoType;
  value: string;
  confidence: IndicatorConfidence;
  category: ThreatCategory;
  severity: ThreatSeverity;
  source: string;
  description?: string;
  tags?: string[];
  ttl?: number;
}): Promise<IndicatorOfCompromise>
```

**IoC Types:**
- `ip` - IP addresses
- `domain` - Domain names
- `url` - URLs
- `hash` - File hashes (MD5, SHA1, SHA256)
- `email` - Email addresses
- `file_path` - File paths
- `registry_key` - Windows registry keys
- `user_agent` - HTTP user agents

**Example:**

```typescript
const ioc = await threatService.addIndicator({
  type: 'ip',
  value: '192.168.1.100',
  confidence: 'high',
  category: 'malware',
  severity: 'critical',
  source: 'internal_detection',
  description: 'C2 server for ransomware campaign',
  tags: ['ransomware', 'c2', 'active'],
  ttl: 7 * 24 * 60 * 60 * 1000  // 7 days
});
```

##### checkIndicator

Checks if a value matches any known indicators.

```typescript
async checkIndicator(
  type: IoType,
  value: string
): Promise<ThreatCheckResult>
```

##### addThreatActor

Adds information about a threat actor.

```typescript
async addThreatActor(actor: {
  name: string;
  aliases?: string[];
  type: 'nation_state' | 'organized_crime' | 'hacktivist' | 'insider' | 'unknown';
  sophistication: 'advanced' | 'intermediate' | 'novice';
  motivation: Array<'financial' | 'espionage' | 'destruction' | 'ideology' | 'unknown'>;
  ttps?: string[];  // MITRE ATT&CK TTPs
  targets?: string[];
  active: boolean;
}): Promise<ThreatActor>
```

##### addCampaign

Tracks a threat campaign.

```typescript
async addCampaign(campaign: {
  name: string;
  description: string;
  actors?: string[];
  indicators?: string[];
  objectives?: string[];
  active: boolean;
  startDate?: Date;
  endDate?: Date;
}): Promise<ThreatCampaign>
```

##### correlateIndicators

Finds relationships between indicators.

```typescript
async correlateIndicators(
  indicatorId: string,
  options?: {
    maxDepth?: number;
    includeIndirect?: boolean;
  }
): Promise<{
  direct: IndicatorOfCompromise[];
  indirect: IndicatorOfCompromise[];
  actors: ThreatActor[];
  campaigns: ThreatCampaign[];
}>
```

##### exportSTIX

Exports threat intelligence in STIX format.

```typescript
exportSTIX(): any
```

##### importSTIX

Imports threat intelligence from STIX format.

```typescript
async importSTIX(bundle: any): Promise<{
  indicators: number;
  actors: number;
  campaigns: number;
  errors: string[];
}>
```

#### Events

- `threat:detected` - When a threat is detected
- `indicator:added` - When a new IoC is added
- `indicator:matched` - When an IoC matches
- `actor:identified` - When a threat actor is identified
- `campaign:detected` - When a campaign is detected

### ComplianceReportGenerator

The ComplianceReportGenerator creates comprehensive compliance reports for multiple frameworks.

#### Import

```typescript
import [ComplianceReportGenerator] from '@bluefly/secure-project';
```

#### Constructor

```typescript
const reportGenerator = new ComplianceReportGenerator(config?: Partial<ComplianceReportConfig>);
```

#### Methods

##### generateReport

Generates a compliance report for a specific framework.

```typescript
async generateReport(params: {
  framework: ComplianceFramework;
  templateId?: string;
  period: { start: Date; end: Date };
  format?: ReportFormat;
  generatedBy: string;
}): Promise<ComplianceReport>
```

**Supported Frameworks:**
- `soc2` - SOC 2 Type II
- `hipaa` - HIPAA Compliance
- `gdpr` - GDPR Compliance
- `pci_dss` - PCI-DSS Compliance
- `iso27001` - ISO 27001
- `nist` - NIST Cybersecurity Framework

**Example:**

```typescript
const report = await reportGenerator.generateReport({
  framework: 'soc2',
  period: {
    start: new Date('2024-01-01'),
    end: new Date('2024-03-31')
  },
  format: 'pdf',
  generatedBy: 'compliance-team'
});

console.log(`Report generated with [report.findings.length] findings`);
console.log(`Compliance score: [report.complianceScore]%`);
```

##### createSchedule

Creates a recurring report schedule.

```typescript
createSchedule(schedule: {
  name: string;
  template: ReportTemplate;
  period: ReportPeriod;
  format: ReportFormat;
  recipients: string[];
  enabled: boolean;
}): ReportSchedule
```

##### exportReport

Exports a report in various formats.

```typescript
async exportReport(
  reportId: string,
  format: ReportFormat
): Promise<string>  // Returns URL or data URI
```

**Supported Formats:**
- `pdf` - PDF document
- `html` - HTML report
- `json` - JSON data
- `csv` - CSV export (findings only)
- `excel` - Excel workbook

##### getMetrics

Gets compliance metrics and trends.

```typescript
getMetrics(): ReportMetrics
```

**Returns:**

```typescript
interface ReportMetrics {
  totalReports: number;
  reportsByFramework: Record<ComplianceFramework, number>;
  averageComplianceScore: number;
  complianceScoreTrend: Array<{ date: Date; score: number }>;
  topFindings: Array<{ type: string; count: number }>;
}
```

#### Report Structure

Generated reports include:

1. **Executive Summary** - High-level overview and compliance status
2. **Compliance Score** - Overall percentage score
3. **Findings** - Detailed list of gaps and issues
4. **Evidence** - Supporting documentation
5. **Metrics** - Key performance indicators
6. **Recommendations** - Prioritized improvement actions

### RiskAssessmentEngine

The RiskAssessmentEngine provides comprehensive risk assessment and management capabilities.

#### Import

```typescript
import [RiskAssessmentEngine] from '@bluefly/secure-project';
```

#### Constructor

```typescript
const riskEngine = new RiskAssessmentEngine(config?: Partial<RiskAssessmentConfig>);
```

#### Methods

##### identifyRisk

Identifies and registers a new risk.

```typescript
async identifyRisk(params: {
  title: string;
  description: string;
  category: RiskCategory;
  owner: string;
  causes?: string[];
  consequences?: string[];
  affectedAssets?: string[];
}): Promise<Risk>
```

**Risk Categories:**
- `strategic` - Strategic business risks
- `operational` - Operational risks
- `financial` - Financial risks
- `compliance` - Compliance and regulatory risks
- `reputational` - Reputational risks
- `security` - Information security risks
- `technical` - Technical/technology risks

**Example:**

```typescript
const risk = await riskEngine.identifyRisk({
  title: 'Data Breach Risk',
  description: 'Risk of unauthorized access to customer data',
  category: 'security',
  owner: 'security-team',
  causes: ['Weak access controls', 'Unpatched vulnerabilities'],
  consequences: ['Financial loss', 'Regulatory fines', 'Reputation damage'],
  affectedAssets: ['customer-database', 'payment-system']
});
```

##### assessRisk

Assesses a risk's likelihood and impact.

```typescript
async assessRisk(
  riskId: string,
  assessment: {
    likelihood: Likelihood;
    impact: Impact;
    vulnerabilities?: string[];
    threats?: string[];
  }
): Promise<Risk>
```

**Likelihood Levels:**
- `almost_certain` - >90% chance
- `likely` - 70-90% chance
- `possible` - 30-70% chance
- `unlikely` - 10-30% chance
- `rare` - <10% chance

**Impact Levels:**
- `catastrophic` - Severe impact on organization
- `major` - Significant impact
- `moderate` - Moderate impact
- `minor` - Minor impact
- `insignificant` - Negligible impact

##### addControl

Adds a control to mitigate a risk.

```typescript
async addControl(
  riskId: string,
  control: {
    name: string;
    description: string;
    type: 'preventive' | 'detective' | 'corrective' | 'compensating';
    implementationStatus: 'planned' | 'partial' | 'full';
    effectiveness: number;  // 0-100%
    implementationCost: number;
    operationalCost: number;
  }
): Promise<Control>
```

##### createTreatmentPlan

Creates a treatment plan for a risk.

```typescript
async createTreatmentPlan(
  riskId: string,
  plan: {
    strategy: TreatmentStrategy;
    description: string;
    actions: Array<{
      description: string;
      assignee: string;
      dueDate: Date;
    }>;
    expectedResidualLikelihood: Likelihood;
    expectedResidualImpact: Impact;
    budget?: number;
  }
): Promise<TreatmentPlan>
```

**Treatment Strategies:**
- `accept` - Accept the risk
- `avoid` - Avoid the risk entirely
- `transfer` - Transfer risk (e.g., insurance)
- `mitigate` - Reduce likelihood or impact

##### getRiskRegister

Gets the complete risk register.

```typescript
getRiskRegister(): RiskRegister
```

##### getHeatMap

Generates a risk heat map for visualization.

```typescript
getHeatMap(): {
  matrix: Record<Likelihood, Record<Impact, Risk[]>>;
  summary: Record<RiskSeverity, number>;
}
```

##### setRiskAppetite

Sets the organization's risk appetite.

```typescript
setRiskAppetite(appetite: {
  category: RiskCategory;
  appetiteLevel: 'averse' | 'minimal' | 'cautious' | 'open' | 'hungry';
  maxInherentRiskScore: number;
  maxResidualRiskScore: number;
  tolerances: Array<{
    metric: string;
    threshold: number;
    unit: string;
  }>;
}): RiskAppetite
```

#### Events

- `risk:identified` - When a new risk is identified
- `risk:assessed` - When a risk is assessed
- `risk:review_due` - When a risk review is due
- `control:added` - When a control is added
- `treatment:created` - When a treatment plan is created
- `appetite:exceeded` - When risk exceeds appetite

## Integration Examples

### Complete Security Setup

```typescript
import {
  AuditLogger,
  PolicyEngine,
  SecurityAlertManager,
  IncidentResponseService,
  VulnerabilityScanner,
  ThreatIntelligenceService,
  ComplianceReportGenerator,
  RiskAssessmentEngine
} from '@bluefly/secure-project';

// Initialize core services
const auditLogger = new AuditLogger({ storage: 'database' });
const policyEngine = new PolicyEngine({ storage: 'database' });
const alertManager = new SecurityAlertManager({ 
  channels: ['email', 'slack'] 
});

// Initialize security services
const incidentService = new IncidentResponseService({
  enableNotifications: true,
  notifications: {
    slack: { webhook: process.env.SLACK_WEBHOOK }
  }
});

const scanner = new VulnerabilityScanner({
  enableScheduling: true,
  scanners: {
    dependency: { enabled: true, tool: 'npm-audit' }
  }
});

const threatService = new ThreatIntelligenceService({
  enableFeeds: true,
  threatFeeds: ['abuse-ch', 'alienvault-otx']
});

const reportGenerator = new ComplianceReportGenerator({
  enableScheduling: true,
  defaultFormat: 'pdf'
});

const riskEngine = new RiskAssessmentEngine({
  enableMonitoring: true,
  reviewCycles: {
    critical: 7,
    high: 30
  }
});

// Wire up event handlers
incidentService.on('incident:created', async (incident) => {
  // Log to audit trail
  await auditLogger.log({
    userId: 'system',
    action: 'incident_created',
    entityType: 'incident',
    entityId: incident.id,
    details: { severity: incident.severity }
  });
  
  // Check threat intelligence
  for (const indicator of incident.indicators || []) {
    const threat = await threatService.checkIndicator('ip', indicator);
    if (threat.matched) {
      await incidentService.updateIncident(incident.id, {
        findings: `Matched threat indicator: [threat.indicator.description]`
      });
    }
  }
});

scanner.on('vulnerability:detected', async (vuln) => {
  if (vuln.severity === 'critical') {
    // Create incident for critical vulnerabilities
    await incidentService.createIncident({
      title: `Critical Vulnerability: [vuln.cve]`,
      description: vuln.description,
      severity: 'high',
      type: 'vulnerability',
      source: 'automated',
      affectedSystems: [vuln.component]
    });
  }
});

// Schedule regular compliance reports
reportGenerator.createSchedule({
  name: 'Quarterly SOC2 Report',
  template: reportGenerator.getTemplate('soc2'),
  period: 'quarterly',
  format: 'pdf',
  recipients: ['compliance@example.com'],
  enabled: true
});
```

### Express.js Integration

```typescript
import express from 'express';
import [ExpressAdapter] from '@bluefly/secure-project';

const app = express();

// Apply security middleware
const securityAdapter = new ExpressAdapter({
  auditLogger,
  policyEngine,
  alertManager
});

app.use(securityAdapter.middleware());

// Protected endpoint
app.get('/api/sensitive-data', async (req, res) => {
  // Access is automatically checked by middleware
  // Audit trail is automatically created
  res.json({ data: 'sensitive information' });
});
```

### Drupal Integration

```typescript
import [createDrupalAdapter] from '@bluefly/secure-project';

const drupalAdapter = createDrupalAdapter({
  auditLogger,
  policyEngine,
  complianceEngine,
  securityAlerts: alertManager
});

// Export for use in Drupal module
export default drupalAdapter;
```

## Best Practices

1. **Always enable audit logging** for compliance requirements
2. **Configure appropriate SLAs** for incident response based on severity
3. **Schedule regular vulnerability scans** (at least weekly)
4. **Keep threat intelligence updated** with regular feed synchronization
5. **Generate compliance reports** before audit periods
6. **Review risk assessments** quarterly or when significant changes occur
7. **Test incident response playbooks** regularly
8. **Monitor security metrics** through dashboards and alerts

## Error Handling

All services follow consistent error handling patterns:

```typescript
try {
  const result = await service.someMethod();
} catch (error) [if (error.code === 'VALIDATION_ERROR') {
    // Handle validation errors] else if (error.code === 'NOT_FOUND') [// Handle not found errors] else [// Handle unexpected errors]
}
```

Common error codes:
- `VALIDATION_ERROR` - Invalid input parameters
- `NOT_FOUND` - Resource not found
- `UNAUTHORIZED` - Insufficient permissions
- `CONFLICT` - Resource conflict (e.g., duplicate)
- `INTERNAL_ERROR` - Internal service error