# Drupal Integration Guide for Secure Project

This guide explains how to integrate the secure_project NPM package with the secure_drupal Drupal module for comprehensive security compliance.

## Table of Contents

1. [Overview](#overview)
2. [Architecture](#architecture)
3. [Installation](#installation)
4. [Configuration](#configuration)
5. [Service Integration](#service-integration)
6. [Plugin Development](#plugin-development)
7. [Event Integration](#event-integration)
8. [ECA Integration](#eca-integration)
9. [Best Practices](#best-practices)
10. [Troubleshooting](#troubleshooting)

## Overview

The secure_drupal module acts as a bridge between Drupal and the secure_project NPM package, providing:

- Seamless integration of security services
- Drupal-specific plugin implementations
- Entity and field-level security controls
- Automated compliance monitoring
- Real-time security alerts

## Architecture

```
┌─────────────────────────────────────────────────────────────┐
│                    Drupal Application                        │
├─────────────────────────────────────────────────────────────┤
│                    secure_drupal Module                      │
│  ┌─────────────┐  ┌──────────────┐  ┌─────────────────┐   │
│  │  Services   │  │   Plugins    │  │   Controllers   │   │
│  │             │  │              │  │                 │   │
│  │ Bridge      │  │ Security     │  │ API             │   │
│  │ Integration │  │ Compliance   │  │ Dashboard       │   │
│  │ Monitoring  │  │ Evidence     │  │ Reports         │   │
│  └─────────────┘  └──────────────┘  └─────────────────┘   │
├─────────────────────────────────────────────────────────────┤
│              SecureProjectBridgeService                      │
│                    (HTTP/Process)                            │
├─────────────────────────────────────────────────────────────┤
│                 secure_project NPM Package                   │
│  ┌─────────────┐  ┌──────────────┐  ┌─────────────────┐   │
│  │   Core      │  │   Security   │  │   Framework     │   │
│  │ Services    │  │  Services    │  │   Adapters     │   │
│  └─────────────┘  └──────────────┘  └─────────────────┘   │
└─────────────────────────────────────────────────────────────┘
```

## Installation

### Prerequisites

- Drupal 10.x or 11.x
- Node.js 18+ and npm
- Composer
- ECA module (optional but recommended)

### Step 1: Install the NPM Package

```bash
# Navigate to a directory outside of Drupal
cd /path/to/secure_project
npm install @bluefly/secure-project
```

### Step 2: Install the Drupal Module

```bash
# Navigate to Drupal root
cd /path/to/drupal
composer require drupal/secure_drupal drupal/eca
drush en secure_drupal eca -y
```

### Step 3: Start the Bridge Server

```bash
# In the secure_project directory
cd /path/to/secure_project
npm run dev  # Starts on http://localhost:3010
```

## Configuration

### Basic Configuration

1. Navigate to `/admin/config/security/secure-drupal`
2. Configure the bridge connection:

```yaml
# HTTP Mode (Recommended for production)
bridge:
  use_http_api: true
  api_base_url: 'http://localhost:3010'
  api_key: 'your-secure-api-key'
  timeout: 30

# Process Mode (Development only)
bridge:
  use_http_api: false
  nodejs_path: '/usr/bin/node'
  script_path: '/path/to/secure_project/scripts/bridge.js'
```

### Service Configuration

Configure individual services through the UI or settings:

```php
// In settings.php
$config['secure_drupal.settings']['services'] = [
  'incident_response' => [
    'enabled' => TRUE,
    'sla' => [
      'critical' => 15,  // minutes
      'high' => 60,
      'medium' => 240,
      'low' => 1440,
    ],
  ],
  'vulnerability_scanner' => [
    'enabled' => TRUE,
    'auto_scan' => TRUE,
    'scan_frequency' => 'daily',
  ],
  'threat_intelligence' => [
    'enabled' => TRUE,
    'feeds' => ['abuse-ch', 'alienvault-otx'],
    'auto_check' => TRUE,
  ],
  'compliance_reporting' => [
    'enabled' => TRUE,
    'frameworks' => ['soc2', 'hipaa', 'gdpr'],
  ],
  'risk_assessment' => [
    'enabled' => TRUE,
    'review_cycles' => [
      'critical' => 7,  // days
      'high' => 30,
      'medium' => 90,
      'low' => 180,
    ],
  ],
];
```

### Notification Configuration

```php
$config['secure_drupal.settings']['notifications'] = [
  'email' => [
    'enabled' => TRUE,
    'recipients' => [
      'security' => 'security@example.com',
      'compliance' => 'compliance@example.com',
    ],
  ],
  'slack' => [
    'enabled' => TRUE,
    'webhook' => 'https://hooks.slack.com/services/YOUR/WEBHOOK/URL',
    'channels' => [
      'incidents' => '#security-incidents',
      'alerts' => '#security-alerts',
    ],
  ],
];
```

## Service Integration

### Using Services in Custom Code

```php
<?php

namespace Drupal\my_module\Controller;

use Drupal\Core\Controller\ControllerBase;
use Symfony\Component\DependencyInjection\ContainerInterface;

class SecurityController extends ControllerBase {
  
  protected $bridge;
  protected $incidentService;
  protected $vulnerabilityService;
  
  public static function create(ContainerInterface $container) {
    $instance = parent::create($container);
    $instance->bridge = $container->get('secure_drupal.bridge');
    $instance->incidentService = $container->get('secure_drupal.incident_response');
    $instance->vulnerabilityService = $container->get('secure_drupal.vulnerability_scanner');
    return $instance;
  }
  
  public function handleSecurityEvent() {
    // Create an incident
    $incident = $this->incidentService->createIncident([
      'title' => 'Suspicious Activity Detected',
      'description' => 'Multiple failed login attempts',
      'severity' => 'high',
      'type' => 'security_breach',
      'source' => 'drupal',
      'affectedUsers' => [$this->currentUser()->id()],
    ]);
    
    // Scan for vulnerabilities
    $scan = $this->vulnerabilityService->scan([
      'type' => 'application',
      'url' => $this->getRequest()->getHost(),
    ]);
    
    return [
      '#markup' => $this->t('Security event processed. Incident ID: @id', [
        '@id' => $incident['id'],
      ]),
    ];
  }
}
```

### Entity Integration

Secure entity operations with automatic compliance tracking:

```php
<?php

namespace Drupal\my_module\Entity;

use Drupal\Core\Entity\ContentEntityBase;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\secure_drupal\Traits\SecureEntityTrait;

/**
 * Defines a secure custom entity.
 *
 * @ContentEntityType(
 *   id = "secure_data",
 *   label = @Translation("Secure Data"),
 *   ...
 * )
 */
class SecureData extends ContentEntityBase {
  use SecureEntityTrait;
  
  /**
   * {@inheritdoc}
   */
  public function preSave(EntityStorageInterface $storage) {
    parent::preSave($storage);
    
    // Automatic compliance validation
    $this->validateCompliance(['gdpr', 'hipaa']);
    
    // Risk assessment for sensitive data
    if ($this->get('sensitivity')->value === 'high') {
      $this->assessRisk([
        'category' => 'data_exposure',
        'assets' => [$this->uuid()],
      ]);
    }
  }
  
  /**
   * [@inheritdoc]
   */
  public function postSave(EntityStorageInterface $storage, $update = TRUE) {
    parent::postSave($storage, $update);
    
    // Audit trail
    $this->auditEntityOperation($update ? 'update' : 'create');
  }
}
```

## Plugin Development

### Creating a Custom Security Control Plugin

```php
<?php

namespace Drupal\my_module\Plugin\SecurityControl;

use Drupal\secure_drupal\Plugin\SecurityControlBase;
use Drupal\secure_drupal\Annotation\SecurityControl;

/**
 * Custom security control for data encryption.
 *
 * @SecurityControl(
 *   id = "data_encryption",
 *   label = @Translation("Data Encryption Control"),
 *   description = @Translation("Ensures sensitive data is encrypted"),
 *   category = "data_protection",
 *   severity = "critical"
 * )
 */
class DataEncryptionControl extends SecurityControlBase [/**
   * {@inheritdoc]
   */
  public function evaluate($context) {
    $entity = $context['entity'];
    
    // Check if sensitive fields are encrypted
    $sensitiveFields = ['ssn', 'credit_card', 'medical_record'];
    $unencrypted = [];
    
    foreach ($sensitiveFields as $field) {
      if ($entity->hasField($field) && !$this->isFieldEncrypted($entity, $field)) {
        $unencrypted[] = $field;
      }
    }
    
    if (!empty($unencrypted)) {
      return $this->fail('Unencrypted sensitive fields: ' . implode(', ', $unencrypted));
    }
    
    return $this->pass('All sensitive fields are encrypted');
  }
  
  /**
   * [@inheritdoc]
   */
  public function remediate($context) {
    $entity = $context['entity'];
    
    // Automatically encrypt sensitive fields
    foreach (['ssn', 'credit_card', 'medical_record'] as $field) {
      if ($entity->hasField($field)) {
        $this->encryptField($entity, $field);
      }
    }
    
    $entity->save();
    return TRUE;
  }
}
```

### Creating a Compliance Framework Plugin

```php
<?php

namespace Drupal\my_module\Plugin\ComplianceFramework;

use Drupal\secure_drupal\Plugin\ComplianceFrameworkBase;
use Drupal\secure_drupal\Annotation\ComplianceFramework;

/**
 * Custom compliance framework for industry-specific requirements.
 *
 * @ComplianceFramework(
 *   id = "custom_industry",
 *   label = @Translation("Custom Industry Compliance"),
 *   version = "1.0",
 *   jurisdiction = "US",
 *   industry = "custom"
 * )
 */
class CustomIndustryFramework extends ComplianceFrameworkBase [/**
   * {@inheritdoc]
   */
  public function getControls() {
    return [
      [
        'id' => 'ci-001',
        'name' => 'Data Retention',
        'description' => 'Ensure data is retained for required period',
        'category' => 'data_management',
        'validationRules' => [
          [
            'type' => 'field_validation',
            'field' => 'retention_period',
            'condition' => 'gte',
            'value' => 7, // years
          ],
        ],
      ],
      // Add more controls...
    ];
  }
  
  /**
   * [@inheritdoc]
   */
  public function validate($entity) {
    $results = parent::validate($entity);
    
    // Add custom validation logic
    if ($entity->getEntityTypeId() === 'node' && $entity->bundle() === 'sensitive_document') {
      // Custom validation for sensitive documents
      if (!$entity->hasField('classification') || $entity->get('classification')->isEmpty()) {
        $results->addViolation('ci-002', 'Document must have security classification');
      }
    }
    
    return $results;
  }
}
```

## Event Integration

### Subscribing to Security Events

```php
<?php

namespace Drupal\my_module\EventSubscriber;

use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Drupal\secure_drupal\Event\SecurityIncidentEvent;
use Drupal\secure_drupal\Event\VulnerabilityDetectedEvent;
use Drupal\secure_drupal\Event\ComplianceViolationEvent;

class SecurityEventSubscriber implements EventSubscriberInterface [/**
   * {@inheritdoc]
   */
  public static function getSubscribedEvents() {
    return [
      SecurityIncidentEvent::class => 'onSecurityIncident',
      VulnerabilityDetectedEvent::class => 'onVulnerabilityDetected',
      ComplianceViolationEvent::class => 'onComplianceViolation',
    ];
  }
  
  public function onSecurityIncident(SecurityIncidentEvent $event) {
    $incident = $event->getIncident();
    
    // Log to Drupal watchdog
    \Drupal::logger('security')->critical('Security incident: @title', [
      '@title' => $incident['title'],
    ]);
    
    // Send additional notifications
    if ($incident['severity'] === 'critical') {
      $this->sendEmergencyNotification($incident);
    }
    
    // Block user if incident involves authentication
    if ($incident['type'] === 'authentication_attack') {
      $this->blockSuspiciousUsers($incident['affectedUsers']);
    }
  }
  
  public function onVulnerabilityDetected(VulnerabilityDetectedEvent $event) {
    $vulnerability = $event->getVulnerability();
    
    // Create a node to track the vulnerability
    $node = \Drupal\node\Entity\Node::create([
      'type' => 'security_issue',
      'title' => $vulnerability['title'],
      'field_severity' => $vulnerability['severity'],
      'field_cve' => $vulnerability['cve'],
      'field_component' => $vulnerability['component'],
      'field_status' => 'open',
    ]);
    $node->save();
    
    // Assign to security team
    $this->assignToSecurityTeam($node);
  }
  
  public function onComplianceViolation(ComplianceViolationEvent $event) {
    $violation = $event->getViolation();
    
    // Queue for compliance review
    $queue = \Drupal::queue('compliance_review');
    $queue->createItem([
      'violation' => $violation,
      'timestamp' => time(),
      'entity' => $event->getEntity()->toArray(),
    ]);
  }
}
```

### Dispatching Custom Events

```php
// In your custom code
$event = new \Drupal\secure_drupal\Event\SecurityAlertEvent(
  'custom_alert',
  'Unusual pattern detected',
  'medium',
  ['pattern' => 'rapid_api_calls', 'count' => 1000]
);

\Drupal::service('event_dispatcher')->dispatch($event);
```

## ECA Integration

### Creating ECA Models for Security Workflows

1. **Incident Response Workflow**

```yaml
# ECA Model: Automated Incident Response
events:
  - plugin: secure_drupal:incident_created
    label: "Incident Created"
    
conditions:
  - plugin: eca_compare
    label: "Is Critical?"
    configuration:
      left_value: "[incident:severity]"
      operator: "=="
      right_value: "critical"
      
actions:
  - plugin: secure_drupal:assign_incident_responder
    label: "Assign to Security Team"
    configuration:
      role: "security_lead"
      
  - plugin: eca_send_email
    label: "Alert Management"
    configuration:
      recipient: "management@example.com"
      subject: "Critical Security Incident: [incident:title]"
      
  - plugin: secure_drupal:execute_playbook
    label: "Run Containment Playbook"
    configuration:
      playbook: "critical_incident_containment"
```

2. **Compliance Monitoring Workflow**

```yaml
# ECA Model: Continuous Compliance Monitoring
events:
  - plugin: eca_cron
    label: "Daily Compliance Check"
    configuration:
      frequency: "0 2 * * *"  # 2 AM daily
      
actions:
  - plugin: secure_drupal:run_compliance_scan
    label: "Scan All Entities"
    configuration:
      frameworks: ["gdpr", "hipaa"]
      entity_types: ["node", "user", "file"]
      
  - plugin: eca_condition_group
    label: "Check for Violations"
    conditions:
      - plugin: secure_drupal:has_compliance_violations
        negate: false
        
  - plugin: secure_drupal:generate_compliance_report
    label: "Generate Report"
    configuration:
      format: "pdf"
      recipients: ["compliance@example.com"]
      
  - plugin: secure_drupal:create_remediation_tasks
    label: "Create JIRA Issues"
    configuration:
      project: "COMPLIANCE"
      issue_type: "task"
```

### Using ECA Conditions

```php
<?php

namespace Drupal\secure_drupal\Plugin\ECA\Condition;

use Drupal\eca\Plugin\ECA\Condition\ConditionBase;

/**
 * Checks if entity has security risks.
 *
 * @EcaCondition(
 *   id = "has_security_risks",
 *   label = @Translation("Entity has security risks"),
 *   description = @Translation("Evaluates if an entity has identified security risks")
 * )
 */
class HasSecurityRisks extends ConditionBase {
  
  public function evaluate() {
    $entity = $this->getTokenService()->getTokenData('entity');
    
    if (!$entity) {
      return FALSE;
    }
    
    $riskService = \Drupal::service('secure_drupal.risk_assessment');
    $risks = $riskService->getEntityRisks($entity);
    
    $threshold = $this->configuration['severity_threshold'] ?? 'high';
    
    foreach ($risks as $risk) {
      if ($this->compareSeverity($risk['severity'], $threshold) >= 0) {
        return TRUE;
      }
    }
    
    return FALSE;
  }
}
```

## Best Practices

### 1. Security Configuration

- Always use HTTPS for bridge communication in production
- Rotate API keys regularly
- Implement IP whitelisting for bridge endpoints
- Enable audit logging for all operations

### 2. Performance Optimization

- Use caching for compliance validations
- Batch API calls to the bridge
- Implement queue workers for heavy operations
- Use lazy loading for security services

### 3. Error Handling

```php
try {
  $result = \Drupal::service('secure_drupal.bridge')->call('checkCompliance', [
    'entity' => $entity->toArray(),
    'frameworks' => ['gdpr', 'hipaa'],
  ]);
} catch (\Drupal\secure_drupal\Exception\BridgeException $e) {
  // Handle bridge communication errors
  \Drupal::logger('secure_drupal')->error('Bridge error: @message', [
    '@message' => $e->getMessage(),
  ]);
  
  // Fallback to cached results
  $result = \Drupal::cache()->get('compliance_' . $entity->id());
} catch (\Exception $e) {
  // Handle other errors
  \Drupal::logger('secure_drupal')->error('Unexpected error: @message', [
    '@message' => $e->getMessage(),
  ]);
}
```

### 4. Monitoring and Alerting

- Set up Drupal watchdog alerts for security events
- Configure email/Slack notifications for critical issues
- Create dashboards for security metrics
- Implement health checks for the bridge

## Troubleshooting

### Common Issues

1. **Bridge Connection Failed**
   ```bash
   # Check bridge status
   curl http://localhost:3010/health
   
   # Check Drupal logs
   drush watchdog:show --type=secure_drupal
   
   # Verify configuration
   drush config:get secure_drupal.settings
   ```

2. **Performance Issues**
   ```php
   // Enable debug mode
   \Drupal::service('secure_drupal.bridge')->enableDebug();
   
   // Check timing
   $timer = \Drupal::service('secure_drupal.performance_monitor');
   $timer->start('compliance_check');
   // ... operation ...
   $timer->stop('compliance_check');
   $duration = $timer->getDuration('compliance_check');
   ```

3. **Compliance Validation Errors**
   ```bash
   # Clear compliance cache
   drush cache:clear compliance
   
   # Re-run validation
   drush secure:validate-entity node 123
   
   # Check detailed logs
   drush sql:query "SELECT * FROM watchdog WHERE type = 'compliance' ORDER BY wid DESC LIMIT 20"
   ```

### Debug Mode

Enable debug mode for detailed logging:

```php
// In settings.php
$config['secure_drupal.settings']['debug'] = TRUE;
$config['secure_drupal.settings']['verbose_logging'] = TRUE;
```

### Health Checks

```bash
# CLI health check
drush secure:health

# Programmatic health check
$health = \Drupal::service('secure_drupal.health')->check();
if (!$health['bridge']['connected']) [// Handle bridge connection issue]
```

## Advanced Topics

### Custom Bridge Endpoints

```php
// In secure_project, add custom endpoint
app.post('/api/custom/drupal-specific', async (req, res) => {
  const { entityType, entityId, operation } = req.body;
  
  // Custom logic for Drupal entities
  const result = await customDrupalHandler(entityType, entityId, operation);
  
  res.json(result);
});
```

### Multi-site Configuration

```php
// In settings.php for each site
$config['secure_drupal.settings']['site_id'] = 'site_1';
$config['secure_drupal.settings']['tenant_isolation'] = TRUE;

// Bridge will maintain separate contexts for each site
```

### Extending Security Services

```php
namespace Drupal\my_module\Service;

use Drupal\secure_drupal\Service\VulnerabilityScannerService;

class CustomVulnerabilityScanner extends VulnerabilityScannerService {
  
  /**
   * Add custom scanning logic.
   */
  public function scan($target, array $options = []) {
    // Call parent scan
    $results = parent::scan($target, $options);
    
    // Add custom scans
    $customResults = $this->performCustomScans($target);
    
    return array_merge($results, $customResults);
  }
  
  protected function performCustomScans($target) {
    // Custom scanning logic
    return [];
  }
}
```

## Resources

- [Secure Project NPM Documentation](https://www.npmjs.com/package/@bluefly/secure-project)
- [Drupal Security Documentation](https://www.drupal.org/docs/security)
- [ECA Module Documentation](https://www.drupal.org/project/eca)
- [Security Best Practices](https://www.drupal.org/docs/security-in-drupal)