# Secure Drupal Restructuring Implementation Guide

This guide provides detailed instructions for implementing the restructuring of the `secure_drupal` module as outlined in the restructuring plan.

## Prerequisites

Before beginning the restructuring process, ensure that you have:

1. A complete backup of the existing codebase
2. A development environment for testing
3. Access to the necessary Git repositories
4. Composer and Drush installed and configured
5. Necessary permissions to create and modify files

## Step 1: Set Up the New Structure

Create the new module directory structure while keeping the old structure intact:

```bash
# Create new module directories
mkdir -p ./secure_project/modules/secure_compliance_suite
mkdir -p ./secure_project/modules/secure_evidence
mkdir -p ./secure_project/modules/secure_audit_tools
mkdir -p ./secure_project/modules/secure_hardening
mkdir -p ./secure_project/modules/secure_auth
mkdir -p ./secure_project/modules/secure_policy_bridge
mkdir -p ./secure_project/modules/secure_sod
mkdir -p ./secure_project/modules/secure_openapi_enforcer
mkdir -p ./secure_project/resources
```

## Step 2: Implement Module Consolidations

### 2.1. Secure Compliance Suite

Consolidate all compliance-related modules into the new secure_compliance_suite module:

1. Create the base module files:

```php
// secure_compliance_suite.info.yml
name: 'Secure Compliance Suite'
type: module
description: 'Comprehensive compliance management with support for multiple frameworks, regimes, and profiles.'
package: 'Security'
core_version_requirement: ^9.3 || ^10
dependencies:
  - secure_drupal:secure_drupal
configure: secure_compliance_suite.settings
```

2. Create a plugin architecture for compliance frameworks:

```php
// src/Annotation/ComplianceFramework.php
<?php

namespace Drupal\secure_compliance_suite\Annotation;

use Drupal\Component\Annotation\Plugin;

/**
 * Defines a Compliance Framework annotation object.
 *
 * @Annotation
 */
class ComplianceFramework extends Plugin {
  /**
   * The plugin ID.
   *
   * @var string
   */
  public $id;

  /**
   * The human-readable name of the compliance framework.
   *
   * @var \Drupal\Core\Annotation\Translation
   */
  public $label;

  /**
   * The description of the compliance framework.
   *
   * @var \Drupal\Core\Annotation\Translation
   */
  public $description;
  
  /**
   * The compliance regime this framework belongs to.
   *
   * @var string
   */
  public $regime;
}
```

3. Create plugin interfaces and base classes:

```php
// src/Plugin/ComplianceFramework/ComplianceFrameworkInterface.php
<?php

namespace Drupal\secure_compliance_suite\Plugin\ComplianceFramework;

use Drupal\Component\Plugin\PluginInspectionInterface;

/**
 * Defines an interface for Compliance Framework plugins.
 */
interface ComplianceFrameworkInterface extends PluginInspectionInterface [// Define methods for compliance frameworks.]

// src/Plugin/ComplianceFramework/ComplianceFrameworkBase.php
<?php

namespace Drupal\secure_compliance_suite\Plugin\ComplianceFramework;

use Drupal\Component\Plugin\PluginBase;

/**
 * Base class for Compliance Framework plugins.
 */
abstract class ComplianceFrameworkBase extends PluginBase implements ComplianceFrameworkInterface {
  // Implement common functionality.
}
```

4. Create specific compliance framework implementations:

```php
// src/Plugin/ComplianceFramework/PciFramework.php
<?php

namespace Drupal\secure_compliance_suite\Plugin\ComplianceFramework;

/**
 * PCI DSS compliance framework.
 *
 * @ComplianceFramework(
 *   id = "pci_dss",
 *   label = @Translation("PCI DSS"),
 *   description = @Translation("Payment Card Industry Data Security Standard compliance framework."),
 *   regime = "pci"
 * )
 */
class PciFramework extends ComplianceFrameworkBase {
  // Implement PCI-specific functionality.
}

// src/Plugin/ComplianceFramework/FedrampFramework.php
<?php

namespace Drupal\secure_compliance_suite\Plugin\ComplianceFramework;

/**
 * FedRAMP compliance framework.
 *
 * @ComplianceFramework(
 *   id = "fedramp",
 *   label = @Translation("FedRAMP"),
 *   description = @Translation("Federal Risk and Authorization Management Program compliance framework."),
 *   regime = "government"
 * )
 */
class FedrampFramework extends ComplianceFrameworkBase {
  // Implement FedRAMP-specific functionality.
}
```

5. Migrate the functionality from the original modules:

   - Copy relevant services from each original module
   - Update namespaces and dependencies
   - Ensure all functionality is preserved
   - Implement a plugin manager for compliance frameworks

### 2.2. Secure Evidence

Consolidate all evidence collection and logging modules:

1. Create the base module files:

```php
// secure_evidence.info.yml
name: 'Secure Evidence'
type: module
description: 'Comprehensive evidence collection, validation, and storage system.'
package: 'Security'
core_version_requirement: ^9.3 || ^10
dependencies:
  - secure_drupal:secure_drupal
configure: secure_evidence.settings
```

2. Create a plugin architecture for evidence providers:

```php
// src/Annotation/EvidenceProvider.php
<?php

namespace Drupal\secure_evidence\Annotation;

use Drupal\Component\Annotation\Plugin;

/**
 * Defines an Evidence Provider annotation object.
 *
 * @Annotation
 */
class EvidenceProvider extends Plugin {
  /**
   * The plugin ID.
   *
   * @var string
   */
  public $id;

  /**
   * The human-readable name of the evidence provider.
   *
   * @var \Drupal\Core\Annotation\Translation
   */
  public $label;

  /**
   * The description of the evidence provider.
   *
   * @var \Drupal\Core\Annotation\Translation
   */
  public $description;
}
```

3. Create plugin interfaces and base classes:

```php
// src/Plugin/EvidenceProvider/EvidenceProviderInterface.php
<?php

namespace Drupal\secure_evidence\Plugin\EvidenceProvider;

use Drupal\Component\Plugin\PluginInspectionInterface;

/**
 * Defines an interface for Evidence Provider plugins.
 */
interface EvidenceProviderInterface extends PluginInspectionInterface [// Define methods for evidence providers.]

// src/Plugin/EvidenceProvider/EvidenceProviderBase.php
<?php

namespace Drupal\secure_evidence\Plugin\EvidenceProvider;

use Drupal\Component\Plugin\PluginBase;

/**
 * Base class for Evidence Provider plugins.
 */
abstract class EvidenceProviderBase extends PluginBase implements EvidenceProviderInterface {
  // Implement common functionality.
}
```

4. Create specific evidence provider implementations:

```php
// src/Plugin/EvidenceProvider/DbLogEvidenceProvider.php
<?php

namespace Drupal\secure_evidence\Plugin\EvidenceProvider;

/**
 * Database log evidence provider.
 *
 * @EvidenceProvider(
 *   id = "dblog",
 *   label = @Translation("Database Log"),
 *   description = @Translation("Collects evidence from the database log."),
 * )
 */
class DbLogEvidenceProvider extends EvidenceProviderBase {
  // Implement dblog-specific functionality.
}

// src/Plugin/EvidenceProvider/ConfigEvidenceProvider.php
<?php

namespace Drupal\secure_evidence\Plugin\EvidenceProvider;

/**
 * Configuration evidence provider.
 *
 * @EvidenceProvider(
 *   id = "config",
 *   label = @Translation("Configuration"),
 *   description = @Translation("Collects evidence from configuration changes."),
 * )
 */
class ConfigEvidenceProvider extends EvidenceProviderBase {
  // Implement configuration-specific functionality.
}
```

### 2.3. Secure Audit Tools

Consolidate all audit, healing, and task tracking modules:

1. Create the base module files:

```php
// secure_audit_tools.info.yml
name: 'Secure Audit Tools'
type: module
description: 'Comprehensive audit, healing, and task tracking system.'
package: 'Security'
core_version_requirement: ^9.3 || ^10
dependencies:
  - secure_drupal:secure_drupal
configure: secure_audit_tools.settings
```

2. Create services for audit, healing, and task tracking:

```php
// src/Service/AuditService.php
<?php

namespace Drupal\secure_audit_tools\Service;

/**
 * Service for security auditing.
 */
class AuditService {
  // Implement audit functionality.
}

// src/Service/HealingService.php
<?php

namespace Drupal\secure_audit_tools\Service;

/**
 * Service for self-healing functionality.
 */
class HealingService {
  // Implement healing functionality.
}

// src/Service/TaskTrackerService.php
<?php

namespace Drupal\secure_audit_tools\Service;

/**
 * Service for task tracking.
 */
class TaskTrackerService {
  // Implement task tracking functionality.
}
```

### 2.4. Secure Hardening

Consolidate all security hardening modules:

1. Create the base module files:

```php
// secure_hardening.info.yml
name: 'Secure Hardening'
type: module
description: 'Comprehensive security hardening, including headers, controls, configuration, and theme security.'
package: 'Security'
core_version_requirement: ^9.3 || ^10
dependencies:
  - secure_drupal:secure_drupal
configure: secure_hardening.settings
```

2. Create submodules for specific hardening areas:

```php
// modules/secure_headers/secure_headers.info.yml
name: 'Secure Headers'
type: module
description: 'Security headers management and enforcement.'
package: 'Security'
core_version_requirement: ^9.3 || ^10
dependencies:
  - secure_drupal:secure_hardening
```

3. Migrate functionality from original modules, maintaining existing APIs where possible.

### 2.5. Secure Auth

Consolidate all authentication modules:

1. Create the base module files:

```php
// secure_auth.info.yml
name: 'Secure Authentication'
type: module
description: 'Comprehensive authentication security, including password policies and advanced authentication methods.'
package: 'Security'
core_version_requirement: ^9.3 || ^10
dependencies:
  - secure_drupal:secure_drupal
configure: secure_auth.settings
```

2. Create submodules for specific authentication features:

```php
// modules/password_policy/password_policy.info.yml
name: 'Password Policy'
type: module
description: 'Configure and enforce password policies.'
package: 'Security'
core_version_requirement: ^9.3 || ^10
dependencies:
  - secure_drupal:secure_auth
```

### 2.6. Module Renamings

Rename modules for clarity and consistency:

1. Rename policy_bridge to secure_policy_bridge:

```php
// secure_policy_bridge.info.yml
name: 'Secure Policy Bridge'
type: module
description: 'Bridge for security policy enforcement and integration.'
package: 'Security'
core_version_requirement: ^9.3 || ^10
dependencies:
  - secure_drupal:secure_drupal
```

2. Rename separation_of_duties to secure_sod:

```php
// secure_sod.info.yml
name: 'Secure Separation of Duties'
type: module
description: 'Enforce separation of duties in security-critical operations.'
package: 'Security'
core_version_requirement: ^9.3 || ^10
dependencies:
  - secure_drupal:secure_drupal
```

3. Rename openapi_compliance_enforcer to secure_openapi_enforcer:

```php
// secure_openapi_enforcer.info.yml
name: 'Secure OpenAPI Enforcer'
type: module
description: 'Enforce OpenAPI compliance in API development.'
package: 'Security'
core_version_requirement: ^9.3 || ^10
dependencies:
  - secure_drupal:secure_drupal
```

### 2.7. Resource Reorganization

Move the resources module to a non-module directory:

1. Create the resources directory:

```bash
mkdir -p /Users/flux423/Sites/_DrupalModules/secure_drupal/resources
```

2. Move relevant files from the original module to this directory.

## Step 3: Update Dependencies and References

1. Update the main secure_drupal.info.yml file to reflect the new module structure:

```yaml
name: 'Secure Drupal'
type: module
description: 'Comprehensive security framework for Drupal.'
package: 'Security'
core_version_requirement: ^9.3 || ^10
dependencies:
  - drupal:system
```

2. Update cross-references between modules to reflect the new structure.

3. Update service definitions in *.services.yml files to reflect new namespaces and class locations.

## Step 4: Implement Backward Compatibility

1. Create alias services for backward compatibility:

```yaml
# secure_drupal.services.yml
services:
  # Original service name pointing to new service
  secure_compliance.manager:
    alias: secure_compliance_suite.manager
    deprecated: 'The "%service_id%" service is deprecated. Use "secure_compliance_suite.manager" instead.'
```

2. Implement service provider classes to handle service replacements:

```php
// src/SecureDrupalServiceProvider.php
<?php

namespace Drupal\secure_drupal;

use Drupal\Core\DependencyInjection\ContainerBuilder;
use Drupal\Core\DependencyInjection\ServiceProviderBase;

/**
 * Service provider for secure_drupal.
 */
class SecureDrupalServiceProvider extends ServiceProviderBase [/**
   * {@inheritdoc]
   */
  public function alter(ContainerBuilder $container) {
    // Handle service replacement for backward compatibility.
    if ($container->hasDefinition('secure_compliance.manager')) {
      $container->setAlias('secure_compliance.manager', 'secure_compliance_suite.manager');
    }
  }
}
```

## Step 5: Testing and Validation

1. Create a testing plan:

   - Unit tests for each consolidated module
   - Integration tests for module interactions
   - Functional tests for UI elements
   - Upgrade tests for backward compatibility

2. Implement and run tests to ensure functionality is preserved.

3. Create a validation script to verify all features are working as expected.

## Step 6: Documentation and Release

1. Update documentation to reflect the new structure:

   - README files for each module
   - API documentation
   - Upgrade guide

2. Create a changelog detailing the restructuring changes.

3. Prepare a release announcement explaining the benefits of the new structure.

## Step 7: Integration with secure_project

1. Update the secure_project repository to work with the new module structure:

   - Update adapter classes
   - Update integration tests
   - Ensure compatibility with the new module organization

2. Test integration between secure_drupal and secure_project to ensure everything works together.

## Rollout Plan

1. **Alpha Release**:
   - Release the restructured modules as an alpha version
   - Gather feedback from early adopters
   - Address any issues identified

2. **Beta Release**:
   - Incorporate feedback from alpha testing
   - Finalize backward compatibility
   - Complete documentation

3. **Stable Release**:
   - Finalize the restructured modules
   - Announce the new structure
   - Provide migration guides for existing users

## Conclusion

This implementation guide provides a detailed roadmap for restructuring the secure_drupal module. By following these steps, the module can be transformed into a more maintainable, clearer, and more user-friendly structure while preserving all existing functionality.