# Security Policy Bridge

## Overview

The Security Policy Bridge module provides automated security policy inheritance and compliance mapping between `secure_drupal` and code generation systems like `code_genesis`. It ensures that all generated code automatically inherits appropriate security policies, compliance requirements, and threat prevention measures.

## Key Features

### 🔒 **Automated Policy Inheritance**
- Automatically applies security policies to code generation requests
- Maps compliance framework requirements to generated code
- Ensures consistent security standards across all generated modules

### 🛡️ **Compliance Integration**
- Supports multiple compliance frameworks (HIPAA, PCI-DSS, FedRAMP, GDPR, etc.)
- Automatically maps compliance requirements to code patterns
- Validates generated code against compliance standards

### 🔍 **Security Validation**
- Real-time validation of generated code against security policies
- Automated detection of security vulnerabilities
- Comprehensive security scoring and reporting

###  **Intelligent Orchestration**
- Event-driven architecture for seamless integration
- AI-enhanced policy analysis and application
- Automated remediation suggestions

## Architecture

### Core Services

```
PolicyInheritanceService
├── Security Control Mapping
├── Compliance Framework Integration
├── Code Pattern Enforcement
└── Validation Rule Application

CodeGenesisIntegration
├── Generation Request Enhancement
├── Post-Generation Validation
├── Security Code Injection
└── Compliance Monitoring

SecurityPolicyInjector
├── Automatic Security Component Injection
├── Compliance Code Pattern Application
├── Threat Prevention Implementation
└── Access Control Integration
```

### Event Integration

The module listens to Code Genesis events:

- **Generation Requested**: Enhances requests with security policies
- **Generation Completed**: Validates and enhances generated code
- **Validation Requested**: Performs security-specific validation

## Installation

1. **Prerequisites**
   ```bash
   # Ensure secure_drupal is installed
   drush en secure_drupal secure_compliance
   
   # Ensure code_genesis is available (optional but recommended)
   drush en code_genesis
   ```

2. **Enable Policy Bridge**
   ```bash
   drush en policy_bridge
   ```

3. **Configure Integration**
   ```bash
   # Configure policy inheritance settings
   drush config-set policy_bridge.settings active_compliance_frameworks.0 'hipaa'
   drush config-set policy_bridge.settings active_compliance_frameworks.1 'gdpr'
   ```

## Configuration

### Basic Configuration

Navigate to `/admin/config/security/policy-bridge` to configure:

- **Active Compliance Frameworks**: Select which compliance standards to enforce
- **Default Security Level**: Set the baseline security requirements
- **Policy Inheritance Rules**: Configure how policies are applied to different code types

### Code Genesis Integration

At `/admin/config/security/policy-bridge/code-genesis`:

- **Auto-Enhancement**: Automatically enhance generation requests with security policies
- **Validation Mode**: Choose validation strictness (permissive, standard, strict)
- **Remediation**: Configure automatic security remediation

### Compliance Mappings

At `/admin/config/security/policy-bridge/mappings`:

- **Framework Mappings**: Map compliance requirements to code patterns
- **Control Implementation**: Configure how security controls are implemented
- **Validation Rules**: Define validation criteria for each compliance framework

## Usage

### Automatic Policy Application

When enabled, the module automatically:

1. **Enhances Generation Requests**
   ```php
   // Original request
   $request = [
     'type' => 'module',
     'requirements' => 'Create a user management module'
   ];
   
   // Automatically enhanced with security policies
   $enhanced_request = [
     'type' => 'module',
     'requirements' => 'Create a user management module with RBAC, audit logging, and data encryption',
     'security_policies' => [...],
     'compliance_requirements' => [...]
   ];
   ```

2. **Validates Generated Code**
   ```php
   // Automatic validation against security policies
   $validation_result = [
     'valid' => true,
     'security_score' => 95,
     'violations' => [],
     'applied_policies' => [...]
   ];
   ```

### Manual Integration

For custom code generation systems:

```php
// Get inherited policies
$policies = \Drupal::service('policy_bridge.policy_inheritance')
  ->getInheritedPolicies('module', ['compliance_level' => 'hipaa']);

// Apply policies to generated code
$enhanced_code = \Drupal::service('policy_bridge.policy_inheritance')
  ->applySecurityPolicies($generated_code, $policies);

// Validate code
$validation = \Drupal::service('policy_bridge.policy_inheritance')
  ->validateGeneratedCode($enhanced_code, $policies);
```

### API Integration

Use REST endpoints for external systems:

```bash
# Get policies for a specific generation type
curl -X GET /api/policy-bridge/policies/module \
  -H "Content-Type: application/json"

# Validate code against policies
curl -X POST /api/policy-bridge/validate \
  -H "Content-Type: application/json" \
  -d '{"code": "...", "type": "module"}'

# Apply policies to code
curl -X POST /api/policy-bridge/apply \
  -H "Content-Type: application/json" \
  -d '{"code": "...", "policies": {...}}'
```

## Policy Types

### Security Controls
- **Access Control**: Permission checks, role-based access
- **Input Validation**: Sanitization, XSS prevention
- **Data Protection**: Encryption, data handling
- **Audit Logging**: Comprehensive activity tracking

### Compliance Requirements
- **HIPAA**: Healthcare data protection
- **PCI-DSS**: Payment card security
- **GDPR**: Data privacy and protection
- **FedRAMP**: Federal security standards
- **SOC 2**: Service organization controls

### Code Patterns
- **Secure Coding**: Safe functions, input validation
- **Architecture**: Security-by-design patterns
- **Dependencies**: Secure module dependencies
- **Configuration**: Secure default settings

## Monitoring & Reporting

### Security Dashboard

Access at `/admin/reports/security/policy-bridge`:

- **Compliance Status**: Overall compliance score
- **Policy Violations**: Active security issues
- **Generated Code Security**: Security metrics for generated modules
- **Trend Analysis**: Security improvement over time

### Violation Reports

View detailed violation reports at `/admin/reports/security/policy-bridge/violations`:

- **Violation Details**: Specific security issues found
- **Remediation Steps**: Automated fix suggestions
- **Impact Assessment**: Risk level and affected components
- **Resolution Tracking**: Fix progress monitoring

## Extension Points

### Custom Policy Providers

Create custom compliance frameworks:

```php
/**
 * @ComplianceFramework(
 *   id = "custom_framework",
 *   label = @Translation("Custom Compliance Framework"),
 *   description = @Translation("Organization-specific compliance requirements")
 * )
 */
class CustomFramework extends ComplianceFrameworkBase [// Implementation]
```

### Security Control Plugins

Add custom security controls:

```php
/**
 * @SecurityControl(
 *   id = "custom_control",
 *   label = @Translation("Custom Security Control"),
 *   category = "data_protection"
 * )
 */
class CustomControl extends SecurityControlBase [// Implementation]
```

### Event Subscribers

Listen to policy events:

```php
class CustomPolicySubscriber implements EventSubscriberInterface {
  public static function getSubscribedEvents() {
    return [
      'policy_bridge.policy_applied' => 'onPolicyApplied',
      'policy_bridge.violation_detected' => 'onViolationDetected',
    ];
  }
}
```

## Integration with Existing Systems

### Drupal Core
- **Configuration Management**: Secure config handling
- **Entity System**: Field-level security
- **User System**: Enhanced authentication/authorization

### Contrib Modules
- **Views**: Secure data display
- **Webform**: Enhanced form security
- **Commerce**: PCI-DSS compliance

### External Systems
- **CI/CD Pipelines**: Automated security validation
- **Code Quality Tools**: Integration with static analysis
- **Monitoring Systems**: Security event forwarding

## Performance Considerations

### Caching Strategy
- **Policy Cache**: 1-hour TTL for inherited policies
- **Validation Cache**: 30-minute TTL for validation results
- **Mapping Cache**: 6-hour TTL for compliance mappings

### Queue Processing
- **Asynchronous Validation**: Heavy validation processed in background
- **Policy Synchronization**: Bulk policy updates via queue
- **Monitoring Updates**: Real-time security metric collection

### Optimization
- **Lazy Loading**: Services loaded only when needed
- **Selective Enhancement**: Apply policies only to relevant code
- **Batch Processing**: Efficient handling of multiple validation requests

## Troubleshooting

### Common Issues

1. **Policies Not Applied**
   - Check module dependencies
   - Verify compliance framework configuration
   - Review permission settings

2. **Validation Failures**
   - Check security control configuration
   - Review compliance mapping settings
   - Validate policy inheritance rules

3. **Performance Issues**
   - Monitor cache hit rates
   - Check queue processing status
   - Review service configuration

### Debug Mode

Enable debug logging:

```php
$config = \Drupal::configFactory()->getEditable('policy_bridge.settings');
$config->set('debug_mode', TRUE);
$config->save();
```

### Log Analysis

Check logs for policy bridge events:

```bash
drush watchdog-show --type=policy_bridge
```

## Security Considerations

### Module Security
- **Restricted Permissions**: Critical operations require admin access
- **Input Validation**: All user inputs validated and sanitized
- **Secure Defaults**: Conservative security settings by default
- **Audit Trail**: All policy changes logged

### Generated Code Security
- **Policy Validation**: All generated code validated before deployment
- **Compliance Verification**: Automated compliance checking
- **Vulnerability Scanning**: Integrated security scanning
- **Remediation Tracking**: Fix implementation monitoring

## Contributing

### Development Setup

1. Clone the repository
2. Install dependencies: `composer install`
3. Run tests: `phpunit`
4. Check coding standards: `phpcs`

### Testing

The module includes comprehensive tests:

- **Unit Tests**: Individual service testing
- **Kernel Tests**: Integration testing
- **Functional Tests**: End-to-end policy application
- **Behavioral Tests**: User interaction testing

### Code Standards

Follow Drupal coding standards:
- PSR-4 autoloading
- Drupal coding standards
- PHPDoc documentation
- Type declarations

## License

GPL-2.0-or-later

## Support

- **Documentation**: Comprehensive inline documentation
- **Issue Queue**: GitHub issues for bug reports
- **Community**: Drupal Slack #security channel
- **Professional Support**: Available through certified partners