#!/usr/bin/env python3
"""
Compliance Validator for Secure Drupal Recipe
Validates configuration against FedRAMP, HIPAA, SOC2, and NIST 800-53 standards
"""

import json
import yaml
import argparse
import sys
from datetime import datetime
from pathlib import Path
import pandas as pd

class ComplianceValidator:
    def __init__(self, recipe_path='/app'):
        self.recipe_path = Path(recipe_path)
        self.recipe = self._load_recipe()
        self.results = {
            'timestamp': datetime.now().isoformat(),
            'recipe_name': self.recipe['name'],
            'compliance_checks': {},
            'overall_score': 0
        }
        
    def _load_recipe(self):
        """Load recipe.yml file"""
        recipe_file = self.recipe_path / 'recipe.yml'
        with open(recipe_file, 'r') as f:
            return yaml.safe_load(f)
    
    def validate_fedramp(self):
        """Validate FedRAMP compliance requirements"""
        print("🏛️  Validating FedRAMP compliance...")
        
        fedramp_controls = {
            'AC-2': {  # Account Management
                'name': 'Account Management',
                'check': lambda: 'user.settings' in self.recipe['config']['import'],
                'severity': 'HIGH'
            },
            'AC-7': {  # Unsuccessful Login Attempts
                'name': 'Unsuccessful Login Attempts',
                'check': lambda: self._check_config('login_security.settings', 'user_wrong_count', 3),
                'severity': 'HIGH'
            },
            'AC-11': {  # Session Lock
                'name': 'Session Lock',
                'check': lambda: self._check_config('autologout.settings', 'timeout', 900, '<='),
                'severity': 'MEDIUM'
            },
            'AU-2': {  # Audit Events
                'name': 'Audit Events',
                'check': lambda: self._check_config('audit_log.settings', 'log_user_operations', True),
                'severity': 'HIGH'
            },
            'IA-2': {  # Multi-Factor Authentication
                'name': 'Multi-Factor Authentication',
                'check': lambda: self._check_config('tfa.settings', 'enabled', True),
                'severity': 'HIGH'
            },
            'IA-5': {  # Authenticator Management
                'name': 'Password Policy',
                'check': lambda: self._check_password_policy(),
                'severity': 'HIGH'
            },
            'SC-8': {  # Transmission Confidentiality
                'name': 'HTTPS Enforcement',
                'check': lambda: self._check_hsts(),
                'severity': 'HIGH'
            },
            'SC-13': {  # Cryptographic Protection
                'name': 'Encryption',
                'check': lambda: 'encrypt' in self.recipe['install'],
                'severity': 'HIGH'
            }
        }
        
        results = self._run_compliance_checks(fedramp_controls)
        self.results['compliance_checks']['fedramp'] = results
        return results
    
    def validate_hipaa(self):
        """Validate HIPAA compliance requirements"""
        print("🏥 Validating HIPAA compliance...")
        
        hipaa_safeguards = {
            '164.308(a)(1)': {  # Security Management Process
                'name': 'Risk Analysis',
                'check': lambda: 'security_review' in self.recipe['install'],
                'severity': 'REQUIRED'
            },
            '164.308(a)(3)': {  # Workforce Security
                'name': 'Access Management',
                'check': lambda: 'field_permissions' in self.recipe['install'],
                'severity': 'REQUIRED'
            },
            '164.308(a)(4)': {  # Information Access Management
                'name': 'Access Control',
                'check': lambda: 'content_access' in self.recipe['install'],
                'severity': 'REQUIRED'
            },
            '164.308(a)(5)': {  # Security Awareness Training
                'name': 'Password Management',
                'check': lambda: 'password_policy' in self.recipe['install'],
                'severity': 'ADDRESSABLE'
            },
            '164.312(a)(1)': {  # Access Control
                'name': 'Unique User Identification',
                'check': lambda: self._check_config('user.settings', 'register', 'admin_only'),
                'severity': 'REQUIRED'
            },
            '164.312(a)(2)': {  # Audit Controls
                'name': 'Audit Logs',
                'check': lambda: 'admin_audit_trail' in self.recipe['install'],
                'severity': 'REQUIRED'
            },
            '164.312(c)(1)': {  # Integrity
                'name': 'Data Integrity Controls',
                'check': lambda: self._check_config('seckit.settings', 'seckit_various.from_origin.checkbox', True),
                'severity': 'ADDRESSABLE'
            },
            '164.312(e)(1)': {  # Transmission Security
                'name': 'Encryption in Transit',
                'check': lambda: self._check_hsts(),
                'severity': 'ADDRESSABLE'
            }
        }
        
        results = self._run_compliance_checks(hipaa_safeguards)
        self.results['compliance_checks']['hipaa'] = results
        return results
    
    def validate_soc2(self):
        """Validate SOC2 compliance requirements"""
        print("📊 Validating SOC2 compliance...")
        
        soc2_criteria = {
            'CC6.1': {  # Logical Access Controls
                'name': 'Logical Access Controls',
                'check': lambda: all([
                    'login_security' in self.recipe['install'],
                    'tfa' in self.recipe['install'],
                    'session_limit' in self.recipe['install']
                ]),
                'severity': 'HIGH'
            },
            'CC6.6': {  # Encryption
                'name': 'Encryption',
                'check': lambda: all([
                    'encrypt' in self.recipe['install'],
                    'key' in self.recipe['install']
                ]),
                'severity': 'HIGH'
            },
            'CC7.2': {  # System Monitoring
                'name': 'System Monitoring',
                'check': lambda: all([
                    'admin_audit_trail' in self.recipe['install'],
                    'login_tracker' in self.recipe['install']
                ]),
                'severity': 'MEDIUM'
            },
            'CC8.1': {  # Change Management
                'name': 'Configuration Management',
                'check': lambda: self._check_config('audit_log.settings', 'log_config_operations', True),
                'severity': 'MEDIUM'
            }
        }
        
        results = self._run_compliance_checks(soc2_criteria)
        self.results['compliance_checks']['soc2'] = results
        return results
    
    def validate_nist(self):
        """Validate NIST 800-53 compliance requirements"""
        print("🔐 Validating NIST 800-53 compliance...")
        
        nist_controls = {
            'AC-2': {  # Account Management
                'name': 'Account Management',
                'check': lambda: self._check_config('user.settings', 'register', 'admin_only'),
                'severity': 'MODERATE'
            },
            'AC-3': {  # Access Enforcement
                'name': 'Access Enforcement',
                'check': lambda: all([
                    'content_access' in self.recipe['install'],
                    'field_permissions' in self.recipe['install']
                ]),
                'severity': 'MODERATE'
            },
            'AU-3': {  # Content of Audit Records
                'name': 'Audit Record Content',
                'check': lambda: self._check_config('audit_log.settings', 'log_user_operations', True),
                'severity': 'LOW'
            },
            'IA-2': {  # Multi-Factor Authentication
                'name': 'Multi-Factor Authentication',
                'check': lambda: 'tfa' in self.recipe['install'],
                'severity': 'MODERATE'
            },
            'SC-8': {  # Transmission Confidentiality
                'name': 'Transmission Protection',
                'check': lambda: self._check_hsts(),
                'severity': 'MODERATE'
            },
            'SI-2': {  # Flaw Remediation
                'name': 'Security Updates',
                'check': lambda: 'security_review' in self.recipe['install'],
                'severity': 'MODERATE'
            }
        }
        
        results = self._run_compliance_checks(nist_controls)
        self.results['compliance_checks']['nist'] = results
        return results
    
    def _check_config(self, config_key, setting, expected_value, operator='=='):
        """Check if a configuration setting meets expected value"""
        config = self.recipe.get('config', {}).get('import', {})
        if config_key not in config:
            return False
        
        config_section = config[config_key]
        
        # Navigate nested settings
        if '.' in setting:
            parts = setting.split('.')
            value = config_section
            for part in parts:
                if isinstance(value, dict) and part in value:
                    value = value[part]
                else:
                    return False
        else:
            value = config_section.get(setting)
        
        if operator == '==':
            return value == expected_value
        elif operator == '<=':
            return value <= expected_value
        elif operator == '>=':
            return value >= expected_value
        else:
            return False
    
    def _check_password_policy(self):
        """Check if password policy meets government standards"""
        policy = self.recipe.get('config', {}).get('import', {}).get(
            'password_policy.password_policy.government_grade', {}
        )
        return (
            policy.get('password_length', 0) >= 14 and
            policy.get('password_character_types', 0) >= 4
        )
    
    def _check_hsts(self):
        """Check if HSTS is properly configured"""
        seckit = self.recipe.get('config', {}).get('import', {}).get('seckit.settings', {})
        hsts = seckit.get('seckit_ssl', {}).get('hsts', {})
        return (
            hsts.get('checkbox', False) is True and
            hsts.get('max_age', 0) >= 31536000
        )
    
    def _run_compliance_checks(self, controls):
        """Run compliance checks and calculate scores"""
        results = {
            'controls': {},
            'passed': 0,
            'failed': 0,
            'score': 0
        }
        
        for control_id, control in controls.items():
            try:
                passed = control['check']()
                results['controls'][control_id] = {
                    'name': control['name'],
                    'status': 'PASS' if passed else 'FAIL',
                    'severity': control['severity']
                }
                if passed:
                    results['passed'] += 1
                else:
                    results['failed'] += 1
            except Exception as e:
                results['controls'][control_id] = {
                    'name': control['name'],
                    'status': 'ERROR',
                    'severity': control['severity'],
                    'error': str(e)
                }
                results['failed'] += 1
        
        total_controls = len(controls)
        results['score'] = (results['passed'] / total_controls * 100) if total_controls > 0 else 0
        
        print(f"   ✓ Passed: {results['passed']}/{total_controls} controls")
        print(f"   ✗ Failed: {results['failed']}/{total_controls} controls")
        print(f"   Score: {results['score']:.1f}%")
        
        return results
    
    def generate_report(self, output_format='json'):
        """Generate compliance report"""
        print("\n📄 Generating compliance report...")
        
        # Calculate overall compliance score
        scores = []
        for standard, results in self.results['compliance_checks'].items():
            scores.append(results['score'])
        
        self.results['overall_score'] = sum(scores) / len(scores) if scores else 0
        
        # Save report
        timestamp = datetime.now().strftime('%Y%m%d_%H%M%S')
        report_dir = Path('/app/compliance_reports')
        report_dir.mkdir(exist_ok=True)
        
        if output_format == 'json':
            report_file = report_dir / f'compliance_report_{timestamp}.json'
            with open(report_file, 'w') as f:
                json.dump(self.results, f, indent=2)
        
        elif output_format == 'csv':
            # Flatten results for CSV
            rows = []
            for standard, results in self.results['compliance_checks'].items():
                for control_id, control in results['controls'].items():
                    rows.append({
                        'Standard': standard.upper(),
                        'Control ID': control_id,
                        'Control Name': control['name'],
                        'Status': control['status'],
                        'Severity': control['severity']
                    })
            
            df = pd.DataFrame(rows)
            report_file = report_dir / f'compliance_report_{timestamp}.csv'
            df.to_csv(report_file, index=False)
        
        print(f"   Report saved to: {report_file}")
        
        # Print summary
        print("\n📊 Compliance Summary:")
        print(f"   Overall Score: {self.results['overall_score']:.1f}%")
        for standard, results in self.results['compliance_checks'].items():
            print(f"   {standard.upper()}: {results['score']:.1f}% ({results['passed']}/{results['passed'] + results['failed']} passed)")
        
        return self.results['overall_score'] >= 80  # 80% threshold for passing
    
    def run(self, standards=None):
        """Run compliance validation"""
        print("🚀 Starting Compliance Validation")
        print("=" * 50)
        
        if standards is None:
            standards = ['fedramp', 'hipaa', 'soc2', 'nist']
        
        validators = {
            'fedramp': self.validate_fedramp,
            'hipaa': self.validate_hipaa,
            'soc2': self.validate_soc2,
            'nist': self.validate_nist
        }
        
        for standard in standards:
            if standard in validators:
                validators[standard]()
        
        passed = self.generate_report()
        
        if passed:
            print("\n✅ Compliance validation PASSED!")
            return 0
        else:
            print("\n❌ Compliance validation FAILED!")
            return 1


if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='Validate Secure Drupal recipe compliance')
    parser.add_argument('--standard', choices=['fedramp', 'hipaa', 'soc2', 'nist', 'all'],
                        default='all', help='Compliance standard to validate')
    parser.add_argument('--format', choices=['json', 'csv'], default='json',
                        help='Output format for report')
    
    args = parser.parse_args()
    
    validator = ComplianceValidator()
    
    if args.standard == 'all':
        exit_code = validator.run()
    else:
        exit_code = validator.run([args.standard])
    
    sys.exit(exit_code)