<?php

/**
 * @file
 * Drush integration for the Secure Drupal recipe.
 */

use Drupal\Core\StringTranslation\StringTranslationTrait;

/**
 * Implements hook_drush_command().
 */
function secure_drupal_drush_command() {
  $items = [];

  $items['secure:drupal-recipe'] = [
    'description' => 'Orchestrates security and compliance checks for a Drupal site.',
    'aliases' => ['sdr', 'secure-recipe'],
    'arguments' => [],
    'options' => [],
    'callback' => 'drush_secure_drupal_recipe',
  ];

  return $items;
}

/**
 * Implements hook_drush_help().
 */
function secure_drupal_drush_help($command) {
  if ($command == 'secure:drupal-recipe') {
    return dt('This command orchestrates security and compliance checks for a Drupal site.');
  }
}

/**
 * Drush command to orchestrate security and compliance checks.
 */
function drush_secure_drupal_recipe() {
  drush_log(dt('Running security scan...'));
  $root = \Drupal::root();
  $command = 'ddev drush -r ' . $root . ' secure:scan -y';
  $output = [];
  $return_var = 0;
  @exec($command, $output, $return_var);
  if ($return_var != 0) {
    drush_log(dt('Security scan failed with error code: ' . $return_var), 'error');
  }

  drush_log(dt('Running compliance checks...'));
  $root = \Drupal::root();
  $command = 'ddev drush -r ' . $root . ' secure:compliance --standard=gdpr -y';
  $output = [];
  $return_var = 0;
  @exec($command, $output, $return_var);
  if ($return_var != 0) {
    drush_log(dt('Compliance checks failed with error code: ' . $return_var), 'error');
  }

  drush_log(dt('Applying security hardening...'));
  $command = 'ddev drush -r ' . $root . ' secure:harden -y';
  $output = [];
  $return_var = 0;
  @exec($command, $output, $return_var);
  if ($return_var != 0) {
    drush_log(dt('Security hardening failed with error code: ' . $return_var), 'error');
  }

  drush_log(dt('Secure Drupal recipe completed.'));
}

/**
 * Implements hook_drush_requirements().
 */
function secure_drupal_drush_requirements($phase) {
  $requirements = [];
  if ($phase == 'runtime') {
    $requirements['secure_drupal_recipe'] = [
      'title' => 'Secure Drupal Recipe',
      'value' => 'Requires Drush 9 or later and the gov_compliance module',
      'severity' => REQUIREMENT_INFO,
    ];
  }
  return $requirements;
}

use Drush\Log\DrushLog;
use Consolidation\OutputFormatters\StructuredData\RowsOfFields;
use Consolidation\OutputFormatters\StructuredData\PropertyList;

/**
 * Localized Drupal translation.
 *
 * @param string $string
 *   String to translate.
 *
 * @return string
 *   Translated string.
 */
function dt(string $string) {
  return \Drupal::translation()->translate($string);
}

/**
 * Localized Drupal logger.
 *
 * @param string $message
 *   Message to log.
 * @param string $type
 *   Log type.
 */
function drush_log(string $message, string $type = 'notice') {
  \Drupal::logger('secure_drupal_recipe')->log($type, $message);
}

define('REQUIREMENT_INFO', 1);

/**
 * Implements hook_drush_version().
 */
function secure_drupal_drush_version() {
  return '8.x-1.0';
}
