#!/usr/bin/env node

/**
 * Security Scanner for Secure Drupal Recipe
 * Performs comprehensive security analysis of the recipe and its configurations
 */

const fs = require('fs').promises;
const path = require('path');
const { execSync } = require('child_process');
const yaml = require('js-yaml');

class SecurityScanner {
  constructor() {
    this.recipePath = '/app';
    this.scanResults = {
      timestamp: new Date().toISOString(),
      vulnerabilities: [],
      misconfigurations: [],
      compliance_gaps: [],
      recommendations: []
    };
  }

  async scanRecipeConfiguration() {
    console.log('🔍 Scanning recipe configuration...');
    
    const recipeFile = path.join(this.recipePath, 'recipe.yml');
    const recipeContent = await fs.readFile(recipeFile, 'utf8');
    const recipe = yaml.load(recipeContent);

    // Check for weak configurations
    const config = recipe.config.import;
    
    // Password policy checks
    if (config['password_policy.password_policy.government_grade'].password_length < 14) {
      this.scanResults.vulnerabilities.push({
        severity: 'HIGH',
        type: 'WEAK_PASSWORD_POLICY',
        message: 'Password length is less than 14 characters',
        recommendation: 'Increase password length to at least 14 characters'
      });
    }

    // Session timeout checks
    if (config['autologout.settings'].timeout > 900) {
      this.scanResults.vulnerabilities.push({
        severity: 'MEDIUM',
        type: 'LONG_SESSION_TIMEOUT',
        message: 'Session timeout exceeds 15 minutes',
        recommendation: 'Reduce session timeout to 900 seconds or less'
      });
    }

    // Encryption checks
    if (!config['encrypt.settings'].default_encryption_profile) {
      this.scanResults.vulnerabilities.push({
        severity: 'CRITICAL',
        type: 'MISSING_ENCRYPTION',
        message: 'No default encryption profile configured',
        recommendation: 'Configure default encryption profile for sensitive data'
      });
    }

    console.log(`   Found ${this.scanResults.vulnerabilities.length} potential issues`);
  }

  async scanDependencies() {
    console.log('📦 Scanning dependencies...');
    
    try {
      // Run npm audit
      const auditResult = execSync('npm audit --json', { encoding: 'utf8' });
      const audit = JSON.parse(auditResult);
      
      if (audit.vulnerabilities) {
        Object.entries(audit.vulnerabilities).forEach(([pkg, vuln]) => {
          this.scanResults.vulnerabilities.push({
            severity: vuln.severity.toUpperCase(),
            type: 'DEPENDENCY_VULNERABILITY',
            package: pkg,
            message: vuln.title || 'Vulnerability in dependency',
            recommendation: `Update ${pkg} to version ${vuln.fixAvailable?.version || 'latest'}`
          });
        });
      }
    } catch (error) {
      console.log('   No npm vulnerabilities found');
    }

    // Check for outdated packages
    try {
      const outdated = execSync('npm outdated --json', { encoding: 'utf8' });
      const packages = JSON.parse(outdated);
      
      Object.entries(packages).forEach(([pkg, info]) => {
        this.scanResults.recommendations.push({
          type: 'OUTDATED_PACKAGE',
          package: pkg,
          current: info.current,
          wanted: info.wanted,
          latest: info.latest,
          message: `Package ${pkg} is outdated`
        });
      });
    } catch (error) {
      console.log('   All packages are up to date');
    }
  }

  async scanSecurityHeaders() {
    console.log('🛡️  Scanning security headers configuration...');
    
    const recipeFile = path.join(this.recipePath, 'recipe.yml');
    const recipeContent = await fs.readFile(recipeFile, 'utf8');
    const recipe = yaml.load(recipeContent);
    const config = recipe.config.import;

    const requiredHeaders = {
      'X-Content-Type-Options': 'nosniff',
      'X-Frame-Options': 'DENY',
      'X-XSS-Protection': '1; mode=block',
      'Referrer-Policy': 'strict-origin-when-cross-origin',
      'Permissions-Policy': true
    };

    const httpHeaders = config['http_response_headers.settings']?.headers || {};

    Object.entries(requiredHeaders).forEach(([header, expectedValue]) => {
      if (!httpHeaders[header]) {
        this.scanResults.misconfigurations.push({
          severity: 'HIGH',
          type: 'MISSING_SECURITY_HEADER',
          header: header,
          message: `Security header ${header} is not configured`,
          recommendation: `Add ${header}: ${typeof expectedValue === 'string' ? expectedValue : 'appropriate value'}`
        });
      } else if (typeof expectedValue === 'string' && httpHeaders[header] !== expectedValue) {
        this.scanResults.misconfigurations.push({
          severity: 'MEDIUM',
          type: 'WEAK_SECURITY_HEADER',
          header: header,
          message: `Security header ${header} has weak value: ${httpHeaders[header]}`,
          recommendation: `Change to ${header}: ${expectedValue}`
        });
      }
    });

    // Check CSP
    const csp = config['csp.settings'];
    if (!csp?.enforce) {
      this.scanResults.misconfigurations.push({
        severity: 'HIGH',
        type: 'CSP_NOT_ENFORCED',
        message: 'Content Security Policy is not enforced',
        recommendation: 'Enable CSP enforcement'
      });
    }
  }

  async scanComplianceGaps() {
    console.log('🏛️  Scanning for compliance gaps...');
    
    const recipeFile = path.join(this.recipePath, 'recipe.yml');
    const recipeContent = await fs.readFile(recipeFile, 'utf8');
    const recipe = yaml.load(recipeContent);

    // FedRAMP specific checks
    const fedrampRequirements = {
      'continuous_monitoring': recipe.install.includes('admin_audit_trail'),
      'incident_response': recipe.config.import['audit_log.settings']?.log_user_operations,
      'access_control': recipe.install.includes('content_access'),
      'system_integrity': recipe.install.includes('security_review'),
      'data_encryption': recipe.install.includes('encrypt')
    };

    Object.entries(fedrampRequirements).forEach(([requirement, met]) => {
      if (!met) {
        this.scanResults.compliance_gaps.push({
          standard: 'FedRAMP',
          requirement: requirement,
          status: 'NOT_MET',
          message: `FedRAMP requirement ${requirement} is not fully implemented`,
          recommendation: `Implement ${requirement} controls`
        });
      }
    });

    // HIPAA specific checks
    const hipaaRequirements = {
      'access_controls': recipe.install.includes('field_permissions'),
      'audit_controls': recipe.config.import['admin_audit_trail.settings'] !== undefined,
      'integrity': recipe.config.import['seckit.settings'] !== undefined,
      'transmission_security': recipe.config.import['seckit.settings']?.seckit_ssl?.hsts?.checkbox === true
    };

    Object.entries(hipaaRequirements).forEach(([requirement, met]) => {
      if (!met) {
        this.scanResults.compliance_gaps.push({
          standard: 'HIPAA',
          requirement: requirement,
          status: 'NOT_MET',
          message: `HIPAA requirement ${requirement} is not fully implemented`,
          recommendation: `Implement ${requirement} safeguards`
        });
      }
    });
  }

  async generateReport() {
    console.log('\n📊 Generating security scan report...');
    
    const reportPath = path.join('/app/scan_results', `security-scan-${Date.now()}.json`);
    await fs.mkdir(path.dirname(reportPath), { recursive: true });
    await fs.writeFile(reportPath, JSON.stringify(this.scanResults, null, 2));
    
    // Generate summary
    const summary = {
      total_vulnerabilities: this.scanResults.vulnerabilities.length,
      critical: this.scanResults.vulnerabilities.filter(v => v.severity === 'CRITICAL').length,
      high: this.scanResults.vulnerabilities.filter(v => v.severity === 'HIGH').length,
      medium: this.scanResults.vulnerabilities.filter(v => v.severity === 'MEDIUM').length,
      low: this.scanResults.vulnerabilities.filter(v => v.severity === 'LOW').length,
      misconfigurations: this.scanResults.misconfigurations.length,
      compliance_gaps: this.scanResults.compliance_gaps.length,
      recommendations: this.scanResults.recommendations.length
    };

    console.log('\n📈 Scan Summary:');
    console.log(`   Total vulnerabilities: ${summary.total_vulnerabilities}`);
    console.log(`     - Critical: ${summary.critical}`);
    console.log(`     - High: ${summary.high}`);
    console.log(`     - Medium: ${summary.medium}`);
    console.log(`     - Low: ${summary.low}`);
    console.log(`   Misconfigurations: ${summary.misconfigurations}`);
    console.log(`   Compliance gaps: ${summary.compliance_gaps}`);
    console.log(`   Recommendations: ${summary.recommendations}`);
    
    if (summary.critical > 0) {
      console.log('\n❌ CRITICAL vulnerabilities found! Immediate action required.');
      process.exit(1);
    }
    
    return summary;
  }

  async run() {
    console.log('🚀 Starting Security Scanner for Secure Drupal Recipe');
    console.log('=' .repeat(50));
    
    try {
      await this.scanRecipeConfiguration();
      await this.scanDependencies();
      await this.scanSecurityHeaders();
      await this.scanComplianceGaps();
      await this.generateReport();
      
      console.log('\n✅ Security scan completed!');
    } catch (error) {
      console.error('\n❌ Security scan failed:', error.message);
      process.exit(1);
    }
  }
}

// Run scanner
if (require.main === module) {
  const scanner = new SecurityScanner();
  scanner.run();
}

module.exports = SecurityScanner;