<?php

namespace Drupal\secure_drupal\Commands;

use Drush\Commands\DrushCommands;
use Drush\Attributes as CLI;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Government-Grade Security Commands for Secure Drupal Recipe.
 * 
 * Provides comprehensive security management, compliance validation,
 * and audit capabilities for federal agencies and healthcare organizations.
 * 
 * Leverages the gov_compliance custom module for enterprise security orchestration.
 */
class SecureDrupalCommands extends DrushCommands {

  /**
   * The module handler.
   */
  protected ModuleHandlerInterface $moduleHandler;

  /**
   * The config factory.
   */
  protected ConfigFactoryInterface $configFactory;

  /**
   * The logger factory.
   */
  protected LoggerChannelFactoryInterface $loggerFactory;

  /**
   * SecureDrupalCommands constructor.
   */
  public function __construct(
    ModuleHandlerInterface $module_handler,
    ConfigFactoryInterface $config_factory,
    LoggerChannelFactoryInterface $logger_factory
  ) {
    parent::__construct();
    $this->moduleHandler = $module_handler;
    $this->configFactory = $config_factory;
    $this->loggerFactory = $logger_factory;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('module_handler'),
      $container->get('config.factory'),
      $container->get('logger.factory')
    );
  }

  /**
   * Deploy government-grade security framework.
   */
  #[CLI\Command(name: 'secure-drupal:deploy', aliases: ['sdd'])]
  #[CLI\Argument(name: 'compliance_level', description: 'Compliance level: fedramp, fedramp-high, hipaa, soc2, nist')]
  #[CLI\Option(name: 'modules', description: 'Comma-separated list of security modules to deploy')]
  #[CLI\Option(name: 'force', description: 'Force deployment even if already configured')]
  #[CLI\Option(name: 'documentation', description: 'Generate compliance documentation')]
  #[CLI\Usage(name: 'secure-drupal:deploy fedramp-high', description: 'Deploy FedRAMP HIGH compliance configuration')]
  #[CLI\Usage(name: 'secure-drupal:deploy hipaa --documentation', description: 'Deploy HIPAA compliance with documentation')]
  public function deploySecurityFramework(string $compliance_level = 'fedramp', array $options = ['modules' => '', 'force' => FALSE, 'documentation' => FALSE]): void {
    $this->output()->writeln("🔒 <info>Deploying Government-Grade Security Framework</info>");
    $this->output()->writeln("📋 Compliance Level: <comment>{$compliance_level}</comment>");

    // Validate compliance level
    $valid_levels = ['fedramp', 'fedramp-high', 'hipaa', 'soc2', 'nist', 'fisma'];
    if (!in_array($compliance_level, $valid_levels)) {
      throw new \InvalidArgumentException("Invalid compliance level. Valid options: " . implode(', ', $valid_levels));
    }

    try {
      // Check if gov_compliance module is available
      if ($this->moduleHandler->moduleExists('gov_compliance')) {
        // Use gov_compliance module for deployment
        $security_orchestration = \Drupal::service('gov_compliance.security_orchestration');
        
        $deployment_config = [
          'compliance_level' => $compliance_level,
          'modules' => $options['modules'] ? explode(',', $options['modules']) : 'all',
          'force_deployment' => $options['force'],
          'generate_documentation' => $options['documentation'],
        ];

        $result = $security_orchestration->deploySecurityFramework($deployment_config);

        if ($result['success']) {
          $this->output()->writeln("✅ <info>Security framework deployed successfully</info>");
          $this->output()->writeln("🔧 Configured modules: <comment>" . implode(', ', $result['configured_modules']) . "</comment>");
          
          if ($options['documentation']) {
            $this->output()->writeln("📄 Documentation generated: <comment>" . $result['documentation_path'] . "</comment>");
          }

          // Display security metrics
          $this->displaySecurityMetrics($result['security_metrics']);
        } else {
          $this->output()->writeln("❌ <error>Deployment failed: " . $result['error'] . "</error>");
          return;
        }
      } else {
        // Fallback to direct configuration
        $this->deploySecurityFrameworkFallback($compliance_level, $options);
      }

    } catch (\Exception $e) {
      $this->output()->writeln("❌ <error>Deployment error: " . $e->getMessage() . "</error>");
      throw $e;
    }
  }

  /**
   * Run comprehensive security audit.
   */
  #[Command(name: 'secure-drupal:audit', aliases: ['sda'])]
  #[Option(name: 'format', description: 'Output format: table, json, csv')]
  public function audit($options = ['format' => 'table']) {
    $this->output()->writeln("<info>Running comprehensive security audit...</info>");
    
    $results = [
      'password_policy' => $this->checkPasswordPolicy(),
      'two_factor_auth' => $this->checkTwoFactorAuth(),
      'session_security' => $this->checkSessionSecurity(),
      'encryption' => $this->checkEncryption(),
      'security_headers' => $this->checkSecurityHeaders(),
      'audit_logging' => $this->checkAuditLogging(),
    ];
    
    $this->displayResults($results, $options['format']);
  }

  /**
   * Generate compliance report.
   */
  #[Command(name: 'secure-drupal:compliance', aliases: ['sdc'])]
  #[Argument(name: 'standard', description: 'Compliance standard: fedramp, hipaa, soc2, nist, all')]
  #[Option(name: 'format', description: 'Output format: table, json, pdf')]
  public function compliance($standard = 'all', $options = ['format' => 'table']) {
    $this->output()->writeln("<info>Generating compliance report for $standard...</info>");
    
    $standards = ($standard === 'all') 
      ? ['fedramp', 'hipaa', 'soc2', 'nist'] 
      : [$standard];
    
    $report = [];
    foreach ($standards as $std) {
      $report[$std] = $this->generateComplianceReport($std);
    }
    
    $this->displayComplianceReport($report, $options['format']);
  }

  /**
   * Install all required security modules.
   */
  #[Command(name: 'secure-drupal:install-modules', aliases: ['sdim'])]
  public function installModules() {
    $this->output()->writeln("<info>Installing security modules...</info>");
    
    $modules = [
      'security_review',
      'password_policy',
      'autologout',
      'tfa',
      'login_security',
      'seckit',
      'captcha',
      'honeypot',
      'shield',
      'encrypt',
      'key',
      'field_encrypt',
      'gdpr',
      'csp',
      'paranoia',
    ];
    
    foreach ($modules as $module) {
      $this->output()->writeln("Installing $module...");
      $this->drush("pm:install $module -y");
    }
    
    $this->output()->writeln("<success>✅ All security modules installed!</success>");
  }

  /**
   * Enable AI security features.
   */
  #[Command(name: 'secure-drupal:enable-ai', aliases: ['sdea'])]
  #[Option(name: 'confirm', description: 'Confirm AI enablement')]
  #[Option(name: 'clearance', description: 'Security clearance level')]
  public function enableAI($options = ['confirm' => false, 'clearance' => '']) {
    if (!$options['confirm']) {
      throw new \Exception('AI features require explicit confirmation. Use --confirm flag.');
    }
    
    if ($options['clearance'] !== 'authorized') {
      throw new \Exception('AI features require authorized clearance. Use --clearance=authorized');
    }
    
    $this->output()->writeln("<info>Enabling AI security features...</info>");
    
    // Enable AI configuration
    \Drupal::configFactory()->getEditable('secure_drupal.settings')
      ->set('ai_enabled', TRUE)
      ->set('ai_clearance', $options['clearance'])
      ->set('ai_enabled_date', date('Y-m-d H:i:s'))
      ->save();
    
    $this->output()->writeln("<success>✅ AI security features enabled!</success>");
  }

  /**
   * Disable AI security features.
   */
  #[Command(name: 'secure-drupal:disable-ai', aliases: ['sdda'])]
  public function disableAI() {
    $this->output()->writeln("<info>Disabling AI security features...</info>");
    
    \Drupal::configFactory()->getEditable('secure_drupal.settings')
      ->set('ai_enabled', FALSE)
      ->set('ai_disabled_date', date('Y-m-d H:i:s'))
      ->save();
    
    $this->output()->writeln("<success>✅ AI security features disabled!</success>");
  }

  /**
   * Apply FedRAMP compliance settings.
   */
  private function applyFedRAMPCompliance() {
    // Apply FedRAMP-specific configurations
    $config = \Drupal::configFactory()->getEditable('password_policy.password_policy.government_grade');
    $config->set('password_length', 14);
    $config->set('password_character_types', 4);
    $config->save();
    
    // Configure session timeout
    $config = \Drupal::configFactory()->getEditable('autologout.settings');
    $config->set('timeout', 900); // 15 minutes
    $config->save();
  }

  /**
   * Apply HIPAA compliance settings.
   */
  private function applyHIPAACompliance() {
    // Enable audit logging
    $config = \Drupal::configFactory()->getEditable('admin_audit_trail.settings');
    $config->set('enabled', TRUE);
    $config->save();
    
    // Enable encryption
    $config = \Drupal::configFactory()->getEditable('encrypt.settings');
    $config->set('default_encryption_profile', 'government_aes');
    $config->save();
  }

  /**
   * Apply SOC2 compliance settings.
   */
  private function applySOC2Compliance() {
    // Configure security headers
    $config = \Drupal::configFactory()->getEditable('seckit.settings');
    $config->set('seckit_ssl.hsts.checkbox', TRUE);
    $config->set('seckit_ssl.hsts.max_age', 31536000);
    $config->save();
  }

  /**
   * Apply NIST compliance settings.
   */
  private function applyNISTCompliance() {
    // Configure access controls
    $config = \Drupal::configFactory()->getEditable('user.settings');
    $config->set('register', 'admin_only');
    $config->save();
  }

  /**
   * Check password policy configuration.
   */
  private function checkPasswordPolicy() {
    $config = \Drupal::config('password_policy.password_policy.government_grade');
    return [
      'status' => $config->get('password_length') >= 14 ? 'PASS' : 'FAIL',
      'details' => 'Password length: ' . $config->get('password_length'),
    ];
  }

  /**
   * Check two-factor authentication.
   */
  private function checkTwoFactorAuth() {
    $config = \Drupal::config('tfa.settings');
    return [
      'status' => $config->get('enabled') ? 'PASS' : 'FAIL',
      'details' => 'TFA enabled: ' . ($config->get('enabled') ? 'Yes' : 'No'),
    ];
  }

  /**
   * Check session security.
   */
  private function checkSessionSecurity() {
    $config = \Drupal::config('autologout.settings');
    return [
      'status' => $config->get('timeout') <= 900 ? 'PASS' : 'FAIL',
      'details' => 'Session timeout: ' . $config->get('timeout') . ' seconds',
    ];
  }

  /**
   * Check encryption configuration.
   */
  private function checkEncryption() {
    $config = \Drupal::config('encrypt.settings');
    return [
      'status' => !empty($config->get('default_encryption_profile')) ? 'PASS' : 'FAIL',
      'details' => 'Encryption profile: ' . $config->get('default_encryption_profile'),
    ];
  }

  /**
   * Check security headers.
   */
  private function checkSecurityHeaders() {
    $config = \Drupal::config('seckit.settings');
    return [
      'status' => $config->get('seckit_ssl.hsts.checkbox') ? 'PASS' : 'FAIL',
      'details' => 'HSTS enabled: ' . ($config->get('seckit_ssl.hsts.checkbox') ? 'Yes' : 'No'),
    ];
  }

  /**
   * Check audit logging.
   */
  private function checkAuditLogging() {
    $config = \Drupal::config('audit_log.settings');
    return [
      'status' => $config->get('log_user_operations') ? 'PASS' : 'FAIL',
      'details' => 'User operations logged: ' . ($config->get('log_user_operations') ? 'Yes' : 'No'),
    ];
  }

  /**
   * Generate compliance report for a standard.
   */
  private function generateComplianceReport($standard) {
    $checks = [];
    
    switch ($standard) {
      case 'fedramp':
        $checks = [
          'password_policy' => $this->checkPasswordPolicy(),
          'session_timeout' => $this->checkSessionSecurity(),
          'two_factor_auth' => $this->checkTwoFactorAuth(),
          'audit_logging' => $this->checkAuditLogging(),
          'encryption' => $this->checkEncryption(),
        ];
        break;
      // Add other standards...
    }
    
    return $checks;
  }

  /**
   * Display results in requested format.
   */
  private function displayResults($results, $format) {
    switch ($format) {
      case 'json':
        $this->output()->writeln(json_encode($results, JSON_PRETTY_PRINT));
        break;
      case 'table':
      default:
        $rows = [];
        foreach ($results as $check => $result) {
          $rows[] = [
            $check,
            $result['status'],
            $result['details'],
          ];
        }
        $this->io()->table(['Check', 'Status', 'Details'], $rows);
        break;
    }
  }

  /**
   * Display compliance report.
   */
  private function displayComplianceReport($report, $format) {
    switch ($format) {
      case 'json':
        $this->output()->writeln(json_encode($report, JSON_PRETTY_PRINT));
        break;
      case 'pdf':
        $this->output()->writeln("<comment>PDF generation would require additional libraries.</comment>");
        // Fall through to table format
      case 'table':
      default:
        foreach ($report as $standard => $checks) {
          $this->output()->writeln("\n<info>$standard Compliance:</info>");
          $rows = [];
          foreach ($checks as $check => $result) {
            $rows[] = [
              $check,
              $result['status'],
              $result['details'],
            ];
          }
          $this->io()->table(['Check', 'Status', 'Details'], $rows);
        }
        break;
    }
  }
}