#!/usr/bin/env node

/**
 * Secure Drupal Recipe Worker Runner
 * Executes all workers in sequence to build and configure the secure Drupal installation
 */

const yaml = require('js-yaml');
const fs = require('fs').promises;
const path = require('path');

const RECIPE_PATH = path.join(__dirname, '..');
const RECIPE_FILE = path.join(RECIPE_PATH, 'recipe.yml');

class SecureDrupalRunner {
  constructor() {
    this.recipe = null;
    this.results = {
      audit: null,
      compliance: null,
      configuration: null,
      documentation: null
    };
  }

  async loadRecipe() {
    console.log('📋 Loading recipe.yml...');
    const recipeContent = await fs.readFile(RECIPE_FILE, 'utf8');
    this.recipe = yaml.load(recipeContent);
    console.log(`✅ Loaded recipe: ${this.recipe.name}`);
    console.log(`   Description: ${this.recipe.description}`);
    console.log(`   Modules to install: ${this.recipe.install.length}`);
    return this.recipe;
  }

  async runSecurityAudit() {
    console.log('\n🔍 Phase 1: Security Audit');
    console.log('=' .repeat(50));
    
    const auditResults = {
      timestamp: new Date().toISOString(),
      recipe_name: this.recipe.name,
      modules_count: this.recipe.install.length,
      security_modules: [],
      configuration_items: [],
      actions: []
    };

    // Analyze security modules
    const securityModules = [
      'security_review', 'password_policy', 'autologout', 'tfa', 
      'login_security', 'seckit', 'captcha', 'honeypot', 'shield',
      'encrypt', 'key', 'field_encrypt', 'gdpr', 'csp', 'paranoia'
    ];

    for (const module of this.recipe.install) {
      if (securityModules.includes(module)) {
        auditResults.security_modules.push(module);
        console.log(`   ✓ Security module found: ${module}`);
      }
    }

    // Analyze configurations
    if (this.recipe.config && this.recipe.config.import) {
      auditResults.configuration_items = Object.keys(this.recipe.config.import);
      console.log(`   ✓ Configuration items: ${auditResults.configuration_items.length}`);
    }

    // Analyze actions
    if (this.recipe.actions) {
      auditResults.actions = this.recipe.actions.length;
      console.log(`   ✓ Security actions: ${auditResults.actions}`);
    }

    console.log(`\n📊 Audit Summary:`);
    console.log(`   - Security modules: ${auditResults.security_modules.length}/${this.recipe.install.length}`);
    console.log(`   - Configurations: ${auditResults.configuration_items.length}`);
    console.log(`   - Actions: ${auditResults.actions}`);

    this.results.audit = auditResults;
    return auditResults;
  }

  async runComplianceValidation() {
    console.log('\n🏛️  Phase 2: Compliance Validation');
    console.log('=' .repeat(50));

    const complianceResults = {
      timestamp: new Date().toISOString(),
      standards: ['FedRAMP', 'HIPAA', 'SOC2', 'NIST 800-53', 'FISMA'],
      validations: {}
    };

    // Check FedRAMP requirements
    complianceResults.validations.fedramp = {
      password_policy: this.recipe.config.import['password_policy.password_policy.government_grade'] ? 'PASS' : 'FAIL',
      session_timeout: this.recipe.config.import['autologout.settings']?.timeout === 900 ? 'PASS' : 'FAIL',
      two_factor_auth: this.recipe.config.import['tfa.settings']?.enabled ? 'PASS' : 'FAIL',
      audit_logging: this.recipe.config.import['audit_log.settings'] ? 'PASS' : 'FAIL',
      encryption: this.recipe.config.import['encrypt.settings'] ? 'PASS' : 'FAIL'
    };

    // Check HIPAA requirements
    complianceResults.validations.hipaa = {
      access_controls: this.recipe.install.includes('field_permissions') ? 'PASS' : 'FAIL',
      audit_controls: this.recipe.config.import['admin_audit_trail.settings'] ? 'PASS' : 'FAIL',
      integrity_controls: this.recipe.config.import['seckit.settings'] ? 'PASS' : 'FAIL',
      transmission_security: this.recipe.config.import['seckit.settings']?.seckit_ssl?.hsts?.checkbox ? 'PASS' : 'FAIL'
    };

    // Check NIST 800-53 requirements
    complianceResults.validations.nist = {
      access_control: this.recipe.install.includes('content_access') ? 'PASS' : 'FAIL',
      audit_accountability: this.recipe.config.import['audit_log.settings'] ? 'PASS' : 'FAIL',
      identification_authentication: this.recipe.config.import['tfa.settings'] ? 'PASS' : 'FAIL',
      system_communications: this.recipe.config.import['csp.settings'] ? 'PASS' : 'FAIL'
    };

    // Print compliance summary
    for (const [standard, checks] of Object.entries(complianceResults.validations)) {
      const passed = Object.values(checks).filter(v => v === 'PASS').length;
      const total = Object.values(checks).length;
      console.log(`\n   ${standard.toUpperCase()} Compliance: ${passed}/${total} checks passed`);
      for (const [check, status] of Object.entries(checks)) {
        console.log(`     - ${check}: ${status === 'PASS' ? '✅' : '❌'} ${status}`);
      }
    }

    this.results.compliance = complianceResults;
    return complianceResults;
  }

  async runModuleConfiguration() {
    console.log('\n⚙️  Phase 3: Module Configuration');
    console.log('=' .repeat(50));

    const configResults = {
      timestamp: new Date().toISOString(),
      configured_modules: [],
      configuration_summary: {}
    };

    // Generate configuration summary
    for (const [configKey, configValue] of Object.entries(this.recipe.config.import || {})) {
      const moduleName = configKey.split('.')[0];
      if (!configResults.configuration_summary[moduleName]) {
        configResults.configuration_summary[moduleName] = [];
      }
      configResults.configuration_summary[moduleName].push(configKey);
      configResults.configured_modules.push(moduleName);
    }

    // Remove duplicates
    configResults.configured_modules = [...new Set(configResults.configured_modules)];

    console.log(`\n   📦 Modules to configure: ${configResults.configured_modules.length}`);
    for (const module of configResults.configured_modules.slice(0, 10)) {
      console.log(`     - ${module}`);
    }
    if (configResults.configured_modules.length > 10) {
      console.log(`     ... and ${configResults.configured_modules.length - 10} more`);
    }

    this.results.configuration = configResults;
    return configResults;
  }

  async generateDocumentation() {
    console.log('\n📄 Phase 4: Documentation Generation');
    console.log('=' .repeat(50));

    const docResults = {
      timestamp: new Date().toISOString(),
      documents_generated: []
    };

    // Generate Security Configuration Report
    const securityReport = {
      title: 'Security Configuration Report',
      generated_at: new Date().toISOString(),
      recipe: this.recipe.name,
      security_modules: this.results.audit.security_modules,
      compliance_standards: this.results.compliance.standards,
      configuration_summary: this.results.configuration.configuration_summary
    };

    const reportPath = path.join(RECIPE_PATH, 'security-configuration-report.json');
    await fs.writeFile(reportPath, JSON.stringify(securityReport, null, 2));
    docResults.documents_generated.push('security-configuration-report.json');
    console.log(`   ✓ Generated: security-configuration-report.json`);

    // Generate Compliance Summary
    const complianceSummary = {
      title: 'Compliance Validation Summary',
      generated_at: new Date().toISOString(),
      recipe: this.recipe.name,
      compliance_results: this.results.compliance.validations
    };

    const compliancePath = path.join(RECIPE_PATH, 'compliance-summary.json');
    await fs.writeFile(compliancePath, JSON.stringify(complianceSummary, null, 2));
    docResults.documents_generated.push('compliance-summary.json');
    console.log(`   ✓ Generated: compliance-summary.json`);

    // Generate Implementation Guide
    const implementationGuide = `# Secure Drupal Recipe Implementation Guide

Generated: ${new Date().toISOString()}

## Recipe Overview
- **Name**: ${this.recipe.name}
- **Description**: ${this.recipe.description}
- **Type**: ${this.recipe.type}

## Security Modules (${this.results.audit.security_modules.length})
${this.results.audit.security_modules.map(m => `- ${m}`).join('\n')}

## Compliance Standards
${this.results.compliance.standards.map(s => `- ${s}`).join('\n')}

## Implementation Steps

1. **Install Drupal 11**
   \`\`\`bash
   composer create-project drupal/recommended-project:^11.0 my-secure-site
   cd my-secure-site
   \`\`\`

2. **Apply Security Recipe**
   \`\`\`bash
   drush recipe secure_drupal
   \`\`\`

3. **Run Security Audit**
   \`\`\`bash
   drush security-review
   \`\`\`

4. **Configure Two-Factor Authentication**
   - Enable TFA for all administrative users
   - Configure backup codes

5. **Review Security Headers**
   - Verify HSTS is enabled
   - Check CSP configuration
   - Confirm X-Frame-Options is set to DENY

## Post-Installation Checklist
- [ ] All admin users have 2FA enabled
- [ ] Password policy is enforced
- [ ] Auto-logout is configured (15 minutes)
- [ ] Security review shows no critical issues
- [ ] HTTPS is enforced site-wide
- [ ] Audit logging is enabled
- [ ] Backup encryption is configured

## Maintenance Schedule
- **Daily**: Review audit logs
- **Weekly**: Run security review
- **Monthly**: Update security modules
- **Quarterly**: Compliance audit
`;

    const guidePath = path.join(RECIPE_PATH, 'IMPLEMENTATION_GUIDE.md');
    await fs.writeFile(guidePath, implementationGuide);
    docResults.documents_generated.push('IMPLEMENTATION_GUIDE.md');
    console.log(`   ✓ Generated: IMPLEMENTATION_GUIDE.md`);

    this.results.documentation = docResults;
    return docResults;
  }

  async run() {
    console.log('🚀 Starting Secure Drupal Recipe Builder');
    console.log('=' .repeat(50));

    try {
      // Load recipe
      await this.loadRecipe();

      // Run all phases
      await this.runSecurityAudit();
      await this.runComplianceValidation();
      await this.runModuleConfiguration();
      await this.generateDocumentation();

      console.log('\n✅ All phases completed successfully!');
      console.log('\n📊 Final Summary:');
      console.log(`   - Security modules: ${this.results.audit.security_modules.length}`);
      console.log(`   - Compliance standards: ${this.results.compliance.standards.length}`);
      console.log(`   - Configured modules: ${this.results.configuration.configured_modules.length}`);
      console.log(`   - Documents generated: ${this.results.documentation.documents_generated.length}`);

      console.log('\n📁 Generated files:');
      for (const doc of this.results.documentation.documents_generated) {
        console.log(`   - ${doc}`);
      }

      console.log('\n🎉 Secure Drupal recipe build complete!');
      console.log('   Next step: Apply the recipe to your Drupal site using:');
      console.log('   drush recipe secure_drupal');

    } catch (error) {
      console.error('\n❌ Error:', error.message);
      process.exit(1);
    }
  }
}

// Run if called directly
if (require.main === module) {
  const runner = new SecureDrupalRunner();
  runner.run();
}

module.exports = SecureDrupalRunner;