# AI Assistant Integration Guide

This document explains how to use the AI Assistant Adapter Framework to generate configuration files for different AI assistants from your coding standards.

## Overview

The AI Assistant Adapter Framework provides a flexible system for transforming your structured coding standards into configurations for different AI assistants such as Claude, GitHub Copilot, and others. This allows you to ensure consistent coding standards across all AI-assisted development in your projects.

## Supported AI Assistants

The framework currently supports the following AI assistants:

- **Claude** (Anthropic) - Generates CLAUDE.md files
- **GitHub Copilot** - Generates JSON configuration files
- **Generic** - Generates standard markdown format for any AI assistant

## Using the CLI

The easiest way to generate AI assistant configurations is through the ReelCode CLI:

```bash
# Generate CLAUDE.md from standards.yaml
tddai generate-ai-config --standards standards.yaml --output CLAUDE.md

# Generate GitHub Copilot configuration
tddai generate-ai-config --standards standards.yaml --type copilot --output .github/copilot/config.json

# Generate with custom sections
tddai generate-ai-config --standards standards.yaml --output CLAUDE.md --sections "Project Guidelines=Follow all security protocols" "Code Review=All changes require review"

# Specify a project ID
tddai generate-ai-config --standards standards.yaml --output CLAUDE.md --project my-project
```

## Programmatic Usage

You can also use the framework programmatically in your Node.js applications:

```typescript
import [StandardsManager] from 'tddai/core';
import { AIAssistantAdapter, AIAssistantType } from 'tddai/ai';

async function generateAIConfigs() {
  // Initialize standards manager
  const manager = new StandardsManager({
    projectId: 'my-project',
    configPath: './standards.yaml'
  });
  await manager.initialize();
  
  // Generate Claude configuration
  await manager.generateAIAssistantConfig({
    assistantType: AIAssistantType.Claude,
    outputPath: './CLAUDE.md',
    assistantOptions: {
      customSections: {
        'Custom Section': 'This is custom content'
      }
    }
  });
  
  // Direct adapter usage
  const copilotAdapter = AIAssistantAdapter.create(AIAssistantType.Copilot);
  const standards = manager.getStandards();
  const config = await copilotAdapter.generateConfig(standards, {
    projectName: 'my-project'
  });
  await copilotAdapter.applyConfig(config, './.github/copilot/config.json');
}
```

## Creating Custom Adapters

You can extend the framework by creating custom adapters for additional AI assistants:

```typescript
import { AIAssistantAdapter, AIAssistantConfigOptions } from 'tddai/ai';
import * as fs from 'fs/promises';
import * as path from 'path';

// Custom adapter implementation
class MyCustomAdapter extends AIAssistantAdapter {
  async generateConfig(standards: any, options?: AIAssistantConfigOptions): Promise<string> {
    // Transform standards into your custom format
    const config = {
      name: standards.name,
      rules: standards.core?.codingStyle || {},
      customData: options?.templateVariables || {}
    };
    
    return JSON.stringify(config, null, 2);
  }
  
  async applyConfig(config: string, outputPath: string): Promise<void> {
    await fs.mkdir(path.dirname(outputPath), { recursive: true });
    await fs.writeFile(outputPath, config, 'utf-8');
  }
}

// Register your custom adapter
AIAssistantAdapter.register('my-custom', MyCustomAdapter);

// Use your custom adapter
const adapter = AIAssistantAdapter.create('my-custom');
```

## Integration with Standards Manager

The `StandardsManager` class provides direct integration with the AI Assistant Adapter framework:

1. Initialize the standards manager with your standards file
2. Call `generateAIAssistantConfig` with the desired assistant type and output path
3. The framework will handle the transformation and file generation

This provides a clean, high-level API for generating AI assistant configurations as part of your development workflow.

## Best Practices

1. **Keep Standards in Version Control**: Store your standards YAML/JSON files in version control to track changes
2. **Generate in CI/CD**: Automatically generate AI assistant configurations as part of your CI/CD pipeline
3. **Project-Specific Standards**: Use project-specific sections for different codebases
4. **Custom Templates**: Create custom templates for more fine-grained control over the generated configurations
5. **Standards Inheritance**: Use the `extends` property to create hierarchical standards

## Example Standards Structure

For reference, here's a simplified example of a standards file:

```yaml
name: Company Coding Standards
version: 1.0
description: Standard coding practices for all company projects

core:
  codingStyle:
    typescript:
      strictMode: true
      preferConst: true
      namingConventions:
        classes: PascalCase
        interfaces: IPascalCase
        variables: camelCase
        constants: UPPER_SNAKE_CASE

  testingRequirements:
    coverageThreshold: 80
    methodology: TDD
    requiredTests:
      - unit
      - integration

  security:
    inputValidation: true
    authenticationRequired: true
```

## Troubleshooting

If you encounter issues with AI assistant configuration generation:

1. Validate your standards file using the standards validator
2. Enable debug mode with the `--debug` flag
3. Check that the output directory exists and is writable
4. Verify that your standards include the sections expected by the specific AI assistant adapter