# TDDAI AI Integration Guide

## Overview

TDDAI enhances AI coding assistants like Claude Code, Cursor, and GitHub Copilot by providing automatic context injection and intelligent project analysis. This allows AI assistants to understand your codebase without manual explanation.

## Key Features

### 1. Automatic Context Injection
- Runs in the background, continuously analyzing your project
- Feeds comprehensive context to AI assistants automatically
- No need to explain project structure, dependencies, or patterns

### 2. Intelligent Code Analysis
- Deep project structure analysis
- Pattern recognition and architecture detection
- Test coverage and quality metrics
- Dependency risk assessment

### 3. AI-to-AI Communication
- Standardized protocol for AI assistant communication
- Real-time updates as code changes
- Proactive warnings and suggestions

### 4. File Organization Enforcement
- Prevents scattered documentation files
- Enforces clean project structure
- Automatic cleanup suggestions

## Getting Started

### Installation

```bash
npm install -g @bluefly/tddai
```

### Quick Setup

```bash
# Setup all AI integrations
tddai ai setup-all

# Start automatic context injection
tddai ai inject

# Analyze project for AI readiness
tddai ai analyze
```

## AI Commands

### Context Injection

Start automatic background context injection:

```bash
tddai ai inject [options]

Options:
  -p, --path <path>      Project path (default: current directory)
  --watch               Enable file watching (default: true)
  --realtime            Enable real-time analysis (default: true)
  --frequency <seconds> Update frequency (default: 30)
```

### Project Analysis

Analyze project for AI optimization:

```bash
tddai ai analyze [options]

Options:
  -p, --path <path>     Project path (default: current directory)
  --detailed           Generate detailed analysis report
  --save <file>        Save analysis to file
```

### Generate AI Prompts

Generate intelligent prompts with full context:

```bash
tddai ai prompt <query> [options]

Options:
  -f, --file <file>    Current file context
  -o, --output <file>  Output file for the prompt
  --json              Output as JSON
```

### Setup AI Assistants

#### Setup Cursor IDE Integration

```bash
tddai ai setup-cursor [options]

Options:
  -p, --path <path>        Project path
  -t, --type <type>        Project type (default: node)
  -f, --framework <fw>     Test framework (default: jest)
  -c, --coverage <number>  Coverage threshold (default: 95)
```

#### Setup Claude Code Integration

```bash
tddai ai setup-claude [options]

Options:
  -p, --path <path>        Project path
  -t, --type <type>        Project type (default: node)
  -f, --framework <fw>     Test framework (default: jest)
  -c, --coverage <number>  Coverage threshold (default: 95)
```

#### Setup All AI Integrations

```bash
tddai ai setup-all [options]

# This creates:
# - .cursorrules (Cursor IDE guidance)
# - CLAUDE.md (Claude Code context)
# - .ai-standards.json (machine-readable standards)
# - .vscode/settings.json (optimized workspace)
```

### Validate AI Integration

Check if AI integration is properly configured:

```bash
tddai ai validate [options]

Options:
  -p, --path <path>    Project path
```

## How It Works

### 1. Project Analysis

TDDAI continuously analyzes your project:

```javascript
// What TDDAI analyzes
{
  projectType: "node",
  frameworks: ["express", "jest"],
  files: 127,
  testCoverage: 92,
  qualityScore: 85,
  architecture: ["MVC", "REST API"],
  dependencies: {
    production: 23,
    development: 15,
    risks: 2
  }
}
```

### 2. Context Generation

Generates comprehensive context for AI assistants:

```javascript
// Generated context includes
{
  projectOverview: "Node.js REST API with Express",
  testingStrategy: "Jest with 95% coverage requirement",
  architecturePatterns: "MVC with service layer",
  criticalInsights: [
    "Low test coverage in controllers/",
    "Circular dependency detected",
    "3 high-complexity functions"
  ],
  recommendations: [
    "Add tests for uncovered functions",
    "Refactor complex authentication logic",
    "Update deprecated dependencies"
  ]
}
```

### 3. Automatic Injection

Context is automatically provided to AI assistants:

```javascript
// .tddai-context.json (automatically updated)
{
  "version": "1.0",
  "type": "context",
  "timestamp": "2024-01-15T10:30:00Z",
  "payload": {
    "context": {
      "projectType": "node",
      "frameworks": ["express", "jest"],
      "currentFile": "src/controllers/userController.js",
      "testCoverage": 87,
      "qualityScore": 82,
      "recommendations": ["Increase test coverage to 95%"]
    }
  }
}
```

## AI Assistant Benefits

### Claude Code
- Automatic project understanding
- Context-aware code suggestions
- Proactive issue detection
- Pattern-matching code generation

### Cursor IDE
- Enhanced AI suggestions via .cursorrules
- Project-specific completions
- TDD workflow enforcement
- Intelligent refactoring

### GitHub Copilot
- Better code predictions
- Project-aligned suggestions
- Test-first development support
- Quality standard enforcement

## Examples

### Example 1: Debugging with Context

Without TDDAI:
```
User: "Why is this function failing?"
Claude: "I need to see the function code, error message, and understand your setup..."
```

With TDDAI:
```
User: "Why is this function failing?"
Claude: "I can see this function in src/services/authService.js has:
- No test coverage (your requirement is 95%)
- A circular dependency with userService
- High complexity score (18, threshold is 15)
The likely issue is... [specific solution]"
```

### Example 2: Writing Tests

Without TDDAI:
```
User: "Write tests for userController"
Claude: "What testing framework do you use? What patterns should I follow?"
```

With TDDAI:
```
User: "Write tests for userController"
Claude: "I'll write Jest tests following your patterns:
- Using supertest for API testing (like in your other controllers)
- Following your AAA pattern (Arrange-Act-Assert)
- Including edge cases for your 95% coverage requirement
- Using your mock factories from __tests__/fixtures/
[Generates tests matching your exact patterns]"
```

### Example 3: Architecture Decisions

Without TDDAI:
```
User: "Should I create a new service for this?"
Claude: "That depends on your architecture..."
```

With TDDAI:
```
User: "Should I create a new service for this?"
Claude: "Yes, based on your MVC + Service Layer architecture:
- Business logic belongs in services/ not controllers/
- You have a similar pattern in userService.js
- Create src/services/paymentService.js
- Add corresponding tests in __tests__/unit/services/
- Follow your dependency injection pattern"
```

## File Organization Integration

TDDAI enforces clean file organization:

```bash
# Check organization
tddai organize validate

# Issues detected:
# - 12 random .md files in src/
# - 5 temporary scripts not cleaned up
# - Documentation scattered across project

# Fix automatically
tddai organize enforce
```

## Performance Considerations

- Minimal overhead: ~50MB memory usage
- File watching: Efficient incremental updates
- Background processing: Non-blocking analysis
- Caching: Avoids redundant analysis

## Troubleshooting

### Context not updating?

```bash
# Check if injection is running
ps aux | grep tddai

# Restart injection
tddai ai inject --watch --realtime
```

### AI assistant not seeing context?

```bash
# Validate setup
tddai ai validate

# Regenerate configuration
tddai ai setup-all
```

### High memory usage?

```bash
# Run with limited scope
tddai ai inject --path src/ --frequency 60
```

## Advanced Configuration

### Custom Context Templates

Create `.tddai/context-template.json`:

```json
{
  "includePatterns": ["src/**/*.js", "tests/**/*.js"],
  "excludePatterns": ["node_modules/**", "dist/**"],
  "contextPriority": {
    "testCoverage": "high",
    "dependencies": "medium",
    "documentation": "low"
  },
  "updateFrequency": 30,
  "maxContextSize": 50000
}
```

### AI Protocol Extensions

Extend the protocol for custom tools:

```javascript
import [AIProtocol] from '@bluefly/tddai';

const protocol = new AIProtocol(projectRoot);

// Create custom message
const customMessage = {
  ...protocol.createContextMessage(context),
  payload: {
    ...basePayload,
    customData: {
      teamStandards: loadTeamStandards(),
      projectMetrics: getProjectMetrics()
    }
  }
};
```

## Best Practices

1. **Run continuously**: Keep `tddai ai inject` running during development
2. **Regular validation**: Run `tddai ai validate` before commits
3. **Clean organization**: Use `tddai organize enforce` regularly
4. **Update AI config**: Re-run `tddai ai setup-all` after major changes
5. **Monitor performance**: Check context file size and update frequency

## Integration with CI/CD

Add to your CI pipeline:

```yaml
# .github/workflows/ai-validation.yml
steps:
  - name: Validate AI Integration
    run: |
      npm install -g @bluefly/tddai
      tddai ai validate
      tddai organize validate
      tddai tdd validate
```

## Future Enhancements

- Multi-language support beyond JavaScript/TypeScript
- IDE plugins for VS Code and IntelliJ
- Cloud-based context sharing for teams
- AI model fine-tuning based on your codebase
- Automated PR reviews with context awareness

## Support

- GitHub Issues: https://github.com/bluefly/tddai/issues
- Documentation: https://github.com/bluefly/tddai/docs
- Community: https://discord.gg/tddai

## Conclusion

TDDAI transforms AI coding assistants from generic helpers into project-aware development partners. By providing automatic context injection and continuous analysis, it eliminates the need to explain your project repeatedly and ensures AI suggestions align with your specific patterns and standards.