# CI/CD Integration

This document provides instructions for integrating ReelCode standards validation with CI/CD systems like GitLab CI and GitHub Actions.

## Overview

ReelCode provides built-in support for integrating standards validation into your CI/CD pipelines. This allows you to:

1. Automatically validate code against defined standards
2. Generate reports in various formats
3. Add validation status to pull requests and merge requests
4. Fail builds when standards are not met

## GitLab CI Integration

### Generating GitLab CI Configuration

Generate a GitLab CI configuration file:

```bash
tddai ci gitlab-config --output .gitlab-ci.yml
```

Options:
- `-o, --output <path>`: Output file path (default: `.gitlab-ci.yml`)
- `-s, --standards <path>`: Path to the standards file
- `-p, --project <id>`: Project ID
- `-n, --node-version <version>`: Node.js version (default: `18`)
- `-t, --template <path>`: Custom template path
- `--include-stages <stages>`: Additional stages to include (comma-separated)

### Merge Request Integration

Generate a merge request comment template:

```bash
tddai ci gitlab-mr-comment --output mr-comment.md
```

Options:
- `-o, --output <path>`: Output file path (default: `mr-comment.md`)
- `-p, --project <id>`: Project ID
- `-t, --template <path>`: Custom template path

### Example GitLab CI Configuration

```yaml
include:
  - template: Security/SAST.gitlab-ci.yml
  - local: '.gitlab/tddai-validation.yml'

stages:
  - validate
  - test
  - build

validate:standards:
  extends: .tddai-validate
  script:
    - tddai validate project --standards .tddai/standards.yaml
    - tddai validate project --output-format markdown --output tddai-report.md
  artifacts:
    paths:
      - tddai-report.md
```

## GitHub Actions Integration

### Generating GitHub Actions Workflow

Generate a GitHub Actions workflow:

```bash
tddai ci github-workflow --output .github/workflows/tddai-validation.yml
```

Options:
- `-o, --output <path>`: Output file path (default: `.github/workflows/tddai-validation.yml`)
- `-s, --standards <path>`: Path to the standards file
- `-p, --project <id>`: Project ID
- `-n, --node-version <version>`: Node.js version (default: `18`)
- `-t, --template <path>`: Custom template path
- `--push-branches <branches>`: Branches to trigger on push (comma-separated, default: `main,master`)
- `--pr-branches <branches>`: Branches to trigger on pull request (comma-separated, default: `main,master`)

### Pull Request Integration

Generate a pull request comment template:

```bash
tddai ci github-pr-comment --output pr-comment.md
```

Options:
- `-o, --output <path>`: Output file path (default: `pr-comment.md`)
- `-p, --project <id>`: Project ID
- `-t, --template <path>`: Custom template path

### Example GitHub Workflow

```yaml
name: ReelCode Validation

on:
  push:
    branches: [main]
  pull_request:
    branches: [main]

jobs:
  validate:
    name: Validate Standards
    runs-on: ubuntu-latest
    steps:
      - uses: actions/checkout@v3
      - uses: actions/setup-node@v3
        with:
          node-version: '18'
      - name: Install ReelCode
        run: npm install -g @bluefly/tddai
      - name: Validate Standards
        run: tddai validate project
```

## Badge Generation

Generate a standards compliance badge for your README:

```bash
tddai ci generate-badge --output badge.svg
```

Options:
- `-o, --output <path>`: Output file path (default: `badge.svg`)
- `-p, --project <id>`: Project ID
- `-s, --standards <path>`: Path to the standards file (default: `./standards.yaml`)
- `-c, --color <color>`: Badge color (default: `green`)

## Reporting Formats

ReelCode supports multiple report formats for CI/CD integration:

### Markdown Reports

```bash
tddai validate project --output-format markdown --output report.md
```

Ideal for:
- Pull request comments
- Documentation
- Human-readable reports

### JSON Reports

```bash
tddai validate project --output-format json --output report.json
```

Ideal for:
- Programmatic processing
- Custom integrations
- Detailed analysis

### JUnit XML Reports

```bash
tddai validate project --output-format junit-xml --output report.xml
```

Ideal for:
- CI/CD test reporting
- Integration with test visualization tools
- Jenkins, GitLab, and other CI systems that support JUnit format

## CI/CD Best Practices

1. **Fail the Build on Standard Violations**
   ```bash
   tddai validate project || exit 1
   ```

2. **Add Standards Badge to README**
   ```markdown
   ![Standards Compliance](https://example.com/badge.svg)
   ```

3. **Include Detailed Reports as Artifacts**
   ```yaml
   artifacts:
     paths:
       - tddai-report.md
       - tddai-report.json
   ```

4. **Comment on Pull Requests/Merge Requests**
   - GitHub Actions: Use the github-script action
   - GitLab CI: Use the GitLab API to post comments

5. **Integrate with Code Quality Tools**
   - SonarQube integration
   - Linting tools coordination

## Examples for Different Project Types

### TypeScript Project

```yaml
validate:typescript:
  extends: .tddai-validate
  script:
    - tddai validate project --include "**/*.ts,**/*.tsx" --include-validators typescript
```

### PHP Project

```yaml
validate:php:
  extends: .tddai-validate
  script:
    - tddai validate project --include "**/*.php" --include-validators php
```

### Multi-Language Project

```yaml
validate:all:
  extends: .tddai-validate
  script:
    - tddai validate project
  parallel:
    matrix:
      - LANGUAGE: [typescript, javascript, php]
```

## Troubleshooting

If you encounter issues with CI/CD integration:

1. **Validation fails but should pass**
   - Check the standards file is being loaded correctly
   - Verify file paths and patterns
   - Use verbose mode for detailed output: `tddai validate project --verbose`

2. **GitHub Actions workflow not running**
   - Ensure the workflow is in the correct directory (`.github/workflows/`)
   - Verify branch names match your repository structure
   - Check permissions for GitHub token

3. **GitLab CI pipeline not running**
   - Validate your `.gitlab-ci.yml` syntax
   - Ensure GitLab CI/CD is enabled for the project
   - Check runner availability

For more detailed information, see the [ReelCode documentation](https://gitlab.bluefly.io/llm/tddai/-/blob/main/README.md).