# Specific Cleanup Examples for Your Projects

## 1. LLM Module - Service Consolidation Example

### BEFORE: 108+ Services with Redundancy
```yaml
# llm.services.yml (BLOATED)
services:
  llm.openai.client:
    class: Drupal\llm\Service\OpenAiClient
    arguments: ['@http_client', '@config.factory', '@logger.factory']
  
  llm.anthropic.client:
    class: Drupal\llm\Service\AnthropicClient
    arguments: ['@http_client', '@config.factory', '@logger.factory']
  
  llm.groq.client:
    class: Drupal\llm\Service\GroqClient
    arguments: ['@http_client', '@config.factory', '@logger.factory']
  
  # ... 20+ more provider clients doing the same thing
  
  llm.cache.responses:
    class: Drupal\llm\Service\ResponseCache
  
  llm.cache.embeddings:
    class: Drupal\llm\Service\EmbeddingCache
  
  llm.cache.tokens:
    class: Drupal\llm\Service\TokenCache
  
  # ... 10+ more cache services
```

### AFTER: Leveraging Contrib + Clean Architecture
```yaml
# llm.services.yml (CLEAN)
services:
  # Use AI module for providers
  llm.provider_manager:
    class: Drupal\llm\Service\LlmProviderManager
    arguments: 
      - '@ai.provider'  # Use contrib AI module
      - '@key.repository'  # Use Key module for API keys
      - '@cache.llm'  # Single cache bin
      - '@logger.channel.llm'
  
  # Single unified cache service
  cache.llm:
    class: Drupal\Core\Cache\CacheBackendInterface
    factory: cache_factory:get
    arguments: [llm]
    tags:
      - { name: cache.bin }
  
  # Core LLM functionality only
  llm.orchestrator:
    class: Drupal\llm\Service\LlmOrchestrator
    arguments: ['@llm.provider_manager', '@plugin.manager.ai_agents']
```

### Implementation Example
```php
// BEFORE: Custom provider implementation
class OpenAiClient {
    private $httpClient;
    private $config;
    private $logger;
    
    public function __construct($http, $config, $logger) {
        $this->httpClient = $http;
        $this->config = $config;
        $this->logger = $logger;
    }
    
    public function complete($prompt) [// 200+ lines of custom HTTP code]
}

// AFTER: Extend AI module
namespace Drupal\llm\Plugin\AiProvider;

use Drupal\ai\Base\AiProviderBase;
use Drupal\ai\OperationType\Chat\ChatInput;
use Drupal\ai\OperationType\Chat\ChatOutput;

/**
 * @AiProvider(
 *   id = "openai_enhanced",
 *   label = @Translation("OpenAI Enhanced"),
 * )
 */
class OpenAiEnhanced extends AiProviderBase [/**
     * {@inheritdoc]
     */
    public function chat(ChatInput $input, $model_id = NULL, array $settings = []): ChatOutput {
        // Only OpenAI-specific enhancements
        $response = parent::chat($input, $model_id, $settings);
        
        // Add your custom logic
        $this->trackUsage($response);
        
        return $response;
    }
    
    protected function trackUsage(ChatOutput $output): void [// Your custom tracking]
}
```

## 2. MCP Client Extras - Clean HTTP Client Usage

### BEFORE: Custom HTTP Implementation
```php
// McpClient.php (BLOATED)
class McpClient {
    private $baseUrl;
    private $timeout = 30;
    
    public function request($method, $endpoint, $data = []) {
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $this->baseUrl . $endpoint);
        curl_setopt($ch, CURLOPT_CUSTOMREQUEST, $method);
        // ... 50+ lines of curl options
        
        $response = curl_exec($ch);
        if (curl_errno($ch)) {
            throw new \Exception(curl_error($ch));
        }
        // ... more error handling
    }
}
```

### AFTER: Using HTTP Client Manager
```yaml
# mcp_client_extras.http_services.yml
mcp_api:
  title: "MCP API"
  api_version: "1.0"
  base_uri: "https://mcp.example.com/api/v1/"
  guzzle_options:
    timeout: 30
    headers:
      Accept: 'application/json'
      Authorization: 'Bearer %key:mcp_api_key%'
```

```php
// McpClient.php (CLEAN)
namespace Drupal\mcp_client_extras\Service;

use Drupal\http_client_manager\HttpClientInterface;

class McpClient {
    
    public function __construct(
        private HttpClientInterface $httpClient,
    ) {}
    
    public function request(string $method, string $endpoint, array $data = []): array {
        $client = $this->httpClient->getClient('mcp_api');
        
        try {
            $response = $client->request($method, $endpoint, [
                'json' => $data,
            ]);
            
            return json_decode($response->getBody()->getContents(), TRUE);
        } catch (GuzzleException $e) {
            throw new McpException($e->getMessage(), $e->getCode(), $e);
        }
    }
}
```

## 3. Recipe Onboarding - State Machine Integration

### BEFORE: Custom State Tracking
```php
// OnboardingStateManager.php (BLOATED)
class OnboardingStateManager {
    const STATE_NEW = 'new';
    const STATE_PROFILE = 'profile';
    const STATE_PREFERENCES = 'preferences';
    const STATE_COMPLETE = 'complete';
    
    public function transitionTo($user, $newState) {
        $currentState = $this->getCurrentState($user);
        
        // 100+ lines of validation logic
        if ($currentState === self::STATE_NEW && $newState === self::STATE_COMPLETE) {
            throw new \Exception('Invalid transition');
        }
        // ... more validation
        
        $user->set('field_onboarding_state', $newState);
        $user->save();
    }
}
```

### AFTER: Using State Machine Module
```yaml
# config/install/state_machine.workflow.onboarding.yml
id: onboarding
label: 'User Onboarding'
states:
  new:
    label: 'New User'
  profile_setup:
    label: 'Profile Setup'
  preferences:
    label: 'Setting Preferences'
  completed:
    label: 'Onboarding Complete'
transitions:
  start:
    label: 'Start Onboarding'
    from: [new]
    to: profile_setup
  set_preferences:
    label: 'Set Preferences'
    from: [profile_setup]
    to: preferences
  complete:
    label: 'Complete Onboarding'
    from: [preferences]
    to: completed
```

```php
// OnboardingManager.php (CLEAN)
namespace Drupal\recipe_onboarding\Service;

use Drupal\state_machine\WorkflowManagerInterface;

class OnboardingManager {
    
    public function __construct(
        private WorkflowManagerInterface $workflowManager,
    ) {}
    
    public function advanceUserState(UserInterface $user, string $transition): void {
        $workflow = $this->workflowManager->getWorkflow('onboarding');
        $state_item = $user->get('field_onboarding_state')->first();
        
        if ($workflow->isTransitionAllowed($state_item, $transition)) {
            $state_item->applyTransition($transition);
            $user->save();
        }
    }
}
```

## 4. AI Provider Apple - Clean Plugin Implementation

### BEFORE: Standalone Implementation
```php
// AppleAiService.php (300+ lines)
class AppleAiService {
    public function __construct() {
        $this->config = \Drupal::config('apple_ai.settings');
        $this->logger = \Drupal::logger('apple_ai');
        $this->cache = \Drupal::cache();
        // ... more services
    }
    
    public function complete($prompt, $options = []) [// Custom implementation]
}
```

### AFTER: AI Module Plugin
```php
namespace Drupal\ai_provider_apple\Plugin\AiProvider;

use Drupal\ai\Base\AiProviderBase;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;

/**
 * @AiProvider(
 *   id = "apple_foundation",
 *   label = @Translation("Apple Foundation Models"),
 *   description = @Translation("Apple's on-device AI models"),
 * )
 */
class AppleFoundationProvider extends AiProviderBase implements ContainerFactoryPluginInterface [/**
     * {@inheritdoc]
     */
    public function getSupportedOperationTypes(): array {
        return ['chat', 'completion', 'embeddings'];
    }
    
    /**
     * [@inheritdoc]
     */
    public function chat(ChatInput $input, $model_id = NULL, array $settings = []): ChatOutput {
        // Apple-specific implementation
        $request = $this->buildAppleRequest($input, $model_id);
        $response = $this->callAppleApi($request);
        
        return new ChatOutput(
            $response['content'],
            $response['model'],
            $response['usage'] ?? []
        );
    }
    
    protected function callAppleApi(array $request): array {
        // Apple-specific API call
        return $this->httpClient->post('apple-ai://foundation', [
            'json' => $request,
        ])->toArray();
    }
}
```

## 5. Gov Compliance - Leveraging Security Modules

### BEFORE: Custom Security Implementation
```php
// SecurityHeaderManager.php (BLOATED)
class SecurityHeaderManager {
    public function setHeaders() {
        header('X-Frame-Options: SAMEORIGIN');
        header('X-Content-Type-Options: nosniff');
        header('X-XSS-Protection: 1; mode=block');
        header('Strict-Transport-Security: max-age=31536000');
        // ... 20+ more headers
    }
}

// PasswordValidator.php (CUSTOM)
class PasswordValidator {
    public function validate($password) {
        if (strlen($password) < 12) {
            return FALSE;
        }
        // ... 100+ lines of validation
    }
}
```

### AFTER: Using Security Contrib Modules
```yaml
# gov_compliance.module
function gov_compliance_install() {
  // Configure SecKit module
  $config = \Drupal::configFactory()->getEditable('seckit.settings');
  $config
    ->set('seckit_xss.csp.enable', TRUE)
    ->set('seckit_xss.csp.policy', "default-src 'self'")
    ->set('seckit_clickjacking.x_frame', 'SAMEORIGIN')
    ->save();
  
  // Configure Password Policy
  $policy = PasswordPolicy::create([
    'id' => 'gov_standard',
    'label' => 'Government Standard',
  ]);
  $policy->save();
}
```

```php
// GovComplianceManager.php (CLEAN)
namespace Drupal\gov_compliance\Service;

class GovComplianceManager {
    
    public function __construct(
        private ConfigFactoryInterface $configFactory,
    ) {}
    
    public function applyGovStandards(): void {
        // Only gov-specific requirements
        $this->configureFedRamp();
        $this->enableAuditLogging();
    }
    
    protected function configureFedRamp(): void {
        // FedRAMP-specific configuration
        $this->configFactory->getEditable('gov_compliance.fedramp')
            ->set('compliance_level', 'moderate')
            ->set('audit_retention_days', 90)
            ->save();
    }
}
```

## 6. Theme Cleanup - LLM Platform Manager

### BEFORE: Duplicate Theme Code
```scss
// custom-admin.scss (2000+ lines)
.admin-toolbar {
  background: $primary-color;
  // ... hundreds of custom styles
}

.dashboard-widget [// ... duplicate of existing patterns]
```

### AFTER: Extending Gin + UI Patterns
```yaml
# llm_platform_manager.info.yml
base theme: gin
libraries-extend:
  gin/gin:
    - llm_platform_manager/enhancements

libraries:
  enhancements:
    css:
      theme:
        css/llm-enhancements.css: {}
```

```scss
// llm-enhancements.scss (200 lines)
// Only custom additions
.llm-ai-widget {
  @extend .gin-card;
  
  &__icon {
    @include gin-icon;
    color: var(--gin-color-primary);
  }
}

// Use CSS custom properties
:root {
  --llm-gradient: linear-gradient(135deg, #667eea 0%, #764ba2 100%);
}
```

## Summary

These examples show how to:
1. **Reduce code** by 70%+ while keeping functionality
2. **Use contrib** modules for common patterns
3. **Keep custom** logic that makes your project unique
4. **Follow standards** for better maintainability
5. **Improve performance** through proper architecture

Each cleanup preserves your project's unique features while removing bloat and following Drupal best practices!