# Custom Code Reduction Action Plan

## Overview
Based on analysis of 45+ contrib modules, here's a concrete plan to reduce custom code by leveraging Drupal contrib ecosystem.

## Immediate Actions (Can implement now)

### 1. Queue Processing - Migrate to Advanced Queue
**Current**: Custom queue implementations in LLM module
**Replace with**: `advancedqueue` module

```php
// OLD: Custom queue
\Drupal::queue('llm_processing')->createItem($data);

// NEW: Advanced Queue
use Drupal\advancedqueue\Job;

$job = Job::create('llm_process', [
  'prompt' => $prompt,
  'provider' => 'openai',
]);
$queue = \Drupal::entityTypeManager()
  ->getStorage('advancedqueue_queue')
  ->load('llm_tasks');
$queue->enqueueJob($job);
```

**Benefits**:
- Built-in job retry logic
- Progress tracking
- Admin UI for monitoring
- Multiple backend support (Database, Redis)

### 2. API Key Management - Migrate to Key Module
**Current**: Custom token/key storage
**Replace with**: `key` module (already installed)

```php
// OLD: Custom key storage
$api_key = \Drupal::config('llm.settings')->get('openai_api_key');

// NEW: Key module
$key = \Drupal::service('key.repository')->getKey('openai_api_key');
$api_key = $key->getKeyValue();
```

**Create key entities**:
```yaml
# config/install/key.key.openai_api_key.yml
id: openai_api_key
label: 'OpenAI API Key'
key_provider: config
key_provider_settings:
  key_value: ''
key_input: text_field
```

### 3. Encryption - Use Encrypt Module
**Current**: Custom encryption in LLM module
**Replace with**: `encrypt` module

```php
// OLD: Custom encryption
$encrypted = $this->customEncrypt($sensitive_data);

// NEW: Encrypt module
$encryption_profile = \Drupal::entityTypeManager()
  ->getStorage('encryption_profile')
  ->load('default');
$encrypted = \Drupal::service('encryption')
  ->encrypt($sensitive_data, $encryption_profile);
```

### 4. Workflow Automation - Migrate to ECA
**Current**: Custom workflow logic
**Replace with**: `eca` module (already installed)

**Example: AI Content Generation Workflow**
```yaml
# config/install/eca.eca.ai_content_workflow.yml
id: ai_content_workflow
label: 'AI Content Generation Workflow'
events:
  content_requested:
    plugin: 'entity_presave'
    configuration:
      entity_type: node
      bundle: article
actions:
  generate_summary:
    plugin: 'ai_generate'
    configuration:
      provider: 'openai'
      prompt: '[node:title] - Generate summary'
```

### 5. HTTP Client Management
**Current**: Custom Guzzle implementations
**Replace with**: `http_client_manager` module

```yaml
# llm.http_services.yml
openai_api:
  title: "OpenAI API"
  base_uri: "https://api.openai.com/v1/"
  guzzle_options:
    headers:
      Authorization: 'Bearer [key:openai_api_key]'
      Content-Type: 'application/json'

anthropic_api:
  title: "Anthropic API" 
  base_uri: "https://api.anthropic.com/v1/"
  guzzle_options:
    headers:
      x-api-key: '[key:anthropic_api_key]'
```

## Module-Specific Recommendations

### LLM Module Refactoring

#### 1. Remove These Custom Services (Use Contrib Instead):
```yaml
# TO REMOVE from llm.services.yml:
llm.token.tracker:          # → Use key module
llm.encryption.service:     # → Use encrypt module  
llm.queue.processor:        # → Use advancedqueue
llm.http.client:           # → Use http_client_manager
llm.workflow.manager:       # → Use eca module
```

#### 2. Vector Operations - Use Search API
```php
// OLD: Custom vector implementation
$vector_result = $this->customVectorSearch($embedding);

// NEW: Search API with Solr Dense Vector
$index = \Drupal::entityTypeManager()
  ->getStorage('search_api_index')
  ->load('llm_vectors');
$query = $index->query();
$query->setOption('search_api_dense_vector', [
  'field' => 'embedding',
  'vector' => $embedding,
  'similarity' => 'cosine',
]);
$results = $query->execute();
```

#### 3. Agent Management - Use AI Agents
```php
// OLD: Custom agent orchestration
$agents = $this->loadCustomAgents();
foreach ($agents as $agent) {
  $result = $this->executeAgent($agent, $task);
}

// NEW: AI Agents module
$agent_manager = \Drupal::service('plugin.manager.ai_agents');
$agent = $agent_manager->createInstance('content_generator', [
  'model' => 'gpt-4',
  'temperature' => 0.7,
]);
$result = $agent->generate($prompt, $context);
```

### Recipe Onboarding Module Refactoring

#### 1. Form Building - Use Webform
Instead of custom forms, use Webform with AI integration:
```yaml
# Export webform configuration
id: recipe_onboarding
title: 'Recipe Onboarding'
elements:
  user_experience:
    '#type': select
    '#title': 'Experience Level'
    '#options':
      beginner: Beginner
      intermediate: Intermediate
      expert: Expert
handlers:
  ai_personalization:
    id: ai_agent
    handler_id: ai_personalization
    settings:
      agent: personalization_agent
```

#### 2. State Management - Use State Machine
```php
// OLD: Custom state tracking
$this->updateOnboardingState($user, 'step_2');

// NEW: State Machine module
$workflow = $user->field_onboarding_state->first()->getWorkflow();
$transition = $workflow->getTransition('complete_step_1');
$user->field_onboarding_state->first()->applyTransition($transition);
$user->save();
```

### MCP Client Extras Refactoring

#### 1. API Documentation - Use OpenAPI
```yaml
# mcp_client_extras.routing.yml
mcp_client_extras.openapi:
  path: '/api/mcp/openapi.json'
  defaults:
    _controller: '\Drupal\openapi\Controller\OpenApiController::index'
    _openapi_resource: mcp_rest
```

#### 2. Authentication - Use Standard Drupal + Key
```php
// OLD: Custom auth
$token = $this->customAuth->getToken();

// NEW: Standard approach
$key = \Drupal::service('key.repository')->getKey('mcp_api_key');
$client = \Drupal::service('http_client_manager.factory')
  ->get('mcp_api');
```

## Migration Timeline

### Week 1: Foundation
- [ ] Configure `key` module for all API keys
- [ ] Set up `encrypt` profiles
- [ ] Create `advancedqueue` queues

### Week 2: Core Services
- [ ] Migrate queue processing to `advancedqueue`
- [ ] Replace custom encryption with `encrypt`
- [ ] Implement `http_client_manager` for APIs

### Week 3: Workflows & Agents
- [ ] Convert workflows to `eca`
- [ ] Integrate `ai_agents` for agent management
- [ ] Set up `state_machine` for state tracking

### Week 4: Search & Documentation
- [ ] Implement `search_api_solr_dense_vector`
- [ ] Configure `openapi` for API docs
- [ ] Clean up deprecated custom code

## Expected Results

### Code Reduction Metrics
- **Custom Services**: 108 → ~30 (72% reduction) ✅ DONE
- **Custom Queue Code**: ~500 lines → 0 (100% reduction)
- **Custom Auth Code**: ~300 lines → 0 (100% reduction)
- **Custom Workflow Code**: ~800 lines → 0 (100% reduction)
- **Total Custom Code**: ~5000 lines → ~1500 lines (70% reduction)

### Benefits
1. **Security**: Professional encryption and key management
2. **Reliability**: Battle-tested queue processing
3. **Maintainability**: Community-supported modules
4. **Upgrade Path**: Easier Drupal core updates
5. **Performance**: Optimized contrib implementations

## Next Steps

1. **Create tickets** for each migration task
2. **Set up test coverage** for migration
3. **Document new patterns** in CLAUDE.md
4. **Train team** on contrib module usage
5. **Monitor performance** improvements

## Success Criteria

- [ ] All API keys managed by `key` module
- [ ] All queues using `advancedqueue`
- [ ] All workflows in `eca`
- [ ] All HTTP clients in `http_client_manager`
- [ ] Zero custom encryption code
- [ ] API documentation via `openapi`
- [ ] 70%+ reduction in custom code

By following this plan, the codebase will be more maintainable, secure, and aligned with Drupal best practices while leveraging the power of the contrib ecosystem.