# TDDAI Integration Examples

This document provides real-world examples of integrating TDDAI into different types of projects.

## Example 1: Express.js API Project

### Initial Setup

```bash
# Create new Express API project
mkdir my-express-api
cd my-express-api
npm init -y

# Install basic dependencies
npm install express
npm install --save-dev typescript @types/node @types/express

# Apply TDDAI API preset
tddai preset apply --name api
```

### Generated Project Structure

```
my-express-api/
├── __tests__/
│   ├── unit/
│   │   ├── controllers/
│   │   ├── services/
│   │   ├── middleware/
│   │   └── utils/
│   ├── integration/
│   │   ├── routes/
│   │   └── api/
│   ├── fixtures/
│   │   ├── mock-data.json
│   │   └── test-users.json
│   └── setup.ts
├── src/
│   ├── controllers/
│   ├── services/
│   ├── middleware/
│   └── app.ts
├── jest.config.js
├── jest.setup.js
├── CLAUDE.md
├── .cursorrules
├── .ai-standards.json
└── tsconfig.json
```

### TDD Development Workflow

1. **Start with failing test (RED)**:

```typescript
// __tests__/unit/controllers/UserController.test.ts
import [UserController] from '../../../src/controllers/UserController';

describe('UserController', () => {
  describe('createUser', () => {
    it('should create a new user with valid data', async () => {
      const userController = new UserController();
      const userData = {
        name: 'John Doe',
        email: 'john@example.com'
      };

      const result = await userController.createUser(userData);

      expect(result).toHaveProperty('id');
      expect(result.name).toBe('John Doe');
      expect(result.email).toBe('john@example.com');
    });
  });
});
```

2. **Run test to see it fail**:

```bash
npm run test:watch
# Test fails - UserController doesn't exist yet
```

3. **Write minimal code to pass (GREEN)**:

```typescript
// src/controllers/UserController.ts
interface User {
  id: string;
  name: string;
  email: string;
}

interface CreateUserData {
  name: string;
  email: string;
}

export class UserController {
  async createUser(userData: CreateUserData): Promise<User> {
    return {
      id: Math.random().toString(36),
      name: userData.name,
      email: userData.email
    };
  }
}
```

4. **Test passes - Refactor**:

```typescript
// src/controllers/UserController.ts (refactored)
import [v4 as uuidv4] from 'uuid';
import [UserService] from '../services/UserService';

export class UserController {
  private userService: UserService;

  constructor(userService = new UserService()) {
    this.userService = userService;
  }

  async createUser(userData: CreateUserData): Promise<User> {
    const user = {
      id: uuidv4(),
      name: userData.name,
      email: userData.email,
      createdAt: new Date()
    };

    return await this.userService.save(user);
  }
}
```

### Integration Testing

```typescript
// __tests__/integration/routes/users.test.ts
import request from 'supertest';
import [app] from '../../../src/app';

describe('User Routes', () => {
  describe('POST /api/users', () => {
    it('should create user and return 201', async () => {
      const userData = {
        name: 'Jane Doe',
        email: 'jane@example.com'
      };

      const response = await request(app)
        .post('/api/users')
        .send(userData)
        .expect(201);

      expect(response.body).toHaveProperty('id');
      expect(response.body.name).toBe('Jane Doe');
    });

    it('should return 400 for invalid email', async () => {
      const userData = {
        name: 'John Doe',
        email: 'invalid-email'
      };

      await request(app)
        .post('/api/users')
        .send(userData)
        .expect(400);
    });
  });
});
```

## Example 2: React Component Library

### Setup

```bash
# Create React component library
mkdir my-component-library
cd my-component-library
npm init -y

# Install React dependencies
npm install react react-dom
npm install --save-dev @types/react @types/react-dom

# Apply TDDAI React preset
tddai preset apply --name react
```

### Component Development with TDD

1. **Write component test first**:

```typescript
// __tests__/components/Button.test.tsx
import { render, screen, fireEvent } from '@testing-library/react';
import [Button] from '../../src/components/Button';

describe('Button', () => {
  it('should render with correct text', () => {
    render(<Button>Click me</Button>);
    
    expect(screen.getByRole('button')).toHaveTextContent('Click me');
  });

  it('should call onClick when clicked', () => {
    const handleClick = jest.fn();
    render(<Button onClick={handleClick}>Click me</Button>);
    
    fireEvent.click(screen.getByRole('button'));
    
    expect(handleClick).toHaveBeenCalledTimes(1);
  });

  it('should be disabled when disabled prop is true', () => {
    render(<Button disabled>Click me</Button>);
    
    expect(screen.getByRole('button')).toBeDisabled();
  });
});
```

2. **Create minimal implementation**:

```typescript
// src/components/Button.tsx
import React from 'react';

interface ButtonProps {
  children: React.ReactNode;
  onClick?: () => void;
  disabled?: boolean;
}

export const Button: React.FC<ButtonProps> = ({
  children,
  onClick,
  disabled = false
}) => {
  return (
    <button onClick={onClick} disabled=[disabled]>
      [children]
    </button>
  );
};
```

3. **All tests pass - Refactor for better styling**:

```typescript
// src/components/Button.tsx (enhanced)
import React from 'react';
import styles from './Button.module.css';

interface ButtonProps {
  children: React.ReactNode;
  onClick?: () => void;
  disabled?: boolean;
  variant?: 'primary' | 'secondary' | 'danger';
  size?: 'small' | 'medium' | 'large';
}

export const Button: React.FC<ButtonProps> = ({
  children,
  onClick,
  disabled = false,
  variant = 'primary',
  size = 'medium'
}) => {
  const className = [
    styles.button,
    styles[variant],
    styles[size],
    disabled && styles.disabled
  ].filter(Boolean).join(' ');

  return (
    <button 
      className={className}
      onClick=[onClick] 
      disabled=[disabled]
    >
      [children]
    </button>
  );
};
```

## Example 3: CLI Tool Development

### Setup

```bash
# Create CLI tool project
mkdir my-cli-tool
cd my-cli-tool
npm init -y

# Apply TDDAI CLI preset
tddai preset apply --name cli
```

### Command Development with TDD

1. **Test command behavior first**:

```typescript
// __tests__/commands/hello.test.ts
import [HelloCommand] from '../../src/commands/HelloCommand';

describe('HelloCommand', () => {
  it('should return greeting with default name', async () => {
    const command = new HelloCommand();
    const result = await command.execute({});
    
    expect(result).toBe('Hello, World!');
  });

  it('should return greeting with custom name', async () => {
    const command = new HelloCommand();
    const result = await command.execute({ name: 'Alice' });
    
    expect(result).toBe('Hello, Alice!');
  });

  it('should handle empty string name', async () => {
    const command = new HelloCommand();
    const result = await command.execute({ name: '' });
    
    expect(result).toBe('Hello, World!');
  });
});
```

2. **Implement command**:

```typescript
// src/commands/HelloCommand.ts
interface HelloOptions {
  name?: string;
}

export class HelloCommand {
  async execute(options: HelloOptions): Promise<string> {
    const name = options.name?.trim() || 'World';
    return `Hello, Name!`;
  }
}
```

3. **Integration test for CLI**:

```typescript
// __tests__/integration/cli.test.ts
import [execSync] from 'child_process';
import path from 'path';

const CLI_PATH = path.join(__dirname, '../../bin/my-cli.js');

describe('CLI Integration', () => {
  it('should run hello command with default name', () => {
    const output = execSync(`node [CLI_PATH] hello`, { encoding: 'utf-8' });
    expect(output.trim()).toBe('Hello, World!');
  });

  it('should run hello command with custom name', () => {
    const output = execSync(`node [CLI_PATH] hello --name Alice`, { encoding: 'utf-8' });
    expect(output.trim()).toBe('Hello, Alice!');
  });
});
```

## Example 4: Drupal Module Development

### Setup

```bash
# Create Drupal module
mkdir my_custom_module
cd my_custom_module

# Apply TDDAI Drupal preset
tddai preset apply --name drupal
```

### Drupal Service Development with TDD

1. **Write service test first**:

```php
<?php

// tests/src/Unit/Services/UserDataServiceTest.php
namespace Drupal\Tests\my_custom_module\Unit\Services;

use Drupal\Tests\UnitTestCase;
use Drupal\my_custom_module\Services\UserDataService;
use Drupal\Core\Entity\EntityTypeManagerInterface;

class UserDataServiceTest extends UnitTestCase {

  public function testGetUserDataReturnsCorrectFormat() {
    $entityTypeManager = $this->createMock(EntityTypeManagerInterface::class);
    $service = new UserDataService($entityTypeManager);
    
    $userData = $service->getUserData(1);
    
    $this->assertIsArray($userData);
    $this->assertArrayHasKey('id', $userData);
    $this->assertArrayHasKey('name', $userData);
    $this->assertArrayHasKey('email', $userData);
  }
}
```

2. **Implement service**:

```php
<?php

// src/Services/UserDataService.php
namespace Drupal\my_custom_module\Services;

use Drupal\Core\Entity\EntityTypeManagerInterface;

class UserDataService {

  protected $entityTypeManager;

  public function __construct(EntityTypeManagerInterface $entity_type_manager) {
    $this->entityTypeManager = $entity_type_manager;
  }

  public function getUserData(int $userId): array {
    $userStorage = $this->entityTypeManager->getStorage('user');
    $user = $userStorage->load($userId);

    if (!$user) {
      return [];
    }

    return [
      'id' => $user->id(),
      'name' => $user->getDisplayName(),
      'email' => $user->getEmail(),
    ];
  }
}
```

## Example 5: NPM Package Migration

### Migrating Existing Package

```bash
# Navigate to existing project
cd my-existing-project

# Analyze current project
tddai migrate analyze

# Create migration plan
tddai migrate plan --output migration-plan.json

# Execute migration (dry run first)
tddai migrate execute --dry-run

# Execute actual migration
tddai migrate execute

# Validate setup
tddai validate --strict
```

### Before Migration

```json
{
  "name": "my-existing-project",
  "scripts": {
    "test": "mocha test/**/*.js",
    "lint": "jshint src/"
  },
  "devDependencies": {
    "mocha": "^8.0.0",
    "chai": "^4.0.0",
    "jshint": "^2.0.0"
  }
}
```

### After Migration

```json
{
  "name": "my-existing-project",
  "scripts": {
    "test": "jest",
    "test:watch": "jest --watch",
    "test:coverage": "jest --coverage",
    "test:unit": "jest --testPathPattern=__tests__/unit",
    "test:integration": "jest --testPathPattern=__tests__/integration",
    "test:ci": "jest --ci --coverage --watchAll=false",
    "lint": "eslint src/**/*.{ts,js}",
    "lint:fix": "eslint src/**/*.{ts,js} --fix",
    "typecheck": "tsc --noEmit",
    "ci": "npm run lint && npm run typecheck && npm run test:ci",
    "tdd:validate": "tddai validate --strict",
    "tdd:monitor": "tddai monitor --interval 30",
    "tdd:workflow": "tddai tdd workflow --step red"
  },
  "devDependencies": {
    "jest": "^29.7.0",
    "@types/jest": "^29.5.8",
    "eslint": "^8.56.0",
    "typescript": "^5.3.3",
    "@bluefly/tddai": "^2.0.0"
  }
}
```

## Example 6: Monorepo Integration

### Setup for Multiple Packages

```bash
# Root of monorepo
cd my-monorepo

# Apply TDDAI to each package
cd packages/api
tddai preset apply --name api

cd ../web-app
tddai preset apply --name react

cd ../shared-utils
tddai preset apply --name library

# Setup workspace-level configuration
cd ../..
tddai setup --workspace
```

### Workspace Package.json

```json
{
  "name": "my-monorepo",
  "private": true,
  "workspaces": [
    "packages/*"
  ],
  "scripts": {
    "test": "npm run test --workspaces",
    "test:coverage": "npm run test:coverage --workspaces",
    "lint": "npm run lint --workspaces",
    "tdd:validate:all": "npm run tdd:validate --workspaces",
    "ci": "npm run lint && npm run test:coverage && npm run tdd:validate:all"
  },
  "devDependencies": {
    "@bluefly/tddai": "^2.0.0"
  }
}
```

## CI/CD Integration Examples

### GitHub Actions

```yaml
# .github/workflows/tddai-validation.yml
name: TDDAI TDD Validation

on:
  push:
    branches: [main, develop]
  pull_request:
    branches: [main]

jobs:
  tdd-validation:
    runs-on: ubuntu-latest
    
    steps:
      - uses: actions/checkout@v4
      
      - uses: actions/setup-node@v4
        with:
          node-version: '20'
          cache: 'npm'
      
      - name: Install dependencies
        run: npm ci
      
      - name: Install TDDAI
        run: npm install -g @bluefly/tddai
      
      - name: Validate TDD compliance
        run: tddai validate --strict --json
      
      - name: Run tests with coverage
        run: npm run test:ci
      
      - name: Check AI integrations
        run: tddai ai validate
```

### GitLab CI

```yaml
# .gitlab-ci.yml
stages:
  - validate
  - test
  - deploy

variables:
  NODE_VERSION: "20"

validate:tdd:
  stage: validate
  image: node:[NODE_VERSION]
  before_script:
    - npm install -g @bluefly/tddai
    - npm ci
  script:
    - tddai validate --strict
    - tddai ai validate
  rules:
    - if: $CI_PIPELINE_SOURCE == "merge_request_event"

test:coverage:
  stage: test
  image: node:[NODE_VERSION]
  before_script:
    - npm ci
  script:
    - npm run test:ci
  coverage: '/Lines:\s*(\d+\.?\d*)%/'
  artifacts:
    reports:
      coverage_report:
        coverage_format: cobertura
        path: coverage/cobertura-coverage.xml
```

## Best Practices from Examples

### 1. Always Start with Tests

```typescript
// ❌ Don't do this
function calculateTotal(items) {
  // Implementation first
  return items.reduce((sum, item) => sum + item.price, 0);
}

// ✅ Do this
describe('calculateTotal', () => {
  it('should calculate total price of items', () => {
    const items = [
      { price: 10.99 },
      { price: 5.50 },
      { price: 2.25 }
    ];
    
    expect(calculateTotal(items)).toBe(18.74);
  });
});

function calculateTotal(items) {
  return items.reduce((sum, item) => sum + item.price, 0);
}
```

### 2. Use AI Integrations Effectively

```markdown
<!-- .cursorrules -->
# Project uses TDDAI - Always write tests first!

When implementing new features:
1. Write failing test (RED)
2. Write minimal code to pass (GREEN)  
3. Refactor while keeping tests green (REFACTOR)

Coverage requirement: 95%
```

### 3. Leverage Presets for Quick Setup

```bash
# Instead of manual configuration
npm install jest --save-dev
# ... manual jest config
# ... manual eslint config
# ... manual typescript config

# Use TDDAI preset
tddai preset apply --name enterprise
# Everything configured automatically!
```

### 4. Monitor Compliance Continuously

```bash
# Add to package.json scripts
"dev": "tddai monitor --interval 30 & npm run dev:server"
"pre-commit": "tddai validate --strict"
```

## Troubleshooting Common Issues

### Issue: Tests Not Running

```bash
# Check TDDAI setup
tddai validate --debug

# Verify jest configuration
npm run test -- --debug

# Check file patterns
tddai migrate status
```

### Issue: Low Coverage

```bash
# Run coverage report
npm run test:coverage

# Check specific files
npm run test:coverage -- --collectCoverageFrom="src/specific-file.ts"

# Use TDDAI monitoring
tddai monitor --interval 10
```

### Issue: AI Integration Not Working

```bash
# Validate AI setup
tddai ai validate

# Regenerate AI configs
tddai ai setup-all --force

# Check file presence
ls -la .cursorrules CLAUDE.md .ai-standards.json
```

---

These examples demonstrate how TDDAI transforms development workflows across different project types, making TDD adoption seamless and AI-powered.