# TDDAI Integration Guide

## Adding TDDAI to Existing Projects

This comprehensive guide shows you how to integrate TDDAI into your existing NPM projects for AI-powered TDD development.

## 🚀 Quick Integration

### 1. Install TDDAI

```bash
# Add to existing project
npm install --save-dev @bluefly/tddai

# Or install globally
npm install -g @bluefly/tddai
```

### 2. Initialize TDDAI in Your Project

```bash
# Navigate to your project
cd your-existing-project

# Initialize TDDAI
tddai setup --type node --framework jest

# Setup AI integrations
tddai ai setup-all
```

### 3. Verify Integration

```bash
# Validate setup
tddai validate --strict

# Start TDD workflow
tddai tdd workflow --step red
```

## 📋 Integration Checklist

Use this checklist when adding TDDAI to any project:

- [ ] Install TDDAI as dev dependency
- [ ] Run `tddai setup` for your project type
- [ ] Configure AI integrations with `tddai ai setup-all`
- [ ] Update package.json scripts
- [ ] Setup CI/CD integration
- [ ] Validate with `tddai validate --strict`
- [ ] Train team on TDD workflow

## 🔧 Project-Specific Integration

### Node.js/TypeScript Projects

```bash
# For TypeScript Node.js projects
tddai setup --type typescript --framework jest --coverage 95

# For JavaScript Node.js projects  
tddai setup --type node --framework jest --coverage 90
```

**Files Created:**
- `jest.config.js` - AI-enhanced Jest configuration
- `jest.setup.js` - Test utilities and mocks
- `__tests__/` - Test directory structure
- `CLAUDE.md` - AI assistant configuration
- `.cursorrules` - Cursor IDE integration
- `.ai-standards.json` - Machine-readable standards

### React Projects

```bash
# For React applications
tddai setup --type react --framework jest --coverage 95
```

**Additional Features:**
- React Testing Library setup
- Component testing templates
- E2E test structure
- Performance testing utilities

### Drupal Module Development

```bash
# For Drupal modules
tddai setup --type drupal --framework phpunit --coverage 90
```

**Drupal-Specific Features:**
- PHPUnit configuration for Drupal
- Kernel test templates
- Unit test structure
- Drupal coding standards integration

### Monorepo Integration

```bash
# For each package in monorepo
cd packages/package-name
tddai setup --type node --framework jest

# Or use workspace mode (coming soon)
tddai setup --workspace --type node --framework jest
```

## 📁 Template Configurations

### Basic Package.json Scripts

TDDAI automatically adds these scripts to your package.json:

```json
{
  "scripts": {
    "test": "jest",
    "test:watch": "jest --watch", 
    "test:coverage": "jest --coverage",
    "test:unit": "jest --testPathPattern=__tests__/unit",
    "test:integration": "jest --testPathPattern=__tests__/integration",
    "test:ci": "jest --ci --coverage --watchAll=false",
    "lint": "eslint src/**/*.{ts,js}",
    "lint:fix": "eslint src/**/*.{ts,js} --fix",
    "typecheck": "tsc --noEmit",
    "ci": "npm run lint && npm run typecheck && npm run test:ci",
    "tdd:validate": "tddai validate --strict",
    "tdd:monitor": "tddai monitor --interval 30",
    "tdd:workflow": "tddai tdd workflow --step red"
  }
}
```

### TypeScript Configuration

TDDAI creates optimized tsconfig.json:

```json
{
  "compilerOptions": {
    "target": "ES2022",
    "module": "commonjs", 
    "moduleResolution": "node",
    "strict": true,
    "skipLibCheck": true,
    "forceConsistentCasingInFileNames": true,
    "declaration": true,
    "declarationMap": true,
    "sourceMap": true,
    "outDir": "./dist",
    "rootDir": "./src",
    "baseUrl": "./src",
    "paths": {
      "@/*": ["*"],
      "@test/*": ["../__tests__/*"]
    }
  },
  "include": ["src/**/*"],
  "exclude": ["node_modules", "dist", "__tests__", "**/*.test.ts"]
}
```

### ESLint Configuration

```json
{
  "extends": [
    "@typescript-eslint/recommended",
    "prettier"
  ],
  "parser": "@typescript-eslint/parser",
  "plugins": ["@typescript-eslint"],
  "rules": {
    "@typescript-eslint/no-unused-vars": "error",
    "@typescript-eslint/no-explicit-any": "warn",
    "@typescript-eslint/explicit-function-return-type": "warn"
  }
}
```

## 🤖 AI Integration Templates

### Cursor IDE Configuration (`.cursorrules`)

```markdown
# TDDAI TDD Standards for Cursor IDE
# Generated by @bluefly/tddai v2.0+

## Project Configuration
- Project Type: TypeScript/Node.js with TDD
- Framework: Jest with 95% coverage requirement
- AI Model: Claude 3.5 Sonnet (optimized for coding)
- Standards: TDDAI AI-Enhanced TDD Framework

## TDD Workflow Rules
1. **RED Phase**: Always write failing tests first
2. **GREEN Phase**: Write minimal code to pass test
3. **REFACTOR Phase**: Clean up code while keeping tests green

## AI Assistant Instructions
- Always suggest writing tests BEFORE implementation
- Ensure all new code has corresponding tests
- Maintain 95%+ test coverage at all times
- Follow TypeScript strict mode and proper typing
```

### Claude Code Configuration (`CLAUDE.md`)

```markdown
# CLAUDE.md - TDDAI AI-Enhanced TDD Framework

## Project Overview
This project follows Test-Driven Development (TDD) methodology with AI integration, using the TDDAI framework for enforced coding standards.

## TDD Workflow (MANDATORY)
1. **Write failing test first (RED)**
2. **Write minimal code to pass test (GREEN)**
3. **Refactor while keeping tests green (REFACTOR)**

## AI Assistant Instructions for Claude Code

### Primary Directive
**ALWAYS follow TDD methodology**: Write tests BEFORE implementation code.

### Test Writing Guidelines
- Unit tests in `__tests__/unit/`
- Integration tests in `__tests__/integration/`
- Maintain 95%+ test coverage
- Include edge cases and error scenarios
```

## 🔄 Migration from Other Frameworks

### From Jest (existing)

```bash
# TDDAI will enhance your existing Jest setup
tddai setup --type node --framework jest

# Validates and enhances existing configuration
tddai validate --strict
```

### From Mocha/Chai

```bash
# Migrate to Jest with TDDAI
npm uninstall mocha chai sinon
npm install --save-dev jest @types/jest

# Setup TDDAI
tddai setup --type node --framework jest

# Convert existing tests (manual process)
# TDDAI provides migration helpers (coming soon)
```

### From No Testing Framework

```bash
# Start fresh with TDDAI
tddai setup --type node --framework jest

# Follow guided setup for first tests
tddai tdd workflow --step red
```

## 📊 CI/CD Integration Templates

### GitHub Actions

```yaml
name: TDDAI TDD Validation

on:
  push:
    branches: [main, develop]
  pull_request:
    branches: [main]

jobs:
  tdd-validation:
    runs-on: ubuntu-latest
    steps:
      - uses: actions/checkout@v4
      - uses: actions/setup-node@v4
        with:
          node-version: '20'
          cache: 'npm'
      
      - name: Install dependencies
        run: npm ci
      
      - name: Install TDDAI
        run: npm install -g @bluefly/tddai
      
      - name: Validate TDD compliance
        run: tddai validate --strict --json
      
      - name: Run tests with coverage
        run: npm run test:ci
      
      - name: Upload coverage
        uses: codecov/codecov-action@v3
```

### GitLab CI

```yaml
stages:
  - validate
  - test
  - build

variables:
  NODE_VERSION: "20"

validate:tdd:
  stage: validate
  image: node:[NODE_VERSION]
  before_script:
    - npm install -g @bluefly/tddai
    - npm ci
  script:
    - tddai validate --strict
  rules:
    - if: $CI_PIPELINE_SOURCE == "merge_request_event"

test:coverage:
  stage: test
  image: node:[NODE_VERSION]
  before_script:
    - npm ci
  script:
    - npm run test:ci
  coverage: '/Lines:\s*(\d+\.?\d*)%/'
  artifacts:
    reports:
      coverage_report:
        coverage_format: cobertura
        path: coverage/cobertura-coverage.xml
```

## 🔌 Programmatic Integration

### Basic API Usage

```typescript
import [TDDAI] from '@bluefly/tddai';

const tddai = new TDDAI({
  projectType: 'node',
  testFramework: 'jest',
  coverageThreshold: 95,
  aiIntegration: true,
  autoSetup: true
});

// Setup TDD framework
await tddai.setupProject('./my-project');

// Validate project compliance
const result = await tddai.validateProject('./my-project');
console.log(`TDD Score: [result.score]/100`);
```

### Advanced Configuration

```typescript
import { TDDAI, CursorIntegration, ClaudeCodeIntegration } from '@bluefly/tddai';

const config = {
  projectType: 'typescript' as const,
  testFramework: 'jest' as const,
  coverageThreshold: 95,
  aiIntegration: true,
  autoSetup: true,
  enableMonitoring: true,
  enableReporting: true
};

const tddai = new TDDAI(config);

// Setup with custom AI integrations
const cursorIntegration = new CursorIntegration('./project', config);
await cursorIntegration.setupCursorIntegration();

const claudeIntegration = new ClaudeCodeIntegration('./project', config);
await claudeIntegration.setupClaudeCodeIntegration();
```

### Custom Presets

```typescript
import [TDDAI] from '@bluefly/tddai';

// Enterprise preset
const enterpriseConfig = {
  projectType: 'typescript',
  testFramework: 'jest',
  coverageThreshold: 90,
  aiIntegration: true,
  autoSetup: true,
  enableMonitoring: true,
  enableReporting: true,
  strictMode: true
};

// Startup preset
const startupConfig = {
  projectType: 'node',
  testFramework: 'jest', 
  coverageThreshold: 80,
  aiIntegration: true,
  autoSetup: true,
  fastSetup: true
};
```

## 📱 Integration Examples

### Express.js API

```bash
cd my-express-api
tddai setup --type node --framework jest
```

**Generated structure:**
```
my-express-api/
├── __tests__/
│   ├── unit/
│   │   ├── controllers/
│   │   ├── services/
│   │   └── utils/
│   ├── integration/
│   │   ├── routes/
│   │   └── middleware/
│   └── fixtures/
├── src/
├── jest.config.js
├── CLAUDE.md
└── .cursorrules
```

### React Component Library

```bash
cd my-component-library
tddai setup --type react --framework jest
```

**Generated structure:**
```
my-component-library/
├── __tests__/
│   ├── unit/
│   │   └── components/
│   ├── integration/
│   └── e2e/
├── src/
│   └── components/
├── jest.config.js
├── jest.setup.js
└── .storybook/ (if detected)
```

### CLI Tool

```bash
cd my-cli-tool
tddai setup --type node --framework jest
```

**CLI-specific features:**
- Command testing utilities
- Integration with commander.js
- E2E command testing
- Help text validation

## 🛠️ Troubleshooting Integration

### Common Issues

#### 1. Existing Test Files Conflict

```bash
# Backup existing tests
mkdir __tests_backup
mv test/ __tests_backup/

# Setup TDDAI
tddai setup --type node --framework jest

# Migrate tests manually to new structure
```

#### 2. TypeScript Configuration Conflicts

```bash
# TDDAI creates tsconfig.tddai.json if conflicts detected
# Merge manually or use TDDAI's configuration

# Override with TDDAI config
tddai setup --type typescript --framework jest --force
```

#### 3. Package.json Script Conflicts

```bash
# TDDAI will prefix conflicting scripts with 'tddai:'
# Example: 'test' becomes 'tddai:test' if conflict

# Use TDDAI scripts
npm run tddai:test
npm run tddai:validate
```

### Debug Mode

```bash
# Enable debug output
tddai setup --debug
tddai validate --debug
tddai monitor --debug
```

## 🎯 Best Practices for Integration

### 1. Gradual Migration

```bash
# Step 1: Install and basic setup
tddai setup --type node --framework jest

# Step 2: Write tests for new features only
tddai tdd workflow --step red

# Step 3: Gradually add tests for existing code
tddai validate --strict

# Step 4: Full AI integration
tddai ai setup-all
```

### 2. Team Onboarding

```bash
# Create onboarding script
echo '#!/bin/bash
npm install
tddai validate --strict
echo "Project ready for TDD development!"
echo "Start with: tddai tdd workflow --step red"
' > setup-dev.sh

chmod +x setup-dev.sh
```

### 3. Continuous Monitoring

```bash
# Add to package.json
"scripts": {
  "dev": "tddai monitor --interval 30 & npm run dev:server",
  "pre-commit": "tddai validate --strict"
}
```

## 📚 Next Steps

After integrating TDDAI:

1. **Train your team** on TDD methodology
2. **Setup CI/CD** with TDDAI validation
3. **Configure AI assistants** (Cursor, Claude Code)
4. **Monitor compliance** in real-time
5. **Iterate and improve** based on validation results

## 🔗 Related Documentation

- [TDDAI CLI Reference](./CLI_REFERENCE.md)
- [AI Integration Guide](./AI_INTEGRATION.md)
- [TDD Best Practices](./TDD_BEST_PRACTICES.md)
- [Troubleshooting](./TROUBLESHOOTING.md)

---

**Ready to revolutionize your development workflow?**

```bash
npm install -g @bluefly/tddai
cd your-project
tddai setup --type node --framework jest
tddai ai setup-all
```

Transform any project into an AI-powered TDD powerhouse! 🚀