# Project Integration Guide

This document provides instructions for integrating ReelCode with specific projects in the BlueFlux ecosystem.

## Overview

ReelCode can be integrated with various project types to enforce consistent coding standards across the organization. The integration process includes:

1. Setting up project-specific standards
2. Generating AI assistant configurations
3. Creating CI/CD integration files
4. Adding project-specific validation scripts

## Supported Project Types

ReelCode supports integration with the following project types:

- **BFAPI**: API Gateway services
- **llm-mcp**: Model Control Plane services
- **BFLLM**: Core LLM services
- **BFRFP**: Content generation services
- **llm-ui**: UI component libraries
- **NATIVE_AIJS**: JavaScript AI framework
- **DRUPAL**: Drupal CMS modules

## Quick Start

The fastest way to integrate ReelCode with your project is to use the `integrate` command:

```bash
tddai integrate project bfapi /path/to/bfapi
```

This will:
1. Create a `.tddai` directory in your project
2. Add project-specific standards files
3. Generate AI assistant configurations (e.g., CLAUDE.md)
4. Add CI/CD configuration templates
5. Create project-specific validation scripts

## Integration Options

```bash
tddai integrate project <project-type> [project-path] [options]
```

Options:
- `-f, --force`: Force overwrite of existing files
- `-s, --standards <path>`: Path to custom standards file
- `-o, --output-dir <path>`: Output directory for generated files (default: `.tddai`)
- `--ai <assistants>`: AI assistants to generate configs for (comma-separated, default: `claude`)
- `--ci <systems>`: CI/CD systems to generate configs for (comma-separated, default: `gitlab,github`)
- `-v, --verbose`: Show verbose output

## Project-Specific Integrations

### BFAPI Integration

```bash
tddai integrate project bfapi /path/to/bfapi
```

The BFAPI integration includes:
- TypeScript standards optimized for API development
- Controller and endpoint naming conventions
- OpenAPI documentation requirements
- API security rules
- GitLab CI configuration for API validation

### llm-mcp Integration

```bash
tddai integrate project llm-mcp /path/to/llm-mcp
```

The llm-mcp integration includes:
- TypeScript standards for model control
- Tensor operation conventions
- Model versioning requirements
- Tool integration standards
- Performance validation scripts

### BFLLM Integration

```bash
tddai integrate project bfllm /path/to/bfllm
```

The BFLLM integration includes:
- LLM-specific coding conventions
- Prompt management standards
- Context handling best practices
- Memory management guidelines
- Embedding standards

### llm-ui Integration

```bash
tddai integrate project llm-ui /path/to/llm-ui
```

The llm-ui integration includes:
- React/TypeScript conventions
- Component structure standards
- Accessibility requirements
- Visual testing integration
- Storybook documentation standards

## Customizing Integration

### Using Custom Standards

You can provide your own standards file:

```bash
tddai integrate project bfapi --standards custom-standards.yaml
```

### Generating Specific AI Configurations

Generate configurations for specific AI assistants:

```bash
tddai integrate project bfapi --ai "claude,copilot,cursor"
```

### Specific CI/CD Integration

Focus on specific CI/CD systems:

```bash
tddai integrate project bfapi --ci "github"
```

## Post-Integration Steps

After integration, you should:

1. **Review Generated Files**: Check the standards and configurations to ensure they meet your project's needs
2. **Commit Changes**: Add the generated files to version control
3. **Configure CI/CD**: Update your existing CI/CD configuration to use the generated files
4. **Update Documentation**: Add information about standards validation to your project README

## Integration Examples

### TypeScript API Project

```bash
# Basic integration
tddai integrate project bfapi ./my-api

# Custom integration with specific AI assistants
tddai integrate project bfapi ./my-api --ai "claude,copilot" --ci "github"

# Integration with custom standards
tddai integrate project bfapi ./my-api --standards ./my-api-standards.yaml
```

### UI Component Library

```bash
# Basic integration
tddai integrate project llm-ui ./ui-components

# Focus on accessibility standards
tddai integrate project llm-ui ./ui-components --standards ./a11y-standards.yaml
```

## Troubleshooting

If you encounter issues with project integration:

1. **Integration Failed**: Check that the project path exists and is a directory
2. **Standards File Not Found**: Verify the standards file path is correct
3. **Integration Not Working**: Use the `--verbose` flag to get detailed output
4. **CI/CD Integration Issues**: Check your CI/CD configuration for syntax errors

For more detailed information, see the [ReelCode documentation](https://gitlab.bluefly.io/llm/tddai/-/blob/main/README.md).