# TDDAI Worker System Documentation

## Overview

The TDDAI Worker System provides a unified framework for executing parallel tasks using Node.js worker threads. It consolidates functionality from three separate worker implementations into a single, cohesive system.

## Architecture

### Core Components

1. **WorkerOrchestrator** - Main orchestration engine
   - Priority-based task scheduling
   - Dependency resolution
   - Event-driven monitoring
   - Automatic retry with exponential backoff

2. **WorkerPool** - Worker thread lifecycle management
   - Dynamic worker allocation
   - Resource pooling
   - Inline worker execution

3. **TaskQueue** - Priority-based task queuing
   - Supports critical, high, medium, and low priorities
   - FIFO within each priority level

4. **WorkerMonitor** - Performance monitoring and reporting
   - Real-time metrics
   - Task timeline visualization
   - Export to JSON/CSV

## Usage

### CLI Commands

```bash
# List all available tasks
tddai worker list

# Run all tasks
tddai worker run

# Run specific task type
tddai worker run --task drupal    # Drupal integration tasks
tddai worker run --task api       # API standardization tasks

# Run with custom worker count
tddai worker run --workers 5

# Run only high-priority tasks
tddai worker run --priority high

# Generate execution report
tddai worker report --format html --output report.html
```

### Programmatic Usage

```typescript
import { WorkerOrchestrator, createDrupalIntegrationTasks } from '@bluefly/tddai';

// Create orchestrator
const orchestrator = new WorkerOrchestrator({
  maxWorkers: 10,
  taskTimeout: 30000,
  retryAttempts: 3,
  monitoring: true
});

// Get tasks
const tasks = createDrupalIntegrationTasks();

// Execute with event monitoring
orchestrator.on('task:complete', ({ task, result }) => {
  console.log(`Completed: [task.name]`);
});

const results = await orchestrator.executeTasks(tasks);

// Get monitoring report
const report = orchestrator.getMonitoringData();
console.log(`Completed [report.completedTasks]/[report.totalTasks] tasks`);

// Cleanup
await orchestrator.shutdown();
```

## Task Types

### 1. Analysis Tasks
- Module structure analysis
- API specification review
- Code coverage analysis
- Dependency mapping

### 2. Generation Tasks
- Test suite generation
- Documentation creation
- SDK generation
- Code scaffolding

### 3. Integration Tasks
- Service configuration
- Module integration
- API consolidation
- Framework setup

### 4. Migration Tasks
- Code migration
- Configuration updates
- Recipe generation
- Automator conversion

### 5. Validation Tasks
- OpenAPI validation
- Test validation
- Configuration checks
- Security audits

### 6. Documentation Tasks
- API documentation
- Module guides
- Integration docs
- Report generation

## Task Definition

```typescript
interface WorkerTask {
  id: string;                      // Unique identifier
  name: string;                    // Display name
  description?: string;            // Optional description
  type: TaskType;                  // analysis|generation|migration|etc
  priority: TaskPriority;          // critical|high|medium|low
  dependencies?: string[];         // Task IDs that must complete first
  timeout?: number;               // Custom timeout in ms
  retryable?: boolean;            // Whether to retry on failure
  data?: any;                     // Task-specific data
}
```

## Available Task Sets

### Drupal Integration Tasks (10 tasks)
- Module analysis and integration
- Test coverage analysis
- Service configuration
- Documentation generation

### API Standardization Tasks (7 tasks)
- 5-Worker API specification
- OpenAPI validation
- Documentation generation

### Apple Provider Tasks (4 tasks)
- Provider development
- Test suite generation
- Privacy UI development
- Model registry

### Migration Tasks (3 tasks)
- Recipe building
- Automator migration
- Authentication decoration

## Performance

- Parallel execution with configurable worker count
- Average task duration: ~300ms
- Throughput: 10-30 tasks/second (depending on task complexity)
- Memory usage: ~30-50MB per worker

## Error Handling

- Automatic retry with exponential backoff
- Configurable retry attempts (default: 3)
- Detailed error reporting
- Graceful degradation on worker failure

## Monitoring & Reporting

### Real-time Monitoring
```bash
# Enable monitoring during execution
tddai worker run --monitoring
```

### Report Generation
```bash
# Generate JSON report
tddai worker report --format json

# Generate HTML report
tddai worker report --format html --output report.html

# Generate Markdown report
tddai worker report --format markdown
```

### Report Contents
- Execution summary
- Task-by-task results
- Performance metrics
- Error details
- Task distribution by type/priority

## Best Practices

1. **Task Granularity**: Keep tasks focused and atomic
2. **Dependencies**: Minimize inter-task dependencies
3. **Timeouts**: Set appropriate timeouts for long-running tasks
4. **Error Handling**: Make tasks idempotent when possible
5. **Monitoring**: Use events for real-time progress tracking

## Extending the System

### Adding New Task Types

1. Define task in appropriate file:
   - `src/tasks/DrupalIntegrationTasks.ts`
   - `src/tasks/ApiStandardizationTasks.ts`

2. Implement task logic in `src/workers/task-executor.ts`

3. Export from task creator functions

### Custom Task Execution

```typescript
const customTasks: WorkerTask[] = [
  {
    id: 'custom-analysis',
    name: 'Custom Analysis',
    type: 'analysis',
    priority: 'high',
    data: { /* custom data */ }
  }
];

const results = await orchestrator.executeTasks(customTasks);
```

## Troubleshooting

### Common Issues

1. **Worker creation fails**: Check Node.js version (requires 18+)
2. **Tasks timeout**: Increase timeout or optimize task logic
3. **Memory issues**: Reduce maxWorkers count
4. **Path errors**: Ensure proper file paths in task data

### Debug Mode

```bash
# Run with debug output
NODE_DEBUG=worker tddai worker run
```

## Migration from Legacy Systems

This system replaces:
- `implement-5-worker-system.js`
- `ai-integration-workers.js`
- `tddai-worker-orchestrator.js`

All functionality has been preserved with improved:
- Type safety
- Error handling
- Performance monitoring
- CLI integration