# TDDAI Simplification Plan - Following Drupal Standards

## Overview
This plan outlines how to simplify TDDAI by following Drupal's architectural principles and "keep it simple" philosophy.

## Core Principles (from Drupal)
1. **Progressive Disclosure of Complexity** - Start simple, add complexity only when needed
2. **Single Responsibility** - Each module/component does one thing well
3. **Composition over Inheritance** - Use services and dependency injection
4. **Convention over Configuration** - Sensible defaults with minimal config

## Phase 1: Consolidate Core Components (Week 1)

### 1.1 Simplify Worker System
**Current**: 19 worker files with complex orchestration
**Target**: 2 task runners

```
src/
├── tasks/
│   ├── TaskRunner.ts         # Simple async task execution
│   └── TaskTypes.ts          # Task interfaces
```

**Actions**:
- Remove WorkerOrchestrator, WorkerPool, WorkerMonitor
- Consolidate 19 workers into 2 simple task runners:
  - `TestTaskRunner` - Handles all test-related tasks
  - `AnalysisTaskRunner` - Handles code analysis tasks
- Use native async/await instead of worker threads for most tasks

### 1.2 Flatten AI Integration
**Current**: 8 AI files with multiple abstraction layers
**Target**: 2 focused modules

```
src/
├── ai/
│   ├── AIProvider.ts         # Direct API integration (OpenAI, Anthropic, Ollama)
│   └── AIHelpers.ts          # Shared utilities
```

**Actions**:
- Remove AIProtocol, ClaudeCodeBridge, ContextGenerator abstractions
- Direct API calls to AI providers (no LangChain)
- Simple provider interface:
  ```typescript
  interface AIProvider {
    generateTests(code: string): Promise<string[]>;
    analyzeCode(code: string): Promise<CodeAnalysis>;
  }
  ```

### 1.3 Unify Documentation
**Current**: 44+ documentation files
**Target**: 3 focused documents

```
/
├── README.md                 # User guide
├── API.md                    # Developer reference
└── examples/                 # Code examples
    └── *.example.js
```

**Actions**:
- Delete all duplicate documentation
- Consolidate into single-purpose files
- Remove marketing language

## Phase 2: Simplify Architecture (Week 2)

### 2.1 Reduce Type Complexity
**Current**: Deep type nesting and central exports
**Target**: Co-located simple types

**Actions**:
- Move types to their implementation files
- Remove unnecessary type exports from index.ts
- Simplify interfaces to essential properties only

### 2.2 Consolidate CLI Commands
**Current**: 60+ npm scripts, complex command tree
**Target**: 10 focused commands (like Drush)

```bash
tddai test              # Run tests
tddai test:generate     # Generate tests
tddai analyze           # Analyze code
tddai setup             # Setup project
tddai validate          # Validate TDD compliance
```

**Actions**:
- Remove duplicate commands
- Flatten command structure
- Clear, verb-based naming

### 2.3 Single Configuration
**Current**: Multiple config files and formats
**Target**: One YAML config

```yaml
# tddai.yml
framework: jest
coverage: 95
ai:
  provider: ollama
  model: llama2
```

**Actions**:
- Remove all config variations
- Single YAML file with schema
- Sensible defaults

## Phase 3: Remove Over-Engineering (Week 3)

### 3.1 Eliminate Unnecessary Dependencies
**Current**: Heavy LangChain dependencies
**Target**: Minimal dependencies

**Remove**:
```json
"@langchain/openai": "^0.0.25",
"@langchain/anthropic": "^0.1.11", 
"@langchain/core": "^0.1.52",
"langchain": "^0.1.30"
```

**Keep**:
```json
"commander": "^11.1.0",
"chalk": "5.3.0",
"js-yaml": "^4.1.0"
```

### 3.2 Simplify Preset System
**Current**: Complex preset resolution
**Target**: Simple templates

```
presets/
├── node.yml
├── drupal.yml
└── react.yml
```

**Actions**:
- Convert to simple YAML templates
- Remove preset resolution logic
- Direct file copying

### 3.3 Remove Abstract Base Classes
**Current**: Unnecessary inheritance hierarchies
**Target**: Composition with services

**Actions**:
- Remove abstract base classes
- Use dependency injection
- Simple service classes

## Phase 4: Drupal Integration (Week 4)

### 4.1 Drush Command Integration
Create Drush commands instead of custom CLI:

```php
namespace Drupal\tddai\Commands;

class TddaiCommands extends DrushCommands {
    #[Command(name: 'tddai:test')]
    public function runTests() {
        // Implementation
    }
}
```

### 4.2 Use Drupal Services
Replace custom implementations with Drupal core:
- Use `ModuleHandler` for module discovery
- Use `FileSystem` service for file operations
- Use Drupal's testing framework

### 4.3 Configuration Management
Use Drupal's config system:
```yaml
# config/install/tddai.settings.yml
framework: phpunit
coverage_threshold: 95
```

## Implementation Priority

### High Priority (Biggest Impact)
1. **Consolidate Workers** - Removes most complexity
2. **Flatten AI Integration** - Simplifies core functionality
3. **Single Configuration** - Reduces confusion

### Medium Priority
4. **Simplify CLI** - Better user experience
5. **Remove LangChain** - Reduces dependencies
6. **Unify Documentation** - Clearer communication

### Low Priority (Nice to Have)
7. **Type System Changes** - Internal improvement
8. **Preset Simplification** - Minor feature
9. **Script Consolidation** - Maintenance improvement

## Success Metrics
- **Code Reduction**: Target 50% fewer files
- **Dependency Reduction**: Remove 4+ major dependencies  
- **API Simplification**: 10 public methods instead of 60+
- **Documentation**: 3 files instead of 44
- **Test Coverage**: Maintain 95% throughout

## Migration Strategy
1. Create feature branch for each phase
2. Maintain backward compatibility during transition
3. Deprecate old APIs with clear migration path
4. Update tests incrementally
5. Release as v3.0.0 with breaking changes documented

## Timeline
- **Week 1**: Core consolidation (Workers, AI, Docs)
- **Week 2**: Architecture simplification
- **Week 3**: Remove over-engineering
- **Week 4**: Drupal integration
- **Week 5**: Testing and documentation
- **Week 6**: Release v3.0.0

This plan follows Drupal's philosophy of starting simple and adding complexity only when proven necessary. The result will be a maintainable, understandable codebase that integrates naturally with Drupal projects.