# AI Coding Standards Schema

This document defines the schema for AI coding standards in the ReelCode system. The standards are defined in YAML or JSON format and are used to configure AI assistants to follow your coding standards consistently.

## Schema Definition

```yaml
version: "1.0"                  # Standards schema version (required)
name: "Project Standards"       # Name of the standards (required)
description: "..."              # Description (optional)
extends: "base-standards"       # Parent standards to extend (optional)

# Core standards that apply to all projects
core:
  # Coding style standards
  codingStyle:
    # TypeScript standards
    typescript:
      strictMode: true          # Whether to enforce TypeScript strict mode
      preferConst: true         # Prefer const over let
      maxLineLength: 100        # Maximum line length
      indentSize: 2             # Indentation size
      indentStyle: "space"      # "space" or "tab"
      
      # Naming conventions
      namingConventions:
        classes: "PascalCase"         # Class naming convention
        interfaces: "PascalCase"      # Interface naming convention
        types: "PascalCase"           # Type naming convention
        enums: "PascalCase"           # Enum naming convention
        constants: "UPPER_SNAKE_CASE" # Constant naming convention
        variables: "camelCase"        # Variable naming convention
        functions: "camelCase"        # Function naming convention
        methods: "camelCase"          # Method naming convention
        properties: "camelCase"       # Property naming convention
        
      # Import rules
      imports:
        preferNamed: true             # Prefer named imports
        orderImports: true            # Order imports alphabetically
        noDefaultExports: false       # Whether to disallow default exports
        
      # TypeScript specific rules
      noImplicitAny: true             # Disallow implicit any
      noExplicitAny: false            # Disallow explicit any
      preferInterface: true           # Prefer interface over type
      preferTypeDeclaration: true     # Prefer type declarations
    
    # JavaScript standards  
    javascript:
      # Similar structure to TypeScript
      
    # PHP standards
    php:
      standard: "PSR-12"              # PSR standard to follow
      strictTypes: true               # Whether to enforce strict types
      
      # Naming conventions
      namingConventions:
        classes: "PascalCase"
        interfaces: "PascalCase"
        traits: "PascalCase"
        constants: "UPPER_SNAKE_CASE"
        properties: "camelCase"
        methods: "camelCase"
        functions: "snake_case"
        variables: "camelCase"
  
  # Testing standards
  testingRequirements:
    coverageThreshold: 90             # Minimum test coverage percentage
    methodology: "TDD"                # Testing methodology (TDD, BDD, etc.)
    requiredTests:                    # Required test types
      - "unit"
      - "integration"
    requiredTestFiles:                # Required test files pattern
      pattern: "Name.test.[ext]"    # Test file naming pattern
    
  # Documentation standards
  documentation:
    required: true                    # Whether documentation is required
    apiDocumentation: "OpenAPI 3.1.0" # API documentation standard
    codeCommentsFormat: "JSDoc"       # Code comments format
    readmeFormat: "standardized"      # README format
    requiredDocs:                     # Required documentation
      - "README.md"
      - "CONTRIBUTING.md"
    
  # Security standards
  security:
    inputValidation: true             # Whether input validation is required
    authenticationRequired: true      # Whether authentication is required
    apiSecurity: true                 # Whether API security is required
    sensitiveDataHandling: "strict"   # How to handle sensitive data
    
  # AI assistant configurations
  aiAssistants:
    # Claude configuration
    claude:
      contextFormat: "markdown"       # Format for context
      promptTemplate: "standardized"  # Prompt template
      systemPrompt: true              # Whether to use system prompt
      
    # GitHub Copilot configuration  
    copilot:
      inline: true                    # Whether to use inline suggestions
      chat: true                      # Whether to use chat
      
    # Cursor configuration
    cursor:
      rules: true                     # Whether to use rules
  
  # Project structure standards
  projectStructure:
    requiredDirectories:              # Required directories
      - "src"
      - "tests"
      - "docs"
    fileNamingPatterns:               # File naming patterns
      'src/components/*.tsx': '^[A-Z][a-zA-Z]+\\.tsx$'
      'src/utils/*.ts': '^[a-z][a-zA-Z]+\\.ts$'
    
# Project-specific standards
projects:
  - id: "bfapi"                       # Project ID
    extends: "core"                   # Extend core standards
    specifics:                        # Project-specific standards
      apiGateway: true                # API gateway specific standards
      middleware: true                # Middleware specific standards
      
  - id: "llm-mcp"                       # Project ID
    extends: "core"                   # Extend core standards
    specifics:                        # Project-specific standards
      toolRegistry: true              # Tool registry specific standards
      contextManagement: true         # Context management specific standards
      
  - id: "drupal_modules"              # Project ID
    extends: "core"                   # Extend core standards
    specifics:                        # Project-specific standards
      drupalCodingStandards: true     # Drupal coding standards
      entityModeling: true            # Entity modeling standards
      serviceDefinition: true         # Service definition standards

# Custom validations
customValidations:
  - name: "copyright-header"           # Custom validation name
    description: "Check for copyright headers"  # Description
    type: "regex"                      # Validation type
    pattern: "^/\\*\\*\\n \\* Copyright"  # Regex pattern
    filePatterns:                      # File patterns to apply to
      - "src/**/*.ts"
      - "src/**/*.tsx"
    message: "Missing copyright header"  # Error message
```

## Inheritance

Standards can inherit from parent standards using the `extends` property. This allows you to create a base set of standards and then extend them for specific projects or requirements.

The inheritance system follows these rules:
1. Child standards inherit all properties from parent standards
2. Child standards can override specific properties
3. Arrays are merged, with child arrays appending to parent arrays
4. Objects are deeply merged

## Example: Basic Standards

```yaml
# base-standards.yaml
version: "1.0"
name: "Base Standards"
core:
  codingStyle:
    typescript:
      strictMode: true
      namingConventions:
        classes: "PascalCase"
        interfaces: "PascalCase"
```

## Example: Project-Specific Standards

```yaml
# project-standards.yaml
version: "1.0"
name: "Project Standards"
extends: "base-standards"
core:
  codingStyle:
    typescript:
      namingConventions:
        interfaces: "IPascalCase"  # Override interface naming
      indentSize: 2                # Add new property
  testingRequirements:             # Add new section
    coverageThreshold: 90
```

When processed, the resulting standards would be:

```yaml
version: "1.0"
name: "Project Standards"
core:
  codingStyle:
    typescript:
      strictMode: true             # Inherited from base
      namingConventions:
        classes: "PascalCase"      # Inherited from base
        interfaces: "IPascalCase"  # Overridden in project
      indentSize: 2                # Added in project
  testingRequirements:
    coverageThreshold: 90          # Added in project
```

## Using the Schema

This schema forms the foundation of the ReelCode AI Standards Management System. It enables:

1. **Standardization**: Consistent coding standards across projects
2. **AI Integration**: Automatic configuration of AI assistants
3. **Validation**: Enforcement of standards in code review and CI/CD
4. **Documentation**: Automatic generation of standards documentation

For implementation details, see the [TDD Implementation Plan](./TDD_IMPLEMENTATION_PLAN.md).