# Workspace-Wide Naming Convention Audit & Fix Plan

## 🔍 **SCOPE ANALYSIS**

Based on comprehensive workspace scan, here are the projects requiring naming convention fixes:

### **📂 Project Structure Identified**

```
/Users/flux423/Sites/LLM/
├── _CommonNPM/tddai/          ✅ FIXED (100% compliant)
├── bfapi/                     ❌ HIGH PRIORITY (437 violations)
├── llmcli/                     ❌ HIGH PRIORITY (296 violations)  
├── llm-ui/                      ❌ MEDIUM PRIORITY (223 violations)
├── llm-mcp/                     ❌ MEDIUM PRIORITY (41 violations)
├── bfrfp/                     ❌ MEDIUM PRIORITY (38 violations)
├── llm-gateway-sdk/               ❌ MEDIUM PRIORITY (22 violations)
├── apple-fm/                  ❌ LOW PRIORITY (15 violations)
├── bfdocs/                    ❌ LOW PRIORITY (8 violations)
├── secure_project/            ❌ LOW PRIORITY (6 violations)
└── bfcicomponents/            ⚠️  PARTIAL (needs review)
```

---

## 🎯 **PRIORITY MATRIX**

### **PHASE 1: Critical Infrastructure (Week 1-2)**

#### **1. bfapi** - API Gateway Service
- **Violations**: 437 files
- **Impact**: 🔴 CRITICAL - Core infrastructure
- **Dependencies**: All other services depend on this
- **Effort**: 15-20 hours

**Key Files to Fix:**
```bash
# Controllers (25+ files)
BridgeController.ts → bridge-controller.ts
InferenceController.ts → inference-controller.ts
ModelRegistryController.ts → model-registry-controller.ts

# Services (20+ files)  
BridgeService.ts → bridge-service.ts
LoadBalancerService.ts → load-balancer-service.ts
ApiCacheManager.ts → api-cache-manager.ts

# Middleware (15+ files)
authMiddleware.ts → auth-middleware.ts
apiRateLimiter.ts → api-rate-limiter.ts
```

#### **2. llmcli** - Development CLI Tool  
- **Violations**: 296 files
- **Impact**: 🟠 HIGH - Developer workflow
- **Dependencies**: Used by all developers daily
- **Effort**: 12-15 hours

---

### **PHASE 2: UI & User-Facing (Week 3)**

#### **3. llm-ui** - Frontend Components
- **Violations**: 223 files  
- **Impact**: 🟡 MEDIUM-HIGH - User interface
- **Dependencies**: UI components used across apps
- **Effort**: 10-12 hours

#### **4. llm-mcp** - Model Control Plane
- **Violations**: 41 files
- **Impact**: 🟡 MEDIUM - Model management  
- **Dependencies**: AI service coordination
- **Effort**: 4-6 hours

---

### **PHASE 3: Supporting Services (Week 4)**

#### **5. bfrfp** - Content Generation
- **Violations**: 38 files
- **Impact**: 🟢 MEDIUM - Content workflows
- **Effort**: 3-5 hours

#### **6. llm-gateway-sdk** - API Client Library
- **Violations**: 22 files  
- **Impact**: 🟢 MEDIUM - Client integrations
- **Effort**: 2-3 hours

---

### **PHASE 4: Specialized Projects (Week 5)**

#### **7. apple-fm** - Apple Foundation Models
- **Violations**: 15 files
- **Impact**: 🔵 LOW - Specialized integration
- **Effort**: 1-2 hours

#### **8. bfdocs** - Documentation System
- **Violations**: 8 files
- **Impact**: 🔵 LOW - Documentation
- **Effort**: 1 hour

#### **9. secure_project** - Security Module  
- **Violations**: 6 files
- **Impact**: 🔵 LOW - Security utilities
- **Effort**: 30 minutes

---

## 📋 **TYPICAL VIOLATIONS PATTERN**

All projects follow similar anti-patterns:

### **File Naming Issues**
```bash
❌ PascalCase Files:
   BridgeController.ts
   ApiCacheManager.ts  
   LoadBalancerService.ts

✅ Should be kebab-case:
   bridge-controller.ts
   api-cache-manager.ts
   load-balancer-service.ts
```

### **Common Violation Categories**
1. **Controllers**: 25-40 files per project (PascalCase → kebab-case)
2. **Services**: 15-25 files per project (PascalCase → kebab-case)  
3. **Middleware**: 10-15 files per project (camelCase → kebab-case)
4. **Utils**: 5-10 files per project (mixed → kebab-case)
5. **Types**: 3-8 files per project (PascalCase → kebab-case)

---

## 🛠 **IMPLEMENTATION STRATEGY**

### **Automated Fix Approach**
```bash
# 1. Systematic file renaming
for file in *Controller.ts; do
  kebab=$(echo $file | sed 's/\\([A-Z]\\)/-\\L\\1/g' | sed 's/^-//')
  mv "$file" "$kebab"
done

# 2. Import statement updates  
find . -name "*.ts" -exec sed -i 's/from.*Controller/from.*controller/g' {} \\;

# 3. Build verification
npm run build && npm test
```

### **Risk Mitigation**
1. **Backup Strategy**: Git branch per project
2. **Incremental Approach**: Fix foundation files first
3. **Build Validation**: Continuous build testing
4. **Import Tracking**: Automated import updates

---

## 📊 **EFFORT ESTIMATION**

### **Total Scope**
- **Files to Rename**: ~1,319 files
- **Import Updates**: ~2,500+ statements  
- **Projects**: 10 major projects
- **Total Effort**: 40-60 hours (5-6 weeks)

### **Weekly Breakdown**
- **Week 1**: bfapi (20 hours)
- **Week 2**: llmcli (15 hours)  
- **Week 3**: llm-ui + llm-mcp (16 hours)
- **Week 4**: bfrfp + llm-gateway-sdk (5 hours)
- **Week 5**: Remaining projects + validation (4 hours)

---

## ✅ **SUCCESS CRITERIA**

### **Per Project**
- [ ] 100% file names follow kebab-case
- [ ] All imports updated correctly
- [ ] Clean TypeScript compilation
- [ ] All tests passing
- [ ] No broken references

### **Workspace-Wide**  
- [ ] Consistent naming across all projects
- [ ] Automated linting rules enforced
- [ ] Documentation updated
- [ ] Developer onboarding guides updated

---

## 🚀 **NEXT STEPS**

### **Immediate Actions**
1. **Start with bfapi** (highest impact, most dependencies)
2. **Create branch**: `naming-fixes-bfapi`
3. **Begin with foundation files**: BaseController, core services
4. **Update imports systematically**
5. **Validate builds after each major change**

### **Recommended Tools**
- **Automated renaming scripts**
- **Import update utilities**
- **ESLint rules for enforcement**
- **Build validation automation**

### **Quality Gates**
- TypeScript compilation ✅
- Unit tests passing ✅  
- Integration tests passing ✅
- No circular dependencies ✅
- Performance unchanged ✅

---

## 📞 **READY TO PROCEED**

The audit is complete. **1,319 naming violations** identified across **10 projects**.

**Recommended approach**: Start with **bfapi** as it's the critical infrastructure that all other services depend on.

Would you like me to:
1. **Start fixing bfapi immediately** 
2. **Create detailed bfapi fix plan first**
3. **Begin with a different project**

All projects are ready for systematic naming convention fixes following the established kebab-case standards.