# Standards Configuration

This guide explains how to configure and customize coding standards for your projects using the AI Coding Standards Tool.

## Configuration File

The AI Coding Standards Tool uses a JSON configuration file (`.acs-standards.json`) to define coding standards. This file should be placed in the root of your project.

### Basic Structure

The configuration file has the following basic structure:

```json
{
  "standards": {
    "[language]": {
      "extends": ["base-standard-1", "base-standard-2"],
      "rules": {
        "rule-name": ["error", { "option1": "value1" }],
        "another-rule": "warn"
      }
    }
  },
  "aiAssistants": {
    "configurePrompts": true,
    "assistants": ["claude", "copilot", "gpt4"],
    "contextInclusion": ["standards", "architecture"]
  },
  "analysis": {
    "ignorePatterns": ["node_modules/**", "dist/**"]
  },
  "reporting": {
    "outputFormat": "html",
    "includeMetrics": true
  }
}
```

## Standards Definition

The `standards` section defines language-specific coding standards. Each language can have its own set of rules and configurations.

### Language-Specific Configuration

```json
"standards": {
  "javascript": {
    "extends": ["airbnb-base"],
    "rules": {
      "max-len": ["error", { "code": 100 }]
    }
  },
  "typescript": {
    "extends": ["airbnb-typescript", "typescript-strict"],
    "rules": {
      "naming-convention": ["error", { "format": "camelCase" }]
    }
  }
}
```

### Extending Base Standards

The `extends` array allows you to inherit rules from predefined standards:

```json
"extends": ["airbnb-base", "prettier"]
```

Available base standards include:
- Standard JavaScript: `standard-js`
- Airbnb: `airbnb-base`
- Google: `google`
- Prettier: `prettier`
- TypeScript Recommended: `typescript-recommended`
- TypeScript Strict: `typescript-strict`

### Rule Configuration

Rules define specific coding standards requirements:

```json
"rules": {
  "max-len": ["error", { "code": 100 }],
  "naming-convention": ["error", {
    "selector": "variable",
    "format": ["camelCase", "UPPER_CASE"]
  }],
  "documentation": "warn",
  "complexity": ["error", { "max": 15 }]
}
```

Each rule can have the following severity levels:
- `"error"`: Rule violations will cause validation to fail
- `"warn"`: Rule violations will generate warnings but not fail validation
- `"off"`: Rule is disabled

Many rules accept options to customize their behavior, provided as an object in the second array element.

## AI-Specific Standards

The tool includes AI-specific standards that guide AI assistants in generating compliant code:

```json
"rules": {
  "ai-assisted": {
    "documentation": "required",
    "testCoverage": 90,
    "promptAnnotations": true,
    "explainability": "high"
  }
}
```

Common AI-specific rules include:
- `documentation`: Requirements for AI-generated code documentation
- `testCoverage`: Expected test coverage for AI-generated code
- `promptAnnotations`: Whether AI should annotate code with reasoning
- `explainability`: Level of explanation required (low, medium, high)

## AI Assistants Configuration

The `aiAssistants` section configures how standards integrate with AI coding assistants:

```json
"aiAssistants": {
  "configurePrompts": true,
  "assistants": ["claude", "copilot", "gpt4"],
  "contextInclusion": ["standards", "architecture", "examples"],
  "promptTemplates": {
    "codeGeneration": "path/to/template.txt",
    "codeReview": "path/to/review-template.txt"
  }
}
```

Key options include:
- `configurePrompts`: Whether to automatically configure AI prompts
- `assistants`: Which AI assistants to configure
- `contextInclusion`: What context to include in AI prompts
- `promptTemplates`: Custom prompt templates for different tasks

## Analysis Configuration

The `analysis` section configures how the tool analyzes your codebase:

```json
"analysis": {
  "ignorePatterns": ["node_modules/**", "dist/**", "*.test.js"],
  "maxFileSize": 1000000,
  "concurrency": 4,
  "cache": true,
  "plugins": ["custom-plugin-1", "custom-plugin-2"]
}
```

Key options include:
- `ignorePatterns`: Files to exclude from analysis
- `maxFileSize`: Maximum file size to analyze
- `concurrency`: Number of parallel analysis processes
- `cache`: Whether to cache analysis results
- `plugins`: Custom analysis plugins to use

## Reporting Configuration

The `reporting` section configures how the tool generates compliance reports:

```json
"reporting": {
  "outputFormat": "html",
  "outputDir": "./standards-reports",
  "includeMetrics": true,
  "includeViolations": true,
  "groupBy": "rule",
  "compareWithPrevious": true
}
```

Key options include:
- `outputFormat`: Report format (html, json, markdown)
- `outputDir`: Directory for report output
- `includeMetrics`: Whether to include compliance metrics
- `includeViolations`: Whether to include rule violations
- `groupBy`: How to group violations (rule, file, severity)
- `compareWithPrevious`: Whether to compare with previous reports

## Environment-Specific Configurations

You can define environment-specific configurations using the `environments` section:

```json
"environments": {
  "development": {
    "rules": {
      "max-len": "warn",
      "documentation": "off"
    }
  },
  "production": {
    "rules": {
      "max-len": "error",
      "documentation": "error"
    }
  }
}
```

To use an environment-specific configuration:

```bash
acs analyze --env production
```

## Configuration Examples

### Minimal Configuration

```json
{
  "standards": {
    "javascript": {
      "extends": ["standard-js"]
    }
  }
}
```

### TypeScript Project

```json
{
  "standards": {
    "typescript": {
      "extends": ["typescript-recommended"],
      "rules": {
        "naming-convention": ["error", {
          "selector": "variable",
          "format": ["camelCase"]
        }],
        "documentation": "error"
      }
    }
  },
  "aiAssistants": {
    "configurePrompts": true,
    "assistants": ["claude"]
  }
}
```

### Full-Featured Configuration

See our [example configurations](../examples/) for more comprehensive examples.

## Configuration Management

### Version Control

It's recommended to commit your `.acs-standards.json` file to version control to ensure consistent standards across your team.

### Standards Inheritance

For projects with multiple packages or components, you can define standards inheritance hierarchies. See [Standards Inheritance](inheritance.md) for details.

### Programmatic Configuration

You can also configure standards programmatically:

```typescript
import [StandardsConfiguration] from '@bluefly/ai-coding-standards';

const config = new StandardsConfiguration();
config.addRule('javascript', 'max-len', ['error', { code: 100 }]);
config.extendStandard('javascript', 'airbnb-base');
config.save('./.acs-standards.json');
```

## Next Steps

- Learn about [Language Support](language-support.md)
- Explore [Custom Standards](custom-standards.md)
- Set up [AI Assistant Integration](../ai-integration/overview.md)