---
id: security-hardening
---
# Security Hardening and Compliance Guide

<!-- title: Security Hardening and Compliance Guide -->
<!-- description: Practical guide for security hardening and compliance in the LLM Platform Platform -->
<!-- lastUpdated: 2024-03-19 -->

## Overview

This guide covers practical security hardening and compliance measures for the LLM Platform Platform, including security controls, compliance requirements, and security monitoring.

## Security Controls

### Input Validation

```typescript
// src/security/validation.pipe.ts
@Injectable()
export class SecurityValidationPipe implements PipeTransform {
  private readonly logger = new Logger(SecurityValidationPipe.name);
  
  transform(value: any, metadata: ArgumentMetadata) {
    try {
      // Sanitize input
      const sanitized = this.sanitizeInput(value);
      
      // Validate against schema
      const schema = this.getSchema(metadata);
      const { error } = schema.validate(sanitized, {
        abortEarly: false,
        stripUnknown: true
      });
      
      if (error) {
        throw new BadRequestException(
          this.formatValidationError(error)
        );
      }
      
      return sanitized;
    } catch (error) {
      this.logger.error('Validation error:', error);
      throw error;
    }
  }
  
  private sanitizeInput(value: any): any {
    if (typeof value === 'string') {
      // Remove potentially dangerous characters
      return value.replace(/[<>'"]/g, '');
    }
    
    if (Array.isArray(value)) {
      return value.map(this.sanitizeInput);
    }
    
    if (typeof value === 'object' && value !== null) {
      return Object.fromEntries(
        Object.entries(value).map(([key, val]) => [
          key,
          this.sanitizeInput(val)
        ])
      );
    }
    
    return value;
  }
  
  private getSchema(metadata: ArgumentMetadata): any {
    // Get schema from metadata
    const schema = Reflect.getMetadata(
      'validation:schema',
      metadata.metatype
    );
    
    if (!schema) {
      throw new Error('Validation schema not found');
    }
    
    return schema;
  }
  
  private formatValidationError(error: any): string {
    return error.details
      .map((detail: any) => detail.message)
      .join(', ');
  }
}

// src/decorators/validate.decorator.ts
export const Validate = (schema: any) => {
  return (target: any, key: string, descriptor: PropertyDescriptor) => {
    Reflect.defineMetadata(
      'validation:schema',
      schema,
      target.constructor
    );
    return descriptor;
  };
};
```

### Rate Limiting

```typescript
// src/security/rate-limit.guard.ts
@Injectable()
export class RateLimitGuard implements CanActivate {
  private readonly redis: Redis;
  private readonly logger = new Logger(RateLimitGuard.name);
  
  constructor(
    private configService: ConfigService
  ) {
    this.redis = new Redis({
      host: this.configService.get('REDIS_HOST'),
      port: this.configService.get('REDIS_PORT'),
      password: this.configService.get('REDIS_PASSWORD')
    });
  }
  
  async canActivate(context: ExecutionContext): Promise<boolean> {
    const request = context.switchToHttp().getRequest();
    const key = this.getRateLimitKey(request);
    
    try {
      const limit = this.getRateLimit(request);
      const window = this.getRateLimitWindow(request);
      
      const current = await this.redis.incr(key);
      if (current === 1) {
        await this.redis.expire(key, window);
      }
      
      if (current > limit) {
        throw new HttpException(
          'Too Many Requests',
          HttpStatus.TOO_MANY_REQUESTS
        );
      }
      
      // Add rate limit headers
      const response = context.switchToHttp().getResponse();
      response.header('X-RateLimit-Limit', limit);
      response.header('X-RateLimit-Remaining', Math.max(0, limit - current));
      response.header(
        'X-RateLimit-Reset',
        Math.ceil(Date.now() / 1000) + window
      );
      
      return true;
    } catch (error) {
      if (error instanceof HttpException) {
        throw error;
      }
      
      this.logger.error('Rate limit error:', error);
      return false;
    }
  }
  
  private getRateLimitKey(request: any): string {
    const { ip, method, path } = request;
    return `ratelimit:${ip}:${method}:${path}`;
  }
  
  private getRateLimit(request: any): number {
    // Get limit from decorator or config
    return Reflect.getMetadata(
      'rate-limit',
      request.route?.handler
    ) || this.configService.get('RATE_LIMIT');
  }
  
  private getRateLimitWindow(request: any): number {
    // Get window from decorator or config
    return Reflect.getMetadata(
      'rate-limit-window',
      request.route?.handler
    ) || this.configService.get('RATE_LIMIT_WINDOW');
  }
}

// src/decorators/rate-limit.decorator.ts
export const RateLimit = (limit: number, window: number) => {
  return (target: any, key: string, descriptor: PropertyDescriptor) => {
    Reflect.defineMetadata('rate-limit', limit, descriptor.value);
    Reflect.defineMetadata(
      'rate-limit-window',
      window,
      descriptor.value
    );
    return descriptor;
  };
};
```

### Security Headers

```typescript
// src/security/security.middleware.ts
@Injectable()
export class SecurityMiddleware implements NestMiddleware {
  use(req: Request, res: Response, next: Function) {
    // Set security headers
    res.setHeader(
      'Content-Security-Policy',
      "default-src 'self'; " +
      "script-src 'self' 'unsafe-inline' 'unsafe-eval'; " +
      "style-src 'self' 'unsafe-inline'; " +
      "img-src 'self' data: https:; " +
      "font-src 'self' data: https:; " +
      "connect-src 'self' https:;"
    );
    
    res.setHeader('X-Content-Type-Options', 'nosniff');
    res.setHeader('X-Frame-Options', 'DENY');
    res.setHeader('X-XSS-Protection', '1; mode=block');
    res.setHeader(
      'Strict-Transport-Security',
      'max-age=31536000; includeSubDomains'
    );
    res.setHeader(
      'Referrer-Policy',
      'strict-origin-when-cross-origin'
    );
    res.setHeader(
      'Permissions-Policy',
      'geolocation=(), microphone=(), camera=()'
    );
    
    // Remove sensitive headers
    res.removeHeader('X-Powered-By');
    res.removeHeader('Server');
    
    next();
  }
}
```

## Compliance Requirements

### Audit Logging

```typescript
// src/security/audit.service.ts
@Injectable()
export class AuditService {
  private readonly logger = new Logger(AuditService.name);
  private readonly elastic: Client;
  
  constructor(
    private configService: ConfigService
  ) {
    this.elastic = new Client({
      node: this.configService.get('ELASTICSEARCH_URL'),
      auth: {
        username: this.configService.get('ELASTICSEARCH_USER'),
        password: this.configService.get('ELASTICSEARCH_PASSWORD')
      }
    });
  }
  
  async logAuditEvent(event: AuditEvent): Promise<void> {
    try {
      await this.elastic.index({
        index: 'audit-logs',
        document: {
          ...event,
          timestamp: new Date().toISOString(),
          environment: this.configService.get('NODE_ENV')
        }
      });
    } catch (error) {
      this.logger.error('Audit logging failed:', error);
      // Fallback to file logging
      this.logger.log('Audit event:', event);
    }
  }
  
  async searchAuditLogs(
    query: AuditLogQuery
  ): Promise<AuditLogSearchResult> {
    try {
      const { body } = await this.elastic.search({
        index: 'audit-logs',
        body: {
          query: {
            bool: {
              must: [
                { term: { 'event.type': query.eventType } },
                { range: {
                  timestamp: {
                    gte: query.startDate,
                    lte: query.endDate
                  }
                }}
              ],
              filter: query.filters
            }
          },
          sort: [{ timestamp: 'desc' }],
          from: query.offset,
          size: query.limit
        }
      });
      
      return {
        total: body.hits.total.value,
        hits: body.hits.hits.map(hit => hit._source),
        aggregations: body.aggregations
      };
    } catch (error) {
      this.logger.error('Audit log search failed:', error);
      throw error;
    }
  }
}

// src/interceptors/audit.interceptor.ts
@Injectable()
export class AuditInterceptor implements NestInterceptor {
  constructor(
    private auditService: AuditService,
    private logger: Logger
  ) {}
  
  intercept(
    context: ExecutionContext,
    next: CallHandler
  ): Observable<any> {
    const request = context.switchToHttp().getRequest();
    const { method, url, user, body } = request;
    
    const startTime = Date.now();
    
    return next.handle().pipe(
      tap({
        next: (response) => {
          this.auditService.logAuditEvent({
            type: 'api_request',
            action: method,
            resource: url,
            user: user?.id,
            status: 'success',
            duration: Date.now() - startTime,
            metadata: {
              requestBody: this.sanitizeBody(body),
              responseStatus: response?.status
            }
          });
        },
        error: (error) => {
          this.auditService.logAuditEvent({
            type: 'api_request',
            action: method,
            resource: url,
            user: user?.id,
            status: 'error',
            duration: Date.now() - startTime,
            metadata: {
              requestBody: this.sanitizeBody(body),
              error: error.message,
              stack: error.stack
            }
          });
        }
      })
    );
  }
  
  private sanitizeBody(body: any): any {
    // Remove sensitive data
    const sanitized = { ...body };
    delete sanitized.password;
    delete sanitized.token;
    delete sanitized.secret;
    return sanitized;
  }
}
```

### Compliance Monitoring

```typescript
// src/security/compliance.service.ts
@Injectable()
export class ComplianceService {
  private readonly logger = new Logger(ComplianceService.name);
  private readonly elastic: Client;
  
  constructor(
    private configService: ConfigService,
    private auditService: AuditService
  ) {
    this.elastic = new Client({
      node: this.configService.get('ELASTICSEARCH_URL'),
      auth: {
        username: this.configService.get('ELASTICSEARCH_USER'),
        password: this.configService.get('ELASTICSEARCH_PASSWORD')
      }
    });
  }
  
  async checkCompliance(): Promise<ComplianceReport> {
    const report: ComplianceReport = {
      timestamp: new Date().toISOString(),
      checks: []
    };
    
    // Check security controls
    report.checks.push(
      await this.checkSecurityControls()
    );
    
    // Check audit logging
    report.checks.push(
      await this.checkAuditLogging()
    );
    
    // Check data protection
    report.checks.push(
      await this.checkDataProtection()
    );
    
    // Check access controls
    report.checks.push(
      await this.checkAccessControls()
    );
    
    return report;
  }
  
  private async checkSecurityControls(): Promise<ComplianceCheck> {
    const check: ComplianceCheck = {
      name: 'security_controls',
      status: 'pass',
      details: []
    };
    
    try {
      // Check rate limiting
      const rateLimitEvents = await this.auditService.searchAuditLogs({
        eventType: 'rate_limit_exceeded',
        startDate: subDays(new Date(), 1).toISOString(),
        endDate: new Date().toISOString()
      });
      
      if (rateLimitEvents.total > 100) {
        check.status = 'fail';
        check.details.push({
          message: 'High rate of rate limit violations',
          count: rateLimitEvents.total
        });
      }
      
      // Check security headers
      const headerEvents = await this.auditService.searchAuditLogs({
        eventType: 'security_header_missing',
        startDate: subDays(new Date(), 1).toISOString(),
        endDate: new Date().toISOString()
      });
      
      if (headerEvents.total > 0) {
        check.status = 'fail';
        check.details.push({
          message: 'Missing security headers detected',
          count: headerEvents.total
        });
      }
    } catch (error) {
      check.status = 'error';
      check.details.push({
        message: 'Security control check failed',
        error: error.message
      });
    }
    
    return check;
  }
  
  private async checkAuditLogging(): Promise<ComplianceCheck> {
    const check: ComplianceCheck = {
      name: 'audit_logging',
      status: 'pass',
      details: []
    };
    
    try {
      // Check log coverage
      const coverage = await this.auditService.searchAuditLogs({
        eventType: 'api_request',
        startDate: subHours(new Date(), 1).toISOString(),
        endDate: new Date().toISOString()
      });
      
      if (coverage.total === 0) {
        check.status = 'fail';
        check.details.push({
          message: 'No audit logs in the last hour'
        });
      }
      
      // Check log retention
      const retention = await this.elastic.indices.getSettings({
        index: 'audit-logs'
      });
      
      const retentionDays = parseInt(
        retention['audit-logs'].settings['index.lifecycle.rollover.after']
      );
      
      if (retentionDays < 90) {
        check.status = 'fail';
        check.details.push({
          message: 'Audit log retention less than 90 days',
          current: retentionDays
        });
      }
    } catch (error) {
      check.status = 'error';
      check.details.push({
        message: 'Audit logging check failed',
        error: error.message
      });
    }
    
    return check;
  }
  
  private async checkDataProtection(): Promise<ComplianceCheck> {
    const check: ComplianceCheck = {
      name: 'data_protection',
      status: 'pass',
      details: []
    };
    
    try {
      // Check encryption
      const encryptionEvents = await this.auditService.searchAuditLogs({
        eventType: 'encryption_error',
        startDate: subDays(new Date(), 1).toISOString(),
        endDate: new Date().toISOString()
      });
      
      if (encryptionEvents.total > 0) {
        check.status = 'fail';
        check.details.push({
          message: 'Encryption errors detected',
          count: encryptionEvents.total
        });
      }
      
      // Check data access
      const accessEvents = await this.auditService.searchAuditLogs({
        eventType: 'data_access',
        startDate: subDays(new Date(), 1).toISOString(),
        endDate: new Date().toISOString()
      });
      
      const unauthorizedAccess = accessEvents.hits.filter(
        (event: any) => !event.authorized
      );
      
      if (unauthorizedAccess.length > 0) {
        check.status = 'fail';
        check.details.push({
          message: 'Unauthorized data access detected',
          count: unauthorizedAccess.length
        });
      }
    } catch (error) {
      check.status = 'error';
      check.details.push({
        message: 'Data protection check failed',
        error: error.message
      });
    }
    
    return check;
  }
  
  private async checkAccessControls(): Promise<ComplianceCheck> {
    const check: ComplianceCheck = {
      name: 'access_controls',
      status: 'pass',
      details: []
    };
    
    try {
      // Check authentication
      const authEvents = await this.auditService.searchAuditLogs({
        eventType: 'authentication',
        startDate: subDays(new Date(), 1).toISOString(),
        endDate: new Date().toISOString()
      });
      
      const failedAuth = authEvents.hits.filter(
        (event: any) => !event.success
      );
      
      if (failedAuth.length > 10) {
        check.status = 'fail';
        check.details.push({
          message: 'High rate of authentication failures',
          count: failedAuth.length
        });
      }
      
      // Check authorization
      const authzEvents = await this.auditService.searchAuditLogs({
        eventType: 'authorization',
        startDate: subDays(new Date(), 1).toISOString(),
        endDate: new Date().toISOString()
      });
      
      const failedAuthz = authzEvents.hits.filter(
        (event: any) => !event.authorized
      );
      
      if (failedAuthz.length > 0) {
        check.status = 'fail';
        check.details.push({
          message: 'Authorization failures detected',
          count: failedAuthz.length
        });
      }
    } catch (error) {
      check.status = 'error';
      check.details.push({
        message: 'Access control check failed',
        error: error.message
      });
    }
    
    return check;
  }
}
```

## Security Monitoring

### Security Monitoring Service

```typescript
// src/security/monitoring.service.ts
@Injectable()
export class SecurityMonitoringService {
  private readonly logger = new Logger(SecurityMonitoringService.name);
  private readonly elastic: Client;
  
  constructor(
    private configService: ConfigService,
    private auditService: AuditService
  ) {
    this.elastic = new Client({
      node: this.configService.get('ELASTICSEARCH_URL'),
      auth: {
        username: this.configService.get('ELASTICSEARCH_USER'),
        password: this.configService.get('ELASTICSEARCH_PASSWORD')
      }
    });
  }
  
  async monitorSecurity(): Promise<void> {
    // Monitor security events
    const events = await this.auditService.searchAuditLogs({
      eventType: 'security_event',
      startDate: subMinutes(new Date(), 5).toISOString(),
      endDate: new Date().toISOString()
    });
    
    // Analyze events
    const analysis = this.analyzeSecurityEvents(events.hits);
    
    // Check for threats
    const threats = this.detectThreats(analysis);
    
    // Alert if needed
    if (threats.length > 0) {
      await this.alertThreats(threats);
    }
  }
  
  private analyzeSecurityEvents(events: any[]): SecurityAnalysis {
    const analysis: SecurityAnalysis = {
      totalEvents: events.length,
      eventTypes: new Map(),
      ipAddresses: new Map(),
      users: new Map(),
      resources: new Map()
    };
    
    events.forEach(event => {
      // Count event types
      analysis.eventTypes.set(
        event.type,
        (analysis.eventTypes.get(event.type) || 0) + 1
      );
      
      // Count IP addresses
      if (event.ip) {
        analysis.ipAddresses.set(
          event.ip,
          (analysis.ipAddresses.get(event.ip) || 0) + 1
        );
      }
      
      // Count users
      if (event.user) {
        analysis.users.set(
          event.user,
          (analysis.users.get(event.user) || 0) + 1
        );
      }
      
      // Count resources
      if (event.resource) {
        analysis.resources.set(
          event.resource,
          (analysis.resources.get(event.resource) || 0) + 1
        );
      }
    });
    
    return analysis;
  }
  
  private detectThreats(
    analysis: SecurityAnalysis
  ): SecurityThreat[] {
    const threats: SecurityThreat[] = [];
    
    // Check for brute force attempts
    for (const [ip, count] of analysis.ipAddresses) {
      if (count > 100) {
        threats.push({
          type: 'brute_force',
          severity: 'high',
          details: {
            ip,
            count,
            message: 'Possible brute force attempt detected'
          }
        });
      }
    }
    
    // Check for suspicious user activity
    for (const [user, count] of analysis.users) {
      if (count > 1000) {
        threats.push({
          type: 'suspicious_activity',
          severity: 'medium',
          details: {
            user,
            count,
            message: 'Unusual user activity detected'
          }
        });
      }
    }
    
    // Check for resource abuse
    for (const [resource, count] of analysis.resources) {
      if (count > 500) {
        threats.push({
          type: 'resource_abuse',
          severity: 'medium',
          details: {
            resource,
            count,
            message: 'Possible resource abuse detected'
          }
        });
      }
    }
    
    return threats;
  }
  
  private async alertThreats(
    threats: SecurityThreat[]
  ): Promise<void> {
    try {
      // Log threats
      this.logger.warn('Security threats detected:', threats);
      
      // Send alerts
      await this.elastic.index({
        index: 'security-alerts',
        document: {
          timestamp: new Date().toISOString(),
          threats
        }
      });
      
      // TODO: Send notifications (email, Slack, etc.)
    } catch (error) {
      this.logger.error('Failed to alert threats:', error);
    }
  }
}
```

## Best Practices

1. **Security Controls**
   - Input validation
   - Rate limiting
   - Security headers
   - Access control
   - Data protection

2. **Compliance Requirements**
   - Audit logging
   - Compliance monitoring
   - Data retention
   - Access reviews
   - Security assessments

3. **Security Monitoring**
   - Event analysis
   - Threat detection
   - Alerting
   - Incident response
   - Security reporting

4. **Security Operations**
   - Regular updates
   - Vulnerability scanning
   - Penetration testing
   - Security training
   - Incident management

## Related Documentation

- [Architecture Guide](../architecture/README)
- [Development Guide](../development/README)
- [Deployment Guide](../deployment/README)
- [Operations Guide](../operations/README)
- [Database Guide](../database/README) 
