---
id: security-implementation
---
# Security Implementation Guide

<!-- title: Security Implementation Guide -->
<!-- description: Practical guide for security implementation in the LLM Platform Platform -->
<!-- lastUpdated: 2024-03-19 -->

## Overview

This guide covers practical security implementation for the LLM Platform Platform, including authentication, authorization, data protection, and security monitoring.

## Authentication Implementation

### JWT Authentication

```typescript
// src/auth/jwt.strategy.ts
@Injectable()
export class JwtStrategy extends PassportStrategy(Strategy) {
  constructor(
    private configService: ConfigService,
    private userService: UserService
  ) {
    super({
      jwtFromRequest: ExtractJwt.fromAuthHeaderAsBearerToken(),
      ignoreExpiration: false,
      secretOrKey: configService.get('JWT_SECRET')
    });
  }
  
  async validate(payload: JwtPayload): Promise<User> {
    const user = await this.userService.findById(payload.sub);
    
    if (!user) {
      throw new UnauthorizedException();
    }
    
    if (!user.isActive) {
      throw new UnauthorizedException('User is inactive');
    }
    
    return user;
  }
}

// src/auth/jwt.guard.ts
@Injectable()
export class JwtAuthGuard extends AuthGuard('jwt') {
  handleRequest(err: any, user: any, info: any) {
    if (err || !user) {
      throw err || new UnauthorizedException();
    }
    return user;
  }
}
```

### API Key Authentication

```typescript
// src/auth/api-key.strategy.ts
@Injectable()
export class ApiKeyStrategy extends PassportStrategy(Strategy, 'api-key') {
  constructor(
    private configService: ConfigService,
    private clientService: ClientService
  ) {
    super({
      apiKeyHeader: 'X-API-Key',
      passReqToCallback: true
    });
  }
  
  async validate(
    request: Request,
    apiKey: string
  ): Promise<Client> {
    const client = await this.clientService.findByApiKey(apiKey);
    
    if (!client) {
      throw new UnauthorizedException('Invalid API key');
    }
    
    if (!client.isActive) {
      throw new UnauthorizedException('Client is inactive');
    }
    
    // Rate limiting
    await this.checkRateLimit(client);
    
    return client;
  }
  
  private async checkRateLimit(client: Client): Promise<void> {
    const key = `rate:${client.id}`;
    const limit = client.rateLimit;
    const window = 60; // 1 minute
    
    const current = await this.redis.incr(key);
    if (current === 1) {
      await this.redis.expire(key, window);
    }
    
    if (current > limit) {
      throw new TooManyRequestsException();
    }
  }
}
```

## Authorization Implementation

### Role-Based Access Control

```typescript
// src/auth/roles.decorator.ts
export const ROLES_KEY = 'roles';
export const Roles = (...roles: string[]) => 
  SetMetadata(ROLES_KEY, roles);

// src/auth/roles.guard.ts
@Injectable()
export class RolesGuard implements CanActivate {
  constructor(
    private reflector: Reflector,
    private userService: UserService
  ) {}
  
  async canActivate(context: ExecutionContext): Promise<boolean> {
    const roles = this.reflector.get<string[]>(
      ROLES_KEY,
      context.getHandler()
    );
    
    if (!roles) {
      return true;
    }
    
    const request = context.switchToHttp().getRequest();
    const user = request.user;
    
    if (!user) {
      throw new UnauthorizedException();
    }
    
    const hasRole = await this.userService.hasRole(user, roles);
    if (!hasRole) {
      throw new ForbiddenException(
        'User does not have required roles'
      );
    }
    
    return true;
  }
}

// Usage in controller
@Controller('agents')
@UseGuards(JwtAuthGuard, RolesGuard)
export class AgentController {
  @Post()
  @Roles('admin', 'agent_manager')
  async create(@Body() input: CreateAgentInput): Promise<Agent> {
    return this.agentService.create(input);
  }
  
  @Get()
  @Roles('admin', 'agent_manager', 'agent_user')
  async list(): Promise<Agent[]> {
    return this.agentService.list();
  }
}
```

### Resource-Based Access Control

```typescript
// src/auth/resource.guard.ts
@Injectable()
export class ResourceGuard implements CanActivate {
  constructor(
    private reflector: Reflector,
    private resourceService: ResourceService
  ) {}
  
  async canActivate(context: ExecutionContext): Promise<boolean> {
    const resourceType = this.reflector.get<string>(
      'resourceType',
      context.getHandler()
    );
    
    if (!resourceType) {
      return true;
    }
    
    const request = context.switchToHttp().getRequest();
    const user = request.user;
    const resourceId = request.params.id;
    
    if (!user || !resourceId) {
      throw new UnauthorizedException();
    }
    
    const resource = await this.resourceService.get(
      resourceType,
      resourceId
    );
    
    const hasAccess = await this.resourceService.hasAccess(
      user,
      resource
    );
    
    if (!hasAccess) {
      throw new ForbiddenException(
        'User does not have access to this resource'
      );
    }
    
    return true;
  }
}

// Usage in controller
@Controller('agents')
@UseGuards(JwtAuthGuard, ResourceGuard)
export class AgentController {
  @Get(':id')
  @SetMetadata('resourceType', 'agent')
  async get(@Param('id') id: string): Promise<Agent> {
    return this.agentService.get(id);
  }
  
  @Put(':id')
  @SetMetadata('resourceType', 'agent')
  async update(
    @Param('id') id: string,
    @Body() input: UpdateAgentInput
  ): Promise<Agent> {
    return this.agentService.update(id, input);
  }
}
```

## Data Protection

### Encryption Service

```typescript
// src/security/encryption.service.ts
@Injectable()
export class EncryptionService {
  private readonly algorithm = 'aes-256-gcm';
  private readonly keyLength = 32;
  private readonly ivLength = 16;
  private readonly saltLength = 64;
  private readonly tagLength = 16;
  
  constructor(
    private configService: ConfigService,
    private redis: Redis
  ) {}
  
  async encrypt(data: string): Promise<string> {
    const key = await this.getEncryptionKey();
    const iv = crypto.randomBytes(this.ivLength);
    const salt = crypto.randomBytes(this.saltLength);
    
    const cipher = crypto.createCipheriv(
      this.algorithm,
      key,
      iv
    );
    
    const encrypted = Buffer.concat([
      cipher.update(data, 'utf8'),
      cipher.final()
    ]);
    
    const tag = cipher.getAuthTag();
    
    // Store salt in Redis with expiration
    await this.redis.set(
      `salt:${salt.toString('hex')}`,
      '1',
      'EX',
      3600 // 1 hour
    );
    
    return Buffer.concat([
      salt,
      iv,
      tag,
      encrypted
    ]).toString('base64');
  }
  
  async decrypt(encrypted: string): Promise<string> {
    const buffer = Buffer.from(encrypted, 'base64');
    
    const salt = buffer.slice(0, this.saltLength);
    const iv = buffer.slice(
      this.saltLength,
      this.saltLength + this.ivLength
    );
    const tag = buffer.slice(
      this.saltLength + this.ivLength,
      this.saltLength + this.ivLength + this.tagLength
    );
    const data = buffer.slice(
      this.saltLength + this.ivLength + this.tagLength
    );
    
    // Verify salt exists
    const exists = await this.redis.exists(
      `salt:${salt.toString('hex')}`
    );
    
    if (!exists) {
      throw new Error('Invalid or expired salt');
    }
    
    const key = await this.getEncryptionKey();
    
    const decipher = crypto.createDecipheriv(
      this.algorithm,
      key,
      iv
    );
    
    decipher.setAuthTag(tag);
    
    return Buffer.concat([
      decipher.update(data),
      decipher.final()
    ]).toString('utf8');
  }
  
  private async getEncryptionKey(): Promise<Buffer> {
    const masterKey = await this.configService.get(
      'ENCRYPTION_KEY'
    );
    
    return crypto.scrypt(
      masterKey,
      'salt',
      this.keyLength
    );
  }
}
```

### Data Access Control

```typescript
// src/security/data-access.service.ts
@Injectable()
export class DataAccessService {
  constructor(
    private encryptionService: EncryptionService,
    private resourceService: ResourceService,
    private auditService: AuditService
  ) {}
  
  async getData(
    user: User,
    resourceType: string,
    resourceId: string
  ): Promise<any> {
    // Check access
    const hasAccess = await this.resourceService.hasAccess(
      user,
      resourceType,
      resourceId
    );
    
    if (!hasAccess) {
      throw new ForbiddenException();
    }
    
    // Get encrypted data
    const encrypted = await this.getEncryptedData(
      resourceType,
      resourceId
    );
    
    // Decrypt data
    const decrypted = await this.encryptionService.decrypt(
      encrypted
    );
    
    // Audit access
    await this.auditService.log({
      user: user.id,
      action: 'read',
      resourceType,
      resourceId,
      timestamp: new Date()
    });
    
    return JSON.parse(decrypted);
  }
  
  async setData(
    user: User,
    resourceType: string,
    resourceId: string,
    data: any
  ): Promise<void> {
    // Check access
    const hasAccess = await this.resourceService.hasAccess(
      user,
      resourceType,
      resourceId,
      'write'
    );
    
    if (!hasAccess) {
      throw new ForbiddenException();
    }
    
    // Encrypt data
    const encrypted = await this.encryptionService.encrypt(
      JSON.stringify(data)
    );
    
    // Store encrypted data
    await this.setEncryptedData(
      resourceType,
      resourceId,
      encrypted
    );
    
    // Audit access
    await this.auditService.log({
      user: user.id,
      action: 'write',
      resourceType,
      resourceId,
      timestamp: new Date()
    });
  }
}
```

## Security Monitoring

### Audit Logging

```typescript
// src/security/audit.service.ts
@Injectable()
export class AuditService {
  constructor(
    private readonly logger: Logger,
    private readonly elasticsearch: Client
  ) {}
  
  async log(entry: AuditEntry): Promise<void> {
    // Log to Elasticsearch
    await this.elasticsearch.index({
      index: 'audit-logs',
      body: {
        ...entry,
        '@timestamp': new Date()
      }
    });
    
    // Log to application logs
    this.logger.log({
      message: 'Audit log entry',
      ...entry
    });
  }
  
  async search(query: AuditQuery): Promise<AuditEntry[]> {
    const response = await this.elasticsearch.search({
      index: 'audit-logs',
      body: {
        query: {
          bool: {
            must: [
              { term: { user: query.user } },
              { term: { action: query.action } },
              { term: { resourceType: query.resourceType } },
              {
                range: {
                  '@timestamp': {
                    gte: query.startDate,
                    lte: query.endDate
                  }
                }
              }
            ]
          }
        },
        sort: [
          { '@timestamp': 'desc' }
        ]
      }
    });
    
    return response.hits.hits.map(hit => hit._source);
  }
}
```

### Security Monitoring

```typescript
// src/security/monitoring.service.ts
@Injectable()
export class SecurityMonitoringService {
  constructor(
    private readonly logger: Logger,
    private readonly prometheus: Registry,
    private readonly alertService: AlertService
  ) {
    this.initializeMetrics();
  }
  
  private initializeMetrics(): void {
    // Failed login attempts
    this.failedLogins = new Counter({
      name: 'security_failed_logins_total',
      help: 'Total number of failed login attempts',
      labelNames: ['user', 'reason']
    });
    
    // API key usage
    this.apiKeyUsage = new Counter({
      name: 'security_api_key_usage_total',
      help: 'Total number of API key usages',
      labelNames: ['client', 'endpoint']
    });
    
    // Resource access
    this.resourceAccess = new Counter({
      name: 'security_resource_access_total',
      help: 'Total number of resource accesses',
      labelNames: ['user', 'resource', 'action']
    });
    
    // Register metrics
    this.prometheus.registerMetric(this.failedLogins);
    this.prometheus.registerMetric(this.apiKeyUsage);
    this.prometheus.registerMetric(this.resourceAccess);
  }
  
  async logFailedLogin(
    user: string,
    reason: string
  ): Promise<void> {
    // Increment metric
    this.failedLogins.inc({ user, reason });
    
    // Log event
    this.logger.warn({
      message: 'Failed login attempt',
      user,
      reason
    });
    
    // Check threshold
    const count = await this.getFailedLoginCount(user);
    if (count >= 5) {
      await this.alertService.alert({
        type: 'security',
        level: 'warning',
        message: `Multiple failed login attempts for user ${user}`,
        details: {
          user,
          count,
          reason
        }
      });
    }
  }
  
  async logApiKeyUsage(
    client: string,
    endpoint: string
  ): Promise<void> {
    // Increment metric
    this.apiKeyUsage.inc({ client, endpoint });
    
    // Log event
    this.logger.debug({
      message: 'API key usage',
      client,
      endpoint
    });
  }
  
  async logResourceAccess(
    user: string,
    resource: string,
    action: string
  ): Promise<void> {
    // Increment metric
    this.resourceAccess.inc({ user, resource, action });
    
    // Log event
    this.logger.debug({
      message: 'Resource access',
      user,
      resource,
      action
    });
  }
}
```

## Best Practices

1. **Authentication**
   - Use JWT for user sessions
   - Use API keys for services
   - Implement rate limiting
   - Rotate keys regularly

2. **Authorization**
   - Use role-based access
   - Use resource-based access
   - Implement least privilege
   - Audit access regularly

3. **Data Protection**
   - Encrypt sensitive data
   - Use secure key management
   - Implement access controls
   - Monitor data access

4. **Security Monitoring**
   - Log security events
   - Monitor metrics
   - Set up alerts
   - Regular audits

## Related Documentation

- [Architecture Guide](../architecture/README)
- [Development Guide](../development/README)
- [Deployment Guide](../deployment/README)
- [Operations Guide](../operations/README)
- [Testing Guide](../testing/README) 
