---
id: shared-VALIDATION_UTILITIES
---
# Validation Utilities

The shared validation utilities provide a comprehensive set of validation functions for ensuring data integrity and consistency across all LLM Platform applications. This module offers powerful tools for validating common data types and formats, allowing developers to enforce data quality standards throughout their applications.

## Installation

The validation utilities are included as part of the shared library. No additional installation is required if you're already using the shared library.

```typescript
// Import the entire validation module
import { validationUtils } from '@bluefly/shared';

// Or import specific validation components
import { 
  string as validateString,
  number as validateNumber,
  object as validateObject,
  array as validateArray,
  domain as validateDomain,
  validate,
  createValidator,
  assertValid
} from '@bluefly/shared/utils/validation-utils';
```

## Key Features

- **Type-specific validation**: Dedicated validation functions for strings, numbers, objects, and arrays
- **Common format validation**: Validate emails, URLs, UUIDs, semantic versions, and more
- **Schema-based validation**: Validate objects against defined schemas
- **Custom validators**: Create custom validation functions
- **Error handling**: Get detailed validation error messages
- **Type safety**: Full TypeScript support

## Basic Usage

### String Validation

```typescript
import { validationUtils } from '@bluefly/shared';

// Check if a string is a valid email
const isValid = validationUtils.string.isEmail('user@example.com');

// Validate a URL
const isValidUrl = validationUtils.string.isUrl('https://bluefly.com/api');

// Check string length
const hasMinLength = validationUtils.string.hasMinLength('password123', 8);

// Validate a UUID
const isValidUuid = validationUtils.string.isUuid('123e4567-e89b-12d3-a456-426614174000');
```

### Number Validation

```typescript
import { validationUtils } from '@bluefly/shared';

// Check if a value is a number
const isNumber = validationUtils.number.isNumber(123);

// Check if a number is within a range
const inRange = validationUtils.number.inRange(25, 0, 100);

// Check if a number is positive
const isPositive = validationUtils.number.isPositive(42);
```

### Object Validation

```typescript
import { validationUtils } from '@bluefly/shared';

// Validate an object against a schema
const config = {
  baseUrl: 'https://api.example.com',
  timeout: 3000,
  retries: 3
};

const result = validationUtils.object.matchesSchema(config, {
  baseUrl: { type: 'string', required: true },
  timeout: { type: 'number', min: 0 },
  retries: { type: 'number', min: 0, max: 10 }
});

if (!result.valid) {
  console.error('Validation errors:', result.errors);
}
```

### Array Validation

```typescript
import { validationUtils } from '@bluefly/shared';

// Check array length
const hasMinItems = validationUtils.array.minLength([1, 2, 3], 2);

// Check for duplicates
const noDuplicates = validationUtils.array.noDuplicates([1, 2, 3, 4]);

// Validate all items in array
const allNumbers = validationUtils.array.allValid(
  [1, 2, 3, 4], 
  (item) => validationUtils.number.isNumber(item)
);

// Validate array of objects against a schema
const users = [
  { id: 1, name: 'John', email: 'john@example.com' },
  { id: 2, name: 'Jane', email: 'jane@example.com' }
];

const validateUsers = validationUtils.array.validateObjects(users, {
  id: { type: 'number', required: true },
  name: { type: 'string', required: true, minLength: 2 },
  email: { 
    type: 'string', 
    required: true,
    custom: (value) => ({
      valid: validationUtils.string.isEmail(value),
      errors: validationUtils.string.isEmail(value) ? undefined : ['Invalid email format']
    })
  }
});
```

### Domain-Specific Validation

```typescript
import { validationUtils } from '@bluefly/shared';

// Validate API key format
const isValidApiKey = validationUtils.domain.isApiKey('api_key_123456789abcdef');

// Validate JWT token format
const isValidJwt = validationUtils.domain.isJwt('eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJzdWIiOiIxMjM0NTY3ODkwIiwibmFtZSI6IkpvaG4gRG9lIiwiaWF0IjoxNTE2MjM5MDIyfQ.SflKxwRJSMeKKF2QT4fwpMeJf36POk6yJV_adQssw5c');

// Validate API configuration
const apiConfig = {
  baseUrl: 'https://api.example.com',
  timeout: 5000,
  retries: 3,
  apiKey: 'api_key_12345'
};

const isValidConfig = validationUtils.domain.isValidApiConfig(apiConfig);
```

### Combining Validators

The library provides tools to combine multiple validators:

```typescript
import { validationUtils } from '@bluefly/shared';

// Create a custom validator by combining multiple validators
const passwordValidator = validationUtils.createValidator<string>(
  (password) => validationUtils.string.hasMinLength(password, 8),
  (password) => /[A-Z]/.test(password), // Has uppercase
  (password) => /[0-9]/.test(password), // Has number
  (password) => /[^A-Za-z0-9]/.test(password) // Has special character
);

// Validate password
const passwordResult = passwordValidator('P@ssw0rd');
if (passwordResult.valid) {
  console.log('Password is valid');
} else {
  console.error('Password validation errors:', passwordResult.errors);
}

// Assert valid (throws error if invalid)
try {
  validationUtils.assertValid(
    'short', 
    [
      (password) => validationUtils.string.hasMinLength(password, 8)
    ], 
    'Password is too short'
  );
} catch (error) {
  console.error(error.message); // "Password is too short: Validation failed at check 1"
}
```

## Creating Custom Validation Schemas

For complex objects, you can create reusable validation schemas:

```typescript
import { validationUtils, ValidationSchema } from '@bluefly/shared';

// Define a user schema
const userSchema: ValidationSchema = {
  id: { type: 'string', required: true },
  username: { type: 'string', required: true, minLength: 3, maxLength: 20 },
  email: { 
    type: 'string', 
    required: true,
    custom: (value) => ({
      valid: validationUtils.string.isEmail(value),
      errors: validationUtils.string.isEmail(value) ? undefined : ['Invalid email format']
    })
  },
  age: { type: 'number', required: false, min: 13 },
  role: { type: 'string', required: false, enum: ['admin', 'user', 'guest'] }
};

// Validate a user object against the schema
const user = {
  id: '12345',
  username: 'john_doe',
  email: 'john@example.com',
  age: 25,
  role: 'user'
};

const result = validationUtils.object.matchesSchema(user, userSchema);
```

## Best Practices

1. **Input Validation**: Always validate user input before processing
2. **Configuration Validation**: Validate configuration objects early in application startup
3. **Error Handling**: Display user-friendly error messages from validation errors
4. **Custom Validators**: Create reusable validation functions for project-specific data types
5. **Schema Definition**: Define schemas for complex data structures used throughout the application

## Integration with Other Modules

The validation utilities work seamlessly with other shared modules:

- **Error Handling**: Validation errors can be converted to standard error objects
- **Logging**: Log validation failures for debugging
- **HTTP Client**: Validate request payloads before sending

Example with error handling:

```typescript
import { validationUtils, createError } from '@bluefly/shared';

function processUserData(userData: any) {
  const result = validationUtils.object.matchesSchema(userData, userSchema);
  
  if (!result.valid) {
    throw createError(
      `Invalid user data: ${result.errors?.join('; ')}`,
      'VALIDATION_ERROR'
    );
  }
  
  // Process valid data...
}
```

## Migration from Legacy Validation

If you're migrating from legacy validation functions, here's how the new functions map to old ones:

| Legacy Function | New Function |
|----------------|--------------|
| `isValidEmail()` | `string.isEmail()` |
| `isValidUrl()` | `string.isUrl()` |
| `isValidSemver()` | `string.isSemver()` |
| `isValidUuid()` | `string.isUuid()` |

The new validation utilities provide backward compatibility, so existing code will continue to work.
