---
id: tdd-enforcement-guide
---
# TDD Enforcement Guide

This guide documents the Test-Driven Development (TDD) enforcement systems in the CI/CD pipeline. The project implements TDD methodology, requiring tests to be written before implementation code.

## Core TDD Principles

1. **RED Phase**: Write failing tests first
2. **GREEN Phase**: Implement minimal code to make tests pass
3. **REFACTOR Phase**: Optimize code while maintaining test coverage

## CI/CD Pipeline Enforcement

The CI/CD pipeline enforces TDD through these mechanisms:

### 1. Test Existence Validation

The pipeline checks that each implementation file has a corresponding test file. This validation runs at the pipeline start to prevent code merges without tests.

```bash
# Run the test existence validation locally
npm run tdd:validate
```

### 2. Test-First Approach Validation

The pipeline verifies git commit history to confirm tests were written before implementation code.

```bash
# Validate the test-first approach locally
npm run tdd:test-first
```

### 3. Coverage Thresholds (95%+)

Code must maintain 95% test coverage across lines, branches, functions, and statements.

```bash
# Run test coverage check locally
npm run test:coverage:check
```

### 4. TDD Compliance Reports

The pipeline generates TDD compliance reports with:
- Test coverage metrics
- TDD compliance metrics
- File-by-file analysis
- Improvement recommendations

```bash
# Generate TDD compliance report locally
npm run tdd:report
```

## Pre-Commit Hooks

Git pre-commit hooks enforce TDD at the local development level:

1. **Test Existence Check**: Blocks commits of implementation files without tests
2. **Test-First Validation**: Confirms tests were written before implementation
3. **Coverage Check**: Verifies coverage thresholds

Install the pre-commit hooks:

```bash
npm run tdd:hooks:install
```

## Running TDD Checks Locally

Before pushing code, run all TDD validations:

```bash
npm run tdd:ci-check
```

This runs the TDD validation suite:
1. Verify tests exist for all files
2. Validate the test-first approach
3. Check test coverage meets 95%+ threshold

## TDD Workflow for New Features

### 1. Create Test Files First

```bash
# Start with a failing test
touch __tests__/my-feature.test.js
```

### 2. Run Tests to See Them Fail (RED Phase)

```bash
npm test
```

### 3. Implement Code to Make Tests Pass (GREEN Phase)

```bash
touch src/my-feature.js
```

### 4. Refactor While Maintaining Tests (REFACTOR Phase)

```bash
npm run test:watch
```

### 5. Verify Coverage

```bash
npm run test:coverage
```

## TDD Compliance Metrics

The TDD compliance report provides metrics on:

1. **Test Coverage**: Percentage of files with corresponding tests
2. **TDD Compliance**: Percentage of files following test-first approach
3. **Code Coverage**: Percentage of code covered by tests
4. **File-by-File Analysis**: Compliance status for each file

## Bypassing TDD Checks (Exceptional Cases Only)

For cases requiring TDD check bypass (legacy code integration, emergency fixes), document the reason:

```bash
git commit --no-verify -m "fix: Emergency security patch (TDD bypass: critical security issue CVE-2023-XXX)"
```

Note: TDD check bypasses should be documented exceptions.

## TDD Enforcement Components

1. **GitLab CI Configuration**: `.gitlab-ci.yml`
2. **TDD Validation Scripts**: `scripts/ci/tdd-validate.sh` and `scripts/ci/validate-test-first.sh`
3. **Pre-Commit Hooks**: `scripts/ci/install-pre-commit-hooks.js`
4. **TDD Compliance Report Generator**: `scripts/ci/generate-tdd-report.js`
5. **Coverage Configuration**: In `package.json` under the `jest` configuration

## TDD Support Resources

- [TDD-ENFORCEMENT-SUMMARY.md](../TDD-ENFORCEMENT-SUMMARY): Current status and recommendations
- [TDD-TASK-TRACKING.md](../TDD-TASK-TRACKING): Tasks for TDD compliance improvement

## Troubleshooting TDD Issues

### Tests Not Being Found

Tests must follow one of these naming patterns:
- `__tests__/file-name.test.js`
- `src/file-name.test.js`
- `test/file-name.test.js`

### Coverage Issues

If coverage is below threshold:
1. Run `npm run test:coverage` to view coverage report
2. Add tests for uncovered lines, branches, and functions
3. Test all edge cases

### Test-First Validation Failures

If test-first validation fails:
1. Check file modification timestamps
2. Ensure tests are committed before implementation
3. Use `git rebase -i` to reorder commits if needed

## Conclusion

The TDD enforcement system requires tests first, maintains coverage thresholds, and enforces development workflow. This approach supports code maintainability and reliability.
