const jwt = require('jsonwebtoken');
const { jwtSecret } = require('../config/auth');
const logger = require('../utils/logger');

/**
 * Authentication middleware to verify JWT tokens
 */
function authenticate(req, res, next) {
  // Get auth header
  const authHeader = req.headers.authorization;
  
  // Check if auth header exists and has the correct format
  if (!authHeader || !authHeader.startsWith('Bearer ')) {
    logger.warn('Authentication failed: No token provided or invalid format');
    return res.status(401).json({
      error: 'unauthorized',
      message: 'Authentication token is required'
    });
  }
  
  // Extract the token
  const token = authHeader.split(' ')[1];
  
  try {
    // Verify the token
    const decoded = jwt.verify(token, jwtSecret);
    
    // Add user info to request object
    req.user = decoded;
    
    // Proceed to the next middleware or route handler
    next();
  } catch (error) {
    logger.warn(`Authentication failed: ${error.message}`);
    
    // Return appropriate error based on JWT error type
    if (error.name === 'TokenExpiredError') {
      return res.status(401).json({
        error: 'token_expired',
        message: 'Authentication token has expired'
      });
    } else {
      return res.status(401).json({
        error: 'invalid_token',
        message: 'Authentication token is invalid or expired'
      });
    }
  }
}

module.exports = {
  authenticate
};