const logger = require('../utils/logger');

/**
 * Global error handler middleware
 * 
 * TODO: Implementation tasks:
 * - [ ] Add error categorization for better reporting
 * - [ ] Implement centralized error logging service
 * - [ ] Create custom error types for business logic errors
 * - [ ] Add support for localized error messages
 * - [ ] Implement graceful handling of database errors
 * - [ ] Add rate limiting error handling
 * - [ ] Run coverage report to verify 95%+ coverage
 */
function errorHandler(err, req, res, next) {
  // Log the error
  logger.error(`Error: ${err.message}`, {
    error: err.toString(),
    stack: err.stack,
    path: req.path,
    method: req.method,
    requestId: req.headers['x-request-id'] || 'unknown'
  });

  // Set default status code and error response
  let statusCode = err.statusCode || 500;
  let errorResponse = {
    error: err.type || 'server_error',
    message: err.message || 'An unexpected error occurred'
  };

  // Add validation details if available
  if (err.details) {
    errorResponse.details = err.details;
  }

  // Don't expose stack trace in production
  if (process.env.NODE_ENV !== 'production' && err.stack) {
    errorResponse.stack = err.stack;
  }

  // Handle specific error types
  if (err.name === 'ValidationError') {
    statusCode = 400;
    errorResponse.error = 'validation_error';
  } else if (err.name === 'UnauthorizedError') {
    statusCode = 401;
    errorResponse.error = 'unauthorized';
  } else if (err.name === 'ForbiddenError') {
    statusCode = 403;
    errorResponse.error = 'forbidden';
  } else if (err.name === 'NotFoundError') {
    statusCode = 404;
    errorResponse.error = 'not_found';
  }

  // Return error response
  res.status(statusCode).json(errorResponse);
}

/**
 * Not found handler middleware for undefined routes
 */
function notFoundHandler(req, res) {
  logger.warn(`Route not found: ${req.method} ${req.originalUrl}`);
  
  res.status(404).json({
    error: 'not_found',
    message: `Route not found: ${req.method} ${req.originalUrl}`
  });
}

// Custom error classes
class ValidationError extends Error {
  constructor(message, details = []) {
    super(message);
    this.name = 'ValidationError';
    this.statusCode = 400;
    this.type = 'validation_error';
    this.details = details;
  }
}

class UnauthorizedError extends Error {
  constructor(message = 'Authentication is required') {
    super(message);
    this.name = 'UnauthorizedError';
    this.statusCode = 401;
    this.type = 'unauthorized';
  }
}

class ForbiddenError extends Error {
  constructor(message = 'Insufficient permissions to access this resource') {
    super(message);
    this.name = 'ForbiddenError';
    this.statusCode = 403;
    this.type = 'forbidden';
  }
}

class NotFoundError extends Error {
  constructor(message = 'The requested resource was not found') {
    super(message);
    this.name = 'NotFoundError';
    this.statusCode = 404;
    this.type = 'not_found';
  }
}

module.exports = {
  errorHandler,
  notFoundHandler,
  ValidationError,
  UnauthorizedError,
  ForbiddenError,
  NotFoundError
};