const { pool } = require('../config/database');
const logger = require('../utils/logger');
const { NotFoundError } = require('../middleware/errorHandler');

/**
 * Service for Model-related database operations
 * 
 * TODO: Implementation tasks:
 * - [ ] Add model creation functionality 
 * - [ ] Implement model version management
 * - [ ] Add batch operations for model updates
 * - [ ] Implement model deletion with validation
 * - [ ] Add caching for frequently accessed models
 * - [ ] Create advanced filtering and search options
 * - [ ] Run coverage report to verify 95%+ coverage
 */
class ModelService {
  /**
   * Get a list of models with optional filtering
   * @param {Object} filters - Filtering options
   * @param {string} [filters.provider] - Filter by provider
   * @param {boolean} [filters.is_active] - Filter by active status
   * @param {number} [filters.limit=20] - Limit results
   * @param {number} [filters.offset=0] - Offset for pagination
   * @returns {Promise<Object>} Models list with pagination info
   */
  async getModels(filters = {}) {
    const { provider, is_active, limit = 20, offset = 0 } = filters;
    
    try {
      // Build WHERE clause for filters
      const whereConditions = [];
      const params = [];
      
      if (provider !== undefined) {
        whereConditions.push('provider = ?');
        params.push(provider);
      }
      
      if (is_active !== undefined) {
        whereConditions.push('is_active = ?');
        params.push(is_active);
      }
      
      const whereClause = whereConditions.length > 0 
        ? `WHERE ${whereConditions.join(' AND ')}` 
        : '';
      
      // Get total count for pagination
      const [countResult] = await pool.query(
        `SELECT COUNT(*) as total FROM models ${whereClause}`,
        params
      );
      const total = countResult[0].total;
      
      // Get models with pagination
      const limitedParams = [...params, limit, offset];
      const [models] = await pool.query(
        `SELECT id, name, provider, base_model, description, capabilities, 
                is_active, created_at, updated_at 
         FROM models 
         ${whereClause} 
         ORDER BY updated_at DESC 
         LIMIT ? OFFSET ?`,
        limitedParams
      );
      
      // Process the models to ensure capabilities is parsed JSON
      const processedModels = models.map(model => ({
        ...model,
        capabilities: typeof model.capabilities === 'string' 
          ? JSON.parse(model.capabilities) 
          : model.capabilities,
        created_at: model.created_at.toISOString(),
        updated_at: model.updated_at.toISOString()
      }));
      
      return {
        models: processedModels,
        pagination: {
          total,
          limit,
          offset
        }
      };
    } catch (error) {
      logger.error('Error getting models:', error);
      throw error;
    }
  }
  
  /**
   * Get a model by ID
   * @param {number} id - Model ID
   * @returns {Promise<Object>} Model details
   * @throws {NotFoundError} If model not found
   */
  async getModelById(id) {
    try {
      const [models] = await pool.query(
        'SELECT * FROM models WHERE id = ?',
        [id]
      );
      
      if (models.length === 0) {
        throw new NotFoundError(`Model with ID ${id} not found`);
      }
      
      const model = models[0];
      
      // Ensure capabilities is parsed JSON
      return {
        ...model,
        capabilities: typeof model.capabilities === 'string' 
          ? JSON.parse(model.capabilities) 
          : model.capabilities,
        created_at: model.created_at.toISOString(),
        updated_at: model.updated_at.toISOString()
      };
    } catch (error) {
      logger.error(`Error getting model by ID ${id}:`, error);
      throw error;
    }
  }
}

module.exports = new ModelService();