#!/usr/bin/env node

/**
 * AI-Powered Documentation Generator
 * 
 * Uses LangChain and available AI providers (Ollama, OpenAI, Anthropic)
 * to automatically generate, enhance, and maintain documentation
 */

const fs = require('fs').promises;
const path = require('path');
const glob = require('glob').promises;
const { ChatOllama } = require('@langchain/community/chat_models/ollama');
const { ChatOpenAI } = require('@langchain/openai');
const { ChatAnthropic } = require('@langchain/anthropic');
const { Document } = require('@langchain/core/documents');
const { RecursiveCharacterTextSplitter } = require('langchain/text_splitter');
const { HNSWLib } = require('@langchain/community/vectorstores/hnswlib');
const { OpenAIEmbeddings } = require('@langchain/openai');
const { OllamaEmbeddings } = require('@langchain/community/embeddings/ollama');

class GitLabDocGenerator {
  constructor(config = {}) {
    this.config = {
      provider: config.provider || 'ollama',
      mode: config.mode || 'enhance',
      filePatterns: config.filePatterns || '**/*.{js,ts,jsx,tsx}',
      outputDir: config.outputDir || 'docs/generated',
      configPath: config.configPath || '~/.ai-config.json',
      chunkSize: config.chunkSize || 2000,
      chunkOverlap: config.chunkOverlap || 200,
      ...config
    };
    
    this.llm = null;
    this.embeddings = null;
    this.vectorStore = null;
    this.processedFiles = [];
    this.report = {
      filesProcessed: 0,
      filesCreated: 0,
      filesEnhanced: 0,
      errors: [],
      startTime: new Date().toISOString()
    };
  }

  async initialize() {
    console.log('🚀 Initializing AI Documentation Generator...');
    
    // Load configuration
    await this.loadConfig();
    
    // Initialize LLM based on provider
    await this.initializeLLM();
    
    // Initialize embeddings for smart search
    await this.initializeEmbeddings();
    
    // Create output directory
    await fs.mkdir(this.config.outputDir, { recursive: true });
  }

  async loadConfig() {
    try {
      const configPath = this.config.configPath.replace('~', process.env.HOME);
      const configContent = await fs.readFile(configPath, 'utf-8');
      const aiConfig = JSON.parse(configContent);
      this.aiConfig = aiConfig;
    } catch (error) {
      console.warn('⚠️  Could not load AI config, using defaults');
      this.aiConfig = { providers: {} };
    }
  }

  async initializeLLM() {
    const provider = this.config.provider;
    
    switch (provider) {
      case 'ollama':
        this.llm = new ChatOllama({
          model: this.aiConfig.providers?.ollama?.model || 'mixtral:latest',
          baseUrl: this.aiConfig.providers?.ollama?.baseUrl || 'http://localhost:11434',
          temperature: 0.3,
        });
        break;
        
      case 'openai':
        if (!process.env.OPENAI_API_KEY && !this.aiConfig.providers?.openai?.apiKey) {
          throw new Error('OpenAI API key not found');
        }
        this.llm = new ChatOpenAI({
          modelName: this.aiConfig.providers?.openai?.model || 'gpt-4',
          temperature: 0.3,
          openAIApiKey: this.aiConfig.providers?.openai?.apiKey || process.env.OPENAI_API_KEY,
        });
        break;
        
      case 'anthropic':
        if (!process.env.ANTHROPIC_API_KEY && !this.aiConfig.providers?.anthropic?.apiKey) {
          throw new Error('Anthropic API key not found');
        }
        this.llm = new ChatAnthropic({
          modelName: this.aiConfig.providers?.anthropic?.model || 'claude-3-haiku-20240307',
          temperature: 0.3,
          anthropicApiKey: this.aiConfig.providers?.anthropic?.apiKey || process.env.ANTHROPIC_API_KEY,
        });
        break;
        
      case 'auto':
        // Try providers in order of preference
        await this.autoSelectProvider();
        break;
        
      default:
        throw new Error(`Unknown provider: ${provider}`);
    }
    
    console.log(`✅ Initialized ${provider} LLM`);
  }

  async autoSelectProvider() {
    // Try Ollama first (free and local)
    try {
      await this.testOllama();
      this.config.provider = 'ollama';
      await this.initializeLLM();
      return;
    } catch (e) {
      console.log('Ollama not available, trying other providers...');
    }
    
    // Try OpenAI if API key exists
    if (process.env.OPENAI_API_KEY || this.aiConfig.providers?.openai?.apiKey) {
      this.config.provider = 'openai';
      await this.initializeLLM();
      return;
    }
    
    // Try Anthropic if API key exists
    if (process.env.ANTHROPIC_API_KEY || this.aiConfig.providers?.anthropic?.apiKey) {
      this.config.provider = 'anthropic';
      await this.initializeLLM();
      return;
    }
    
    throw new Error('No AI provider available. Please install Ollama or provide API keys.');
  }

  async testOllama() {
    const response = await fetch('http://localhost:11434/api/tags');
    if (!response.ok) throw new Error('Ollama not running');
  }

  async initializeEmbeddings() {
    if (this.config.provider === 'ollama') {
      this.embeddings = new OllamaEmbeddings({
        model: 'nomic-embed-text',
        baseUrl: this.aiConfig.providers?.ollama?.baseUrl || 'http://localhost:11434',
      });
    } else {
      this.embeddings = new OpenAIEmbeddings({
        openAIApiKey: this.aiConfig.providers?.openai?.apiKey || process.env.OPENAI_API_KEY,
      });
    }
  }

  async run() {
    await this.initialize();
    
    console.log(`📚 Running in ${this.config.mode} mode...`);
    
    switch (this.config.mode) {
      case 'create':
        await this.createDocumentation();
        break;
      case 'enhance':
        await this.enhanceDocumentation();
        break;
      case 'review':
        await this.reviewDocumentation();
        break;
      case 'translate':
        await this.translateDocumentation();
        break;
      default:
        throw new Error(`Unknown mode: ${this.config.mode}`);
    }
    
    await this.generateReport();
    console.log('✅ Documentation generation complete!');
  }

  async createDocumentation() {
    const files = await this.findFiles();
    console.log(`Found ${files.length} files to document`);
    
    for (const file of files) {
      try {
        await this.documentFile(file);
        this.report.filesProcessed++;
      } catch (error) {
        console.error(`Error documenting ${file}:`, error.message);
        this.report.errors.push({ file, error: error.message });
      }
    }
  }

  async documentFile(filePath) {
    console.log(`📄 Documenting ${filePath}...`);
    
    const content = await fs.readFile(filePath, 'utf-8');
    const ext = path.extname(filePath);
    const language = this.getLanguageFromExt(ext);
    
    // Create prompts based on file type
    const prompts = this.getDocumentationPrompts(language, content, filePath);
    
    // Generate documentation sections
    const sections = {};
    for (const [section, prompt] of Object.entries(prompts)) {
      const response = await this.llm.invoke(prompt);
      sections[section] = response.content;
    }
    
    // Create markdown documentation
    const markdown = this.createMarkdown(filePath, sections, language);
    
    // Save documentation
    const docPath = this.getDocPath(filePath);
    await fs.mkdir(path.dirname(docPath), { recursive: true });
    await fs.writeFile(docPath, markdown);
    
    this.report.filesCreated++;
    console.log(`✅ Created documentation: ${docPath}`);
  }

  getDocumentationPrompts(language, content, filePath) {
    const baseContext = `You are an expert technical writer creating comprehensive documentation.
    File: ${filePath}
    Language: ${language}
    
    Code:
    \`\`\`${language}
    ${content}
    \`\`\`
    `;
    
    return {
      overview: `${baseContext}
      
      Write a comprehensive overview of this code file. Include:
      1. Purpose and main functionality
      2. Key components/classes/functions
      3. Dependencies and imports
      4. Role in the larger system
      
      Format as markdown with clear sections.`,
      
      api: `${baseContext}
      
      Document the API/interface of this code. Include:
      1. All public functions/methods with parameters and return types
      2. Classes and their public members
      3. Exported constants and types
      4. Usage examples for each
      
      Format as markdown with code examples.`,
      
      architecture: `${baseContext}
      
      Explain the architectural patterns and design decisions:
      1. Design patterns used
      2. Data flow and state management
      3. Error handling approach
      4. Performance considerations
      5. Security considerations
      
      Format as markdown with diagrams where helpful (use mermaid syntax).`,
      
      examples: `${baseContext}
      
      Create practical usage examples:
      1. Basic usage example
      2. Advanced usage with all options
      3. Integration example with other components
      4. Common patterns and best practices
      5. What to avoid (anti-patterns)
      
      Format as markdown with runnable code examples.`,
      
      testing: `${baseContext}
      
      Create a testing guide:
      1. What should be tested
      2. Example test cases
      3. Mock/stub requirements
      4. Edge cases to consider
      5. Performance testing suggestions
      
      Format as markdown with example test code.`
    };
  }

  async enhanceDocumentation() {
    console.log('🔍 Scanning for existing documentation to enhance...');
    
    // Find existing markdown files
    const docFiles = await glob(`${this.config.outputDir}/**/*.md`);
    const codeFiles = await this.findFiles();
    
    // Build vector store for intelligent enhancement
    await this.buildVectorStore(codeFiles);
    
    for (const docFile of docFiles) {
      try {
        await this.enhanceDoc(docFile);
        this.report.filesEnhanced++;
      } catch (error) {
        console.error(`Error enhancing ${docFile}:`, error.message);
        this.report.errors.push({ file: docFile, error: error.message });
      }
    }
  }

  async buildVectorStore(files) {
    console.log('🧠 Building vector store for intelligent search...');
    
    const documents = [];
    const splitter = new RecursiveCharacterTextSplitter({
      chunkSize: this.config.chunkSize,
      chunkOverlap: this.config.chunkOverlap,
    });
    
    for (const file of files) {
      const content = await fs.readFile(file, 'utf-8');
      const chunks = await splitter.createDocuments(
        [content],
        [{ source: file, type: 'code' }]
      );
      documents.push(...chunks);
    }
    
    // Create vector store
    this.vectorStore = await HNSWLib.fromDocuments(documents, this.embeddings);
    console.log(`✅ Indexed ${documents.length} code chunks`);
  }

  async enhanceDoc(docPath) {
    console.log(`✨ Enhancing ${docPath}...`);
    
    const content = await fs.readFile(docPath, 'utf-8');
    
    // Find related code using vector search
    const relevantDocs = await this.vectorStore.similaritySearch(content, 5);
    
    const enhancePrompt = `You are enhancing existing documentation with additional insights.
    
    Current Documentation:
    ${content}
    
    Related Code Context:
    ${relevantDocs.map(doc => `File: ${doc.metadata.source}\n${doc.pageContent}`).join('\n---\n')}
    
    Enhance this documentation by:
    1. Adding any missing important details from the code
    2. Updating examples to match current implementation
    3. Adding cross-references to related components
    4. Improving clarity and organization
    5. Adding diagrams where helpful (mermaid syntax)
    6. Ensuring accuracy with the actual code
    
    Return the enhanced documentation in markdown format.
    Preserve the overall structure but improve and expand content.`;
    
    const response = await this.llm.invoke(enhancePrompt);
    
    // Save enhanced documentation
    await fs.writeFile(docPath, response.content);
    console.log(`✅ Enhanced documentation saved`);
  }

  async reviewDocumentation() {
    console.log('🔍 Reviewing documentation for issues...');
    
    const docFiles = await glob(`${this.config.outputDir}/**/*.md`);
    const reviews = [];
    
    for (const docFile of docFiles) {
      const review = await this.reviewDoc(docFile);
      reviews.push(review);
    }
    
    // Generate review report
    const reportPath = path.join(this.config.outputDir, 'review-report.md');
    await this.generateReviewReport(reviews, reportPath);
  }

  async reviewDoc(docPath) {
    const content = await fs.readFile(docPath, 'utf-8');
    
    const reviewPrompt = `You are a technical documentation reviewer.
    
    Review this documentation:
    ${content}
    
    Check for:
    1. Accuracy and completeness
    2. Clarity and readability
    3. Proper formatting and structure
    4. Missing sections or information
    5. Outdated information
    6. Grammar and spelling
    7. Code example correctness
    
    Provide a structured review with:
    - Overall score (1-10)
    - Issues found (categorized by severity)
    - Specific improvement suggestions
    - What's done well
    
    Format as JSON.`;
    
    const response = await this.llm.invoke(reviewPrompt);
    
    try {
      return {
        file: docPath,
        review: JSON.parse(response.content)
      };
    } catch (e) {
      return {
        file: docPath,
        review: { error: 'Could not parse review', raw: response.content }
      };
    }
  }

  async findFiles() {
    const patterns = this.config.filePatterns.split(',');
    const files = [];
    
    for (const pattern of patterns) {
      const matched = await glob(pattern.trim(), {
        ignore: ['**/node_modules/**', '**/dist/**', '**/build/**', '**/*.test.*', '**/*.spec.*']
      });
      files.push(...matched);
    }
    
    return files;
  }

  getLanguageFromExt(ext) {
    const langMap = {
      '.js': 'javascript',
      '.jsx': 'javascript',
      '.ts': 'typescript', 
      '.tsx': 'typescript',
      '.php': 'php',
      '.py': 'python',
      '.java': 'java',
      '.go': 'go',
      '.rs': 'rust',
      '.rb': 'ruby',
      '.swift': 'swift',
      '.kt': 'kotlin',
      '.scala': 'scala',
      '.r': 'r',
      '.cpp': 'cpp',
      '.c': 'c',
      '.cs': 'csharp',
      '.lua': 'lua',
      '.dart': 'dart',
      '.elm': 'elm',
      '.ex': 'elixir'
    };
    
    return langMap[ext] || 'text';
  }

  getDocPath(codePath) {
    const relativePath = path.relative(process.cwd(), codePath);
    const parsed = path.parse(relativePath);
    const docName = `${parsed.name}.md`;
    
    return path.join(this.config.outputDir, parsed.dir, docName);
  }

  createMarkdown(filePath, sections, language) {
    const relativePath = path.relative(process.cwd(), filePath);
    
    return `# ${path.basename(filePath)}

**File**: \`${relativePath}\`  
**Language**: ${language}  
**Generated**: ${new Date().toISOString()}  
**AI Model**: ${this.config.provider}  

## Overview

${sections.overview}

## API Reference

${sections.api}

## Architecture

${sections.architecture}

## Usage Examples

${sections.examples}

## Testing Guide

${sections.testing}

---

*This documentation was automatically generated by the AI Documentation Generator.*  
*Provider: ${this.config.provider} | Mode: ${this.config.mode}*
`;
  }

  async generateReport() {
    this.report.endTime = new Date().toISOString();
    this.report.duration = new Date(this.report.endTime) - new Date(this.report.startTime);
    
    const reportPath = path.join(this.config.outputDir, 'report.json');
    await fs.writeFile(reportPath, JSON.stringify(this.report, null, 2));
    
    console.log('\n📊 Documentation Generation Report:');
    console.log(`Files Processed: ${this.report.filesProcessed}`);
    console.log(`Files Created: ${this.report.filesCreated}`);
    console.log(`Files Enhanced: ${this.report.filesEnhanced}`);
    console.log(`Errors: ${this.report.errors.length}`);
    console.log(`Duration: ${Math.round(this.report.duration / 1000)}s`);
  }

  async generateReviewReport(reviews, reportPath) {
    const summary = {
      totalFiles: reviews.length,
      averageScore: 0,
      criticalIssues: 0,
      warnings: 0,
      suggestions: 0
    };
    
    let reportMd = `# Documentation Review Report

Generated: ${new Date().toISOString()}

## Summary

`;
    
    reviews.forEach(r => {
      if (r.review.score) {
        summary.averageScore += r.review.score;
      }
    });
    
    summary.averageScore = summary.averageScore / reviews.length;
    
    reportMd += `- **Files Reviewed**: ${summary.totalFiles}
- **Average Score**: ${summary.averageScore.toFixed(1)}/10
- **Critical Issues**: ${summary.criticalIssues}
- **Warnings**: ${summary.warnings}
- **Suggestions**: ${summary.suggestions}

## Detailed Reviews

`;
    
    reviews.forEach(r => {
      reportMd += `### ${r.file}

**Score**: ${r.review.score || 'N/A'}/10

`;
      
      if (r.review.issues) {
        reportMd += `**Issues**:\n`;
        Object.entries(r.review.issues).forEach(([severity, issues]) => {
          reportMd += `- **${severity}**: ${issues.length} issues\n`;
          issues.forEach(issue => {
            reportMd += `  - ${issue}\n`;
          });
        });
      }
      
      reportMd += '\n---\n\n';
    });
    
    await fs.writeFile(reportPath, reportMd);
    console.log(`✅ Review report saved to ${reportPath}`);
  }
}

// Export for use in CI/CD
module.exports = { GitLabDocGenerator };

// Run if called directly
if (require.main === module) {
  const generator = new GitLabDocGenerator({
    provider: process.env.AI_PROVIDER || 'ollama',
    mode: process.env.AI_MODE || 'enhance',
    filePatterns: process.env.FILE_PATTERNS || '**/*.{js,ts,jsx,tsx}',
    outputDir: process.env.DOC_OUTPUT_DIR || 'docs/generated'
  });
  
  generator.run().catch(err => {
    console.error('❌ Error:', err);
    process.exit(1);
  });
}