#!/usr/bin/env node
/**
 * Install Git Pre-Commit Hooks for TDD Enforcement
 * 
 * This script installs pre-commit hooks that enforce TDD methodology:
 * 1. Validate that tests exist for all modified files
 * 2. Ensure tests are written before implementation (based on file timestamps)
 * 3. Check test coverage for modified files
 *
 * TODO: Implementation tasks:
 * - [ ] Create contributor guidelines emphasizing TDD
 * - [ ] Document testing approach for each component
 * - [ ] Create test examples for new contributors
 * - [ ] Document CI/CD integration with TDD
 * - [ ] Implement auto-installation of hooks during setup
 * - [ ] Add support for advanced coverage checks
 * - [ ] Create interactive tutorial for TDD workflow
 */

const fs = require('fs');
const path = require('path');
const { execSync } = require('child_process');

const HOOKS_DIR = path.resolve(process.cwd(), '.git/hooks');
const PRE_COMMIT_HOOK = path.join(HOOKS_DIR, 'pre-commit');
const TDD_HOOK_SCRIPT = path.resolve(process.cwd(), 'scripts/ci/pre-commit-tdd.sh');

// Create hooks directory if it doesn't exist
if (!fs.existsSync(HOOKS_DIR)) {
  console.log('Creating hooks directory...');
  fs.mkdirSync(HOOKS_DIR, { recursive: true });
}

// Create TDD pre-commit hook script
console.log('Creating TDD pre-commit hook script...');
const preCommitScript = `#!/bin/bash
# Pre-commit hook for TDD enforcement
# Created by install-pre-commit-hooks.js

BLUE='\033[0;34m'
GREEN='\033[0;32m'
RED='\033[0;31m'
YELLOW='\033[1;33m'
NC='\033[0m' # No Color

echo -e "\${BLUE}======================================================\${NC}"
echo -e "\${BLUE}      RUNNING TDD ENFORCEMENT PRE-COMMIT HOOK        \${NC}"
echo -e "\${BLUE}======================================================\${NC}"
echo ""

# Get staged JS/TS files
STAGED_FILES=$(git diff --cached --name-only --diff-filter=ACM | grep -E '\\.(js|jsx|ts|tsx)$' | grep -v '\\.test\\.' || true)

if [ -z "$STAGED_FILES" ]; then
  echo -e "\${GREEN}No JavaScript/TypeScript implementation files staged for commit.\${NC}"
  exit 0
fi

echo -e "\${YELLOW}Checking TDD compliance for staged files:\${NC}"
echo "$STAGED_FILES"
echo ""

FAILED=0

# Check if there are corresponding test files
for FILE in $STAGED_FILES; do
  FILE_DIR=$(dirname "$FILE")
  FILE_BASE=$(basename "$FILE" | sed -E 's/\\.(js|ts|jsx|tsx)$//')
  
  # Check for test files in various common test locations
  POSSIBLE_TEST_PATHS=(
    "$FILE_DIR/__tests__/${FILE_BASE}.test.js"
    "$FILE_DIR/__tests__/${FILE_BASE}.test.ts"
    "__tests__/$FILE_DIR/${FILE_BASE}.test.js"
    "__tests__/$FILE_DIR/${FILE_BASE}.test.ts"
    "__tests__/${FILE_BASE}.test.js"
    "__tests__/${FILE_BASE}.test.ts"
    "test/$FILE_DIR/${FILE_BASE}.test.js"
    "test/$FILE_DIR/${FILE_BASE}.test.ts"
    "tests/$FILE_DIR/${FILE_BASE}.test.js"
    "tests/$FILE_DIR/${FILE_BASE}.test.ts"
    "$FILE_DIR/${FILE_BASE}.test.js"
    "$FILE_DIR/${FILE_BASE}.test.ts"
  )
  
  TEST_EXISTS=0
  TEST_FILE=""
  
  for TEST_PATH in "\${POSSIBLE_TEST_PATHS[@]}"; do
    if [ -f "$TEST_PATH" ]; then
      TEST_EXISTS=1
      TEST_FILE="$TEST_PATH"
      break
    fi
  done
  
  if [ $TEST_EXISTS -eq 0 ]; then
    echo -e "\${RED}✗ No test found for:\${NC} $FILE"
    echo -e "  \${YELLOW}Expected test locations:\${NC}"
    echo -e "  - $FILE_DIR/__tests__/${FILE_BASE}.test.js"
    echo -e "  - __tests__/$FILE_DIR/${FILE_BASE}.test.js"
    echo -e "  - __tests__/${FILE_BASE}.test.js"
    echo -e "  - $FILE_DIR/${FILE_BASE}.test.js"
    FAILED=1
  else
    echo -e "\${GREEN}✓ Test found:\${NC} $TEST_FILE"
    
    # Check if test is also staged
    if ! git diff --cached --name-only | grep -q "$TEST_FILE"; then
      # Test file exists but is not staged - check if it was created after the implementation
      if [ -f "$TEST_FILE" ] && [ -f "$FILE" ]; then
        IMPL_MTIME=$(stat -c %Y "$FILE" 2>/dev/null || stat -f %m "$FILE")
        TEST_MTIME=$(stat -c %Y "$TEST_FILE" 2>/dev/null || stat -f %m "$TEST_FILE")
        
        if [ $TEST_MTIME -gt $IMPL_MTIME ]; then
          echo -e "\${YELLOW}⚠ Warning: Test file was modified after implementation.\${NC}"
          echo -e "  \${YELLOW}For true TDD, tests should be written before implementation.\${NC}"
          echo -e "  \${YELLOW}Consider including test changes in this commit.\${NC}"
        fi
      fi
    fi
  fi
done

# Show helpful message if failures
if [ $FAILED -ne 0 ]; then
  echo ""
  echo -e "\${RED}TDD VALIDATION FAILED!\${NC}"
  echo -e "\${RED}Please ensure all implementation files have corresponding test files.\${NC}"
  echo -e "\${YELLOW}Remember: In TDD, tests must be written before implementation.\${NC}"
  echo ""
  echo -e "To bypass this check (not recommended), use: git commit --no-verify"
  exit 1
fi

echo ""
echo -e "\${GREEN}TDD VALIDATION PASSED!\${NC}"
exit 0
`;

fs.writeFileSync(TDD_HOOK_SCRIPT, preCommitScript);
execSync(`chmod +x ${TDD_HOOK_SCRIPT}`);

// Install the pre-commit hook
console.log('Installing pre-commit hook...');
const hookContent = `#!/bin/bash
# TDD enforcement pre-commit hook
# This is a wrapper that calls the TDD enforcement script

${TDD_HOOK_SCRIPT}
`;

fs.writeFileSync(PRE_COMMIT_HOOK, hookContent);
execSync(`chmod +x ${PRE_COMMIT_HOOK}`);

console.log('Pre-commit hooks installed successfully!');
console.log(`You can find the pre-commit hook at: ${PRE_COMMIT_HOOK}`);
console.log(`You can find the TDD hook script at: ${TDD_HOOK_SCRIPT}`);
console.log('\nTo bypass the TDD checks in special cases, use: git commit --no-verify');