#!/bin/bash
# TDD Validation Script
# Validates that tests exist for all implementation files
#
# TODO: Implementation tasks:
# - [ ] Configure CI to enforce TDD workflow
# - [ ] Create test coverage reports as part of CI pipeline
# - [ ] Configure test coverage thresholds (95%+)
# - [ ] Add test report visualization
# - [ ] Support multi-language test detection
# - [ ] Implement granular reporting of coverage
# - [ ] Add historical compliance tracking

set -e

BLUE='\033[0;34m'
GREEN='\033[0;32m'
RED='\033[0;31m'
YELLOW='\033[1;33m'
NC='\033[0m' # No Color

echo -e "${BLUE}======================================================${NC}"
echo -e "${BLUE}           TDD VALIDATION CHECK                       ${NC}"
echo -e "${BLUE}======================================================${NC}"
echo ""

# Get all modified files in the current branch/PR
if [ -n "$CI_MERGE_REQUEST_TARGET_BRANCH_NAME" ]; then
  # Running in a merge request
  TARGET_BRANCH="$CI_MERGE_REQUEST_TARGET_BRANCH_NAME"
else
  # Running on a branch
  TARGET_BRANCH="origin/main"
fi

echo -e "${YELLOW}Comparing against:${NC} $TARGET_BRANCH"
echo ""

# Get all modified JS/TS files that are not test files
MODIFIED_FILES=$(git diff --name-only "$TARGET_BRANCH" | grep -E '\.jsx?$|\.tsx?$' | grep -v '\.test\.' || true)

if [ -z "$MODIFIED_FILES" ]; then
  echo -e "${GREEN}No implementation files were modified. TDD check passed!${NC}"
  exit 0
fi

echo -e "${YELLOW}Checking test files for these implementation files:${NC}"
echo "$MODIFIED_FILES"
echo ""

FAILED=0
PASSED=0
TOTAL=0

for FILE in $MODIFIED_FILES; do
  TOTAL=$((TOTAL + 1))
  
  # Determine the expected test file path
  FILE_DIR=$(dirname "$FILE")
  FILE_BASE=$(basename "$FILE" | sed -E 's/\.(js|ts|jsx|tsx)$//')
  
  # Check for test files in various common test locations
  TEST_EXISTS=0
  
  # Test files in __tests__ directory (Jest standard)
  POSSIBLE_TEST_PATHS=(
    "$FILE_DIR/__tests__/${FILE_BASE}.test.js"
    "$FILE_DIR/__tests__/${FILE_BASE}.test.ts"
    "__tests__/$FILE_DIR/${FILE_BASE}.test.js"
    "__tests__/$FILE_DIR/${FILE_BASE}.test.ts"
    "__tests__/${FILE_BASE}.test.js"
    "__tests__/${FILE_BASE}.test.ts"
    "test/$FILE_DIR/${FILE_BASE}.test.js"
    "test/$FILE_DIR/${FILE_BASE}.test.ts"
    "tests/$FILE_DIR/${FILE_BASE}.test.js"
    "tests/$FILE_DIR/${FILE_BASE}.test.ts"
    "$FILE_DIR/${FILE_BASE}.test.js"
    "$FILE_DIR/${FILE_BASE}.test.ts"
  )
  
  for TEST_PATH in "${POSSIBLE_TEST_PATHS[@]}"; do
    if [ -f "$TEST_PATH" ]; then
      TEST_EXISTS=1
      echo -e "${GREEN}✓ Test found:${NC} $TEST_PATH"
      break
    fi
  done
  
  if [ $TEST_EXISTS -eq 0 ]; then
    echo -e "${RED}✗ No test found for:${NC} $FILE"
    echo -e "  ${YELLOW}Expected test locations:${NC}"
    echo -e "  - $FILE_DIR/__tests__/${FILE_BASE}.test.js"
    echo -e "  - __tests__/$FILE_DIR/${FILE_BASE}.test.js"
    echo -e "  - __tests__/${FILE_BASE}.test.js"
    echo -e "  - $FILE_DIR/${FILE_BASE}.test.js"
    FAILED=$((FAILED + 1))
  else
    PASSED=$((PASSED + 1))
  fi
done

echo ""
echo -e "${BLUE}======================================================${NC}"
echo -e "${BLUE}                TDD VALIDATION RESULTS                ${NC}"
echo -e "${BLUE}======================================================${NC}"
echo -e "Total files checked: ${TOTAL}"
echo -e "Passed: ${GREEN}${PASSED}${NC}"
echo -e "Failed: ${RED}${FAILED}${NC}"
echo ""

if [ $FAILED -gt 0 ]; then
  echo -e "${RED}TDD VALIDATION FAILED!${NC}"
  echo -e "${RED}Please ensure all implementation files have corresponding test files.${NC}"
  echo -e "${YELLOW}Remember: In TDD, tests must be written before implementation.${NC}"
  exit 1
else
  echo -e "${GREEN}TDD VALIDATION PASSED!${NC}"
  echo -e "${GREEN}All implementation files have corresponding test files.${NC}"
  exit 0
fi