#!/usr/bin/env node

/**
 * Claude Directory Documentation Generator
 * 
 * This script automates the creation and maintenance of documentation
 * for the Bluefly AI Platform, with a focus on CLAUDE.md integration.
 */

const fs = require('fs');
const path = require('path');
const { execSync } = require('child_process');

// Configuration
const PROJECT_ROOT = process.cwd();
const CLAUDE_DIR = path.join(PROJECT_ROOT, '.claude');
const DOCS_DIR = path.join(PROJECT_ROOT, 'docs');
const README_PATH = path.join(CLAUDE_DIR, 'README.md');
const DOCS_README_PATH = path.join(DOCS_DIR, 'README.md');
const TEMPLATES_DIR = path.join(__dirname, 'templates');
const CLAUDE_MD_PATH = path.join(PROJECT_ROOT, 'CLAUDE.md');
const DATE_FORMAT = new Intl.DateTimeFormat('en-US', { 
  year: 'numeric', 
  month: 'long', 
  day: 'numeric' 
});

// Ensure the .claude directory exists
if (!fs.existsSync(CLAUDE_DIR)) {
  fs.mkdirSync(CLAUDE_DIR, { recursive: true });
  console.log(`Created directory: ${CLAUDE_DIR}`);
}

// Ensure the docs directory exists
if (!fs.existsSync(DOCS_DIR)) {
  fs.mkdirSync(DOCS_DIR, { recursive: true });
  console.log(`Created directory: ${DOCS_DIR}`);
}

// Command line arguments
const args = process.argv.slice(2);
const command = args[0];

if (!command) {
  printUsage();
  process.exit(1);
}

// Execute the appropriate command
switch (command) {
  case 'create':
    createDocFile(args[1], args[2], args[3], args[4]);
    break;
  case 'update':
    updateDocFile(args[1], args[2]);
    break;
  case 'validate':
    validateDocStructure(args[1]);
    break;
  case 'sync':
    syncFromClaude(args[1]);
    break;
  case 'index':
    updateReadmeIndex(args[1]);
    break;
  case 'db-schema':
    createDbSchemaDoc(args[1], args[2]);
    break;
  case 'api-doc':
    createApiDoc(args[1], args[2]);
    break;
  case 'help':
    printUsage();
    break;
  default:
    console.error(`Unknown command: ${command}`);
    printUsage();
    process.exit(1);
}

/**
 * Prints usage information
 */
function printUsage() {
  console.log(`
Claude Documentation Generator

Usage:
  node docgen-claude.js <command> [options]

Commands:
  create <name> [category] [template] [location]  Create a new documentation file
  update <name> [location]                        Update an existing documentation file
  validate [location]                             Validate the documentation structure
  sync [location]                                 Sync content from CLAUDE.md to target directory
  index [location]                                Update the README.md index
  db-schema <name> [category]                     Create a database schema documentation file
  api-doc <name> [category]                       Create an API documentation file
  help                                            Show this help message

Options:
  name        The name of the documentation file
  category    The category for the documentation file
  template    The template to use (standard, architecture, db_schema, api)
  location    The target location (.claude or docs)

Examples:
  node docgen-claude.js create api_guidelines "Development Standards"
  node docgen-claude.js create service_architecture "Core Documentation" architecture
  node docgen-claude.js create user_database "Database Schemas" db_schema docs
  node docgen-claude.js update testing_requirements
  node docgen-claude.js validate
  node docgen-claude.js sync
  node docgen-claude.js index docs
  node docgen-claude.js db-schema user_database "Database Schemas"
  node docgen-claude.js api-doc user_api "API Documentation"
`);
}

/**
 * Creates a new documentation file
 * @param {string} name - The name of the documentation file
 * @param {string} category - The category for the documentation file
 * @param {string} templateName - Optional template name (defaults to "standard")
 * @param {string} location - Target location (.claude or docs)
 */
function createDocFile(name, category, templateName = 'standard', location = '.claude') {
  if (!name) {
    console.error('Error: Missing file name');
    printUsage();
    process.exit(1);
  }

  // Determine target directory
  const targetDir = location === 'docs' ? DOCS_DIR : CLAUDE_DIR;
  
  // Create category directory if it exists and we're in docs mode
  let finalTargetDir = targetDir;
  if (location === 'docs' && category) {
    const categoryDir = path.join(targetDir, category.toLowerCase().replace(/\s+/g, '_'));
    if (!fs.existsSync(categoryDir)) {
      fs.mkdirSync(categoryDir, { recursive: true });
      console.log(`Created category directory: ${categoryDir}`);
      
      // Create README.md in the category directory if it doesn't exist
      const categoryReadmePath = path.join(categoryDir, 'README.md');
      if (!fs.existsSync(categoryReadmePath)) {
        const categoryReadmeContent = `# ${category}\n\nThis directory contains documentation for ${category.toLowerCase()} in the Bluefly AI Platform.\n\n## Documents\n\n`;
        fs.writeFileSync(categoryReadmePath, categoryReadmeContent);
        console.log(`Created category README.md at ${categoryReadmePath}`);
      }
    }
    finalTargetDir = categoryDir;
  }

  // Convert name to proper file format
  const fileName = name.toLowerCase().replace(/\s+/g, '_') + '.md';
  const filePath = path.join(finalTargetDir, fileName);

  // Check if file already exists
  if (fs.existsSync(filePath)) {
    console.error(`Error: File ${fileName} already exists at ${filePath}`);
    process.exit(1);
  }

  // Load template
  const templatePath = path.join(TEMPLATES_DIR, `${templateName}.md`);
  if (!fs.existsSync(templatePath)) {
    console.error(`Error: Template file not found: ${templatePath}`);
    console.log(`Available templates: ${fs.readdirSync(TEMPLATES_DIR).filter(f => f.endsWith('.md') && f !== 'README.md').join(', ')}`);
    process.exit(1);
  }

  let template = fs.readFileSync(templatePath, 'utf8');

  // Replace placeholders
  const title = name.split('_').map(word => word.charAt(0).toUpperCase() + word.slice(1)).join(' ');
  
  template = template
    .replace(/{{TITLE}}/g, title)
    .replace(/{{TOPIC}}/g, title.toLowerCase())
    .replace(/{{OVERVIEW_PLACEHOLDER}}/g, 'Add an overview of this topic here.')
    .replace(/{{COMPONENTS_PLACEHOLDER}}/g, '- Component 1\n- Component 2\n- Component 3')
    .replace(/{{DETAILS_PLACEHOLDER}}/g, 'Add detailed information here.')
    .replace(/{{LANGUAGE}}/g, 'javascript')
    .replace(/{{CODE_EXAMPLE_PLACEHOLDER}}/g, '// Add code example here\nconst example = "This is an example";\nconsole.log(example);')
    .replace(/{{BEST_PRACTICES_PLACEHOLDER}}/g, '1. Best practice 1\n2. Best practice 2\n3. Best practice 3')
    .replace(/{{RELATED_DOC_1}}/g, 'Related Document 1')
    .replace(/{{RELATED_DOC_1_PATH}}/g, './related_document_1.md')
    .replace(/{{RELATED_DOC_2}}/g, 'Related Document 2')
    .replace(/{{RELATED_DOC_2_PATH}}/g, './related_document_2.md')
    .replace(/{{REFERENCES_PLACEHOLDER}}/g, '- Reference 1\n- Reference 2\n- Reference 3')
    .replace(/{{VERSION}}/g, '1.0.0')
    .replace(/{{LAST_UPDATED}}/g, DATE_FORMAT.format(new Date()))
    .replace(/{{STATUS}}/g, 'Draft');

  // Architecture-specific replacements
  if (templateName === 'architecture') {
    template = template
      .replace(/{{DIAGRAM_PLACEHOLDER}}/g, 
`┌───────────────────┐     ┌───────────────────┐
│                   │     │                   │
│  Component 1      │────►│  Component 2      │
│                   │     │                   │
└───────────────────┘     └───────────────────┘
                              │
                              ▼
                          ┌───────────────────┐
                          │                   │
                          │  Component 3      │
                          │                   │
                          └───────────────────┘`)
      .replace(/{{RESPONSIBILITIES_PLACEHOLDER}}/g, '- Component 1: Responsible for X\n- Component 2: Responsible for Y\n- Component 3: Responsible for Z')
      .replace(/{{DATA_FLOW_PLACEHOLDER}}/g, '1. Data flows from Component 1 to Component 2\n2. Component 2 processes the data\n3. Component 2 sends processed data to Component 3')
      .replace(/{{INTEGRATION_POINTS_PLACEHOLDER}}/g, '- Component 1 integrates with External System A\n- Component 2 integrates with Database B\n- Component 3 integrates with API C')
      .replace(/{{DEPLOYMENT_PLACEHOLDER}}/g, '- Component 1: Deployed as a Docker container\n- Component 2: Deployed as a Kubernetes pod\n- Component 3: Deployed as a serverless function')
      .replace(/{{SECURITY_PLACEHOLDER}}/g, '- All communication is encrypted using TLS\n- Authentication is handled via JWT\n- Authorization uses RBAC')
      .replace(/{{PERFORMANCE_PLACEHOLDER}}/g, '- Component 1: Handles 1000 requests/second\n- Component 2: Processes data in <50ms\n- Component 3: Scales horizontally for increased load')
      .replace(/{{SCALABILITY_PLACEHOLDER}}/g, '- Horizontal scaling using Kubernetes\n- Auto-scaling based on CPU and memory metrics\n- Load balancing using ingress controller')
      .replace(/{{MONITORING_PLACEHOLDER}}/g, '- Prometheus metrics collection\n- Grafana dashboards for visualization\n- Alerting via AlertManager')
      .replace(/{{ERROR_HANDLING_PLACEHOLDER}}/g, '- Circuit breaker pattern for external services\n- Retry mechanism for transient failures\n- Fallback mechanisms for degraded operation')
      .replace(/{{CONFIGURATION_PLACEHOLDER}}/g, '- Configuration via environment variables\n- Secrets managed via Kubernetes secrets\n- Feature flags for controlled rollout');
  }

  // Database schema specific replacements
  if (templateName === 'db_schema') {
    template = template
      .replace(/{{DIAGRAM_PLACEHOLDER}}/g,
`┌───────────────────┐     ┌───────────────────┐
│                   │     │                   │
│  Table1           │─┬──►│  Table2           │
│                   │ │   │                   │
└───────────────────┘ │   └───────────────────┘
                      │      │
                      │      ▼
                      │   ┌───────────────────┐
                      └──►│                   │
                          │  Table3           │
                          │                   │
                          └───────────────────┘`)
      .replace(/{{TABLE1_NAME}}/g, 'Users')
      .replace(/{{TABLE1_DESCRIPTION}}/g, 'Stores user account information.')
      .replace(/{{TABLE1_COLUMNS}}/g, 
`| id | INTEGER | PRIMARY KEY, AUTO_INCREMENT | Unique identifier for the user |
| username | VARCHAR(50) | NOT NULL, UNIQUE | User's login name |
| email | VARCHAR(100) | NOT NULL, UNIQUE | User's email address |
| password_hash | VARCHAR(255) | NOT NULL | Hashed user password |
| created_at | TIMESTAMP | NOT NULL, DEFAULT CURRENT_TIMESTAMP | Account creation timestamp |
| updated_at | TIMESTAMP | NOT NULL, DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP | Last update timestamp |
| status | ENUM('active', 'inactive', 'suspended') | NOT NULL, DEFAULT 'active' | Current account status |`)
      .replace(/{{TABLE1_INDEXES}}/g, 
`- PRIMARY KEY (id)
- UNIQUE INDEX idx_username (username)
- UNIQUE INDEX idx_email (email)
- INDEX idx_status (status)`)
      .replace(/{{TABLE2_NAME}}/g, 'Profiles')
      .replace(/{{TABLE2_DESCRIPTION}}/g, 'Stores additional user profile information.')
      .replace(/{{TABLE2_COLUMNS}}/g, 
`| id | INTEGER | PRIMARY KEY, AUTO_INCREMENT | Unique identifier for the profile |
| user_id | INTEGER | NOT NULL, FOREIGN KEY | Reference to users.id |
| first_name | VARCHAR(50) | NULL | User's first name |
| last_name | VARCHAR(50) | NULL | User's last name |
| bio | TEXT | NULL | User's biography |
| avatar_url | VARCHAR(255) | NULL | URL to user's avatar image |
| created_at | TIMESTAMP | NOT NULL, DEFAULT CURRENT_TIMESTAMP | Profile creation timestamp |
| updated_at | TIMESTAMP | NOT NULL, DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP | Last update timestamp |`)
      .replace(/{{TABLE2_INDEXES}}/g, 
`- PRIMARY KEY (id)
- FOREIGN KEY fk_user_profile (user_id) REFERENCES users(id) ON DELETE CASCADE
- INDEX idx_user_id (user_id)`)
      .replace(/{{RELATIONSHIPS_PLACEHOLDER}}/g, 
`- One-to-One: Each Users record has exactly one corresponding Profiles record
- One-to-Many: Each Users record can have multiple records in other tables (like Posts, Comments, etc.)
- Many-to-Many: Users can belong to multiple Groups and each Group can have multiple Users (via a junction table)`)
      .replace(/{{DATA_FLOW_PLACEHOLDER}}/g, 
`1. User registration creates a new record in the Users table
2. Profile creation adds a record to the Profiles table with the corresponding user_id
3. Authentication queries the Users table to validate credentials
4. Profile updates modify the corresponding record in the Profiles table`)
      .replace(/{{MIGRATION_PLACEHOLDER}}/g, 
`- Add new columns with NULL constraint first, then populate data, then add NOT NULL constraints if needed
- Use transaction-safe operations for schema changes
- Schedule migrations during low-traffic periods
- Create backup before significant schema changes
- Consider using tools like Liquibase or Flyway for migration management`)
      .replace(/{{QUERY_EXAMPLE_PLACEHOLDER}}/g, 
`-- Get user with profile information
SELECT u.id, u.username, u.email, p.first_name, p.last_name, p.bio
FROM users u
JOIN profiles p ON u.id = p.user_id
WHERE u.status = 'active'
ORDER BY u.created_at DESC
LIMIT 10;

-- Update user status
UPDATE users
SET status = 'suspended'
WHERE id = 123;`)
      .replace(/{{PERFORMANCE_PLACEHOLDER}}/g, 
`- Keep indexed columns as small as possible
- Use appropriate data types (e.g., INT for IDs, not VARCHAR)
- Consider partitioning large tables
- Use EXPLAIN to analyze query performance
- Set appropriate buffer pool size for InnoDB
- Consider using covering indexes for frequent queries
- Monitor slow queries and optimize as needed`)
      .replace(/{{SECURITY_PLACEHOLDER}}/g, 
`- Never store plaintext passwords, use secure hashing algorithms
- Use prepared statements to prevent SQL injection
- Implement column-level encryption for sensitive data
- Use TLS for database connections
- Implement proper access controls and least privilege
- Audit database access
- Regularly update and patch database software`);
  }

  // API documentation specific replacements
  if (templateName === 'api') {
    template = template
      .replace(/{{BASE_URL}}/g, 'https://api.bluefly.io/v1')
      .replace(/{{AUTHENTICATION_PLACEHOLDER}}/g, "This API uses JSON Web Tokens (JWT) for authentication. To authenticate your requests, include an Authorization header with a Bearer token.\n\n```\nAuthorization: Bearer eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9...\n```\n\nTo obtain a token, make a POST request to the /auth endpoint with your credentials.")
      .replace(/{{HEADERS_PLACEHOLDER}}/g, "| Authorization | Bearer {jwt_token} | Required for authenticated endpoints |\n| Content-Type | application/json | Required for endpoints accepting request body |\n| Accept | application/json | Specifies the response format |\n| X-API-Key | your_api_key | Required for endpoints requiring API key |\n| X-Request-ID | request_uuid | Optional request identifier for tracking |")
      .replace(/{{ENDPOINT1_NAME}}/g, 'Authenticate')
      .replace(/{{ENDPOINT1_DESCRIPTION}}/g, 'Authenticates a user and returns a JWT token.')
      .replace(/{{ENDPOINT1_URL}}/g, '/auth')
      .replace(/{{ENDPOINT1_METHOD}}/g, 'POST')
      .replace(/{{ENDPOINT1_AUTH_REQUIRED}}/g, 'No')
      .replace(/{{ENDPOINT1_REQUEST_PARAMS_TYPE}}/g, 'Request Body')
      .replace(/{{ENDPOINT1_REQUEST_PARAMS}}/g, "{\n  \"username\": \"jdoe\",\n  \"password\": \"secure_password\"\n}")
      .replace(/{{ENDPOINT1_SUCCESS_CODE}}/g, '200 OK')
      .replace(/{{ENDPOINT1_SUCCESS_RESPONSE}}/g, "{\n  \"token\": \"eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9...\",\n  \"expires_at\": \"2025-06-01T00:00:00Z\",\n  \"user\": {\n    \"id\": 123,\n    \"username\": \"jdoe\",\n    \"email\": \"jdoe@example.com\"\n  }\n}")
      .replace(/{{ENDPOINT1_ERROR1_CODE}}/g, '401 Unauthorized')
      .replace(/{{ENDPOINT1_ERROR1_RESPONSE}}/g, "{\n  \"error\": \"invalid_credentials\",\n  \"message\": \"Invalid username or password\"\n}")
      .replace(/{{ENDPOINT1_ERROR2_CODE}}/g, '429 Too Many Requests')
      .replace(/{{ENDPOINT1_ERROR2_RESPONSE}}/g, "{\n  \"error\": \"rate_limit_exceeded\",\n  \"message\": \"Too many authentication attempts\",\n  \"retry_after\": 60\n}")
      .replace(/{{ENDPOINT1_EXAMPLE}}/g, "curl -X POST https://api.bluefly.io/v1/auth \\\n  -H \"Content-Type: application/json\" \\\n  -d '{\"username\": \"jdoe\", \"password\": \"secure_password\"}'")
      
      .replace(/{{ENDPOINT2_NAME}}/g, 'Get User Profile')
      .replace(/{{ENDPOINT2_DESCRIPTION}}/g, 'Retrieves the profile of the authenticated user.')
      .replace(/{{ENDPOINT2_URL}}/g, '/users/me')
      .replace(/{{ENDPOINT2_METHOD}}/g, 'GET')
      .replace(/{{ENDPOINT2_AUTH_REQUIRED}}/g, 'Yes')
      .replace(/{{ENDPOINT2_REQUEST_PARAMS_TYPE}}/g, 'Query Parameters')
      .replace(/{{ENDPOINT2_REQUEST_PARAMS}}/g, "{\n  \"include\": \"preferences,settings\"  // Optional fields to include\n}")
      .replace(/{{ENDPOINT2_SUCCESS_CODE}}/g, '200 OK')
      .replace(/{{ENDPOINT2_SUCCESS_RESPONSE}}/g, "{\n  \"id\": 123,\n  \"username\": \"jdoe\",\n  \"email\": \"jdoe@example.com\",\n  \"first_name\": \"John\",\n  \"last_name\": \"Doe\",\n  \"created_at\": \"2024-01-15T10:30:45Z\",\n  \"preferences\": {\n    \"theme\": \"dark\",\n    \"notifications\": true\n  },\n  \"settings\": {\n    \"two_factor_auth\": true,\n    \"language\": \"en\"\n  }\n}")
      .replace(/{{ENDPOINT2_ERROR1_CODE}}/g, '401 Unauthorized')
      .replace(/{{ENDPOINT2_ERROR1_RESPONSE}}/g, "{\n  \"error\": \"invalid_token\",\n  \"message\": \"Authentication token is invalid or expired\"\n}")
      .replace(/{{ENDPOINT2_EXAMPLE}}/g, "curl -X GET https://api.bluefly.io/v1/users/me?include=preferences,settings \\\n  -H \"Authorization: Bearer eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9...\"")
      .replace(/{{MODEL1_NAME}}/g, 'User')
      .replace(/{{MODEL1_DESCRIPTION}}/g, 'Represents a user in the system.')
      .replace(/{{MODEL1_SCHEMA}}/g, "{\n  \"id\": \"integer\",       // Unique identifier\n  \"username\": \"string\",  // Username (unique)\n  \"email\": \"string\",     // Email address (unique)\n  \"first_name\": \"string\",\n  \"last_name\": \"string\",\n  \"status\": \"string\",    // \"active\", \"inactive\", \"suspended\"\n  \"created_at\": \"string (ISO 8601 datetime)\",\n  \"updated_at\": \"string (ISO 8601 datetime)\",\n  \"preferences\": {       // Optional user preferences\n    \"theme\": \"string\",\n    \"notifications\": \"boolean\"\n  }\n}")
      .replace(/{{MODEL2_NAME}}/g, 'Authentication Token')
      .replace(/{{MODEL2_DESCRIPTION}}/g, 'Represents an authentication token returned after successful login.')
      .replace(/{{MODEL2_SCHEMA}}/g, "{\n  \"token\": \"string\",     // JWT token\n  \"expires_at\": \"string (ISO 8601 datetime)\",\n  \"user\": {              // User information\n    \"id\": \"integer\",\n    \"username\": \"string\",\n    \"email\": \"string\"\n  }\n}")
      .replace(/{{ERROR_CODES_PLACEHOLDER}}/g, "| 400 | bad_request | The request was malformed or missing required parameters |\n| 401 | unauthorized | Authentication is required or credentials are invalid |\n| 403 | forbidden | The authenticated user lacks permission for this operation |\n| 404 | not_found | The requested resource was not found |\n| 422 | validation_error | The request data failed validation |\n| 429 | rate_limit_exceeded | Too many requests, rate limit exceeded |\n| 500 | server_error | An unexpected server error occurred |")
      .replace(/{{RATE_LIMITING_PLACEHOLDER}}/g, "The API implements rate limiting to ensure fair usage and system stability:\n\n- Authentication endpoints: 10 requests per minute per IP address\n- Standard endpoints: 60 requests per minute per authenticated user\n- Bulk operations: 10 requests per minute per authenticated user\n\nWhen rate limits are exceeded, the API returns a 429 Too Many Requests response with a Retry-After header indicating the number of seconds to wait before retrying.")
      .replace(/{{VERSIONING_PLACEHOLDER}}/g, "The API uses URL-based versioning with the format \"/v{version_number}\".

- Current stable version: v1
- Beta features: v2 (subject to change)

Breaking changes are only introduced in new major versions. Minor updates and bug fixes are applied to the current version without changing the API contract.")
      .replace(/{{PAGINATION_PLACEHOLDER}}/g, "List endpoints support pagination using the following query parameters:\n\n- \\"page\\": Page number (1-based, default: 1)\n- \\"per_page\\": Number of items per page (default: 20, max: 100)\n\nResponse includes pagination metadata:\n\n```json\n{\n  \"data\": [...],\n  \"pagination\": {\n    \"total_items\": 243,\n    \"total_pages\": 13,\n    \"current_page\": 1,\n    \"per_page\": 20,\n    \"next_page\": 2,\n    \"prev_page\": null\n  }\n}\n```\n\nFor large datasets, consider using cursor-based pagination with the \\"cursor\\" and \\"limit\\" parameters.")
      .replace(/{{WORKFLOW_EXAMPLE_PLACEHOLDER}}/g, "// Complete authentication and data retrieval workflow\nasync function getUserData() {\n  // 1. Authenticate\n  const authResponse = await fetch('https://api.bluefly.io/v1/auth', {\n    method: 'POST',\n    headers: { 'Content-Type': 'application/json' },\n    body: JSON.stringify({\n      username: 'jdoe',\n      password: 'secure_password'\n    })\n  });\n  \n  const authData = await authResponse.json();\n  const token = authData.token;\n  \n  // 2. Get user profile\n  const profileResponse = await fetch('https://api.bluefly.io/v1/users/me', {\n    headers: {\n      'Authorization': 'Bearer ' + token,\n      'Accept': 'application/json'\n    }\n  });\n  \n  const profileData = await profileResponse.json();\n  \n  // 3. Update user preferences\n  await fetch('https://api.bluefly.io/v1/users/me/preferences', {\n    method: 'PATCH',\n    headers: {\n      'Authorization': 'Bearer ' + token,\n      'Content-Type': 'application/json'\n    },\n    body: JSON.stringify({\n      theme: 'dark',\n      notifications: true\n    })\n  });\n  \n  return profileData;\n}");
  }

  // Write the file
  fs.writeFileSync(filePath, template);
  console.log(`Created documentation file: ${filePath}`);

  // Update README index if category is provided
  if (category) {
    if (location === 'docs') {
      addToDocsReadmeIndex(fileName, title, category, finalTargetDir);
    } else {
      addToReadmeIndex(fileName, title, category);
    }
  }
}

/**
 * Creates a database schema documentation file
 * @param {string} name - The name of the database
 * @param {string} category - The category for the documentation file
 */
function createDbSchemaDoc(name, category = 'Database Schemas') {
  createDocFile(name, category, 'db_schema', 'docs');
}

/**
 * Creates an API documentation file
 * @param {string} name - The name of the API
 * @param {string} category - The category for the documentation file
 */
function createApiDoc(name, category = 'API Documentation') {
  createDocFile(name, category, 'api', 'docs');
}

/**
 * Updates an existing documentation file
 * @param {string} name - The name of the documentation file
 * @param {string} location - Target location (.claude or docs)
 */
function updateDocFile(name, location = '.claude') {
  if (!name) {
    console.error('Error: Missing file name');
    printUsage();
    process.exit(1);
  }

  // Determine target directory
  const targetDir = location === 'docs' ? DOCS_DIR : CLAUDE_DIR;

  // Convert name to proper file format
  const fileName = name.toLowerCase().endsWith('.md') 
    ? name.toLowerCase()
    : name.toLowerCase() + '.md';
  
  // Try to find the file in the target directory or its subdirectories
  let filePath;
  if (location === 'docs') {
    // Check if the file exists directly in the docs directory
    const directPath = path.join(targetDir, fileName);
    if (fs.existsSync(directPath)) {
      filePath = directPath;
    } else {
      // Look for the file in subdirectories
      let found = false;
      const subdirs = fs.readdirSync(targetDir, { withFileTypes: true })
        .filter(dirent => dirent.isDirectory())
        .map(dirent => dirent.name);
      
      for (const subdir of subdirs) {
        const subdirPath = path.join(targetDir, subdir, fileName);
        if (fs.existsSync(subdirPath)) {
          filePath = subdirPath;
          found = true;
          break;
        }
      }
      
      if (!found) {
        console.error(`Error: File ${fileName} not found in ${targetDir} or its subdirectories`);
        process.exit(1);
      }
    }
  } else {
    filePath = path.join(targetDir, fileName);
    
    // Check if file exists
    if (!fs.existsSync(filePath)) {
      console.error(`Error: File ${fileName} not found at ${filePath}`);
      process.exit(1);
    }
  }

  // Read the file
  let content = fs.readFileSync(filePath, 'utf8');

  // Update the last updated date
  content = content.replace(
    /\*\*Last Updated\*\*: .*/,
    `**Last Updated**: ${DATE_FORMAT.format(new Date())}`
  );

  // Write the file
  fs.writeFileSync(filePath, content);
  console.log(`Updated documentation file: ${filePath}`);
}

/**
 * Validates the documentation structure
 * @param {string} location - Target location (.claude or docs)
 */
function validateDocStructure(location = '.claude') {
  const targetDir = location === 'docs' ? DOCS_DIR : CLAUDE_DIR;
  const targetReadme = location === 'docs' ? DOCS_README_PATH : README_PATH;

  if (!fs.existsSync(targetReadme)) {
    console.error(`Error: README.md not found at ${targetReadme}`);
    
    // Create a basic README.md if in docs mode
    if (location === 'docs') {
      const basicReadme = `# Bluefly AI Platform Documentation

This directory contains documentation for the Bluefly AI Platform.

## Documentation Categories

`;
      fs.writeFileSync(targetReadme, basicReadme);
      console.log(`Created basic README.md at ${targetReadme}`);
    } else {
      process.exit(1);
    }
  }

  // Read README.md
  const readme = fs.readFileSync(targetReadme, 'utf8');

  if (location === '.claude') {
    // Find all links in README.md
    const linkRegex = /\[.*\]\((\.\/.*\.md)\)/g;
    const links = [];
    let match;
    while ((match = linkRegex.exec(readme)) !== null) {
      links.push(match[1]);
    }

    // Check if all linked files exist
    let missingFiles = false;
    for (const link of links) {
      const filePath = path.join(targetDir, path.basename(link));
      if (!fs.existsSync(filePath)) {
        console.error(`Error: Linked file not found: ${filePath}`);
        missingFiles = true;
      }
    }

    // Find all .md files in .claude directory
    const files = fs.readdirSync(targetDir)
      .filter(file => file.endsWith('.md') && file !== 'README.md');

    // Check if all .md files are linked in README.md
    let unlinkedFiles = false;
    for (const file of files) {
      const linkPath = `./${file}`;
      if (!links.includes(linkPath)) {
        console.error(`Warning: File not linked in README.md: ${file}`);
        unlinkedFiles = true;
      }
    }

    // Validate documentation content
    let invalidContent = false;
    for (const file of files) {
      const filePath = path.join(targetDir, file);
      const content = fs.readFileSync(filePath, 'utf8');
      
      // Check if file has a title
      if (!content.match(/^#\s+.*/)) {
        console.error(`Error: File ${file} is missing a title (# Title)`);
        invalidContent = true;
      }
      
      // Check if file has a version and last updated date
      if (!content.match(/\*\*Version\*\*:/) || !content.match(/\*\*Last Updated\*\*:/)) {
        console.error(`Error: File ${file} is missing version or last updated information`);
        invalidContent = true;
      }
    }

    if (!missingFiles && !unlinkedFiles && !invalidContent) {
      console.log('Documentation structure validation successful.');
    } else {
      console.error('Documentation structure validation failed.');
      process.exit(1);
    }
  } else {
    // For docs directory, we validate subdirectories and their READMEs
    const subdirs = fs.readdirSync(targetDir, { withFileTypes: true })
      .filter(dirent => dirent.isDirectory())
      .map(dirent => dirent.name);
    
    // Check that each subdirectory has a README.md
    let missingReadmes = false;
    for (const subdir of subdirs) {
      const subdirReadmePath = path.join(targetDir, subdir, 'README.md');
      if (!fs.existsSync(subdirReadmePath)) {
        console.error(`Error: Subdirectory ${subdir} is missing a README.md`);
        missingReadmes = true;
      }
    }
    
    // Validate documentation content in all .md files
    let invalidContent = false;
    const validateDir = (dir) => {
      const files = fs.readdirSync(dir, { withFileTypes: true });
      for (const file of files) {
        const filePath = path.join(dir, file.name);
        if (file.isDirectory()) {
          validateDir(filePath);
        } else if (file.name.endsWith('.md') && file.name !== 'README.md') {
          const content = fs.readFileSync(filePath, 'utf8');
          
          // Check if file has a title
          if (!content.match(/^#\s+.*/)) {
            console.error(`Error: File ${filePath} is missing a title (# Title)`);
            invalidContent = true;
          }
          
          // Check if file has a version and last updated date
          if (!content.match(/\*\*Version\*\*:/) || !content.match(/\*\*Last Updated\*\*:/)) {
            console.error(`Error: File ${filePath} is missing version or last updated information`);
            invalidContent = true;
          }
        }
      }
    };
    
    validateDir(targetDir);
    
    if (!missingReadmes && !invalidContent) {
      console.log('Documentation structure validation successful.');
    } else {
      console.error('Documentation structure validation failed.');
      process.exit(1);
    }
  }
}

/**
 * Syncs content from CLAUDE.md to target directory
 * @param {string} location - Target location (.claude or docs)
 */
function syncFromClaude(location = '.claude') {
  if (!fs.existsSync(CLAUDE_MD_PATH)) {
    console.error(`Error: CLAUDE.md not found at ${CLAUDE_MD_PATH}`);
    process.exit(1);
  }

  // Determine target directory
  const targetDir = location === 'docs' ? DOCS_DIR : CLAUDE_DIR;
  const targetReadme = location === 'docs' ? DOCS_README_PATH : README_PATH;

  // Create target directory if it doesn't exist
  if (!fs.existsSync(targetDir)) {
    fs.mkdirSync(targetDir, { recursive: true });
    console.log(`Created directory: ${targetDir}`);
  }

  // Read CLAUDE.md
  const claudeMd = fs.readFileSync(CLAUDE_MD_PATH, 'utf8');

  // Extract sections based on headings
  const sectionRegex = /^#+\s+(.*?)\n([\s\S]*?)(?=^#+\s+|\Z)/gm;
  const sections = [];
  let match;
  while ((match = sectionRegex.exec(claudeMd)) !== null) {
    sections.push({
      title: match[1].trim(),
      content: match[0].trim()
    });
  }

  // Create README.md if it doesn't exist
  if (!fs.existsSync(targetReadme)) {
    let readmeContent;
    
    if (location === '.claude') {
      readmeContent = `# Bluefly AI Platform Documentation

This directory contains guidance for Claude Code (claude.ai/code) when working with code in this repository.

🚦 **IMPORTANT:** All development in this project follows strict Test Driven Development (TDD): tests are always written first, and only then is code written to pass those tests.

## Important File Management Instructions
- **NEVER DELETE FILES**: Instead of deleting files, move them to \`process.cwd()/__DELETE_LATER\`
- This applies to all Claude Code windows working on this project
- **NOTE**: This is NOT a monorepo project - each directory represents a separate, independent project with its own configuration, dependencies, and deployment pipeline

## Documentation Index

`;
    } else {
      readmeContent = `# Bluefly AI Platform Documentation

This directory contains comprehensive documentation for the Bluefly AI Platform.

## Documentation Categories

`;
    }
    
    fs.writeFileSync(targetReadme, readmeContent);
    console.log(`Created README.md at ${targetReadme}`);
  }

  // Map sections to files and categories
  const sectionToCategory = {
    'Service Architecture': 'Core Documentation',
    'Testing Requirements': 'Development Standards',
    'Drupal LLM Platform': 'Implementation Details',
    'TDD Enforcement': 'Development Standards',
    'Field ECA Bridge': 'Implementation Details',
    'Vision': 'Reference & Support',
    'Roadmap': 'Reference & Support',
    'Project Structure': 'Core Documentation',
    'Development Commands': 'Core Documentation'
  };

  for (const section of sections) {
    const fileName = section.title.toLowerCase().replace(/\s+/g, '_') + '.md';
    const category = sectionToCategory[section.title] || 'Reference & Support';
    
    if (location === '.claude') {
      const filePath = path.join(targetDir, fileName);
      
      // Skip if file already exists
      if (fs.existsSync(filePath)) {
        console.log(`Skipping existing file: ${fileName}`);
        continue;
      }

      // Add version and last updated information if not present
      let content = section.content;
      if (!content.match(/\*\*Version\*\*:/) || !content.match(/\*\*Last Updated\*\*:/)) {
        content += `\n\n---\n\n**Version**: 1.0.0  \n**Last Updated**: ${DATE_FORMAT.format(new Date())}  \n**Status**: Imported from CLAUDE.md`;
      }

      // Write the file
      fs.writeFileSync(filePath, content);
      console.log(`Created documentation file from section: ${filePath}`);

      // Add to README index
      addToReadmeIndex(fileName, section.title, category);
    } else {
      // For docs directory, create a category subdirectory
      const categoryDir = path.join(targetDir, category.toLowerCase().replace(/\s+/g, '_'));
      if (!fs.existsSync(categoryDir)) {
        fs.mkdirSync(categoryDir, { recursive: true });
        console.log(`Created category directory: ${categoryDir}`);
        
        // Create README.md in the category directory
        const categoryReadmePath = path.join(categoryDir, 'README.md');
        const categoryReadmeContent = `# ${category}

This directory contains documentation for ${category.toLowerCase()} in the Bluefly AI Platform.

## Documents

`;
        fs.writeFileSync(categoryReadmePath, categoryReadmeContent);
        console.log(`Created category README.md at ${categoryReadmePath}`);
      }
      
      const filePath = path.join(categoryDir, fileName);
      
      // Skip if file already exists
      if (fs.existsSync(filePath)) {
        console.log(`Skipping existing file: ${fileName}`);
        continue;
      }

      // Add version and last updated information if not present
      let content = section.content;
      if (!content.match(/\*\*Version\*\*:/) || !content.match(/\*\*Last Updated\*\*:/)) {
        content += `\n\n---\n\n**Version**: 1.0.0  \n**Last Updated**: ${DATE_FORMAT.format(new Date())}  \n**Status**: Imported from CLAUDE.md`;
      }

      // Write the file
      fs.writeFileSync(filePath, content);
      console.log(`Created documentation file from section: ${filePath}`);

      // Add to category README.md
      addToCategoryReadme(fileName, section.title, categoryDir);
      
      // Add category to main README.md
      addCategoryToDocsReadme(category, categoryDir);
    }
  }

  console.log(`Sync from CLAUDE.md to ${location} completed.`);
}

/**
 * Updates the README.md index
 * @param {string} location - Target location (.claude or docs)
 */
function updateReadmeIndex(location = '.claude') {
  const targetDir = location === 'docs' ? DOCS_DIR : CLAUDE_DIR;
  const targetReadme = location === 'docs' ? DOCS_README_PATH : README_PATH;

  if (!fs.existsSync(targetReadme)) {
    // Create a basic README.md if it doesn't exist
    let basicReadme;
    
    if (location === '.claude') {
      basicReadme = `# Bluefly AI Platform Documentation

This directory contains guidance for Claude Code (claude.ai/code) when working with code in this repository.

🚦 **IMPORTANT:** All development in this project follows strict Test Driven Development (TDD): tests are always written first, and only then is code written to pass those tests.

## Important File Management Instructions
- **NEVER DELETE FILES**: Instead of deleting files, move them to \`process.cwd()/__DELETE_LATER\`
- This applies to all Claude Code windows working on this project
- **NOTE**: This is NOT a monorepo project - each directory represents a separate, independent project with its own configuration, dependencies, and deployment pipeline

## Documentation Index

`;
    } else {
      basicReadme = `# Bluefly AI Platform Documentation

This directory contains comprehensive documentation for the Bluefly AI Platform.

## Documentation Categories

`;
    }
    
    fs.writeFileSync(targetReadme, basicReadme);
    console.log(`Created basic README.md at ${targetReadme}`);
  }

  if (location === '.claude') {
    // Read README.md
    let readme = fs.readFileSync(targetReadme, 'utf8');

    // Find the index section
    const indexSectionRegex = /(## Documentation Index\n\n)([\s\S]*?)(\n\n##|$)/;
    const indexSectionMatch = readme.match(indexSectionRegex);
    
    if (!indexSectionMatch) {
      console.error('Error: Could not find Documentation Index section in README.md');
      process.exit(1);
    }

    // Get all .md files in .claude directory
    const files = fs.readdirSync(targetDir)
      .filter(file => file.endsWith('.md') && file !== 'README.md');

    // Extract metadata from files
    const fileData = [];
    for (const file of files) {
      const filePath = path.join(targetDir, file);
      const content = fs.readFileSync(filePath, 'utf8');
      
      // Extract title
      const titleMatch = content.match(/^#\s+(.*)/);
      const title = titleMatch ? titleMatch[1].trim() : file.replace('.md', '').split('_').map(word => word.charAt(0).toUpperCase() + word.slice(1)).join(' ');
      
      // Extract description (first paragraph after title)
      const descriptionMatch = content.match(/^#\s+.*\n\n(.*?)(\n\n|$)/);
      const description = descriptionMatch ? descriptionMatch[1].trim() : '';

      // Categorize files based on filename patterns
      let category = 'Reference & Support';
      if (file.includes('test') || file.includes('tdd')) {
        category = 'Development Standards';
      } else if (file.includes('architecture') || file.includes('structure') || file.includes('commands')) {
        category = 'Core Documentation';
      } else if (file.includes('pipeline') || file.includes('deploy') || file.includes('monitor')) {
        category = 'DevOps & Operations';
      } else if (file.includes('drupal') || file.includes('module') || file.includes('field')) {
        category = 'Implementation Details';
      }

      fileData.push({
        file,
        title,
        description,
        category
      });
    }

    // Group files by category
    const categories = {};
    for (const data of fileData) {
      if (!categories[data.category]) {
        categories[data.category] = [];
      }
      categories[data.category].push(data);
    }

    // Generate index content
    let indexContent = '';
    for (const category in categories) {
      indexContent += `### ${category}\n`;
      for (const data of categories[category]) {
        indexContent += `- [${data.title}](./${data.file})${data.description ? ' - ' + data.description : ''}\n`;
      }
      indexContent += '\n';
    }

    // Update README.md
    readme = readme.replace(indexSectionMatch[0], `${indexSectionMatch[1]}${indexContent}${indexSectionMatch[3].startsWith('\n\n##') ? indexSectionMatch[3] : '\n\n' + indexSectionMatch[3]}`);
    fs.writeFileSync(targetReadme, readme);
    console.log(`Updated README.md index at ${targetReadme}`);
  } else {
    // For docs directory, we update the main README.md with categories
    let readme = fs.readFileSync(targetReadme, 'utf8');
    
    // Find the categories section
    const categoriesSectionRegex = /(## Documentation Categories\n\n)([\s\S]*?)(\n\n##|$)/;
    const categoriesSectionMatch = readme.match(categoriesSectionRegex);
    
    if (!categoriesSectionMatch) {
      console.error('Error: Could not find Documentation Categories section in README.md');
      process.exit(1);
    }
    
    // Get all subdirectories in docs directory
    const subdirs = fs.readdirSync(targetDir, { withFileTypes: true })
      .filter(dirent => dirent.isDirectory())
      .map(dirent => dirent.name);
    
    // Generate categories content
    let categoriesContent = '';
    for (const subdir of subdirs) {
      const readmePath = path.join(targetDir, subdir, 'README.md');
      let categoryTitle = subdir.split('_').map(word => word.charAt(0).toUpperCase() + word.slice(1)).join(' ');
      
      // Extract title from README.md if available
      if (fs.existsSync(readmePath)) {
        const readme = fs.readFileSync(readmePath, 'utf8');
        const titleMatch = readme.match(/^#\s+(.*)/);
        if (titleMatch) {
          categoryTitle = titleMatch[1].trim();
        }
      }
      
      categoriesContent += `### [${categoryTitle}](./${subdir}/README.md)\n\n`;
      
      // Count the number of documentation files
      const docFiles = fs.readdirSync(path.join(targetDir, subdir))
        .filter(file => file.endsWith('.md') && file !== 'README.md');
      
      categoriesContent += `${docFiles.length} document${docFiles.length !== 1 ? 's' : ''}\n\n`;
    }
    
    // Update README.md
    readme = readme.replace(categoriesSectionMatch[0], `${categoriesSectionMatch[1]}${categoriesContent}${categoriesSectionMatch[3].startsWith('\n\n##') ? categoriesSectionMatch[3] : '\n\n' + categoriesSectionMatch[3]}`);
    fs.writeFileSync(targetReadme, readme);
    console.log(`Updated README.md categories at ${targetReadme}`);
  }
}

/**
 * Adds a file to the README.md index
 * @param {string} fileName - The name of the documentation file
 * @param {string} title - The title of the documentation file
 * @param {string} category - The category for the documentation file
 */
function addToReadmeIndex(fileName, title, category) {
  if (!fs.existsSync(README_PATH)) {
    console.error(`Error: README.md not found at ${README_PATH}`);
    return;
  }

  // Read README.md
  let readme = fs.readFileSync(README_PATH, 'utf8');

  // Find or create the category section
  const categoryRegex = new RegExp(`### ${category}\\n([\\s\\S]*?)\\n(###|$)`);
  const categoryMatch = readme.match(categoryRegex);

  if (categoryMatch) {
    // Category exists, add the file to it
    const categoryContent = categoryMatch[1];
    const newCategoryContent = `${categoryContent}- [${title}](./${fileName})\n`;
    readme = readme.replace(categoryMatch[0], `### ${category}\n${newCategoryContent}\n${categoryMatch[2]}`);
  } else {
    // Category doesn't exist, create it
    const indexSectionRegex = /(## Documentation Index\n\n)([\s\S]*?)(\n\n##|$)/;
    const indexSectionMatch = readme.match(indexSectionRegex);

    if (indexSectionMatch) {
      const indexContent = indexSectionMatch[2];
      const newIndexContent = `${indexContent}\n### ${category}\n- [${title}](./${fileName})\n`;
      readme = readme.replace(indexSectionMatch[0], `${indexSectionMatch[1]}${newIndexContent}${indexSectionMatch[3]}`);
    } else {
      // Documentation Index section doesn't exist, create it
      readme += `\n## Documentation Index\n\n### ${category}\n- [${title}](./${fileName})\n`;
    }
  }

  // Write the updated README.md
  fs.writeFileSync(README_PATH, readme);
  console.log(`Added ${fileName} to ${category} in README.md`);
}

/**
 * Adds a file to the docs README.md index
 * @param {string} fileName - The name of the documentation file
 * @param {string} title - The title of the documentation file
 * @param {string} category - The category for the documentation file
 * @param {string} categoryDir - The directory path for the category
 */
function addToDocsReadmeIndex(fileName, title, category, categoryDir) {
  const categoryReadmePath = path.join(categoryDir, 'README.md');
  
  if (!fs.existsSync(categoryReadmePath)) {
    // Create category README.md if it doesn't exist
    const categoryReadmeContent = `# ${category}

This directory contains documentation for ${category.toLowerCase()} in the Bluefly AI Platform.

## Documents

- [${title}](./${fileName})
`;
    fs.writeFileSync(categoryReadmePath, categoryReadmeContent);
    console.log(`Created category README.md at ${categoryReadmePath}`);
  } else {
    // Update existing category README.md
    let categoryReadme = fs.readFileSync(categoryReadmePath, 'utf8');
    
    // Find the documents section
    const documentsSectionRegex = /(## Documents\n\n)([\s\S]*?)(\n\n##|$)/;
    const documentsSectionMatch = categoryReadme.match(documentsSectionRegex);
    
    if (documentsSectionMatch) {
      // Documents section exists, add the file to it
      const documentsContent = documentsSectionMatch[2];
      const newDocumentsContent = `${documentsContent}- [${title}](./${fileName})\n`;
      categoryReadme = categoryReadme.replace(documentsSectionMatch[0], `${documentsSectionMatch[1]}${newDocumentsContent}${documentsSectionMatch[3]}`);
    } else {
      // Documents section doesn't exist, create it
      categoryReadme += `\n## Documents\n\n- [${title}](./${fileName})\n`;
    }
    
    // Write the updated category README.md
    fs.writeFileSync(categoryReadmePath, categoryReadme);
    console.log(`Added ${fileName} to ${categoryReadmePath}`);
  }
  
  // Add category to main README.md if needed
  addCategoryToDocsReadme(category, categoryDir);
}

/**
 * Adds a file to the category README.md
 * @param {string} fileName - The name of the documentation file
 * @param {string} title - The title of the documentation file
 * @param {string} categoryDir - The directory path for the category
 */
function addToCategoryReadme(fileName, title, categoryDir) {
  const categoryReadmePath = path.join(categoryDir, 'README.md');
  
  if (!fs.existsSync(categoryReadmePath)) {
    console.error(`Error: Category README.md not found at ${categoryReadmePath}`);
    return;
  }
  
  // Read category README.md
  let categoryReadme = fs.readFileSync(categoryReadmePath, 'utf8');
  
  // Find the documents section
  const documentsSectionRegex = /(## Documents\n\n)([\s\S]*?)(\n\n##|$)/;
  const documentsSectionMatch = categoryReadme.match(documentsSectionRegex);
  
  if (documentsSectionMatch) {
    // Documents section exists, add the file to it if not already present
    const documentsContent = documentsSectionMatch[2];
    if (!documentsContent.includes(`[${title}](./${fileName})`)) {
      const newDocumentsContent = `${documentsContent}- [${title}](./${fileName})\n`;
      categoryReadme = categoryReadme.replace(documentsSectionMatch[0], `${documentsSectionMatch[1]}${newDocumentsContent}${documentsSectionMatch[3]}`);
    }
  } else {
    // Documents section doesn't exist, create it
    categoryReadme += `\n## Documents\n\n- [${title}](./${fileName})\n`;
  }
  
  // Write the updated category README.md
  fs.writeFileSync(categoryReadmePath, categoryReadme);
  console.log(`Added ${fileName} to ${categoryReadmePath}`);
}

/**
 * Adds a category to the main docs README.md
 * @param {string} category - The category name
 * @param {string} categoryDir - The directory path for the category
 */
function addCategoryToDocsReadme(category, categoryDir) {
  if (!fs.existsSync(DOCS_README_PATH)) {
    // Create main README.md if it doesn't exist
    const docsReadmeContent = `# Bluefly AI Platform Documentation

This directory contains comprehensive documentation for the Bluefly AI Platform.

## Documentation Categories

### [${category}](${path.relative(DOCS_DIR, path.join(categoryDir, 'README.md'))})

`;
    fs.writeFileSync(DOCS_README_PATH, docsReadmeContent);
    console.log(`Created main docs README.md at ${DOCS_README_PATH}`);
  } else {
    // Update existing main README.md
    let docsReadme = fs.readFileSync(DOCS_README_PATH, 'utf8');
    
    // Check if category is already in the README
    const categoryLinkRegex = new RegExp(`\\[${category}\\]\\(${path.relative(DOCS_DIR, path.join(categoryDir, 'README.md')).replace(/\//g, '\\/')}\\)`);
    if (!docsReadme.match(categoryLinkRegex)) {
      // Find the categories section
      const categoriesSectionRegex = /(## Documentation Categories\n\n)([\s\S]*?)(\n\n##|$)/;
      const categoriesSectionMatch = docsReadme.match(categoriesSectionRegex);
      
      if (categoriesSectionMatch) {
        // Categories section exists, add the category to it
        const categoriesContent = categoriesSectionMatch[2];
        const newCategoriesContent = `${categoriesContent}### [${category}](${path.relative(DOCS_DIR, path.join(categoryDir, 'README.md'))})\n\n`;
        docsReadme = docsReadme.replace(categoriesSectionMatch[0], `${categoriesSectionMatch[1]}${newCategoriesContent}${categoriesSectionMatch[3]}`);
      } else {
        // Categories section doesn't exist, create it
        docsReadme += `\n## Documentation Categories\n\n### [${category}](${path.relative(DOCS_DIR, path.join(categoryDir, 'README.md'))})\n\n`;
      }
      
      // Write the updated main README.md
      fs.writeFileSync(DOCS_README_PATH, docsReadme);
      console.log(`Added category ${category} to main docs README.md`);
    }
  }
}

// Make the script executable
if (require.main === module) {
  try {
    // Make the script executable
    if (process.platform !== 'win32') {
      execSync(`chmod +x ${__filename}`);
    }
  } catch (error) {
    console.error('Error making script executable:', error);
  }
}