#!/usr/bin/env node

/**
 * Enhanced Data Collectors for Comprehensive Documentation Aggregation
 * 
 * This script provides specialized collectors for different project types
 * to extract maximum information during build-time aggregation.
 * 
 * Features:
 * - Specialized collectors for each project type (API, CLI, UI, etc.)
 * - Code analysis for automatic documentation generation
 * - OpenAPI specification parsing
 * - Test coverage analysis
 * - Performance metrics extraction
 * - Security configuration detection
 */

const fs = require('fs').promises;
const path = require('path');
const yaml = require('yaml');

class EnhancedCollectors {
  constructor() {
    this.collectors = {
      'Core Services': this.collectCoreServiceData.bind(this),
      'AI Services': this.collectAIServiceData.bind(this),
      'Protocol Services': this.collectProtocolServiceData.bind(this),
      'Frontend': this.collectFrontendData.bind(this),
      'Client Libraries': this.collectClientLibraryData.bind(this),
      'Developer Tools': this.collectDeveloperToolData.bind(this),
      'DevOps': this.collectDevOpsData.bind(this),
      'Drupal Integration': this.collectDrupalData.bind(this)
    };
  }

  async collectProjectData(projectKey, projectConfig, projectPath) {
    const collector = this.collectors[projectConfig.category];
    if (collector) {
      return await collector(projectKey, projectConfig, projectPath);
    }
    return await this.collectGenericData(projectKey, projectConfig, projectPath);
  }

  // Core Services Collector (bfapi)
  async collectCoreServiceData(projectKey, projectConfig, projectPath) {
    const data = await this.collectGenericData(projectKey, projectConfig, projectPath);
    
    // Collect API-specific data
    data.api = await this.collectOpenAPIData(projectPath);
    data.routes = await this.collectExpressRoutes(projectPath);
    data.controllers = await this.collectControllers(projectPath);
    data.middleware = await this.collectMiddleware(projectPath);
    data.services = await this.collectServices(projectPath);
    data.performance = await this.collectPerformanceMetrics(projectPath);
    
    return data;
  }

  // AI Services Collector (bfllm, bfrfp)
  async collectAIServiceData(projectKey, projectConfig, projectPath) {
    const data = await this.collectGenericData(projectKey, projectConfig, projectPath);
    
    // Collect AI-specific data
    data.models = await this.collectModelInfo(projectPath);
    data.providers = await this.collectAIProviders(projectPath);
    data.algorithms = await this.collectAlgorithms(projectPath);
    data.training = await this.collectTrainingConfig(projectPath);
    data.inference = await this.collectInferenceConfig(projectPath);
    
    return data;
  }

  // Protocol Services Collector (llm-mcp)
  async collectProtocolServiceData(projectKey, projectConfig, projectPath) {
    const data = await this.collectGenericData(projectKey, projectConfig, projectPath);
    
    // Collect protocol-specific data
    data.protocols = await this.collectProtocolDefinitions(projectPath);
    data.grpc = await this.collectGRPCServices(projectPath);
    data.mcp = await this.collectMCPConfiguration(projectPath);
    data.tools = await this.collectToolDefinitions(projectPath);
    
    return data;
  }

  // Frontend Collector (llm-ui)
  async collectFrontendData(projectKey, projectConfig, projectPath) {
    const data = await this.collectGenericData(projectKey, projectConfig, projectPath);
    
    // Collect frontend-specific data
    data.components = await this.collectReactComponents(projectPath);
    data.hooks = await this.collectReactHooks(projectPath);
    data.styles = await this.collectStyleConfiguration(projectPath);
    data.storybook = await this.collectStorybookStories(projectPath);
    data.tests = await this.collectFrontendTests(projectPath);
    
    return data;
  }

  // Client Libraries Collector (llm-gateway-sdk)
  async collectClientLibraryData(projectKey, projectConfig, projectPath) {
    const data = await this.collectGenericData(projectKey, projectConfig, projectPath);
    
    // Collect SDK-specific data
    data.sdk = await this.collectSDKMethods(projectPath);
    data.types = await this.collectTypeDefinitions(projectPath);
    data.examples = await this.collectUsageExamples(projectPath);
    data.generation = await this.collectGenerationConfig(projectPath);
    
    return data;
  }

  // Developer Tools Collector (llmcli)
  async collectDeveloperToolData(projectKey, projectConfig, projectPath) {
    const data = await this.collectGenericData(projectKey, projectConfig, projectPath);
    
    // Collect CLI-specific data
    data.commands = await this.collectCLICommands(projectPath);
    data.plugins = await this.collectCLIPlugins(projectPath);
    data.configurations = await this.collectCLIConfigs(projectPath);
    data.automation = await this.collectAutomationScripts(projectPath);
    
    return data;
  }

  // DevOps Collector (bfcicomponents)
  async collectDevOpsData(projectKey, projectConfig, projectPath) {
    const data = await this.collectGenericData(projectKey, projectConfig, projectPath);
    
    // Collect CI/CD-specific data
    data.components = await this.collectCIComponents(projectPath);
    data.templates = await this.collectCITemplates(projectPath);
    data.pipelines = await this.collectPipelineConfigs(projectPath);
    data.coverage = await this.collectTestCoverage(projectPath);
    
    return data;
  }

  // Drupal Integration Collector (_DrupalModules)
  async collectDrupalData(projectKey, projectConfig, projectPath) {
    const data = await this.collectGenericData(projectKey, projectConfig, projectPath);
    
    // Collect Drupal-specific data
    data.modules = await this.collectDrupalModules(projectPath);
    data.recipes = await this.collectDrupalRecipes(projectPath);
    data.entities = await this.collectDrupalEntities(projectPath);
    data.services = await this.collectDrupalServices(projectPath);
    data.hooks = await this.collectDrupalHooks(projectPath);
    
    return data;
  }

  // Generic Data Collector
  async collectGenericData(projectKey, projectConfig, projectPath) {
    const data = {
      ...projectConfig,
      projectPath,
      timestamp: new Date().toISOString(),
      structure: await this.analyzeProjectStructure(projectPath),
      dependencies: await this.analyzeDependencies(projectPath),
      security: await this.analyzeSecurityConfig(projectPath),
      docker: await this.analyzeDockerConfig(projectPath),
      git: await this.analyzeGitConfig(projectPath),
      documentation: await this.analyzeDocumentationQuality(projectPath),
      metrics: await this.calculateProjectMetrics(projectPath)
    };

    return data;
  }

  // Specialized Collection Methods

  async collectOpenAPIData(projectPath) {
    try {
      const openApiPath = path.join(projectPath, 'openapi.yaml');
      const content = await fs.readFile(openApiPath, 'utf-8');
      const spec = yaml.parse(content);
      
      return {
        version: spec.openapi,
        title: spec.info?.title,
        version_info: spec.info?.version,
        endpoints: this.extractEndpoints(spec),
        schemas: Object.keys(spec.components?.schemas || {}),
        security: spec.security,
        servers: spec.servers
      };
    } catch (error) {
      return null;
    }
  }

  async collectExpressRoutes(projectPath) {
    const routes = [];
    try {
      const routesDir = path.join(projectPath, 'src', 'routes');
      const routeFiles = await fs.readdir(routesDir).catch(() => []);
      
      for (const file of routeFiles) {
        if (file.endsWith('.routes.ts') || file.endsWith('.routes.js')) {
          const routePath = path.join(routesDir, file);
          const content = await fs.readFile(routePath, 'utf-8');
          routes.push({
            file: file,
            methods: this.extractRouteMethods(content),
            middleware: this.extractRouteMiddleware(content)
          });
        }
      }
    } catch (error) {
      // Routes directory not found
    }
    
    return routes;
  }

  async collectControllers(projectPath) {
    const controllers = [];
    try {
      const controllersDir = path.join(projectPath, 'src', 'controllers');
      const controllerFiles = await fs.readdir(controllersDir).catch(() => []);
      
      for (const file of controllerFiles) {
        if (file.endsWith('.controller.ts') || file.endsWith('.controller.js')) {
          const controllerPath = path.join(controllersDir, file);
          const content = await fs.readFile(controllerPath, 'utf-8');
          controllers.push({
            file: file,
            methods: this.extractControllerMethods(content),
            exports: this.extractControllerExports(content)
          });
        }
      }
    } catch (error) {
      // Controllers directory not found
    }
    
    return controllers;
  }

  async collectCLICommands(projectPath) {
    const commands = [];
    try {
      const srcDir = path.join(projectPath, 'src');
      const commandsDir = path.join(srcDir, 'commands');
      
      // Try both src/commands and src/cli patterns
      const dirs = [commandsDir, path.join(srcDir, 'cli')];
      
      for (const dir of dirs) {
        try {
          const files = await fs.readdir(dir);
          for (const file of files) {
            if (file.endsWith('.ts') || file.endsWith('.js')) {
              const filePath = path.join(dir, file);
              const content = await fs.readFile(filePath, 'utf-8');
              commands.push({
                name: path.basename(file, path.extname(file)),
                file: file,
                description: this.extractCommandDescription(content),
                options: this.extractCommandOptions(content),
                examples: this.extractCommandExamples(content)
              });
            }
          }
        } catch (e) {
          // Directory doesn't exist
        }
      }
    } catch (error) {
      // Commands not found
    }
    
    return commands;
  }

  async collectReactComponents(projectPath) {
    const components = [];
    try {
      const srcDir = path.join(projectPath, 'src');
      const componentsDirs = [
        path.join(srcDir, 'components'),
        path.join(srcDir, 'lib'),
        srcDir
      ];
      
      for (const dir of componentsDirs) {
        try {
          await this.findReactComponents(dir, components);
        } catch (e) {
          // Directory doesn't exist
        }
      }
    } catch (error) {
      // No React components found
    }
    
    return components;
  }

  async findReactComponents(dir, components, relativePath = '') {
    try {
      const entries = await fs.readdir(dir, { withFileTypes: true });
      
      for (const entry of entries) {
        const fullPath = path.join(dir, entry.name);
        const relPath = path.join(relativePath, entry.name);
        
        if (entry.isDirectory() && !entry.name.startsWith('.') && entry.name !== 'node_modules') {
          await this.findReactComponents(fullPath, components, relPath);
        } else if (entry.isFile() && this.isReactComponent(entry.name)) {
          const content = await fs.readFile(fullPath, 'utf-8');
          components.push({
            name: path.basename(entry.name, path.extname(entry.name)),
            file: relPath,
            props: this.extractComponentProps(content),
            exports: this.extractComponentExports(content),
            hooks: this.extractUsedHooks(content)
          });
        }
      }
    } catch (error) {
      // Error reading directory
    }
  }

  async collectDrupalModules(projectPath) {
    const modules = [];
    try {
      const entries = await fs.readdir(projectPath, { withFileTypes: true });
      
      for (const entry of entries) {
        if (entry.isDirectory() && !entry.name.startsWith('.')) {
          const modulePath = path.join(projectPath, entry.name);
          const infoFile = path.join(modulePath, `${entry.name}.info.yml`);
          
          try {
            const infoContent = await fs.readFile(infoFile, 'utf-8');
            const info = yaml.parse(infoContent);
            
            modules.push({
              name: entry.name,
              info: info,
              hooks: await this.extractDrupalHooksFromModule(modulePath),
              services: await this.extractDrupalModuleServices(modulePath),
              routes: await this.extractDrupalRoutes(modulePath),
              permissions: await this.extractDrupalPermissions(modulePath)
            });
          } catch (e) {
            // Not a Drupal module
          }
        }
      }
    } catch (error) {
      // Error reading modules
    }
    
    return modules;
  }

  // Analysis Methods

  async analyzeProjectStructure(projectPath) {
    const structure = {};
    
    try {
      const entries = await fs.readdir(projectPath, { withFileTypes: true });
      
      for (const entry of entries) {
        if (entry.isDirectory() && !entry.name.startsWith('.') && entry.name !== 'node_modules') {
          structure[entry.name] = await this.getDirectoryInfo(path.join(projectPath, entry.name));
        }
      }
    } catch (error) {
      // Error analyzing structure
    }
    
    return structure;
  }

  async getDirectoryInfo(dirPath) {
    try {
      const entries = await fs.readdir(dirPath);
      return {
        fileCount: entries.filter(e => !e.startsWith('.')).length,
        hasIndex: entries.includes('index.ts') || entries.includes('index.js'),
        hasTests: entries.some(e => e.includes('.test.') || e.includes('.spec.')),
        types: this.categorizeFiles(entries)
      };
    } catch (error) {
      return { error: error.message };
    }
  }

  async analyzeDependencies(projectPath) {
    try {
      const packageJsonPath = path.join(projectPath, 'package.json');
      const content = await fs.readFile(packageJsonPath, 'utf-8');
      const packageJson = JSON.parse(content);
      
      return {
        dependencies: Object.keys(packageJson.dependencies || {}),
        devDependencies: Object.keys(packageJson.devDependencies || {}),
        peerDependencies: Object.keys(packageJson.peerDependencies || {}),
        scripts: Object.keys(packageJson.scripts || {}),
        version: packageJson.version,
        engines: packageJson.engines
      };
    } catch (error) {
      return null;
    }
  }

  async analyzeSecurityConfig(projectPath) {
    const security = {};
    
    // Check for common security files
    const securityFiles = [
      '.env.example',
      'security.json',
      'auth.config.js',
      'helmet.config.js'
    ];
    
    for (const file of securityFiles) {
      try {
        await fs.access(path.join(projectPath, file));
        security[file] = true;
      } catch (e) {
        security[file] = false;
      }
    }
    
    return security;
  }

  async analyzeDockerConfig(projectPath) {
    try {
      const dockerfilePath = path.join(projectPath, 'Dockerfile');
      const content = await fs.readFile(dockerfilePath, 'utf-8');
      
      return {
        exists: true,
        baseImage: this.extractDockerBaseImage(content),
        multiStage: content.includes('FROM') && content.split('FROM').length > 2,
        size: content.length
      };
    } catch (error) {
      return { exists: false };
    }
  }

  async analyzeGitConfig(projectPath) {
    try {
      const gitIgnorePath = path.join(projectPath, '.gitignore');
      const content = await fs.readFile(gitIgnorePath, 'utf-8');
      
      return {
        hasGitignore: true,
        rules: content.split('\n').filter(line => line.trim() && !line.startsWith('#')).length
      };
    } catch (error) {
      return { hasGitignore: false };
    }
  }

  async analyzeDocumentationQuality(projectPath) {
    const docsPath = path.join(projectPath, 'docs');
    const quality = {
      hasDocsFolder: false,
      hasReadme: false,
      docFiles: 0,
      completeness: 0
    };
    
    try {
      await fs.access(docsPath);
      quality.hasDocsFolder = true;
      
      const docFiles = await this.scanDocumentationFiles(docsPath);
      quality.docFiles = docFiles.length;
      quality.completeness = this.calculateDocumentationCompleteness(docFiles);
    } catch (e) {
      // No docs folder
    }
    
    try {
      await fs.access(path.join(projectPath, 'README.md'));
      quality.hasReadme = true;
    } catch (e) {
      // No README
    }
    
    return quality;
  }

  async scanDocumentationFiles(docsPath) {
    const files = [];
    try {
      const entries = await fs.readdir(docsPath, { withFileTypes: true });
      
      for (const entry of entries) {
        if (entry.isFile() && this.isDocumentationFile(entry.name)) {
          files.push(entry.name);
        } else if (entry.isDirectory()) {
          const subFiles = await this.scanDocumentationFiles(path.join(docsPath, entry.name));
          files.push(...subFiles);
        }
      }
    } catch (error) {
      // Error scanning docs
    }
    
    return files;
  }

  calculateDocumentationCompleteness(docFiles) {
    const requiredDocs = ['index.md', 'architecture.md', 'api.md', 'setup.md'];
    const foundDocs = docFiles.filter(file => 
      requiredDocs.some(required => file.toLowerCase().includes(required.toLowerCase()))
    );
    
    return Math.round((foundDocs.length / requiredDocs.length) * 100);
  }

  async calculateProjectMetrics(projectPath) {
    const metrics = {
      totalFiles: 0,
      codeFiles: 0,
      testFiles: 0,
      documentationFiles: 0,
      configFiles: 0,
      lastModified: null
    };
    
    try {
      await this.countFiles(projectPath, metrics);
    } catch (error) {
      // Error calculating metrics
    }
    
    return metrics;
  }

  async countFiles(dirPath, metrics, depth = 0) {
    if (depth > 5) return; // Prevent deep recursion
    
    try {
      const entries = await fs.readdir(dirPath, { withFileTypes: true });
      
      for (const entry of entries) {
        if (this.shouldSkipEntry(entry.name)) continue;
        
        const fullPath = path.join(dirPath, entry.name);
        
        if (entry.isDirectory()) {
          await this.countFiles(fullPath, metrics, depth + 1);
        } else if (entry.isFile()) {
          metrics.totalFiles++;
          
          const stats = await fs.stat(fullPath);
          if (!metrics.lastModified || stats.mtime > new Date(metrics.lastModified)) {
            metrics.lastModified = stats.mtime.toISOString();
          }
          
          // Categorize file
          if (this.isCodeFile(entry.name)) metrics.codeFiles++;
          else if (this.isTestFile(entry.name)) metrics.testFiles++;
          else if (this.isDocumentationFile(entry.name)) metrics.documentationFiles++;
          else if (this.isConfigFile(entry.name)) metrics.configFiles++;
        }
      }
    } catch (error) {
      // Error counting files
    }
  }

  // Utility Methods and Extraction Functions

  shouldSkipEntry(name) {
    const skipPatterns = [
      'node_modules', '.git', '.next', 'dist', 'build', 'coverage',
      '.cache', '.tmp', '.temp', 'vendor', '__pycache__'
    ];
    return name.startsWith('.') || skipPatterns.includes(name);
  }

  isCodeFile(filename) {
    const extensions = ['.ts', '.js', '.tsx', '.jsx', '.py', '.php', '.go', '.rs', '.java'];
    return extensions.some(ext => filename.endsWith(ext)) && !this.isTestFile(filename);
  }

  isTestFile(filename) {
    return filename.includes('.test.') || filename.includes('.spec.') || filename.includes('__tests__');
  }

  isDocumentationFile(filename) {
    const extensions = ['.md', '.rst', '.txt'];
    return extensions.some(ext => filename.toLowerCase().endsWith(ext));
  }

  isConfigFile(filename) {
    const configFiles = [
      'package.json', 'tsconfig.json', 'jest.config.js', 'webpack.config.js',
      'vite.config.ts', 'rollup.config.js', '.eslintrc.js', '.babelrc',
      'Dockerfile', 'docker-compose.yml', '.gitlab-ci.yml'
    ];
    return configFiles.includes(filename) || filename.endsWith('.config.js') || filename.endsWith('.config.ts');
  }

  isReactComponent(filename) {
    return (filename.endsWith('.tsx') || filename.endsWith('.jsx')) && 
           !this.isTestFile(filename) && 
           /^[A-Z]/.test(path.basename(filename, path.extname(filename)));
  }

  categorizeFiles(entries) {
    const types = {};
    for (const entry of entries) {
      const ext = path.extname(entry).toLowerCase();
      types[ext] = (types[ext] || 0) + 1;
    }
    return types;
  }

  extractEndpoints(spec) {
    const endpoints = [];
    if (spec.paths) {
      for (const [path, methods] of Object.entries(spec.paths)) {
        for (const [method, details] of Object.entries(methods)) {
          if (typeof details === 'object' && details.operationId) {
            endpoints.push({
              path,
              method: method.toUpperCase(),
              operationId: details.operationId,
              summary: details.summary,
              tags: details.tags
            });
          }
        }
      }
    }
    return endpoints;
  }

  extractRouteMethods(content) {
    const methods = [];
    const routeRegex = /router\.(get|post|put|delete|patch)\s*\(\s*['"`]([^'"`]+)['"`]/g;
    let match;
    
    while ((match = routeRegex.exec(content)) !== null) {
      methods.push({
        method: match[1].toUpperCase(),
        path: match[2]
      });
    }
    
    return methods;
  }

  extractControllerMethods(content) {
    const methods = [];
    const methodRegex = /(?:async\s+)?(\w+)\s*\([^)]*\)\s*[:{]/g;
    let match;
    
    while ((match = methodRegex.exec(content)) !== null) {
      if (!['constructor', 'toString', 'valueOf'].includes(match[1])) {
        methods.push(match[1]);
      }
    }
    
    return methods;
  }

  extractDockerBaseImage(dockerContent) {
    const fromMatch = dockerContent.match(/^FROM\s+([^\s]+)/m);
    return fromMatch ? fromMatch[1] : 'unknown';
  }

  // Placeholder methods for collectors (to be implemented as needed)
  async collectMiddleware(projectPath) { return []; }
  async collectServices(projectPath) { return []; }
  async collectPerformanceMetrics(projectPath) { return {}; }
  async collectModelInfo(projectPath) { return []; }
  async collectAIProviders(projectPath) { return []; }
  async collectAlgorithms(projectPath) { return []; }
  async collectTrainingConfig(projectPath) { return {}; }
  async collectInferenceConfig(projectPath) { return {}; }
  async collectProtocolDefinitions(projectPath) { return []; }
  async collectGRPCServices(projectPath) { return []; }
  async collectMCPConfiguration(projectPath) { return {}; }
  async collectToolDefinitions(projectPath) { return []; }
  async collectReactHooks(projectPath) { return []; }
  async collectStyleConfiguration(projectPath) { return {}; }
  async collectStorybookStories(projectPath) { return []; }
  async collectFrontendTests(projectPath) { return []; }
  async collectSDKMethods(projectPath) { return []; }
  async collectTypeDefinitions(projectPath) { return []; }
  async collectUsageExamples(projectPath) { return []; }
  async collectGenerationConfig(projectPath) { return {}; }
  async collectCLIPlugins(projectPath) { return []; }
  async collectCLIConfigs(projectPath) { return []; }
  async collectAutomationScripts(projectPath) { return []; }
  async collectCIComponents(projectPath) { return []; }
  async collectCITemplates(projectPath) { return []; }
  async collectPipelineConfigs(projectPath) { return []; }
  async collectTestCoverage(projectPath) { return {}; }
  async collectDrupalRecipes(projectPath) { return []; }
  async collectDrupalEntities(projectPath) { return []; }
  async collectDrupalServices(projectPath) { return []; }
  async collectDrupalHooks(projectPath) { return []; }
  async extractDrupalHooksFromModule(modulePath) { return []; }
  async extractDrupalModuleServices(modulePath) { return []; }
  async extractDrupalRoutes(modulePath) { return []; }
  async extractDrupalPermissions(modulePath) { return []; }
  
  extractRouteMiddleware(content) { return []; }
  extractControllerExports(content) { return []; }
  extractCommandDescription(content) { return ''; }
  extractCommandOptions(content) { return []; }
  extractCommandExamples(content) { return []; }
  extractComponentProps(content) { return []; }
  extractComponentExports(content) { return []; }
  extractUsedHooks(content) { return []; }
}

module.exports = EnhancedCollectors;