#!/bin/bash

# Script to merge files from feature branches into merge-feature-branches for bfdocs

# Set up some variables
REPO_PATH="./bfdocs"
TARGET_BRANCH="merge-feature-branches"
TEMP_DIR="${REPO_PATH}/temp_merge_files"

# Create a temporary directory for files
mkdir -p "${TEMP_DIR}"

# Function to copy specific files from a branch
copy_files_from_branch() {
    local branch=$1
    local file_pattern=$2
    local dest_dir=$3
    
    echo "Copying $file_pattern from $branch branch to $dest_dir"
    
    # Switch to the source branch
    git checkout "$branch"
    
    # Find and copy files matching the pattern
    find . -path "$file_pattern" -type f | while read file; do
        target_file="${TEMP_DIR}/${file#./}"
        mkdir -p "$(dirname "$target_file")"
        cp "$file" "$target_file"
        echo "  Copied: $file"
    done
    
    # Switch back to target branch
    git checkout "$TARGET_BRANCH"
    
    # Copy files from temp directory to destination
    if [ -n "$(find "${TEMP_DIR}" -type f)" ]; then
        cp -r "${TEMP_DIR}"/* "${dest_dir}/"
        echo "  Files copied to destination"
    else
        echo "  No matching files found"
    fi
    
    # Clean up temp directory
    rm -rf "${TEMP_DIR:?}"/*
}

# Main script execution
cd "${REPO_PATH}" || { echo "Could not change to repository directory"; exit 1; }

# Make sure we start on the target branch
git checkout "${TARGET_BRANCH}"

# Create temporary directory
mkdir -p "${TEMP_DIR}"

echo "Starting feature branch merges..."

# Fetch remote branches
git fetch origin

# Copy VitePress 1.0 updates
echo "Processing VitePress 1.0 branch..."
copy_files_from_branch "origin/feature/vitepress-1.0" "./.vitepress/*" "${REPO_PATH}"
copy_files_from_branch "origin/feature/vitepress-1.0" "./docs/*" "${REPO_PATH}"
copy_files_from_branch "origin/feature/vitepress-1.0" "./src/*" "${REPO_PATH}"
copy_files_from_branch "origin/feature/vitepress-1.0" "./*.json" "${REPO_PATH}"
copy_files_from_branch "origin/feature/vitepress-1.0" "./*.js" "${REPO_PATH}"

# Copy configuration fixes
echo "Processing local changes branch..."
copy_files_from_branch "origin/feature/keep-local-changes" "./*.config.*" "${REPO_PATH}"
copy_files_from_branch "origin/feature/keep-local-changes" "./jest.*" "${REPO_PATH}"
copy_files_from_branch "origin/feature/keep-local-changes" "./babel.*" "${REPO_PATH}"

# Copy validation fixes
echo "Processing validation fixes branch..."
copy_files_from_branch "origin/validation-fixes-20250415" "./scripts/*" "${REPO_PATH}"
copy_files_from_branch "origin/validation-fixes-20250415" "./tests/*" "${REPO_PATH}"

# Clean up
rm -rf "${TEMP_DIR}"

echo "Feature branch merges completed!"