#!/usr/bin/env node

/**
 * Simple DB Schema Document Generator
 * 
 * This script creates a database schema documentation file in the docs directory.
 */

const fs = require('fs');
const path = require('path');
const { execSync } = require('child_process');

// Configuration
const PROJECT_ROOT = process.cwd();
const DOCS_DIR = path.join(PROJECT_ROOT, 'docs');
const TEMPLATES_DIR = path.join(__dirname, 'templates');
const DATE_FORMAT = new Intl.DateTimeFormat('en-US', { 
  year: 'numeric', 
  month: 'long', 
  day: 'numeric' 
});

// Command line arguments
const args = process.argv.slice(2);
const dbName = args[0] || 'sample_database';
const category = args[1] || 'Database Schemas';

// Ensure the docs directory exists
if (!fs.existsSync(DOCS_DIR)) {
  fs.mkdirSync(DOCS_DIR, { recursive: true });
  console.log(`Created directory: ${DOCS_DIR}`);
}

// Create category directory if it doesn't exist
const categoryDir = path.join(DOCS_DIR, category.toLowerCase().replace(/\s+/g, '_'));
if (!fs.existsSync(categoryDir)) {
  fs.mkdirSync(categoryDir, { recursive: true });
  console.log(`Created category directory: ${categoryDir}`);
  
  // Create README.md in the category directory if it doesn't exist
  const categoryReadmePath = path.join(categoryDir, 'README.md');
  if (!fs.existsSync(categoryReadmePath)) {
    const categoryReadmeContent = `# ${category}\n\nThis directory contains documentation for ${category.toLowerCase()} in the Bluefly AI Platform.\n\n## Documents\n\n`;
    fs.writeFileSync(categoryReadmePath, categoryReadmeContent);
    console.log(`Created category README.md at ${categoryReadmePath}`);
  }
}

// Convert name to proper file format
const fileName = dbName.toLowerCase().replace(/\s+/g, '_') + '.md';
const filePath = path.join(categoryDir, fileName);

// Check if file already exists
if (fs.existsSync(filePath)) {
  console.error(`Error: File ${fileName} already exists at ${filePath}`);
  process.exit(1);
}

// Create DB Schema content
const title = dbName.split('_').map(word => word.charAt(0).toUpperCase() + word.slice(1)).join(' ');
const content = `# ${title} Database Schema

This document describes the database schema for ${title.toLowerCase()} in the Bluefly AI Platform.

## Overview

The ${title} database stores information about LLM platform models, users, configurations, and interactions.

## Entity Relationship Diagram

\`\`\`
┌───────────────────┐     ┌───────────────────┐
│                   │     │                   │
│  Models           │─┬──►│  ModelVersions    │
│                   │ │   │                   │
└───────────────────┘ │   └───────────────────┘
                      │      │
                      │      ▼
                      │   ┌───────────────────┐
                      └──►│                   │
                          │  Deployments      │
                          │                   │
                          └───────────────────┘
\`\`\`

## Tables

### Models

Stores the core model information.

| Column | Type | Constraints | Description |
|--------|------|-------------|-------------|
| id | INTEGER | PRIMARY KEY, AUTO_INCREMENT | Unique identifier for the model |
| name | VARCHAR(100) | NOT NULL, UNIQUE | Model name |
| provider | VARCHAR(50) | NOT NULL | AI provider (e.g., OpenAI, Anthropic) |
| base_model | VARCHAR(100) | NOT NULL | Base model identifier |
| description | TEXT | NULL | Model description |
| capabilities | JSON | NULL | JSON object defining model capabilities |
| created_at | TIMESTAMP | NOT NULL, DEFAULT CURRENT_TIMESTAMP | Record creation timestamp |
| updated_at | TIMESTAMP | NOT NULL, DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP | Last update timestamp |
| is_active | BOOLEAN | NOT NULL, DEFAULT TRUE | Whether the model is active |

#### Indexes

- PRIMARY KEY (id)
- UNIQUE INDEX idx_model_name (name)
- INDEX idx_provider (provider)
- INDEX idx_base_model (base_model)
- INDEX idx_is_active (is_active)

### ModelVersions

Stores different versions of each model.

| Column | Type | Constraints | Description |
|--------|------|-------------|-------------|
| id | INTEGER | PRIMARY KEY, AUTO_INCREMENT | Unique identifier for the model version |
| model_id | INTEGER | NOT NULL, FOREIGN KEY | Reference to models.id |
| version | VARCHAR(50) | NOT NULL | Version identifier |
| parameters | BIGINT | NULL | Number of parameters |
| context_window | INTEGER | NULL | Context window size in tokens |
| input_token_cost | DECIMAL(10,6) | NULL | Cost per input token |
| output_token_cost | DECIMAL(10,6) | NULL | Cost per output token |
| created_at | TIMESTAMP | NOT NULL, DEFAULT CURRENT_TIMESTAMP | Record creation timestamp |
| updated_at | TIMESTAMP | NOT NULL, DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP | Last update timestamp |
| is_default | BOOLEAN | NOT NULL, DEFAULT FALSE | Whether this is the default version |

#### Indexes

- PRIMARY KEY (id)
- FOREIGN KEY fk_model_version (model_id) REFERENCES models(id) ON DELETE CASCADE
- UNIQUE INDEX idx_model_version (model_id, version)
- INDEX idx_is_default (is_default)

## Relationships

- One-to-Many: Each Models record can have multiple ModelVersions records
- One-to-Many: Each Models record can have multiple Deployments records
- Many-to-One: Each Deployments record belongs to one Models record

## Data Flow

1. New models are added to the Models table
2. Model versions are tracked in the ModelVersions table with relationship to the parent model
3. Deployments are created referencing a specific model
4. User interactions query model capabilities and version information to determine optimal models for specific tasks

## Migration Considerations

- Add new columns with NULL constraint first, then populate data, then add NOT NULL constraints if needed
- Use transaction-safe operations for schema changes
- Schedule migrations during low-traffic periods
- Create backup before significant schema changes
- Consider using tools like Liquibase or Flyway for migration management

## Query Examples

\`\`\`sql
-- Get all active models with their default versions
SELECT m.id, m.name, m.provider, m.base_model, mv.version, mv.parameters, mv.context_window
FROM models m
JOIN model_versions mv ON m.id = mv.model_id
WHERE m.is_active = TRUE AND mv.is_default = TRUE
ORDER BY m.provider, m.name;

-- Find models with specific capabilities
SELECT m.id, m.name, m.provider, m.base_model
FROM models m
WHERE JSON_EXTRACT(m.capabilities, '$.vision') = TRUE
  AND JSON_EXTRACT(m.capabilities, '$.streaming') = TRUE;
\`\`\`

## Performance Considerations

- Keep indexed columns as small as possible
- Use appropriate data types (e.g., INT for IDs, not VARCHAR)
- Consider partitioning the model_versions table if it grows very large
- Use EXPLAIN to analyze query performance
- Set appropriate buffer pool size for InnoDB
- Consider using covering indexes for frequent queries
- Monitor slow queries and optimize as needed

## Security Considerations

- Implement proper access controls and least privilege
- Audit sensitive operations (model activation/deactivation)
- Use TLS for database connections
- Implement row-level security if needed for multi-tenant deployments
- Regularly update and patch database software
- Ensure proper backup and recovery procedures

## Related Documentation

- [Model Registry API](../api_documentation/model_registry_api.md)
- [Deployment System](../core_documentation/deployment_system.md)

## References

- [Database Design Best Practices](https://example.com/db-best-practices)
- [MySQL Performance Tuning](https://example.com/mysql-performance)
- [Security Guidelines](https://example.com/security-guidelines)

---

**Version**: 1.0.0  
**Last Updated**: ${DATE_FORMAT.format(new Date())}  
**Status**: Draft`;

// Write the file
fs.writeFileSync(filePath, content);
console.log(`Created database schema documentation file: ${filePath}`);

// Add to category README.md
const categoryReadmePath = path.join(categoryDir, 'README.md');
if (fs.existsSync(categoryReadmePath)) {
  let categoryReadme = fs.readFileSync(categoryReadmePath, 'utf8');
  
  // Find the documents section
  const documentsSectionRegex = /(## Documents\n\n)([\s\S]*?)(\n\n##|$)/;
  const documentsSectionMatch = categoryReadme.match(documentsSectionRegex);
  
  if (documentsSectionMatch) {
    // Documents section exists, add the file to it if not already present
    const documentsContent = documentsSectionMatch[2];
    if (!documentsContent.includes(`[${title}](./${fileName})`)) {
      const newDocumentsContent = `${documentsContent}- [${title} Database Schema](./${fileName})\n`;
      categoryReadme = categoryReadme.replace(documentsSectionMatch[0], `${documentsSectionMatch[1]}${newDocumentsContent}${documentsSectionMatch[3]}`);
    }
  } else {
    // Documents section doesn't exist, create it
    categoryReadme += `\n## Documents\n\n- [${title} Database Schema](./${fileName})\n`;
  }
  
  // Write the updated category README.md
  fs.writeFileSync(categoryReadmePath, categoryReadme);
  console.log(`Added ${fileName} to ${categoryReadmePath}`);
}

// Add category to main README.md
const docsReadmePath = path.join(DOCS_DIR, 'README.md');
if (!fs.existsSync(docsReadmePath)) {
  // Create main README.md if it doesn't exist
  const docsReadmeContent = `# Bluefly AI Platform Documentation

This directory contains comprehensive documentation for the Bluefly AI Platform.

## Documentation Categories

### [${category}](${path.relative(DOCS_DIR, path.join(categoryDir, 'README.md'))})

`;
  fs.writeFileSync(docsReadmePath, docsReadmeContent);
  console.log(`Created main docs README.md at ${docsReadmePath}`);
} else {
  // Update existing main README.md
  let docsReadme = fs.readFileSync(docsReadmePath, 'utf8');
  
  // Check if category is already in the README
  const categoryLinkRegex = new RegExp(`\\[${category}\\]\\(${path.relative(DOCS_DIR, path.join(categoryDir, 'README.md')).replace(/\//g, '\\/')}\\)`);
  if (!docsReadme.match(categoryLinkRegex)) {
    // Find the categories section
    const categoriesSectionRegex = /(## Documentation Categories\n\n)([\s\S]*?)(\n\n##|$)/;
    const categoriesSectionMatch = docsReadme.match(categoriesSectionRegex);
    
    if (categoriesSectionMatch) {
      // Categories section exists, add the category to it
      const categoriesContent = categoriesSectionMatch[2];
      const newCategoriesContent = `${categoriesContent}### [${category}](${path.relative(DOCS_DIR, path.join(categoryDir, 'README.md'))})\n\n`;
      docsReadme = docsReadme.replace(categoriesSectionMatch[0], `${categoriesSectionMatch[1]}${newCategoriesContent}${categoriesSectionMatch[3]}`);
    } else {
      // Categories section doesn't exist, create it
      docsReadme += `\n## Documentation Categories\n\n### [${category}](${path.relative(DOCS_DIR, path.join(categoryDir, 'README.md'))})\n\n`;
    }
    
    // Write the updated main README.md
    fs.writeFileSync(docsReadmePath, docsReadme);
    console.log(`Added category ${category} to main docs README.md`);
  }
}

console.log(`Database schema documentation created successfully!`);

// Make the script executable
if (require.main === module) {
  try {
    // Make the script executable
    if (process.platform !== 'win32') {
      execSync(`chmod +x ${__filename}`);
    }
  } catch (error) {
    console.error('Error making script executable:', error);
  }
}