#!/usr/bin/env node

/**
 * Project Documentation Synchronizer
 * 
 * This script ensures all projects have complete /docs directories
 * with architectural documentation templates and keeps them in sync.
 */

const fs = require('fs').promises;
const path = require('path');
const DocumentationAggregator = require('./aggregate-docs');

class ProjectDocsSyncer {
  constructor(configPath = './docs-config.json') {
    this.aggregator = new DocumentationAggregator(configPath);
  }

  async syncAllProjects() {
    console.log('🔄 Synchronizing project documentation...');
    
    await this.aggregator.loadConfig();
    
    for (const [projectKey, projectConfig] of Object.entries(this.aggregator.config.projects)) {
      try {
        await this.syncProject(projectKey, projectConfig);
      } catch (error) {
        console.error(`❌ Failed to sync ${projectKey}:`, error.message);
      }
    }
    
    console.log('✅ Documentation synchronization complete!');
  }

  async syncProject(projectKey, projectConfig) {
    console.log(`🔄 Syncing ${projectKey}...`);
    
    const projectPath = path.resolve(projectConfig.path);
    const docsPath = path.join(projectPath, projectConfig.docsPath);
    
    // Ensure docs directory exists
    await fs.mkdir(docsPath, { recursive: true });
    
    // Create standard documentation structure
    await this.ensureDocumentationStructure(projectKey, projectConfig, docsPath);
    
    // Update documentation templates if needed
    await this.updateDocumentationTemplates(projectKey, projectConfig, docsPath);
    
    console.log(`✅ Synced ${projectKey}`);
  }

  async ensureDocumentationStructure(projectKey, projectConfig, docsPath) {
    const standardStructure = [
      'index.md',
      'architecture.md', 
      'api.md',
      'setup.md',
      'examples',
      'integration',
      'troubleshooting'
    ];

    for (const item of standardStructure) {
      const itemPath = path.join(docsPath, item);
      
      try {
        await fs.access(itemPath);
        console.log(`📄 ${item} exists for ${projectKey}`);
      } catch (e) {
        if (item.includes('.')) {
          // It's a file - create from template
          const content = await this.getTemplateContent(item, projectKey, projectConfig);
          await fs.writeFile(itemPath, content);
          console.log(`📄 Created ${item} for ${projectKey}`);
        } else {
          // It's a directory
          await fs.mkdir(itemPath, { recursive: true });
          
          // Create README.md in the directory
          const readmeContent = await this.getDirectoryReadmeContent(item, projectKey, projectConfig);
          await fs.writeFile(path.join(itemPath, 'README.md'), readmeContent);
          console.log(`📁 Created ${item}/ directory for ${projectKey}`);
        }
      }
    }
  }

  async updateDocumentationTemplates(projectKey, projectConfig, docsPath) {
    // Check if templates need updating based on project changes
    const projectPath = path.resolve(projectConfig.path);
    
    try {
      // Extract current project information
      const currentInfo = await this.extractProjectInfo(projectPath);
      
      // Update architecture.md if project structure changed
      const archPath = path.join(docsPath, 'architecture.md');
      const archContent = await fs.readFile(archPath, 'utf-8');
      
      if (this.needsArchitectureUpdate(archContent, currentInfo)) {
        const updatedContent = await this.updateArchitectureDoc(archContent, currentInfo, projectKey, projectConfig);
        await fs.writeFile(archPath, updatedContent);
        console.log(`📄 Updated architecture.md for ${projectKey}`);
      }
      
    } catch (error) {
      console.warn(`⚠️  Could not update templates for ${projectKey}:`, error.message);
    }
  }

  async extractProjectInfo(projectPath) {
    const info = {
      hasPackageJson: false,
      hasDockerfile: false,
      hasKubernetes: false,
      dependencies: [],
      scripts: [],
      technologies: []
    };

    try {
      // Check for package.json
      const packageJsonPath = path.join(projectPath, 'package.json');
      try {
        const packageContent = await fs.readFile(packageJsonPath, 'utf-8');
        const packageJson = JSON.parse(packageContent);
        info.hasPackageJson = true;
        info.dependencies = Object.keys(packageJson.dependencies || {});
        info.scripts = Object.keys(packageJson.scripts || {});
      } catch (e) {
        // No package.json
      }

      // Check for Dockerfile
      try {
        await fs.access(path.join(projectPath, 'Dockerfile'));
        info.hasDockerfile = true;
        info.technologies.push('Docker');
      } catch (e) {
        // No Dockerfile
      }

      // Check for Kubernetes
      try {
        await fs.access(path.join(projectPath, 'k8s'));
        info.hasKubernetes = true;
        info.technologies.push('Kubernetes');
      } catch (e) {
        // No k8s directory
      }

      // Detect other technologies
      const techFiles = [
        { file: 'tsconfig.json', tech: 'TypeScript' },
        { file: 'jest.config.js', tech: 'Jest' },
        { file: 'babel.config.js', tech: 'Babel' },
        { file: '.gitlab-ci.yml', tech: 'GitLab CI' },
        { file: 'docker-compose.yml', tech: 'Docker Compose' }
      ];

      for (const { file, tech } of techFiles) {
        try {
          await fs.access(path.join(projectPath, file));
          info.technologies.push(tech);
        } catch (e) {
          // File doesn't exist
        }
      }

    } catch (error) {
      console.warn('Could not extract project info:', error.message);
    }

    return info;
  }

  needsArchitectureUpdate(content, currentInfo) {
    // Simple heuristic - check if major technologies are mentioned
    const mentionedTechs = currentInfo.technologies.filter(tech => 
      content.toLowerCase().includes(tech.toLowerCase())
    );
    
    // Update if less than half of detected technologies are mentioned
    return mentionedTechs.length < currentInfo.technologies.length / 2;
  }

  async updateArchitectureDoc(content, currentInfo, projectKey, projectConfig) {
    // Update the technical stack section
    const techStackRegex = /## Technical Stack([\\s\\S]*?)##/;
    const match = content.match(techStackRegex);
    
    if (match) {
      const newTechStack = `## Technical Stack

${projectConfig.architecture ? `
- **Language**: ${projectConfig.architecture.language || 'Not specified'}
- **Port**: ${projectConfig.architecture.port || 'N/A'}
- **Type**: ${projectConfig.architecture.type || 'Not specified'}
${projectConfig.architecture.database ? `- **Database**: ${projectConfig.architecture.database}` : ''}
` : ''}
${currentInfo.technologies.length > 0 ? `
- **Technologies**: ${currentInfo.technologies.join(', ')}
` : ''}
${currentInfo.hasDockerfile ? '- **Containerization**: Docker' : ''}
${currentInfo.hasKubernetes ? '- **Orchestration**: Kubernetes' : ''}

##`;

      return content.replace(techStackRegex, newTechStack);
    }
    
    return content;
  }

  async getTemplateContent(filename, projectKey, projectConfig) {
    // Reuse templates from aggregator
    switch (filename) {
      case 'index.md':
        return this.aggregator.generateProjectIndexTemplate(projectKey, projectConfig);
      case 'architecture.md':
        return this.aggregator.generateArchitectureTemplate(projectKey, projectConfig);
      case 'api.md':
        return this.aggregator.generateApiTemplate(projectKey, projectConfig);
      case 'setup.md':
        return this.aggregator.generateSetupTemplate(projectKey, projectConfig);
      default:
        return this.generateGenericTemplate(filename, projectKey, projectConfig);
    }
  }

  async getDirectoryReadmeContent(dirName, projectKey, projectConfig) {
    const templates = {
      'examples': `# ${projectConfig.name} - Examples

This directory contains usage examples and tutorials for ${projectConfig.name}.

## Available Examples

- [Basic Usage](./basic-usage.md) - Getting started with ${projectConfig.name}
- [Advanced Configuration](./advanced-config.md) - Advanced setup and configuration
- [Integration Examples](./integration.md) - Integration with other services

## Running Examples

Follow the setup instructions in [../setup.md](../setup.md) first, then:

\`\`\`bash
# Navigate to project directory
cd ${projectKey}

# Run specific example
npm run example:basic  # or relevant command
\`\`\`

## Contributing Examples

When adding new examples:

1. Create a new .md file with descriptive name
2. Include working code samples
3. Add setup instructions
4. Update this README with link to your example

---

*Part of the [${projectConfig.name}](../index.md) documentation.*
`,

      'integration': `# ${projectConfig.name} - Integration Guide

This directory contains integration documentation for ${projectConfig.name}.

## Integration Types

### Internal Services
${projectConfig.architecture?.dependencies ? 
  projectConfig.architecture.dependencies.map(dep => `- [${dep} Integration](./internal/${dep}.md)`).join('\\n') :
  '- No internal service dependencies'
}

### External Services
- [Third-party APIs](./external/apis.md)
- [Webhooks](./external/webhooks.md)
- [Authentication](./external/auth.md)

### Platform Integration
- [Drupal Integration](./platform/drupal.md)
- [GitLab CI/CD](./platform/gitlab.md)
- [Kubernetes](./platform/kubernetes.md)

## Integration Patterns

- [Event-driven Integration](./patterns/events.md)
- [API-first Integration](./patterns/api.md)
- [Data Synchronization](./patterns/sync.md)

---

*Part of the [${projectConfig.name}](../index.md) documentation.*
`,

      'troubleshooting': `# ${projectConfig.name} - Troubleshooting

Common issues and solutions for ${projectConfig.name}.

## Common Issues

### Startup Problems

**Service won't start**
\`\`\`bash
# Check if port is available
netstat -tulpn | grep ${projectConfig.architecture?.port || '3000'}

# Check environment variables
printenv | grep ${projectKey.toUpperCase()}

# Check logs
${projectConfig.architecture?.containerized ? `docker logs ${projectKey}` : 'tail -f logs/app.log'}
\`\`\`

**Dependencies not found**
\`\`\`bash
# Verify all dependencies are running
docker-compose ps

# Restart dependencies
docker-compose restart
\`\`\`

### Configuration Issues

**Environment variables not loaded**
\`\`\`bash
# Check .env file exists
ls -la .env

# Verify syntax
cat .env | grep -v '^#' | grep '='
\`\`\`

**Database connection failed**
\`\`\`bash
# Test database connection
${projectConfig.architecture?.database ? 'pg_isready -h localhost -p 5432' : 'echo "No database configured"'}
\`\`\`

### Performance Issues

**High memory usage**
- Check for memory leaks in application logs
- Monitor with \`htop\` or \`docker stats\`
- Review configuration for memory limits

**Slow response times**
- Enable debug logging
- Check database query performance
- Review network connectivity to dependencies

## Debug Mode

Enable debug logging:

\`\`\`bash
${projectConfig.architecture?.language === 'TypeScript/Node.js' ? 'DEBUG=* npm start' : 'LOG_LEVEL=debug npm start'}
\`\`\`

## Health Checks

\`\`\`bash
# Basic health check
curl http://localhost:${projectConfig.architecture?.port || '3000'}/health

# Detailed diagnostics
curl http://localhost:${projectConfig.architecture?.port || '3000'}/diagnostics
\`\`\`

## Getting Help

- **Issues**: [${projectConfig.repository}/issues](${projectConfig.repository}/issues)
- **Documentation**: [Platform Hub](https://gitlab.bluefly.io/ecosystem/bfdocs)
- **Logs**: Check application and system logs for detailed error messages

---

*Part of the [${projectConfig.name}](../index.md) documentation.*
`
    };

    return templates[dirName] || `# ${projectConfig.name} - ${dirName.charAt(0).toUpperCase() + dirName.slice(1)}

Documentation for ${dirName} related to ${projectConfig.name}.

---

*Part of the [${projectConfig.name}](../index.md) documentation.*
`;
  }

  generateGenericTemplate(filename, projectKey, projectConfig) {
    const title = filename.replace('.md', '').split('-').map(word => 
      word.charAt(0).toUpperCase() + word.slice(1)
    ).join(' ');

    return `---
title: ${projectConfig.name} - ${title}
description: ${title} documentation for ${projectConfig.name}
category: ${projectConfig.category}
---

# ${projectConfig.name} - ${title}

${title} documentation for ${projectConfig.description}

## Overview

[Add overview content here]

## Details

[Add detailed content here]

---

*This documentation is part of the [${projectConfig.name}](./index.md) project in the [Bluefly LLM Platform](https://gitlab.bluefly.io/ecosystem/bfdocs) ecosystem.*
`;
  }
}

// Run if called directly
if (require.main === module) {
  const syncer = new ProjectDocsSyncer();
  syncer.syncAllProjects().catch(console.error);
}

module.exports = ProjectDocsSyncer;