#!/usr/bin/env node

/**
 * @file update-docs-structure.js
 * @description Updates documentation structure with OpenAPI compliance and proper organization
 */

const fs = require('fs').promises;
const path = require('path');
const { execSync } = require('child_process');

const DOCS_ROOT = path.resolve(__dirname, '../docs');
const CLAUDE_DOCS = path.resolve(__dirname, '../../.claude');

// OpenAPI-compliant documentation structure
const DOCS_STRUCTURE = {
  'openapi': {
    'index.yaml': null,
    'components.yaml': null,
    'paths.yaml': null,
    'schemas.yaml': null
  },
  'architecture': {
    'index.md': null,
    'emergent-consciousness.md': null,
    'system-design.md': null,
    'security.md': null,
    'performance.md': null,
    'data': {
      'schema.md': null,
      'models.md': null
    }
  },
  'development': {
    'index.md': null,
    'environment.md': null,
    'testing.md': null,
    'best-practices.md': null,
    'configuration.md': null,
    'tdd': {
      'index.md': null,
      'enforcement.md': null,
      'coverage.md': null
    }
  },
  'integration': {
    'index.md': null,
    'drupal': {
      'index.md': null,
      'modules.md': null,
      'recipes.md': null,
      'field-eca.md': null
    },
    'openapi': {
      'index.md': null,
      'validation.md': null,
      'security.md': null
    },
    'monitoring': {
      'index.md': null,
      'metrics.md': null,
      'alerts.md': null
    }
  },
  'operations': {
    'index.md': null,
    'deployment': {
      'index.md': null,
      'environments.md': null,
      'ci-cd.md': null
    },
    'monitoring': {
      'index.md': null,
      'observability.md': null,
      'logging.md': null
    },
    'security': {
      'index.md': null,
      'compliance.md': null,
      'auditing.md': null
    },
    'performance': {
      'index.md': null,
      'optimization.md': null,
      'scaling.md': null
    }
  },
  'reference': {
    'index.md': null,
    'api': {
      'index.md': null,
      'endpoints.md': null,
      'authentication.md': null,
      'errors.md': null
    },
    'cli': {
      'index.md': null,
      'commands.md': null,
      'configuration.md': null
    },
    'recipes': {
      'index.md': null,
      'drupal.md': null,
      'llm.md': null,
      'integration.md': null
    }
  }
};

// File mapping from .claude to new structure
const FILE_MAPPING = {
  'service_architecture.md': 'architecture/system-design.md',
  'component_relationships.md': 'architecture/emergent-consciousness.md',
  'database_schema.md': 'architecture/data/schema.md',
  'project_organization.md': 'architecture/index.md',
  'development_commands.md': 'reference/cli/commands.md',
  'contributing.md': 'development/best-practices.md',
  'testing_requirements.md': 'development/testing.md',
  'TEST_COVERAGE_BEST_PRACTICES.md': 'development/tdd/coverage.md',
  'tdd_enforcement.md': 'development/tdd/enforcement.md',
  'api_guidelines.md': 'integration/openapi/index.md',
  'monitoring_observability.md': 'operations/monitoring/observability.md',
  'security_guidelines.md': 'operations/security/index.md',
  'deployment_guides.md': 'operations/deployment/index.md',
  'troubleshooting.md': 'operations/monitoring/index.md',
  'performance_optimization.md': 'operations/performance/optimization.md',
  'drupal_llm_platform.md': 'integration/drupal/index.md',
  'field_eca_bridge.md': 'integration/drupal/field-eca.md'
};

async function createDirectoryStructure() {
  console.log('Creating OpenAPI-compliant documentation structure...');
  
  for (const [dir, subdirs] of Object.entries(DOCS_STRUCTURE)) {
    const dirPath = path.join(DOCS_ROOT, dir);
    await fs.mkdir(dirPath, { recursive: true });
    
    if (typeof subdirs === 'object') {
      for (const [subdir, _] of Object.entries(subdirs)) {
        if (subdir.endsWith('.md') || subdir.endsWith('.yaml')) continue;
        await fs.mkdir(path.join(dirPath, subdir), { recursive: true });
      }
    }
  }
}

async function createOpenAPIFiles() {
  console.log('Creating OpenAPI specification files...');
  
  const openapiBase = {
    openapi: '3.1.0',
    info: {
      title: 'Bluefly.io LLM Platform API',
      version: '1.0.0',
      description: 'API documentation for the Bluefly.io LLM Platform'
    },
    servers: [
      {
        url: 'https://api.bluefly.io/v1',
        description: 'Production API server'
      }
    ]
  };

  // Create base OpenAPI files
  await fs.writeFile(
    path.join(DOCS_ROOT, 'openapi/index.yaml'),
    JSON.stringify(openapiBase, null, 2)
  );

  // Create component schemas
  const componentsBase = {
    components: {
      schemas: {},
      securitySchemes: {
        bearerAuth: {
          type: 'http',
          scheme: 'bearer',
          bearerFormat: 'JWT'
        }
      }
    }
  };

  await fs.writeFile(
    path.join(DOCS_ROOT, 'openapi/components.yaml'),
    JSON.stringify(componentsBase, null, 2)
  );
}

async function moveFiles() {
  console.log('Moving documentation files to new structure...');
  
  for (const [src, dest] of Object.entries(FILE_MAPPING)) {
    const srcPath = path.join(CLAUDE_DOCS, src);
    const destPath = path.join(DOCS_ROOT, dest);
    
    try {
      await fs.access(srcPath);
      await fs.mkdir(path.dirname(destPath), { recursive: true });
      await fs.copyFile(srcPath, destPath);
      console.log(`Moved ${src} to ${dest}`);
    } catch (error) {
      if (error.code === 'ENOENT') {
        console.warn(`Warning: Source file ${src} not found`);
      } else {
        console.error(`Error moving ${src}:`, error);
      }
    }
  }
}

async function validateOpenAPI() {
  console.log('Validating OpenAPI specifications...');
  
  try {
    execSync('npx @redocly/cli lint openapi/*.yaml', { stdio: 'inherit' });
  } catch (error) {
    console.error('OpenAPI validation failed:', error);
    process.exit(1);
  }
}

async function updateVuePressConfig() {
  console.log('Updating VuePress configuration...');
  
  const configPath = path.join(__dirname, '../.vuepress/config.js');
  const config = require(configPath);
  
  // Update sidebar structure to match new docs
  config.themeConfig.sidebar = {
    '/architecture/': [
      {
        title: 'Architecture',
        collapsable: false,
        children: [
          ['', 'Overview'],
          ['emergent-consciousness', 'Emergent Consciousness'],
          ['system-design', 'System Design'],
          ['data/schema', 'Data Schema'],
          ['security', 'Security'],
          ['performance', 'Performance']
        ]
      }
    ],
    '/development/': [
      {
        title: 'Development',
        collapsable: false,
        children: [
          ['', 'Overview'],
          ['environment', 'Environment'],
          ['testing', 'Testing'],
          ['best-practices', 'Best Practices'],
          ['configuration', 'Configuration'],
          ['tdd/', 'TDD Guide']
        ]
      }
    ],
    '/integration/': [
      {
        title: 'Integration',
        collapsable: false,
        children: [
          ['', 'Overview'],
          ['drupal/', 'Drupal Integration'],
          ['openapi/', 'OpenAPI Integration'],
          ['monitoring/', 'Monitoring']
        ]
      }
    ],
    '/operations/': [
      {
        title: 'Operations',
        collapsable: false,
        children: [
          ['', 'Overview'],
          ['deployment/', 'Deployment'],
          ['monitoring/', 'Monitoring'],
          ['security/', 'Security'],
          ['performance/', 'Performance']
        ]
      }
    ],
    '/reference/': [
      {
        title: 'Reference',
        collapsable: false,
        children: [
          ['', 'Overview'],
          ['api/', 'API Reference'],
          ['cli/', 'CLI Reference'],
          ['recipes/', 'Recipes']
        ]
      }
    ]
  };

  await fs.writeFile(
    configPath,
    `module.exports = ${JSON.stringify(config, null, 2)}`
  );
}

async function main() {
  try {
    await createDirectoryStructure();
    await createOpenAPIFiles();
    await moveFiles();
    await validateOpenAPI();
    await updateVuePressConfig();
    
    console.log('\nDocumentation structure update complete!');
    console.log('Next steps:');
    console.log('1. Review the updated documentation structure in bfdocs/docs/');
    console.log('2. Update OpenAPI specifications with actual API endpoints');
    console.log('3. Run "npm run docs:dev" to preview the documentation');
    console.log('4. Commit changes and push to GitLab');
  } catch (error) {
    console.error('Error during documentation update:', error);
    process.exit(1);
  }
}

if (require.main === module) {
  main();
}

module.exports = {
  createDirectoryStructure,
  createOpenAPIFiles,
  moveFiles,
  validateOpenAPI,
  updateVuePressConfig
}; 