/**
 * @module DocumentationAggregator
 * @description Service to aggregate documentation from all projects
 *
 * TODO: Implementation tasks:
 * - [ ] Review tests for aggregation functionality
 * - [ ] Implement validation of cross-references between projects
 * - [ ] Add support for generating search index with better keyword extraction
 * - [ ] Create integrated navigation across all project documentation
 * - [ ] Implement better content sanitization for safety
 * - [ ] Add link validation with external URL safety checks
 * - [ ] Run coverage report to verify 95%+ coverage
 */

const fs = require('fs').promises;
const path = require('path');

/**
 * Service for aggregating documentation from all projects
 * @class
 */
class DocumentationAggregator {
  /**
   * Create a DocumentationAggregator
   * @param {string} configPath - Path to configuration file
   */
  constructor(configPath = './docs-config.json') {
    this.configPath = configPath;
    this.config = null;
    this.stats = {
      projectsProcessed: 0,
      filesGenerated: 0,
      errors: []
    };
  }

  /**
   * Load configuration file
   * @returns {Promise<Object>} Configuration object
   * @throws {Error} If configuration cannot be loaded
   */
  async loadConfig() {
    try {
      const configContent = await fs.readFile(this.configPath, 'utf-8');
      this.config = JSON.parse(configContent);
      return this.config;
    } catch (error) {
      throw new Error(`Failed to load configuration: ${error.message}`);
    }
  }

  /**
   * Aggregate documentation from all projects
   * @param {string} outputDir - Output directory
   * @returns {Promise<Object>} Aggregation report
   */
  async aggregateAllProjects(outputDir = './public') {
    if (!this.config) {
      await this.loadConfig();
    }

    // Ensure output directory exists
    try {
      await fs.mkdir(outputDir, { recursive: true });
    } catch (error) {
      // Ignore if directory already exists
    }

    // Process all projects in parallel for better performance
    const projectsData = {};
    const projectEntries = Object.entries(this.config.projects);
    
    // Process projects in batches to avoid overwhelming the system
    const BATCH_SIZE = 3; // Process 3 projects concurrently
    
    for (let i = 0; i < projectEntries.length; i += BATCH_SIZE) {
      const batch = projectEntries.slice(i, i + BATCH_SIZE);
      
      const batchPromises = batch.map(async ([projectKey, projectConfig]) => {
        try {
          const projectData = await this.processProject(projectKey, projectConfig);
          return { projectKey, projectData, success: true };
        } catch (error) {
          this.stats.errors.push({
            project: projectKey,
            error: error.message
          });
          return { projectKey, error, success: false };
        }
      });
      
      const batchResults = await Promise.allSettled(batchPromises);
      
      // Collect successful results
      batchResults.forEach((result) => {
        if (result.status === 'fulfilled' && result.value.success) {
          projectsData[result.value.projectKey] = result.value.projectData;
          this.stats.projectsProcessed++;
        }
      });
    }

    // Generate projects.json
    await fs.writeFile(
      path.join(outputDir, 'projects.json'),
      JSON.stringify(projectsData, null, 2),
      'utf-8'
    );
    this.stats.filesGenerated++;

    // Generate search index
    await this.generateSearchIndex(projectsData, outputDir);
    this.stats.filesGenerated++;

    // Generate HTML files
    await this.generateHtmlFiles(projectsData, outputDir);

    return this.stats;
  }

  /**
   * Process a single project
   * @param {string} projectKey - Project key
   * @param {Object} projectConfig - Project configuration
   * @returns {Promise<Object>} Project data
   */
  async processProject(projectKey, projectConfig) {
    const projectPath = path.resolve(projectConfig.path);
    const docsPath = path.join(projectPath, projectConfig.docsPath || 'docs');
    
    // Get documentation files
    const files = await this.getDocumentationFiles(docsPath);
    
    // Process files and extract metadata
    const processedFiles = await this.processDocumentationFiles(files, docsPath, projectKey);
    
    return {
      name: projectConfig.name,
      description: projectConfig.description,
      category: projectConfig.category,
      version: projectConfig.version,
      repository: projectConfig.repository,
      architecture: projectConfig.architecture,
      files: processedFiles
    };
  }

  /**
   * Get documentation files from a directory
   * @param {string} docsPath - Documentation directory path
   * @returns {Promise<string[]>} List of file paths
   */
  async getDocumentationFiles(docsPath) {
    try {
      const files = await this.readDirRecursive(docsPath);
      return files.filter(file => file.endsWith('.md'));
    } catch (error) {
      return [];
    }
  }

  /**
   * Read directory recursively
   * @param {string} dir - Directory path
   * @param {string} [baseDir] - Base directory for relative paths
   * @returns {Promise<string[]>} List of file paths
   */
  async readDirRecursive(dir, baseDir = dir) {
    const entries = await fs.readdir(dir, { withFileTypes: true });
    
    const files = await Promise.all(entries.map(async entry => {
      const res = path.resolve(dir, entry.name);
      if (entry.isDirectory()) {
        return this.readDirRecursive(res, baseDir);
      } else {
        return res;
      }
    }));
    
    return files.flat();
  }

  /**
   * Process documentation files
   * @param {string[]} files - List of file paths
   * @param {string} docsPath - Documentation directory path
   * @param {string} projectKey - Project key
   * @returns {Promise<Object[]>} Processed files
   */
  async processDocumentationFiles(files, docsPath, projectKey) {
    const processedFiles = [];
    
    for (const file of files) {
      try {
        const relativePath = path.relative(docsPath, file);
        const content = await fs.readFile(file, 'utf-8');
        
        // Extract frontmatter
        const frontmatter = this.extractFrontmatter(content);
        
        processedFiles.push({
          path: relativePath,
          title: frontmatter.title || path.basename(file, '.md'),
          description: frontmatter.description || '',
          category: frontmatter.category || '',
          content: this.removeFrontmatter(content)
        });
      } catch (error) {
        // Skip files that cannot be processed
      }
    }
    
    return processedFiles;
  }

  /**
   * Extract frontmatter from content
   * @param {string} content - File content
   * @returns {Object} Frontmatter object
   */
  extractFrontmatter(content) {
    const frontmatterMatch = content.match(/^---\s*\n([\s\S]*?)\n---/);
    
    if (frontmatterMatch) {
      const frontmatterText = frontmatterMatch[1];
      const frontmatter = {};
      
      // Parse frontmatter lines
      frontmatterText.split('\n').forEach(line => {
        const match = line.match(/^([^:]+):\s*(.*)$/);
        if (match) {
          frontmatter[match[1].trim()] = match[2].trim();
        }
      });
      
      return frontmatter;
    }
    
    return {};
  }

  /**
   * Remove frontmatter from content
   * @param {string} content - File content
   * @returns {string} Content without frontmatter
   */
  removeFrontmatter(content) {
    return content.replace(/^---\s*\n[\s\S]*?\n---\s*\n/, '');
  }

  /**
   * Generate search index
   * @param {Object} projectsData - Projects data
   * @param {string} outputDir - Output directory
   * @returns {Promise<void>}
   */
  async generateSearchIndex(projectsData, outputDir) {
    const searchIndex = [];
    
    for (const [projectKey, projectData] of Object.entries(projectsData)) {
      for (const file of projectData.files) {
        // Extract keywords from content
        const keywords = this.extractKeywords(file.content);
        
        searchIndex.push({
          id: `${projectKey}/${file.path}`,
          title: file.title,
          description: file.description,
          project: projectKey,
          path: file.path,
          keywords
        });
      }
    }
    
    await fs.writeFile(
      path.join(outputDir, 'search-index.json'),
      JSON.stringify(searchIndex, null, 2),
      'utf-8'
    );
  }

  /**
   * Extract keywords from content
   * @param {string} content - Content to extract keywords from
   * @returns {string[]} List of keywords
   */
  extractKeywords(content) {
    // Simple keyword extraction
    const words = content
      .toLowerCase()
      .replace(/[^\w\s]/g, '')
      .split(/\s+/)
      .filter(word => word.length > 3);
    
    // Remove duplicates and limit to 20 keywords
    return [...new Set(words)].slice(0, 20);
  }

  /**
   * Generate HTML files
   * @param {Object} projectsData - Projects data
   * @param {string} outputDir - Output directory
   * @returns {Promise<void>}
   */
  async generateHtmlFiles(projectsData, outputDir) {
    // Generate index.html
    const indexHtml = this.generateIndexHtml(projectsData);
    await fs.writeFile(path.join(outputDir, 'index.html'), indexHtml, 'utf-8');
    this.stats.filesGenerated++;
    
    // Generate project HTML files
    for (const [projectKey, projectData] of Object.entries(projectsData)) {
      const projectHtml = this.generateProjectHtml(projectKey, projectData);
      await fs.writeFile(path.join(outputDir, `${projectKey}.html`), projectHtml, 'utf-8');
      this.stats.filesGenerated++;
    }
  }

  /**
   * Generate index HTML
   * @param {Object} projectsData - Projects data
   * @returns {string} HTML content
   */
  generateIndexHtml(projectsData) {
    return `<!DOCTYPE html>
<html lang="en">
<head>
  <meta charset="UTF-8">
  <meta name="viewport" content="width=device-width, initial-scale=1.0">
  <title>Bluefly LLM Platform Documentation</title>
  <style>
    /* Basic styles */
    body { font-family: -apple-system, BlinkMacSystemFont, "Segoe UI", Roboto, Helvetica, Arial, sans-serif; line-height: 1.6; max-width: 1200px; margin: 0 auto; padding: 2rem; }
    h1 { margin-bottom: 2rem; }
    .project-grid { display: grid; grid-template-columns: repeat(auto-fill, minmax(300px, 1fr)); gap: 2rem; }
    .project-card { border: 1px solid #eee; border-radius: 8px; padding: 1.5rem; transition: transform 0.2s; }
    .project-card:hover { transform: translateY(-5px); box-shadow: 0 10px 20px rgba(0,0,0,0.1); }
    .project-title { margin-top: 0; }
    .project-category { display: inline-block; background: #f0f0f0; padding: 0.2rem 0.5rem; border-radius: 4px; font-size: 0.8rem; margin-bottom: 1rem; }
    .project-description { color: #666; }
    .project-link { display: inline-block; margin-top: 1rem; color: #0066cc; text-decoration: none; }
    .project-link:hover { text-decoration: underline; }
  </style>
</head>
<body>
  <h1>Bluefly LLM Platform Documentation</h1>
  
  <div class="project-grid">
    ${Object.entries(projectsData).map(([projectKey, projectData]) => `
    <div class="project-card">
      <span class="project-category">${projectData.category || 'Uncategorized'}</span>
      <h2 class="project-title">${projectData.name}</h2>
      <p class="project-description">${projectData.description || ''}</p>
      <a href="${projectKey}.html" class="project-link">View Documentation →</a>
    </div>
    `).join('')}
  </div>
</body>
</html>`;
  }

  /**
   * Generate project HTML
   * @param {string} projectKey - Project key
   * @param {Object} projectData - Project data
   * @returns {string} HTML content
   */
  generateProjectHtml(projectKey, projectData) {
    return `<!DOCTYPE html>
<html lang="en">
<head>
  <meta charset="UTF-8">
  <meta name="viewport" content="width=device-width, initial-scale=1.0">
  <title>${projectData.name} - Documentation</title>
  <style>
    /* Basic styles */
    body { font-family: -apple-system, BlinkMacSystemFont, "Segoe UI", Roboto, Helvetica, Arial, sans-serif; line-height: 1.6; display: grid; grid-template-columns: 250px 1fr; min-height: 100vh; margin: 0; }
    .sidebar { padding: 2rem; background: #f5f5f5; border-right: 1px solid #eee; }
    .content { padding: 2rem; max-width: 800px; }
    .home-link { display: block; margin-bottom: 2rem; font-weight: bold; color: #333; text-decoration: none; }
    .home-link:hover { color: #0066cc; }
    .nav-title { font-size: 0.8rem; text-transform: uppercase; color: #999; margin: 1.5rem 0 0.5rem; }
    .nav-link { display: block; padding: 0.2rem 0; color: #333; text-decoration: none; }
    .nav-link:hover { color: #0066cc; }
    .project-header { margin-bottom: 2rem; padding-bottom: 1rem; border-bottom: 1px solid #eee; }
    .project-title { margin: 0; }
    .project-category { display: inline-block; background: #f0f0f0; padding: 0.2rem 0.5rem; border-radius: 4px; font-size: 0.8rem; margin: 0.5rem 0; }
    .project-description { color: #666; }
  </style>
</head>
<body>
  <div class="sidebar">
    <a href="index.html" class="home-link">← All Projects</a>
    
    <div class="nav-title">Documentation</div>
    ${projectData.files
      .filter(file => !file.path.includes('/'))
      .map(file => `<a href="#${file.path}" class="nav-link">${file.title}</a>`)
      .join('')}
  </div>
  
  <div class="content">
    <div class="project-header">
      <h1 class="project-title">${projectData.name}</h1>
      <span class="project-category">${projectData.category || 'Uncategorized'}</span>
      <p class="project-description">${projectData.description || ''}</p>
    </div>
    
    ${projectData.files
      .filter(file => !file.path.includes('/'))
      .map(file => `
      <div id="${file.path}">
        <h2>${file.title}</h2>
        <div>${file.content}</div>
      </div>
      `).join('')}
  </div>
</body>
</html>`;
  }
}

module.exports = DocumentationAggregator;